package Builtin "
    Copyright (c) PELAB, Dept. Computer and Information Science, Linkoping University,
    Sweden, in the following denoted PELAB/LIU.

   
    This file is part of OpenModelica, in the following denoted OSM.


    The OSM License, Version 1.1, March 4, 2005.


    1 Preface

    The aim of this license is to lay down the conditions enabling you to use, 
    modify and circulate OSM. However, PELAB/LIU remain the authors of 
    OSM and so retain property rights and the use of all ancillary rights.

    2 Definitions

    OSM is defined as all successive versions of the OSM software and their 
    documentation that have been developed by PELAB/LIU and including
    accepted contributions from other contributors according to this license.

    OSM DERIVED SOFTWARE is defined as all or part of OSM that you have 
    modified and/or translated and/or adapted.


    3 Dual License

    OSM is made available under the OSM licensing scheme, which is a dual
    licensing scheme with two options, a) and b):

    a) OSM OPEN SOURCE LICENSE:

    If you wish to write Open Source software you can use the Open Source version 
    of OSM, released under the OSM license which include GPL as its open source 
    licensing option. If you use the OSM Open Source version you must release your 
    Application using OSM including this Application's source code under the GPL as well.

    This OSM license text, and Copyright (c) PELAB/Linkoping University, must
    be present in your copy of OSM and in OSM DERIVED SOFTWARE.

    You should have received a copy of the GPL - GNU General Public License
    along with OpenModelica; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.


    b) OSM COMMERCIAL LICENSE

    If you are using OSM commercially - that is, for commercial usage or
    for creating proprietary software for sale or use in a commercial setting
    - you must purchase a commercial license of OSM from PELAB/L
  *
 ** file:	 Builtin.rml
 ** module:      Builtin
 ** description: Builting tyepes and variables
 **
 ** RCS: $Id: Builtin.rml 1799 2005-06-02 09:50:11Z levsa $
 **
 ** This module defines the builtin types, variables and functions in
 ** Modelica.  The only exported functions are `initial_env' and
 ** simple_initial_env.
 ** 
 ** There are several builtin attributes defined in the builtin types, such as
 ** unit, start, etc. 
 ** 
 *"

public import OpenModelica.Compiler.Absyn;

public import OpenModelica.Compiler.SCode;

public import OpenModelica.Compiler.Env;

protected import OpenModelica.Compiler.Types "";

protected import OpenModelica.Compiler.ClassInf;

protected import OpenModelica.Compiler.Debug;

protected import OpenModelica.Compiler.Print;

protected constant SCode_Class rl_type=SCode.CLASS("RealType",false,false,SCode.R_PREDEFINED_REAL(),
          SCode.PARTS({},{},{},{},{},NONE)) "* - The primitive types 
 **
 ** These are the primitive types that are used to build the types
 ** `Real', `Integer' etc. 
 *";

protected constant SCode_Class int_type=SCode.CLASS("IntegerType",false,false,SCode.R_PREDEFINED_INT(),
          SCode.PARTS({},{},{},{},{},NONE));

protected constant SCode_Class str_type=SCode.CLASS("StringType",false,false,SCode.R_PREDEFINED_STRING(),
          SCode.PARTS({},{},{},{},{},NONE));

protected constant SCode_Class bool_type=SCode.CLASS("BooleanType",false,false,SCode.R_PREDEFINED_BOOL(),
          SCode.PARTS({},{},{},{},{},NONE));

protected constant SCode_Class enum_type=SCode.CLASS("EnumType",false,false,SCode.R_PREDEFINED_ENUM(),
          SCode.PARTS({},{},{},{},{},NONE));

protected constant SCode_Element unit=SCode.COMPONENT("unit",true,false,false,
          SCode.ATTR({},false,SCode.RW(),SCode.PARAM(),Absyn.BIDIR()),Absyn.IDENT("StringType"),SCode.MOD(false,{},SOME(Absyn.STRING(""))),
          NONE,NONE) "* This `unit' component is used in several places below, and it is
 ** declared once here to make the definitions below easier to read. ";

protected constant SCode_Element quantity=SCode.COMPONENT("quantity",true,false,false,
          SCode.ATTR({},false,SCode.RW(),SCode.PARAM(),Absyn.BIDIR()),Absyn.IDENT("StringType"),SCode.MOD(false,{},SOME(Absyn.STRING(""))),
          NONE,NONE);

protected constant SCode_Element display_unit=SCode.COMPONENT("displayUnit",true,false,false,
          SCode.ATTR({},false,SCode.RW(),SCode.PARAM(),Absyn.BIDIR()),Absyn.IDENT("StringType"),SCode.MOD(false,{},SOME(Absyn.STRING(""))),
          NONE,NONE);

protected constant SCode_Element min=SCode.COMPONENT("min",true,false,false,
          SCode.ATTR({},false,SCode.RW(),SCode.PARAM(),Absyn.BIDIR()),Absyn.IDENT("RealType"),SCode.MOD(false,{},SOME(Absyn.REAL(-1e+99))),
          NONE,NONE);

protected constant SCode_Element max=SCode.COMPONENT("max",true,false,false,
          SCode.ATTR({},false,SCode.RW(),SCode.PARAM(),Absyn.BIDIR()),Absyn.IDENT("RealType"),SCode.MOD(false,{},SOME(Absyn.REAL(1e+99))),
          NONE,NONE);

protected constant SCode_Element real_start=SCode.COMPONENT("start",true,false,false,
          SCode.ATTR({},false,SCode.RW(),SCode.PARAM(),Absyn.BIDIR()),Absyn.IDENT("RealType"),SCode.MOD(false,{},SOME(Absyn.REAL(0.0))),
          NONE,NONE);

protected constant SCode_Element integer_start=SCode.COMPONENT("start",true,false,false,
          SCode.ATTR({},false,SCode.RW(),SCode.PARAM(),Absyn.BIDIR()),Absyn.IDENT("IntegerType"),
          SCode.MOD(false,{},SOME(Absyn.INTEGER(0))),NONE,NONE);

protected constant SCode_Element string_start=SCode.COMPONENT("start",true,false,false,
          SCode.ATTR({},false,SCode.RW(),SCode.PARAM(),Absyn.BIDIR()),Absyn.IDENT("StringType"),SCode.MOD(false,{},SOME(Absyn.STRING(""))),
          NONE,NONE);

protected constant SCode_Element boolean_start=SCode.COMPONENT("start",true,false,false,
          SCode.ATTR({},false,SCode.RW(),SCode.PARAM(),Absyn.BIDIR()),Absyn.IDENT("BooleanType"),
          SCode.MOD(false,{},SOME(Absyn.BOOL(false))),NONE,NONE);

protected constant SCode_Element fixed=SCode.COMPONENT("fixed",true,false,false,
          SCode.ATTR({},false,SCode.RW(),SCode.PARAM(),Absyn.BIDIR()),Absyn.IDENT("BooleanType"),
          SCode.MOD(false,{},SOME(Absyn.BOOL(false))),NONE,NONE);

protected constant SCode_Element nominal=SCode.COMPONENT("nominal",true,false,false,
          SCode.ATTR({},false,SCode.RW(),SCode.PARAM(),Absyn.BIDIR()),Absyn.IDENT("RealType"),SCode.MOD(false,{},NONE),NONE,NONE) " Should be true for variables";

protected constant SCode_Element state_select=SCode.COMPONENT("stateSelect",true,false,false,
          SCode.ATTR({},false,SCode.RW(),SCode.PARAM(),Absyn.BIDIR()),Absyn.IDENT("StateSelect"),
          SCode.MOD(false,{},
          SOME(
          Absyn.CREF(
          Absyn.CREF_QUAL("StateSelect",{},Absyn.CREF_IDENT("default",{}))))),NONE,NONE);

protected 
type SCode_ElementList = list<SCode.Element>;
constant SCode_ElementList state_select_comps={
          SCode.COMPONENT("never",true,false,false,
          SCode.ATTR({},false,SCode.RO(),SCode.CONST(),Absyn.BIDIR()),Absyn.IDENT("EnumType"),SCode.NOMOD(),NONE,NONE),
          SCode.COMPONENT("avoid",true,false,false,
          SCode.ATTR({},false,SCode.RO(),SCode.CONST(),Absyn.BIDIR()),Absyn.IDENT("EnumType"),SCode.NOMOD(),NONE,NONE),
          SCode.COMPONENT("default",true,false,false,
          SCode.ATTR({},false,SCode.RO(),SCode.CONST(),Absyn.BIDIR()),Absyn.IDENT("EnumType"),SCode.NOMOD(),NONE,NONE),
          SCode.COMPONENT("prefer",true,false,false,
          SCode.ATTR({},false,SCode.RO(),SCode.CONST(),Absyn.BIDIR()),Absyn.IDENT("EnumType"),SCode.NOMOD(),NONE,NONE),
          SCode.COMPONENT("always",true,false,false,
          SCode.ATTR({},false,SCode.RO(),SCode.CONST(),Absyn.BIDIR()),Absyn.IDENT("EnumType"),SCode.NOMOD(),NONE,NONE)} " The StateSelect enumeration ";

protected constant SCode_Class state_select_type=SCode.CLASS("StateSelect",false,false,SCode.R_ENUMERATION(),
          SCode.PARTS(state_select_comps,{},{},{},{},NONE));

protected constant SCode_Class real_type=SCode.CLASS("Real",false,false,SCode.R_PREDEFINED_REAL(),
          SCode.PARTS(
          {unit,quantity,display_unit,min,max,real_start,fixed,
          nominal,state_select},{},{},{},{},NONE)) "* - The `Real' type ";

protected constant SCode_Class integer_type=SCode.CLASS("Integer",false,false,SCode.R_PREDEFINED_INT(),
          SCode.PARTS({quantity,min,max,integer_start,fixed},{},{},{},{},NONE)) "* - The `Integer' type ";

protected constant SCode_Class string_type=SCode.CLASS("String",false,false,SCode.R_PREDEFINED_STRING(),
          SCode.PARTS({quantity,string_start},{},{},{},{},NONE)) "* - The `String' type ";

protected constant SCode_Class boolean_type=SCode.CLASS("Boolean",false,false,SCode.R_PREDEFINED_BOOL(),
          SCode.PARTS({quantity,boolean_start,fixed},{},{},{},{},NONE)) "* - The `Boolean' type ";

protected constant Types_Var time_var=Types.VAR("time",
          Types.ATTR(false,SCode.RO(),SCode.VAR(),Absyn.BIDIR()),false,(Types.T_REAL({}),NONE),Types.UNBOUND()) "* - The `time' variable ";

protected 
replaceable type Type_a;

type VType_aOption = Option<Type_a>;

type TupleTypes_TTypeVType_aOption = tuple<Types.TType,VType_aOption>;
constant TupleTypes_TTypeVType_aOption nil2real=(Types.T_FUNCTION({},(Types.T_REAL({}),NONE)),NONE) "* - Some assorted function types ";

protected constant TupleTypes_TTypeVType_aOption nil2bool=(Types.T_FUNCTION({},(Types.T_REAL({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption real2real=(
          Types.T_FUNCTION({("x",(Types.T_REAL({}),NONE))},(Types.T_REAL({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption real2int=(
          Types.T_FUNCTION({("x",(Types.T_REAL({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption int2real=(
          Types.T_FUNCTION({("x",(Types.T_INTEGER({}),NONE))},(Types.T_REAL({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption real_real2real=(
          Types.T_FUNCTION(
          {("x",(Types.T_REAL({}),NONE)),("y",(Types.T_REAL({}),NONE))},(Types.T_REAL({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption int2int=(
          Types.T_FUNCTION({("x",(Types.T_INTEGER({}),NONE))},
          (Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption int_int2int=(
          Types.T_FUNCTION(
          {("x",(Types.T_INTEGER({}),NONE)),
          ("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption bool2bool=(
          Types.T_FUNCTION({("x",(Types.T_BOOL({}),NONE))},(Types.T_BOOL({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption string2string=(
          Types.T_FUNCTION({("x",(Types.T_STRING({}),NONE))},(Types.T_STRING({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption real_real2bool=(
          Types.T_FUNCTION(
          {("x",(Types.T_REAL({}),NONE)),("y",(Types.T_REAL({}),NONE))},(Types.T_BOOL({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimint2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimint2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_INTEGER({}),NONE)),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimint2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_INTEGER({}),NONE)),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimint2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_INTEGER({}),NONE)),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimint2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_INTEGER({}),NONE)),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimint2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_INTEGER({}),NONE)),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimint2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_INTEGER({}),NONE)),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimint2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_INTEGER({}),NONE)),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimreal2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_REAL({}),NONE)),
          NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimreal2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_REAL({}),NONE)),
          NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimreal2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_REAL({}),NONE)),
          NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimreal2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_REAL({}),NONE)),
          NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimreal2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_REAL({}),NONE)),
          NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimreal2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_REAL({}),NONE)),
          NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimreal2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_REAL({}),NONE)),
          NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimreal2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_REAL({}),NONE)),
          NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimreal2real=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_REAL({}),NONE)),
          NONE))},(Types.T_REAL({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimreal2real=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_REAL({}),NONE)),
          NONE))},(Types.T_REAL({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimreal2real=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_REAL({}),NONE)),
          NONE))},(Types.T_REAL({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimreal2real=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_REAL({}),NONE)),
          NONE))},(Types.T_REAL({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimreal2real=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_REAL({}),NONE)),
          NONE))},(Types.T_REAL({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimreal2real=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_REAL({}),NONE)),
          NONE))},(Types.T_REAL({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimreal2real=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_REAL({}),NONE)),
          NONE))},(Types.T_REAL({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimreal2real=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_REAL({}),NONE)),
          NONE))},(Types.T_REAL({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimstring2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_STRING({}),NONE)),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimstring2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_STRING({}),NONE)),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimstring2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_STRING({}),NONE)),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimstring2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_STRING({}),NONE)),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimstring2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_STRING({}),NONE)),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimstring2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_STRING({}),NONE)),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimstring2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_STRING({}),NONE)),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimstring2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_STRING({}),NONE)),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimstring2string=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_STRING({}),NONE)),NONE))},(Types.T_STRING({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimstring2string=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_STRING({}),NONE)),NONE))},(Types.T_STRING({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimstring2string=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_STRING({}),NONE)),NONE))},(Types.T_STRING({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimstring2string=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_STRING({}),NONE)),NONE))},(Types.T_STRING({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimstring2string=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_STRING({}),NONE)),NONE))},(Types.T_STRING({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimstring2string=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_STRING({}),NONE)),NONE))},(Types.T_STRING({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimstring2string=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_STRING({}),NONE)),NONE))},(Types.T_STRING({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimstring2string=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_STRING({}),NONE)),NONE))},(Types.T_STRING({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimbool2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_BOOL({}),NONE)),
          NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimbool2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_BOOL({}),NONE)),
          NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimbool2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_BOOL({}),NONE)),
          NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimbool2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_BOOL({}),NONE)),
          NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimbool2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_BOOL({}),NONE)),
          NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimbool2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_BOOL({}),NONE)),
          NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimbool2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_BOOL({}),NONE)),
          NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimbool2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_BOOL({}),NONE)),
          NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimbool2bool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_BOOL({}),NONE)),
          NONE))},(Types.T_BOOL({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimbool2bool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_BOOL({}),NONE)),
          NONE))},(Types.T_BOOL({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimbool2bool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_BOOL({}),NONE)),
          NONE))},(Types.T_BOOL({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimbool2bool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_BOOL({}),NONE)),
          NONE))},(Types.T_BOOL({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimbool2bool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_BOOL({}),NONE)),
          NONE))},(Types.T_BOOL({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimbool2bool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_BOOL({}),NONE)),
          NONE))},(Types.T_BOOL({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimbool2bool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_BOOL({}),NONE)),
          NONE))},(Types.T_BOOL({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimbool2bool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_BOOL({}),NONE)),
          NONE))},(Types.T_BOOL({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimint_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimint_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_INTEGER({}),NONE)),NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimint_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_INTEGER({}),NONE)),NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimint_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_INTEGER({}),NONE)),NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimint_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_INTEGER({}),NONE)),NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimint_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_INTEGER({}),NONE)),NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimint_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_INTEGER({}),NONE)),NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimint_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_INTEGER({}),NONE)),NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimreal_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_REAL({}),NONE)),
          NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimreal_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_REAL({}),NONE)),
          NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimreal_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_REAL({}),NONE)),
          NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimreal_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_REAL({}),NONE)),
          NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimreal_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_REAL({}),NONE)),
          NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimreal_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_REAL({}),NONE)),
          NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimreal_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_REAL({}),NONE)),
          NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimreal_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_REAL({}),NONE)),
          NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimstring_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_STRING({}),NONE)),NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimstring_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_STRING({}),NONE)),NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimstring_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_STRING({}),NONE)),NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimstring_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_STRING({}),NONE)),NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimstring_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_STRING({}),NONE)),NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimstring_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_STRING({}),NONE)),NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimstring_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_STRING({}),NONE)),NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimstring_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_STRING({}),NONE)),NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimbool_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_BOOL({}),NONE)),
          NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimbool_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_BOOL({}),NONE)),
          NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimbool_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_BOOL({}),NONE)),
          NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimbool_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_BOOL({}),NONE)),
          NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimbool_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_BOOL({}),NONE)),
          NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimbool_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_BOOL({}),NONE)),
          NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimbool_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_BOOL({}),NONE)),
          NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimbool_int2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_BOOL({}),NONE)),
          NONE)),("y",(Types.T_INTEGER({}),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimint2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimint2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimint2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimint2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimint2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimint2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimint2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimint2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimint2matrixint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimint2matrixint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimint2matrixint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimint2matrixint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimint2matrixint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimint2matrixint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimint2matrixint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimint2matrixint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimint2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimint2array2dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimint2array3dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimint2array4dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimint2array5dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimint2array6dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimint2array7dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimint2array8dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimreal2array1dimreal=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_REAL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimreal2array2dimreal=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_REAL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimreal2array3dimreal=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_REAL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimreal2array4dimreal=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_REAL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimreal2array5dimreal=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_REAL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimreal2array6dimreal=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_REAL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimreal2array7dimreal=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_REAL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimreal2array8dimreal=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_REAL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimstring2array1dimstring=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_STRING({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimstring2array2dimstring=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_STRING({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimstring2array3dimstring=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_STRING({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimstring2array4dimstring=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_STRING({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimstring2array5dimstring=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_STRING({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimstring2array6dimstring=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_STRING({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimstring2array7dimstring=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_STRING({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimstring2array8dimstring=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_STRING({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimbool2array1dimbool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_BOOL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimbool2array2dimbool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_BOOL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimbool2array3dimbool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_BOOL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimbool2array4dimbool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_BOOL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimbool2array5dimbool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_BOOL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimbool2array6dimbool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_BOOL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimbool2array7dimbool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_BOOL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimbool2array8dimbool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_BOOL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimreal2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_REAL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimreal2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_REAL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimreal2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_REAL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimreal2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_REAL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimreal2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_REAL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimreal2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_REAL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimreal2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_REAL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimreal2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_REAL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimreal2vectorreal=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_REAL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimreal2vectorreal=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_REAL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimreal2vectorreal=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_REAL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimreal2vectorreal=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_REAL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimreal2vectorreal=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_REAL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimreal2vectorreal=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_REAL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimreal2vectorreal=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_REAL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimreal2vectorreal=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_REAL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimreal2matrixreal=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_REAL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimreal2matrixreal=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_REAL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimreal2matrixreal=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_REAL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimreal2matrixreal=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_REAL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimreal2matrixreal=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_REAL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimreal2matrixreal=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_REAL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimreal2matrixreal=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_REAL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimreal2matrixreal=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_REAL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimstring2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimstring2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimstring2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimstring2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimstring2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimstring2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimstring2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimstring2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimstring2vectorstring=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_STRING({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimstring2vectorstring=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_STRING({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimstring2vectorstring=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_STRING({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimstring2vectorstring=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_STRING({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimstring2vectorstring=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_STRING({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimstring2vectorstring=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_STRING({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimstring2vectorstring=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_STRING({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimstring2vectorstring=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_STRING({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimstring2matrixstring=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_STRING({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimstring2matrixstring=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_STRING({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimstring2matrixstring=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_STRING({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimstring2matrixstring=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_STRING({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimstring2matrixstring=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_STRING({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimstring2matrixstring=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_STRING({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimstring2matrixstring=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_STRING({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimstring2matrixstring=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_STRING({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimbool2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimbool2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimbool2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimbool2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimbool2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimbool2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimbool2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimbool2vectorint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimbool2vectorbool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_BOOL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimbool2vectorbool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_BOOL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimbool2vectorbool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_BOOL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimbool2vectorbool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_BOOL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimbool2vectorbool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_BOOL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimbool2vectorbool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_BOOL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimbool2vectorbool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_BOOL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimbool2vectorbool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_BOOL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimbool2matrixbool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_BOOL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimbool2matrixbool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_BOOL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimbool2matrixbool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_BOOL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimbool2matrixbool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_BOOL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimbool2matrixbool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_BOOL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimbool2matrixbool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_BOOL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimbool2matrixbool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_BOOL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimbool2matrixbool=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_BOOL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption int2matrixint=(
          Types.T_FUNCTION({("x",(Types.T_INTEGER({}),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption real2matrixreal=(
          Types.T_FUNCTION({("x",(Types.T_REAL({}),NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption string2matrixstring=(
          Types.T_FUNCTION({("x",(Types.T_STRING({}),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_STRING({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption bool2matrixbool=(
          Types.T_FUNCTION({("x",(Types.T_BOOL({}),NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_BOOL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption vector_vector2int=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),
          ("y",
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE))},(Types.T_INTEGER({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption vector_vector2real=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_REAL({}),NONE)),
          NONE)),
          ("y",
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_REAL({}),NONE)),
          NONE))},(Types.T_REAL({}),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption int2array1dimint=(
          Types.T_FUNCTION({("x",(Types.T_INTEGER({}),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption int2array2dimint=(
          Types.T_FUNCTION({("x",(Types.T_INTEGER({}),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption int2array3dimint=(
          Types.T_FUNCTION({("x",(Types.T_INTEGER({}),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption int2array4dimint=(
          Types.T_FUNCTION({("x",(Types.T_INTEGER({}),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption int2array5dimint=(
          Types.T_FUNCTION({("x",(Types.T_INTEGER({}),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption int2array6dimint=(
          Types.T_FUNCTION({("x",(Types.T_INTEGER({}),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption int2array7dimint=(
          Types.T_FUNCTION({("x",(Types.T_INTEGER({}),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption int2array8dimint=(
          Types.T_FUNCTION({("x",(Types.T_INTEGER({}),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption n1int2arrayint=(
          Types.T_FUNCTION({("x1",(Types.T_INTEGER({}),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption n2int2arrayint=(
          Types.T_FUNCTION(
          {("x1",(Types.T_INTEGER({}),NONE)),
          ("x2",(Types.T_INTEGER({}),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption n3int2arrayint=(
          Types.T_FUNCTION(
          {("x1",(Types.T_INTEGER({}),NONE)),
          ("x2",(Types.T_INTEGER({}),NONE)),("x3",(Types.T_INTEGER({}),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption n4int2arrayint=(
          Types.T_FUNCTION(
          {("x1",(Types.T_INTEGER({}),NONE)),
          ("x2",(Types.T_INTEGER({}),NONE)),("x3",(Types.T_INTEGER({}),NONE)),("x4",(Types.T_INTEGER({}),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption n5int2arrayint=(
          Types.T_FUNCTION(
          {("x1",(Types.T_INTEGER({}),NONE)),
          ("x2",(Types.T_INTEGER({}),NONE)),("x3",(Types.T_INTEGER({}),NONE)),("x4",(Types.T_INTEGER({}),NONE)),
          ("x5",(Types.T_INTEGER({}),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption n6int2arrayint=(
          Types.T_FUNCTION(
          {("x1",(Types.T_INTEGER({}),NONE)),
          ("x2",(Types.T_INTEGER({}),NONE)),("x3",(Types.T_INTEGER({}),NONE)),("x4",(Types.T_INTEGER({}),NONE)),
          ("x5",(Types.T_INTEGER({}),NONE)),("x6",(Types.T_INTEGER({}),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption n7int2arrayint=(
          Types.T_FUNCTION(
          {("x1",(Types.T_INTEGER({}),NONE)),
          ("x2",(Types.T_INTEGER({}),NONE)),("x3",(Types.T_INTEGER({}),NONE)),("x4",(Types.T_INTEGER({}),NONE)),
          ("x5",(Types.T_INTEGER({}),NONE)),("x6",(Types.T_INTEGER({}),NONE)),("x7",(Types.T_INTEGER({}),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption n8int2arrayint=(
          Types.T_FUNCTION(
          {("x1",(Types.T_INTEGER({}),NONE)),
          ("x2",(Types.T_INTEGER({}),NONE)),("x3",(Types.T_INTEGER({}),NONE)),("x4",(Types.T_INTEGER({}),NONE)),
          ("x5",(Types.T_INTEGER({}),NONE)),("x6",(Types.T_INTEGER({}),NONE)),("x7",(Types.T_INTEGER({}),NONE)),
          ("x8",(Types.T_INTEGER({}),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption n1real2arrayreal=(
          Types.T_FUNCTION({("x1",(Types.T_REAL({}),NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption n2real2arrayreal=(
          Types.T_FUNCTION(
          {("x1",(Types.T_REAL({}),NONE)),
          ("x2",(Types.T_REAL({}),NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption n3real2arrayreal=(
          Types.T_FUNCTION(
          {("x1",(Types.T_REAL({}),NONE)),
          ("x2",(Types.T_REAL({}),NONE)),("x3",(Types.T_REAL({}),NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption n4real2arrayreal=(
          Types.T_FUNCTION(
          {("x1",(Types.T_REAL({}),NONE)),
          ("x2",(Types.T_REAL({}),NONE)),("x3",(Types.T_REAL({}),NONE)),("x4",(Types.T_REAL({}),NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption n5real2arrayreal=(
          Types.T_FUNCTION(
          {("x1",(Types.T_REAL({}),NONE)),
          ("x2",(Types.T_REAL({}),NONE)),("x3",(Types.T_REAL({}),NONE)),("x4",(Types.T_REAL({}),NONE)),
          ("x5",(Types.T_REAL({}),NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption n6real2arrayreal=(
          Types.T_FUNCTION(
          {("x1",(Types.T_REAL({}),NONE)),
          ("x2",(Types.T_REAL({}),NONE)),("x3",(Types.T_REAL({}),NONE)),("x4",(Types.T_REAL({}),NONE)),
          ("x5",(Types.T_REAL({}),NONE)),("x6",(Types.T_REAL({}),NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption n7real2arrayreal=(
          Types.T_FUNCTION(
          {("x1",(Types.T_REAL({}),NONE)),
          ("x2",(Types.T_REAL({}),NONE)),("x3",(Types.T_REAL({}),NONE)),("x4",(Types.T_REAL({}),NONE)),
          ("x5",(Types.T_REAL({}),NONE)),("x6",(Types.T_REAL({}),NONE)),("x7",(Types.T_REAL({}),NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption n8real2arrayreal=(
          Types.T_FUNCTION(
          {("x1",(Types.T_REAL({}),NONE)),
          ("x2",(Types.T_REAL({}),NONE)),("x3",(Types.T_REAL({}),NONE)),("x4",(Types.T_REAL({}),NONE)),
          ("x5",(Types.T_REAL({}),NONE)),("x6",(Types.T_REAL({}),NONE)),("x7",(Types.T_REAL({}),NONE)),
          ("x8",(Types.T_REAL({}),NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption int_int2vectorreal=(
          Types.T_FUNCTION(
          {("x",(Types.T_INTEGER({}),NONE)),
          ("y",(Types.T_INTEGER({}),NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimint2array3dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimreal2array3dimreal=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_REAL({}),NONE)),
          NONE))},
          (Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_REAL({}),NONE)),
          NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2real=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE))},(Types.T_INTEGER({}),NONE)),NONE) " T_ARRAY is apperantly not constant. To bad! ";

protected constant TupleTypes_TTypeVType_aOption array2dimint2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE) " Legal syntax: val array2one= (Types.T_FUNCTION({(\"x\",(Types.T_ARRAY(1,(Types.T_REAL({}),NONE)),NONE))}, TYPES.T_INTEGER) 
 For size(A) to transpose A 
 val array1dimint2array1dimint = ... already defined ";

protected constant TupleTypes_TTypeVType_aOption array3dimint2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimint2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimint2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimint2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimint2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimint2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array9dimint2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(9)),(Types.T_INTEGER({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimreal2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_REAL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimreal2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_REAL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimreal2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_REAL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimreal2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_REAL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimreal2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_REAL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimreal2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_REAL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimreal2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_REAL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimreal2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_REAL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array9dimreal2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(9)),(Types.T_REAL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimstring2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimstring2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimstring2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimstring2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimstring2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimstring2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimstring2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimstring2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array9dimstring2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (
          Types.T_ARRAY(Types.DIM(SOME(9)),(Types.T_STRING({}),NONE)),NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array1dimbool2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array2dimbool2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(2)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array3dimbool2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(3)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array4dimbool2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(4)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array5dimbool2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(5)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array6dimbool2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(6)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array7dimbool2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(7)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array8dimbool2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(8)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

protected constant TupleTypes_TTypeVType_aOption array9dimbool2array1dimint=(
          Types.T_FUNCTION(
          {
          ("x",
          (Types.T_ARRAY(Types.DIM(SOME(9)),(Types.T_BOOL({}),NONE)),
          NONE))},
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE)),NONE);

public function simple_initial_env "
val array2array=  (Types.T_FUNCTION({(\"x\",(Types.T_ARRAY)},
				      (Types.T_ARRAY),NONE)
val array_array2array=
val int2array= (Types.T_FUNCTION(\"x\",(Types.T_ARRAY(1,_)),NONE) 
   Specifierar en vector, array of dimension one 
   zeroes, ones, fill? 
   
val real_real_int2array
val array2real
val array_array2int

  * - Initial environment 
  * function: simple_initial_env
 ** 
 ** The initial environment where instantiation takes place is built
 ** up using this function.  It creates an empty environment and adds
 ** all the built-in types to it.
 **
 ** This only creates a minimal environment, useful for debugging purposes.
 *"
  output Env_FrameList env;
protected 
  type Env_FrameList = list<Env.Frame>;
algorithm 
  env := Env.open_scope(Env.empty_env, false, NONE) "Debug.fprint (\"insttr\",\"Creating initial env.\n\") &";
  env := Env.extend_frame_c(env, rl_type);
  env := Env.extend_frame_c(env, int_type);
  env := Env.extend_frame_c(env, str_type);
  env := Env.extend_frame_c(env, bool_type);
  env := Env.extend_frame_c(env, enum_type);
  env := Env.extend_frame_c(env, real_type);
  env := Env.extend_frame_c(env, integer_type);
  env := Env.extend_frame_c(env, string_type);
  env := Env.extend_frame_c(env, boolean_type);
  env := Env.extend_frame_c(env, state_select_type);
end simple_initial_env;

public function initial_env "* function: initial_env
 ** 
 ** The initial environment where instantiation takes place is built
 ** up using this function.  It creates an empty environment and adds
 ** all the built-in definitions to it.
 *"
  output Env_FrameList env;
protected 
  type Env_FrameList = list<Env.Frame>;
  Env_FrameList envb;
algorithm 
  env := Env.open_scope(Env.empty_env, false, NONE) "Debug.fprint (\"insttr\",\"Creating initial env.\n\") &";
  env := Env.extend_frame_c(env, rl_type);
  env := Env.extend_frame_c(env, int_type);
  env := Env.extend_frame_c(env, str_type);
  env := Env.extend_frame_c(env, bool_type);
  env := Env.extend_frame_c(env, enum_type);
  env := Env.extend_frame_c(env, real_type);
  env := Env.extend_frame_c(env, integer_type);
  env := Env.extend_frame_c(env, string_type);
  env := Env.extend_frame_c(env, boolean_type);
  env := Env.extend_frame_c(env, state_select_type);
  env := Env.extend_frame_v(env, time_var, NONE, false);
  env := Env.extend_frame_t(env, "inital", nil2real) " non-functions ";
  env := Env.extend_frame_t(env, "terminal", nil2real);
  env := Env.extend_frame_t(env, "event", bool2bool);
  env := Env.extend_frame_t(env, "switch", bool2bool);
  env := Env.extend_frame_t(env, "timeEvent", real_real2bool);
  env := Env.extend_frame_t(env, "der", real2real);
  env := Env.extend_frame_t(env, "div", real_real2real) " non-differentiable functions ";
  env := Env.extend_frame_t(env, "rem", real_real2real);
  env := Env.extend_frame_t(env, "ceil", real2real);
  envb := Env.extend_frame_t(env, "floor", real2real);
  env := Env.extend_frame_t(envb, "integer", real2int);
  env := Env.extend_frame_t(env, "abs", real2real) " differentiable functions ";
  env := Env.extend_frame_t(env, "sign", real2real);
  env := Env.extend_frame_t(env, "sin", real2real) " Not in the report ";
  env := Env.extend_frame_t(env, "cos", real2real);
  env := Env.extend_frame_t(env, "tan", real2real);
  env := Env.extend_frame_t(env, "asin", real2real);
  env := Env.extend_frame_t(env, "acos", real2real);
  env := Env.extend_frame_t(env, "atan", real2real);
  env := Env.extend_frame_t(env, "exp", real2real);
  env := Env.extend_frame_t(env, "log", real2real);
  env := Env.extend_frame_t(env, "log10", real2real);
  env := Env.extend_frame_t(env, "ndims", array1dimint2int) " PR. Add the built in array functions here.  Also do it for real, string and bool";
  env := Env.extend_frame_t(env, "ndims", array2dimint2int);
  env := Env.extend_frame_t(env, "ndims", array3dimint2int);
  env := Env.extend_frame_t(env, "ndims", array4dimint2int);
  env := Env.extend_frame_t(env, "ndims", array5dimint2int);
  env := Env.extend_frame_t(env, "ndims", array6dimint2int);
  env := Env.extend_frame_t(env, "ndims", array7dimint2int);
  env := Env.extend_frame_t(env, "ndims", array8dimint2int);
  env := Env.extend_frame_t(env, "ndims", array1dimreal2int);
  env := Env.extend_frame_t(env, "ndims", array2dimreal2int);
  env := Env.extend_frame_t(env, "ndims", array3dimreal2int);
  env := Env.extend_frame_t(env, "ndims", array4dimreal2int);
  env := Env.extend_frame_t(env, "ndims", array5dimreal2int);
  env := Env.extend_frame_t(env, "ndims", array6dimreal2int);
  env := Env.extend_frame_t(env, "ndims", array7dimreal2int);
  env := Env.extend_frame_t(env, "ndims", array8dimreal2int);
  env := Env.extend_frame_t(env, "ndims", array1dimstring2int);
  env := Env.extend_frame_t(env, "ndims", array2dimstring2int);
  env := Env.extend_frame_t(env, "ndims", array3dimstring2int);
  env := Env.extend_frame_t(env, "ndims", array4dimstring2int);
  env := Env.extend_frame_t(env, "ndims", array5dimstring2int);
  env := Env.extend_frame_t(env, "ndims", array6dimstring2int);
  env := Env.extend_frame_t(env, "ndims", array7dimstring2int);
  env := Env.extend_frame_t(env, "ndims", array8dimstring2int);
  env := Env.extend_frame_t(env, "ndims", array1dimbool2int);
  env := Env.extend_frame_t(env, "ndims", array2dimbool2int);
  env := Env.extend_frame_t(env, "ndims", array3dimbool2int);
  env := Env.extend_frame_t(env, "ndims", array4dimbool2int);
  env := Env.extend_frame_t(env, "ndims", array5dimbool2int);
  env := Env.extend_frame_t(env, "ndims", array6dimbool2int);
  env := Env.extend_frame_t(env, "ndims", array7dimbool2int);
  env := Env.extend_frame_t(env, "ndims", array8dimbool2int);
  env := Env.extend_frame_t(env, "size", array1dimint_int2int) " Debug. Print.print_buf \"\n Just before the built in built in size.\" &	";
  env := Env.extend_frame_t(env, "size", array2dimint_int2int);
  env := Env.extend_frame_t(env, "size", array3dimint_int2int);
  env := Env.extend_frame_t(env, "size", array4dimint_int2int);
  env := Env.extend_frame_t(env, "size", array5dimint_int2int);
  env := Env.extend_frame_t(env, "size", array6dimint_int2int);
  env := Env.extend_frame_t(env, "size", array7dimint_int2int);
  env := Env.extend_frame_t(env, "size", array8dimint_int2int);
  env := Env.extend_frame_t(env, "size", array1dimreal_int2int);
  env := Env.extend_frame_t(env, "size", array2dimreal_int2int);
  env := Env.extend_frame_t(env, "size", array3dimreal_int2int);
  env := Env.extend_frame_t(env, "size", array4dimreal_int2int);
  env := Env.extend_frame_t(env, "size", array5dimreal_int2int);
  env := Env.extend_frame_t(env, "size", array6dimreal_int2int);
  env := Env.extend_frame_t(env, "size", array7dimreal_int2int);
  env := Env.extend_frame_t(env, "size", array8dimreal_int2int);
  env := Env.extend_frame_t(env, "size", array1dimstring_int2int);
  env := Env.extend_frame_t(env, "size", array2dimstring_int2int);
  env := Env.extend_frame_t(env, "size", array3dimstring_int2int);
  env := Env.extend_frame_t(env, "size", array4dimstring_int2int);
  env := Env.extend_frame_t(env, "size", array5dimstring_int2int);
  env := Env.extend_frame_t(env, "size", array6dimstring_int2int);
  env := Env.extend_frame_t(env, "size", array7dimstring_int2int);
  env := Env.extend_frame_t(env, "size", array8dimstring_int2int);
  env := Env.extend_frame_t(env, "size", array1dimbool_int2int);
  env := Env.extend_frame_t(env, "size", array2dimbool_int2int);
  env := Env.extend_frame_t(env, "size", array3dimbool_int2int);
  env := Env.extend_frame_t(env, "size", array4dimbool_int2int);
  env := Env.extend_frame_t(env, "size", array5dimbool_int2int);
  env := Env.extend_frame_t(env, "size", array6dimbool_int2int);
  env := Env.extend_frame_t(env, "size", array7dimbool_int2int);
  env := Env.extend_frame_t(env, "size", array8dimbool_int2int);
  env := Env.extend_frame_t(env, "size", array1dimint2vectorint);
  env := Env.extend_frame_t(env, "size", array2dimint2vectorint);
  env := Env.extend_frame_t(env, "size", array3dimint2vectorint);
  env := Env.extend_frame_t(env, "size", array4dimint2vectorint);
  env := Env.extend_frame_t(env, "size", array5dimint2vectorint);
  env := Env.extend_frame_t(env, "size", array6dimint2vectorint);
  env := Env.extend_frame_t(env, "size", array7dimint2vectorint);
  env := Env.extend_frame_t(env, "size", array8dimint2vectorint);
  env := Env.extend_frame_t(env, "size", array1dimreal2vectorint);
  env := Env.extend_frame_t(env, "size", array2dimreal2vectorint);
  env := Env.extend_frame_t(env, "size", array3dimreal2vectorint);
  env := Env.extend_frame_t(env, "size", array4dimreal2vectorint);
  env := Env.extend_frame_t(env, "size", array5dimreal2vectorint);
  env := Env.extend_frame_t(env, "size", array6dimreal2vectorint);
  env := Env.extend_frame_t(env, "size", array7dimreal2vectorint);
  env := Env.extend_frame_t(env, "size", array8dimreal2vectorint);
  env := Env.extend_frame_t(env, "size", array1dimstring2vectorint);
  env := Env.extend_frame_t(env, "size", array2dimstring2vectorint);
  env := Env.extend_frame_t(env, "size", array3dimstring2vectorint);
  env := Env.extend_frame_t(env, "size", array4dimstring2vectorint);
  env := Env.extend_frame_t(env, "size", array5dimstring2vectorint);
  env := Env.extend_frame_t(env, "size", array6dimstring2vectorint);
  env := Env.extend_frame_t(env, "size", array7dimstring2vectorint);
  env := Env.extend_frame_t(env, "size", array8dimstring2vectorint);
  env := Env.extend_frame_t(env, "size", array1dimbool2vectorint);
  env := Env.extend_frame_t(env, "size", array2dimbool2vectorint);
  env := Env.extend_frame_t(env, "size", array3dimbool2vectorint);
  env := Env.extend_frame_t(env, "size", array4dimbool2vectorint);
  env := Env.extend_frame_t(env, "size", array5dimbool2vectorint);
  env := Env.extend_frame_t(env, "size", array6dimbool2vectorint);
  env := Env.extend_frame_t(env, "size", array7dimbool2vectorint);
  env := Env.extend_frame_t(env, "size", array8dimbool2vectorint);
  env := Env.extend_frame_t(env, "size", array1dimint2array1dimint) " size(A) ";
  env := Env.extend_frame_t(env, "size", array2dimint2array1dimint);
  env := Env.extend_frame_t(env, "size", array3dimint2array1dimint);
  env := Env.extend_frame_t(env, "size", array4dimint2array1dimint);
  env := Env.extend_frame_t(env, "size", array5dimint2array1dimint);
  env := Env.extend_frame_t(env, "size", array6dimint2array1dimint);
  env := Env.extend_frame_t(env, "size", array7dimint2array1dimint);
  env := Env.extend_frame_t(env, "size", array8dimint2array1dimint);
  env := Env.extend_frame_t(env, "size", array9dimint2array1dimint);
  env := Env.extend_frame_t(env, "size", array1dimreal2array1dimint);
  env := Env.extend_frame_t(env, "size", array2dimreal2array1dimint);
  env := Env.extend_frame_t(env, "size", array3dimreal2array1dimint);
  env := Env.extend_frame_t(env, "size", array4dimreal2array1dimint);
  env := Env.extend_frame_t(env, "size", array5dimreal2array1dimint);
  env := Env.extend_frame_t(env, "size", array6dimreal2array1dimint);
  env := Env.extend_frame_t(env, "size", array7dimreal2array1dimint);
  env := Env.extend_frame_t(env, "size", array8dimreal2array1dimint);
  env := Env.extend_frame_t(env, "size", array9dimreal2array1dimint);
  env := Env.extend_frame_t(env, "size", array1dimstring2array1dimint);
  env := Env.extend_frame_t(env, "size", array2dimstring2array1dimint);
  env := Env.extend_frame_t(env, "size", array3dimstring2array1dimint);
  env := Env.extend_frame_t(env, "size", array4dimstring2array1dimint);
  env := Env.extend_frame_t(env, "size", array5dimstring2array1dimint);
  env := Env.extend_frame_t(env, "size", array6dimstring2array1dimint);
  env := Env.extend_frame_t(env, "size", array7dimstring2array1dimint);
  env := Env.extend_frame_t(env, "size", array8dimstring2array1dimint);
  env := Env.extend_frame_t(env, "size", array9dimstring2array1dimint);
  env := Env.extend_frame_t(env, "size", array1dimbool2array1dimint);
  env := Env.extend_frame_t(env, "size", array2dimbool2array1dimint);
  env := Env.extend_frame_t(env, "size", array3dimbool2array1dimint);
  env := Env.extend_frame_t(env, "size", array4dimbool2array1dimint);
  env := Env.extend_frame_t(env, "size", array5dimbool2array1dimint);
  env := Env.extend_frame_t(env, "size", array6dimbool2array1dimint);
  env := Env.extend_frame_t(env, "size", array7dimbool2array1dimint);
  env := Env.extend_frame_t(env, "size", array8dimbool2array1dimint);
  env := Env.extend_frame_t(env, "size", array9dimbool2array1dimint);
  env := Env.extend_frame_t(env, "scalar", array1dimint2int);
  env := Env.extend_frame_t(env, "scalar", array2dimint2int);
  env := Env.extend_frame_t(env, "scalar", array3dimint2int);
  env := Env.extend_frame_t(env, "scalar", array4dimint2int);
  env := Env.extend_frame_t(env, "scalar", array5dimint2int);
  env := Env.extend_frame_t(env, "scalar", array6dimint2int);
  env := Env.extend_frame_t(env, "scalar", array7dimint2int);
  env := Env.extend_frame_t(env, "scalar", array8dimint2int);
  env := Env.extend_frame_t(env, "scalar", array1dimreal2real);
  env := Env.extend_frame_t(env, "scalar", array2dimreal2real);
  env := Env.extend_frame_t(env, "scalar", array3dimreal2real);
  env := Env.extend_frame_t(env, "scalar", array4dimreal2real);
  env := Env.extend_frame_t(env, "scalar", array5dimreal2real);
  env := Env.extend_frame_t(env, "scalar", array6dimreal2real);
  env := Env.extend_frame_t(env, "scalar", array7dimreal2real);
  env := Env.extend_frame_t(env, "scalar", array8dimreal2real);
  env := Env.extend_frame_t(env, "scalar", array1dimbool2bool);
  env := Env.extend_frame_t(env, "scalar", array2dimbool2bool);
  env := Env.extend_frame_t(env, "scalar", array3dimbool2bool);
  env := Env.extend_frame_t(env, "scalar", array4dimbool2bool);
  env := Env.extend_frame_t(env, "scalar", array5dimbool2bool);
  env := Env.extend_frame_t(env, "scalar", array6dimbool2bool);
  env := Env.extend_frame_t(env, "scalar", array7dimbool2bool);
  env := Env.extend_frame_t(env, "scalar", array8dimbool2bool);
  env := Env.extend_frame_t(env, "scalar", array1dimstring2string);
  env := Env.extend_frame_t(env, "scalar", array2dimstring2string);
  env := Env.extend_frame_t(env, "scalar", array3dimstring2string);
  env := Env.extend_frame_t(env, "scalar", array4dimstring2string);
  env := Env.extend_frame_t(env, "scalar", array5dimstring2string);
  env := Env.extend_frame_t(env, "scalar", array6dimstring2string);
  env := Env.extend_frame_t(env, "scalar", array7dimstring2string);
  env := Env.extend_frame_t(env, "scalar", array8dimstring2string);
  env := Env.extend_frame_t(env, "vector", array1dimint2vectorint) " PR. 991024 Continue here.";
  env := Env.extend_frame_t(env, "vector", array2dimint2vectorint);
  env := Env.extend_frame_t(env, "vector", array3dimint2vectorint);
  env := Env.extend_frame_t(env, "vector", array4dimint2vectorint);
  env := Env.extend_frame_t(env, "vector", array5dimint2vectorint);
  env := Env.extend_frame_t(env, "vector", array6dimint2vectorint);
  env := Env.extend_frame_t(env, "vector", array7dimint2vectorint);
  env := Env.extend_frame_t(env, "vector", array8dimint2vectorint);
  env := Env.extend_frame_t(env, "vector", array1dimreal2vectorreal);
  env := Env.extend_frame_t(env, "vector", array2dimreal2vectorreal);
  env := Env.extend_frame_t(env, "vector", array3dimreal2vectorreal);
  env := Env.extend_frame_t(env, "vector", array4dimreal2vectorreal);
  env := Env.extend_frame_t(env, "vector", array5dimreal2vectorreal);
  env := Env.extend_frame_t(env, "vector", array6dimreal2vectorreal);
  env := Env.extend_frame_t(env, "vector", array7dimreal2vectorreal);
  env := Env.extend_frame_t(env, "vector", array8dimreal2vectorreal);
  env := Env.extend_frame_t(env, "vector", array1dimbool2vectorbool);
  env := Env.extend_frame_t(env, "vector", array2dimbool2vectorbool);
  env := Env.extend_frame_t(env, "vector", array3dimbool2vectorbool);
  env := Env.extend_frame_t(env, "vector", array4dimbool2vectorbool);
  env := Env.extend_frame_t(env, "vector", array5dimbool2vectorbool);
  env := Env.extend_frame_t(env, "vector", array6dimbool2vectorbool);
  env := Env.extend_frame_t(env, "vector", array7dimbool2vectorbool);
  env := Env.extend_frame_t(env, "vector", array8dimbool2vectorbool);
  env := Env.extend_frame_t(env, "vector", array1dimstring2vectorstring);
  env := Env.extend_frame_t(env, "vector", array2dimstring2vectorstring);
  env := Env.extend_frame_t(env, "vector", array3dimstring2vectorstring);
  env := Env.extend_frame_t(env, "vector", array4dimstring2vectorstring);
  env := Env.extend_frame_t(env, "vector", array5dimstring2vectorstring);
  env := Env.extend_frame_t(env, "vector", array6dimstring2vectorstring);
  env := Env.extend_frame_t(env, "vector", array7dimstring2vectorstring);
  env := Env.extend_frame_t(env, "vector", array8dimstring2vectorstring);
  env := Env.extend_frame_t(env, "matrix", int2matrixint);
  env := Env.extend_frame_t(env, "matrix", real2matrixreal);
  env := Env.extend_frame_t(env, "matrix", string2matrixstring);
  env := Env.extend_frame_t(env, "matrix", bool2matrixbool);
  env := Env.extend_frame_t(env, "matrix", array1dimint2matrixint);
  env := Env.extend_frame_t(env, "matrix", array2dimint2matrixint);
  env := Env.extend_frame_t(env, "matrix", array3dimint2matrixint);
  env := Env.extend_frame_t(env, "matrix", array4dimint2matrixint);
  env := Env.extend_frame_t(env, "matrix", array5dimint2matrixint);
  env := Env.extend_frame_t(env, "matrix", array6dimint2matrixint);
  env := Env.extend_frame_t(env, "matrix", array7dimint2matrixint);
  env := Env.extend_frame_t(env, "matrix", array8dimint2matrixint);
  env := Env.extend_frame_t(env, "matrix", array1dimreal2matrixreal);
  env := Env.extend_frame_t(env, "matrix", array2dimreal2matrixreal);
  env := Env.extend_frame_t(env, "matrix", array3dimreal2matrixreal);
  env := Env.extend_frame_t(env, "matrix", array4dimreal2matrixreal);
  env := Env.extend_frame_t(env, "matrix", array5dimreal2matrixreal);
  env := Env.extend_frame_t(env, "matrix", array6dimreal2matrixreal);
  env := Env.extend_frame_t(env, "matrix", array7dimreal2matrixreal);
  env := Env.extend_frame_t(env, "matrix", array8dimreal2matrixreal);
  env := Env.extend_frame_t(env, "matrix", array1dimbool2matrixbool);
  env := Env.extend_frame_t(env, "matrix", array2dimbool2matrixbool);
  env := Env.extend_frame_t(env, "matrix", array3dimbool2matrixbool);
  env := Env.extend_frame_t(env, "matrix", array4dimbool2matrixbool);
  env := Env.extend_frame_t(env, "matrix", array5dimbool2matrixbool);
  env := Env.extend_frame_t(env, "matrix", array6dimbool2matrixbool);
  env := Env.extend_frame_t(env, "matrix", array7dimbool2matrixbool);
  env := Env.extend_frame_t(env, "matrix", array8dimbool2matrixbool);
  env := Env.extend_frame_t(env, "matrix", array1dimstring2matrixstring);
  env := Env.extend_frame_t(env, "matrix", array2dimstring2matrixstring);
  env := Env.extend_frame_t(env, "matrix", array3dimstring2matrixstring);
  env := Env.extend_frame_t(env, "matrix", array4dimstring2matrixstring);
  env := Env.extend_frame_t(env, "matrix", array5dimstring2matrixstring);
  env := Env.extend_frame_t(env, "matrix", array6dimstring2matrixstring);
  env := Env.extend_frame_t(env, "matrix", array7dimstring2matrixstring);
  env := Env.extend_frame_t(env, "matrix", array8dimstring2matrixstring);
  env := Env.extend_frame_t(env, "transpose", array2dimint2matrixint);
  env := Env.extend_frame_t(env, "transpose", array3dimint2matrixint);
  env := Env.extend_frame_t(env, "transpose", array4dimint2matrixint);
  env := Env.extend_frame_t(env, "transpose", array5dimint2matrixint);
  env := Env.extend_frame_t(env, "transpose", array6dimint2matrixint);
  env := Env.extend_frame_t(env, "transpose", array2dimreal2matrixreal);
  env := Env.extend_frame_t(env, "transpose", array3dimreal2matrixreal);
  env := Env.extend_frame_t(env, "transpose", array4dimreal2matrixreal);
  env := Env.extend_frame_t(env, "transpose", array5dimreal2matrixreal);
  env := Env.extend_frame_t(env, "transpose", array6dimreal2matrixreal);
  env := Env.extend_frame_t(env, "transpose", array2dimbool2matrixbool);
  env := Env.extend_frame_t(env, "transpose", array3dimbool2matrixbool);
  env := Env.extend_frame_t(env, "transpose", array4dimbool2matrixbool);
  env := Env.extend_frame_t(env, "transpose", array5dimbool2matrixbool);
  env := Env.extend_frame_t(env, "transpose", array6dimbool2matrixbool);
  env := Env.extend_frame_t(env, "transpose", array2dimstring2matrixstring);
  env := Env.extend_frame_t(env, "transpose", array3dimstring2matrixstring);
  env := Env.extend_frame_t(env, "transpose", array4dimstring2matrixstring);
  env := Env.extend_frame_t(env, "transpose", array5dimstring2matrixstring);
  env := Env.extend_frame_t(env, "transpose", array6dimstring2matrixstring);
  env := Env.extend_frame_t(env, "outerproduct", vector_vector2int) " Only real and int makes sense here. And maybe bool. ";
  env := Env.extend_frame_t(env, "outerproduct", vector_vector2real);
  env := Env.extend_frame_t(env, "identity", int2array1dimint);
  env := Env.extend_frame_t(env, "identity", int2array2dimint);
  env := Env.extend_frame_t(env, "identity", int2array3dimint);
  env := Env.extend_frame_t(env, "identity", int2array4dimint);
  env := Env.extend_frame_t(env, "identity", int2array5dimint);
  env := Env.extend_frame_t(env, "identity", int2array6dimint);
  env := Env.extend_frame_t(env, "identity", int2array7dimint);
  env := Env.extend_frame_t(env, "identity", int2array8dimint);
  env := Env.extend_frame_t(env, "initial", nil2bool);
  env := Env.extend_frame_t(env, "terminal", nil2bool);
  env := Env.extend_frame_t(env, "diagonal", array1dimint2matrixint);
  env := Env.extend_frame_t(env, "diagonal", array1dimreal2matrixreal);
  env := Env.extend_frame_t(env, "diagonal", array1dimbool2matrixbool);
  env := Env.extend_frame_t(env, "diagonal", array1dimstring2matrixstring);
  env := Env.extend_frame_t(env, "zeros", n1int2arrayint) " There is a problem to represents these functions where you do not 
 know how many arguments they will take. In this implementation up to 8 arguments are supported. ";
  env := Env.extend_frame_t(env, "zeros", n2int2arrayint);
  env := Env.extend_frame_t(env, "zeros", n3int2arrayint);
  env := Env.extend_frame_t(env, "zeros", n4int2arrayint);
  env := Env.extend_frame_t(env, "zeros", n5int2arrayint);
  env := Env.extend_frame_t(env, "zeros", n6int2arrayint);
  env := Env.extend_frame_t(env, "zeros", n7int2arrayint);
  env := Env.extend_frame_t(env, "zeros", n8int2arrayint);
  env := Env.extend_frame_t(env, "ones", n1int2arrayint);
  env := Env.extend_frame_t(env, "ones", n2int2arrayint);
  env := Env.extend_frame_t(env, "ones", n3int2arrayint);
  env := Env.extend_frame_t(env, "ones", n4int2arrayint);
  env := Env.extend_frame_t(env, "ones", n5int2arrayint);
  env := Env.extend_frame_t(env, "ones", n6int2arrayint);
  env := Env.extend_frame_t(env, "ones", n7int2arrayint);
  env := Env.extend_frame_t(env, "ones", n8int2arrayint);
  env := Env.extend_frame_t(env, "array", n1int2arrayint);
  env := Env.extend_frame_t(env, "array", n2int2arrayint);
  env := Env.extend_frame_t(env, "array", n3int2arrayint);
  env := Env.extend_frame_t(env, "array", n4int2arrayint);
  env := Env.extend_frame_t(env, "array", n5int2arrayint);
  env := Env.extend_frame_t(env, "array", n6int2arrayint);
  env := Env.extend_frame_t(env, "array", n7int2arrayint);
  env := Env.extend_frame_t(env, "array", n8int2arrayint);
  env := Env.extend_frame_t(env, "array", n1real2arrayreal);
  env := Env.extend_frame_t(env, "array", n2real2arrayreal);
  env := Env.extend_frame_t(env, "array", n3real2arrayreal);
  env := Env.extend_frame_t(env, "array", n4real2arrayreal);
  env := Env.extend_frame_t(env, "array", n5real2arrayreal);
  env := Env.extend_frame_t(env, "array", n6real2arrayreal);
  env := Env.extend_frame_t(env, "array", n7real2arrayreal);
  env := Env.extend_frame_t(env, "array", n8real2arrayreal);
  env := Env.extend_frame_t(env, "linspace", int_int2vectorreal) "	Fill is impossible to do with this scemen, because the type of the array is unkown during the compile time of omc.

	Env.extend_frame_t(env, \"fill\", array2int) 		     => env &
";
  env := Env.extend_frame_t(env, "min", int_int2int);
  env := Env.extend_frame_t(env, "min", real_real2real);
  env := Env.extend_frame_t(env, "min", array1dimint2int);
  env := Env.extend_frame_t(env, "min", array2dimint2int);
  env := Env.extend_frame_t(env, "min", array3dimint2int);
  env := Env.extend_frame_t(env, "min", array4dimint2int);
  env := Env.extend_frame_t(env, "min", array5dimint2int);
  env := Env.extend_frame_t(env, "min", array6dimint2int);
  env := Env.extend_frame_t(env, "min", array7dimint2int);
  env := Env.extend_frame_t(env, "min", array8dimint2int);
  env := Env.extend_frame_t(env, "min", array1dimreal2real);
  env := Env.extend_frame_t(env, "min", array2dimreal2real);
  env := Env.extend_frame_t(env, "min", array3dimreal2real);
  env := Env.extend_frame_t(env, "min", array4dimreal2real);
  env := Env.extend_frame_t(env, "min", array5dimreal2real);
  env := Env.extend_frame_t(env, "min", array6dimreal2real);
  env := Env.extend_frame_t(env, "min", array7dimreal2real);
  env := Env.extend_frame_t(env, "min", array8dimreal2real);
  env := Env.extend_frame_t(env, "max", int_int2int);
  env := Env.extend_frame_t(env, "max", real_real2real);
  env := Env.extend_frame_t(env, "max", array1dimint2int);
  env := Env.extend_frame_t(env, "max", array2dimint2int);
  env := Env.extend_frame_t(env, "max", array3dimint2int);
  env := Env.extend_frame_t(env, "max", array4dimint2int);
  env := Env.extend_frame_t(env, "max", array5dimint2int);
  env := Env.extend_frame_t(env, "max", array6dimint2int);
  env := Env.extend_frame_t(env, "max", array7dimint2int);
  env := Env.extend_frame_t(env, "max", array8dimint2int);
  env := Env.extend_frame_t(env, "max", array1dimreal2real);
  env := Env.extend_frame_t(env, "max", array2dimreal2real);
  env := Env.extend_frame_t(env, "max", array3dimreal2real);
  env := Env.extend_frame_t(env, "max", array4dimreal2real);
  env := Env.extend_frame_t(env, "max", array5dimreal2real);
  env := Env.extend_frame_t(env, "max", array6dimreal2real);
  env := Env.extend_frame_t(env, "max", array7dimreal2real);
  env := Env.extend_frame_t(env, "max", array8dimreal2real);
  env := Env.extend_frame_t(env, "noEvent", real2real);
  env := Env.extend_frame_t(env, "sum", array1dimint2int);
  env := Env.extend_frame_t(env, "sum", array2dimint2int);
  env := Env.extend_frame_t(env, "sum", array3dimint2int);
  env := Env.extend_frame_t(env, "sum", array4dimint2int);
  env := Env.extend_frame_t(env, "sum", array5dimint2int);
  env := Env.extend_frame_t(env, "sum", array6dimint2int);
  env := Env.extend_frame_t(env, "sum", array7dimint2int);
  env := Env.extend_frame_t(env, "sum", array8dimint2int);
  env := Env.extend_frame_t(env, "sum", array1dimreal2real);
  env := Env.extend_frame_t(env, "sum", array2dimreal2real);
  env := Env.extend_frame_t(env, "sum", array3dimreal2real);
  env := Env.extend_frame_t(env, "sum", array4dimreal2real);
  env := Env.extend_frame_t(env, "sum", array5dimreal2real);
  env := Env.extend_frame_t(env, "sum", array6dimreal2real);
  env := Env.extend_frame_t(env, "sum", array7dimreal2real);
  env := Env.extend_frame_t(env, "sum", array8dimreal2real);
  env := Env.extend_frame_t(env, "product", array1dimint2int);
  env := Env.extend_frame_t(env, "product", array2dimint2int);
  env := Env.extend_frame_t(env, "product", array3dimint2int);
  env := Env.extend_frame_t(env, "product", array4dimint2int);
  env := Env.extend_frame_t(env, "product", array5dimint2int);
  env := Env.extend_frame_t(env, "product", array6dimint2int);
  env := Env.extend_frame_t(env, "product", array7dimint2int);
  env := Env.extend_frame_t(env, "product", array8dimint2int);
  env := Env.extend_frame_t(env, "product", array1dimreal2real);
  env := Env.extend_frame_t(env, "product", array2dimreal2real);
  env := Env.extend_frame_t(env, "product", array3dimreal2real);
  env := Env.extend_frame_t(env, "product", array4dimreal2real);
  env := Env.extend_frame_t(env, "product", array5dimreal2real);
  env := Env.extend_frame_t(env, "product", array6dimreal2real);
  env := Env.extend_frame_t(env, "product", array7dimreal2real);
  env := Env.extend_frame_t(env, "product", array8dimreal2real);
  env := Env.extend_frame_t(env, "pre", real2real);
  env := Env.extend_frame_t(env, "pre", int2int);
  env := Env.extend_frame_t(env, "pre", bool2bool);
  env := Env.extend_frame_t(env, "pre", string2string);
  env := Env.extend_frame_t(env, "symmetric", array1dimint2array1dimint);
  env := Env.extend_frame_t(env, "symmetric", array2dimint2array2dimint);
  env := Env.extend_frame_t(env, "symmetric", array3dimint2array3dimint);
  env := Env.extend_frame_t(env, "symmetric", array4dimint2array4dimint);
  env := Env.extend_frame_t(env, "symmetric", array5dimint2array5dimint);
  env := Env.extend_frame_t(env, "symmetric", array6dimint2array6dimint);
  env := Env.extend_frame_t(env, "symmetric", array7dimint2array7dimint);
  env := Env.extend_frame_t(env, "symmetric", array8dimint2array8dimint);
  env := Env.extend_frame_t(env, "symmetric", array1dimreal2array1dimreal);
  env := Env.extend_frame_t(env, "symmetric", array2dimreal2array2dimreal);
  env := Env.extend_frame_t(env, "symmetric", array3dimreal2array3dimreal);
  env := Env.extend_frame_t(env, "symmetric", array4dimreal2array4dimreal);
  env := Env.extend_frame_t(env, "symmetric", array5dimreal2array5dimreal);
  env := Env.extend_frame_t(env, "symmetric", array6dimreal2array6dimreal);
  env := Env.extend_frame_t(env, "symmetric", array7dimreal2array7dimreal);
  env := Env.extend_frame_t(env, "symmetric", array8dimreal2array8dimreal);
  env := Env.extend_frame_t(env, "symmetric", array1dimstring2array1dimstring);
  env := Env.extend_frame_t(env, "symmetric", array2dimstring2array2dimstring);
  env := Env.extend_frame_t(env, "symmetric", array3dimstring2array3dimstring);
  env := Env.extend_frame_t(env, "symmetric", array4dimstring2array4dimstring);
  env := Env.extend_frame_t(env, "symmetric", array5dimstring2array5dimstring);
  env := Env.extend_frame_t(env, "symmetric", array6dimstring2array6dimstring);
  env := Env.extend_frame_t(env, "symmetric", array7dimstring2array7dimstring);
  env := Env.extend_frame_t(env, "symmetric", array8dimstring2array8dimstring);
  env := Env.extend_frame_t(env, "symmetric", array1dimbool2array1dimbool);
  env := Env.extend_frame_t(env, "symmetric", array2dimbool2array2dimbool);
  env := Env.extend_frame_t(env, "symmetric", array3dimbool2array3dimbool);
  env := Env.extend_frame_t(env, "symmetric", array4dimbool2array4dimbool);
  env := Env.extend_frame_t(env, "symmetric", array5dimbool2array5dimbool);
  env := Env.extend_frame_t(env, "symmetric", array6dimbool2array6dimbool);
  env := Env.extend_frame_t(env, "symmetric", array7dimbool2array7dimbool);
  env := Env.extend_frame_t(env, "symmetric", array8dimbool2array8dimbool);
  env := Env.extend_frame_t(env, "cross", array1dimint2array1dimint);
  env := Env.extend_frame_t(env, "cross", array1dimreal2array1dimreal);
  env := Env.extend_frame_t(env, "skew", array1dimint2array3dimint);
  env := Env.extend_frame_t(env, "skew", array1dimreal2array3dimreal);
  env := Env.extend_frame_t(env, "sqrt", int2real);
  env := Env.extend_frame_t(env, "sqrt", real2real) "Debug.	Print.print_buf \"\n Just AFTER the built in array functions part.\" ";
end initial_env;
end Builtin;

