package Ceval "
 Copyright (c) PELAB, Dept. Computer and Information Science, Linkoping University,
 Sweden, in the following denoted PELAB/LIU.
 
 
 This file is part of OpenModelica, in the following denoted OSM.
 
 
 The OSM License, Version 1.1, March 4, 2005.
 

 1 Preface

 The aim of this license is to lay down the conditions enabling you to use, 
 modify and circulate OSM. However, PELAB/LIU remain the authors of 
 OSM and so retain property rights and the use of all ancillary rights.

 2 Definitions

 OSM is defined as all successive versions of the OSM software and their 
 documentation that have been developed by PELAB/LIU and including
 accepted contributions from other contributors according to this license.

 OSM DERIVED SOFTWARE is defined as all or part of OSM that you have 
 modified and/or translated and/or adapted.


 3 Dual License

 OSM is made available under the OSM licensing scheme, which is a dual
 licensing scheme with two options, a) and b):

a) OSM OPEN SOURCE LICENSE:

If you wish to write Open Source software you can use the Open Source version 
of OSM, released under the OSM license which include GPL as its open source 
licensing option. If you use the OSM Open Source version you must release your 
Application using OSM including this Application's source code under the GPL as well.

This OSM license text, and Copyright (c) PELAB/Linkoping University, must
be present in your copy of OSM and in OSM DERIVED SOFTWARE.

You should have received a copy of the GPL - GNU General Public License
along with OpenModelica; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.


b) OSM COMMERCIAL LICENSE

If you are using OSM commercially - that is, for commercial usage or
for creating proprietary software for sale or use in a commercial setting
- you must purchase a commercial license of OSM from PELAB/LIU, which allows
you to use OSM without releasing your Application under the GPL.

Comment: Payments for 
  *
 ** file:	 Ceval.rml
 ** module:      Ceval
 ** description: Constant propagation of expressions
 **
 ** RCS: $Id: Ceval.rml 1801 2005-06-03 06:46:30Z x02lucpo $
 ** 
 ** This module handles constant propagation (or evaluation)
 ** When elaborating expressions, in the Static module, expressions are checked to 
 ** find out its type. It also checks whether the expressions are constant and the function 
 ** ceval in this module will then evaluate the expression to a constant value, defined
 ** in the Values module.
 **
 ** Input: 
 **	Env: Environment with bindings
 **	Exp: Expression to check for constant evaluation
 **	Bool flag determines whether the current instantiation is implicit
 **	InteractiveSymbolTable is optional, and used in interactive mode,
 **	e.g. from mosh
 **	
 ** Output:
 **	Value: The evaluated value
 **     InteractiveSymbolTable: Modified symbol table
 **     Subscript list : Evaluates subscripts and generates constant expressions. 
 *"

public import OpenModelica.Compiler.Env;

public import OpenModelica.Compiler.Exp;

public import OpenModelica.Compiler.Interactive;

public import OpenModelica.Compiler.Values;

protected import OpenModelica.Compiler.DAELow;

protected import OpenModelica.Compiler.SimCodegen;

protected import OpenModelica.Compiler.Static;

protected import OpenModelica.Compiler.Print;

protected import OpenModelica.Compiler.Types;

protected import OpenModelica.Compiler.ModUtil;

protected import OpenModelica.Compiler.System;

protected import OpenModelica.Compiler.Absyn;

protected import OpenModelica.Compiler.SCode;

protected import OpenModelica.Compiler.Inst;

protected import OpenModelica.Compiler.Lookup;

protected import OpenModelica.Compiler.Dump;

protected import OpenModelica.Compiler.DAE;

protected import OpenModelica.Compiler.Debug;

protected import OpenModelica.Compiler.Util;

protected import OpenModelica.Compiler.ClassInf;

protected import OpenModelica.Compiler.RTOpts;

protected import OpenModelica.Compiler.Parser;

protected import OpenModelica.Compiler.Prefix;

protected import OpenModelica.Compiler.Codegen;

protected import OpenModelica.Compiler.ClassLoader;

protected import OpenModelica.Compiler.Derive;

protected function ceval_builtin
  input Env.Env in_env;
  input Exp.Exp in_exp;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  input IntegerOption in_integeroption;
  output Values.Value out_value;
  output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
protected 
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
  type IntegerOption = Option<Integer>;
algorithm 
  (out_value,out_interactive_interactivesymboltableoption):=
  matchcontinue (in_env,in_exp,in_boolean,in_interactive_interactivesymboltableoption,in_integeroption)
    local
      type Env_FrameList = list<Env.Frame>;
      function FuncType1
        input Env_FrameList in_env_framelist;
        input Exp_ExpList in_exp_explist;
        input Boolean in_boolean;
        input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
        output Values.Value out_value;
        output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
      protected 
        type Exp_ExpList = list<Exp.Exp>;
      end FuncType1;
      type Exp_ExpList = list<Exp.Exp>;
      type Values_ValueList = list<Values.Value>;
      Values.Value v,newval;
      Interactive_InteractiveSymbolTableOption st;
      Env_FrameList env;
      Exp.Exp exp,dim,e;
      Boolean impl,builtin;
      FuncType1 handler;
      String id;
      Exp_ExpList args,expl;
      Values_ValueList vallst;
      Absyn.Path funcpath;
    case (env,Exp.SIZE(exp,SOME(dim)),impl,st,_) "impl  Handle size separately "
      equation 
        (v,st) = ceval_builtin_size(env, exp, dim, impl, st); then (v,st);
    case (env,Exp.SIZE(exp,NONE),impl,st,_)
      equation 
        (v,st) = ceval_builtin_size_matrix(env, exp, impl, st); then (v,st);
    case (env,Exp.CALL(Absyn.IDENT(id),args,_,builtin),impl,st,_) "as true"
      equation 
        handler = ceval_builtin_handler(id);
        (v,st) = handler(env, args, impl, st); then (v,st);
    case (env,(e = Exp.CALL(funcpath,expl,_,(builtin = true))),impl,(st = NONE),_)
      equation 
        vallst = ceval_list(env, expl, impl, st);
        newval = ceval_call_function(env, e, vallst); then (newval,st);
  end matchcontinue;
end ceval_builtin;

protected function ceval_builtin_handler
  input Absyn.Ident in_ident;
  output FuncType1 out_functype1;
protected 
  function FuncType1
    input Env.Env in_env;
    input Exp_ExpList in_exp_explist;
    input Boolean in_boolean;
    input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
    output Values.Value out_value;
    output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
  protected 
    type Exp_ExpList = list<Exp.Exp>;
    type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
  end FuncType1;
algorithm 
  out_functype1:=
  matchcontinue (in_ident)
    local String id;
    case "floor" then ceval_builtin_floor; 
    case "ceil" then ceval_builtin_ceil; 
    case "abs" then ceval_builtin_abs; 
    case "sqrt" then ceval_builtin_sqrt; 
    case "div" then ceval_builtin_div; 
    case "sin" then ceval_builtin_sin; 
    case "cos" then ceval_builtin_cos; 
    case "tan" then ceval_builtin_tan; 
    case "integer" then ceval_builtin_integer; 
    case "mod" then ceval_builtin_mod; 
    case "rem" then ceval_builtin_rem; 
    case "diagonal" then ceval_builtin_diagonal; 
    case "differentiate" then ceval_builtin_differentiate; 
    case "simplify" then ceval_builtin_simplify; 
    case id
      equation 
        Debug.fprint("ceval", "No ceval_builtin_handler found for: ");
        Debug.fprintln("ceval", id); then fail();
  end matchcontinue;
end ceval_builtin_handler;

public function ceval
  input Env.Env in_env;
  input Exp.Exp in_exp;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  input IntegerOption in_integeroption;
  output Values.Value out_value;
  output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
protected 
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
  type IntegerOption = Option<Integer>;
algorithm 
  (out_value,out_interactive_interactivesymboltableoption):=
  matchcontinue (in_env,in_exp,in_boolean,in_interactive_interactivesymboltableoption,in_integeroption)
    local
      type Env_FrameList = list<Env.Frame>;
      type Values_ValueList = list<Values.Value>;
      type Exp_ExpList = list<Exp.Exp>;
      type TupleExp_ExpBoolean = tuple<Exp.Exp,Boolean>;
      type TupleExp_ExpBooleanList = list<TupleExp_ExpBoolean>;
      type TupleExp_ExpBooleanListList = list<TupleExp_ExpBooleanList>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOption = tuple<Absyn.Path,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOptionList = list<TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOption>;
      Integer x,dim,l,lhv,rhv,res,start_1,stop_1,step_1,i,indx_1,indx;
      Interactive_InteractiveSymbolTableOption st,st_1,st_2,st_3;
      Real lhvr,rhvr,resr,sum,div,x_1,diff,r;
      String funcstr,infilename,outfilename,str,iter;
      Boolean impl,builtin,b,b_1;
      Absyn.Exp exp_1,exp;
      Env_FrameList env;
      Absyn.Element elt_1,elt;
      Absyn.Code c;
      Values_ValueList es_1,elts,vallst,vlst1,vlst2,reslst,aval,rhvals,lhvals,arr,ivals,rvals,vallst_1,vals;
      Exp_ExpList es,expl;
      TupleExp_ExpBooleanListList expll;
      Values.Value v,newval,value,sval,elt1,elt2,lhs_1,rhs_1;
      Exp.Exp lh,rh,e,lhs,rhs,start,stop,step,e1,e2,iterexp;
      Absyn.Path funcpath,func;
      Absyn.Program p;
      TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOptionList cflist;
      Exp.Operator relop;
    case (_,Exp.ICONST(x),_,st,_) then (Values.INTEGER(x),st); 
    case (_,Exp.RCONST(x),_,st,_)
      local Real x; then (Values.REAL(x),st);
    case (_,Exp.SCONST(x),_,st,_)
      local String x; then (Values.STRING(x),st);
    case (_,Exp.BCONST(x),_,st,_)
      local Boolean x; then (Values.BOOL(x),st);
    case (_,Exp.END(),_,st,SOME(dim)) then (Values.INTEGER(dim),st); 
    case (_,Exp.END(),_,st,NONE)
      equation 
        Print.print_error_buf("# 'end' cannot be used outside array subscript\n"); then fail();
    case (env,Exp.CODE(Absyn.C_EXPRESSION(exp),_),impl,st,_)
      equation 
        exp_1 = ceval_ast_exp(env, exp, impl, st); then (Values.CODE(Absyn.C_EXPRESSION(exp_1)),st);
    case (env,Exp.CODE(Absyn.C_EXPRESSION(exp),_),impl,st,_)
      equation 
        exp_1 = ceval_ast_exp(env, exp, impl, st); then (Values.CODE(Absyn.C_EXPRESSION(exp_1)),st);
    case (env,Exp.CODE(Absyn.C_ELEMENT(elt),_),impl,st,_)
      equation 
        elt_1 = ceval_ast_elt(env, elt, impl, st); then (Values.CODE(Absyn.C_ELEMENT(elt_1)),st);
    case (env,Exp.CODE(c,_),_,st,_) then (Values.CODE(c),st); 
    case (env,Exp.ARRAY(_,_,es),impl,st,_)
      equation 
        es_1 = ceval_list(env, es, impl, st);
        l = list_length(es_1); then (Values.ARRAY(es_1),st);
    case (env,Exp.MATRIX(_,_,expll),impl,st,_)
      equation 
        elts = ceval_matrixelt(env, expll, impl); then (Values.ARRAY(elts),st);
    case (env,Exp.CREF(c,_),(impl = false),SOME(st),_)
      local
        Exp.ComponentRef c;
        Interactive.InteractiveSymbolTable st;
      equation 
        v = ceval_cref(env, c, false) " When in interactive mode, always evalutate crefs, i.e non-implicit
	   * mode.."; then (v,SOME(st));
    case (env,Exp.CREF(c,_),impl,st,_)
      local Exp.ComponentRef c;
      equation 
        v = ceval_cref(env, c, impl); then (v,st);
    case (env,exp,impl,st,dim)
      local
        Exp.Exp exp;
        IntegerOption dim;
      equation 
        (v,st_1) = ceval_builtin(env, exp, impl, st, dim); then (v,st_1);
    case (env,Exp.BINARY(lh,Exp.POW(Exp.INT()),rh),impl,st,dim)
      local IntegerOption dim;
      equation 
        (Values.INTEGER(lhv),st_1) = ceval(env, lh, impl, st, dim);
        (Values.INTEGER(rhv),st_2) = ceval(env, rh, impl, st_1, dim);
        lhvr = int_real(lhv);
        rhvr = int_real(rhv);
        resr = real_pow(lhvr, rhvr);
        res = real_int(resr); then (Values.INTEGER(res),st_2);
    case (env,Exp.BINARY(lh,Exp.POW(Exp.REAL()),rh),impl,st,dim)
      local
        Real lhv;
        IntegerOption dim;
      equation 
        (Values.REAL(lhv),st_1) = ceval(env, lh, impl, st, dim);
        (Values.INTEGER(rhv),st_2) = ceval(env, rh, impl, st_1, dim);
        rhvr = int_real(rhv);
        resr = real_pow(lhv, rhvr); then (Values.REAL(resr),st_2);
    case (env,Exp.BINARY(lh,Exp.POW(Exp.REAL()),rh),impl,st,dim)
      local
        Real rhv;
        IntegerOption dim;
      equation 
        (Values.INTEGER(lhv),st_1) = ceval(env, lh, impl, st, dim);
        (Values.REAL(rhv),st_2) = ceval(env, rh, impl, st_1, dim);
        lhvr = int_real(lhv);
        resr = real_pow(lhvr, rhv); then (Values.REAL(resr),st_2);
    case (env,Exp.BINARY(lh,Exp.POW(Exp.REAL()),rh),impl,st,dim)
      local
        Real lhv,rhv;
        IntegerOption dim;
      equation 
        (Values.REAL(lhv),st_1) = ceval(env, lh, impl, st, dim);
        (Values.REAL(rhv),st_2) = ceval(env, rh, impl, st_1, dim);
        resr = real_pow(lhv, rhv); then (Values.REAL(resr),st_2);
    case (env,(e = Exp.CALL(funcpath,expl,_,(builtin = false))),(impl = false),(st = NONE),_) " end luc_pop   FIXME: disabled calling of functions during ceval for now. Some
 * functions call other functions, which must be generated together in
 * order that compilation works   
 axiom	ceval (env, e as Exp.CALL(funcpath,expl,_,builtin as false),impl as false,st as NONE,_) => fail
   Call functions in non-interactive mode.   FIXME: functions are always generated. Put back the check
 and write another rule for the false case that generates the function "
      equation 
        vallst = ceval_list(env, expl, impl, st);
        newval = ceval_call_function(env, e, vallst); then (newval,st);
    case (env,(e = Exp.CALL(_,_,_,_)),(impl = false),NONE,_) "	ceval (env, e as Exp.CALL(funcpath,expl,_,_),false,st as SOME(Interactive.SYMBOLTABLE(_,_,_,_,cflist)),_) => (newval,st)
   FIXME: Fix this in order to allow calling of constant-marked function
 * calls. One should call the function as if we were in interactive
 * mode, and put the result instead of the function call  	 

 rule	Print.print_buf \"# Can't call builtin functions at compile time\n\" &
 Print.print_buf \"    expression: \" & Exp.print_exp e & Print.print_buf \"\n\"
 -------------------------------------------------------
 ceval (env, e as Exp.CALL(_,_,_,builtin as true),impl as false,NONE,_) => fail
 "
      equation 
        Print.print_buf("# Can't call functions at compile time\n");
        Print.print_buf("    expression: ");
        Exp.print_exp(e);
        Print.print_buf("\n"); then fail();
    case (env,(e = Exp.CALL(_,_,_,_)),(impl = true),NONE,_) then fail(); 
    case (env,(e = Exp.CALL(_,_,_,_)),(impl = true),SOME(st),_)
      local Interactive.InteractiveSymbolTable st;
      equation 
        (value,st) = ceval_interactive_functions(env, e, st); then (value,SOME(st));
    case (env,(e = Exp.CALL(func,expl,_,_)),(impl = true),(st = SOME(_)),_)
      equation 
        vallst = ceval_list(env, expl, impl, st) " Call of record constructors, etc., i.e. functions that can be 
	 constant propagated. ";
        newval = ceval_function(env, func, vallst, impl); then (newval,st);
    case (env,(e = Exp.CALL(func,expl,_,_)),(impl = true),(st = SOME(Interactive.SYMBOLTABLE(p,_,_,_,cflist))),_)
      equation 
        true = Static.is_function_in_cflist(cflist, func) " Call externally implemented functions. ";
        vallst = ceval_list(env, expl, impl, st);
        funcstr = ModUtil.path_string2(func, "_");
        infilename = string_append(funcstr, "_in.txt");
        outfilename = string_append(funcstr, "_out.txt");
        Values.write_to_file_as_args(vallst, infilename);
        System.execute_function(funcstr);
        newval = System.read_values_from_file(outfilename); then (newval,st);
    case (env,Exp.BINARY(lh,Exp.ADD(Exp.STRING()),rh),impl,st,_) "* Strings *"
      local String lhv,rhv;
      equation 
        (Values.STRING(lhv),_) = ceval(env, lh, impl, st, NONE);
        (Values.STRING(rhv),_) = ceval(env, rh, impl, st, NONE);
        str = string_append(lhv, rhv); then (Values.STRING(str),st);
    case (env,Exp.BINARY(lh,Exp.ADD(Exp.REAL()),rh),impl,st,dim) "*  Numerical "
      local
        Real lhv,rhv;
        IntegerOption dim;
      equation 
        (Values.REAL(lhv),st_1) = ceval(env, lh, impl, st, dim);
        (Values.REAL(rhv),st_2) = ceval(env, rh, impl, st_1, dim);
        sum = lhv +. rhv; then (Values.REAL(sum),st_2);
    case (env,Exp.BINARY(lh,Exp.ADD_ARR(_),rh),impl,st,dim)
      local IntegerOption dim;
      equation 
        (Values.ARRAY(vlst1),st_1) = ceval(env, lh, impl, st, dim);
        (Values.ARRAY(vlst2),st_2) = ceval(env, rh, impl, st_1, dim);
        reslst = Values.add_elementwise_arrayelt(vlst1, vlst2); then (Values.ARRAY(reslst),st_2);
    case (env,Exp.BINARY(lh,Exp.SUB_ARR(_),rh),impl,st,dim)
      local IntegerOption dim;
      equation 
        (Values.ARRAY(vlst1),st_1) = ceval(env, lh, impl, st, dim);
        (Values.ARRAY(vlst2),st_2) = ceval(env, rh, impl, st_1, dim);
        reslst = Values.sub_elementwise_arrayelt(vlst1, vlst2); then (Values.ARRAY(reslst),st_2);
    case (env,Exp.BINARY(lh,Exp.MUL_SCALAR_ARRAY(_),rh),impl,st,dim)
      local IntegerOption dim;
      equation 
        (sval,st_1) = ceval(env, lh, impl, st, dim);
        (Values.ARRAY(aval),st_2) = ceval(env, rh, impl, st_1, dim);
        reslst = Values.mult_scalar_arrayelt(sval, aval); then (Values.ARRAY(reslst),st_2);
    case (env,Exp.BINARY(lh,Exp.MUL_ARRAY_SCALAR(_),rh),impl,st,dim)
      local IntegerOption dim;
      equation 
        (sval,st_1) = ceval(env, rh, impl, st, dim);
        (Values.ARRAY(aval),st_2) = ceval(env, lh, impl, st_1, dim);
        reslst = Values.mult_scalar_arrayelt(sval, aval); then (Values.ARRAY(reslst),st_2);
    case (env,Exp.BINARY(lh,Exp.DIV_ARRAY_SCALAR(_),rh),impl,st,dim)
      local IntegerOption dim;
      equation 
        (sval,st_1) = ceval(env, rh, impl, st, dim);
        (Values.ARRAY(aval),st_2) = ceval(env, lh, impl, st_1, dim);
        reslst = Values.div_arrayelt_scalar(sval, aval); then (Values.ARRAY(reslst),st_2);
    case (env,Exp.BINARY(lh,Exp.MUL_SCALAR_PRODUCT(_),rh),impl,st,dim)
      local
        Values.Value res;
        IntegerOption dim;
      equation 
        (Values.ARRAY(rhvals),st_1) = ceval(env, rh, impl, st, dim);
        (Values.ARRAY(lhvals),st_2) = ceval(env, lh, impl, st_1, dim);
        res = Values.mult_scalar_product(rhvals, lhvals); then (res,st_2);
    case (env,Exp.BINARY(lh,Exp.MUL_MATRIX_PRODUCT(_),rh),impl,st,dim)
      local
        Values.Value res;
        IntegerOption dim;
      equation 
        (Values.ARRAY(lhvals = elt1 :: _),st_1) = ceval(env, lh, impl, st, dim) " {{..}..{..}} * {...} ";
        (Values.ARRAY(rhvals = elt2 :: _),st_2) = ceval(env, rh, impl, st_1, dim);
        true = Values.is_array(elt1);
        false = Values.is_array(elt2);
        res = Values.mult_scalar_product(lhvals, rhvals); then (res,st_2);
    case (env,Exp.BINARY(lh,Exp.MUL_MATRIX_PRODUCT(_),rh),impl,st,dim)
      local
        Values.Value res;
        IntegerOption dim;
      equation 
        (Values.ARRAY(rhvals = elt1 :: _),st_1) = ceval(env, rh, impl, st, dim) " {...} * {{..}..{..}} ";
        (Values.ARRAY(lhvals = elt2 :: _),st_2) = ceval(env, lh, impl, st_1, dim);
        true = Values.is_array(elt1);
        false = Values.is_array(elt2);
        res = Values.mult_scalar_product(lhvals, rhvals); then (res,st_2);
    case (env,Exp.BINARY(lh,Exp.MUL_MATRIX_PRODUCT(_),rh),impl,st,dim)
      local
        Values_ValueList res;
        IntegerOption dim;
      equation 
        (Values.ARRAY(rhvals = elt1 :: _),st_1) = ceval(env, rh, impl, st, dim) " {{..}..{..}} * {{..}..{..}} ";
        (Values.ARRAY(lhvals = elt2 :: _),st_2) = ceval(env, lh, impl, st_1, dim);
        true = Values.is_array(elt1);
        true = Values.is_array(elt2);
        res = Values.mult_matrix(lhvals, rhvals); then (Values.ARRAY(res),st_2);
    case (env,Exp.BINARY(lh,Exp.ADD(Exp.INT()),rh),impl,st,dim)
      local
        Integer sum;
        IntegerOption dim;
      equation 
        (Values.INTEGER(lhv),st_1) = ceval(env, lh, impl, st, dim);
        (Values.INTEGER(rhv),st_2) = ceval(env, rh, impl, st_1, dim);
        sum = lhv + rhv; then (Values.INTEGER(sum),st_2);
    case (env,Exp.BINARY(lh,Exp.SUB(Exp.REAL()),rh),impl,st,dim) ""
      local
        Real lhv,rhv;
        IntegerOption dim;
      equation 
        (Values.REAL(lhv),st_1) = ceval(env, lh, impl, st, dim);
        (Values.REAL(rhv),st_2) = ceval(env, rh, impl, st_1, dim);
        sum = lhv -. rhv; then (Values.REAL(sum),st_2);
    case (env,Exp.BINARY(lh,Exp.SUB(Exp.INT()),rh),impl,st,dim)
      local
        Integer sum;
        IntegerOption dim;
      equation 
        (Values.INTEGER(lhv),st_1) = ceval(env, lh, impl, st, dim);
        (Values.INTEGER(rhv),st_2) = ceval(env, rh, impl, st_1, dim);
        sum = lhv - rhv; then (Values.INTEGER(sum),st_2);
    case (env,Exp.BINARY(lh,Exp.MUL(Exp.REAL()),rh),impl,st,dim) ""
      local
        Real lhv,rhv;
        IntegerOption dim;
      equation 
        (Values.REAL(lhv),st_1) = ceval(env, lh, impl, st, dim);
        (Values.REAL(rhv),st_2) = ceval(env, rh, impl, st_1, dim);
        sum = lhv*.rhv; then (Values.REAL(sum),st_2);
    case (env,Exp.BINARY(lh,Exp.DIV(Exp.REAL()),rh),impl,st,dim)
      local
        Real lhv,rhv;
        IntegerOption dim;
      equation 
        (Values.REAL(lhv),st_1) = ceval(env, lh, impl, st, dim);
        (Values.REAL(rhv),st_2) = ceval(env, rh, impl, st_1, dim);
        div = lhv/.rhv; then (Values.REAL(div),st_2);
    case (env,Exp.BINARY(lh,Exp.DIV(Exp.REAL()),rh),impl,st,dim)
      local
        Real lhv,rhv;
        IntegerOption dim;
      equation 
        (Values.REAL(lhv),st_1) = ceval(env, lh, impl, st, dim);
        (Values.REAL(rhv),st_2) = ceval(env, rh, impl, st_1, dim);
        failure(_ = real_div(lhv, rhv));
        Print.print_error_buf("#Error, division by zero.\n"); then fail();
    case (env,Exp.BINARY(lh,Exp.MUL(Exp.INT()),rh),impl,st,dim)
      local
        Integer sum;
        IntegerOption dim;
      equation 
        (Values.INTEGER(lhv),st_1) = ceval(env, lh, impl, st, dim);
        (Values.INTEGER(rhv),st_2) = ceval(env, rh, impl, st_1, dim);
        sum = lhv*rhv; then (Values.INTEGER(sum),st_2);
    case (env,Exp.UNARY(Exp.UMINUS(Exp.REAL()),exp),impl,st,dim) ""
      local
        Real x;
        Exp.Exp exp;
        IntegerOption dim;
      equation 
        (Values.REAL(x),st_1) = ceval(env, exp, impl, st, dim);
        x_1 = -.x; then (Values.REAL(x_1),st_1);
    case (env,Exp.UNARY(Exp.UMINUS(Exp.INT()),exp),impl,st,dim)
      local
        Integer x_1;
        Exp.Exp exp;
        IntegerOption dim;
      equation 
        (Values.INTEGER(x),st_1) = ceval(env, exp, impl, st, dim);
        x_1 = -x; then (Values.INTEGER(x_1),st_1);
    case (env,Exp.UNARY(Exp.UPLUS(Exp.REAL()),exp),impl,st,dim) ""
      local
        Real x;
        Exp.Exp exp;
        IntegerOption dim;
      equation 
        (Values.REAL(x),st_1) = ceval(env, exp, impl, st, dim);
        x_1 = -.x; then (Values.REAL(x_1),st_1);
    case (env,Exp.UNARY(Exp.UPLUS(Exp.INT()),exp),impl,st,dim)
      local
        Integer x_1;
        Exp.Exp exp;
        IntegerOption dim;
      equation 
        (Values.INTEGER(x),st_1) = ceval(env, exp, impl, st, dim);
        x_1 = -x; then (Values.INTEGER(x_1),st_1);
    case (env,Exp.LBINARY(lh,Exp.AND(),rh),impl,st,dim) "*  Logical "
      local
        Boolean lhv,rhv,x;
        IntegerOption dim;
      equation 
        (Values.BOOL(lhv),st_1) = ceval(env, lh, impl, st, dim);
        (Values.BOOL(rhv),st_2) = ceval(env, rh, impl, st_1, dim);
        x = bool_and(lhv, rhv); then (Values.BOOL(x),st_2);
    case (env,Exp.LBINARY(lh,Exp.OR(),rh),impl,st,dim)
      local
        Boolean lhv,rhv,x;
        IntegerOption dim;
      equation 
        (Values.BOOL(lhv),st_1) = ceval(env, lh, impl, st, dim);
        (Values.BOOL(rhv),st_2) = ceval(env, rh, impl, st_1, dim);
        x = bool_or(lhv, rhv); then (Values.BOOL(x),st_2);
    case (env,Exp.LUNARY(Exp.NOT(),e),impl,st,dim)
      local IntegerOption dim;
      equation 
        (Values.BOOL(b),st_1) = ceval(env, e, impl, st, dim);
        b_1 = bool_not(b); then (Values.BOOL(b_1),st_1);
    case (env,Exp.RELATION(lhs,relop,rhs),impl,st,dim) "*  Relations "
      local IntegerOption dim;
      equation 
        (lhs_1,st_1) = ceval(env, lhs, impl, st, dim);
        (rhs_1,st_2) = ceval(env, rhs, impl, st_1, dim);
        v = ceval_relation(lhs_1, relop, rhs_1); then (v,st_2);
    case (env,Exp.RANGE(Exp.INT(),start,NONE,stop),impl,st,dim) ""
      local IntegerOption dim;
      equation 
        (Values.INTEGER(start_1),st_1) = ceval(env, start, impl, st, dim);
        (Values.INTEGER(stop_1),st_2) = ceval(env, stop, impl, st_1, dim);
        arr = ceval_range(start_1, 1, stop_1); then (Values.ARRAY(arr),st_1);
    case (env,Exp.RANGE(Exp.INT(),start,SOME(step),stop),impl,st,dim)
      local IntegerOption dim;
      equation 
        (Values.INTEGER(start_1),st_1) = ceval(env, start, impl, st, dim);
        (Values.INTEGER(step_1),st_2) = ceval(env, step, impl, st_1, dim);
        (Values.INTEGER(stop_1),st_3) = ceval(env, stop, impl, st_2, dim);
        arr = ceval_range(start_1, step_1, stop_1); then (Values.ARRAY(arr),st_3);
    case (env,Exp.RANGE(Exp.REAL(),start,NONE,stop),impl,st,dim)
      local
        Real start_1,stop_1,step;
        IntegerOption dim;
      equation 
        (Values.REAL(start_1),st_1) = ceval(env, start, impl, st, dim);
        (Values.REAL(stop_1),st_2) = ceval(env, stop, impl, st_1, dim);
        diff = stop_1 -. start_1;
        step = int_real(1);
        arr = ceval_range_real(start_1, step, stop_1) " bug in rml, 1.0 => 0.0 in cygwin  "; then (Values.ARRAY(arr),st_2);
    case (env,Exp.RANGE(Exp.REAL(),start,SOME(step),stop),impl,st,dim)
      local
        Real start_1,step_1,stop_1;
        IntegerOption dim;
      equation 
        (Values.REAL(start_1),st_1) = ceval(env, start, impl, st, dim);
        (Values.REAL(step_1),st_2) = ceval(env, step, impl, st_1, dim);
        (Values.REAL(stop_1),st_3) = ceval(env, stop, impl, st_2, dim);
        arr = ceval_range_real(start_1, step_1, stop_1); then (Values.ARRAY(arr),st_3);
    case (env,Exp.CAST(Exp.REAL(),e),impl,st,dim)
      local IntegerOption dim;
      equation 
        (Values.INTEGER(i),st_1) = ceval(env, e, impl, st, dim);
        r = int_real(i); then (Values.REAL(r),st_1);
    case (env,Exp.CAST(Exp.REAL(),e),impl,st,dim)
      local IntegerOption dim;
      equation 
        (Values.ARRAY(ivals),st_1) = ceval(env, e, impl, st, dim);
        rvals = Values.type_convert(Exp.INT(), Exp.REAL(), ivals); then (Values.ARRAY(rvals),st_1);
    case (env,Exp.CAST(Exp.REAL(),(e = Exp.ARRAY(Exp.INT(),_,expl))),impl,st,dim)
      local IntegerOption dim;
      equation 
        (Values.ARRAY(vallst),st_1) = ceval(env, e, impl, st, dim);
        vallst_1 = Values.type_convert(Exp.INT(), Exp.REAL(), vallst); then (Values.ARRAY(vallst_1),st_1);
    case (env,Exp.CAST(Exp.REAL(),(e = Exp.RANGE(Exp.INT(),_,_,_))),impl,st,dim)
      local IntegerOption dim;
      equation 
        (Values.ARRAY(vallst),st_1) = ceval(env, e, impl, st, dim);
        vallst_1 = Values.type_convert(Exp.INT(), Exp.REAL(), vallst); then (Values.ARRAY(vallst_1),st_1);
    case (env,Exp.CAST(Exp.REAL(),(e = Exp.MATRIX(Exp.INT(),_,_))),impl,st,dim)
      local IntegerOption dim;
      equation 
        (Values.ARRAY(vallst),st_1) = ceval(env, e, impl, st, dim);
        vallst_1 = Values.type_convert(Exp.INT(), Exp.REAL(), vallst); then (Values.ARRAY(vallst_1),st_1);
    case (env,Exp.IFEXP(b,e1,e2),impl,st,dim)
      local
        Exp.Exp b;
        IntegerOption dim;
      equation 
        (Values.BOOL(true),st_1) = ceval(env, b, impl, st, dim) " Ifexp, true branch ";
        (v,st_2) = ceval(env, e1, impl, st_1, dim); then (v,st_2);
    case (env,Exp.IFEXP(b,e1,e2),impl,st,dim)
      local
        Exp.Exp b;
        IntegerOption dim;
      equation 
        (Values.BOOL(false),st_1) = ceval(env, b, impl, st, dim) " Ifexp, false branch ";
        (v,st_2) = ceval(env, e2, impl, st_1, dim); then (v,st_2);
    case (env,Exp.ASUB(e,indx),impl,st,dim)
      local IntegerOption dim;
      equation 
        (Values.ARRAY(vals),st_1) = ceval(env, e, impl, st, dim) " asub ";
        indx_1 = indx - 1;
        v = list_nth(vals, indx_1); then (v,st_1);
    case (env,Exp.REDUCTION(p,exp,iter,iterexp),impl,st,dim)
      local
        Absyn.Path p;
        Exp.Exp exp;
        IntegerOption dim;
      equation 
        Print.print_buf("#-- ceval reduction\n"); then fail();
    case (env,e,_,_,_) "   ceval can apparently fail and that is ok, catched by other rules..."
      equation 
        Debug.fprint("failtrace", "- ceval failed: ") " (v,st) ";
        str = Exp.print_exp_str(e);
        Debug.fprint("failtrace", str);
        Debug.fprint("failtrace", "\n"); then fail();
  end matchcontinue;
end ceval;

protected function ceval_call_function
  input Env.Env in_env;
  input Exp.Exp in_exp;
  input Values_ValueList in_values_valuelist;
  output Values.Value out_value;
protected 
  type Values_ValueList = list<Values.Value>;
algorithm 
  out_value:=
  matchcontinue (in_env,in_exp,in_values_valuelist)
    local
      type Env_FrameList = list<Env.Frame>;
      type Exp_ExpList = list<Exp.Exp>;
      String funcstr,infilename,outfilename,str;
      Values.Value newval;
      Env_FrameList env;
      Exp.Exp e;
      Absyn.Path funcpath;
      Exp_ExpList expl;
      Boolean builtin;
      Values_ValueList vallst;
    case (env,(e = Exp.CALL(funcpath,expl,_,builtin)),vallst) " Call functions in non-interactive mode.   FIXME: functions are always generated. Put back the check
	           and write another rule for the false case that generates the function "
      equation 
        ceval_generate_function(env, funcpath) " Static.is_function_in_cflist(cflist,funcpath) => true & ";
        funcstr = ModUtil.path_string2(funcpath, "_");
        infilename = string_append(funcstr, "_in.txt");
        outfilename = string_append(funcstr, "_out.txt");
        Values.write_to_file_as_args(vallst, infilename);
        System.execute_function(funcstr);
        newval = System.read_values_from_file(outfilename); then newval;
    case (env,e,_)
      equation 
        Debug.fprint("failtrace", "- ceval_call_function failed: ");
        str = Exp.print_exp_str(e);
        Debug.fprintln("failtrace", str); then fail();
  end matchcontinue;
end ceval_call_function;

protected function ceval_function
  input Env.Env in_env;
  input Absyn.Path in_path;
  input Values_ValueList in_values_valuelist;
  input Boolean in_boolean;
  output Values.Value out_value;
protected 
  type Values_ValueList = list<Values.Value>;
algorithm 
  out_value:=
  matchcontinue (in_env,in_path,in_values_valuelist,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      type StringList = list<String>;
      type DAE_ElementList = list<DAE.Element>;
      SCode.Class c;
      Env_FrameList env_1,env;
      StringList compnames;
      Types.Mod mod;
      DAE_ElementList dae;
      String s;
      Values.Value value;
      Absyn.Path funcname;
      Values_ValueList vallst;
      Boolean impl;
    case (env,funcname,vallst,impl) "impl  For record constructors "
      equation 
        (_,_) = Lookup.lookup_record_constructor_class(env, funcname);
        (c,env_1) = Lookup.lookup_class(env, funcname, false);
        compnames = SCode.component_names(c);
        mod = Types.values_to_mods(vallst, compnames);
        (dae,_,_,_,_) = Inst.inst_class(env_1, mod, Prefix.NOPRE(), {}, c, {}, impl, Inst.TOP_CALL());
        DAE.dump_elements(dae);
        s = Print.get_string();
        print(s);
        print("\n");
        value = DAE.dae_to_record_value(funcname, dae, impl); then value;
    case (env,funcname,vallst,(impl = true))
      equation 
        Debug.fprint("failtrace", 
          "ceval_function: Don't know what to do. impl was always false before:");
        s = Absyn.path_string(funcname);
        Debug.fprintln("failtrace", s); then fail();
  end matchcontinue;
end ceval_function;

protected function ceval_ast_exp "impl"
  input Env.Env in_env;
  input Absyn.Exp in_exp;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Absyn.Exp out_exp;
protected 
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  out_exp:=
  matchcontinue (in_env,in_exp,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      type TupleAbsyn_ExpAbsyn_Exp = tuple<Absyn.Exp,Absyn.Exp>;
      type TupleAbsyn_ExpAbsyn_ExpList = list<TupleAbsyn_ExpAbsyn_Exp>;
      type Absyn_ExpList = list<Absyn.Exp>;
      type Absyn_ExpListList = list<Absyn_ExpList>;
      Absyn.Exp e,e1_1,e2_1,e1,e2,e_1,cond_1,then_1,else_1,cond,then_,else_,exp,e3_1,e3;
      Env_FrameList env;
      Absyn.Operator op;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st;
      TupleAbsyn_ExpAbsyn_ExpList nest_1,nest;
      Absyn.ComponentRef cr;
      Absyn.FunctionArgs fa;
      Absyn_ExpList expl_1,expl;
    case (_,(e = Absyn.INTEGER(_)),_,_) then e; 
    case (_,(e = Absyn.REAL(_)),_,_) then e; 
    case (_,(e = Absyn.CREF(_)),_,_) then e; 
    case (_,(e = Absyn.STRING(_)),_,_) then e; 
    case (_,(e = Absyn.BOOL(_)),_,_) then e; 
    case (env,Absyn.BINARY(e1,op,e2),impl,st)
      equation 
        e1_1 = ceval_ast_exp(env, e1, impl, st);
        e2_1 = ceval_ast_exp(env, e2, impl, st); then Absyn.BINARY(e1_1,op,e2_1);
    case (env,Absyn.UNARY(op,e),impl,st)
      equation 
        e_1 = ceval_ast_exp(env, e, impl, st); then Absyn.UNARY(op,e_1);
    case (env,Absyn.LBINARY(e1,op,e2),impl,st)
      equation 
        e1_1 = ceval_ast_exp(env, e1, impl, st);
        e2_1 = ceval_ast_exp(env, e2, impl, st); then Absyn.LBINARY(e1_1,op,e2_1);
    case (env,Absyn.LUNARY(op,e),impl,st)
      equation 
        e_1 = ceval_ast_exp(env, e, impl, st); then Absyn.LUNARY(op,e_1);
    case (env,Absyn.RELATION(e1,op,e2),impl,st)
      equation 
        e1_1 = ceval_ast_exp(env, e1, impl, st);
        e2_1 = ceval_ast_exp(env, e2, impl, st); then Absyn.RELATION(e1_1,op,e2_1);
    case (env,Absyn.IFEXP(cond,then_,else_,nest),impl,st)
      equation 
        cond_1 = ceval_ast_exp(env, cond, impl, st);
        then_1 = ceval_ast_exp(env, then_, impl, st);
        else_1 = ceval_ast_exp(env, else_, impl, st);
        nest_1 = ceval_ast_expexp_list(env, nest, impl, st); then Absyn.IFEXP(cond_1,then_1,else_1,nest_1);
    case (env,Absyn.CALL(Absyn.CREF_IDENT("Eval",{}),Absyn.FUNCTIONARGS({e},{})),impl,st)
      local Exp.Exp e_1;
      equation 
        (e_1,_,_) = Static.elab_exp(env, e, impl, st);
        (Values.CODE(Absyn.C_EXPRESSION(exp)),_) = ceval(env, e_1, impl, st, NONE); then exp;
    case (env,(e = Absyn.CALL(cr,fa)),_,_) then e; 
    case (env,Absyn.ARRAY(expl),impl,st)
      equation 
        expl_1 = ceval_ast_exp_list(env, expl, impl, st); then Absyn.ARRAY(expl_1);
    case (env,Absyn.MATRIX(expl),impl,st)
      local Absyn_ExpListList expl_1,expl;
      equation 
        expl_1 = ceval_ast_exp_list_list(env, expl, impl, st); then Absyn.MATRIX(expl_1);
    case (env,Absyn.RANGE(e1,SOME(e2),e3),impl,st)
      equation 
        e1_1 = ceval_ast_exp(env, e1, impl, st);
        e2_1 = ceval_ast_exp(env, e2, impl, st);
        e3_1 = ceval_ast_exp(env, e3, impl, st); then Absyn.RANGE(e1_1,SOME(e2_1),e3_1);
    case (env,Absyn.RANGE(e1,NONE,e3),impl,st)
      equation 
        e1_1 = ceval_ast_exp(env, e1, impl, st);
        e3_1 = ceval_ast_exp(env, e3, impl, st); then Absyn.RANGE(e1_1,NONE,e3_1);
    case (env,Absyn.TUPLE(expl),impl,st)
      equation 
        expl_1 = ceval_ast_exp_list(env, expl, impl, st); then Absyn.TUPLE(expl_1);
    case (env,Absyn.END,_,_) then Absyn.END; 
    case (env,(e = Absyn.CODE(_)),_,_) then e; 
  end matchcontinue;
end ceval_ast_exp;

protected function ceval_ast_exp_list "impl"
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Absyn_ExpList out_absyn_explist;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  out_absyn_explist:=
  matchcontinue (in_env,in_absyn_explist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      Env_FrameList env;
      Absyn.Exp e_1,e;
      Absyn_ExpList res,es;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st;
    case (env,{},_,_) then {}; 
    case (env,e :: es,impl,st)
      equation 
        e_1 = ceval_ast_exp(env, e, impl, st);
        res = ceval_ast_exp_list(env, es, impl, st); then e :: res;
  end matchcontinue;
end ceval_ast_exp_list;

protected function ceval_ast_exp_list_list "impl"
  input Env.Env in_env;
  input Absyn_ExpListList in_absyn_explistlist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Absyn_ExpListList out_absyn_explistlist;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
  type Absyn_ExpListList = list<Absyn_ExpList>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  out_absyn_explistlist:=
  matchcontinue (in_env,in_absyn_explistlist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      Env_FrameList env;
      Absyn_ExpList e_1,e;
      Absyn_ExpListList res,es;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st;
    case (env,{},_,_) then {}; 
    case (env,e :: es,impl,st)
      equation 
        e_1 = ceval_ast_exp_list(env, e, impl, st);
        res = ceval_ast_exp_list_list(env, es, impl, st); then e :: res;
  end matchcontinue;
end ceval_ast_exp_list_list;

protected function ceval_ast_expexp_list
  input Env.Env in_env;
  input TupleAbsyn_ExpAbsyn_ExpList in_tupleabsyn_expabsyn_explist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output TupleAbsyn_ExpAbsyn_ExpList out_tupleabsyn_expabsyn_explist;
protected 
  type TupleAbsyn_ExpAbsyn_Exp = tuple<Absyn.Exp,Absyn.Exp>;
  type TupleAbsyn_ExpAbsyn_ExpList = list<TupleAbsyn_ExpAbsyn_Exp>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  out_tupleabsyn_expabsyn_explist:=
  matchcontinue (in_env,in_tupleabsyn_expabsyn_explist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      Absyn.Exp e1_1,e2_1,e1,e2;
      TupleAbsyn_ExpAbsyn_ExpList res,xs;
      Env_FrameList env;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st;
    case (_,{},_,_) then {}; 
    case (env,(e1,e2) :: xs,impl,st)
      equation 
        e1_1 = ceval_ast_exp(env, e1, impl, st);
        e2_1 = ceval_ast_exp(env, e2, impl, st);
        res = ceval_ast_expexp_list(env, xs, impl, st); then (e1_1,e2_1) :: res;
  end matchcontinue;
end ceval_ast_expexp_list;

protected function ceval_ast_elt "impl"
  input Env.Env in_env;
  input Absyn.Element in_element;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Absyn.Element out_element;
protected 
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  out_element:=
  matchcontinue (in_env,in_element,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
      type Env_FrameList = list<Env.Frame>;
      type Absyn_ElementSpecOption = Option<Absyn.ElementSpec>;
      Absyn_ComponentItemList citems_1,citems;
      Env_FrameList env;
      Boolean f,r,impl;
      Absyn.InnerOuter io;
      String id,file;
      Absyn.ElementAttributes attr;
      Absyn.Path tp;
      Integer line;
      Absyn_ElementSpecOption c;
      Interactive_InteractiveSymbolTableOption st;
    case (env,Absyn.ELEMENT(f,r,io,id,Absyn.COMPONENTS(attr,tp,citems),file,line,c),impl,st) "impl"
      equation 
        citems_1 = ceval_ast_citems(env, citems, impl, st); then Absyn.ELEMENT(f,r,io,id,Absyn.COMPONENTS(attr,tp,citems_1),file,line,c);
  end matchcontinue;
end ceval_ast_elt;

protected function ceval_ast_citems "impl"
  input Env.Env in_env;
  input Absyn_ComponentItemList in_absyn_componentitemlist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Absyn_ComponentItemList out_absyn_componentitemlist;
protected 
  type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  out_absyn_componentitemlist:=
  matchcontinue (in_env,in_absyn_componentitemlist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Absyn_ModificationOption = Option<Absyn.Modification>;
      type Absyn_SubscriptList = list<Absyn.Subscript>;
      type Env_FrameList = list<Env.Frame>;
      type Absyn_CommentOption = Option<Absyn.Comment>;
      Absyn_ComponentItemList res,xs;
      Absyn_ModificationOption modopt_1,modopt;
      Absyn_SubscriptList ad_1,ad;
      Env_FrameList env;
      String id;
      Absyn_CommentOption cmt;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st;
      Absyn.ComponentItem x;
    case (_,{},_,_) then {}; 
    case (env,Absyn.COMPONENTITEM(Absyn.COMPONENT(id,ad,modopt),cmt) :: xs,impl,st) " If one component fails, the rest should still succeed "
      equation 
        res = ceval_ast_citems(env, xs, impl, st);
        modopt_1 = ceval_ast_modopt(env, modopt, impl, st);
        ad_1 = ceval_ast_arraydim(env, ad, impl, st); then Absyn.COMPONENTITEM(Absyn.COMPONENT(id,ad_1,modopt_1),cmt) :: res;
    case (env,x :: xs,impl,st) " If one component fails, the rest should still succeed "
      equation 
        res = ceval_ast_citems(env, xs, impl, st); then x :: res;
  end matchcontinue;
end ceval_ast_citems;

protected function ceval_ast_modopt "impl"
  input Env.Env in_env;
  input Absyn_ModificationOption in_absyn_modificationoption;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Absyn_ModificationOption out_absyn_modificationoption;
protected 
  type Absyn_ModificationOption = Option<Absyn.Modification>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  out_absyn_modificationoption:=
  matchcontinue (in_env,in_absyn_modificationoption,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      Absyn.Modification res,mod;
      Env_FrameList env;
      Boolean st;
      Interactive_InteractiveSymbolTableOption impl;
    case (env,SOME(mod),st,impl) "impl"
      equation 
        res = ceval_ast_modification(env, mod, st, impl); then SOME(res);
    case (env,NONE,_,_) then NONE; 
  end matchcontinue;
end ceval_ast_modopt;

protected function ceval_ast_modification "impl"
  input Env.Env in_env;
  input Absyn.Modification in_modification;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Absyn.Modification out_modification;
protected 
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  out_modification:=
  matchcontinue (in_env,in_modification,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Absyn_ElementArgList = list<Absyn.ElementArg>;
      type Env_FrameList = list<Env.Frame>;
      Absyn.Exp e_1,e;
      Absyn_ElementArgList eltargs_1,eltargs;
      Env_FrameList env;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st;
    case (env,Absyn.CLASSMOD(eltargs,SOME(e)),impl,st) "impl"
      equation 
        e_1 = ceval_ast_exp(env, e, impl, st);
        eltargs_1 = ceval_ast_eltargs(env, eltargs, impl, st); then Absyn.CLASSMOD(eltargs_1,SOME(e_1));
    case (env,Absyn.CLASSMOD(eltargs,NONE),impl,st)
      equation 
        eltargs_1 = ceval_ast_eltargs(env, eltargs, impl, st); then Absyn.CLASSMOD(eltargs_1,NONE);
  end matchcontinue;
end ceval_ast_modification;

protected function ceval_ast_eltargs
  input Env.Env in_env;
  input Absyn_ElementArgList in_absyn_elementarglist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Absyn_ElementArgList out_absyn_elementarglist;
protected 
  type Absyn_ElementArgList = list<Absyn.ElementArg>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  out_absyn_elementarglist:=
  matchcontinue (in_env,in_absyn_elementarglist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      type StringOption = Option<String>;
      Env_FrameList env;
      Absyn.Modification mod_1,mod;
      Absyn_ElementArgList res,args;
      Boolean b,impl;
      Absyn.Each e;
      Absyn.ComponentRef cr;
      StringOption stropt;
      Interactive_InteractiveSymbolTableOption st;
      Absyn.ElementArg m;
    case (env,{},_,_) then {}; 
    case (env,Absyn.MODIFICATION(b,e,cr,SOME(mod),stropt) :: args,impl,st) "*TODO: look through redeclarations for Eval(var) as well *"
      equation 
        mod_1 = ceval_ast_modification(env, mod, impl, st);
        res = ceval_ast_eltargs(env, args, impl, st); then Absyn.MODIFICATION(b,e,cr,SOME(mod_1),stropt) :: res;
    case (env,m :: args,impl,st) "*TODO: look through redeclarations for Eval(var) as well *"
      equation 
        res = ceval_ast_eltargs(env, args, impl, st); then m :: res;
  end matchcontinue;
end ceval_ast_eltargs;

protected function ceval_ast_arraydim "impl"
  input Env.Env in_env;
  input Absyn.ArrayDim in_arraydim;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Absyn.ArrayDim out_arraydim;
protected 
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  out_arraydim:=
  matchcontinue (in_env,in_arraydim,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      type Absyn_SubscriptList = list<Absyn.Subscript>;
      Env_FrameList env;
      Absyn_SubscriptList res,xs;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st;
      Absyn.Exp e_1,e;
    case (env,{},_,_) then {}; 
    case (env,Absyn.NOSUB :: xs,impl,st)
      equation 
        res = ceval_ast_arraydim(env, xs, impl, st); then Absyn.NOSUB :: res;
    case (env,Absyn.SUBSCRIPT(e) :: xs,impl,st)
      equation 
        res = ceval_ast_arraydim(env, xs, impl, st);
        e_1 = ceval_ast_exp(env, e, impl, st); then Absyn.SUBSCRIPT(e) :: res;
  end matchcontinue;
end ceval_ast_arraydim;

protected function ceval_interactive_functions
  input Env.Env in_env;
  input Exp.Exp in_exp;
  input Interactive.InteractiveSymbolTable in_interactivesymboltable;
  output Values.Value out_value;
  output Interactive.InteractiveSymbolTable out_interactivesymboltable;
algorithm 
  (out_value,out_interactivesymboltable):=
  matchcontinue (in_env,in_exp,in_interactivesymboltable)
    local
      type SCode_ClassList = list<SCode.Class>;
      type Env_FrameList = list<Env.Frame>;
      type Interactive_InstantiatedClassList = list<Interactive.InstantiatedClass>;
      type Interactive_InteractiveVariableList = list<Interactive.InteractiveVariable>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOption = tuple<Absyn.Path,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOptionList = list<TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOption>;
      type DAE_ElementList = list<DAE.Element>;
      type IntegerList = list<Integer>;
      type IntegerListArray = IntegerList[:];
      type TupleIntegerIntegerDAELow_Equation = tuple<Integer,Integer,DAELow.Equation>;
      type TupleIntegerIntegerDAELow_EquationList = list<TupleIntegerIntegerDAELow_Equation>;
      type IntegerVector = Integer[:];
      type IntegerListList = list<IntegerList>;
      type StringList = list<String>;
      type Exp_ExpList = list<Exp.Exp>;
      Absyn.Path path,p1,classname,classpath;
      SCode_ClassList p_1,sp,fp;
      Env_FrameList env;
      SCode.Class c;
      String s1,str,varid,res,omhome,pathstr,cname_str,filename,funcfilename,init_filename,makefilename,pd,s_call,sim_call,result_file,pwd,filename_1,plotCmd,tmpPlotFile,call,str_1,scriptstr,msg,res_1,mp,name;
      Exp.ComponentRef cr,fcr,cref;
      Interactive.InteractiveSymbolTable st,newst,st_1;
      Absyn.Program p,pnew,newp;
      Interactive_InstantiatedClassList ic,ic_1;
      Interactive_InteractiveVariableList iv;
      TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOptionList cf;
      TupleTypes_TTypeAbsyn_PathOption tp,simType;
      Absyn.Class class_;
      DAE.DAElist dae;
      DAE_ElementList dael;
      DAELow.DAELow daelow,dlow,dlow_1,indexed_dlow,indexed_dlow_1;
      DAELow.Variables vars;
      DAELow.EquationArray eqnarr;
      IntegerListArray m,mt,mT;
      TupleIntegerIntegerDAELow_EquationList jac;
      Real starttime_r,stoptime_r,delta_time,interval_r,stepsize_r,t1,t2,time;
      Integer interval_i,size,rest;
      IntegerVector ass1,ass2;
      IntegerListList comps;
      Values.Value simValue,value,v;
      Exp.Exp starttime,stoptime,interval,exp,funcref,exp1,exp2,res1,exp3;
      StringList vars_1,vars_2,args;
      Interactive.InteractiveStmts istmts;
    case (env,Exp.CALL(Absyn.IDENT("lookupClass"),{Exp.CREF(cr,_)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        path = Static.component_ref_to_path(cr);
        p_1 = SCode.elaborate(p);
        env = Inst.make_env_from_program(p_1, Absyn.IDENT(""));
        (c,env) = Lookup.lookup_class(env, path, true);
        SOME(p1) = Env.get_env_path(env);
        s1 = ModUtil.path_string(p1);
        Print.print_buf("Found class ");
        Print.print_buf(s1);
        Print.print_buf("\n\n");
        str = Print.get_string(); then (Values.STRING(str),st);
    case (env,Exp.CALL(Absyn.IDENT("typeOf"),{Exp.CREF(Exp.CREF_IDENT(varid,_),_)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        tp = Interactive.get_type_of_variable(varid, iv);
        str = Types.unparse_type(tp); then (Values.STRING(str),st);
    case (env,Exp.CALL(Absyn.IDENT("clear"),{},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        newst = Interactive.SYMBOLTABLE(Absyn.PROGRAM({},Absyn.TOP),{},{},{},{}); then (Values.BOOL(true),newst);
    case (env,Exp.CALL(Absyn.IDENT("clearVariables"),{},_,_),(st = Interactive.SYMBOLTABLE(p,fp,ic,iv,cf)))
      equation 
        newst = Interactive.SYMBOLTABLE(p,fp,ic,{},cf); then (Values.BOOL(true),newst);
    case (env,Exp.CALL(Absyn.IDENT("clearCache"),{},_,_),(st = Interactive.SYMBOLTABLE(p,fp,ic,iv,cf)))
      equation 
        newst = Interactive.SYMBOLTABLE(p,fp,{},iv,cf); then (Values.BOOL(true),newst);
    case (env,Exp.CALL(Absyn.IDENT("list"),{},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        str = Dump.unparse_str(p) " ,false "; then (Values.STRING(str),st);
    case (env,Exp.CALL(Absyn.IDENT("list"),{Exp.CREF(cr,_)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        path = Static.component_ref_to_path(cr);
        class_ = Interactive.get_pathed_class_in_program(path, p);
        str = Dump.unparse_str(Absyn.PROGRAM({class_},Absyn.TOP)) " ,false "; then (Values.STRING(str),st);
    case (env,Exp.CALL(Absyn.IDENT("jacobian"),{Exp.CREF(cr,_)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        path = Static.component_ref_to_path(cr);
        p_1 = SCode.elaborate(p);
        (dae = DAE.DAE(dael),env) = Inst.instantiate_class(p_1, path);
        ic_1 = Interactive.add_instantiated_class(ic, Interactive.INSTCLASS(path,dael,env));
        (daelow = DAELow.DAELOW(vars,_,eqnarr,_,_,_,_)) = DAELow.lower(dae);
        m = DAELow.incidence_matrix(daelow);
        mt = DAELow.transpose_matrix(m);
        jac = DAELow.calculate_jacobian(vars, eqnarr, m, mt);
        res = DAELow.dump_jacobian_str(jac); then (Values.STRING(res),Interactive.SYMBOLTABLE(p,sp,ic_1,iv,cf));
    case (env,Exp.CALL(Absyn.IDENT("translateModel"),{Exp.CREF(cr,_)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        omhome = System.read_env("OPENMODELICAHOME");
        path = Static.component_ref_to_path(cr);
        pathstr = Absyn.path_string(path);
        Print.print_error_buf("#Failed to instantiate ");
        Print.print_error_buf(pathstr);
        Print.print_error_buf(" .\n");
        str = Print.get_error_string(); then (Values.STRING(str),st);
    case (env,Exp.CALL(Absyn.IDENT("translateModel"),{Exp.CREF(cr,_)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf))) then (Values.STRING("The environment variable OPENMODELICAHOME was not set"),st); 
    case (env,Exp.CALL(Absyn.IDENT("simulate"),{Exp.CREF(cr,_),starttime,stoptime,interval},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf))) " FIXME: Should ceval be called with impl=true here? "
      equation 
        (Values.REAL(starttime_r),SOME(st)) = ceval(env, starttime, true, SOME(st), NONE) " Build and simulate model ";
        (Values.REAL(stoptime_r),SOME(st)) = ceval(env, stoptime, true, SOME(st), NONE);
        (Values.INTEGER(interval_i),SOME(st)) = ceval(env, interval, true, SOME(st), NONE);
        delta_time = stoptime_r -. starttime_r;
        interval_r = int_real(interval_i);
        stepsize_r = delta_time/.interval_r;
        classname = Static.component_ref_to_path(cr);
        p_1 = SCode.elaborate(p);
        (dae = DAE.DAE(dael),env) = Inst.instantiate_class(p_1, classname);
        ic_1 = Interactive.add_instantiated_class(ic, Interactive.INSTCLASS(classname,dael,env));
        dlow = DAELow.lower(dae);
        m = DAELow.incidence_matrix(dlow);
        mT = DAELow.transpose_matrix(m);
        (ass1,ass2,dlow_1,m,mT) = DAELow.matching_algorithm(dlow, m, mT);
        (comps) = DAELow.strong_components(m, mT, ass1, ass2);
        indexed_dlow = DAELow.translate_dae(dlow_1);
        indexed_dlow_1 = DAELow.calculate_values(indexed_dlow);
        cname_str = Absyn.path_string(classname);
        filename = Util.string_append_list({cname_str,".cpp"});
        funcfilename = Util.string_append_list({cname_str,"_functions.cpp"});
        init_filename = Util.string_append_list({cname_str,"_init.txt"});
        makefilename = Util.string_append_list({cname_str,".makefile"});
        SimCodegen.generate_functions(dae, indexed_dlow_1, classname, funcfilename);
        SimCodegen.generate_simulation_code(dae, indexed_dlow_1, ass1, ass2, m, mT, comps, classname, 
          filename);
        SimCodegen.generate_init_data(indexed_dlow_1, classname, init_filename, starttime_r, 
          stoptime_r, stepsize_r);
        SimCodegen.generate_makefile(makefilename, classname);
        pd = System.path_delimiter() "	Util.string_append_list({\"make -f \",cname_str, \".makefile\n\"}) => s_call &
          ";
        omhome = System.read_env("OPENMODELICAHOME");
        s_call = Util.string_append_list(
          {omhome,pd,"Compiler",pd,"scripts",pd,"Compile ",
          makefilename});
        0 = System.system_call(s_call);
        sim_call = Util.string_append_list({".",pd,cname_str});
        _ = System.system_call(sim_call);
        result_file = Util.string_append_list({cname_str,"_res.plt"});
        simValue = Values.RECORD(Absyn.IDENT("SimulationResult"),
          {Values.STRING(result_file)},{"resultFile"});
        simType = (
          Types.T_COMPLEX(ClassInf.RECORD("SimulationResult"),
          {
          Types.VAR("resultFile",
          Types.ATTR(false,SCode.RO,SCode.VAR,Absyn.BIDIR),false,(Types.T_STRING({}),NONE),Types.UNBOUND())}),NONE);
        newst = Interactive.add_var_to_symboltable("currentSimulationResult", simValue, simType, st);
        Print.clear_error_buf(); then (simValue,newst);
    case (env,Exp.CALL(Absyn.IDENT("simulate"),{Exp.CREF(cr,_),starttime,stoptime,interval},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        omhome = System.read_env("OPENMODELICAHOME") " simulation fail for som other reason than OPENMODELICAHOME not being set.";
        str = Print.get_error_string();
        res = Util.string_append_list({"Simulation failed.\n",str,"\n"});
        simValue = Values.RECORD(Absyn.IDENT("SimulationResult"),{Values.STRING(res)},
          {"resultFile"}); then (simValue,st);
    case (env,Exp.CALL(Absyn.IDENT("simulate"),{Exp.CREF(cr,_),starttime,stoptime,interval},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        simValue = Values.RECORD(Absyn.IDENT("SimulationResult"),
          {
          Values.STRING(
          "Simulation Failed. Environment variable OPENMODELICAHOME not set.")},{"resultFile"}); then (simValue,st);
    case (env,Exp.CALL(Absyn.IDENT("instantiateModel"),{Exp.CREF(cr,_)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        path = Static.component_ref_to_path(cr);
        p_1 = SCode.elaborate(p);
        (dae = DAE.DAE(dael),env) = Inst.instantiate_class(p_1, path);
        ic_1 = Interactive.add_instantiated_class(ic, Interactive.INSTCLASS(path,dael,env));
        str = DAE.dump_str(dae); then (Values.STRING(str),Interactive.SYMBOLTABLE(p,sp,ic_1,iv,cf));
    case (env,Exp.CALL(Absyn.IDENT("instantiateModel"),{Exp.CREF(cr,_)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        path = Static.component_ref_to_path(cr);
        p_1 = SCode.elaborate(p);
        str = Print.get_error_string() " we do not want error msg twice.. ";
        failure((dae = DAE.DAE(dael),env) = Inst.instantiate_class(p_1, path));
        Print.clear_error_buf();
        Print.print_error_buf(str);
        str = Print.get_error_string(); then (Values.STRING(str),Interactive.SYMBOLTABLE(p,sp,ic,iv,cf));
    case (env,Exp.CALL(Absyn.IDENT("readSimulationResult"),{Exp.SCONST(filename),Exp.ARRAY(_,_,vars),Exp.ICONST(size)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      local Exp_ExpList vars;
      equation 
        vars_1 = Util.list_map(vars, Exp.print_exp_str);
        pwd = System.pwd();
        pd = System.path_delimiter();
        filename_1 = Util.string_append_list({pwd,pd,filename});
        value = System.read_ptolemyplot_dataset(filename_1, vars_1, size); then (value,st);
    case (env,Exp.CALL(Absyn.IDENT("readSimulationResult"),{Exp.SCONST(filename),Exp.ARRAY(_,_,vars),Exp.ICONST(size)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      local Exp_ExpList vars;
      equation 
        Print.print_error_buf(
          "#Error reading simulation result. Check that filename and variablenames are correct.\n"); then fail();
    case (env,Exp.CALL(Absyn.IDENT("plot"),{Exp.ARRAY(_,_,vars)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      local
        Integer res;
        Exp_ExpList vars;
      equation 
        vars_1 = Util.list_map(vars, Exp.print_exp_str) " plot ";
        vars_2 = Util.list_union_elt("time", vars_1);
        (Values.RECORD(_,{Values.STRING(filename)},_),_) = ceval(env, 
          Exp.CREF(Exp.CREF_IDENT("currentSimulationResult",{}),Exp.OTHER()), true, SOME(st), NONE);
        value = System.read_ptolemyplot_dataset(filename, vars_2, 0);
        pwd = System.pwd();
        omhome = System.read_env("OPENMODELICAHOME");
        pd = System.path_delimiter();
        plotCmd = Util.string_append_list({omhome,pd,"Compiler",pd,"scripts",pd,"doPlot"});
        tmpPlotFile = Util.string_append_list({pwd,pd,"tmpPlot.plt"});
        res = Values.write_ptolemyplot_dataset(tmpPlotFile, value, vars_2, "Plot by OpenModelica");
        call = Util.string_append_list({plotCmd," \"",tmpPlotFile,"\""});
        _ = System.system_call(call); then (Values.BOOL(true),st);
    case (env,Exp.CALL(Absyn.IDENT("plot"),{Exp.ARRAY(_,_,vars)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      local Exp_ExpList vars;
      equation 
        vars_1 = Util.list_map(vars, Exp.print_exp_str) " Catch error reading simulation file. ";
        vars_2 = Util.list_union_elt("time", vars_1);
        (Values.RECORD(_,{Values.STRING(filename)},_),_) = ceval(env, 
          Exp.CREF(Exp.CREF_IDENT("currentSimulationResult",{}),Exp.OTHER()), true, SOME(st), NONE);
        failure(_ = System.read_ptolemyplot_dataset(filename, vars_2, 0)); then (Values.STRING("Error reading the simulation result."),st);
    case (env,Exp.CALL(Absyn.IDENT("plot"),{Exp.ARRAY(_,_,vars)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      local Exp_ExpList vars;
      equation 
        vars_1 = Util.list_map(vars, Exp.print_exp_str) " Catch error reading simulation file. ";
        vars_2 = Util.list_union_elt("time", vars_1);
        failure((_,_) = ceval(env, 
          Exp.CREF(Exp.CREF_IDENT("currentSimulationResult",{}),Exp.OTHER()), true, SOME(st), NONE)); then (Values.STRING("No simulation result to plot."),st);
    case (env,Exp.CALL(Absyn.IDENT("plot"),{Exp.ARRAY(_,_,vars)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      local Exp_ExpList vars; then (Values.STRING("Unknown error while plotting"),st);
    case (env,Exp.CALL(Absyn.IDENT("timing"),{exp},_,_),st)
      equation 
        t1 = System.time();
        (value,SOME(st_1)) = ceval(env, exp, true, SOME(st), NONE);
        t2 = System.time();
        time = t2 -. t1; then (Values.REAL(time),st_1);
    case (env,Exp.CALL(Absyn.IDENT("setCompiler"),{Exp.SCONST(str)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        System.set_c_compiler(str); then (Values.BOOL(true),st);
    case (env,Exp.CALL(Absyn.IDENT("setCompilerFlags"),{Exp.SCONST(str)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        System.set_c_flags(str); then (Values.BOOL(true),st);
    case (env,Exp.CALL(Absyn.IDENT("setDebugFlags"),{Exp.SCONST(str)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        str_1 = string_append("+d=", str);
        args = RTOpts.args({str_1}); then (Values.BOOL(true),st);
    case (env,Exp.CALL(Absyn.IDENT("cd"),{Exp.SCONST(str)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      local Integer res;
      equation 
        res = System.cd(str);
        (res == 0) = true;
        str_1 = System.pwd(); then (Values.STRING(str_1),st);
    case (env,Exp.CALL(Absyn.IDENT("cd"),{},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        str_1 = System.pwd(); then (Values.STRING(str_1),st);
    case (env,Exp.CALL(Absyn.IDENT("system"),{Exp.SCONST(str)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      local Integer res;
      equation 
        res = System.system_call(str); then (Values.INTEGER(res),st);
    case (env,Exp.CALL(Absyn.IDENT("readFile"),{Exp.SCONST(str)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        str_1 = System.read_file(str); then (Values.STRING(str_1),st);
    case (env,Exp.CALL(Absyn.IDENT("getErrorString"),{},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        str = Print.get_error_string();
        Print.clear_error_buf(); then (Values.STRING(str),st);
    case (env,Exp.CALL(Absyn.IDENT("runScript"),{Exp.SCONST(str)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        scriptstr = System.read_file(str);
        (istmts,msg) = Parser.parsestringexp(scriptstr);
        equality(msg = "Ok");
        (res,newst) = Interactive.evaluate(istmts, st, true);
        res_1 = Util.string_append_list({res,"\ntrue"}); then (Values.STRING(res_1),newst);
    case (env,Exp.CALL(Absyn.IDENT("runScript"),{Exp.SCONST(str)},_,_),st) then (Values.BOOL(false),st); 
    case (env,Exp.CALL(Absyn.IDENT("generateCode"),{(funcref = Exp.CREF(fcr,_))},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        path = Static.component_ref_to_path(fcr) " SCode.elaborate(p) => p' & ";
        ceval_generate_function(env, path) "	& Inst.instantiate_implicit(p') => d & "; then (Values.BOOL(true),st);
    case (env,Exp.CALL(Absyn.IDENT("loadModel"),{Exp.CREF(cr,_)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf))) " add path to symboltable for compiled functions
	  Interactive.SYMBOLTABLE(p,sp,ic,iv,(path,t)::cf),
	  but where to get t? "
      local Absyn.Program p_1;
      equation 
        mp = System.modelicapath();
        path = Static.component_ref_to_path(cr);
        pnew = ClassLoader.load_class(path, mp);
        p_1 = Interactive.update_program(pnew, p);
        str = Print.get_string();
        newst = Interactive.SYMBOLTABLE(p_1,sp,{},iv,cf); then (Values.BOOL(true),newst);
    case (env,Exp.CALL(Absyn.IDENT("loadModel"),{Exp.CREF(cr,_)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        path = Static.component_ref_to_path(cr);
        pathstr = ModUtil.path_string(path);
        str = Util.string_append_list({"Class ",pathstr," not found in MODELICAPATH.\n"});
        Print.print_error_buf(str); then (Values.BOOL(false),st);
    case (env,Exp.CALL(Absyn.IDENT("loadFile"),{Exp.SCONST(name)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      local Absyn.Program p1;
      equation 
        0 = System.regular_file_exist(name);
        p1 = Parser.parse(name);
        newp = Interactive.update_program(p1, p); then (Values.BOOL(true),Interactive.SYMBOLTABLE(newp,sp,{},iv,cf));
    case (env,Exp.CALL(Absyn.IDENT("loadFile"),{Exp.SCONST(name)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf))) "* it the rule above have failed then check if file exists *  without this omc crashes "
      equation 
        rest = System.regular_file_exist(name);
        (rest > 0) = true; then (Values.BOOL(false),st);
    case (env,Exp.CALL(Absyn.IDENT("loadFile"),{Exp.SCONST(name)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        failure(_ = Parser.parse(name)); then (Values.BOOL(false),st);
    case (env,Exp.CALL(Absyn.IDENT("saveModel"),{Exp.SCONST(filename),Exp.CREF(cr,_)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        classpath = Static.component_ref_to_path(cr);
        class_ = Interactive.get_pathed_class_in_program(classpath, p);
        str = Dump.unparse_str(Absyn.PROGRAM({class_},Absyn.TOP)) " ,true ";
        System.write_file(filename, str); then (Values.BOOL(true),st);
    case (env,Exp.CALL(Absyn.IDENT("save"),{Exp.CREF(cr,_)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      local Absyn.Program p_1;
      equation 
        classpath = Static.component_ref_to_path(cr);
        (p_1,filename) = Interactive.get_contained_class_and_file(classpath, p);
        str = Dump.unparse_str(p_1) " ,true ";
        System.write_file(filename, str); then (Values.BOOL(true),st);
    case (env,Exp.CALL(Absyn.IDENT("save"),{Exp.CREF(cr,_)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf))) then (Values.BOOL(false),st); 
    case (env,Exp.CALL(Absyn.IDENT("saveAll"),{Exp.SCONST(filename)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        str = Dump.unparse_str(p) " ,true ";
        System.write_file(filename, str); then (Values.BOOL(true),st);
    case (env,Exp.CALL(Absyn.IDENT("saveModel"),{Exp.SCONST(name),Exp.CREF(cr,_)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        classpath = Static.component_ref_to_path(cr) "Error writing to file ";
        class_ = Interactive.get_pathed_class_in_program(classpath, p);
        str = Dump.unparse_str(Absyn.PROGRAM({class_},Absyn.TOP)) " ,true ";
        Print.print_error_buf("Error writing to file.\n") " ,true "; then (Values.BOOL(false),st);
    case (env,Exp.CALL(Absyn.IDENT("saveModel"),{Exp.SCONST(name),Exp.CREF(cr,_)},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        classpath = Static.component_ref_to_path(cr) "Error writing to file ";
        Print.print_error_buf("Error unknown class.\n") "Error writing to file "; then (Values.BOOL(false),st);
    case (env,Exp.CALL(Absyn.IDENT("help"),{},_,_),(st = Interactive.SYMBOLTABLE(p,sp,ic,iv,cf)))
      equation 
        omhome = System.read_env("OPENMODELICAHOME");
        pd = System.path_delimiter();
        filename = Util.string_append_list({omhome,pd,"Compiler",pd,"helptext.txt"});
        print(filename);
        str = System.read_file(filename); then (Values.STRING(str),st);
    case (env,Exp.CALL(Absyn.IDENT("getUnit"),{Exp.CREF(cref,_),Exp.CREF(classname,_)},_,_),st)
      local Exp.ComponentRef classname;
      equation 
        (v,st_1) = get_builtin_attribute(classname, cref, "unit", st); then (v,st_1);
    case (env,Exp.CALL(Absyn.IDENT("getQuantity"),{Exp.CREF(cref,_),Exp.CREF(classname,_)},_,_),st)
      local Exp.ComponentRef classname;
      equation 
        (v,st_1) = get_builtin_attribute(classname, cref, "quantity", st); then (v,st_1);
    case (env,Exp.CALL(Absyn.IDENT("getDisplayUnit"),{Exp.CREF(cref,_),Exp.CREF(classname,_)},_,_),st)
      local Exp.ComponentRef classname;
      equation 
        (v,st_1) = get_builtin_attribute(classname, cref, "displayUnit", st); then (v,st_1);
    case (env,Exp.CALL(Absyn.IDENT("getMin"),{Exp.CREF(cref,_),Exp.CREF(classname,_)},_,_),st)
      local Exp.ComponentRef classname;
      equation 
        (v,st_1) = get_builtin_attribute(classname, cref, "min", st); then (v,st_1);
    case (env,Exp.CALL(Absyn.IDENT("getMax"),{Exp.CREF(cref,_),Exp.CREF(classname,_)},_,_),st)
      local Exp.ComponentRef classname;
      equation 
        (v,st_1) = get_builtin_attribute(classname, cref, "max", st); then (v,st_1);
    case (env,Exp.CALL(Absyn.IDENT("getStart"),{Exp.CREF(cref,_),Exp.CREF(classname,_)},_,_),st)
      local Exp.ComponentRef classname;
      equation 
        (v,st_1) = get_builtin_attribute(classname, cref, "start", st); then (v,st_1);
    case (env,Exp.CALL(Absyn.IDENT("getFixed"),{Exp.CREF(cref,_),Exp.CREF(classname,_)},_,_),st)
      local Exp.ComponentRef classname;
      equation 
        (v,st_1) = get_builtin_attribute(classname, cref, "fixed", st); then (v,st_1);
    case (env,Exp.CALL(Absyn.IDENT("getNominal"),{Exp.CREF(cref,_),Exp.CREF(classname,_)},_,_),st)
      local Exp.ComponentRef classname;
      equation 
        (v,st_1) = get_builtin_attribute(classname, cref, "nominal", st); then (v,st_1);
    case (env,Exp.CALL(Absyn.IDENT("getStateSelect"),{Exp.CREF(cref,_),Exp.CREF(classname,_)},_,_),st)
      local Exp.ComponentRef classname;
      equation 
        (v,st_1) = get_builtin_attribute(classname, cref, "stateSelect", st); then (v,st_1);
    case (env,Exp.CALL(Absyn.IDENT("exp_contains"),{exp1,exp2},_,_),st) " FIXME: test code: remove "
      local Boolean res;
      equation 
        res = Exp.exp_contains(exp1, exp2); then (Values.BOOL(res),st);
    case (env,Exp.CALL(Absyn.IDENT("solve"),{exp1,exp2,exp3},_,_),st)
      equation 
        res1 = Exp.solve(exp1, exp2, exp3);
        res = Exp.print_exp_str(res1); then (Values.STRING(res),st);
  end matchcontinue;
end ceval_interactive_functions;

protected function get_builtin_attribute
  input Exp.ComponentRef in_componentref1;
  input Exp.ComponentRef in_componentref2;
  input String in_string3;
  input Interactive.InteractiveSymbolTable in_interactivesymboltable4;
  output Values.Value out_value;
  output Interactive.InteractiveSymbolTable out_interactivesymboltable;
algorithm 
  (out_value,out_interactivesymboltable):=
  matchcontinue (in_componentref1,in_componentref2,in_string3,in_interactivesymboltable4)
    local
      type DAE_ElementList = list<DAE.Element>;
      type Env_FrameList = list<Env.Frame>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type SCode_ClassList = list<SCode.Class>;
      type Interactive_InstantiatedClassList = list<Interactive.InstantiatedClass>;
      type Interactive_InteractiveVariableList = list<Interactive.InteractiveVariable>;
      type TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOption = tuple<Absyn.Path,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOptionList = list<TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOption>;
      type Connect_SetList = list<Connect.Set>;
      type Types_VarList = list<Types.Var>;
      Absyn.Path classname_1;
      DAE_ElementList dae,dae1;
      Env_FrameList env,env_1,env3,env4;
      Exp.ComponentRef cref_1,classname,cref;
      Types.Attributes attr;
      TupleTypes_TTypeAbsyn_PathOption ty;
      Exp.Exp exp;
      String str,n,attribute;
      Interactive.InteractiveSymbolTable st;
      Absyn.Program p;
      SCode_ClassList sp,p_1;
      Interactive_InstantiatedClassList ic,ic_1;
      Interactive_InteractiveVariableList vars;
      TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOptionList cf;
      SCode.Class c;
      Boolean encflag;
      SCode.Restriction r;
      ClassInf.State ci_state,ci_state_1;
      Connect_SetList csets_1;
      Types_VarList tys;
      Values.Value v;
    case (classname,cref,"stateSelect",(st = Interactive.SYMBOLTABLE(p,sp,ic,vars,cf)))
      equation 
        classname_1 = Static.component_ref_to_path(classname) " Check cached instantiated class ";
        Interactive.INSTCLASS(_,dae,env) = Interactive.get_instantiated_class(ic, classname_1);
        cref_1 = Exp.join_crefs(cref, Exp.CREF_IDENT("stateSelect",{}));
        (attr,ty,Types.EQBOUND(exp,_)) = Lookup.lookup_var(env, cref_1);
        str = Exp.print_exp_str(exp); then (Values.STRING(str),st);
    case (classname,cref,"stateSelect",Interactive.SYMBOLTABLE(p,sp,ic,vars,cf))
      equation 
        classname_1 = Static.component_ref_to_path(classname);
        p_1 = SCode.elaborate(p);
        env = Inst.make_env_from_program(p_1, Absyn.IDENT(""));
        (c = SCode.CLASS(n,_,encflag,r,_),env_1) = Lookup.lookup_class(env, classname_1, true);
        env3 = Env.open_scope(env_1, encflag, SOME(n));
        ci_state = ClassInf.start(r, n);
        (dae1,env4,csets_1,ci_state_1,tys) = Inst.inst_class_in(env3, Types.NOMOD, Prefix.NOPRE, {}, ci_state, c, false, 
          {}, false, false);
        cref_1 = Exp.join_crefs(cref, Exp.CREF_IDENT("stateSelect",{}));
        (attr,ty,Types.EQBOUND(exp,_)) = Lookup.lookup_var(env4, cref_1);
        ic_1 = Interactive.add_instantiated_class(ic, Interactive.INSTCLASS(classname_1,dae1,env4));
        str = Exp.print_exp_str(exp); then (Values.STRING(str),Interactive.SYMBOLTABLE(p,sp,ic_1,vars,cf));
    case (classname,cref,attribute,(st = Interactive.SYMBOLTABLE(p,sp,ic,vars,cf)))
      equation 
        classname_1 = Static.component_ref_to_path(classname);
        Interactive.INSTCLASS(_,dae,env) = Interactive.get_instantiated_class(ic, classname_1);
        cref_1 = Exp.join_crefs(cref, Exp.CREF_IDENT(attribute,{}));
        (attr,ty,Types.VALBOUND(v)) = Lookup.lookup_var(env, cref_1); then (v,st);
    case (classname,cref,attribute,(st = Interactive.SYMBOLTABLE(p,sp,ic,vars,cf)))
      equation 
        classname_1 = Static.component_ref_to_path(classname);
        p_1 = SCode.elaborate(p);
        env = Inst.make_env_from_program(p_1, Absyn.IDENT(""));
        (c = SCode.CLASS(n,_,encflag,r,_),env_1) = Lookup.lookup_class(env, classname_1, true);
        env3 = Env.open_scope(env_1, encflag, SOME(n));
        ci_state = ClassInf.start(r, n);
        (dae1,env4,csets_1,ci_state_1,tys) = Inst.inst_class_in(env3, Types.NOMOD, Prefix.NOPRE, {}, ci_state, c, false, 
          {}, false, false);
        cref_1 = Exp.join_crefs(cref, Exp.CREF_IDENT(attribute,{}));
        (attr,ty,Types.VALBOUND(v)) = Lookup.lookup_var(env4, cref_1);
        ic_1 = Interactive.add_instantiated_class(ic, Interactive.INSTCLASS(classname_1,dae1,env4)); then (v,Interactive.SYMBOLTABLE(p,sp,ic_1,vars,cf));
  end matchcontinue;
end get_builtin_attribute;

protected function ceval_matrixelt
  input Env.Env in_env;
  input TupleExp_ExpBooleanListList in_tupleexp_expbooleanlistlist;
  input Boolean in_boolean;
  output Values_ValueList out_values_valuelist;
protected 
  type TupleExp_ExpBoolean = tuple<Exp.Exp,Boolean>;
  type TupleExp_ExpBooleanList = list<TupleExp_ExpBoolean>;
  type TupleExp_ExpBooleanListList = list<TupleExp_ExpBooleanList>;
  type Values_ValueList = list<Values.Value>;
algorithm 
  out_values_valuelist:=
  matchcontinue (in_env,in_tupleexp_expbooleanlistlist,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      Values.Value v;
      Values_ValueList vl;
      Env_FrameList env;
      TupleExp_ExpBooleanList expl;
      TupleExp_ExpBooleanListList expll;
      Boolean impl;
    case (env,expl :: expll,impl)
      equation 
        v = ceval_matrixeltrow(env, expl, impl);
        vl = ceval_matrixelt(env, expll, impl); then v :: vl;
    case (_,{},_) then {}; 
  end matchcontinue;
end ceval_matrixelt;

protected function ceval_matrixeltrow
  input Env.Env in_env;
  input TupleExp_ExpBooleanList in_tupleexp_expbooleanlist;
  input Boolean in_boolean;
  output Values.Value out_value;
protected 
  type TupleExp_ExpBoolean = tuple<Exp.Exp,Boolean>;
  type TupleExp_ExpBooleanList = list<TupleExp_ExpBoolean>;
algorithm 
  out_value:=
  matchcontinue (in_env,in_tupleexp_expbooleanlist,in_boolean)
    local
      type Values_ValueList = list<Values.Value>;
      type Env_FrameList = list<Env.Frame>;
      Values.Value res;
      Values_ValueList resl;
      Env_FrameList env;
      Exp.Exp e;
      TupleExp_ExpBooleanList rest;
      Boolean impl;
    case (env,(e,_) :: rest,impl)
      equation 
        (res,_) = ceval(env, e, impl, NONE, NONE);
        Values.ARRAY(resl) = ceval_matrixeltrow(env, rest, impl); then Values.ARRAY(res :: resl);
    case (env,{},_) then Values.ARRAY({}); 
  end matchcontinue;
end ceval_matrixeltrow;

protected function ceval_builtin_size
  input Env.Env in_env1;
  input Exp.Exp in_exp2;
  input Exp.Exp in_exp3;
  input Boolean in_boolean4;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption5;
  output Values.Value out_value;
  output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
protected 
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  (out_value,out_interactive_interactivesymboltableoption):=
  matchcontinue (in_env1,in_exp2,in_exp3,in_boolean4,in_interactive_interactivesymboltableoption5)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type IntegerList = list<Integer>;
      type Env_FrameList = list<Env.Frame>;
      Types.Attributes attr;
      TupleTypes_TTypeAbsyn_PathOption tp;
      Types.Binding bind,binding;
      IntegerList sizelst;
      Integer dim,dim_1,v,dimv;
      Interactive_InteractiveSymbolTableOption st_1,st;
      Env_FrameList env;
      Exp.ComponentRef cr;
      Boolean impl;
      Exp.Exp exp;
      String expstr;
      Exp.Type crtp;
      Values.Value v2;
    case (env,Exp.CREF(cr,tp),dim,impl,st)
      equation 
        (attr,tp,bind) = Lookup.lookup_var(env, cr) " If dimensions known, always ceval ";
        true = Types.dimensions_known(tp);
        sizelst = Types.get_dimension_sizes(tp);
        (Values.INTEGER(dim),st_1) = ceval(env, dim, impl, st, NONE);
        dim_1 = dim - 1;
        v = list_nth(sizelst, dim_1); then (Values.INTEGER(v),st_1);
    case (env,Exp.CREF(cr,tp),dim,(impl = true),st)
      local Exp.Exp dim;
      equation 
        (attr,tp,bind) = Lookup.lookup_var(env, cr) " If dimensions not known and impl=true, just silently fail ";
        false = Types.dimensions_known(tp); then fail();
    case (env,Exp.CREF(cr,tp),dim,(impl = false),st)
      local Exp.Exp dim;
      equation 
        (attr,tp,bind) = Lookup.lookup_var(env, cr) " If dimensions not known and impl=false, error message ";
        false = Types.dimensions_known(tp);
        Print.print_error_buf(
          "#-- ceval_builtin_size: Dimensions must be known when instantiating models\n"); then fail();
    case (env,(exp = Exp.CREF(cr,crtp)),dim,(impl = false),st)
      local Exp.Exp dim;
      equation 
        (attr,tp,Types.UNBOUND()) = Lookup.lookup_var(env, cr) "For crefs without value binding";
        Print.print_error_buf(
          "#-- ceval_builtin_size: Value of cref not known when instantiating: ") "For crefs without value binding";
        expstr = Exp.print_exp_str(exp);
        Print.print_error_buf(expstr);
        Print.print_error_buf("\n"); then fail();
    case (env,(exp = Exp.CREF(cr,crtp)),dim,(impl = true),st)
      local Exp.Exp dim;
      equation 
        (attr,tp,Types.UNBOUND()) = Lookup.lookup_var(env, cr) "For crefs without value binding. If impl=true just silently fail "; then fail();
    case (env,(exp = Exp.CREF(cr,crtp)),dim,impl,st)
      local
        Values.Value v;
        Exp.Exp dim;
      equation 
        (attr,tp,binding) = Lookup.lookup_var(env, cr) "For crefs with value binding";
        (Values.INTEGER(dimv),st_1) = ceval(env, dim, impl, st, NONE);
        v = ceval_cref_binding(env, cr, binding, impl);
        v2 = ceval_builtin_size_2(v, dimv); then (v2,st_1);
    case (env,exp,dim,impl,st)
      local
        Values.Value v;
        Exp.Exp dim;
      equation 
        (v,st_1) = ceval(env, exp, impl, st, NONE) " try to ceval expression, for constant expressions ";
        (Values.INTEGER(dimv),st_1) = ceval(env, dim, impl, st, NONE);
        v2 = ceval_builtin_size_2(v, dimv); then (v2,st_1);
    case (env,exp,dim,impl,st)
      local Exp.Exp dim;
      equation 
        Print.print_error_buf("#-- ceval_builtin_size failed: ");
        expstr = Exp.print_exp_str(exp);
        Print.print_error_buf(expstr);
        Print.print_error_buf("\n"); then fail();
  end matchcontinue;
end ceval_builtin_size;

protected function ceval_builtin_size_2
  input Values.Value in_value;
  input Integer in_integer;
  output Values.Value out_value;
algorithm 
  out_value:=
  matchcontinue (in_value,in_integer)
    local
      type Values_ValueList = list<Values.Value>;
      Integer dim,ind_1,ind;
      Values_ValueList lst;
      Values.Value l;
    case (Values.ARRAY(lst),1)
      equation 
        dim = list_length(lst); then Values.INTEGER(dim);
    case (Values.ARRAY(l :: lst),ind)
      local Values.Value dim;
      equation 
        ind_1 = ind - 1;
        dim = ceval_builtin_size_2(l, ind_1); then dim;
    case (_,_)
      equation 
        Debug.fprint("failtrace", "- ceval_builtin_size_2 failed\n"); then fail();
  end matchcontinue;
end ceval_builtin_size_2;

protected function ceval_builtin_abs
  input Env.Env in_env;
  input Exp_ExpList in_exp_explist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Values.Value out_value;
  output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
protected 
  type Exp_ExpList = list<Exp.Exp>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  (out_value,out_interactive_interactivesymboltableoption):=
  matchcontinue (in_env,in_exp_explist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      Real rv,rv_1;
      Env_FrameList env;
      Exp.Exp exp;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st;
      Integer iv;
    case (env,{exp},impl,st)
      equation 
        (Values.REAL(rv),_) = ceval(env, exp, impl, st, NONE);
        rv_1 = real_abs(rv); then (Values.REAL(rv_1),st);
    case (env,{exp},impl,st)
      equation 
        (Values.INTEGER(iv),_) = ceval(env, exp, impl, st, NONE);
        iv = int_abs(iv); then (Values.INTEGER(iv),st);
  end matchcontinue;
end ceval_builtin_abs;

protected function ceval_builtin_floor
  input Env.Env in_env;
  input Exp_ExpList in_exp_explist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Values.Value out_value;
  output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
protected 
  type Exp_ExpList = list<Exp.Exp>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  (out_value,out_interactive_interactivesymboltableoption):=
  matchcontinue (in_env,in_exp_explist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      Real rv,rv_1;
      Env_FrameList env;
      Exp.Exp exp;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st;
    case (env,{exp},impl,st)
      equation 
        (Values.REAL(rv),_) = ceval(env, exp, impl, st, NONE) "print \"# floor function not implemented CEVAL yet\n\" &";
        rv_1 = real_floor(rv); then (Values.REAL(rv_1),st);
  end matchcontinue;
end ceval_builtin_floor;

protected function ceval_builtin_ceil
  input Env.Env in_env;
  input Exp_ExpList in_exp_explist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Values.Value out_value;
  output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
protected 
  type Exp_ExpList = list<Exp.Exp>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  (out_value,out_interactive_interactivesymboltableoption):=
  matchcontinue (in_env,in_exp_explist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      Real rv,rv_1,rvt,rv_2;
      Integer ri,ri_1;
      Env_FrameList env;
      Exp.Exp exp;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st;
    case (env,{exp},impl,st)
      equation 
        (Values.REAL(rv),_) = ceval(env, exp, impl, st, NONE);
        rv_1 = real_floor(rv);
        ri = real_int(rv_1);
        rvt = int_real(ri);
        (rvt ==. rv) = true; then (Values.REAL(rv_1),st);
    case (env,{exp},impl,st)
      equation 
        (Values.REAL(rv),_) = ceval(env, exp, impl, st, NONE);
        rv_1 = real_floor(rv);
        ri = real_int(rv_1);
        ri_1 = ri + 1;
        rv_2 = int_real(ri_1); then (Values.REAL(rv_2),st);
  end matchcontinue;
end ceval_builtin_ceil;

protected function ceval_builtin_sqrt
  input Env.Env in_env;
  input Exp_ExpList in_exp_explist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Values.Value out_value;
  output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
protected 
  type Exp_ExpList = list<Exp.Exp>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  (out_value,out_interactive_interactivesymboltableoption):=
  matchcontinue (in_env,in_exp_explist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      Real rv,rv_1;
      Env_FrameList env;
      Exp.Exp exp;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st;
    case (env,{exp},impl,st)
      equation 
        (Values.REAL(rv),_) = ceval(env, exp, impl, st, NONE);
        (rv <. 0.0) = true;
        Print.print_error_buf("# sqrt - the input is negative\n"); then fail();
    case (env,{exp},impl,st)
      equation 
        (Values.REAL(rv),_) = ceval(env, exp, impl, st, NONE) "* print \"# sqrt function not implemented CEVAL yet\n\" &*";
        rv_1 = real_sqrt(rv); then (Values.REAL(rv_1),st);
  end matchcontinue;
end ceval_builtin_sqrt;

protected function ceval_builtin_sin
  input Env.Env in_env;
  input Exp_ExpList in_exp_explist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Values.Value out_value;
  output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
protected 
  type Exp_ExpList = list<Exp.Exp>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  (out_value,out_interactive_interactivesymboltableoption):=
  matchcontinue (in_env,in_exp_explist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      Real rv,rv_1;
      Env_FrameList env;
      Exp.Exp exp;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st;
    case (env,{exp},impl,st)
      equation 
        (Values.REAL(rv),_) = ceval(env, exp, impl, st, NONE);
        rv_1 = real_sin(rv); then (Values.REAL(rv_1),st);
  end matchcontinue;
end ceval_builtin_sin;

protected function ceval_builtin_cos
  input Env.Env in_env;
  input Exp_ExpList in_exp_explist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Values.Value out_value;
  output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
protected 
  type Exp_ExpList = list<Exp.Exp>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  (out_value,out_interactive_interactivesymboltableoption):=
  matchcontinue (in_env,in_exp_explist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      Real rv,rv_1;
      Env_FrameList env;
      Exp.Exp exp;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st;
    case (env,{exp},impl,st)
      equation 
        (Values.REAL(rv),_) = ceval(env, exp, impl, st, NONE);
        rv_1 = real_cos(rv); then (Values.REAL(rv_1),st);
  end matchcontinue;
end ceval_builtin_cos;

protected function ceval_builtin_tan
  input Env.Env in_env;
  input Exp_ExpList in_exp_explist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Values.Value out_value;
  output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
protected 
  type Exp_ExpList = list<Exp.Exp>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  (out_value,out_interactive_interactivesymboltableoption):=
  matchcontinue (in_env,in_exp_explist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      Real rv,sv,cv,rv_1;
      Env_FrameList env;
      Exp.Exp exp;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st;
    case (env,{exp},impl,st)
      equation 
        (Values.REAL(rv),_) = ceval(env, exp, impl, st, NONE) " tan is not implemented in RML for some strange reason...
	 use the formula tan(x) = sin(x) / cos(x) and hope cos(x) != 0 
	 ";
        sv = real_sin(rv);
        cv = real_cos(rv);
        rv_1 = sv/.cv; then (Values.REAL(rv_1),st);
  end matchcontinue;
end ceval_builtin_tan;

protected function ceval_builtin_div
  input Env.Env in_env;
  input Exp_ExpList in_exp_explist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Values.Value out_value;
  output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
protected 
  type Exp_ExpList = list<Exp.Exp>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  (out_value,out_interactive_interactivesymboltableoption):=
  matchcontinue (in_env,in_exp_explist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      Real rv1,rv2,rv_1,rv_2;
      Integer ri,ri_1,ri1,ri2;
      Env_FrameList env;
      Exp.Exp exp1,exp2;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st;
    case (env,{exp1,exp2},impl,st)
      equation 
        (Values.REAL(rv1),_) = ceval(env, exp1, impl, st, NONE);
        (Values.REAL(rv2),_) = ceval(env, exp2, impl, st, NONE);
        rv_1 = rv1/.rv2;
        ri = real_int(rv_1);
        rv_2 = int_real(ri); then (Values.REAL(rv_2),st);
    case (env,{exp1,exp2},impl,st)
      equation 
        (Values.INTEGER(ri),_) = ceval(env, exp1, impl, st, NONE);
        rv1 = int_real(ri);
        (Values.REAL(rv2),_) = ceval(env, exp2, impl, st, NONE);
        rv_1 = rv1/.rv2;
        ri_1 = real_int(rv_1);
        rv_2 = int_real(ri_1); then (Values.REAL(rv_2),st);
    case (env,{exp1,exp2},impl,st)
      equation 
        (Values.REAL(rv1),_) = ceval(env, exp1, impl, st, NONE);
        (Values.INTEGER(ri),_) = ceval(env, exp2, impl, st, NONE);
        rv2 = int_real(ri);
        rv_1 = rv1/.rv2;
        ri_1 = real_int(rv_1);
        rv_2 = int_real(ri_1); then (Values.REAL(rv_2),st);
    case (env,{exp1,exp2},impl,st)
      equation 
        (Values.INTEGER(ri1),_) = ceval(env, exp1, impl, st, NONE);
        (Values.INTEGER(ri2),_) = ceval(env, exp2, impl, st, NONE);
        ri_1 = ri1/ri2; then (Values.INTEGER(ri_1),st);
    case (env,{exp1,exp2},impl,st)
      equation 
        (Values.REAL(rv2),_) = ceval(env, exp2, impl, st, NONE);
        (rv2 ==. 0.0) = true;
        Print.print_error_buf("#Error, division by zero.\n"); then fail();
    case (env,{exp1,exp2},impl,st)
      equation 
        (Values.INTEGER(ri2),_) = ceval(env, exp2, impl, st, NONE);
        (ri2 == 0) = true;
        Print.print_error_buf("#Error, division by zero.\n"); then fail();
  end matchcontinue;
end ceval_builtin_div;

protected function ceval_builtin_mod
  input Env.Env in_env;
  input Exp_ExpList in_exp_explist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Values.Value out_value;
  output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
protected 
  type Exp_ExpList = list<Exp.Exp>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  (out_value,out_interactive_interactivesymboltableoption):=
  matchcontinue (in_env,in_exp_explist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      Real rv1,rv2,rva,rvb,rvc,rvd;
      Env_FrameList env;
      Exp.Exp exp1,exp2;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st;
      Integer ri,ri1,ri2,ri_1;
    case (env,{exp1,exp2},impl,st)
      equation 
        (Values.REAL(rv1),_) = ceval(env, exp1, impl, st, NONE);
        (Values.REAL(rv2),_) = ceval(env, exp2, impl, st, NONE);
        rva = rv1/.rv2;
        rvb = real_floor(rva);
        rvc = rvb*.rv2;
        rvd = rv1 -. rvc; then (Values.REAL(rvd),st);
    case (env,{exp1,exp2},impl,st)
      equation 
        (Values.INTEGER(ri),_) = ceval(env, exp1, impl, st, NONE);
        rv1 = int_real(ri);
        (Values.REAL(rv2),_) = ceval(env, exp2, impl, st, NONE);
        rva = rv1/.rv2;
        rvb = real_floor(rva);
        rvc = rvb*.rv2;
        rvd = rv1 -. rvc; then (Values.REAL(rvd),st);
    case (env,{exp1,exp2},impl,st)
      equation 
        (Values.REAL(rv1),_) = ceval(env, exp1, impl, st, NONE);
        (Values.INTEGER(ri),_) = ceval(env, exp2, impl, st, NONE);
        rv2 = int_real(ri);
        rva = rv1/.rv2;
        rvb = real_floor(rva);
        rvc = rvb*.rv2;
        rvd = rv1 -. rvc; then (Values.REAL(rvd),st);
    case (env,{exp1,exp2},impl,st)
      equation 
        (Values.INTEGER(ri1),_) = ceval(env, exp1, impl, st, NONE);
        (Values.INTEGER(ri2),_) = ceval(env, exp2, impl, st, NONE);
        rv1 = int_real(ri1);
        rv2 = int_real(ri2);
        rva = rv1/.rv2;
        rvb = real_floor(rva);
        rvc = rvb*.rv2;
        rvd = rv1 -. rvc;
        ri_1 = real_int(rvd); then (Values.INTEGER(ri_1),st);
    case (env,{exp1,exp2},impl,st)
      equation 
        (Values.REAL(rv2),_) = ceval(env, exp2, impl, st, NONE);
        (rv2 ==. 0.0) = true;
        Print.print_error_buf("#Second argument in mod should be non-zero.\n"); then fail();
    case (env,{exp1,exp2},impl,st)
      equation 
        (Values.INTEGER(ri2),_) = ceval(env, exp2, impl, st, NONE);
        (ri2 == 0) = true;
        Print.print_error_buf("#Second argument in mod should be non-zero.\n"); then fail();
  end matchcontinue;
end ceval_builtin_mod;

protected function ceval_builtin_differentiate
  input Env.Env in_env;
  input Exp_ExpList in_exp_explist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Values.Value out_value;
  output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
protected 
  type Exp_ExpList = list<Exp.Exp>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  (out_value,out_interactive_interactivesymboltableoption):=
  matchcontinue (in_env,in_exp_explist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp differentiated_exp,differentiated_exp_1,exp1;
      String ret_val;
      Env_FrameList env;
      Exp.ComponentRef cr;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st;
    case (env,{exp1,Exp.CREF(cr,_)},impl,st)
      equation 
        differentiated_exp = Derive.differentiate_exp(exp1, cr);
        differentiated_exp_1 = Exp.simplify(differentiated_exp);
        ret_val = Exp.print_exp_str(differentiated_exp_1) " this is wrong...   this should be used instead but unelab_exp must be able to unelaborate a complete exp 
           now it doesn't so the expression is returned as string Exp.unelab_exp(differentiated_exp') => absyn_exp "; then (Values.STRING(ret_val),st);
    case (_,_,_,st) "           =>  (Values.CODE(Absyn.C_EXPRESSION(absyn_exp)),st) "
      equation 
        Print.print_buf("#Differentiation failed\n"); then fail();
  end matchcontinue;
end ceval_builtin_differentiate;

protected function ceval_builtin_simplify
  input Env.Env in_env;
  input Exp_ExpList in_exp_explist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Values.Value out_value;
  output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
protected 
  type Exp_ExpList = list<Exp.Exp>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  (out_value,out_interactive_interactivesymboltableoption):=
  matchcontinue (in_env,in_exp_explist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp exp1_1,exp1;
      String ret_val;
      Env_FrameList env;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st;
    case (env,{exp1},impl,st)
      equation 
        exp1_1 = Exp.simplify(exp1);
        ret_val = Exp.print_exp_str(exp1_1) " this should be used instead but unelab_exp must be able to unelaborate a complete exp  Exp.unelab_exp(simplifyd_exp') => absyn_exp "; then (Values.STRING(ret_val),st);
    case (_,_,_,st) "           =>  (Values.CODE(Absyn.C_EXPRESSION(absyn_exp)),st) "
      equation 
        Print.print_buf("#simplification failed\n"); then fail();
  end matchcontinue;
end ceval_builtin_simplify;

protected function ceval_builtin_rem
  input Env.Env in_env;
  input Exp_ExpList in_exp_explist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Values.Value out_value;
  output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
protected 
  type Exp_ExpList = list<Exp.Exp>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  (out_value,out_interactive_interactivesymboltableoption):=
  matchcontinue (in_env,in_exp_explist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      Real rv1,rv2,rva,rvb,rvd;
      Env_FrameList env;
      Exp.Exp exp1,exp2;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st;
      Integer ri,ri1,ri2,ri_1;
    case (env,{exp1,exp2},impl,st)
      equation 
        (Values.REAL(rv1),_) = ceval(env, exp1, impl, st, NONE);
        (Values.REAL(rv2),_) = ceval(env, exp2, impl, st, NONE);
        rva = rv1/.rv2;
        rvb = rva*.rv2;
        rvd = rv1 -. rvb; then (Values.REAL(rvd),st);
    case (env,{exp1,exp2},impl,st)
      equation 
        (Values.INTEGER(ri),_) = ceval(env, exp1, impl, st, NONE);
        rv1 = int_real(ri);
        (Values.REAL(rv2),_) = ceval(env, exp2, impl, st, NONE);
        rva = rv1/.rv2;
        rvb = rva*.rv2;
        rvd = rv1 -. rvb; then (Values.REAL(rvd),st);
    case (env,{exp1,exp2},impl,st)
      equation 
        (Values.REAL(rv1),_) = ceval(env, exp1, impl, st, NONE);
        (Values.INTEGER(ri),_) = ceval(env, exp2, impl, st, NONE);
        rv2 = int_real(ri);
        rva = rv1/.rv2;
        rvb = rva*.rv2;
        rvd = rv1 -. rvb; then (Values.REAL(rvd),st);
    case (env,{exp1,exp2},impl,st)
      equation 
        (Values.INTEGER(ri1),_) = ceval(env, exp1, impl, st, NONE);
        (Values.INTEGER(ri2),_) = ceval(env, exp2, impl, st, NONE);
        rv1 = int_real(ri1);
        rv2 = int_real(ri2);
        rva = rv1/.rv2;
        rvb = rva*.rv2;
        rvd = rv1 -. rvb;
        ri_1 = real_int(rvd); then (Values.INTEGER(ri_1),st);
    case (env,{exp1,exp2},impl,st)
      equation 
        (Values.REAL(rv2),_) = ceval(env, exp2, impl, st, NONE);
        (rv2 ==. 0.0) = true;
        Print.print_error_buf("#Second argument in rem should be non-zero.\n"); then fail();
    case (env,{exp1,exp2},impl,st)
      equation 
        (Values.INTEGER(ri2),_) = ceval(env, exp2, impl, st, NONE);
        (ri2 == 0) = true;
        Print.print_error_buf("#Second argument in rem should be non-zero.\n"); then fail();
  end matchcontinue;
end ceval_builtin_rem;

protected function ceval_builtin_integer
  input Env.Env in_env;
  input Exp_ExpList in_exp_explist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Values.Value out_value;
  output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
protected 
  type Exp_ExpList = list<Exp.Exp>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  (out_value,out_interactive_interactivesymboltableoption):=
  matchcontinue (in_env,in_exp_explist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      Real rv;
      Integer ri;
      Env_FrameList env;
      Exp.Exp exp;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st;
    case (env,{exp},impl,st)
      equation 
        (Values.REAL(rv),_) = ceval(env, exp, impl, st, NONE);
        ri = real_int(rv); then (Values.INTEGER(ri),st);
  end matchcontinue;
end ceval_builtin_integer;

protected function ceval_generate_function
  input Env.Env in_env;
  input Absyn.Path in_path;
algorithm 
  _:=
  matchcontinue (in_env,in_path)
    local
      type Env_FrameList = list<Env.Frame>;
      String pathstr,gencodestr,filename;
      Env_FrameList env;
      Absyn.Path path;
    case (env,path)
      equation 
        Debug.fprintln("ceval", "/*- ceval_generate_function starting*/");
        pathstr = ModUtil.path_string2(path, "_");
        (gencodestr,_) = ceval_generate_function_str(path, env, {});
        filename = string_append(pathstr, ".c");
        Print.clear_buf();
        Print.print_buf(
          "#include \"modelica.h\"\n#include <stdio.h>\n#include <stdlib.h>\n#include <errno.h>\n");
        Print.print_buf(gencodestr);
        Print.print_buf("\nint main(int argc, char** argv)\n");
        Print.print_buf("{\n\n  if (argc != 3)\n");
        Print.print_buf(
          "{\n      fprintf(stderr,\"# Incorrect number of arguments\\n\");\n");
        Print.print_buf("return 1;\n    }\n");
        Print.print_buf(pathstr);
        Print.print_buf("_read_call_write(argv[1],argv[2]);\n  return 0;\n}\n");
        Print.write_buf(filename) "
	  Print.print_buf \"#endif /*\" & Print.print_buf defmacro & Print.print_buf \"*/\n\" & 
          ";
        System.compile_c_file(filename); then ();
    case (_,_)
      equation 
        Debug.fprint("failtrace", "/*- ceval_generate_function failed*/\n"); then fail();
  end matchcontinue;
end ceval_generate_function;

protected function ceval_generate_function_str " Generates a function with the given path, and all functions that are called
 * within that function. The string list contains names of functions already
 * generated, which won't be generated again "
  input Absyn.Path in_path;
  input Env.Env in_env;
  input Absyn_PathList in_absyn_pathlist;
  output String out_string;
  output Absyn_PathList out_absyn_pathlist;
protected 
  type Absyn_PathList = list<Absyn.Path>;
algorithm 
  (out_string,out_absyn_pathlist):=
  matchcontinue (in_path,in_env,in_absyn_pathlist)
    local
      type Env_FrameList = list<Env.Frame>;
      type DAE_ElementList = list<DAE.Element>;
      type StringList = list<String>;
      Absyn.Path gfmember,path;
      Env_FrameList env,env_1,env_2;
      Absyn_PathList gflist,calledfuncs,gflist_1;
      SCode.Class cls;
      DAE_ElementList d;
      StringList debugfuncs,calledfuncsstrs,calledfuncsstrs_1;
      String debugfuncsstr,funcname,funccom,thisfuncstr,resstr;
      DAE.DAElist d_1;
    case (path,env,gflist) " If getmember succeeds, path is in generated functions list, so do nothing "
      equation 
        gfmember = Util.list_getmember_p(path, gflist, ModUtil.path_equal); then ("",gflist);
    case (path,env,gflist) " If getmember fails, path is not in generated functions list, hence
	 * generate it "
      equation 
        failure(_ = Util.list_getmember_p(path, gflist, ModUtil.path_equal));
        Debug.fprintln("ceval", "/*- ceval_generate_function_str starting*/");
        (cls,env_1) = Lookup.lookup_class(env, path, false);
        Debug.fprintln("ceval", "/*- ceval_generate_function_str instantiating*/");
        (env_2,d) = Inst.implicit_function_instantiation(env_1, Types.NOMOD, Prefix.NOPRE, {}, cls, {}, false);
        Debug.fprint("ceval", 
          "/*- ceval_generate_function_str getting functions: ");
        calledfuncs = SimCodegen.get_called_functions_in_function(path, DAE.DAE(d));
        debugfuncs = Util.list_map(calledfuncs, Absyn.path_string);
        debugfuncsstr = Util.string_delimit_list(debugfuncs, ", ");
        Debug.fprint("ceval", debugfuncsstr);
        Debug.fprintln("ceval", "*/");
        (calledfuncsstrs,gflist_1) = ceval_generate_function_str_list(calledfuncs, env, gflist);
        Debug.fprint("ceval", "/*- ceval_generate_function_str prefixing dae */");
        d_1 = ModUtil.string_prefix_params(DAE.DAE(d));
        Print.clear_buf();
        funcname = Absyn.path_string(path);
        funccom = Util.string_append_list({"/*---FUNC: ",funcname," ---*/\n\n"});
        Print.print_buf(funccom);
        Debug.fprintln("ceval", 
          "/* - ceval_generate_function_str generating functions */");
        Codegen.generate_functions(d_1);
        thisfuncstr = Print.get_string();
        calledfuncsstrs_1 = Util.list_append_elt(thisfuncstr, calledfuncsstrs);
        resstr = Util.string_delimit_list(calledfuncsstrs_1, "\n\n"); then (resstr,path :: gflist);
    case (_,_,_)
      equation 
        Debug.fprint("failtrace", "/*- ceval_generate_function_str failed*/\n"); then fail();
  end matchcontinue;
end ceval_generate_function_str;

protected function ceval_generate_function_str_list
  input Absyn_PathList in_absyn_pathlist1;
  input Env.Env in_env2;
  input Absyn_PathList in_absyn_pathlist3;
  output StringList out_stringlist;
  output Absyn_PathList out_absyn_pathlist;
protected 
  type Absyn_PathList = list<Absyn.Path>;
  type StringList = list<String>;
algorithm 
  (out_stringlist,out_absyn_pathlist):=
  matchcontinue (in_absyn_pathlist1,in_env2,in_absyn_pathlist3)
    local
      type Env_FrameList = list<Env.Frame>;
      Env_FrameList env;
      Absyn_PathList gflist,gflist_1,gflist_2,rest;
      String firststr;
      StringList reststr;
      Absyn.Path first;
    case ({},env,gflist) then ({},gflist); 
    case (first :: rest,env,gflist)
      equation 
        (firststr,gflist_1) = ceval_generate_function_str(first, env, gflist);
        (reststr,gflist_2) = ceval_generate_function_str_list(rest, env, gflist_1); then (firststr :: reststr,gflist_2);
  end matchcontinue;
end ceval_generate_function_str_list;

protected function ceval_builtin_diagonal
  input Env.Env in_env;
  input Exp_ExpList in_exp_explist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Values.Value out_value;
  output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
protected 
  type Exp_ExpList = list<Exp.Exp>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  (out_value,out_interactive_interactivesymboltableoption):=
  matchcontinue (in_env,in_exp_explist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Values_ValueList = list<Values.Value>;
      type Env_FrameList = list<Env.Frame>;
      Values_ValueList rv2,retExp;
      Integer dimension,correctDimension;
      String dimensionString;
      Env_FrameList env;
      Exp.Exp exp;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st;
    case (env,{exp},impl,st)
      equation 
        (Values.ARRAY(rv2),_) = ceval(env, exp, impl, st, NONE);
        dimension = list_length(rv2);
        correctDimension = dimension + 1;
        retExp = ceval_builtin_diagonal2(env, exp, impl, st, correctDimension, 1, {});
        dimensionString = int_string(dimension);
        Debug.fcall("ceval", Print.print_buf, "== dimensionString ");
        Debug.fcall("ceval", Print.print_buf, dimensionString);
        Debug.fcall("ceval", Print.print_buf, "\n"); then (Values.ARRAY(retExp),st);
    case (_,_,_,_)
      equation 
        print(
          "#-- ceval_builtin_diagonal: Couldn't elaborate ceval_builtin_diagonal()\n"); then fail();
  end matchcontinue;
end ceval_builtin_diagonal;

protected function ceval_builtin_diagonal2
  input Env.Env in_env1;
  input Exp.Exp in_exp2;
  input Boolean in_boolean3;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption4;
  input Integer in_integer5;
  input Integer in_integer6;
  input Values_ValueList in_values_valuelist7;
  output Values_ValueList out_values_valuelist;
protected 
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
  type Values_ValueList = list<Values.Value>;
algorithm 
  out_values_valuelist:=
  matchcontinue (in_env1,in_exp2,in_boolean3,in_interactive_interactivesymboltableoption4,in_integer5,in_integer6,in_values_valuelist7)
    local
      type Env_FrameList = list<Env.Frame>;
      Real rv2;
      Integer correctDim,correctPlace,newRow,matrixDimension,row;
      Values_ValueList zeroList,listWithElement,retExp,appendedList,listIN,list_;
      Env_FrameList env;
      Exp.Exp s1;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st;
      String RowString,matrixDimensionString;
    case (env,s1,impl,st,matrixDimension,row,{})
      equation 
        (Values.REAL(rv2),_) = ceval(env, Exp.ASUB(s1,row), impl, st, NONE);
        correctDim = matrixDimension - 1;
        zeroList = Util.list_fill(Values.REAL(0.0), correctDim);
        correctPlace = row - 1;
        listWithElement = Util.list_replaceat(Values.REAL(rv2), correctPlace, zeroList);
        newRow = row + 1;
        retExp = ceval_builtin_diagonal2(env, s1, impl, st, matrixDimension, newRow, 
          {Values.ARRAY(listWithElement)}); then retExp;
    case (env,s1,impl,st,matrixDimension,row,listIN)
      equation 
        (Values.REAL(rv2),_) = ceval(env, Exp.ASUB(s1,row), impl, st, NONE);
        failure(equality(matrixDimension = row));
        correctDim = matrixDimension - 1;
        zeroList = Util.list_fill(Values.REAL(0.0), correctDim);
        correctPlace = row - 1;
        listWithElement = Util.list_replaceat(Values.REAL(rv2), correctPlace, zeroList);
        newRow = row + 1;
        appendedList = list_append(listIN, {Values.ARRAY(listWithElement)});
        retExp = ceval_builtin_diagonal2(env, s1, impl, st, matrixDimension, newRow, appendedList); then retExp;
    case (env,s1,impl,st,matrixDimension,row,{})
      local Integer rv2;
      equation 
        (Values.INTEGER(rv2),_) = ceval(env, Exp.ASUB(s1,row), impl, st, NONE);
        correctDim = matrixDimension - 1;
        zeroList = Util.list_fill(Values.INTEGER(0), correctDim);
        correctPlace = row - 1;
        listWithElement = Util.list_replaceat(Values.INTEGER(rv2), correctPlace, zeroList);
        newRow = row + 1;
        retExp = ceval_builtin_diagonal2(env, s1, impl, st, matrixDimension, newRow, 
          {Values.ARRAY(listWithElement)}); then retExp;
    case (env,s1,impl,st,matrixDimension,row,listIN)
      local Integer rv2;
      equation 
        (Values.INTEGER(rv2),_) = ceval(env, Exp.ASUB(s1,row), impl, st, NONE);
        failure(equality(matrixDimension = row));
        correctDim = matrixDimension - 1;
        zeroList = Util.list_fill(Values.INTEGER(0), correctDim);
        correctPlace = row - 1;
        listWithElement = Util.list_replaceat(Values.INTEGER(rv2), correctPlace, zeroList);
        newRow = row + 1;
        appendedList = list_append(listIN, {Values.ARRAY(listWithElement)});
        retExp = ceval_builtin_diagonal2(env, s1, impl, st, matrixDimension, newRow, appendedList); then retExp;
    case (env,s1,impl,st,matrixDimension,row,listIN)
      equation 
        equality(matrixDimension = row); then listIN;
    case (_,_,_,_,matrixDimension,row,list_)
      equation 
        print(
          "#-- ceval_builtin_diagonal2: Couldn't elaborate ceval_builtin_diagonal2()\n");
        RowString = int_string(row);
        matrixDimensionString = int_string(matrixDimension); then fail();
  end matchcontinue;
end ceval_builtin_diagonal2;

protected function ceval_builtin_size_matrix
  input Env.Env in_env;
  input Exp.Exp in_exp;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Values.Value out_value;
  output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
protected 
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  (out_value,out_interactive_interactivesymboltableoption):=
  matchcontinue (in_env,in_exp,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type IntegerList = list<Integer>;
      type Env_FrameList = list<Env.Frame>;
      Types.Attributes attr;
      TupleTypes_TTypeAbsyn_PathOption tp;
      Types.Binding bind;
      IntegerList sizelst;
      Values.Value v;
      Env_FrameList env;
      Exp.ComponentRef cr;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st,st_1;
      Exp.Exp exp;
    case (env,Exp.CREF(cr,tp),impl,st)
      equation 
        (attr,tp,bind) = Lookup.lookup_var(env, cr);
        sizelst = Types.get_dimension_sizes(tp);
        v = Values.intlist_to_value(sizelst); then (v,st);
    case (env,exp,impl,st)
      equation 
        (v,st_1) = ceval(env, exp, impl, st, NONE);
        tp = Types.type_of_value(v);
        sizelst = Types.get_dimension_sizes(tp);
        v = Values.intlist_to_value(sizelst); then (v,st);
  end matchcontinue;
end ceval_builtin_size_matrix;

protected function ceval_relation
  input Values.Value in_value1;
  input Exp.Operator in_operator2;
  input Values.Value in_value3;
  output Values.Value out_value;
algorithm 
  out_value:=
  matchcontinue (in_value1,in_operator2,in_value3)
    local
      Values.Value v,v1,v2;
      Exp.Type t;
      Boolean b;
      Integer i1,i2;
    case (v1,Exp.GREATER(t),v2)
      equation 
        v = ceval_relation(v2, Exp.LESS(t), v1); then v;
    case (Values.INTEGER(i1),Exp.LESS(Exp.INT()),Values.INTEGER(i2)) "*  Integers "
      equation 
        b = (i1 < i2); then Values.BOOL(b);
    case (Values.INTEGER(i1),Exp.EQUAL(Exp.INT()),Values.INTEGER(i2))
      equation 
        b = (i1 == i2); then Values.BOOL(b);
    case (Values.INTEGER(i1),Exp.NEQUAL(Exp.INT()),Values.INTEGER(i2)) "*  Reals "
      equation 
        b = (i1 <> i2); then Values.BOOL(b);
    case (Values.REAL(i1),Exp.LESS(Exp.REAL()),Values.REAL(i2)) "*  Reals "
      local Real i1,i2;
      equation 
        b = (i1 <. i2); then Values.BOOL(b);
    case (Values.REAL(i1),Exp.EQUAL(Exp.REAL()),Values.REAL(i2))
      local Real i1,i2;
      equation 
        b = (i1 ==. i2); then Values.BOOL(b);
    case (Values.REAL(i1),Exp.NEQUAL(Exp.REAL()),Values.REAL(i2))
      local Real i1,i2;
      equation 
        b = (i1 <>. i2); then Values.BOOL(b);
    case (_,_,_)
      equation 
        Print.print_buf("- ceval_relation failed\n"); then fail();
  end matchcontinue;
end ceval_relation;

protected function ceval_range "* function: ceval_range
 **
 ** This function evaluates a range expression.  It only handles integers.
 *"
  input Integer in_integer1;
  input Integer in_integer2;
  input Integer in_integer3;
  output Values_ValueList out_values_valuelist;
protected 
  type Values_ValueList = list<Values.Value>;
algorithm 
  out_values_valuelist:=
  matchcontinue (in_integer1,in_integer2,in_integer3)
    local
      Integer start,stop,j,d,k,step;
      Boolean b1,b2,c1,b3,b4,c2;
      Values_ValueList res;
    case (start,_,stop)
      equation 
        (start == stop) = true " e.g. 1:1 => {1} "; then {Values.INTEGER(start)};
    case (j,d,k)
      equation 
        b1 = (j > k) "if d > 0 and j>k or if d < 0 and j<k ";
        b2 = (d > 0);
        c1 = bool_and(b1, b2);
        b3 = (j < k);
        b4 = (d < 0);
        c2 = bool_and(b3, b4);
        true = bool_or(c1, c1); then {};
    case (start,step,stop)
      equation 
        res = ceval_range2(start, step, stop); then res;
  end matchcontinue;
end ceval_range;

protected function ceval_range2
  input Integer in_integer1;
  input Integer in_integer2;
  input Integer in_integer3;
  output Values_ValueList out_values_valuelist;
protected 
  type Values_ValueList = list<Values.Value>;
algorithm 
  out_values_valuelist:=
  matchcontinue (in_integer1,in_integer2,in_integer3)
    local
      Integer start,stop,next,step;
      Values_ValueList l;
    case (start,_,stop)
      equation 
        (start > stop) = true; then {};
    case (start,step,stop)
      equation 
        (start > stop) = false;
        next = start + step " redundant ";
        l = ceval_range2(next, step, stop); then Values.INTEGER(start) :: l;
  end matchcontinue;
end ceval_range2;

protected function ceval_range_real "* function: ceval_range_real
 **
 ** This function evaluates a range expression.  It only handles reals
 *"
  input Real in_real1;
  input Real in_real2;
  input Real in_real3;
  output Values_ValueList out_values_valuelist;
protected 
  type Values_ValueList = list<Values.Value>;
algorithm 
  out_values_valuelist:=
  matchcontinue (in_real1,in_real2,in_real3)
    local
      Real start,stop,j,d,k;
      Boolean b1,b2,c1,b3,b4,c2;
      Values_ValueList res;
    case (start,_,stop)
      equation 
        (start ==. stop) = true " e.g. 1:1 => {1} "; then {Values.REAL(start)};
    case (j,d,k)
      equation 
        b1 = (j >. k) "if d > 0 and j>k or if d < 0 and j<k ";
        b2 = (d >. 0.0);
        c1 = bool_and(b1, b2);
        b3 = (j <. k);
        b4 = (d <. 0.0);
        c2 = bool_and(b3, b4);
        true = bool_or(c1, c1); then {};
    case (j,d,k)
      equation 
        res = ceval_range_real2(j, d, k); then res;
  end matchcontinue;
end ceval_range_real;

protected function ceval_range_real2
  input Real in_real1;
  input Real in_real2;
  input Real in_real3;
  output Values_ValueList out_values_valuelist;
protected 
  type Values_ValueList = list<Values.Value>;
algorithm 
  out_values_valuelist:=
  matchcontinue (in_real1,in_real2,in_real3)
    local
      Real start,stop,next,step;
      Values_ValueList l;
    case (start,_,stop)
      equation 
        (start >. stop) = true; then {};
    case (start,step,stop)
      equation 
        (start >. stop) = false;
        next = start +. step " redundant ";
        l = ceval_range_real2(next, step, stop); then Values.REAL(start) :: l;
  end matchcontinue;
end ceval_range_real2;

public function ceval_list
  input Env.Env in_env;
  input Exp_ExpList in_exp_explist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Values_ValueList out_values_valuelist;
protected 
  type Exp_ExpList = list<Exp.Exp>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
  type Values_ValueList = list<Values.Value>;
algorithm 
  out_values_valuelist:=
  matchcontinue (in_env,in_exp_explist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      Env_FrameList env;
      Values.Value v;
      Exp.Exp exp;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st;
      Values_ValueList vs;
      Exp_ExpList exps;
    case (env,{},_,_) then {}; 
    case (env,{exp},impl,st)
      equation 
        (v,_) = ceval(env, exp, impl, st, NONE); then {v};
    case (env,exp :: exps,impl,st)
      equation 
        (v,_) = ceval(env, exp, impl, st, NONE);
        vs = ceval_list(env, exps, impl, st); then v :: vs;
  end matchcontinue;
end ceval_list;

protected function ceval_cref ""
  input Env.Env in_env;
  input Exp.ComponentRef in_componentref;
  input Boolean in_boolean;
  output Values.Value out_value;
algorithm 
  out_value:=
  matchcontinue (in_env,in_componentref,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      Types.Binding binding;
      Values.Value v;
      Env_FrameList env;
      Exp.ComponentRef c;
      Boolean impl;
      String str;
    case (env,c,impl) "impl  Uncomment this when struct params work    rule	(* Parameters should not be evaluated. Exception: structural parameters *)
          Lookup.lookup_var (env, c) => (Types.ATTR(_,_,SCode.PARAM,_),_,binding) 
          -------------------------------------
          ceval_cref (env,c) => fail   FIXME: For now, we don't try to check binding if implicit instantiation  
          rule	Debug.fprintln(\"ceval\", \"ceval_cref: Skipping ceval_cref_binding because implicit inst\")
	  ----------------------------------------------------------------------------
	  ceval_cref (env,c,impl as true) => fail

           as false"
      equation 
        (_,_,binding) = Lookup.lookup_var(env, c);
        v = ceval_cref_binding(env, c, binding, impl); then v;
    case (_,c,(impl = false)) " default "
      equation 
        Print.print_error_buf("# No constant binding for ") "as false";
        str = Exp.print_component_ref_str(c);
        Print.print_error_buf(str);
        Print.print_error_buf("\n"); then fail();
  end matchcontinue;
end ceval_cref;

protected function ceval_cref_binding
  input Env.Env in_env;
  input Exp.ComponentRef in_componentref;
  input Types.Binding in_binding;
  input Boolean in_boolean;
  output Values.Value out_value;
algorithm 
  out_value:=
  matchcontinue (in_env,in_componentref,in_binding,in_boolean)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type IntegerList = list<Integer>;
      type Env_FrameList = list<Env.Frame>;
      type Exp_SubscriptList = list<Exp.Subscript>;
      TupleTypes_TTypeAbsyn_PathOption tp;
      IntegerList sizelst;
      Values.Value res,v;
      Env_FrameList env;
      String id,rfn,iter,expstr;
      Exp_SubscriptList subsc;
      Boolean impl;
      Exp.Exp elexp,iterexp,exp;
    case (env,Exp.CREF_IDENT(id,subsc),Types.VALBOUND(v),impl) "impl  FIXME: For now, we don't try to check binding if implicit instantiation   
          rule	Debug.fprintln(\"ceval\", \"ceval_cref_binding: shouldn't have been called.\") &
	  Debug.fprintln(\"ceval\", \"Skipping because implicit inst\")
	  ----------------------------------------------------------------------------
	  ceval_cref_binding (_,_,_,impl as true ) => fail
          "
      equation 
        (_,tp,_) = Lookup.lookup_var(env, Exp.CREF_IDENT(id,{}));
        sizelst = Types.get_dimension_sizes(tp);
        res = ceval_subscript_value(env, subsc, v, sizelst, impl); then res;
    case (env,_,Types.UNBOUND(),(impl = false))
      equation 
        Print.print_buf("- ceval_cref_binding failed (UNBOUND)\n"); then fail();
    case (env,_,Types.UNBOUND(),(impl = true))
      equation 
        Debug.fprint("ceval", 
          "#- ceval_cref__binding: Ignoring unbound when implicit"); then fail();
    case (env,Exp.CREF_IDENT(id,subsc),Types.EQBOUND(exp,true),impl)
      equation 
        Exp.REDUCTION(Absyn.IDENT(rfn),elexp,iter,iterexp) = exp;
        equality(rfn = "array");
        Debug.fprintln("ceval", "#-- ceval_cref_binding Array evaluation"); then fail();
    case (env,Exp.CREF_IDENT(id,subsc),Types.EQBOUND(exp,true),impl)
      equation 
        Exp.REDUCTION(Absyn.IDENT(rfn),elexp,iter,iterexp) = exp;
        failure(equality(rfn = "array"));
        (_,tp,_) = Lookup.lookup_var(env, Exp.CREF_IDENT(id,{}));
        sizelst = Types.get_dimension_sizes(tp);
        (v,_) = ceval(env, exp, impl, NONE, NONE);
        res = ceval_subscript_value(env, subsc, v, sizelst, impl); then res;
    case (env,Exp.CREF_IDENT(id,subsc),Types.EQBOUND(exp,true),impl)
      equation 
        (_,tp,_) = Lookup.lookup_var(env, Exp.CREF_IDENT(id,{}));
        sizelst = Types.get_dimension_sizes(tp);
        (v,_) = ceval(env, exp, impl, NONE, NONE);
        res = ceval_subscript_value(env, subsc, v, sizelst, impl); then res;
    case (env,_,Types.EQBOUND(exp,false),impl)
      equation 
        Debug.fprint("ceval", 
          "#- ceval_cref__binding failed (nonconstant EQBOUND(");
        expstr = Exp.print_exp_str(exp);
        Debug.fprint("ceval", expstr);
        Debug.fprintln("ceval", "))"); then fail();
  end matchcontinue;
end ceval_cref_binding;

protected function ceval_subscript_value
  input Env.Env in_env;
  input Exp_SubscriptList in_exp_subscriptlist;
  input Values.Value in_value;
  input IntegerList in_integerlist;
  input Boolean in_boolean;
  output Values.Value out_value;
protected 
  type Exp_SubscriptList = list<Exp.Subscript>;
  type IntegerList = list<Integer>;
algorithm 
  out_value:=
  matchcontinue (in_env,in_exp_subscriptlist,in_value,in_integerlist,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      type Values_ValueList = list<Values.Value>;
      Integer n,n_1,dim;
      Values.Value subval,res,v;
      Env_FrameList env;
      Exp.Exp exp;
      Exp_SubscriptList subs;
      Values_ValueList lst;
      IntegerList dims;
      Boolean impl;
    case (env,Exp.INDEX(exp) :: subs,Values.ARRAY(lst),dim :: dims,impl) "impl"
      equation 
        (Values.INTEGER(n),_) = ceval(env, exp, impl, NONE, SOME(dim));
        n_1 = n - 1;
        subval = list_nth(lst, n_1);
        res = ceval_subscript_value(env, subs, subval, dims, impl); then res;
    case (env,{},v,_,_) then v; 
  end matchcontinue;
end ceval_subscript_value;

public function ceval_subscripts
  input Env.Env in_env;
  input Exp_SubscriptList in_exp_subscriptlist;
  input IntegerList in_integerlist;
  input Boolean in_boolean;
  output Exp_SubscriptList out_exp_subscriptlist;
protected 
  type Exp_SubscriptList = list<Exp.Subscript>;
  type IntegerList = list<Integer>;
algorithm 
  out_exp_subscriptlist:=
  matchcontinue (in_env,in_exp_subscriptlist,in_integerlist,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      Exp.Subscript sub_1,sub;
      Exp_SubscriptList subs_1,subs;
      Env_FrameList env;
      Integer dim;
      IntegerList dims;
      Boolean impl;
    case (_,{},_,_) then {}; 
    case (env,sub :: subs,dim :: dims,impl)
      equation 
        sub_1 = ceval_subscript(env, sub, dim, impl);
        subs_1 = ceval_subscripts(env, subs, dims, impl); then sub_1 :: subs_1;
  end matchcontinue;
end ceval_subscripts;

protected function ceval_subscript "* function: ceval_subscript
 **
 ** This function relates a subscript to its canonical forms, which is
 ** when all expressions are evaluated to constant values.
 *"
  input Env.Env in_env;
  input Exp.Subscript in_subscript;
  input Integer in_integer;
  input Boolean in_boolean;
  output Exp.Subscript out_subscript;
algorithm 
  out_subscript:=
  matchcontinue (in_env,in_subscript,in_integer,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      Env_FrameList env;
      Values.Value v1;
      Exp.Exp e1_1,e1;
      Integer dim;
      Boolean impl;
    case (env,Exp.WHOLEDIM(),_,_) then Exp.WHOLEDIM(); 
    case (env,Exp.INDEX(e1),dim,impl)
      equation 
        (v1,_) = ceval(env, e1, impl, NONE, SOME(dim));
        e1_1 = Static.value_exp(v1); then Exp.INDEX(e1_1);
    case (env,Exp.SLICE(e1),dim,impl)
      equation 
        (v1,_) = ceval(env, e1, impl, NONE, SOME(dim));
        e1_1 = Static.value_exp(v1); then Exp.SLICE(e1_1);
  end matchcontinue;
end ceval_subscript;
end Ceval;

