package ClassInf "
    Copyright (c) PELAB, Dept. Computer and Information Science, Linkoping University,
    Sweden, in the following denoted PELAB/LIU.

   
    This file is part of OpenModelica, in the following denoted OSM.


    The OSM License, Version 1.1, March 4, 2005.


    1 Preface

    The aim of this license is to lay down the conditions enabling you to use, 
    modify and circulate OSM. However, PELAB/LIU remain the authors of 
    OSM and so retain property rights and the use of all ancillary rights.

    2 Definitions

    OSM is defined as all successive versions of the OSM software and their 
    documentation that have been developed by PELAB/LIU and including
    accepted contributions from other contributors according to this license.

    OSM DERIVED SOFTWARE is defined as all or part of OSM that you have 
    modified and/or translated and/or adapted.


    3 Dual License

    OSM is made available under the OSM licensing scheme, which is a dual
    licensing scheme with two options, a) and b):

    a) OSM OPEN SOURCE LICENSE:

    If you wish to write Open Source software you can use the Open Source version 
    of OSM, released under the OSM license which include GPL as its open source 
    licensing option. If you use the OSM Open Source version you must release your 
    Application using OSM including this Application's source code under the GPL as well.

    This OSM license text, and Copyright (c) PELAB/Linkoping University, must
    be present in your copy of OSM and in OSM DERIVED SOFTWARE.

    You should have received a copy of the GPL - GNU General Public License
    along with OpenModelica; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.


    b) OSM COMMERCIAL LICENSE

    If you are using OSM commercially - that is, for commercial usage or
    for creating proprietary software for sale or use in a commercial setting
    - you must purchase a commercial license of OSM from PELAB/L
  *
 ** file:	 ClassInf.rml
 ** module:      ClassInf
 ** description: Class restrictions
 **
 ** RCS:	 $Id: ClassInf.rml 1733 2005-04-27 06:47:45Z haklu $
 **
 ** This module deals with class inference, i.e. determining if a
 ** class definition adhers to one of the class restrictions, and, if
 ** specifically declared in a restrictied form, if it breaks that
 ** restriction.
 **
 ** The inference is implemented as a finite state machine.  The
 ** function `start' initializes a new machine, and the function
 ** `trans' signals transitions in the machine.  Finally, the state
 ** can be checked agains a restriction with the `valid' function.
 ** 
 *"

public import OpenModelica.Compiler.SCode;

public 
uniontype State "* - Machine states "
  record UNKNOWN
    String x1;
  end UNKNOWN;
  record MODEL
    String x1;
  end MODEL;
  record RECORD
    String x1;
  end RECORD;
  record BLOCK
    String x1;
  end BLOCK;
  record CONNECTOR
    String x1;
  end CONNECTOR;
  record TYPE
    String x1;
  end TYPE;
  record PACKAGE
    String x1;
  end PACKAGE;
  record FUNCTION
    String x1;
  end FUNCTION;
  record ENUMERATION
    String x1;
  end ENUMERATION;
  record HAS_EQUATIONS
    String x1;
  end HAS_EQUATIONS;
  record IS_NEW
    String x1;
  end IS_NEW;
  record TYPE_INTEGER
    String x1;
  end TYPE_INTEGER;
  record TYPE_REAL
    String x1;
  end TYPE_REAL;
  record TYPE_STRING
    String x1;
  end TYPE_STRING;
  record TYPE_BOOL
    String x1;
  end TYPE_BOOL;
  record TYPE_ENUM
    String x1;
  end TYPE_ENUM;
end State;

public 
uniontype Event "* - Events "
  record FOUND_EQUATION " There are definitions inside the current definition "
  end FOUND_EQUATION;
  record NEWDEF " This is not a derived class  * The `Event' type contains the different events during "
  end NEWDEF;
end Event;

protected import OpenModelica.Compiler.Print;

protected import OpenModelica.Compiler.Absyn;

public function print_state "* - Printing 
  
  * Some functions for printing error and debug information about the 
  * state machine.
 **
 ** The code is excluded from the report.
 *
  !ignorecode
  * function: print_state "
  input State in_state;
algorithm 
  _:=
  matchcontinue (in_state)
    local String s;
    case UNKNOWN(s)
      equation 
        Print.print_buf("UNKNOWN ");
        Print.print_buf(s); then ();
    case MODEL(s)
      equation 
        Print.print_buf("MODEL ");
        Print.print_buf(s); then ();
    case RECORD(s)
      equation 
        Print.print_buf("RECORD ");
        Print.print_buf(s); then ();
    case BLOCK(s)
      equation 
        Print.print_buf("BLOCK ");
        Print.print_buf(s); then ();
    case CONNECTOR(s)
      equation 
        Print.print_buf("CONNECTOR ");
        Print.print_buf(s); then ();
    case TYPE(s)
      equation 
        Print.print_buf("TYPE ");
        Print.print_buf(s); then ();
    case PACKAGE(s)
      equation 
        Print.print_buf("PACKAGE ");
        Print.print_buf(s); then ();
    case FUNCTION(s)
      equation 
        Print.print_buf("FUNCTION ");
        Print.print_buf(s); then ();
    case TYPE_INTEGER(s)
      equation 
        Print.print_buf("TYPE_INTEGER ");
        Print.print_buf(s); then ();
    case TYPE_REAL(s)
      equation 
        Print.print_buf("TYPE_REAL ");
        Print.print_buf(s); then ();
    case TYPE_STRING(s)
      equation 
        Print.print_buf("TYPE_STRING ");
        Print.print_buf(s); then ();
    case TYPE_BOOL(s)
      equation 
        Print.print_buf("TYPE_BOOL ");
        Print.print_buf(s); then ();
    case IS_NEW(s)
      equation 
        Print.print_buf("IS_NEW ");
        Print.print_buf(s); then ();
    case HAS_EQUATIONS(s)
      equation 
        Print.print_buf("HAS_EQUATIONS ");
        Print.print_buf(s); then ();
  end matchcontinue;
end print_state;

public function get_state_name
  input State in_state;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_state)
    local String s;
    case UNKNOWN(s) then s; 
    case MODEL(s) then s; 
    case RECORD(s) then s; 
    case BLOCK(s) then s; 
    case CONNECTOR(s) then s; 
    case TYPE(s) then s; 
    case PACKAGE(s) then s; 
    case FUNCTION(s) then s; 
    case TYPE_INTEGER(s) then s; 
    case TYPE_REAL(s) then s; 
    case TYPE_STRING(s) then s; 
    case TYPE_BOOL(s) then s; 
    case IS_NEW(s) then s; 
    case HAS_EQUATIONS(s) then s; 
  end matchcontinue;
end get_state_name;

protected function print_event "* function: print_event "
  input Event in_event;
algorithm 
  _:=
  matchcontinue (in_event)
    case FOUND_EQUATION()
      equation 
        Print.print_buf("FOUND_EQUATION"); then ();
    case NEWDEF()
      equation 
        Print.print_buf("NEWDEF"); then ();
  end matchcontinue;
end print_event;

public function start "!includecode
  * - Transitions 
  * function: start
 **
 ** This is the  state machine initialization function.
 *"
  input SCode.Restriction in_restriction;
  input String in_string;
  output State out_state;
algorithm 
  out_state:=
  matchcontinue (in_restriction,in_string)
    local String s;
    case (SCode.R_CLASS(),s) then UNKNOWN(s); 
    case (SCode.R_MODEL(),s) then MODEL(s); 
    case (SCode.R_RECORD(),s) then RECORD(s); 
    case (SCode.R_BLOCK(),s) then BLOCK(s); 
    case (SCode.R_CONNECTOR(),s) then CONNECTOR(s); 
    case (SCode.R_TYPE(),s) then TYPE(s); 
    case (SCode.R_PACKAGE(),s) then PACKAGE(s); 
    case (SCode.R_FUNCTION(),s) then FUNCTION(s); 
    case (SCode.R_EXT_FUNCTION(),s) then FUNCTION(s); 
    case (SCode.R_ENUMERATION(),s) then ENUMERATION(s); 
    case (SCode.R_PREDEFINED_INT(),s) then TYPE_INTEGER(s); 
    case (SCode.R_PREDEFINED_REAL(),s) then TYPE_REAL(s); 
    case (SCode.R_PREDEFINED_STRING(),s) then TYPE_STRING(s); 
    case (SCode.R_PREDEFINED_BOOL(),s) then TYPE_BOOL(s); 
    case (SCode.R_PREDEFINED_ENUM(),s) then TYPE_ENUM(s); 
  end matchcontinue;
end start;

public function trans "* function: trans
 **
 ** This is the state machine transition function.  It describes the
 ** transitions between states at different events.
 *"
  input State in_state;
  input Event in_event;
  output State out_state;
algorithm 
  out_state:=
  matchcontinue (in_state,in_event)
    local
      String s;
      State st;
      Event ev;
    case (UNKNOWN(s),NEWDEF()) then IS_NEW(s); 
    case (MODEL(s),NEWDEF()) then MODEL(s); 
    case (RECORD(s),NEWDEF()) then RECORD(s); 
    case (BLOCK(s),NEWDEF()) then BLOCK(s); 
    case (CONNECTOR(s),NEWDEF()) then CONNECTOR(s); 
    case (TYPE(s),NEWDEF())
      equation 
        Print.print_error_buf("# In class ");
        Print.print_error_buf(s);
        Print.print_error_buf("\n");
        Print.print_error_buf("# 'type' can only be derived from predefined types\n"); then fail();
    case (PACKAGE(s),NEWDEF()) then PACKAGE(s); 
    case (FUNCTION(s),NEWDEF()) then FUNCTION(s); 
    case (ENUMERATION(s),NEWDEF()) then ENUMERATION(s); 
    case (IS_NEW(s),NEWDEF()) then IS_NEW(s); 
    case (TYPE_INTEGER(s),NEWDEF()) then TYPE_INTEGER(s); 
    case (TYPE_REAL(s),NEWDEF()) then TYPE_REAL(s); 
    case (TYPE_STRING(s),NEWDEF()) then TYPE_STRING(s); 
    case (TYPE_BOOL(s),NEWDEF()) then TYPE_BOOL(s); 
    case (TYPE_ENUM(s),NEWDEF()) then TYPE_ENUM(s);  "* Event `FOUND_EQUATION' "
    case (UNKNOWN(s),FOUND_EQUATION()) then HAS_EQUATIONS(s); 
    case (IS_NEW(s),FOUND_EQUATION()) then HAS_EQUATIONS(s); 
    case (MODEL(s),FOUND_EQUATION()) then MODEL(s); 
    case (RECORD(s),FOUND_EQUATION())
      equation 
        Print.print_error_buf("# In class ");
        Print.print_error_buf(s);
        Print.print_error_buf("\n");
        Print.print_error_buf("# Equations not allowed in records\n"); then fail();
    case (BLOCK(s),FOUND_EQUATION()) then BLOCK(s); 
    case (CONNECTOR(s),FOUND_EQUATION())
      equation 
        Print.print_error_buf("# In class ");
        Print.print_error_buf(s);
        Print.print_error_buf("\n");
        Print.print_error_buf("# Equations not allowed in connectors\n"); then fail();
    case (TYPE(s),FOUND_EQUATION()) then fail(); 
    case (PACKAGE(s),FOUND_EQUATION()) then fail(); 
    case (FUNCTION(s),FOUND_EQUATION()) then fail(); 
    case (HAS_EQUATIONS(s),FOUND_EQUATION()) then HAS_EQUATIONS(s); 
    case (st,ev)
      equation 
        Print.print_buf("- trans failed: ");
        print_state(st);
        Print.print_buf(", ");
        print_event(ev);
        Print.print_buf("\n"); then fail();
  end matchcontinue;
end trans;

public function valid "* function: valid
 **
 ** This is the validity function which determines if a state is valid
 ** according to one of the restrictions.  This means, that if a class
 ** definition is to be used as, say, a connector, the state of the
 ** state machine is checked against the `SCode.R_CONNECTOR'
 ** restriction using this function to find out if it is an error to
 ** use this class definition as a connector.
 *"
  input State in_state;
  input SCode.Restriction in_restriction;
algorithm 
  _:=
  matchcontinue (in_state,in_restriction)
    local String s;
    case (UNKNOWN(s),_) then (); 
    case (IS_NEW(s),SCode.R_CLASS()) then (); 
    case (HAS_EQUATIONS(s),SCode.R_CLASS()) then (); 
    case (MODEL(s),SCode.R_MODEL()) then (); 
    case (IS_NEW(s),SCode.R_MODEL()) then (); 
    case (HAS_EQUATIONS(s),SCode.R_MODEL()) then (); 
    case (RECORD(s),SCode.R_RECORD()) then (); 
    case (IS_NEW(s),SCode.R_RECORD()) then (); 
    case (BLOCK(s),SCode.R_BLOCK()) then (); 
    case (HAS_EQUATIONS(s),SCode.R_BLOCK()) then (); 
    case (CONNECTOR(_),SCode.R_CONNECTOR()) then (); 
    case (IS_NEW(_),SCode.R_CONNECTOR()) then (); 
    case (TYPE_INTEGER(_),SCode.R_CONNECTOR()) then (); 
    case (TYPE_REAL(_),SCode.R_CONNECTOR()) then (); 
    case (TYPE_STRING(_),SCode.R_CONNECTOR()) then (); 
    case (TYPE_BOOL(_),SCode.R_CONNECTOR()) then (); 
    case (TYPE(s),SCode.R_TYPE()) then (); 
    case (TYPE_INTEGER(s),SCode.R_TYPE()) then (); 
    case (TYPE_REAL(s),SCode.R_TYPE()) then (); 
    case (TYPE_STRING(s),SCode.R_TYPE()) then (); 
    case (TYPE_BOOL(s),SCode.R_TYPE()) then (); 
    case (IS_NEW(s),SCode.R_PACKAGE()) then (); 
    case (PACKAGE(s),SCode.R_PACKAGE()) then (); 
    case (IS_NEW(s),SCode.R_FUNCTION()) then (); 
    case (FUNCTION(s),SCode.R_FUNCTION()) then (); 
  end matchcontinue;
end valid;

public function assert_valid "* function: assert_valid
 **
 ** This function has the same semantical meaning as the function
 ** `valid'.  However, it prints an error message when it fails.
 *"
  input State in_state;
  input SCode.Restriction in_restriction;
algorithm 
  _:=
  matchcontinue (in_state,in_restriction)
    local
      State st;
      SCode.Restriction re;
      String str;
    case (st,re)
      equation 
        valid(st, re); then ();
    case (st,re)
      equation 
        Print.print_error_buf("# Restriction violation: ");
        str = get_state_name(st);
        Print.print_error_buf(str);
        Print.print_error_buf(" is not a ");
        str = SCode.restr_string(re);
        Print.print_error_buf(str);
        Print.print_error_buf("\n"); then fail();
  end matchcontinue;
end assert_valid;

public function matching_state
  input State in_state;
  input StateList in_statelist;
  output Boolean out_boolean;
protected 
  type StateList = list<State>;
algorithm 
  out_boolean:=
  matchcontinue (in_state,in_statelist)
    local
      State st,first;
      StateList rest;
      Boolean res;
    case (st,{}) then false; 
    case (UNKNOWN(_),UNKNOWN(_) :: rest) then true; 
    case (MODEL(_),MODEL(_) :: rest) then true; 
    case (RECORD(_),RECORD(_) :: rest) then true; 
    case (BLOCK(_),BLOCK(_) :: rest) then true; 
    case (CONNECTOR(_),CONNECTOR(_) :: rest) then true; 
    case (TYPE(_),TYPE(_) :: rest) then true; 
    case (PACKAGE(_),PACKAGE(_) :: rest) then true; 
    case (FUNCTION(_),FUNCTION(_) :: rest) then true; 
    case (ENUMERATION(_),ENUMERATION(_) :: rest) then true; 
    case (HAS_EQUATIONS(_),HAS_EQUATIONS(_) :: rest) then true; 
    case (IS_NEW(_),IS_NEW(_) :: rest) then true; 
    case (TYPE_INTEGER(_),TYPE_INTEGER(_) :: rest) then true; 
    case (TYPE_REAL(_),TYPE_REAL(_) :: rest) then true; 
    case (TYPE_STRING(_),TYPE_STRING(_) :: rest) then true; 
    case (TYPE_BOOL(_),TYPE_BOOL(_) :: rest) then true; 
    case (TYPE_ENUM(_),TYPE_ENUM(_) :: rest) then true; 
    case (st,first :: rest)
      equation 
        res = matching_state(st, rest); then res;
  end matchcontinue;
end matching_state;

public function is_function
  input State in_state;
algorithm 
  _:=
  matchcontinue (in_state)
    case FUNCTION(_) then (); 
  end matchcontinue;
end is_function;
end ClassInf;

