package Derive "
    Copyright (c) PELAB, Dept. Computer and Information Science, Linkoping University,
    Sweden, in the following denoted PELAB/LIU.

   
    This file is part of OpenModelica, in the following denoted OSM.


    The OSM License, Version 1.1, March 4, 2005.


    1 Preface

    The aim of this license is to lay down the conditions enabling you to use, 
    modify and circulate OSM. However, PELAB/LIU remain the authors of 
    OSM and so retain property rights and the use of all ancillary rights.

    2 Definitions

    OSM is defined as all successive versions of the OSM software and their 
    documentation that have been developed by PELAB/LIU and including
    accepted contributions from other contributors according to this license.

    OSM DERIVED SOFTWARE is defined as all or part of OSM that you have 
    modified and/or translated and/or adapted.


    3 Dual License

    OSM is made available under the OSM licensing scheme, which is a dual
    licensing scheme with two options, a) and b):

    a) OSM OPEN SOURCE LICENSE:

    If you wish to write Open Source software you can use the Open Source version 
    of OSM, released under the OSM license which include GPL as its open source 
    licensing option. If you use the OSM Open Source version you must release your 
    Application using OSM including this Application's source code under the GPL as well.

    This OSM license text, and Copyright (c) PELAB/Linkoping University, must
    be present in your copy of OSM and in OSM DERIVED SOFTWARE.

    You should have received a copy of the GPL - GNU General Public License
    along with OpenModelica; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.


    b) OSM COMMERCIAL LICENSE

    If you are using OSM commercially - that is, for commercial usage or
    for creating proprietary software for sale or use in a commercial setting
    - you must purchase a commercial license of OSM from PELAB/L
  *
 ** file:	 Derive.rml
 ** module:      Derive
 ** description: Differentiation of equations from DAELow
 **
 ** RCS: $Id: Derive.rml 1733 2005-04-27 06:47:45Z haklu $
 ** 
 ** This module is responsible for symbolic differentiation of equations and 
 ** expressions. Is is currently (2004-09-28) only used by the solve function in 
 ** the exp module for solving equations.
 ** 
 ** The symbolic differentiation is used in the Newton-Raphson method and in
 ** index reduction.
 ** 
 ** 
 *"

public import OpenModelica.Compiler.DAELow;

public import OpenModelica.Compiler.Exp;

protected import OpenModelica.Compiler.Absyn;

protected import OpenModelica.Compiler.Util;

protected import OpenModelica.Compiler.Print;

protected import OpenModelica.Compiler.Graphviz;

public function differentiate_equation_time " time varying variables "
  input DAELow.Equation in_equation;
  input DAELow.Variables in_variables;
  output DAELow.Equation out_equation;
algorithm 
  out_equation:=
  matchcontinue (in_equation,in_variables)
    local
      Exp.Exp e1_1,e2_1,e1_2,e2_2,e1,e2;
      DAELow.Variables timevars;
    case (DAELow.EQUATION(e1,e2),timevars) " time varying variables "
      equation 
        e1_1 = differentiate_exp_time(e1, timevars);
        e2_1 = differentiate_exp_time(e2, timevars);
        e1_2 = Exp.simplify(e1_1);
        e2_2 = Exp.simplify(e2_1) "& Exp.simplify(e1'') => e1'' &
	Exp.simplify(e2'') => e2'' "; then DAELow.EQUATION(e1_2,e2_2);
    case (DAELow.ALGORITHM(_,_,_),_)
      equation 
        print("-differentiate_equation_time on algorithm not impl yet.\n"); then fail();
    case (_,_)
      equation 
        print("-differentiate_equation_time faile\n"); then fail();
  end matchcontinue;
end differentiate_equation_time;

public function differentiate_exp_time "* function: differentiate_exp_time
 ** This function differentiates expressions with respect to the 'time' variable.
 ** All other variables that are varying over time are given as the second variable.
 ** For instance, given the model:
 ** model test 
 **   Real x,y; 
 **   parameter Real PI=3.14; 
 ** equation
 **   x+y=5*PI;	
 ** end test;
 ** gives
 ** differentiate_exp_time('x+y=5*PI', {x,y}) => der(x)+der(y)=0
 *"
  input Exp.Exp in_exp;
  input DAELow.Variables in_variables;
  output Exp.Exp out_exp;
algorithm 
  out_exp:=
  matchcontinue (in_exp,in_variables)
    local
      type Exp_ExpList = list<Exp.Exp>;
      Exp.Type tp;
      Exp.Exp e,e_1,e1_1,e2_1,e1,e2,e3_1,e3;
      Exp.ComponentRef cr;
      DAELow.Variables timevars,tv;
      Exp.Operator op,rel;
      Exp_ExpList expl_1,expl;
      Absyn.Path a;
      Boolean b,c;
      Integer i;
      String str;
    case (Exp.ICONST(_),_) then Exp.RCONST(0.0); 
    case (Exp.RCONST(_),_) then Exp.RCONST(0.0); 
    case (Exp.CREF(Exp.CREF_IDENT("time",{}),tp),_) then Exp.RCONST(1.0); 
    case ((e = Exp.CREF(cr,tp)),timevars)
      equation 
        (_,_) = DAELow.get_var(cr, timevars); then Exp.CALL(Absyn.IDENT("der"),{e},false,true);
    case (Exp.CALL(Absyn.IDENT("sin"),{e},false,true),timevars)
      equation 
        e_1 = differentiate_exp_time(e, timevars) " der(sin(x)) = der(x)*cos(x) "; then Exp.BINARY(e_1,Exp.MUL(Exp.REAL()),
          Exp.CALL(Absyn.IDENT("cos"),{e},false,true));
    case ((e = Exp.CREF(cr,tp)),timevars) then Exp.RCONST(0.0); 
    case (Exp.BINARY(e1,Exp.ADD(tp),e2),tv)
      equation 
        e1_1 = differentiate_exp_time(e1, tv);
        e2_1 = differentiate_exp_time(e2, tv); then Exp.BINARY(e1_1,Exp.ADD(tp),e2_1);
    case (Exp.BINARY(e1,Exp.SUB(tp),e2),tv)
      equation 
        e1_1 = differentiate_exp_time(e1, tv);
        e2_1 = differentiate_exp_time(e2, tv); then Exp.BINARY(e1_1,Exp.SUB(tp),e2_1);
    case (Exp.BINARY(e1,Exp.MUL(tp),e2),tv) " f'g + fg' "
      equation 
        e1_1 = differentiate_exp_time(e1, tv);
        e2_1 = differentiate_exp_time(e2, tv); then Exp.BINARY(Exp.BINARY(e1,Exp.MUL(tp),e2_1),Exp.ADD(tp),
          Exp.BINARY(e1_1,Exp.MUL(tp),e2));
    case (Exp.BINARY(e1,Exp.DIV(tp),e2),tv) " (f'g - fg' ) / g^2 "
      equation 
        e1_1 = differentiate_exp_time(e1, tv);
        e2_1 = differentiate_exp_time(e2, tv); then Exp.BINARY(
          Exp.BINARY(Exp.BINARY(e1_1,Exp.MUL(tp),e2),Exp.SUB(tp),
          Exp.BINARY(e1,Exp.MUL(tp),e2_1)),Exp.DIV(tp),Exp.BINARY(e2,Exp.MUL(tp),e2));
    case (Exp.UNARY(op,e),tv)
      equation 
        e_1 = differentiate_exp_time(e, tv); then Exp.UNARY(op,e_1);
    case (Exp.LBINARY(e1,op,e2),tv)
      equation 
        Exp.BINARY(e1_1,op,e2_1) = differentiate_exp_time(Exp.BINARY(e1,op,e2), tv); then Exp.LBINARY(e1_1,op,e2_1);
    case (Exp.LUNARY(op,e),tv)
      equation 
        e_1 = differentiate_exp_time(e, tv); then Exp.LUNARY(op,e_1);
    case (Exp.RELATION(e1,rel,e2),tv)
      equation 
        e1_1 = differentiate_exp_time(e1, tv);
        e2_1 = differentiate_exp_time(e2, tv); then Exp.RELATION(e1_1,rel,e2_1);
    case (Exp.IFEXP(e1,e2,e3),tv)
      equation 
        e1_1 = differentiate_exp_time(e1, tv);
        e2_1 = differentiate_exp_time(e2, tv);
        e3_1 = differentiate_exp_time(e3, tv); then Exp.IFEXP(e1_1,e2_1,e3_1);
    case (Exp.CALL((a = Absyn.IDENT("der")),expl,b,c),tv)
      equation 
        expl_1 = Util.list_map_1(expl, differentiate_exp_time, tv); then Exp.CALL(a,expl_1,b,c);
    case (Exp.CALL(a,expl,b,c),tv)
      equation 
        print("Error. differentiating of function not impl. yet.\n");
        expl_1 = Util.list_map_1(expl, differentiate_exp_time, tv); then Exp.CALL(a,expl_1,b,c);
    case (Exp.ARRAY(tp,b,expl),tv)
      equation 
        expl_1 = Util.list_map_1(expl, differentiate_exp_time, tv); then Exp.ARRAY(tp,b,expl_1);
    case ((e = Exp.MATRIX(_,_,_)),_)
      equation 
        Print.print_buf("differentiate_exp_time on MATRIX not impl.\n");
        print("differentiate_exp_time on MATRIX not impl.\n"); then e;
    case (Exp.TUPLE(expl),tv)
      equation 
        expl_1 = Util.list_map_1(expl, differentiate_exp_time, tv); then Exp.TUPLE(expl_1);
    case (Exp.CAST(tp,e),tv)
      equation 
        e_1 = differentiate_exp_time(e, tv); then Exp.CAST(tp,e_1);
    case (Exp.ASUB(e,i),tv)
      equation 
        e_1 = differentiate_exp_time(e, tv); then Exp.ASUB(e,i);
    case (Exp.REDUCTION(a,e1,b,e2),tv)
      local String b;
      equation 
        e1_1 = differentiate_exp_time(e1, tv);
        e2_1 = differentiate_exp_time(e2, tv); then Exp.REDUCTION(a,e1_1,b,e2_1);
    case (e,tv)
      equation 
        str = Exp.print_exp_str(e);
        print("-differentiate_exp_time on ");
        print(str);
        print(" failed\n"); then fail();
  end matchcontinue;
end differentiate_exp_time;

public function differentiate_exp "* function: differenatiate_exp
 ** This function differentiates expressions with respect to a given variable, given as second argument.
 ** For example.
 ** differentiate_exp('2*x*y+2*x+y',x) => 2*x+2
 *"
  input Exp.Exp in_exp;
  input Exp.ComponentRef in_componentref;
  output Exp.Exp out_exp;
algorithm 
  out_exp:=
  matchcontinue (in_exp,in_componentref)
    local
      type Exp_ExpList = list<Exp.Exp>;
      Real rval;
      Exp.ComponentRef cr,crx,tv;
      Exp.Exp e,e1_1,e2_1,e1,e2,const_one,e_1,exp_1,exp,e3_1,e3;
      Exp.Type tp;
      Exp.Operator op,rel;
      Boolean b,c;
      Exp_ExpList expl_1,expl;
      Absyn.Path a;
      Integer i;
    case (Exp.ICONST(_),_) then Exp.RCONST(0.0); 
    case (Exp.RCONST(_),_) then Exp.RCONST(0.0); 
    case (Exp.CREF(cr,_),crx)
      equation 
        true = Exp.cref_equal(cr, crx) " D(x)/dx => 1 ";
        rval = int_real(1) " Since bug in RML makes 1.0 into 0.0 "; then Exp.RCONST(rval);
    case ((e = Exp.CREF(cr,_)),crx)
      equation 
        false = Exp.cref_equal(cr, crx) " D(c)/dx => 0 "; then Exp.RCONST(0.0);
    case (Exp.BINARY(e1,Exp.ADD(tp),e2),tv)
      equation 
        e1_1 = differentiate_exp(e1, tv);
        e2_1 = differentiate_exp(e2, tv); then Exp.BINARY(e1_1,Exp.ADD(tp),e2_1);
    case (Exp.BINARY(e1,Exp.SUB(tp),e2),tv)
      equation 
        e1_1 = differentiate_exp(e1, tv);
        e2_1 = differentiate_exp(e2, tv); then Exp.BINARY(e1_1,Exp.SUB(tp),e2_1);
    case (Exp.BINARY((e1 = Exp.CREF(cr,_)),Exp.POW(tp),e2),tv) " ax^(a-1) "
      equation 
        true = Exp.cref_equal(cr, tv) " a^x => ax^(a-1) ";
        false = Exp.exp_contains(e2, Exp.CREF(tv,tp));
        const_one = differentiate_exp(Exp.CREF(tv,tp), tv); then Exp.BINARY(e2,Exp.MUL(tp),
          Exp.BINARY(e1,Exp.POW(tp),Exp.BINARY(e2,Exp.SUB(tp),const_one)));
    case (Exp.BINARY(e1,Exp.MUL(tp),e2),tv) " f'g + fg' "
      equation 
        e1_1 = differentiate_exp(e1, tv);
        e2_1 = differentiate_exp(e2, tv); then Exp.BINARY(Exp.BINARY(e1,Exp.MUL(tp),e2_1),Exp.ADD(tp),
          Exp.BINARY(e1_1,Exp.MUL(tp),e2));
    case (Exp.BINARY(e1,Exp.DIV(tp),e2),tv) " (f'g - fg' ) / g^2 "
      equation 
        e1_1 = differentiate_exp(e1, tv);
        e2_1 = differentiate_exp(e2, tv); then Exp.BINARY(
          Exp.BINARY(Exp.BINARY(e1_1,Exp.MUL(tp),e2),Exp.SUB(tp),
          Exp.BINARY(e1,Exp.MUL(tp),e2_1)),Exp.DIV(tp),Exp.BINARY(e2,Exp.MUL(tp),e2));
    case (Exp.UNARY(op,e),tv)
      equation 
        e_1 = differentiate_exp(e, tv); then Exp.UNARY(op,e_1);
    case (Exp.CALL(Absyn.IDENT("sin"),exp :: {},b,c),tv)
      equation 
        true = Exp.exp_contains(exp, Exp.CREF(tv,Exp.REAL()));
        exp_1 = differentiate_exp(exp, tv); then Exp.BINARY(Exp.CALL(Absyn.IDENT("cos"),{exp},b,c),Exp.MUL(Exp.REAL()),
          exp_1);
    case (Exp.CALL(Absyn.IDENT("cos"),exp :: {},b,c),tv)
      equation 
        true = Exp.exp_contains(exp, Exp.CREF(tv,Exp.REAL()));
        exp_1 = differentiate_exp(exp, tv); then Exp.BINARY(
          Exp.UNARY(Exp.UMINUS(Exp.REAL()),
          Exp.CALL(Absyn.IDENT("sin"),{exp},b,c)),Exp.MUL(Exp.REAL()),exp_1);
    case (Exp.CALL(Absyn.IDENT("exp"),exp :: {},b,c),tv)
      equation 
        true = Exp.exp_contains(exp, Exp.CREF(tv,Exp.REAL())) " exp(x) => x' * exp(x) ";
        exp_1 = differentiate_exp(exp, tv); then Exp.BINARY(Exp.CALL(Absyn.IDENT("exp"),exp :: {},b,c),
          Exp.MUL(Exp.REAL()),exp_1);
    case (Exp.CALL(Absyn.IDENT("log"),exp :: {},b,c),tv)
      equation 
        true = Exp.exp_contains(exp, Exp.CREF(tv,Exp.REAL())) " log(x) => x' * 1/x ";
        exp_1 = differentiate_exp(exp, tv); then Exp.BINARY(exp_1,Exp.MUL(Exp.REAL()),
          Exp.BINARY(Exp.RCONST(1.0),Exp.DIV(Exp.REAL()),exp));
    case (Exp.CALL(Absyn.IDENT("log10"),exp :: {},b,c),tv)
      equation 
        true = Exp.exp_contains(exp, Exp.CREF(tv,Exp.REAL())) " log10(x) => x'*1/(x*log(10)) ";
        exp_1 = differentiate_exp(exp, tv); then Exp.BINARY(exp_1,Exp.MUL(Exp.REAL()),
          Exp.BINARY(Exp.RCONST(1.0),Exp.DIV(Exp.REAL()),
          Exp.BINARY(exp,Exp.MUL(Exp.REAL()),
          Exp.CALL(Absyn.IDENT("log"),{Exp.RCONST(10.0)},b,c))));
    case (Exp.CALL(Absyn.IDENT("sqrt"),exp :: {},b,c),tv)
      equation 
        true = Exp.exp_contains(exp, Exp.CREF(tv,Exp.REAL())) " sqrt(x) => 1(2 * sqrt(x)) * der(x) ";
        exp_1 = differentiate_exp(exp, tv); then Exp.BINARY(
          Exp.BINARY(Exp.RCONST(1.0),Exp.DIV(Exp.REAL()),
          Exp.BINARY(Exp.RCONST(2.0),Exp.MUL(Exp.REAL()),
          Exp.CALL(Absyn.IDENT("sqrt"),exp :: {},b,c))),Exp.MUL(Exp.REAL()),exp_1);
    case (Exp.CALL(Absyn.IDENT("tan"),exp :: {},b,c),tv)
      equation 
        true = Exp.exp_contains(exp, Exp.CREF(tv,Exp.REAL())) " tan x => 1/((cos x)^2) ";
        exp_1 = differentiate_exp(exp, tv); then Exp.BINARY(
          Exp.BINARY(Exp.RCONST(1.0),Exp.DIV(Exp.REAL()),
          Exp.BINARY(Exp.CALL(Absyn.IDENT("cos"),{exp},b,c),Exp.POW(Exp.REAL()),
          Exp.RCONST(2.0))),Exp.MUL(Exp.REAL()),exp_1);
    case (Exp.LBINARY(e1,op,e2),tv)
      equation 
        Exp.BINARY(e1_1,op,e2_1) = differentiate_exp(Exp.BINARY(e1,op,e2), tv); then Exp.LBINARY(e1_1,op,e2_1);
    case (Exp.LUNARY(op,e),tv)
      equation 
        e_1 = differentiate_exp(e, tv); then Exp.LUNARY(op,e_1);
    case (Exp.RELATION(e1,rel,e2),tv)
      equation 
        e1_1 = differentiate_exp(e1, tv);
        e2_1 = differentiate_exp(e2, tv); then Exp.RELATION(e1_1,rel,e2_1);
    case (Exp.IFEXP(e1,e2,e3),tv)
      equation 
        e1_1 = differentiate_exp(e1, tv);
        e2_1 = differentiate_exp(e2, tv);
        e3_1 = differentiate_exp(e3, tv); then Exp.IFEXP(e1_1,e2_1,e3_1);
    case (Exp.CALL((a = Absyn.IDENT("der")),expl,b,c),tv)
      equation 
        expl_1 = Util.list_map_1(expl, differentiate_exp, tv); then Exp.CALL(a,expl_1,b,c);
    case (Exp.ARRAY(tp,b,expl),tv)
      equation 
        expl_1 = Util.list_map_1(expl, differentiate_exp, tv); then Exp.ARRAY(tp,b,expl_1);
    case (Exp.TUPLE(expl),tv)
      equation 
        expl_1 = Util.list_map_1(expl, differentiate_exp, tv); then Exp.TUPLE(expl_1);
    case (Exp.CAST(tp,e),tv)
      equation 
        e_1 = differentiate_exp(e, tv); then Exp.CAST(tp,e_1);
    case (Exp.ASUB(e,i),tv)
      equation 
        e_1 = differentiate_exp(e, tv); then Exp.ASUB(e,i);
    case (Exp.REDUCTION(a,e1,b,e2),tv)
      local String b;
      equation 
        e1_1 = differentiate_exp(e1, tv);
        e2_1 = differentiate_exp(e2, tv); then Exp.REDUCTION(a,e1_1,b,e2_1);
    case (e,cr)
      equation 
        false = Exp.exp_contains(e, Exp.CREF(cr,Exp.REAL())) " If the expression does not contain the variable,
	 the derivative is zero. For efficiency reasons this rule
	 is last. Otherwise expressions is allways traversed twice 
	 when differentiating."; then Exp.RCONST(0.0);
  end matchcontinue;
end differentiate_exp;
end Derive;

