package DumpGraphviz "
    Copyright (c) PELAB, Dept. Computer and Information Science, Linkoping University,
    Sweden, in the following denoted PELAB/LIU.

   
    This file is part of OpenModelica, in the following denoted OSM.


    The OSM License, Version 1.1, March 4, 2005.


    1 Preface

    The aim of this license is to lay down the conditions enabling you to use, 
    modify and circulate OSM. However, PELAB/LIU remain the authors of 
    OSM and so retain property rights and the use of all ancillary rights.

    2 Definitions

    OSM is defined as all successive versions of the OSM software and their 
    documentation that have been developed by PELAB/LIU and including
    accepted contributions from other contributors according to this license.

    OSM DERIVED SOFTWARE is defined as all or part of OSM that you have 
    modified and/or translated and/or adapted.


    3 Dual License

    OSM is made available under the OSM licensing scheme, which is a dual
    licensing scheme with two options, a) and b):

    a) OSM OPEN SOURCE LICENSE:

    If you wish to write Open Source software you can use the Open Source version 
    of OSM, released under the OSM license which include GPL as its open source 
    licensing option. If you use the OSM Open Source version you must release your 
    Application using OSM including this Application's source code under the GPL as well.

    This OSM license text, and Copyright (c) PELAB/Linkoping University, must
    be present in your copy of OSM and in OSM DERIVED SOFTWARE.

    You should have received a copy of the GPL - GNU General Public License
    along with OpenModelica; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.


    b) OSM COMMERCIAL LICENSE

    If you are using OSM commercially - that is, for commercial usage or
    for creating proprietary software for sale or use in a commercial setting
    - you must purchase a commercial license of OSM from PELAB/L
  *
 ** file:	 Dump.rml
 ** module:      Dump
 ** description: debug printing
 **
 ** RCS: $Id: DumpGraphviz.rml 1786 2005-05-16 14:07:48Z petar $
 **
 ** Printing routines for debugging of the AST.  These functions do
 ** nothing but print the data structures to the standard output.
 **
 ** The implementation of the functions are excluded from the report,
 ** as they occupy a lot of space and do not convey any useful
 ** semantic information.
 *"

public import OpenModelica.Compiler.Absyn;

public import OpenModelica.Compiler.Debug;

public import OpenModelica.Compiler.Graphviz;

public 
type Ident = String;

public 
type Node = Graphviz.Node;

protected import OpenModelica.Compiler.ClassInf "!ignorecode";

protected import OpenModelica.Compiler.Dump;

public function dump " Relations "
  input Absyn.Program p;
protected 
  Graphviz.Node r;
algorithm 
  r := build_graphviz(p) " print \"> Beginning of rule dump\n\" & ";
  Graphviz.dump(r) " & print \"> End of rule dump\n\" ";
end dump;

protected function build_graphviz
  input Absyn.Program in_program;
  output Node out_node;
algorithm 
  out_node:=
  matchcontinue (in_program)
    local
      type NodeList = list<Graphviz.Node>;
      type Absyn_ClassList = list<Absyn.Class>;
      NodeList nl;
      Absyn_ClassList cs;
    case (Absyn.PROGRAM(cs,_))
      equation 
        nl = print_classes(cs) " print \"> Beginning of rule build_graphviz\n\" &   & print \"> End of rule build_graphviz\n\" "; then Graphviz.NODE("ROOT",{},nl);
  end matchcontinue;
end build_graphviz;

protected function print_classes
  input Absyn_ClassList in_absyn_classlist;
  output NodeList out_nodelist;
protected 
  type Absyn_ClassList = list<Absyn.Class>;
  type NodeList = list<Node>;
algorithm 
  out_nodelist:=
  matchcontinue (in_absyn_classlist)
    local
      Graphviz.Node node;
      NodeList nl;
      Absyn.Class c;
      Absyn_ClassList cs;
    case {} then {}; 
    case c :: cs
      equation 
        node = print_class(c) " print \"> Beginning of rule print_classes\n\" & ";
        nl = print_classes(cs) " & print \"> End of rule print_classes\n\" "; then node :: nl;
  end matchcontinue;
end print_classes;

protected function print_class
  input Absyn.Class in_class;
  output Node out_node;
algorithm 
  out_node:=
  matchcontinue (in_class)
    local
      type NodeList = list<Graphviz.Node>;
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      Ident rs,n;
      NodeList nl;
      Boolean p,f,e;
      Absyn.Restriction r;
      Absyn_ClassPartList parts;
    case (Absyn.CLASS(n,p,f,e,r,Absyn.PARTS(parts,_),_))
      equation 
        rs = Absyn.restr_string(r) " print \"> Beginning of rule print_class\n\" & ";
        nl = print_parts(parts) " & print \"> End of rule print_class\n\" "; then Graphviz.NODE(rs,{},nl);
  end matchcontinue;
end print_class;

protected function print_parts
  input Absyn_ClassPartList in_absyn_classpartlist;
  output NodeList out_nodelist;
protected 
  type Absyn_ClassPartList = list<Absyn.ClassPart>;
  type NodeList = list<Node>;
algorithm 
  out_nodelist:=
  matchcontinue (in_absyn_classpartlist)
    local
      Graphviz.Node node;
      NodeList nl;
      Absyn.ClassPart c;
      Absyn_ClassPartList cs;
    case {} then {}; 
    case c :: cs
      equation 
        node = print_class_part(c) " print \"> Beginning of rule print_parts\n\" & ";
        nl = print_parts(cs) " & print \"> End of rule print_parts\n\" "; then node :: nl;
  end matchcontinue;
end print_parts;

protected function print_class_part ""
  input Absyn.ClassPart in_classpart;
  output Node out_node;
algorithm 
  out_node:=
  matchcontinue (in_classpart)
    local
      type NodeList = list<Graphviz.Node>;
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      type Absyn_EquationItemList = list<Absyn.EquationItem>;
      type Absyn_AlgorithmItemList = list<Absyn.AlgorithmItem>;
      NodeList nl;
      Absyn_ElementItemList el;
      Absyn_EquationItemList eqs;
    case (Absyn.PUBLIC(el))
      equation 
        nl = print_elementitems(el) " print \"> Beginning of rule print_class_part\n\" &   & print \"> End of rule print_class_part\n\" "; then Graphviz.NODE("PUBLIC",{},nl);
    case (Absyn.PROTECTED(el))
      equation 
        nl = print_elementitems(el) " print \"> Beginning of rule print_class_part\n\" &   & print \"> End of rule print_class_part\n\" "; then Graphviz.NODE("PROTECTED",{},nl);
    case (Absyn.EQUATIONS(eqs))
      equation 
        nl = print_equations(eqs); then Graphviz.NODE("EQUATIONS",{},nl);
    case (Absyn.ALGORITHMS(eqs))
      local Absyn_AlgorithmItemList eqs;
      equation 
        nl = print_algorithms(eqs); then Graphviz.NODE("ALGORITHMS",{},nl);
    case (_) "
  rule	print_constraints eqs => nl
	------------------------
	print_class_part(Absyn.CONSTRAINTS(eqs)) => Graphviz.NODE(\"CONSTRAINTS\",{},nl)
"
      equation 
        print("") " print \"> Error in print_class_part\n\" "; then Graphviz.NODE("PART_ERROR",{},{});
  end matchcontinue;
end print_class_part;

protected function print_elementitems ""
  input Absyn_ElementItemList in_absyn_elementitemlist;
  output NodeList out_nodelist;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
  type NodeList = list<Node>;
algorithm 
  out_nodelist:=
  matchcontinue (in_absyn_elementitemlist)
    local
      NodeList nl;
      Absyn_ElementItemList el;
      Graphviz.Node node;
      Absyn.Element e;
    case {} then {}; 
    case (Absyn.ANNOTATIONITEM(_) :: el)
      equation 
        nl = print_elementitems(el); then nl;
    case (Absyn.ELEMENTITEM(e) :: el)
      equation 
        node = print_element(e) " print \"> Beginning of rule print_elements\n\" & ";
        nl = print_elementitems(el) " & print \"> End of rule print_elements\n\" "; then node :: nl;
  end matchcontinue;
end print_elementitems;

protected function make_bool_attr ""
  input String str;
  input Boolean flag;
  output Graphviz.Attribute out_attribute;
protected 
  Ident s;
algorithm 
  s := select_string(flag, "true", "false");
  out_attribute := Graphviz.ATTR(str,s);
end make_bool_attr;

protected function make_leaf
  input String in_string;
  input Graphviz_AttributeList in_graphviz_attributelist;
  output Node out_node;
protected 
  type Graphviz_AttributeList = list<Graphviz.Attribute>;
algorithm 
  out_node:=
  matchcontinue (in_string,in_graphviz_attributelist)
    local
      Ident str;
      Graphviz_AttributeList al;
    case (str,al) then Graphviz.NODE(str,al,{}); 
  end matchcontinue;
end make_leaf;

protected function print_element
  input Absyn.Element in_element;
  output Node out_node;
algorithm 
  out_node:=
  matchcontinue (in_element)
    local
      Graphviz.Attribute fa;
      Graphviz.Node elsp;
      Boolean final_;
      Absyn.ElementSpec spec;
    case (Absyn.ELEMENT(final_,_,_,_,spec,_,_,_))
      equation 
        fa = make_bool_attr("final", final_) " print \"> Beginning of rule print_element\n\" & ";
        elsp = print_elementspec(spec) " & print \"> End of rule print_element\n\" "; then Graphviz.NODE("ELEMENT",{fa},{elsp});
  end matchcontinue;
end print_element;

protected function print_path
  input Absyn.Path p;
  output Node pn;
protected 
  Ident s;
algorithm 
  s := Absyn.path_string(p);
  pn := make_leaf(s, {});
end print_path;

protected function print_elementspec
  input Absyn.ElementSpec in_elementspec;
  output Node out_node;
algorithm 
  out_node:=
  matchcontinue (in_elementspec)
    local
      type Absyn_ElementArgList = list<Absyn.ElementArg>;
      type NodeList = list<Graphviz.Node>;
      type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
      Graphviz.Node nl,en,pn;
      Graphviz.Attribute ra;
      Boolean repl;
      Absyn.Class cl;
      Absyn.Path p;
      Absyn_ElementArgList l;
      NodeList cns;
      Absyn.ElementAttributes attr;
      Absyn_ComponentItemList cs;
    case (Absyn.CLASSDEF(repl,cl))
      equation 
        nl = print_class(cl);
        ra = make_bool_attr("replaceable", repl); then Graphviz.NODE("CLASSDEF",{ra},{});
    case (Absyn.EXTENDS(p,l))
      equation 
        en = print_path(p); then Graphviz.NODE("EXTENDS",{},{en});
    case (Absyn.COMPONENTS(attr,p,cs))
      equation 
        pn = print_path(p);
        cns = print_components(cs); then Graphviz.NODE("COMPONENTS",{},pn :: cns);
    case (_)
      equation 
        print(""); then Graphviz.NODE("ELSPEC_ERROR",{},{});
  end matchcontinue;
end print_elementspec;

protected function print_components
  input Absyn_ComponentItemList in_absyn_componentitemlist;
  output NodeList out_nodelist;
protected 
  type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
  type NodeList = list<Node>;
algorithm 
  out_nodelist:=
  matchcontinue (in_absyn_componentitemlist)
    local
      Graphviz.Node n;
      NodeList nl;
      Absyn.ComponentItem c;
      Absyn_ComponentItemList cs;
    case {} then {}; 
    case c :: cs
      equation 
        n = print_componentitem(c);
        nl = print_components(cs); then n :: nl;
  end matchcontinue;
end print_components;

protected function print_componentitem
  input Absyn.ComponentItem in_componentitem;
  output Node out_node;
algorithm 
  out_node:=
  matchcontinue (in_componentitem)
    local
      type Absyn_SubscriptList = list<Absyn.Subscript>;
      type Absyn_ModificationOption = Option<Absyn.Modification>;
      Graphviz.Node nn;
      Ident n;
      Absyn_SubscriptList a;
      Absyn_ModificationOption m;
    case (Absyn.COMPONENTITEM(Absyn.COMPONENT(n,a,m),_))
      equation 
        print("");
        nn = Graphviz.NODE(n,{},{}); then Graphviz.LNODE("COMPONENT",{n},{},{nn});
  end matchcontinue;
end print_componentitem;

protected function print_equations
  input Absyn_EquationItemList in_absyn_equationitemlist;
  output NodeList out_nodelist;
protected 
  type Absyn_EquationItemList = list<Absyn.EquationItem>;
  type NodeList = list<Node>;
algorithm 
  out_nodelist:=
  matchcontinue (in_absyn_equationitemlist)
    local
      type Absyn_CommentOption = Option<Absyn.Comment>;
      Graphviz.Node node;
      NodeList nl;
      Absyn.Equation eq;
      Absyn_CommentOption ann;
      Absyn_EquationItemList el;
    case {} then {}; 
    case Absyn.EQUATIONITEM(eq,ann) :: el
      equation 
        node = print_equation(eq);
        nl = print_equations(el); then node :: nl;
  end matchcontinue;
end print_equations;

protected function print_equation
  input Absyn.Equation in_equation;
  output Node out_node;
algorithm 
  out_node:=
  matchcontinue (in_equation)
    local
      type NodeList = list<Graphviz.Node>;
      type Absyn_EquationItemList = list<Absyn.EquationItem>;
      Ident s1,s2,s,s_1,s_2,es,n;
      Absyn.Exp e1,e2,e;
      Absyn.ComponentRef c1,c2;
      NodeList eqn;
      Absyn_EquationItemList eqs;
    case (Absyn.EQ_EQUALS(e1,e2))
      equation 
        s1 = Dump.print_exp_str(e1);
        s2 = Dump.print_exp_str(e2);
        s = string_append(s1, " = ");
        s_1 = string_append(s, s2); then Graphviz.LNODE("EQ_EQUALS",{s_1},{},{});
    case (Absyn.EQ_CONNECT(c1,c2))
      equation 
        s1 = Dump.print_component_ref_str(c1);
        s2 = Dump.print_component_ref_str(c2);
        s = string_append("connect(", s1);
        s_1 = string_append(s, s2);
        s_2 = string_append(s_1, ")"); then Graphviz.LNODE("EQ_CONNECT",{s_2},{},{});
    case (Absyn.EQ_FOR(n,e,eqs))
      equation 
        eqn = print_equations(eqs);
        es = Dump.print_exp_str(e); then Graphviz.LNODE("EQ_FOR",{n,es},{},eqn);
    case (_) then Graphviz.NODE("EQ_ERROR",{},{}); 
  end matchcontinue;
end print_equation;

protected function print_algorithms
  input Absyn_AlgorithmItemList in_absyn_algorithmitemlist;
  output NodeList out_nodelist;
protected 
  type Absyn_AlgorithmItemList = list<Absyn.AlgorithmItem>;
  type NodeList = list<Node>;
algorithm 
  out_nodelist:=
  matchcontinue (in_absyn_algorithmitemlist)
    local
      Graphviz.Node node;
      NodeList nl;
      Absyn.AlgorithmItem e;
      Absyn_AlgorithmItemList el;
    case {} then {}; 
    case e :: el
      equation 
        node = print_algorithmitem(e);
        nl = print_algorithms(el); then node :: nl;
  end matchcontinue;
end print_algorithms;

protected function print_algorithmitem
  input Absyn.AlgorithmItem in_algorithmitem;
  output Node out_node;
algorithm 
  out_node:=
  matchcontinue (in_algorithmitem)
    local
      Graphviz.Node node;
      Absyn.Algorithm alg;
    case (Absyn.ALGORITHMITEM(alg,_))
      equation 
        node = print_algorithm(alg); then node;
    case (_) then Graphviz.NODE("ALG_ERROR",{},{}); 
  end matchcontinue;
end print_algorithmitem;

protected function print_algorithm
  input Absyn.Algorithm in_algorithm;
  output Node out_node;
algorithm 
  out_node:=
  matchcontinue (in_algorithm)
    local
      Absyn.ComponentRef cr;
      Absyn.Exp e;
    case (Absyn.ALG_ASSIGN(cr,e))
      equation 
        print(""); then Graphviz.NODE("ALG_ASSIGN",{},{});
    case (_)
      equation 
        print(""); then Graphviz.NODE("ALG_ERROR",{},{});
  end matchcontinue;
end print_algorithm;

protected function variability_symbol "
relation print_constraints: Absyn.Constraint list => Node list =

  axiom	print_constraints {} => {}

  rule	print_constraint e => node &
	print_constraints el => nl
	--------------------------
	print_constraints e::el => node::nl
	
end


relation print_constraint: Absyn.Constraint => Node =

  rule	print \"\" 
	------------------
	print_constraint (Absyn.INEQ_EXPR(e)) => Graphviz.NODE(\"INEQ_EXPR\",{},{})

  rule	print \"\"
	--------
	print_constraint (_) => Graphviz.NODE(\"ALG_ERROR\",{},{})
end
"
  input Absyn.Variability in_variability;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_variability)
    case (Absyn.VAR()) then ""; 
    case (Absyn.DISCRETE()) then "DISCRETE"; 
    case (Absyn.PARAM()) then "PARAM"; 
    case (Absyn.CONST()) then "CONST"; 
  end matchcontinue;
end variability_symbol;

protected function direction_symbol
  input Absyn.Direction in_direction;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_direction)
    case (Absyn.BIDIR()) then ""; 
    case (Absyn.INPUT()) then "INPUT"; 
    case (Absyn.OUTPUT()) then "OUTPUT"; 
  end matchcontinue;
end direction_symbol;

protected function select_string
  input Boolean in_boolean1;
  input String in_string2;
  input String in_string3;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_boolean1,in_string2,in_string3)
    local Ident a,b;
    case (true,a,b) then a; 
    case (false,a,b) then b; 
  end matchcontinue;
end select_string;
end DumpGraphviz;

