package Mod "
    Copyright (c) PELAB, Dept. Computer and Information Science, Linkoping University,
    Sweden, in the following denoted PELAB/LIU.

   
    This file is part of OpenModelica, in the following denoted OSM.


    The OSM License, Version 1.1, March 4, 2005.


    1 Preface

    The aim of this license is to lay down the conditions enabling you to use, 
    modify and circulate OSM. However, PELAB/LIU remain the authors of 
    OSM and so retain property rights and the use of all ancillary rights.

    2 Definitions

    OSM is defined as all successive versions of the OSM software and their 
    documentation that have been developed by PELAB/LIU and including
    accepted contributions from other contributors according to this license.

    OSM DERIVED SOFTWARE is defined as all or part of OSM that you have 
    modified and/or translated and/or adapted.


    3 Dual License

    OSM is made available under the OSM licensing scheme, which is a dual
    licensing scheme with two options, a) and b):

    a) OSM OPEN SOURCE LICENSE:

    If you wish to write Open Source software you can use the Open Source version 
    of OSM, released under the OSM license which include GPL as its open source 
    licensing option. If you use the OSM Open Source version you must release your 
    Application using OSM including this Application's source code under the GPL as well.

    This OSM license text, and Copyright (c) PELAB/Linkoping University, must
    be present in your copy of OSM and in OSM DERIVED SOFTWARE.

    You should have received a copy of the GPL - GNU General Public License
    along with OpenModelica; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.


    b) OSM COMMERCIAL LICENSE

    If you are using OSM commercially - that is, for commercial usage or
    for creating proprietary software for sale or use in a commercial setting
    - you must purchase a commercial license of OSM from PELAB/L
  *
 ** file:        Mod.rml
 ** module:      Mod
 ** description: Modification handling
 **
 ** RCS: $Id: Mod.rml 1733 2005-04-27 06:47:45Z haklu $
 **
 ** Modifications are simply the same kind of modifications used in
 ** the `Absyn' module.
 **
 ** This type is very similar to `SCode.Mod'.  The main difference is
 ** that it uses `Exp.Exp' for the expressions.  Expressions stored
 ** here are prefixed and typechecked.
 **
 ** The datatype itself is moved to the Types module, in Types.rml, to prevent
 ** circular dependencies.
 **
 *"

public import OpenModelica.Compiler.Absyn;

public import OpenModelica.Compiler.Env;

public import OpenModelica.Compiler.Exp;

public import OpenModelica.Compiler.Prefix;

public import OpenModelica.Compiler.SCode;

public import OpenModelica.Compiler.Types;

public 
type Ident = String;

protected import OpenModelica.Compiler.Dump;

protected import OpenModelica.Compiler.Debug;

protected import OpenModelica.Compiler.Print;

protected import OpenModelica.Compiler.Inst;

protected import OpenModelica.Compiler.Static;

protected import OpenModelica.Compiler.Values;

protected import OpenModelica.Compiler.Util;

public function elab_mod "* function: elab_mod
 **
 ** This function elaborates on the expressions in a modification and
 ** turns them into global expressions.  This is done because the
 ** expressions in modifications must be elaborated on in the context
 ** they are provided in, and not the context they are used in.
 *"
  input Env.Env in_env;
  input Prefix.Prefix in_prefix;
  input SCode.Mod in_mod;
  input Boolean in_boolean;
  output Types.Mod out_mod;
algorithm 
  out_mod:=
  matchcontinue (in_env,in_prefix,in_mod,in_boolean)
    local
      type Types_SubModList = list<Types.SubMod>;
      type Env_FrameList = list<Env.Frame>;
      type SCode_SubModList = list<SCode.SubMod>;
      type TupleSCode_ElementTypes_Mod = tuple<SCode.Element,Types.Mod>;
      type TupleSCode_ElementTypes_ModList = list<TupleSCode_ElementTypes_Mod>;
      type SCode_ElementList = list<SCode.Element>;
      Boolean impl,final_;
      Types_SubModList subs_1;
      Env_FrameList env;
      Prefix.Prefix pre;
      SCode.Mod m,mod;
      SCode_SubModList subs;
      Exp.Exp e_1,e_2;
      Types.Properties prop;
      Absyn.Exp e;
      TupleSCode_ElementTypes_ModList elist_1;
      SCode_ElementList elist;
    case (_,_,SCode.NOMOD(),impl) then Types.NOMOD(); 
    case (env,pre,(m = SCode.MOD(final_,subs,NONE)),impl)
      equation 
        subs_1 = elab_submods(env, pre, subs, impl); then Types.MOD(final_,subs_1,NONE);
    case (env,pre,(m = SCode.MOD(final_,subs,SOME(e))),impl)
      equation 
        subs_1 = elab_submods(env, pre, subs, impl);
        (e_1,prop,_) = Static.elab_exp(env, e, impl, NONE);
        e_2 = Prefix.prefix_exp(env, e_1, pre) " Bug: will cause elaboration of parameters without value to fail,
	 But this can be ok, since a modifier is present, giving it a value 
	 from outer modifications.. "; then Types.MOD(final_,subs_1,SOME(Types.TYPED(e_2,prop)));
    case (env,pre,(m = SCode.REDECL(final_,elist)),impl)
      equation 
        elist_1 = Inst.add_nomod(elist); then Types.REDECL(final_,elist_1);
    case (_,pre,mod,impl) then fail(); 
  end matchcontinue;
end elab_mod;

public function unelab_mod
  input Types.Mod in_mod;
  output SCode.Mod out_mod;
algorithm 
  out_mod:=
  matchcontinue (in_mod)
    local
      type SCode_SubModList = list<SCode.SubMod>;
      type Types_SubModList = list<Types.SubMod>;
      type SCode_ElementList = list<SCode.Element>;
      type TupleSCode_ElementTypes_Mod = tuple<SCode.Element,Types.Mod>;
      type TupleSCode_ElementTypes_ModList = list<TupleSCode_ElementTypes_Mod>;
      SCode_SubModList subs_1;
      Types.Mod m,mod;
      Boolean final_;
      Types_SubModList subs;
      Absyn.Exp e,e_1;
      Types.Properties p;
      SCode_ElementList elist_1;
      TupleSCode_ElementTypes_ModList elist;
    case (Types.NOMOD()) then SCode.NOMOD(); 
    case ((m = Types.MOD(final_,subs,NONE)))
      equation 
        subs_1 = unelab_submods(subs); then SCode.MOD(final_,subs_1,NONE);
    case ((m = Types.MOD(final_,subs,SOME(Types.UNTYPED(e)))))
      equation 
        subs_1 = unelab_submods(subs); then SCode.MOD(final_,subs_1,SOME(e));
    case ((m = Types.MOD(final_,subs,SOME(Types.TYPED(e,p)))))
      local Exp.Exp e;
      equation 
        subs_1 = unelab_submods(subs);
        e_1 = Exp.unelab_exp(e); then SCode.MOD(final_,subs_1,SOME(e_1));
    case ((m = Types.REDECL(final_,elist)))
      equation 
        elist_1 = Util.list_map(elist, Util.tuple2_1); then SCode.REDECL(final_,elist_1);
    case (mod)
      equation 
        Print.print_buf("#-- elab_untyped_mod failed:\n");
        print("- unelab_mod failed\n"); then fail();
  end matchcontinue;
end unelab_mod;

protected function unelab_submods
  input Types_SubModList in_types_submodlist;
  output SCode_SubModList out_scode_submodlist;
protected 
  type Types_SubModList = list<Types.SubMod>;
  type SCode_SubModList = list<SCode.SubMod>;
algorithm 
  out_scode_submodlist:=
  matchcontinue (in_types_submodlist)
    local
      SCode_SubModList x_1,xs_1,res;
      Types.SubMod x;
      Types_SubModList xs;
    case ({}) then {}; 
    case (x :: xs)
      equation 
        x_1 = unelab_submod(x);
        xs_1 = unelab_submods(xs);
        res = list_append(x_1, xs_1); then res;
  end matchcontinue;
end unelab_submods;

protected function unelab_submod "* function: unelab_submod
 **
 ** This function unelaborates on a submodification.
 *"
  input Types.SubMod in_submod;
  output SCode_SubModList out_scode_submodlist;
protected 
  type SCode_SubModList = list<SCode.SubMod>;
algorithm 
  out_scode_submodlist:=
  matchcontinue (in_submod)
    local
      type Absyn_SubscriptList = list<Absyn.Subscript>;
      type IntegerList = list<Integer>;
      SCode.Mod m_1;
      Ident i;
      Types.Mod m;
      Absyn_SubscriptList ss_1;
      IntegerList ss;
    case (Types.NAMEMOD(i,m))
      equation 
        m_1 = unelab_mod(m); then {SCode.NAMEMOD(i,m_1)};
    case (Types.IDXMOD(ss,m))
      equation 
        ss_1 = unelab_subscript(ss);
        m_1 = unelab_mod(m); then {SCode.IDXMOD(ss_1,m_1)};
  end matchcontinue;
end unelab_submod;

protected function unelab_subscript
  input IntegerList in_integerlist;
  output SCode_SubscriptList out_scode_subscriptlist;
protected 
  type IntegerList = list<Integer>;
  type SCode_SubscriptList = list<SCode.Subscript>;
algorithm 
  out_scode_subscriptlist:=
  matchcontinue (in_integerlist)
    local
      type Absyn_SubscriptList = list<Absyn.Subscript>;
      Absyn_SubscriptList xs;
      Integer i;
      IntegerList is;
    case ({}) then {}; 
    case (i :: is)
      equation 
        xs = unelab_subscript(is); then Absyn.SUBSCRIPT(Absyn.INTEGER(i)) :: xs;
  end matchcontinue;
end unelab_subscript;

public function update_mod "* function: update_mod
 **
 ** This function updates and untyped modification to a typed one, by looking
 ** up the type of the modifier in the environment and update it.
 *"
  input Env.Env in_env;
  input Prefix.Prefix in_prefix;
  input Types.Mod in_mod;
  input Boolean in_boolean;
  output Types.Mod out_mod;
algorithm 
  out_mod:=
  matchcontinue (in_env,in_prefix,in_mod,in_boolean)
    local
      type Types_SubModList = list<Types.SubMod>;
      type Env_FrameList = list<Env.Frame>;
      Boolean impl,f;
      Types.Mod m;
      Types_SubModList subs_1,subs;
      Exp.Exp e_1,e_2;
      Types.Properties prop,p;
      Env_FrameList env;
      Prefix.Prefix pre;
      Absyn.Exp e;
    case (_,_,Types.NOMOD(),impl) then Types.NOMOD(); 
    case (_,_,(m = Types.REDECL(_,_)),impl) then m; 
    case (env,pre,(m = Types.MOD(f,subs,SOME(Types.UNTYPED(e)))),impl)
      equation 
        subs_1 = update_submods(env, pre, subs, impl);
        (e_1,prop,_) = Static.elab_exp(env, e, impl, NONE);
        e_2 = Prefix.prefix_exp(env, e_1, pre);
        Debug.fprint("updmod", "Updated mod: ");
        Debug.fcall("updmod", print_mod, 
          Types.MOD(f,subs_1,SOME(Types.TYPED(e_2,prop)))); then Types.MOD(f,subs_1,SOME(Types.TYPED(e_2,prop)));
    case (env,pre,Types.MOD(f,subs,SOME(Types.TYPED(e,p))),impl)
      local Exp.Exp e;
      equation 
        subs_1 = update_submods(env, pre, subs, impl); then Types.MOD(f,subs_1,SOME(Types.TYPED(e,p)));
    case (env,pre,Types.MOD(f,subs,NONE),impl)
      equation 
        subs_1 = update_submods(env, pre, subs, impl); then Types.MOD(f,subs_1,NONE);
    case (env,pre,m,impl)
      equation 
        Print.print_buf("- update_mod failed\n mod:");
        print_mod(m);
        Print.print_buf("\n"); then fail();
  end matchcontinue;
end update_mod;

protected function update_submods
  input Env.Env in_env;
  input Prefix.Prefix in_prefix;
  input Types_SubModList in_types_submodlist;
  input Boolean in_boolean;
  output Types_SubModList out_types_submodlist;
protected 
  type Types_SubModList = list<Types.SubMod>;
algorithm 
  out_types_submodlist:=
  matchcontinue (in_env,in_prefix,in_types_submodlist,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      Boolean impl;
      Types_SubModList x_1,xs_1,res,xs;
      Env_FrameList env;
      Prefix.Prefix pre;
      Types.SubMod x;
    case (_,_,{},impl) then {}; 
    case (env,pre,x :: xs,impl)
      equation 
        x_1 = update_submod(env, pre, x, impl);
        xs_1 = update_submods(env, pre, xs, impl);
        res = insert_submods(x_1, xs_1, env, pre); then res;
  end matchcontinue;
end update_submods;

protected function update_submod
  input Env.Env in_env;
  input Prefix.Prefix in_prefix;
  input Types.SubMod in_submod;
  input Boolean in_boolean;
  output Types_SubModList out_types_submodlist;
protected 
  type Types_SubModList = list<Types.SubMod>;
algorithm 
  out_types_submodlist:=
  matchcontinue (in_env,in_prefix,in_submod,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      Types.Mod m_1,m;
      Env_FrameList env;
      Prefix.Prefix pre;
      Ident i;
      Boolean impl;
      Types_SubModList smods;
    case (env,pre,Types.NAMEMOD(i,m),impl) "impl"
      equation 
        m_1 = update_mod(env, pre, m, impl); then {Types.NAMEMOD(i,m_1)};
    case (env,pre,Types.IDXMOD(_,m),impl)
      equation 
        m_1 = update_mod(env, pre, m, impl) "Static.elab_subscripts (env,ss) => (ss',true) &";
        smods = make_idxmods({}, m_1);
        Print.print_buf("#Error, not implemented updating of IDXMOD yet\n"); then smods;
  end matchcontinue;
end update_submod;

public function elab_untyped_mod "* function elab_untyped_mod
 ** This function is used to convert SCode.Mod into Mod, without 
 ** adding correct type information. Instead, a undefined type will be 
 ** given to the modification. This is used when modifications of e.g. 
 ** elements in base classes used. For instance,
 ** model test extends A(x=y); end test; // both x and y are defined in A
 ** The modifier x=y must be merged with outer modifiers, thus it needs 
 ** to be converted to Mod.
 ** Notice that the correct type information must be updated later on.
 *"
  input SCode.Mod in_mod;
  input Env.Env in_env;
  input Prefix.Prefix in_prefix;
  output Types.Mod out_mod;
algorithm 
  out_mod:=
  matchcontinue (in_mod,in_env,in_prefix)
    local
      type Types_SubModList = list<Types.SubMod>;
      type SCode_SubModList = list<SCode.SubMod>;
      type Env_FrameList = list<Env.Frame>;
      type TupleSCode_ElementTypes_Mod = tuple<SCode.Element,Types.Mod>;
      type TupleSCode_ElementTypes_ModList = list<TupleSCode_ElementTypes_Mod>;
      type SCode_ElementList = list<SCode.Element>;
      Types_SubModList subs_1;
      SCode.Mod m,mod;
      Boolean final_;
      SCode_SubModList subs;
      Env_FrameList env;
      Prefix.Prefix pre;
      Absyn.Exp e;
      TupleSCode_ElementTypes_ModList elist_1;
      SCode_ElementList elist;
    case (SCode.NOMOD(),_,_) then Types.NOMOD(); 
    case ((m = SCode.MOD(final_,subs,NONE)),env,pre)
      equation 
        subs_1 = elab_untyped_submods(subs, env, pre); then Types.MOD(final_,subs_1,NONE);
    case ((m = SCode.MOD(final_,subs,SOME(e))),env,pre)
      equation 
        subs_1 = elab_untyped_submods(subs, env, pre); then Types.MOD(final_,subs_1,SOME(Types.UNTYPED(e)));
    case ((m = SCode.REDECL(final_,elist)),env,pre)
      equation 
        elist_1 = Inst.add_nomod(elist); then Types.REDECL(final_,elist_1);
    case (mod,env,pre)
      equation 
        Print.print_buf("#-- elab_untyped_mod failed:\n");
        SCode.print_mod(mod);
        Print.print_buf("\n"); then fail();
  end matchcontinue;
end elab_untyped_mod;

protected function elab_submods
  input Env.Env in_env;
  input Prefix.Prefix in_prefix;
  input SCode_SubModList in_scode_submodlist;
  input Boolean in_boolean;
  output Types_SubModList out_types_submodlist;
protected 
  type SCode_SubModList = list<SCode.SubMod>;
  type Types_SubModList = list<Types.SubMod>;
algorithm 
  out_types_submodlist:=
  matchcontinue (in_env,in_prefix,in_scode_submodlist,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      Boolean impl;
      Types_SubModList x_1,xs_1,res;
      Env_FrameList env;
      Prefix.Prefix pre;
      SCode.SubMod x;
      SCode_SubModList xs;
    case (_,_,{},impl) then {}; 
    case (env,pre,x :: xs,impl)
      equation 
        x_1 = elab_submod(env, pre, x, impl);
        xs_1 = elab_submods(env, pre, xs, impl);
        res = insert_submods(x_1, xs_1, env, pre); then res;
  end matchcontinue;
end elab_submods;

protected function elab_submod "* function: elab_submod
 **
 ** This function elaborates on a submodification, turning an
 ** `SCode.SubMod' into one or more `Types.SubMod's.
 *"
  input Env.Env in_env;
  input Prefix.Prefix in_prefix;
  input SCode.SubMod in_submod;
  input Boolean in_boolean;
  output Types_SubModList out_types_submodlist;
protected 
  type Types_SubModList = list<Types.SubMod>;
algorithm 
  out_types_submodlist:=
  matchcontinue (in_env,in_prefix,in_submod,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      type Exp_SubscriptList = list<Exp.Subscript>;
      type Absyn_SubscriptList = list<Absyn.Subscript>;
      Types.Mod m_1;
      Env_FrameList env;
      Prefix.Prefix pre;
      Ident i;
      SCode.Mod m;
      Boolean impl;
      Exp_SubscriptList ss_1;
      Types_SubModList smods;
      Absyn_SubscriptList ss;
    case (env,pre,SCode.NAMEMOD(i,m),impl) "impl"
      equation 
        m_1 = elab_mod(env, pre, m, impl); then {Types.NAMEMOD(i,m_1)};
    case (env,pre,SCode.IDXMOD(ss,m),impl)
      equation 
        (ss_1,true) = Static.elab_subscripts(env, ss, impl);
        m_1 = elab_mod(env, pre, m, impl);
        smods = make_idxmods(ss_1, m_1); then smods;
  end matchcontinue;
end elab_submod;

protected function elab_untyped_submods
  input SCode_SubModList in_scode_submodlist;
  input Env.Env in_env;
  input Prefix.Prefix in_prefix;
  output Types_SubModList out_types_submodlist;
protected 
  type SCode_SubModList = list<SCode.SubMod>;
  type Types_SubModList = list<Types.SubMod>;
algorithm 
  out_types_submodlist:=
  matchcontinue (in_scode_submodlist,in_env,in_prefix)
    local
      type Env_FrameList = list<Env.Frame>;
      Types_SubModList x_1,xs_1,res;
      SCode.SubMod x;
      SCode_SubModList xs;
      Env_FrameList env;
      Prefix.Prefix pre;
    case ({},_,_) then {}; 
    case (x :: xs,env,pre)
      equation 
        x_1 = elab_untyped_submod(x, env, pre);
        xs_1 = elab_untyped_submods(xs, env, pre);
        res = insert_submods(x_1, xs_1, env, pre); then res;
  end matchcontinue;
end elab_untyped_submods;

protected function elab_untyped_submod
  input SCode.SubMod in_submod;
  input Env.Env in_env;
  input Prefix.Prefix in_prefix;
  output Types_SubModList out_types_submodlist;
protected 
  type Types_SubModList = list<Types.SubMod>;
algorithm 
  out_types_submodlist:=
  matchcontinue (in_submod,in_env,in_prefix)
    local
      type Env_FrameList = list<Env.Frame>;
      type Absyn_SubscriptList = list<Absyn.Subscript>;
      Types.Mod m_1;
      Ident i;
      SCode.Mod m;
      Env_FrameList env;
      Prefix.Prefix pre;
      Absyn_SubscriptList subcr;
    case (SCode.NAMEMOD(i,m),env,pre)
      equation 
        m_1 = elab_untyped_mod(m, env, pre); then {Types.NAMEMOD(i,m_1)};
    case (SCode.IDXMOD(subcr,m),env,pre)
      equation 
        m_1 = elab_untyped_mod(m, env, pre); then {Types.IDXMOD({-1},m_1)};
  end matchcontinue;
end elab_untyped_submod;

protected function make_idxmods "* function: make_idxmods
 **
 ** From a list of list of integers, this function creates a list of
 ** sub-modifications of the `IDXMOD' variety.
 *"
  input Exp_SubscriptList in_exp_subscriptlist;
  input Types.Mod in_mod;
  output Types_SubModList out_types_submodlist;
protected 
  type Exp_SubscriptList = list<Exp.Subscript>;
  type Types_SubModList = list<Types.SubMod>;
algorithm 
  out_types_submodlist:=
  matchcontinue (in_exp_subscriptlist,in_mod)
    local
      type Exp_ExpList = list<Exp.Exp>;
      Integer x;
      Types.Mod m;
      Types_SubModList mods,mods_1;
      Exp_SubscriptList xs;
    case ({Exp.INDEX(Exp.ICONST(x))},m) then {Types.IDXMOD({x},m)}; 
    case (Exp.INDEX(Exp.ICONST(x)) :: xs,m)
      equation 
        mods = make_idxmods(xs, m);
        mods_1 = prefix_idxmods(mods, x); then mods_1;
    case (Exp.SLICE(Exp.ARRAY(_,_,x)) :: xs,m)
      local Exp_ExpList x;
      equation 
        Print.print_buf("= expand_slice\n");
        mods = expand_slice(x, xs, 1, m); then mods;
    case (Exp.WHOLEDIM() :: xs,m)
      equation 
        Print.print_buf("# Sorry, [:] slices are not handled in modifications\n"); then fail();
  end matchcontinue;
end make_idxmods;

protected function prefix_idxmods "* function: prefix_idxmods
 **
 ** This function adds a subscript to each `Types.IDXMOD' in a list of
 ** submodifications.
 *"
  input Types_SubModList in_types_submodlist;
  input Integer in_integer;
  output Types_SubModList out_types_submodlist;
protected 
  type Types_SubModList = list<Types.SubMod>;
algorithm 
  out_types_submodlist:=
  matchcontinue (in_types_submodlist,in_integer)
    local
      type IntegerList = list<Integer>;
      Types_SubModList mods_1,mods;
      IntegerList l;
      Types.Mod m;
      Integer i;
    case ({},_) then {}; 
    case (Types.IDXMOD(l,m) :: mods,i)
      equation 
        mods_1 = prefix_idxmods(mods, i); then Types.IDXMOD(i :: l,m) :: mods_1;
  end matchcontinue;
end prefix_idxmods;

protected function expand_slice
  input Exp_ExpList in_exp_explist;
  input Exp_SubscriptList in_exp_subscriptlist;
  input Integer in_integer;
  input Types.Mod in_mod;
  output Types_SubModList out_types_submodlist;
protected 
  type Exp_ExpList = list<Exp.Exp>;
  type Exp_SubscriptList = list<Exp.Subscript>;
  type Types_SubModList = list<Types.SubMod>;
algorithm 
  out_types_submodlist:=
  matchcontinue (in_exp_explist,in_exp_subscriptlist,in_integer,in_mod)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      Exp.Exp e_1,x,e;
      TupleTypes_TTypeAbsyn_PathOption t_1,t;
      Types_SubModList mods1,mods2,mods;
      Integer n_1,n;
      Exp_ExpList xs;
      Exp_SubscriptList ss;
      Types.Mod m,mod;
      Boolean final_,const_;
    case ({},_,_,_) then {}; 
    case ({},_,_,_) then {}; 
    case (x :: xs,ss,n,(m = Types.MOD(final_,{},SOME(Types.TYPED(e,Types.PROP(t,const_))))))
      equation 
        e_1 = Exp.simplify(Exp.ASUB(e,n));
        t_1 = Types.unlift_array(t);
        mods1 = make_idxmods(Exp.INDEX(x) :: ss, 
          Types.MOD(final_,{},SOME(Types.TYPED(e_1,Types.PROP(t_1,const_)))));
        n_1 = n + 1;
        mods2 = expand_slice(xs, ss, n_1, m);
        mods = list_append(mods1, mods2); then mods;
    case (_,_,_,mod)
      equation 
        Print.print_error_buf("# Illegal modification (");
        print_mod(mod);
        Print.print_buf(")\n"); then fail();
  end matchcontinue;
end expand_slice;

protected function expand_list "* function: expand_list
 **
 ** This utility function takes a list of integer values and a list of
 ** list of integers, and for each integer in the first and each list
 ** in the second list creates a
 ** list with that integer as head and the second list as tail. All
 ** resulting lists are collected in a list and returned.
 *"
  input Values_ValueList in_values_valuelist;
  input IntegerListList in_integerlistlist;
  output IntegerListList out_integerlistlist;
protected 
  type Values_ValueList = list<Values.Value>;
  type IntegerList = list<Integer>;
  type IntegerListList = list<IntegerList>;
algorithm 
  out_integerlistlist:=
  matchcontinue (in_values_valuelist,in_integerlistlist)
    local
      IntegerListList l1,l2,l,yy,ys;
      Values_ValueList xx,xs;
      Integer x;
      IntegerList y;
    case ({},_) then {}; 
    case (_,{}) then {}; 
    case ((xx = Values.INTEGER(x) :: xs),(yy = y :: ys))
      equation 
        l1 = expand_list(xx, ys);
        l2 = expand_list(xs, yy);
        l = list_append(l1, l2); then ((x::y)::l);
  end matchcontinue;
end expand_list;

protected function insert_submods
  input Types_SubModList in_types_submodlist1;
  input Types_SubModList in_types_submodlist2;
  input Env.Env in_env3;
  input Prefix.Prefix in_prefix4;
  output Types_SubModList out_types_submodlist;
protected 
  type Types_SubModList = list<Types.SubMod>;
algorithm 
  out_types_submodlist:=
  matchcontinue (in_types_submodlist1,in_types_submodlist2,in_env3,in_prefix4)
    local
      type Env_FrameList = list<Env.Frame>;
      Types_SubModList x_1,xs_1,l,xs,y;
      Types.SubMod x;
      Env_FrameList env;
      Prefix.Prefix pre;
    case ({},_,_,_) then {}; 
    case (x :: xs,y,env,pre)
      equation 
        x_1 = insert_submod(x, y, env, pre);
        xs_1 = insert_submods(xs, y, env, pre);
        l = list_append(x_1, xs_1); then l;
  end matchcontinue;
end insert_submods;

protected function insert_submod
  input Types.SubMod in_submod;
  input Types_SubModList in_types_submodlist;
  input Env.Env in_env;
  input Prefix.Prefix in_prefix;
  output Types_SubModList out_types_submodlist;
protected 
  type Types_SubModList = list<Types.SubMod>;
algorithm 
  out_types_submodlist:=
  matchcontinue (in_submod,in_types_submodlist,in_env,in_prefix)
    local
      type Env_FrameList = list<Env.Frame>;
      type IntegerList = list<Integer>;
      Types.SubMod sub,sub1;
      Types.Mod m,m1,m2;
      Ident n1,n2;
      Types_SubModList tail,sub2;
      Env_FrameList env;
      Prefix.Prefix pre;
      IntegerList i1,i2;
    case (sub,{},_,_) then {sub}; 
    case (Types.NAMEMOD(n1,m1),Types.NAMEMOD(n2,m2) :: tail,env,pre)
      equation 
        equality(n1 = n2);
        m = merge(m1, m2, env, pre); then Types.NAMEMOD(n1,m) :: tail;
    case (Types.IDXMOD(i1,m1),Types.IDXMOD(i2,m2) :: tail,env,pre)
      equation 
        equality(i1 = i2);
        m = merge(m1, m2, env, pre); then Types.IDXMOD(i1,m) :: tail;
    case (sub1,sub2,_,_) then sub1 :: sub2; 
  end matchcontinue;
end insert_submod;

public function lookup_modification_p "*
 ** - Lookup
 *
  * function: lookup_modification_p
 ** 
 ** This function extracts a modification from inside another
 ** modification, using a name to look up submodifications.
 *"
  input Types.Mod in_mod;
  input Absyn.Path in_path;
  output Types.Mod out_mod;
algorithm 
  out_mod:=
  matchcontinue (in_mod,in_path)
    local
      Types.Mod mod,m,mod_1;
      Ident n;
      Absyn.Path p;
    case (m,Absyn.IDENT(n))
      equation 
        mod = lookup_comp_modification(m, n); then mod;
    case (m,Absyn.QUALIFIED(n,p))
      equation 
        mod = lookup_comp_modification(m, n);
        mod_1 = lookup_modification_p(mod, p); then mod_1;
    case (_,_)
      equation 
        Print.print_buf("- lookup_modification_p failed\n"); then fail();
  end matchcontinue;
end lookup_modification_p;

public function lookup_comp_modification "* function: lookup_comp_modification
 **
 ** This function is used to look up an identifier in a modification.
 *"
  input Types.Mod in_mod;
  input Absyn.Ident in_ident;
  output Types.Mod out_mod;
algorithm 
  out_mod:=
  matchcontinue (in_mod,in_ident)
    local
      type Types_SubModList = list<Types.SubMod>;
      Types.Mod mod;
      Types_SubModList subs;
      Ident n,i;
    case (Types.NOMOD(),_) then Types.NOMOD(); 
    case (Types.REDECL(_,_),_) then Types.NOMOD(); 
    case (Types.MOD(_,subs,_),n)
      equation 
        mod = lookup_comp_modification2(subs, n); then mod;
    case (mod,i)
      equation 
        Print.print_buf("# Mod.lookup_comp_modification(");
        print_mod(mod);
        Print.print_buf(",");
        Print.print_buf(i);
        Print.print_buf(") failed\n");
        print("- lookup_comp_modification failed\n"); then fail();
  end matchcontinue;
end lookup_comp_modification;

protected function lookup_comp_modification2 "* function: lookup_comp_modification2
 ** 
 ** This function is just a helper to `lookup_comp_modification'.
 *"
  input Types_SubModList in_types_submodlist;
  input Absyn.Ident in_ident;
  output Types.Mod out_mod;
protected 
  type Types_SubModList = list<Types.SubMod>;
algorithm 
  out_mod:=
  matchcontinue (in_types_submodlist,in_ident)
    local
      Ident n,m;
      Types.Mod mod;
      Types.SubMod x;
      Types_SubModList xs;
    case ({},_) then Types.NOMOD(); 
    case (Types.NAMEMOD(n,mod) :: _,m)
      equation 
        equality(n = m); then mod;
    case (x :: xs,n)
      equation 
        mod = lookup_comp_modification2(xs, n); then mod;
    case (_,_)
      equation 
        Print.print_buf("- lookup_comp_modification2 failed\n"); then fail();
  end matchcontinue;
end lookup_comp_modification2;

public function lookup_idx_modification "* function: lookup_idx_modification
 **
 ** This function extracts modifications to an array element, using an
 ** integer to index the modification.
 *"
  input Types.Mod in_mod;
  input Integer in_integer;
  output Types.Mod out_mod;
algorithm 
  out_mod:=
  matchcontinue (in_mod,in_integer)
    local
      type Types_SubModList = list<Types.SubMod>;
      type Types_EqModOption = Option<Types.EqMod>;
      Types.Mod mod_1,mod_2,mod_3,mod;
      Types_SubModList subs_1,subs;
      Types_EqModOption eq_1,eq;
      Boolean f;
      Integer idx;
      Ident str,s;
    case (Types.NOMOD(),_) then Types.NOMOD(); 
    case (Types.REDECL(_,_),_) then Types.NOMOD(); 
    case (Types.MOD(f,subs,eq),idx)
      equation 
        (mod_1,subs_1) = lookup_idx_modification2(subs, NONE, idx);
        mod_2 = merge(Types.MOD(f,subs_1,NONE), mod_1, {}, Prefix.NOPRE());
        eq_1 = index_eqmod(eq, {idx});
        mod_3 = merge(mod_2, Types.MOD(false,{},eq_1), {}, Prefix.NOPRE()); then mod_3;
    case (mod,idx)
      equation 
        Debug.fprint("failtrace", "-lookup_idx_modification(");
        str = print_mod_str(mod);
        Debug.fprint("failtrace", str);
        Debug.fprint("failtrace", ",");
        s = int_string(idx);
        Debug.fprint("failtrace", s);
        Debug.fprint("failtrace", ") failed\n"); then fail();
  end matchcontinue;
end lookup_idx_modification;

protected function lookup_idx_modification2
  input Types_SubModList in_types_submodlist;
  input Types_EqModOption in_types_eqmodoption;
  input Integer in_integer;
  output Types.Mod out_mod;
  output Types_SubModList out_types_submodlist;
protected 
  type Types_SubModList = list<Types.SubMod>;
  type Types_EqModOption = Option<Types.EqMod>;
algorithm 
  (out_mod,out_types_submodlist):=
  matchcontinue (in_types_submodlist,in_types_eqmodoption,in_integer)
    local
      type IntegerList = list<Integer>;
      Types_SubModList subs_1,subs,xs_1;
      Integer x,y,idx;
      Types.Mod mod,mod_1;
      Types_EqModOption eq;
      IntegerList xs;
    case ({},_,_) then (Types.NOMOD(),{}); 
    case (Types.IDXMOD({x},mod) :: subs,eq,y) " FIXME: Redeclaration "
      equation 
        equality(x = y);
        (Types.NOMOD(),subs_1) = lookup_idx_modification2(subs, eq, y); then (mod,subs_1);
    case (Types.IDXMOD(x :: xs,mod) :: subs,eq,y)
      equation 
        equality(x = y);
        (mod_1,subs_1) = lookup_idx_modification2(subs, eq, y); then (mod_1,Types.IDXMOD(xs,mod) :: subs_1);
    case (Types.IDXMOD(x :: xs,mod) :: subs,eq,y)
      equation 
        failure(equality(x = y));
        (mod_1,subs_1) = lookup_idx_modification2(subs, eq, y); then (mod_1,subs_1);
    case (x :: xs,eq,idx)
      local
        Types.SubMod x;
        Types_SubModList xs;
      equation 
        (mod,xs_1) = lookup_idx_modification2(xs, eq, idx); then (mod,x :: xs_1);
    case (_,_,_)
      equation 
        Print.print_buf("- lookup_idx_modification2 failed\n"); then fail();
  end matchcontinue;
end lookup_idx_modification2;

protected function index_eqmod "* function: index_eqmod
 **
 ** If there is an equation modification, this function can subscript
 ** it using the provided indexing expressions.  This is used when a
 ** modification equates an array variable with an array expression.
 ** This expression will be expanded to produce one equation
 ** expression per array component.
 *"
  input Types_EqModOption in_types_eqmodoption;
  input IntegerList in_integerlist;
  output Types_EqModOption out_types_eqmodoption;
protected 
  type Types_EqModOption = Option<Types.EqMod>;
  type IntegerList = list<Integer>;
algorithm 
  out_types_eqmodoption:=
  matchcontinue (in_types_eqmodoption,in_integerlist)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      Types_EqModOption e;
      TupleTypes_TTypeAbsyn_PathOption t_1,t;
      Exp.Exp exp;
      Boolean c;
      Integer x;
      IntegerList xs;
    case (NONE,_) then NONE; 
    case (e,{}) then e; 
    case (SOME(Types.TYPED(e,Types.PROP(t,c))),x :: xs)
      equation 
        t_1 = Types.unlift_array(t);
        exp = Exp.simplify(Exp.ASUB(e,x));
        e = index_eqmod(SOME(Types.TYPED(exp,Types.PROP(t_1,c))), xs); then e;
  end matchcontinue;
end index_eqmod;

public function merge "*
 **
 ** - Merging
 **
 ** The merge function merges to modifications to one. The first
 ** argument is the \"outer\" modification that should take precedence over
 ** the \"inner\" modifications.
 **
 *
  * function: merge
 ** 
 ** This function merges to modificiations into one.  The first
 ** modifications takes precedence over the second.
 *"
  input Types.Mod in_mod1;
  input Types.Mod in_mod2;
  input Env.Env in_env3;
  input Prefix.Prefix in_prefix4;
  output Types.Mod out_mod;
algorithm 
  out_mod:=
  matchcontinue (in_mod1,in_mod2,in_env3,in_prefix4)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type Absyn_CommentOption = Option<Absyn.Comment>;
      type Env_FrameList = list<Env.Frame>;
      type TupleSCode_ElementTypes_Mod = tuple<SCode.Element,Types.Mod>;
      type TupleSCode_ElementTypes_ModList = list<TupleSCode_ElementTypes_Mod>;
      type Types_SubModList = list<Types.SubMod>;
      type Types_EqModOption = Option<Types.EqMod>;
      Types.Mod m,m1_1,m2_1,m_2,mod,mods,outer_,inner_;
      Boolean f1,f,r,p,f2,final_;
      Ident id1,id2;
      SCode.Attributes attr;
      Absyn.Path tp;
      SCode.Mod m1,m2;
      Absyn_PathOption bc,bc2;
      Absyn_CommentOption comment,comment2;
      Env_FrameList env;
      Prefix.Prefix pre;
      TupleSCode_ElementTypes_ModList els;
      Types_SubModList subs,subs1,subs2;
      Types_EqModOption ass,ass1,ass2;
    case (Types.NOMOD(),Types.NOMOD(),_,_) then Types.NOMOD(); 
    case (Types.NOMOD(),m,_,_) then m; 
    case (m,Types.NOMOD(),_,_) then m; 
    case (Types.REDECL(f1,{(SCode.COMPONENT(id1,f,r,p,attr,tp,m1,bc,comment),_)}),Types.REDECL(f2,{(SCode.COMPONENT(id2,_,_,_,_,_,m2,bc2,comment2),_)}),env,pre) "  rule	(* If redeclaring same component*)
	id1 = id2 &
	elab_mod(env,pre,m1) => m1' &
	elab_mod(env,pre,m2) => m2' &
	merge(m1',m2',env,pre) => m'' 

 (* DO SOMETHING INTELLIGENT WITH THE COMMENTS!!! *)
	--------------------------------------
	merge(Types.REDECL(f1,{(SCode.COMPONENT(id1,f,r,p,attr,tp,m1,bc,comment),_)}),
	      Types.REDECL(f2,{(SCode.COMPONENT(id2,_,_,_,_,_,m2,bc2,comment2),_)})
			       ,env,pre) 
	  => Types.REDECL(f1,{(SCode.COMPONENT(id1,f,r,p,attr,tp,SCode.NOMOD,bc,comment),m'')})
"
      equation 
        equality(id1 = id2) " If didn't work, try with untyped modifications";
        m1_1 = elab_untyped_mod(m2, env, pre);
        m2_1 = elab_untyped_mod(m2, env, pre);
        m_2 = merge(m1_1, m2_1, env, pre); then Types.REDECL(f1,
          {
          (
          SCode.COMPONENT(id1,f,r,p,attr,tp,SCode.NOMOD(),bc,comment),m_2)});
    case ((mod = Types.REDECL(f1,(els = {(SCode.COMPONENT(id1,_,_,_,_,_,_,_,_),_)}))),(mods = Types.MOD(_,subs,_)),env,pre) then mod;  "luc_pop : this shoud return the first mod because it have been merged in merge_subs  => Types.MOD(false,Types.NAMEMOD(id1,Types.REDECL(f1,els))::subs,
		       NONE) end luc_pop"
    case (Types.MOD(_,subs,_),Types.REDECL(f1,(els = {(SCode.COMPONENT(id1,_,_,_,_,_,_,_,_),_)})),env,pre) then Types.MOD(false,Types.NAMEMOD(id1,Types.REDECL(f1,els)) :: subs,NONE);  "luc_pop: this is needed to catch it there are the same.  it matches a a dc-motor with RampVoltage "
    case (outer_,inner_,_,_) "luc_pop: this is needed to catch it there are the same.  it matches a a dc-motor with RampVoltage   end luc_pop"
      equation 
        equality(outer_ = inner_); then outer_;
    case (_,(m = Types.MOD(true,_,_)),_,_)
      equation 
        Print.print_buf("# trying to modify final element\n") " end luc_pop"; then fail();
    case (Types.MOD(final_,subs1,ass1),Types.MOD(false,subs2,ass2),env,pre)
      equation 
        subs = merge_subs(subs1, subs2, env, pre);
        ass = merge_eq(ass1, ass2); then Types.MOD(final_,subs,ass);
    case (outer_,inner_,_,_) then outer_; 
  end matchcontinue;
end merge;

protected function merge_subs
  input Types_SubModList in_types_submodlist1;
  input Types_SubModList in_types_submodlist2;
  input Env.Env in_env3;
  input Prefix.Prefix in_prefix4;
  output Types_SubModList out_types_submodlist;
protected 
  type Types_SubModList = list<Types.SubMod>;
algorithm 
  out_types_submodlist:=
  matchcontinue (in_types_submodlist1,in_types_submodlist2,in_env3,in_prefix4)
    local
      type Env_FrameList = list<Env.Frame>;
      Types_SubModList s1,s1_1,ss,s2;
      Types.SubMod s_1,s;
      Env_FrameList env;
      Prefix.Prefix pre;
    case (s1,{},_,_) then s1; 
    case (s1,s :: s2,env,pre)
      equation 
        (s1_1,s_1) = merge_subs2(s1, s, env, pre);
        ss = merge_subs(s1_1, s2, env, pre); then s_1 :: ss;
  end matchcontinue;
end merge_subs;

protected function merge_subs2
  input Types_SubModList in_types_submodlist;
  input Types.SubMod in_submod;
  input Env.Env in_env;
  input Prefix.Prefix in_prefix;
  output Types_SubModList out_types_submodlist;
  output Types.SubMod out_submod;
protected 
  type Types_SubModList = list<Types.SubMod>;
algorithm 
  (out_types_submodlist,out_submod):=
  matchcontinue (in_types_submodlist,in_submod,in_env,in_prefix)
    local
      type Env_FrameList = list<Env.Frame>;
      type IntegerList = list<Integer>;
      Types.SubMod m,s,s1,s2;
      Types.Mod m1,m2;
      Ident n1,n2;
      Types_SubModList ss,ss_1;
      Env_FrameList env;
      Prefix.Prefix pre;
      IntegerList i1,i2;
    case ({},m,_,_) then ({},m); 
    case (Types.NAMEMOD(n1,m1) :: ss,Types.NAMEMOD(n2,m2),env,pre) " Modifications in the list take precedence "
      local Types.Mod m;
      equation 
        equality(n1 = n2);
        m = merge(m1, m2, env, pre); then (ss,Types.NAMEMOD(n1,m));
    case (Types.IDXMOD(i1,m1) :: ss,Types.IDXMOD(i2,m2),env,pre)
      local Types.Mod m;
      equation 
        equality(i1 = i2);
        m = merge(m1, m2, env, pre); then (ss,Types.IDXMOD(i1,m));
    case (s1 :: ss,s2,env,pre)
      equation 
        (ss_1,s) = merge_subs2(ss, s2, env, pre); then (s1 :: ss_1,s);
  end matchcontinue;
end merge_subs2;

protected function merge_eq "* function: merge_eq
 ** 
 ** The outer modification, given in the first argument, takes
 ** precedence over the inner modifications.
 *"
  input Types_EqModOption in_types_eqmodoption1;
  input Types_EqModOption in_types_eqmodoption2;
  output Types_EqModOption out_types_eqmodoption;
protected 
  type Types_EqModOption = Option<Types.EqMod>;
algorithm 
  out_types_eqmodoption:=
  matchcontinue (in_types_eqmodoption1,in_types_eqmodoption2)
    local Types_EqModOption e;
    case ((e = SOME(Types.TYPED(_,_))),_) then e; 
    case ((e = SOME(Types.UNTYPED(_))),_) then e; 
    case (NONE,e) then e; 
  end matchcontinue;
end merge_eq;

public function mod_equation "* function: mod_equation
 ** 
 ** This function simply extracts the equation part of a modification.
 *"
  input Types.Mod in_mod;
  output Types_EqModOption out_types_eqmodoption;
protected 
  type Types_EqModOption = Option<Types.EqMod>;
algorithm 
  out_types_eqmodoption:=
  matchcontinue (in_mod)
    local Types_EqModOption e;
    case Types.NOMOD() then NONE; 
    case Types.REDECL(_,_) then NONE; 
    case Types.MOD(_,_,e) then e; 
  end matchcontinue;
end mod_equation;

public function print_mod_str "* - Printing 
  !ignorecode
  * function: print_mod
 **
 ** This function prints a modification. It uses a few other function
 ** to do its stuff.
 **
 ** The functions are excluded from the report for brevity.
 *"
  input Types.Mod in_mod;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_mod)
    local
      type SCode_ElementList = list<SCode.Element>;
      type IdentList = list<Ident>;
      type TupleSCode_ElementTypes_Mod = tuple<SCode.Element,Types.Mod>;
      type TupleSCode_ElementTypes_ModList = list<TupleSCode_ElementTypes_Mod>;
      type Types_SubModList = list<Types.SubMod>;
      type Types_EqModOption = Option<Types.EqMod>;
      SCode_ElementList elist_1;
      Ident final_str,str,res,s1_1,s2;
      IdentList str_lst,s1;
      Boolean final_;
      TupleSCode_ElementTypes_ModList elist;
      Types_SubModList subs;
      Types_EqModOption eq;
    case (Types.NOMOD()) then "()"; 
    case Types.REDECL(final_,elist)
      equation 
        Print.print_buf("(redeclare(");
        elist_1 = Util.list_map(elist, Util.tuple2_1);
        final_str = Util.if_(final_, "final", "");
        str_lst = Util.list_map(elist_1, SCode.print_element_str);
        str = Util.string_delimit_list(str_lst, ", ");
        Print.print_buf("))");
        res = Util.string_append_list({"(redeclare(",final_str,str,"))"}); then res;
    case Types.MOD(final_,subs,eq)
      equation 
        final_str = Util.if_(final_, "final", "");
        s1 = print_subs1_str(subs);
        s1_1 = Util.string_delimit_list(s1, ",");
        s2 = print_eqmod_str(eq);
        str = Util.string_append_list({final_str,s1_1,s2}); then str;
  end matchcontinue;
end print_mod_str;

public function print_mod
  input Types.Mod m;
protected 
  Ident str;
algorithm 
  str := print_mod_str(m);
  Print.print_buf(str);
end print_mod;

protected function print_subs1_str ""
  input Types_SubModList in_types_submodlist;
  output StringList out_stringlist;
protected 
  type Types_SubModList = list<Types.SubMod>;
  type StringList = list<String>;
algorithm 
  out_stringlist:=
  matchcontinue (in_types_submodlist)
    local
      type IdentList = list<Ident>;
      Ident s1;
      IdentList res;
      Types.SubMod x;
      Types_SubModList xs;
    case {} then {}; 
    case x :: xs
      equation 
        s1 = print_sub_str(x);
        res = print_subs1_str(xs); then s1 :: res;
  end matchcontinue;
end print_subs1_str;

protected function print_sub_str ""
  input Types.SubMod in_submod;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_submod)
    local
      type IntegerList = list<Integer>;
      Ident mod_str,res,n,str;
      Types.Mod mod;
      IntegerList ss;
    case Types.NAMEMOD(n,mod)
      equation 
        mod_str = print_mod_str(mod);
        res = string_append(n, mod_str); then res;
    case Types.IDXMOD(ss,mod)
      equation 
        str = print_subscripts_str(ss);
        mod_str = print_mod_str(mod);
        res = string_append(str, mod_str); then res;
  end matchcontinue;
end print_sub_str;

protected function print_subscripts_str ""
  input IntegerList in_integerlist;
  output String out_string;
protected 
  type IntegerList = list<Integer>;
algorithm 
  out_string:=
  matchcontinue (in_integerlist)
    local
      Ident s,str,res;
      Integer x;
      IntegerList xs;
    case ({}) then "[]"; 
    case x :: xs
      equation 
        Print.print_buf("[");
        s = int_string(x);
        str = print_subscripts2_str(xs);
        res = Util.string_append_list({"[",s,str,"]"}); then res;
  end matchcontinue;
end print_subscripts_str;

protected function print_subscripts2_str ""
  input IntegerList in_integerlist;
  output String out_string;
protected 
  type IntegerList = list<Integer>;
algorithm 
  out_string:=
  matchcontinue (in_integerlist)
    local
      Ident s,str,res;
      Integer x;
      IntegerList xs;
    case ({}) then ""; 
    case x :: xs
      equation 
        Print.print_buf(",");
        s = int_string(x);
        str = print_subscripts2_str(xs);
        res = Util.string_append_list({",",s,str}); then res;
  end matchcontinue;
end print_subscripts2_str;

protected function print_eqmod_str ""
  input Types_EqModOption in_types_eqmodoption;
  output String out_string;
protected 
  type Types_EqModOption = Option<Types.EqMod>;
algorithm 
  out_string:=
  matchcontinue (in_types_eqmodoption)
    local
      Ident str,str2,res;
      Exp.Exp e;
      Types.Properties prop;
    case NONE then ""; 
    case SOME(Types.TYPED(e,prop))
      equation 
        str = Exp.print_exp_str(e);
        str2 = Types.print_prop_str(prop);
        res = Util.string_append_list({" = (typed)",str,str2}); then res;
    case SOME(Types.UNTYPED(e))
      local Absyn.Exp e;
      equation 
        str = Dump.print_exp_str(e);
        res = string_append(" =(untyped) ", str); then res;
  end matchcontinue;
end print_eqmod_str;
end Mod;

