package Prefix "
    Copyright (c) PELAB, Dept. Computer and Information Science, Linkoping University,
    Sweden, in the following denoted PELAB/LIU.

   
    This file is part of OpenModelica, in the following denoted OSM.


    The OSM License, Version 1.1, March 4, 2005.


    1 Preface

    The aim of this license is to lay down the conditions enabling you to use, 
    modify and circulate OSM. However, PELAB/LIU remain the authors of 
    OSM and so retain property rights and the use of all ancillary rights.

    2 Definitions

    OSM is defined as all successive versions of the OSM software and their 
    documentation that have been developed by PELAB/LIU and including
    accepted contributions from other contributors according to this license.

    OSM DERIVED SOFTWARE is defined as all or part of OSM that you have 
    modified and/or translated and/or adapted.


    3 Dual License

    OSM is made available under the OSM licensing scheme, which is a dual
    licensing scheme with two options, a) and b):

    a) OSM OPEN SOURCE LICENSE:

    If you wish to write Open Source software you can use the Open Source version 
    of OSM, released under the OSM license which include GPL as its open source 
    licensing option. If you use the OSM Open Source version you must release your 
    Application using OSM including this Application's source code under the GPL as well.

    This OSM license text, and Copyright (c) PELAB/Linkoping University, must
    be present in your copy of OSM and in OSM DERIVED SOFTWARE.

    You should have received a copy of the GPL - GNU General Public License
    along with OpenModelica; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.


    b) OSM COMMERCIAL LICENSE

    If you are using OSM commercially - that is, for commercial usage or
    for creating proprietary software for sale or use in a commercial setting
    - you must purchase a commercial license of OSM from PELAB/L
  *
 ** file:	 Prefix.rml
 ** module:      Prefix
 ** description: Prefix management
 **
 ** RCS: $Id: Prefix.rml 1733 2005-04-27 06:47:45Z haklu $
 **
 ** When instantiating an expression, there is a prefix that 
 ** has to be added to each variable name to be able to use it in the 
 ** flattened equation set.
 ** 
 ** A prefix for a variable x could be for example a.b.c so that the 
 ** fully qualified name is a.b.c.x. 
 **
 *"

public import OpenModelica.Compiler.Absyn;

public import OpenModelica.Compiler.Exp;

public import OpenModelica.Compiler.Env;

public import OpenModelica.Compiler.Lookup;

public 
type IntegerList = list<Integer>;

uniontype Prefix
  record NOPRE
  end NOPRE;
  record PRE
    String x1;
    IntegerList x2;
    Prefix x3;
  end PRE;
end Prefix;

protected import OpenModelica.Compiler.Util;

protected import OpenModelica.Compiler.Print;

public function print_prefix_str
  input Prefix in_prefix;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_prefix)
    local
      String str,s,rest_1,s_1,s_2;
      Prefix rest;
    case NOPRE() then "<NOPRE>"; 
    case PRE(str,{},NOPRE()) then str; 
    case PRE(str,_,NOPRE())
      equation 
        s = string_append(str, "[]"); then s;
    case PRE(str,{},rest)
      equation 
        rest_1 = print_prefix_str(rest);
        s = string_append(rest_1, ".");
        s_1 = string_append(s, str); then s_1;
    case PRE(str,_,rest)
      equation 
        rest_1 = print_prefix_str(rest);
        s = string_append(rest_1, ".");
        s_1 = string_append(s, str);
        s_2 = string_append(s_1, "[]"); then s_2;
  end matchcontinue;
end print_prefix_str;

public function print_prefix
  input Prefix p;
protected 
  String s;
algorithm 
  s := print_prefix_str(p);
  Print.print_buf(s);
end print_prefix;

public function prefix_add "* function: prefix_add
 **
 ** This function is used to extend a prefix with another level.  If
 ** the prefix `a.b{10}.c' is extended with `d' and an empty subscript
 ** list, the resulting prefix is `a.b{10}.c.d'.  Remember that
 ** prefixes components are stored in the opposite order from the
 ** normal order used when displaying them.
 *"
  input Exp.Ident in_ident;
  input IntegerList in_integerlist;
  input Prefix in_prefix;
  output Prefix out_prefix;
algorithm 
  out_prefix:=
  matchcontinue (in_ident,in_integerlist,in_prefix)
    local
      String i;
      IntegerList s;
      Prefix p;
    case (i,s,p) then PRE(i,s,p); 
  end matchcontinue;
end prefix_add;

public function prefix_path "* function: prefix_path
 **
 ** Prefix a `Path' variable by adding the supplied prefix to it and
 ** returning a new `Path'.
 *"
  input Absyn.Path in_path;
  input Prefix in_prefix;
  output Absyn.Path out_path;
algorithm 
  out_path:=
  matchcontinue (in_path,in_prefix)
    local
      Absyn.Path p,p_1;
      String s;
      Prefix ss;
    case (p,NOPRE()) then p; 
    case (p,PRE(s,_,ss))
      equation 
        p_1 = prefix_path(Absyn.QUALIFIED(s,p), ss); then p_1;
  end matchcontinue;
end prefix_path;

public function prefix_to_path "* function: prefix_to_path
 **
 ** Convert a Prefix to a `Path'
 *"
  input Prefix in_prefix;
  output Absyn.Path out_path;
algorithm 
  out_path:=
  matchcontinue (in_prefix)
    local
      String s;
      Absyn.Path p;
      Prefix ss;
    case NOPRE()
      equation 
        Print.print_buf("#-- Error: Cannot convert empty prefix to a path\n"); then fail();
    case PRE(s,_,NOPRE()) then Absyn.IDENT(s); 
    case PRE(s,_,ss)
      equation 
        p = prefix_to_path(ss); then Absyn.QUALIFIED(s,p);
  end matchcontinue;
end prefix_to_path;

public function prefix_cref "* function: prefix_cref
 **
 ** Prefix a `ComponentRef' variable by adding the supplied prefix to
 ** it and returning a new `ComponentRef'.
 **
 ** LS: Changed to call prefix_to_cref which is more general now
 *"
  input Prefix pre;
  input Exp.ComponentRef cref;
  output Exp.ComponentRef cref_1;
protected 
  Exp.ComponentRef cref_1;
algorithm 
  cref_1 := prefix_to_cref2(pre, SOME(cref));
end prefix_cref;

public function prefix_to_cref "* function: prefix_to_cref
 **
 ** Convert a prefix to a component reference.
 *"
  input Prefix pre;
  output Exp.ComponentRef cref_1;
protected 
  Exp.ComponentRef cref_1;
algorithm 
  cref_1 := prefix_to_cref2(pre, NONE);
end prefix_to_cref;

protected function prefix_to_cref2 "* function: prefix_to_cref2
 **
 ** Convert a prefix to a component reference. Converting NOPRE with no
 ** component reference is an error because a component reference cannot be
 ** empty
 *"
  input Prefix in_prefix;
  input Exp_ComponentRefOption in_exp_componentrefoption;
  output Exp.ComponentRef out_componentref;
protected 
  type Exp_ComponentRefOption = Option<Exp.ComponentRef>;
algorithm 
  out_componentref:=
  matchcontinue (in_prefix,in_exp_componentrefoption)
    local
      type Exp_SubscriptList = list<Exp.Subscript>;
      Exp.ComponentRef cref,cref_1;
      Exp_SubscriptList s_1;
      String i;
      IntegerList s;
      Prefix xs;
    case (NOPRE(),NONE)
      equation 
        Print.print_buf("#-- Cannot convert empty prefix to component reference\n"); then fail();
    case (NOPRE(),SOME(cref)) then cref; 
    case (PRE(i,s,xs),NONE)
      equation 
        s_1 = Exp.int_subscripts(s);
        cref_1 = prefix_to_cref2(xs, SOME(Exp.CREF_IDENT(i,s_1))); then cref_1;
    case (PRE(i,s,xs),SOME(cref))
      equation 
        s_1 = Exp.int_subscripts(s);
        cref_1 = prefix_to_cref2(xs, SOME(Exp.CREF_QUAL(i,s_1,cref))); then cref_1;
  end matchcontinue;
end prefix_to_cref2;

public function prefix_exp "* function: prefix_exp
 **
 ** Add the supplied prefix to all component references in an
 ** expression.
 *"
  input Env.Env in_env;
  input Exp.Exp in_exp;
  input Prefix in_prefix;
  output Exp.Exp out_exp;
algorithm 
  out_exp:=
  matchcontinue (in_env,in_exp,in_prefix)
    local
      type Env_FrameList = list<Env.Frame>;
      type Exp_ExpList = list<Exp.Exp>;
      type BooleanList = list<Boolean>;
      type TupleExp_ExpBoolean = tuple<Exp.Exp,Boolean>;
      type TupleExp_ExpBooleanList = list<TupleExp_ExpBoolean>;
      type TupleExp_ExpBooleanListList = list<TupleExp_ExpBooleanList>;
      Exp.Exp e,e1_1,e2_1,e1,e2,e3_1,e3,cref_1,dim_1,cref,dim,start_1,stop_1,start,stop,step_1,step,e_1,exp_1,iterexp_1,exp,iterexp;
      Exp.ComponentRef p_1,p;
      Env_FrameList env;
      Exp.Type t;
      Prefix pre;
      Exp.Operator o;
      Exp_ExpList es_1,es,el,el_1;
      Absyn.Path f,fcn;
      Boolean b,bi,a;
      BooleanList bl;
      TupleExp_ExpBooleanList x_1,x;
      TupleExp_ExpBooleanListList xs_1,xs;
      String id;
    case (_,(e = Exp.ICONST(_)),_) then e; 
    case (_,(e = Exp.RCONST(_)),_) then e; 
    case (_,(e = Exp.SCONST(_)),_) then e; 
    case (_,(e = Exp.BCONST(_)),_) then e; 
    case (env,Exp.CREF(p,t),pre)
      equation 
        (_,_,_) = Lookup.lookup_var_local(env, p);
        p_1 = prefix_cref(pre, p); then Exp.CREF(p_1,t);
    case (env,(e = Exp.CREF(p,_)),pre)
      equation 
        failure((_,_,_) = Lookup.lookup_var_local(env, p)); then e;
    case (env,Exp.BINARY(e1,o,e2),p)
      local Prefix p;
      equation 
        e1_1 = prefix_exp(env, e1, p);
        e2_1 = prefix_exp(env, e2, p); then Exp.BINARY(e1_1,o,e2_1);
    case (env,Exp.UNARY(o,e1),p)
      local Prefix p;
      equation 
        e1_1 = prefix_exp(env, e1, p); then Exp.UNARY(o,e1_1);
    case (env,Exp.LBINARY(e1,o,e2),p)
      local Prefix p;
      equation 
        e1_1 = prefix_exp(env, e1, p);
        e2_1 = prefix_exp(env, e2, p); then Exp.LBINARY(e1_1,o,e2_1);
    case (env,Exp.LUNARY(o,e1),p)
      local Prefix p;
      equation 
        e1_1 = prefix_exp(env, e1, p); then Exp.LUNARY(o,e1_1);
    case (env,Exp.RELATION(e1,o,e2),p)
      local Prefix p;
      equation 
        e1_1 = prefix_exp(env, e1, p);
        e2_1 = prefix_exp(env, e2, p); then Exp.RELATION(e1_1,o,e2_1);
    case (env,Exp.IFEXP(e1,e2,e3),p)
      local Prefix p;
      equation 
        e1_1 = prefix_exp(env, e1, p);
        e2_1 = prefix_exp(env, e2, p);
        e3_1 = prefix_exp(env, e3, p); then Exp.IFEXP(e1_1,e2_1,e3_1);
    case (env,Exp.SIZE(cref,SOME(dim)),p)
      local Prefix p;
      equation 
        cref_1 = prefix_exp(env, cref, p);
        dim_1 = prefix_exp(env, dim, p); then Exp.SIZE(cref_1,SOME(dim_1));
    case (env,Exp.SIZE(cref,NONE),p)
      local Prefix p;
      equation 
        cref_1 = prefix_exp(env, cref, p); then Exp.SIZE(cref_1,NONE);
    case (env,Exp.CALL(f,es,b,bi),p)
      local Prefix p;
      equation 
        es_1 = prefix_exp_list(env, es, p); then Exp.CALL(f,es_1,b,bi);
    case (env,Exp.ARRAY(t,a,{}),p)
      local Prefix p; then Exp.ARRAY(t,a,{});
    case (env,Exp.ARRAY(t,a,es),p)
      local Prefix p;
      equation 
        es_1 = prefix_exp_list(env, es, p); then Exp.ARRAY(t,a,es_1);
    case (env,Exp.TUPLE(es),p)
      local Prefix p;
      equation 
        es_1 = prefix_exp_list(env, es, p); then Exp.TUPLE(es_1);
    case (env,Exp.MATRIX(t,a,{}),p)
      local
        Integer a;
        Prefix p; then Exp.MATRIX(t,a,{});
    case (env,Exp.MATRIX(t,a,x :: xs),p)
      local
        Integer b,a;
        Prefix p;
      equation 
        el = Util.list_map(x, Util.tuple2_1);
        bl = Util.list_map(x, Util.tuple2_2);
        el_1 = prefix_exp_list(env, el, p);
        x_1 = Util.list_thread_tuple(el_1, bl);
        Exp.MATRIX(t,b,xs_1) = prefix_exp(env, Exp.MATRIX(t,a,xs), p); then Exp.MATRIX(t,a,x_1 :: xs_1);
    case (env,Exp.RANGE(t,start,NONE,stop),p)
      local Prefix p;
      equation 
        start_1 = prefix_exp(env, start, p);
        stop_1 = prefix_exp(env, stop, p); then Exp.RANGE(t,start_1,NONE,stop_1);
    case (env,Exp.RANGE(t,start,SOME(step),stop),p)
      local Prefix p;
      equation 
        start_1 = prefix_exp(env, start, p);
        step_1 = prefix_exp(env, step, p);
        stop_1 = prefix_exp(env, stop, p); then Exp.RANGE(t,start_1,SOME(step_1),stop_1);
    case (env,Exp.CAST(Exp.REAL(),e),p)
      local Prefix p;
      equation 
        e_1 = prefix_exp(env, e, p); then Exp.CAST(Exp.REAL(),e_1);
    case (env,Exp.REDUCTION(fcn,exp,id,iterexp),p)
      local Prefix p;
      equation 
        exp_1 = prefix_exp(env, exp, p);
        iterexp_1 = prefix_exp(env, iterexp, p); then Exp.REDUCTION(fcn,exp_1,id,iterexp_1);
    case (_,e,_)
      equation 
        Print.print_buf("- prefix_exp failed\n");
        Print.print_buf("  expression: ");
        Exp.print_exp(e);
        Print.print_buf("\n"); then fail();
  end matchcontinue;
end prefix_exp;

public function prefix_exp_list "* function: prefix_exp_list
 ** 
 ** This function prefixes a list of expressions using the
 ** `prefix_exp' function.
 *"
  input Env.Env in_env;
  input Exp_ExpList in_exp_explist;
  input Prefix in_prefix;
  output Exp_ExpList out_exp_explist;
protected 
  type Exp_ExpList = list<Exp.Exp>;
algorithm 
  out_exp_explist:=
  matchcontinue (in_env,in_exp_explist,in_prefix)
    local
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp e_1,e;
      Exp_ExpList es_1,es;
      Env_FrameList env;
      Prefix p;
    case (_,{},_) then {}; 
    case (env,e :: es,p)
      equation 
        e_1 = prefix_exp(env, e, p);
        es_1 = prefix_exp_list(env, es, p); then e_1 :: es_1;
  end matchcontinue;
end prefix_exp_list;

public function prefix_cref_list "* function: prefix_cref_list
 ** 
 ** This function prefixes a list of component references using the
 ** `prefix_cref function.
 *"
  input Prefix in_prefix;
  input Exp_ComponentRefList in_exp_componentreflist;
  output Exp_ComponentRefList out_exp_componentreflist;
protected 
  type Exp_ComponentRefList = list<Exp.ComponentRef>;
algorithm 
  out_exp_componentreflist:=
  matchcontinue (in_prefix,in_exp_componentreflist)
    local
      Exp.ComponentRef cr_1,cr;
      Exp_ComponentRefList crlist_1,crlist;
      Prefix p;
    case (_,{}) then {}; 
    case (p,cr :: crlist)
      equation 
        cr_1 = prefix_cref(p, cr);
        crlist_1 = prefix_cref_list(p, crlist); then cr_1 :: crlist_1;
  end matchcontinue;
end prefix_cref_list;
end Prefix;

