package Types "
    Copyright (c) PELAB, Dept. Computer and Information Science, Linkoping University,
    Sweden, in the following denoted PELAB/LIU.

   
    This file is part of OpenModelica, in the following denoted OSM.


    The OSM License, Version 1.1, March 4, 2005.


    1 Preface

    The aim of this license is to lay down the conditions enabling you to use, 
    modify and circulate OSM. However, PELAB/LIU remain the authors of 
    OSM and so retain property rights and the use of all ancillary rights.

    2 Definitions

    OSM is defined as all successive versions of the OSM software and their 
    documentation that have been developed by PELAB/LIU and including
    accepted contributions from other contributors according to this license.

    OSM DERIVED SOFTWARE is defined as all or part of OSM that you have 
    modified and/or translated and/or adapted.


    3 Dual License

    OSM is made available under the OSM licensing scheme, which is a dual
    licensing scheme with two options, a) and b):

    a) OSM OPEN SOURCE LICENSE:

    If you wish to write Open Source software you can use the Open Source version 
    of OSM, released under the OSM license which include GPL as its open source 
    licensing option. If you use the OSM Open Source version you must release your 
    Application using OSM including this Application's source code under the GPL as well.

    This OSM license text, and Copyright (c) PELAB/Linkoping University, must
    be present in your copy of OSM and in OSM DERIVED SOFTWARE.

    You should have received a copy of the GPL - GNU General Public License
    along with OpenModelica; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.


    b) OSM COMMERCIAL LICENSE

    If you are using OSM commercially - that is, for commercial usage or
    for creating proprietary software for sale or use in a commercial setting
    - you must purchase a commercial license of OSM from PELAB/L
  *
 ** file:	 Types.rml
 ** module:      Types
 ** description: Type system
 **
 ** RCS: $Id: Types.rml 1733 2005-04-27 06:47:45Z haklu $
 **
 ** This file specifies the type system, as defined in the modelica
 ** specification. It contains an RML type called `Type' which 
 ** defines types. It also contains functions for
 ** determining subtyping etc.
 **
 ** There are a few known problems with this module.  It currently
 ** depends on `SCode.Attributes', which in turn depends on
 ** `Absyn.ArrayDim'.  However, the only things used from those
 ** modules are constants that could be moved to their own modules.
 *"

public import OpenModelica.Compiler.ClassInf;

public import OpenModelica.Compiler.Absyn;

public import OpenModelica.Compiler.Exp;

public import OpenModelica.Compiler.Values;

public import OpenModelica.Compiler.SCode;

public 
type Ident = String "* - Identifiers ";

public 
uniontype Var "* - Variables "
  record VAR
    Ident x1 " name ";
    Attributes x2 " attributes ";
    Boolean x3 " protected ";
    Type x4 " type ";
    Binding x5 " equation modification ";
  end VAR;
end Var;

public 
uniontype Attributes
  record ATTR
    Boolean x1 " flow ";
    SCode.Accessibility x2;
    SCode.Variability x3 " parameter ";
    Absyn.Direction x4;
  end ATTR;
end Attributes;

public 
uniontype Binding
  record UNBOUND
  end UNBOUND;
  record EQBOUND
    Exp.Exp x1;
    Boolean x2 " constant ";
  end EQBOUND;
  record VALBOUND
    Values.Value x1;
  end VALBOUND;
end Binding;

public 
type Absyn_PathOption = Option<Absyn.Path>;

type Type = tuple<TType,Absyn_PathOption> "* - Types ";

public 
type VarList = list<Var>;

type StringList = list<String>;

type FuncArgList = list<FuncArg>;

type TypeList = list<Type>;

uniontype TType "*
	 **  A Type is a tuple of a TType (containing the actual type) and a optional classname
	 **  for the class where the type originates from.
	 *"
  record T_INTEGER
    VarList x1;
  end T_INTEGER;
  record T_REAL
    VarList x1;
  end T_REAL;
  record T_STRING
    VarList x1;
  end T_STRING;
  record T_BOOL
    VarList x1;
  end T_BOOL;
  record T_ENUM
  end T_ENUM;
  record T_ENUMERATION
    StringList x1;
    VarList x2;
  end T_ENUMERATION;
  record T_ARRAY
    ArrayDim x1;
    Type x2;
  end T_ARRAY;
  record T_COMPLEX
    ClassInf.State x1 " The type of. a class ";
    VarList x2;
  end T_COMPLEX;
  record T_FUNCTION
    FuncArgList x1;
    Type x2 " Only single-result ";
  end T_FUNCTION;
  record T_TUPLE
    TypeList x1 " For functions returning 
							   multiple values. ";
  end T_TUPLE;
  record T_NOTYPE " Used when type is not yet determined "
  end T_NOTYPE;
end TType;

public 
type IntegerOption = Option<Integer>;

uniontype ArrayDim
  record DIM
    IntegerOption x1;
  end DIM;
end ArrayDim;

public 
type FuncArg = tuple<Ident,Type>;

public 
type ConstList = list<Const>;

uniontype Const "* - Expression properties 
   A tuple is added to the Types. This is used by functions whom
returns multiple arguments. 
   Used by split_props  "
  record CONST
    Boolean x1;
  end CONST;
  record TUPLE_CONST
    ConstList x1;
  end TUPLE_CONST;
end Const;

public 
uniontype Properties
  record PROP
    Type x1;
    Boolean x2 " type   if the type is a tuple, each element 
				          have a const flag.   Type is meant to be T_TUPLE ";
  end PROP;
  record PROP_TUPLE
    Type x1;
    Const x2 " The elements might be 
							    tuple themselfs. ";
  end PROP_TUPLE;
end Properties;

public 
uniontype EqMod "	* P.R 1.1 for multiple return arguments from functions, 
	*  one constant flag for each return argument. 
	*

  * The datatype `Properties' contain information about an
   ** expression.  The properties are created by analyzing the
   ** expressions. 
  * To generate the correct set of equations, the translator has to 
  * differentiate between the primitive types `Real', `Integer', 
  * `String', `Boolean' and types directly derived from then from 
  * other, complex types.  For arrays and matrices the type 
  * `T_ARRAY' is used, with the first argument being the number of 
  * dimensions, and the second being the type of the objects in the 
  * array.  The `Type' type is used to store 
  * information about whether a class is derived from a primitive 
  * type, and whether a variable is of one of these types.  
  * - Modification datatype, was originally in Mod "
  record TYPED
    Exp.Exp x1;
    Properties x2;
  end TYPED;
  record UNTYPED
    Absyn.Exp x1;
  end UNTYPED;
end EqMod;

public 
type IntegerList = list<Integer>;

uniontype SubMod
  record NAMEMOD
    Ident x1;
    Mod x2;
  end NAMEMOD;
  record IDXMOD
    IntegerList x1;
    Mod x2;
  end IDXMOD;
end SubMod;

public 
type SubModList = list<SubMod>;

type EqModOption = Option<EqMod>;

type TupleSCode_ElementMod = tuple<SCode.Element,Mod>;

type TupleSCode_ElementModList = list<TupleSCode_ElementMod>;

uniontype Mod
  record MOD
    Boolean x1 " final ";
    SubModList x2;
    EqModOption x3;
  end MOD;
  record REDECL
    Boolean x1 " final ";
    TupleSCode_ElementModList x2;
  end REDECL;
  record NOMOD
  end NOMOD;
end Mod;

protected import OpenModelica.Compiler.Dump;

protected import OpenModelica.Compiler.Debug;

protected import OpenModelica.Compiler.Print;

protected import OpenModelica.Compiler.Util;

protected import OpenModelica.Compiler.Static;

public function is_array
  input Type in_type;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_type)
    case ((T_ARRAY(_,_),_)) then true; 
    case ((_,_)) then false; 
  end matchcontinue;
end is_array;

public function is_string
  input Type in_type;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_type)
    case ((T_STRING(_),_)) then true; 
    case ((_,_)) then false; 
  end matchcontinue;
end is_string;

public function is_array_or_string
  input Type in_type;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_type)
    local Type ty;
    case ty
      equation 
        true = is_array(ty); then true;
    case ty
      equation 
        true = is_string(ty); then true;
    case _ then false; 
  end matchcontinue;
end is_array_or_string;

public function ndims
  input Type in_type;
  output Integer out_integer;
algorithm 
  out_integer:=
  matchcontinue (in_type)
    local
      Integer n;
      Type t;
    case ((T_ARRAY(_,t),_))
      equation 
        n = ndims(t); then n + 1;
    case ((_,_)) then 0; 
  end matchcontinue;
end ndims;

public function dimensions_known
  input Type in_type;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_type)
    local Type tp;
    case (tp)
      equation 
        {} = get_dimension_sizes(tp); then false;
    case (tp)
      equation 
        _ = get_dimension_sizes(tp); then true;
  end matchcontinue;
end dimensions_known;

public function get_dimension_sizes
  input Type in_type;
  output IntegerList out_integerlist;
algorithm 
  out_integerlist:=
  matchcontinue (in_type)
    local
      IntegerList res;
      Integer i;
      Type tp;
    case ((T_ARRAY(DIM(SOME(i)),tp),_))
      equation 
        res = get_dimension_sizes(tp); then i :: res;
    case ((_,_)) then {}; 
  end matchcontinue;
end get_dimension_sizes;

public function values_to_mods "* function: values_to_mods
 ** author: PA
 **
 ** This function takes a list of values and convert into a Modification.
 **  Used for record construction evaluation. PersonRecord(\"name\",45) has a value list 
 ** { \"name\",45 } that needs to be converted into a modifier for the record class
 **  PersonRecord (\"name,45)
 **  FIXME: How about other value types, e.g. array, enum etc 
*"
  input Values_ValueList in_values_valuelist;
  input IdentList in_identlist;
  output Mod out_mod;
protected 
  type Values_ValueList = list<Values.Value>;
  type IdentList = list<Ident>;
algorithm 
  out_mod:=
  matchcontinue (in_values_valuelist,in_identlist)
    local
      SubModList res;
      Integer i;
      Values_ValueList rest,vals;
      Ident id,s,cname_str,vs;
      IdentList ids,val_names;
      Real r;
      Boolean b;
      Exp.Exp rec_call;
      VarList varlst;
      Absyn.Path cname;
      Values.Value v;
    case ({},_) then MOD(false,{},NONE); 
    case (Values.INTEGER(i) :: rest,id :: ids)
      equation 
        MOD(_,res,_) = values_to_mods(rest, ids); then MOD(false,
          NAMEMOD(id,
          MOD(false,{},
          SOME(TYPED(Exp.ICONST(i),PROP((T_INTEGER({}),NONE),false))))) :: res,NONE);
    case (Values.REAL(r) :: rest,id :: ids)
      equation 
        MOD(_,res,_) = values_to_mods(rest, ids); then MOD(false,
          NAMEMOD(id,
          MOD(false,{},
          SOME(TYPED(Exp.RCONST(r),PROP((T_REAL({}),NONE),false))))) :: res,NONE);
    case (Values.STRING(s) :: rest,id :: ids)
      equation 
        MOD(_,res,_) = values_to_mods(rest, ids); then MOD(false,
          NAMEMOD(id,
          MOD(false,{},
          SOME(TYPED(Exp.SCONST(s),PROP((T_STRING({}),NONE),false))))) :: res,NONE);
    case (Values.BOOL(b) :: rest,id :: ids)
      equation 
        MOD(_,res,_) = values_to_mods(rest, ids); then MOD(false,
          NAMEMOD(id,
          MOD(false,{},
          SOME(TYPED(Exp.BCONST(b),PROP((T_BOOL({}),NONE),false))))) :: res,NONE);
    case (Values.RECORD(cname,vals,val_names) :: rest,id :: ids)
      equation 
        MOD(_,res,_) = values_to_mods(rest, ids);
        rec_call = values_to_record_constructor_call(cname, vals);
        varlst = values_to_vars(vals, val_names);
        cname_str = Absyn.path_string(cname); then MOD(false,
          NAMEMOD(id,
          MOD(false,{},
          SOME(
          TYPED(rec_call,
          PROP((T_COMPLEX(ClassInf.RECORD(cname_str),varlst),NONE),false))))) :: res,NONE);
    case (v :: _,_)
      equation 
        Debug.fprint("failtrace", "-values_to_mods failed for value: ");
        vs = Values.val_string(v);
        Debug.fprint("failtrace", vs);
        Debug.fprint("failtrace", "\n"); then fail();
  end matchcontinue;
end values_to_mods;

protected function values_to_record_constructor_call
  input Absyn.Path funcname;
  input Values_ValueList values;
  output Exp.Exp out_exp;
protected 
  type Values_ValueList = list<Values.Value>;
  type Exp_ExpList = list<Exp.Exp>;
  Exp_ExpList expl;
algorithm 
  expl := Util.list_map(values, Static.value_exp);
  out_exp := Exp.CALL(funcname,expl,false,false);
end values_to_record_constructor_call;

public function values_to_vars "* function values_to_vars
 ** 
 ** Translates a list of Values.Value to a Var list, using a list
 ** of identifiers as component names.
 ** Used e.g. when retrieving the type of a record value.
 *"
  input Values_ValueList in_values_valuelist;
  input Exp_IdentList in_exp_identlist;
  output VarList out_varlist;
protected 
  type Values_ValueList = list<Values.Value>;
  type Exp_IdentList = list<Exp.Ident>;
algorithm 
  out_varlist:=
  matchcontinue (in_values_valuelist,in_exp_identlist)
    local
      type IdentList = list<Ident>;
      Type tp;
      VarList rest;
      Values.Value v;
      Values_ValueList vs;
      Ident id;
      IdentList ids;
    case ({},{}) then {}; 
    case (v :: vs,id :: ids)
      equation 
        tp = type_of_value(v);
        rest = values_to_vars(vs, ids); then VAR(id,ATTR(false,SCode.RW(),SCode.VAR(),Absyn.BIDIR()),false,
          tp,UNBOUND()) :: rest;
    case (_,_)
      equation 
        Debug.fprint("failtrace", "-values_to_vars failed\n"); then fail();
  end matchcontinue;
end values_to_vars;

public function type_of_value "* function: type_of_value
 ** author: PA
 **
 ** Returns the type of a Values.Value.
 ** Some information is lost in the translation, like attributes
 ** of the builtin type.
 *"
  input Values.Value in_value;
  output Type out_type;
algorithm 
  out_type:=
  matchcontinue (in_value)
    local
      type Values_ValueList = list<Values.Value>;
      type IdentList = list<Ident>;
      Type tp;
      Integer dim1;
      Values.Value w,v;
      Values_ValueList vs,vl;
      TypeList ts;
      VarList vars;
      Ident cname_str;
      Absyn.Path cname;
      IdentList ids;
    case (Values.INTEGER(_)) then ((T_INTEGER({}),NONE)); 
    case (Values.REAL(_)) then ((T_REAL({}),NONE)); 
    case (Values.STRING(_)) then ((T_STRING({}),NONE)); 
    case (Values.BOOL(_)) then ((T_BOOL({}),NONE)); 
    case (Values.ENUM(_)) then ((T_ENUM(),NONE)); 
    case ((w = Values.ARRAY(v :: vs)))
      equation 
        tp = type_of_value(v);
        dim1 = list_length(v :: vs); then ((T_ARRAY(DIM(SOME(dim1)),tp),NONE));
    case ((w = Values.TUPLE(vs)))
      equation 
        ts = Util.list_map(vs, type_of_value); then ((T_TUPLE(ts),NONE));
    case Values.RECORD(cname,vl,ids)
      equation 
        vars = values_to_vars(vl, ids);
        cname_str = Absyn.path_string(cname); then ((T_COMPLEX(ClassInf.RECORD(cname_str),vars),NONE));
    case (v)
      local Ident vs;
      equation 
        Debug.fprint("failtrace", "-type_of_values failed: ");
        vs = Values.val_string(v);
        Debug.fprintln("failtrace", vs); then fail();
  end matchcontinue;
end type_of_value;

public function basic_type "* function: basic_type
 **
 ** Test whether a type is one of the builtin types.
 *"
  input Type in_type;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_type)
    case ((T_INTEGER(_),_)) then true; 
    case ((T_REAL(_),_)) then true; 
    case ((T_STRING(_),_)) then true; 
    case ((T_BOOL(_),_)) then true; 
    case ((T_ENUM(),_)) then true; 
    case ((T_ARRAY(_,_),_)) then false; 
    case ((T_COMPLEX(_,_),_)) then false; 
    case ((T_ENUMERATION(_,_),_)) then false; 
  end matchcontinue;
end basic_type;

public function equivtypes "* function: equivtypes
 **
 ** This is the type equivalence function.  It is defined in terms of
 ** the subtype function.  Two types are considered equivalent if they
 ** are subtypes of each other.
 *"
  input Type in_type1;
  input Type in_type2;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_type1,in_type2)
    local Type t1,t2;
    case (t1,t2)
      equation 
        true = subtype(t1, t2);
        true = subtype(t2, t1); then true;
    case (t1,t2) then false; 
  end matchcontinue;
end equivtypes;

public function subtype "* function: subtype
 **
 ** Is the first type a subtype of the second type?  This function
 ** specifies the rules for subtyping in Modelica.
 *"
  input Type in_type1;
  input Type in_type2;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_type1,in_type2)
    local
      type IdentList = list<Ident>;
      Boolean res;
      Ident l1,l2;
      IdentList rest1,rest2;
      VarList vl1,vl2,els1,els2;
      Absyn_PathOption p1,p2;
      Type t1,t2;
      Integer i1,i2;
      ClassInf.State st1,st2;
      TypeList type_list1,type_list2;
    case ((T_INTEGER(_),_),(T_INTEGER(_),_)) then true; 
    case ((T_REAL(_),_),(T_REAL(_),_)) then true; 
    case ((T_STRING(_),_),(T_STRING(_),_)) then true; 
    case ((T_BOOL(_),_),(T_BOOL(_),_)) then true; 
    case ((T_ENUM(),_),(T_ENUM(),_)) then true; 
    case ((T_ENUMERATION(l1 :: rest1,vl1),p1),(T_ENUMERATION(l2 :: rest2,vl2),p2))
      equation 
        equality(l2 = l1);
        res = subtype((T_ENUMERATION(rest1,vl1),p1), 
          (T_ENUMERATION(rest2,vl2),p2)); then res;
    case ((T_ENUMERATION({},_),_),(T_ENUMERATION(_,_),_)) then true; 
    case ((T_ARRAY(_,t1),_),(T_ARRAY(DIM(NONE),t2),_))
      equation 
        true = subtype(t1, t2); then true;
    case ((T_ARRAY(DIM(NONE),t1),_),(T_ARRAY(_,t2),_))
      equation 
        true = subtype(t1, t2); then true;
    case ((T_ARRAY(DIM(SOME(i1)),t1),_),(T_ARRAY(DIM(SOME(i2)),t2),_))
      equation 
        equality(i1 = i2);
        true = subtype(t1, t2); then true;
    case ((T_COMPLEX(st1,els1),_),(T_COMPLEX(st2,els2),_))
      equation 
        true = subtype_varlist(els1, els2); then true;
    case ((T_TUPLE(type_list1),_),(T_TUPLE(type_list2),_)) " PR. Check of tuples, similar to complex. Just that
	     identifier name do not have to be checked. Only types are
	 checked. "
      equation 
        true = subtype_typelist(type_list1, type_list2); then true;
    case (t1,t2) then false;  " What? If not subtye should return false. Doesn't mean no matching rule

  rule	Debug.fprint (\"tytr\", \"subtype: no matching subtype rule.\n\") &
	Debug.fcall (\"tytr\", print_type, t1) &
	Debug.fprint (\"tytr\", \" <> \") &
	Debug.fcall (\"tytr\", print_type, t2) &
	Debug.fprint (\"tytr\", \"\n\")
	-----------------------------------
	subtype(t1,t2) => false
"
  end matchcontinue;
end subtype;

protected function subtype_typelist "* PR. function: subtype_typelist
 **
 ** This function checks if the both `Type' lists matches types, element
 ** by element.
 *"
  input TypeList in_typelist1;
  input TypeList in_typelist2;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_typelist1,in_typelist2)
    local
      Type t1,t2;
      TypeList rest1,rest2;
    case ({},{}) then true; 
    case (t1 :: rest1,t2 :: rest2)
      equation 
        true = subtype(t1, t2);
        true = subtype_typelist(rest1, rest2); then true;
    case (_,_) then false; 
  end matchcontinue;
end subtype_typelist;

protected function subtype_varlist "* function: subtype_varlist
 **
 ** This function checks if the `Var' list in the first list is a
 ** subset of the list in the second argument.  More precisely, it
 ** checks if, for each `Var' in the second list there is a `Var' in
 ** the first list with a type that is a subtype of the `Var' in the
 ** second list.
 *"
  input VarList in_varlist1;
  input VarList in_varlist2;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_varlist1,in_varlist2)
    local
      Type t1,t2;
      VarList l,vs;
      Ident n;
    case (_,{}) then true; 
    case (l,VAR(n,_,_,t2,_) :: vs)
      equation 
        VAR(_,_,_,t1,_) = varlist_lookup(l, n);
        true = subtype(t1, t2);
        true = subtype_varlist(l, vs); then true;
    case (_,_) then false; 
  end matchcontinue;
end subtype_varlist;

protected function varlist_lookup "* function: varlist_lookup
 **
 ** Given a list of `Var' and a name, this function finds any `Var'
 ** with the given name.
 *"
  input VarList in_varlist;
  input Ident in_ident;
  output Var out_var;
algorithm 
  out_var:=
  matchcontinue (in_varlist,in_ident)
    local
      Var v;
      Ident n,name;
      VarList vs;
    case ((v = VAR(n,_,_,_,_)) :: _,name)
      equation 
        equality(n = name); then v;
    case (v :: vs,name)
      equation 
        v = varlist_lookup(vs, name); then v;
  end matchcontinue;
end varlist_lookup;

public function lookup_component "* function: lookup_component
 **
 ** This function finds a subcomponent by name.
 *"
  input Type in_type;
  input Ident in_ident;
  output Var out_var;
algorithm 
  out_var:=
  matchcontinue (in_type,in_ident)
    local
      Var v;
      Type t,ty,ty_1;
      Ident n,id;
      ClassInf.State st;
      VarList cs;
      Attributes attr;
      Boolean prot;
      Binding bnd;
      ArrayDim dim;
    case (t,n)
      equation 
        true = basic_type(t);
        v = lookup_in_builtin(t, n); then v;
    case ((T_COMPLEX(st,cs),_),id)
      equation 
        v = lookup_component2(cs, id); then v;
    case ((T_ARRAY(dim,(T_COMPLEX(st,cs),_)),_),id)
      equation 
        VAR(n,attr,prot,ty,bnd) = lookup_component2(cs, id);
        ty_1 = (T_ARRAY(dim,ty),NONE); then VAR(n,attr,prot,ty_1,bnd);
    case (_,id) then fail(); 
  end matchcontinue;
end lookup_component;

protected function lookup_in_builtin "* function: lookup_in_builtin
 **
 ** Since builtin types are not represented as T_COMPLEX, special care
 ** is needed to be able to lookup the attributes (`start' etc) in
 ** them.
 **
 ** This is not a complete solution.  The current way of mapping the
 ** both the Modelica type `Real' and the simple type `RealType' to
 ** `T_REAL' is a bit problematic, since it doesn't make a
 ** difference between `Real' and `RealType', which makes the
 ** translator accept things like `x.start.start.start'.
 *"
  input Type in_type;
  input Ident in_ident;
  output Var out_var;
algorithm 
  out_var:=
  matchcontinue (in_type,in_ident)
    local
      Var v;
      VarList cs;
      Ident id;
    case ((T_REAL(cs),_),id) " Real "
      equation 
        v = lookup_component2(cs, id); then v;
    case ((T_INTEGER(cs),_),id)
      equation 
        v = lookup_component2(cs, id); then v;
    case ((T_STRING(cs),_),id)
      equation 
        v = lookup_component2(cs, id); then v;
    case ((T_BOOL(cs),_),id)
      equation 
        v = lookup_component2(cs, id); then v;
    case ((T_ENUM(),_),"quantity") then VAR("quantity",
          ATTR(false,SCode.RW(),SCode.PARAM(),Absyn.BIDIR()),false,(T_STRING({}),NONE),VALBOUND(Values.STRING(""))); 
    case ((T_ENUM(),_),"min") then VAR("min",ATTR(false,SCode.RW(),SCode.PARAM(),Absyn.BIDIR()),
          false,(T_ENUM(),NONE),UNBOUND()); 
    case ((T_ENUM(),_),"max") then VAR("max",ATTR(false,SCode.RW(),SCode.PARAM(),Absyn.BIDIR()),
          false,(T_ENUM(),NONE),UNBOUND()); 
    case ((T_ENUM(),_),"start") then VAR("start",ATTR(false,SCode.RW(),SCode.PARAM(),Absyn.BIDIR()),
          false,(T_BOOL({}),NONE),UNBOUND()); 
    case ((T_ENUM(),_),"fixed") then VAR("fixed",ATTR(false,SCode.RW(),SCode.PARAM(),Absyn.BIDIR()),
          false,(T_BOOL({}),NONE),UNBOUND()); 
    case ((T_ENUM(),_),"enable") then VAR("enable",
          ATTR(false,SCode.RW(),SCode.PARAM(),Absyn.BIDIR()),false,(T_BOOL({}),NONE),VALBOUND(Values.BOOL(true))); 
  end matchcontinue;
end lookup_in_builtin;

protected function lookup_component2 "* function: lookup_component2
 **
 ** This function finds a named `Var' in a list of `Var's, comparing
 ** the name against the second argument to this function.
 *"
  input VarList in_varlist;
  input Ident in_ident;
  output Var out_var;
algorithm 
  out_var:=
  matchcontinue (in_varlist,in_ident)
    local
      Var v;
      Ident n,m;
      VarList vs;
    case ((v = VAR(n,_,_,_,_)) :: _,m)
      equation 
        equality(n = m); then v;
    case (v :: vs,n)
      equation 
        v = lookup_component2(vs, n); then v;
  end matchcontinue;
end lookup_component2;

public function make_array "* function: make_array
 **  This function makes an array type given a Type and an Absyn.ArrayDim
 *"
  input Type in_type;
  input Absyn.ArrayDim in_arraydim;
  output Type out_type;
algorithm 
  out_type:=
  matchcontinue (in_type,in_arraydim)
    local
      type Absyn_SubscriptList = list<Absyn.Subscript>;
      Type t;
      Integer len;
      Absyn_SubscriptList l;
    case (t,{}) then t; 
    case (t,l)
      equation 
        len = list_length(l); then ((T_ARRAY(DIM(SOME(len)),t),NONE));
  end matchcontinue;
end make_array;

public function lift_array "* function: lift_array
 **
 ** This function turns a type into an array of that type.  If the
 ** type already is an array, aonther dimension is simply added.
 *"
  input Type in_type;
  input IntegerOption in_integeroption;
  output Type out_type;
algorithm 
  out_type:=
  matchcontinue (in_type,in_integeroption)
    local
      Type ty;
      IntegerOption i;
    case (ty,i) then ((T_ARRAY(DIM(i),ty),NONE));  " PR  axiom	lift_array (ty,i) => T_ARRAY(DIM(i),ty) "
  end matchcontinue;
end lift_array;

public function unlift_array "* function: unlift_array
 **
 ** This function turns an array of a type into that type.
 *"
  input Type in_type;
  output Type out_type;
algorithm 
  out_type:=
  matchcontinue (in_type)
    local Type ty;
    case ((T_ARRAY(DIM(_),ty),_)) then ty; 
  end matchcontinue;
end unlift_array;

public function array_element_type "* function: array_element_type
 **
 ** This function turns an array into the element type
 ** of the array.
 *"
  input Type in_type;
  output Type out_type;
algorithm 
  out_type:=
  matchcontinue (in_type)
    local Type ty_1,ty,t;
    case ((T_ARRAY(DIM(_),ty),_))
      equation 
        ty_1 = array_element_type(ty); then ty_1;
    case t then t; 
  end matchcontinue;
end array_element_type;

public function unparse_type "* function: unparse_type
 **
 ** This function prints a Modelica type as a piece of Modelica code.
*"
  input Type in_type;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_type)
    local
      type IdentList = list<Ident>;
      Ident s1,str,tys,dims,res,vstr,name,paramstr,restypestr,tystr;
      IdentList l,dimlststr,dimlststr_1,vars,paramstrs,tystrs;
      Type ty,t,restype;
      IntegerList dimlst;
      VarList vs;
      ClassInf.State ci_state;
      FuncArgList params;
    case ((T_INTEGER(_),_)) then "Integer"; 
    case ((T_REAL(_),_)) then "Real"; 
    case ((T_STRING(_),_)) then "String"; 
    case ((T_BOOL(_),_)) then "Boolean"; 
    case ((T_ENUMERATION(l,_),_))
      equation 
        s1 = Util.string_delimit_list(l, ",");
        str = Util.string_append_list({"enumeration(",s1,")"}); then str;
    case ((t = (T_ARRAY(_,_),_)))
      equation 
        (ty,dimlst) = flatten_array_type(t);
        dimlststr = Util.list_map(dimlst, int_string);
        dimlststr_1 = list_reverse(dimlststr);
        tys = unparse_type(ty);
        dims = Util.string_delimit_list(dimlststr_1, ", ");
        res = Util.string_append_list({tys,"[",dims,"]"}); then res;
    case (((t = T_COMPLEX(ClassInf.RECORD(name),vs)),_))
      local TType t;
      equation 
        vars = Util.list_map(vs, unparse_var);
        vstr = Util.string_append_list(vars);
        res = Util.string_append_list({"record ",name,"\n",vstr,"end record;"}); then res;
    case ((T_COMPLEX(ci_state,vs),_))
      equation 
        res = ClassInf.get_state_name(ci_state); then res;
    case ((T_FUNCTION(params,restype),_))
      equation 
        paramstrs = Util.list_map(params, unparse_param);
        paramstr = Util.string_delimit_list(paramstrs, ", ");
        restypestr = unparse_type(restype);
        res = Util.string_append_list({"/* function(",paramstr,") => ",restypestr,"*/"}); then res;
    case ((T_TUPLE(tys),_))
      local TypeList tys;
      equation 
        tystrs = Util.list_map(tys, unparse_type);
        tystr = Util.string_delimit_list(tystrs, ", ");
        res = Util.string_append_list({"/* (",tystr,") */"}); then res;
    case ((T_NOTYPE(),_)) then "/* #NOTYPE# */"; 
    case (ty) then "/* unparse_type: not implemented yet */\n"; 
  end matchcontinue;
end unparse_type;

public function unparse_const
  input Const in_const;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_const)
    local
      type IdentList = list<Ident>;
      Ident bstr,res,res_1;
      Boolean b;
      IdentList strlist;
      ConstList constlist;
    case CONST(b)
      equation 
        bstr = Util.bool_string(b); then bstr;
    case TUPLE_CONST(constlist)
      equation 
        strlist = Util.list_map(constlist, unparse_const);
        res = Util.string_delimit_list(strlist, ", ");
        res_1 = Util.string_append_list({"(",res,")"}); then res_1;
  end matchcontinue;
end unparse_const;

public function print_type "* function: print_type
 **
 ** This function prints a textual description of a Modelica type.  If
 ** the type is not one of the primitive types, it simply prints
 ** `composite'.  The actual code is expluded from the report.
 *
  !ignorecode"
  input Type in_type;
algorithm 
  _:=
  matchcontinue (in_type)
    local
      type IdentList = list<Ident>;
      VarList vars;
      IdentList l;
      ClassInf.State st;
      ArrayDim dim;
      Type t,restype;
      FuncArgList params;
      TypeList tys;
    case ((T_INTEGER(vars),_))
      equation 
        Print.print_buf("Integer");
        Print.print_buf(" (");
        Dump.print_list(vars, print_var, ", ");
        Print.print_buf(")"); then ();
    case ((T_REAL(vars),_))
      equation 
        Print.print_buf("Real");
        Print.print_buf(" (");
        Dump.print_list(vars, print_var, ", ");
        Print.print_buf(")"); then ();
    case ((T_STRING(vars),_))
      equation 
        Print.print_buf("String");
        Print.print_buf(" (");
        Dump.print_list(vars, print_var, ", ");
        Print.print_buf(")"); then ();
    case ((T_BOOL(vars),_))
      equation 
        Print.print_buf("Boolean");
        Print.print_buf(" (");
        Dump.print_list(vars, print_var, ", ");
        Print.print_buf(")"); then ();
    case ((T_ENUM(),_))
      equation 
        Print.print_buf("EnumType"); then ();
    case ((T_ENUMERATION(l,vars),_))
      equation 
        Print.print_buf("enumeration((");
        Dump.print_list(l, print, ", ");
        Print.print_buf(") ");
        Print.print_buf(", (");
        Dump.print_list(vars, print_var, ", ");
        Print.print_buf(")"); then ();
    case ((T_COMPLEX(st,vars),_))
      equation 
        Print.print_buf("composite(");
        Print.print_buf(", (");
        ClassInf.print_state(st);
        Print.print_buf(", (");
        Dump.print_list(vars, print_var, ", ");
        Print.print_buf(")"); then ();
    case ((T_ARRAY(dim,t),_))
      equation 
        Print.print_buf("array[");
        print_arraydim(dim);
        Print.print_buf("] of ");
        print_type(t);
        Print.print_buf(")"); then ();
    case ((T_FUNCTION(params,restype),_))
      equation 
        Print.print_buf("function(");
        print_params(params);
        Print.print_buf(" => ");
        print_type(restype);
        Print.print_buf(")"); then ();
    case ((T_TUPLE(tys),_))
      equation 
        Print.print_buf("(");
        Dump.print_list(tys, print_type, ", ");
        Print.print_buf(")"); then ();
    case ((T_NOTYPE(),_))
      equation 
        Print.print_buf("#NOTYPE#"); then ();
    case ((_,_))
      equation 
        Print.print_buf("print_type failed!\n"); then ();
  end matchcontinue;
end print_type;

protected function print_arraydim
  input ArrayDim ad;
protected 
  Ident s;
algorithm 
  s := get_arraydim_str(ad);
  Print.print_buf(s);
end print_arraydim;

protected function get_arraydim_str
  input ArrayDim in_arraydim;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_arraydim)
    local
      Ident s;
      Integer i;
    case DIM(NONE) then ":"; 
    case DIM(SOME(i))
      equation 
        s = int_string(i); then s;
    case _ then "#STRANGE#"; 
  end matchcontinue;
end get_arraydim_str;

public function print_params
  input FuncArgList in_funcarglist;
algorithm 
  _:=
  matchcontinue (in_funcarglist)
    local
      Ident n;
      Type t;
      FuncArgList params;
    case {} then (); 
    case {(n,t)}
      equation 
        Print.print_buf(n);
        Print.print_buf(" :: ");
        print_type(t); then ();
    case ((n,t) :: params)
      equation 
        Print.print_buf(n);
        Print.print_buf(" :: ");
        print_type(t);
        Print.print_buf(" * ");
        print_params(params); then ();
  end matchcontinue;
end print_params;

protected function unparse_var "!includecode"
  input Var in_var;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_var)
    local
      Ident t,res,n;
      Attributes attr;
      Boolean prot;
      Type typ;
      Binding bind;
    case VAR(n,attr,prot,typ,bind)
      equation 
        t = unparse_type(typ);
        res = Util.string_append_list({t," ",n,";\n"}); then res;
  end matchcontinue;
end unparse_var;

protected function unparse_param
  input FuncArg in_funcarg;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_funcarg)
    local
      Ident tstr,res,id;
      Type ty;
    case ((id,ty))
      equation 
        tstr = unparse_type(ty);
        res = Util.string_append_list({id,":",tstr}); then res;
  end matchcontinue;
end unparse_param;

public function print_var " LS "
  input Var in_var;
algorithm 
  _:=
  matchcontinue (in_var)
    local
      Ident vs,n;
      SCode.Variability var;
      Boolean prot;
      Type typ;
      Binding bind;
    case VAR(n,ATTR(_,_,var,_),prot,typ,bind)
      equation 
        print_type(typ);
        Print.print_buf(" ");
        Print.print_buf(n);
        Print.print_buf(" ");
        vs = SCode.variability_string(var);
        Print.print_buf(vs);
        Print.print_buf(" ");
        print_binding(bind); then ();
  end matchcontinue;
end print_var;

public function print_binding " LS "
  input Binding in_binding;
algorithm 
  _:=
  matchcontinue (in_binding)
    local
      Exp.Exp exp;
      Boolean f;
      Values.Value v;
    case UNBOUND()
      equation 
        Print.print_buf("UNBOUND"); then ();
    case EQBOUND(exp,f)
      equation 
        Print.print_buf("EQBOUND: ");
        Exp.print_exp(exp);
        Dump.print_select(f, " true", " false"); then ();
    case VALBOUND(v)
      equation 
        Print.print_buf("VALBOUND: ");
        Values.print_val(v); then ();
  end matchcontinue;
end print_binding;

public function print_binding_str
  input Binding in_binding;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_binding)
    local
      Ident str,str2,res,s;
      Exp.Exp exp;
      Boolean f;
      Values.Value v;
    case UNBOUND() then "UNBOUND"; 
    case EQBOUND(exp,f)
      equation 
        str = Exp.print_exp_str(exp);
        str2 = Dump.select_string(f, " true", " false");
        res = Util.string_append_list({"EQBOUND(",str,", ",str2,")"}); then res;
    case VALBOUND(v)
      equation 
        s = Values.unparse_values({v});
        res = Util.string_append_list({"VALBOUND(",s,")"}); then res;
  end matchcontinue;
end print_binding_str;

public function make_function_type " LS "
  input Absyn.Path p;
  input VarList vl;
  output Type out_type;
protected 
  VarList invl,outvl;
  FuncArgList fargs;
  Type rettype;
algorithm 
  invl := get_input_vars(vl);
  outvl := get_output_vars(vl);
  fargs := make_fargs_list(invl);
  rettype := make_return_type(outvl) "	& Debug.fprint (\"ft\", \" <fargs: \") & 
	Debug.fprint_list (\"ft\", fargs, print_farg, \", \") &
	Debug.fprint (\"ft\", \" >\") &

	Debug.fprint (\"ft\", \" <rettype: \") & 
	Debug.fcall (\"ft\", print_type, rettype) &
	Debug.fprint (\"ft\", \" >\")
";
  out_type := (T_FUNCTION(fargs, rettype), SOME(p));
end make_function_type;

public function make_enumeration_type
  input Absyn.Path in_path;
  input VarList in_varlist;
  output Type out_type;
algorithm 
  out_type:=
  matchcontinue (in_path,in_varlist)
    local
      type IdentList = list<Ident>;
      IdentList strs;
      Absyn.Path p;
      Ident name;
      VarList xs;
    case (p,VAR(name,_,_,_,_) :: xs)
      equation 
        ((T_ENUMERATION(strs,{}),_)) = make_enumeration_type(p, xs); then ((T_ENUMERATION(name :: strs,{}),SOME(p)));
    case (p,{}) then ((T_ENUMERATION({},{}),SOME(p))); 
  end matchcontinue;
end make_enumeration_type;

public function print_farg
  input FuncArg in_funcarg;
algorithm 
  _:=
  matchcontinue (in_funcarg)
    local
      Ident n;
      Type ty;
    case ((n,ty))
      equation 
        print_type(ty);
        Print.print_buf(" ");
        Print.print_buf(n); then ();
  end matchcontinue;
end print_farg;

protected function get_input_vars " LS "
  input VarList vl;
  output VarList vl_1;
protected 
  VarList vl_1;
algorithm 
  vl_1 := get_vars(vl, is_input_var);
end get_input_vars;

protected function get_output_vars " LS "
  input VarList vl;
  output VarList vl_1;
protected 
  VarList vl_1;
algorithm 
  vl_1 := get_vars(vl, is_output_var);
end get_output_vars;

public function get_classname
  input Type in_type;
  output Absyn.Path out_path;
algorithm 
  out_path:=
  matchcontinue (in_type)
    local Absyn.Path p;
    case ((_,SOME(p))) then p; 
  end matchcontinue;
end get_classname;

public function get_vars " LS "
  input VarList in_varlist;
  input FuncType1 in_functype1;
  output VarList out_varlist;
protected 
  function FuncType1
    input Var in_var;
  end FuncType1;
algorithm 
  out_varlist:=
  matchcontinue (in_varlist,in_functype1)
    local
      VarList vl_1,vl;
      Var v;
      FuncType1 cond;
    case ({},_) then {}; 
    case (v :: vl,cond)
      equation 
        cond(v);
        vl_1 = get_vars(vl, cond); then v :: vl_1;
    case (v :: vl,cond)
      equation 
        failure(cond(v));
        vl_1 = get_vars(vl, cond); then vl_1;
  end matchcontinue;
end get_vars;

protected function is_input_var " LS "
  input Var in_var;
algorithm 
  _:=
  matchcontinue (in_var)
    local
      Ident n;
      Attributes attr;
      Type ty;
      Binding bnd;
    case VAR(n,attr,false,ty,bnd) " LS: false means not protected, hence we ignore protected variables "
      equation 
        true = is_input_attr(attr); then ();
  end matchcontinue;
end is_input_var;

protected function is_output_var
  input Var in_var;
algorithm 
  _:=
  matchcontinue (in_var)
    local
      Ident n;
      Attributes attr;
      Type ty;
      Binding bnd;
    case VAR(n,attr,false,ty,bnd) " LS: false means not protected, hence we ignore protected variables "
      equation 
        true = is_output_attr(attr); then ();
  end matchcontinue;
end is_output_var;

public function is_input_attr
  input Attributes in_attributes;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_attributes)
    case ATTR(_,_,_,Absyn.INPUT()) then true; 
  end matchcontinue;
end is_input_attr;

public function is_output_attr
  input Attributes in_attributes;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_attributes)
    case ATTR(_,_,_,Absyn.OUTPUT()) then true; 
  end matchcontinue;
end is_output_attr;

public function is_bidir_attr
  input Attributes in_attributes;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_attributes)
    case ATTR(_,_,_,Absyn.BIDIR()) then true; 
  end matchcontinue;
end is_bidir_attr;

protected function make_fargs_list " LS "
  input VarList in_varlist;
  output FuncArgList out_funcarglist;
algorithm 
  out_funcarglist:=
  matchcontinue (in_varlist)
    local
      FuncArgList fargl;
      Ident n;
      Attributes attr;
      Boolean pr;
      Type ty;
      Binding bnd;
      VarList vl;
    case {} then {}; 
    case (VAR(n,attr,pr,ty,bnd) :: vl)
      equation 
        fargl = make_fargs_list(vl) "is_input_var v &"; then (n,ty) :: fargl;
    case (VAR(n,attr,pr,ty,bnd) :: vl)
      equation 
        Print.print_buf("#-- Not an input var:");
        Print.print_buf(n); then fail();
  end matchcontinue;
end make_fargs_list;

protected function make_return_type " LS "
  input VarList in_varlist;
  output Type out_type;
algorithm 
  out_type:=
  matchcontinue (in_varlist)
    local
      Type ty;
      Var var;
      TypeList tys;
      VarList vl;
    case {} then ((T_NOTYPE(),NONE)); 
    case {var}
      equation 
        ty = make_return_type_single(var); then ty;
    case vl
      equation 
        tys = make_return_type_tuple(vl); then ((T_TUPLE(tys),NONE));
  end matchcontinue;
end make_return_type;

protected function make_return_type_single " LS "
  input Var in_var;
  output Type out_type;
algorithm 
  out_type:=
  matchcontinue (in_var)
    local
      Ident n;
      Attributes attr;
      Boolean pr;
      Type ty;
      Binding bnd;
    case VAR(n,attr,pr,ty,bnd) then ty; 
  end matchcontinue;
end make_return_type_single;

protected function make_return_type_tuple " LS "
  input VarList in_varlist;
  output TypeList out_typelist;
algorithm 
  out_typelist:=
  matchcontinue (in_varlist)
    local
      TypeList tys;
      Ident n;
      Attributes attr;
      Boolean pr;
      Type ty;
      Binding bnd;
      VarList vl;
    case {} then {}; 
    case VAR(n,attr,pr,ty,bnd) :: vl
      equation 
        tys = make_return_type_tuple(vl); then ty :: tys;
  end matchcontinue;
end make_return_type_tuple;

public function is_parameter " LS "
  input Var in_var;
algorithm 
  _:=
  matchcontinue (in_var)
    local
      Ident n;
      Boolean fl;
      SCode.Accessibility ac;
      Absyn.Direction dir;
      Type ty;
      Binding bnd;
    case VAR(n,ATTR(fl,ac,SCode.PARAM(),dir),false,ty,bnd) then (); 
  end matchcontinue;
end is_parameter;

public function contain_real " Returns true if a buitlin type, or array-type is Real "
  input TypeList in_typelist;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_typelist)
    local
      Boolean r1,r2,res;
      ArrayDim d;
      Type tp;
      TypeList xs;
    case ((T_ARRAY(d,tp),_) :: xs)
      equation 
        r1 = contain_real({tp});
        r2 = contain_real(xs);
        res = bool_or(r1, r2); then res;
    case ((T_REAL(_),_) :: _) then true; 
    case (_ :: xs)
      equation 
        res = contain_real(xs); then res;
    case (_) then false; 
  end matchcontinue;
end contain_real;

public function flatten_array_type
  input Type in_type;
  output Type out_type;
  output IntegerList out_integerlist;
algorithm 
  (out_type,out_integerlist):=
  matchcontinue (in_type)
    local
      Type ty_1,ty;
      IntegerList dimlist_1,dimlist;
      Integer dim;
    case ((T_ARRAY(DIM(NONE),ty),_))
      equation 
        (ty_1,dimlist_1) = flatten_array_type(ty); then (ty_1,dimlist_1);
    case ((T_ARRAY(DIM(SOME(dim)),ty),_))
      equation 
        (ty_1,dimlist) = flatten_array_type(ty);
        dimlist_1 = list_append(dimlist, {dim}); then (ty_1,dimlist_1);
    case ty then (ty,{}); 
  end matchcontinue;
end flatten_array_type;

public function get_type_name
  input Type in_type;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_type)
    local
      type IdentList = list<Ident>;
      Ident n,dimstr,tystr,str;
      ClassInf.State st;
      Type ty,arrayty;
      IntegerList dims;
      IdentList dimstrs;
    case ((T_INTEGER(_),_)) then "Integer"; 
    case ((T_REAL(_),_)) then "Real"; 
    case ((T_STRING(_),_)) then "String"; 
    case ((T_BOOL(_),_)) then "Boolean"; 
    case ((T_COMPLEX(st,_),_))
      equation 
        n = ClassInf.get_state_name(st); then n;
    case ((arrayty = (T_ARRAY(_,_),_)))
      equation 
        (ty,dims) = flatten_array_type(arrayty);
        dimstrs = Util.list_map(dims, int_string);
        dimstr = Util.string_delimit_list(dimstrs, ", ");
        tystr = get_type_name(ty);
        str = Util.string_append_list({tystr,"[",dimstr,"]"}); then str;
    case ((_,_)) then "Not nameable type or no type"; 
  end matchcontinue;
end get_type_name;

public function is_prop_all_const "* LS: If PROP_TUPLE, returns true if all of the flags are true *"
  input Properties in_properties;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_properties)
    local
      Boolean constant_,res;
      Ident str;
      Properties prop;
    case PROP(_,constant_) then constant_; 
    case PROP_TUPLE(_,constant_)
      local Const constant_;
      equation 
        res = is_prop_tuple_all_const(constant_); then res;
    case prop
      equation 
        Debug.fprint("failtrace", "- is_prop_all_const failed: ");
        str = print_prop_str(prop);
        Debug.fprintln("failtrace", str); then fail();
  end matchcontinue;
end is_prop_all_const;

public function is_prop_any_const "* LS: If PROP_TUPLE, returns true if any of the flags are true *"
  input Properties in_properties;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_properties)
    local
      Boolean constant_,res;
      Ident str;
      Properties prop;
    case PROP(_,constant_) then constant_; 
    case PROP_TUPLE(_,constant_)
      local Const constant_;
      equation 
        res = is_prop_tuple_any_const(constant_); then res;
    case prop
      equation 
        Debug.fprint("failtrace", "- is_prop_any_const failed: ");
        str = print_prop_str(prop);
        Debug.fprintln("failtrace", str); then fail();
  end matchcontinue;
end is_prop_any_const;

protected function is_prop_tuple_any_const "* LS: *"
  input Const in_const;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_const)
    local
      Boolean f,res;
      Const first,const_;
      ConstList rest;
      Ident str;
    case CONST(f) then f; 
    case TUPLE_CONST(first :: rest)
      equation 
        true = is_prop_tuple_any_const(first); then true;
    case TUPLE_CONST(first :: {})
      equation 
        false = is_prop_tuple_any_const(first); then false;
    case TUPLE_CONST(first :: rest)
      equation 
        false = is_prop_tuple_any_const(first);
        res = is_prop_tuple_any_const(TUPLE_CONST(rest)); then res;
    case const_
      equation 
        Debug.fprint("failtrace", "- is_prop_tuple_any_const failed: ");
        str = unparse_const(const_);
        Debug.fprintln("failtrace", str); then fail();
  end matchcontinue;
end is_prop_tuple_any_const;

protected function is_prop_tuple_all_const "* LS: *"
  input Const in_const;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_const)
    local
      Boolean f,res;
      Const first,const_;
      ConstList rest;
      Ident str;
    case CONST(f) then f; 
    case TUPLE_CONST(first :: rest)
      equation 
        false = is_prop_tuple_all_const(first); then false;
    case TUPLE_CONST(first :: {})
      equation 
        true = is_prop_tuple_all_const(first); then true;
    case TUPLE_CONST(first :: rest)
      equation 
        true = is_prop_tuple_all_const(first);
        res = is_prop_tuple_all_const(TUPLE_CONST(rest)); then res;
    case const_
      equation 
        Debug.fprint("failtrace", "- is_prop_tuple_all_const failed: ");
        str = unparse_const(const_);
        Debug.fprintln("failtrace", str); then fail();
  end matchcontinue;
end is_prop_tuple_all_const;

public function is_prop_array
  input Properties p;
  output Boolean b;
protected 
  Type t;
algorithm 
  t := get_prop_type(p);
  b := is_type_array(t);
end is_prop_array;

public function get_prop_type "* LS: *"
  input Properties in_properties;
  output Type out_type;
algorithm 
  out_type:=
  matchcontinue (in_properties)
    local Type ty;
    case PROP(ty,_) then ty; 
    case PROP_TUPLE(ty,_) then ty; 
  end matchcontinue;
end get_prop_type;

protected function is_type_array
  input Type in_type;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_type)
    case ((T_ARRAY(_,_),_)) then true; 
    case ((_,_)) then false; 
  end matchcontinue;
end is_type_array;

public function elab_type
  input Type in_type;
  output Exp.Type out_type;
algorithm 
  out_type:=
  matchcontinue (in_type)
    local
      Type et,t;
      Exp.Type t_1;
    case ((T_INTEGER(_),_)) then Exp.INT(); 
    case ((T_REAL(_),_)) then Exp.REAL(); 
    case ((T_BOOL(_),_)) then Exp.BOOL(); 
    case ((T_STRING(_),_)) then Exp.STRING(); 
    case ((T_ENUM(),_)) then Exp.ENUM(); 
    case ((t = (T_ARRAY(_,_),_)))
      equation 
        et = array_element_type(t);
        t_1 = elab_type(et); then t_1;
    case ((_,_)) then Exp.OTHER(); 
  end matchcontinue;
end elab_type;

public function match_prop "* function: match_prop
 **
 ** This is basically a wrapper aroune `match_type'.  It matches an
 ** expression with properties with another set of properties.  If
 ** necessary, the expression is modified to match.  The only relevant
 ** property is the type.
 **
 *"
  input Exp.Exp in_exp1;
  input Properties in_properties2;
  input Properties in_properties3;
  output Exp.Exp out_exp;
algorithm 
  out_exp:=
  matchcontinue (in_exp1,in_properties2,in_properties3)
    local
      Exp.Exp e_1,e;
      Type gt,et;
    case (e,PROP(gt,_),PROP(et,_))
      equation 
        Debug.print("Debug: match prop.");
        (e_1,_) = match_type(e, gt, et); then e_1;
    case (e,PROP_TUPLE(gt,_),PROP_TUPLE(et,_))
      equation 
        Debug.print("\nDebug: match prop (PROP TUPLE). ");
        (e_1,_) = match_type(e, gt, et); then e_1;
  end matchcontinue;
end match_prop;

public function match_type "* function: match_type
 **
 ** This function matches an expression with an expected type, and
 ** converts the expression to the expected type if necessary.
 *"
  input Exp.Exp in_exp1;
  input Type in_type2;
  input Type in_type3;
  output Exp.Exp out_exp;
  output Type out_type;
algorithm 
  (out_exp,out_type):=
  matchcontinue (in_exp1,in_type2,in_type3)
    local
      Exp.Exp e,e_1;
      Type e_type,expected_type,e_type_1;
    case (e,e_type,expected_type)
      equation 
        true = subtype(e_type, expected_type); then (e,e_type);
    case (e,e_type,expected_type)
      equation 
        false = subtype(e_type, expected_type);
        (e_1,e_type_1) = type_convert(e, e_type, expected_type) "	Debug.fprint(\"sei\", \"trying type convert\n\") &"; then (e_1,e_type_1);
  end matchcontinue;
end match_type;

public function match_type_list "* function: match_type_list
 **
 ** This function matches a list of types, with a list of other types.
 ** Type conversion is disredaded, but an expression is given (the rhs of a tuple assignment)
 ** if such conversions should be implemented
 *"
  input Exp.Exp in_exp1;
  input TypeList in_typelist2;
  input TypeList in_typelist3;
  output Exp.Exp out_exp;
  output TypeList out_typelist;
algorithm 
  (out_exp,out_typelist):=
  matchcontinue (in_exp1,in_typelist2,in_typelist3)
    local
      Exp.Exp e,e_1,e_2;
      Type tp,t1,t2;
      TypeList res,ts1,ts2;
    case (e,{},{}) then (e,{}); 
    case (e,t1 :: ts1,t2 :: ts2)
      equation 
        (e_1,tp) = match_type(e, t1, t2);
        (e_2,res) = match_type_list(e_1, ts1, ts2); then (e_1,tp :: res);
  end matchcontinue;
end match_type_list;

public function type_convert "* function: type_convert
 **
 ** This functions converts the expression in the first argument to
 ** the type specified in the third argument.  The current type of the
 ** expression is given in the second argument.
 **
 ** If no type conversion is possible, this function fails.
 *"
  input Exp.Exp in_exp1;
  input Type in_type2;
  input Type in_type3;
  output Exp.Exp out_exp;
  output Type out_type;
algorithm 
  (out_exp,out_type):=
  matchcontinue (in_exp1,in_type2,in_type3)
    local
      type Exp_ExpList = list<Exp.Exp>;
      type TupleExp_ExpBoolean = tuple<Exp.Exp,Boolean>;
      type TupleExp_ExpBooleanList = list<TupleExp_ExpBoolean>;
      type TupleExp_ExpBooleanListList = list<TupleExp_ExpBooleanList>;
      type IdentList = list<Ident>;
      Exp_ExpList elist_1,elist;
      Exp.Type at,t;
      Boolean a;
      Integer dim1,dim2,nmax,dim11,dim22;
      Type ty1,ty2,t1,t2,t_1;
      Absyn_PathOption p,p1,p2;
      Exp.Exp begin_1,step_1,stop_1,begin,step,stop,e_1,e,exp;
      TupleExp_ExpBooleanListList ell_1,ell;
      TypeList tys_1,tys1,tys2;
      IdentList l;
      VarList v;
    case (Exp.ARRAY(_,_,elist),(T_ARRAY(DIM(SOME(dim1)),ty1),_),(T_ARRAY(DIM(SOME(dim2)),ty2),p))
      equation 
        (dim1 == dim2) = true;
        elist_1 = type_convert_array(elist, ty1, ty2);
        at = elab_type(ty2);
        a = is_array(ty2); then (Exp.ARRAY(at,a,elist_1),(T_ARRAY(DIM(SOME(dim1)),ty2),p));
    case (Exp.RANGE(t,begin,SOME(step),stop),(T_ARRAY(DIM(SOME(dim1)),ty1),_),(T_ARRAY(DIM(SOME(dim2)),ty2),p))
      equation 
        (dim1 == dim2) = true;
        (begin_1,_) = type_convert(begin, ty1, ty2);
        (step_1,_) = type_convert(step, ty1, ty2);
        (stop_1,_) = type_convert(stop, ty1, ty2);
        at = elab_type(ty2);
        a = is_array(ty2); then (Exp.RANGE(at,begin_1,SOME(step_1),stop_1),(T_ARRAY(DIM(SOME(dim1)),ty2),p));
    case (Exp.RANGE(t,begin,NONE,stop),(T_ARRAY(DIM(SOME(dim1)),ty1),_),(T_ARRAY(DIM(SOME(dim2)),ty2),p))
      equation 
        (dim1 == dim2) = true;
        (begin_1,_) = type_convert(begin, ty1, ty2);
        (stop_1,_) = type_convert(stop, ty1, ty2);
        at = elab_type(ty2);
        a = is_array(ty2); then (Exp.RANGE(at,begin_1,NONE,stop_1),(T_ARRAY(DIM(SOME(dim1)),ty2),p));
    case (Exp.MATRIX(_,nmax,ell),(T_ARRAY(DIM(SOME(dim1)),(T_ARRAY(DIM(SOME(dim11)),t1),_)),_),(T_ARRAY(DIM(SOME(dim2)),(T_ARRAY(DIM(SOME(dim22)),t2),p1)),p2))
      equation 
        (dim1 == dim2) = true;
        (dim11 == dim22) = true;
        ell_1 = type_convert_matrix(ell, t1, t2);
        at = elab_type(t2); then (Exp.MATRIX(at,nmax,ell_1),(T_ARRAY(DIM(SOME(dim1)),(T_ARRAY(DIM(SOME(dim11)),t2),p1)),
          p2));
    case (Exp.MATRIX(_,nmax,ell),(T_ARRAY(DIM(SOME(dim1)),(T_ARRAY(DIM(SOME(dim11)),t1),_)),_),(T_ARRAY(DIM(NONE),(T_ARRAY(DIM(SOME(dim22)),t2),p1)),p2))
      equation 
        (dim11 == dim22) = true;
        ell_1 = type_convert_matrix(ell, t1, t2);
        at = elab_type(t2); then (Exp.MATRIX(at,nmax,ell_1),(T_ARRAY(DIM(SOME(dim1)),(T_ARRAY(DIM(SOME(dim11)),t2),p1)),
          p2));
    case (Exp.ARRAY(_,_,elist),(T_ARRAY(DIM(NONE),ty1),_),(T_ARRAY(DIM(SOME(dim2)),ty2),p2))
      equation 
        (elist_1) = type_convert_array(elist, ty1, ty2) " This rule is used to ensure that casts are made on each   element, instead of on the whole array ";
        at = elab_type(ty2);
        a = is_array(ty2); then (Exp.ARRAY(at,a,elist_1),(T_ARRAY(DIM(NONE),ty2),p2));
    case (Exp.ARRAY(_,_,elist),(T_ARRAY(DIM(SOME(dim1)),ty1),_),(T_ARRAY(DIM(NONE),ty2),p2))
      equation 
        (elist_1) = type_convert_array(elist, ty1, ty2) " This rule is used to ensure that casts are made on each   element, instead of on the whole array ";
        at = elab_type(ty2);
        a = is_array(ty2); then (Exp.ARRAY(at,a,elist_1),(T_ARRAY(DIM(SOME(dim1)),ty2),p2));
    case (e,(T_ARRAY(DIM(NONE),ty1),_),(T_ARRAY(DIM(SOME(dim2)),ty2),p2))
      equation 
        (e_1,t_1) = type_convert(e, ty1, ty2); then (e_1,(T_ARRAY(DIM(NONE),t_1),p2));
    case (e,(T_ARRAY(DIM(NONE),ty1),_),(T_ARRAY(DIM(NONE),ty2),p2))
      equation 
        (e_1,t_1) = type_convert(e, ty1, ty2); then (e_1,(T_ARRAY(DIM(NONE),t_1),p2));
    case (e,(T_ARRAY(DIM(SOME(dim1)),ty1),_),(T_ARRAY(DIM(NONE),ty2),p2))
      equation 
        (e_1,t_1) = type_convert(e, ty1, ty2); then (e_1,(T_ARRAY(DIM(SOME(dim1)),t_1),p2));
    case (Exp.TUPLE(elist),(T_TUPLE(tys1),_),(T_TUPLE(tys2),p2))
      equation 
        (elist_1,tys_1) = type_convert_list(elist, tys1, tys2); then (Exp.TUPLE(elist_1),(T_TUPLE(tys_1),p2));
    case (exp,(T_ENUM(),_),(T_ENUMERATION(l,v),p2)) then (exp,(T_ENUMERATION(l,v),p2)); 
    case (e,(T_INTEGER(v),_),(T_REAL(_),p)) then (Exp.CAST(Exp.REAL(),e),(T_REAL(v),p)); 
  end matchcontinue;
end type_convert;

public function type_convert_array
  input Exp_ExpList in_exp_explist1;
  input Type in_type2;
  input Type in_type3;
  output Exp_ExpList out_exp_explist;
protected 
  type Exp_ExpList = list<Exp.Exp>;
algorithm 
  out_exp_explist:=
  matchcontinue (in_exp_explist1,in_type2,in_type3)
    local
      Exp_ExpList rest_1,rest;
      Exp.Exp first_1,first;
      Type ty1,ty2;
    case ({},_,_) then {}; 
    case (first :: rest,ty1,ty2) "  rule	Print.print_buf \"- type conversion of array failed exp=\" &
	Exp.print_exp e &
	Print.print_buf \"t1 = \" &
	print_type t1 & 
	Print.print_buf \" t2 = \" &
	print_type t2 &
	Print.print_buf \"\n\" 
	-------------------------------
	type_convert_array (e::_,t1,t2) => fail "
      equation 
        rest_1 = type_convert_array(rest, ty1, ty2);
        (first_1,_) = type_convert(first, ty1, ty2); then first_1 :: rest_1;
  end matchcontinue;
end type_convert_array;

protected function type_convert_matrix
  input TupleExp_ExpBooleanListList in_tupleexp_expbooleanlistlist1;
  input Type in_type2;
  input Type in_type3;
  output TupleExp_ExpBooleanListList out_tupleexp_expbooleanlistlist;
protected 
  type TupleExp_ExpBoolean = tuple<Exp.Exp,Boolean>;
  type TupleExp_ExpBooleanList = list<TupleExp_ExpBoolean>;
  type TupleExp_ExpBooleanListList = list<TupleExp_ExpBooleanList>;
algorithm 
  out_tupleexp_expbooleanlistlist:=
  matchcontinue (in_tupleexp_expbooleanlistlist1,in_type2,in_type3)
    local
      TupleExp_ExpBooleanListList rest_1,rest;
      TupleExp_ExpBooleanList first_1,first;
      Type ty1,ty2;
    case ({},_,_) then {}; 
    case (first :: rest,ty1,ty2) "  rule	Print.print_buf \"- type conversion of matrix failed\" &
	Print.print_buf \"t1 = \" &
	print_type t1 & 
	Print.print_buf \" t2 = \" &
	print_type t2 &
	Print.print_buf \"\n\" 
	-------------------------------
	type_convert_matrix (e::_,t1,t2) => fail "
      equation 
        rest_1 = type_convert_matrix(rest, ty1, ty2);
        first_1 = type_convert_matrix_row(first, ty1, ty2); then first_1 :: rest_1;
  end matchcontinue;
end type_convert_matrix;

protected function type_convert_matrix_row
  input TupleExp_ExpBooleanList in_tupleexp_expbooleanlist1;
  input Type in_type2;
  input Type in_type3;
  output TupleExp_ExpBooleanList out_tupleexp_expbooleanlist;
protected 
  type TupleExp_ExpBoolean = tuple<Exp.Exp,Boolean>;
  type TupleExp_ExpBooleanList = list<TupleExp_ExpBoolean>;
algorithm 
  out_tupleexp_expbooleanlist:=
  matchcontinue (in_tupleexp_expbooleanlist1,in_type2,in_type3)
    local
      TupleExp_ExpBooleanList rest;
      Exp.Exp exp_1,exp;
      Type newt,t1,t2;
      Boolean a;
    case ({},_,_) then {}; 
    case ((exp,_) :: rest,t1,t2)
      equation 
        rest = type_convert_matrix_row(rest, t1, t2);
        (exp_1,newt) = type_convert(exp, t1, t2);
        a = is_array(t2); then (exp_1,a) :: rest;
  end matchcontinue;
end type_convert_matrix_row;

protected function type_convert_list
  input Exp_ExpList in_exp_explist1;
  input TypeList in_typelist2;
  input TypeList in_typelist3;
  output Exp_ExpList out_exp_explist;
  output TypeList out_typelist;
protected 
  type Exp_ExpList = list<Exp.Exp>;
algorithm 
  (out_exp_explist,out_typelist):=
  matchcontinue (in_exp_explist1,in_typelist2,in_typelist3)
    local
      Exp_ExpList rest_1,rest;
      TypeList tyrest_1,ty1rest,ty2rest;
      Exp.Exp first_1,first;
      Type ty_1,ty1,ty2;
    case ({},_,_) then ({},{}); 
    case (first :: rest,ty1 :: ty1rest,ty2 :: ty2rest)
      equation 
        (rest_1,tyrest_1) = type_convert_list(rest, ty1rest, ty2rest);
        (first_1,ty_1) = type_convert(first, ty1, ty2); then (first_1 :: rest_1,ty_1 :: tyrest_1);
  end matchcontinue;
end type_convert_list;

public function match_with_promote " Allow Integer => Real "
  input Properties in_properties1;
  input Properties in_properties2;
  input Boolean in_boolean3;
  output Properties out_properties;
algorithm 
  out_properties:=
  matchcontinue (in_properties1,in_properties2,in_boolean3)
    local
      Type t,t1,t2;
      Boolean c,c1,c2,havereal;
      ArrayDim dim,dim1,dim2;
      Absyn_PathOption p2,p;
      VarList v;
    case (PROP((T_ARRAY(dim1,t1),_),c1),PROP((T_ARRAY(dim2,t2),p2),c2),havereal) " Allow Integer => Real "
      equation 
        PROP(t,c) = match_with_promote(PROP(t1,c1), PROP(t2,c2), havereal);
        dim = dim1; then PROP((T_ARRAY(dim,t),p2),c);
    case (PROP(t1,c1),PROP((T_ARRAY(DIM(SOME(1)),t2),p2),c2),havereal)
      equation 
        false = is_array(t1);
        PROP(t,c) = match_with_promote(PROP(t1,c1), PROP(t2,c2), havereal); then PROP((T_ARRAY(DIM(SOME(1)),t),p2),c);
    case (PROP((T_ARRAY(DIM(SOME(1)),t1),p),c1),PROP(t2,c2),havereal)
      equation 
        false = is_array(t2);
        PROP(t,c) = match_with_promote(PROP(t1,c1), PROP(t2,c2), havereal); then PROP((T_ARRAY(DIM(SOME(1)),t),p),c);
    case (PROP(t1,c1),PROP(t2,c2),false)
      equation 
        false = is_array(t1);
        false = is_array(t2);
        equality(t1 = t2);
        t = t1;
        c = bool_and(c1, c2); then PROP(t,c);
    case (PROP((T_REAL(v),_),c1),PROP((T_REAL(_),p2),c2),true)
      equation 
        c = bool_and(c1, c2) " Have real and both Real "; then PROP((T_REAL(v),p2),c);
    case (PROP((T_INTEGER(_),_),c1),PROP((T_REAL(v),p2),c2),true)
      equation 
        c = bool_and(c1, c2) " Have real and first Integer "; then PROP((T_REAL(v),p2),c);
    case (PROP((T_REAL(v),_),c1),PROP((T_INTEGER(_),p2),c2),true)
      equation 
        c = bool_and(c1, c2) " Have real and second Integer "; then PROP((T_REAL(v),p2),c);
    case (PROP((T_INTEGER(_),_),c1),PROP((T_INTEGER(_),p2),c2),true)
      equation 
        c = bool_and(c1, c2) " Have real and both Integer "; then PROP((T_REAL({}),p2),c);
  end matchcontinue;
end match_with_promote;

public function print_prop_str
  input Properties in_properties;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_properties)
    local
      Ident ty_str,const_str,res;
      Type ty;
      Boolean const_;
    case PROP(ty,const_)
      equation 
        ty_str = unparse_type(ty);
        const_str = Util.if_(const_, "const", "non_const");
        res = Util.string_append_list({"PROP(",ty_str,const_str,")"}); then res;
    case PROP_TUPLE(ty,const_)
      local Const const_;
      equation 
        ty_str = unparse_type(ty);
        const_str = unparse_const(const_);
        res = Util.string_append_list({"PROP_TUPLE(",ty_str,", ",const_str,")"}); then res;
  end matchcontinue;
end print_prop_str;

public function print_prop
  input Properties p;
protected 
  Ident str;
algorithm 
  str := print_prop_str(p);
  Print.print_buf(str);
end print_prop;

public function flow_variables "* function: flow_variables
 **
 ** This function retrieves all variables names that are flow variables, and 
 ** prepends the prefix given as an 'Exp.ComponentRef'
*"
  input VarList in_varlist;
  input Exp.ComponentRef in_componentref;
  output Exp_ComponentRefList out_exp_componentreflist;
protected 
  type Exp_ComponentRefList = list<Exp.ComponentRef>;
algorithm 
  out_exp_componentreflist:=
  matchcontinue (in_varlist,in_componentref)
    local
      Exp.ComponentRef cr_1,cr;
      Exp_ComponentRefList res;
      Ident id;
      VarList vs;
    case ({},_) then {}; 
    case (VAR(id,ATTR(true,_,_,_),_,_,_) :: vs,cr)
      equation 
        cr_1 = Exp.join_crefs(cr, Exp.CREF_IDENT(id,{}));
        res = flow_variables(vs, cr); then cr_1 :: res;
    case (_ :: vs,cr)
      equation 
        res = flow_variables(vs, cr); then res;
  end matchcontinue;
end flow_variables;

public function get_all_exps "* function: get_all_exps
 ** 
 ** This function goes through the Type structure and finds all the
 ** expressions and returns them in a list
 *"
  input Type in_type;
  output Exp_ExpList out_exp_explist;
protected 
  type Exp_ExpList = list<Exp.Exp>;
algorithm 
  out_exp_explist:=
  matchcontinue (in_type)
    local
      Exp_ExpList exps;
      TType ttype;
      Absyn_PathOption pathopt;
    case ((ttype,pathopt))
      equation 
        exps = get_all_exps_tt(ttype); then exps;
  end matchcontinue;
end get_all_exps;

protected function get_all_exps_tt "* function: get_all_exps_tt
 ** 
 ** This function goes through the TType structure and finds all the
 ** expressions and returns them in a list
 *"
  input TType in_ttype;
  output Exp_ExpList out_exp_explist;
protected 
  type Exp_ExpList = list<Exp.Exp>;
algorithm 
  out_exp_explist:=
  matchcontinue (in_ttype)
    local
      type IdentList = list<Ident>;
      type Exp_ExpListList = list<Exp_ExpList>;
      Exp_ExpList exps,tyexps;
      VarList vars;
      IdentList strs;
      ArrayDim dim;
      Type ty;
      ClassInf.State cinf;
      TypeList tys;
      Exp_ExpListList explists,explist;
      FuncArgList fargs;
    case T_INTEGER(vars)
      equation 
        exps = get_all_exps_vars(vars); then exps;
    case T_REAL(vars)
      equation 
        exps = get_all_exps_vars(vars); then exps;
    case T_STRING(vars)
      equation 
        exps = get_all_exps_vars(vars); then exps;
    case T_BOOL(vars)
      equation 
        exps = get_all_exps_vars(vars); then exps;
    case T_ENUMERATION(strs,vars)
      equation 
        exps = get_all_exps_vars(vars); then exps;
    case T_ARRAY(dim,ty)
      equation 
        exps = get_all_exps(ty); then exps;
    case T_COMPLEX(cinf,vars)
      equation 
        exps = get_all_exps_vars(vars); then exps;
    case T_FUNCTION(fargs,ty)
      equation 
        tys = Util.list_map(fargs, get_funcarg_type);
        explists = Util.list_map(tys, get_all_exps);
        tyexps = get_all_exps(ty);
        exps = Util.list_flatten(tyexps :: explists); then exps;
    case T_TUPLE(tys)
      equation 
        explist = Util.list_map(tys, get_all_exps);
        exps = Util.list_flatten(explist); then exps;
    case _
      equation 
        Debug.fprintln("failtrace", "-- get_all_exps_tt failed"); then fail();
  end matchcontinue;
end get_all_exps_tt;

protected function get_all_exps_vars
  input VarList vars;
  output Exp_ExpList exps;
protected 
  type Exp_ExpList = list<Exp.Exp>;
  type Exp_ExpListList = list<Exp_ExpList>;
  Exp_ExpListList explist;
algorithm 
  explist := Util.list_map(vars, get_all_exps_var);
  exps := Util.list_flatten(explist);
end get_all_exps_vars;

protected function get_all_exps_var
  input Var in_var;
  output Exp_ExpList out_exp_explist;
protected 
  type Exp_ExpList = list<Exp.Exp>;
algorithm 
  out_exp_explist:=
  matchcontinue (in_var)
    local
      Exp_ExpList tyexps,bndexp,exps;
      Ident id;
      Attributes attr;
      Boolean prot;
      Type ty;
      Binding bnd;
    case VAR(id,attr,prot,ty,bnd)
      equation 
        tyexps = get_all_exps(ty);
        bndexp = get_all_exps_binding(bnd);
        exps = list_append(tyexps, bndexp); then exps;
  end matchcontinue;
end get_all_exps_var;

protected function get_all_exps_binding
  input Binding in_binding;
  output Exp_ExpList out_exp_explist;
protected 
  type Exp_ExpList = list<Exp.Exp>;
algorithm 
  out_exp_explist:=
  matchcontinue (in_binding)
    local
      Exp.Exp exp;
      Boolean cnst;
      Values.Value v;
    case EQBOUND(exp,cnst) then {exp}; 
    case UNBOUND() then {}; 
    case VALBOUND(v) then {}; 
    case _
      equation 
        Debug.fprintln("failtrace", "-- get_all_exps_binding failed"); then fail();
  end matchcontinue;
end get_all_exps_binding;

protected function get_funcarg_type
  input FuncArg in_funcarg;
  output Type out_type;
algorithm 
  out_type:=
  matchcontinue (in_funcarg)
    local
      Ident id;
      Type ty;
    case ((id,ty)) then ty; 
  end matchcontinue;
end get_funcarg_type;
end Types;

