package Values "
    Copyright (c) PELAB, Dept. Computer and Information Science, Linkoping University,
    Sweden, in the following denoted PELAB/LIU.

   
    This file is part of OpenModelica, in the following denoted OSM.


    The OSM License, Version 1.1, March 4, 2005.


    1 Preface

    The aim of this license is to lay down the conditions enabling you to use, 
    modify and circulate OSM. However, PELAB/LIU remain the authors of 
    OSM and so retain property rights and the use of all ancillary rights.

    2 Definitions

    OSM is defined as all successive versions of the OSM software and their 
    documentation that have been developed by PELAB/LIU and including
    accepted contributions from other contributors according to this license.

    OSM DERIVED SOFTWARE is defined as all or part of OSM that you have 
    modified and/or translated and/or adapted.


    3 Dual License

    OSM is made available under the OSM licensing scheme, which is a dual
    licensing scheme with two options, a) and b):

    a) OSM OPEN SOURCE LICENSE:

    If you wish to write Open Source software you can use the Open Source version 
    of OSM, released under the OSM license which include GPL as its open source 
    licensing option. If you use the OSM Open Source version you must release your 
    Application using OSM including this Application's source code under the GPL as well.

    This OSM license text, and Copyright (c) PELAB/Linkoping University, must
    be present in your copy of OSM and in OSM DERIVED SOFTWARE.

    You should have received a copy of the GPL - GNU General Public License
    along with OpenModelica; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.


    b) OSM COMMERCIAL LICENSE

    If you are using OSM commercially - that is, for commercial usage or
    for creating proprietary software for sale or use in a commercial setting
    - you must purchase a commercial license of OSM from PELAB/L
  *
 ** file:        Values.rml
 ** module:      Values
 ** description: Evaluated expression values
 **
 ** RCS: $Id: Values.rml 1733 2005-04-27 06:47:45Z haklu $
 ** 
 ** The module `Values' contains data structures for representing
 ** constant Modelica values.  These include integer, real, string and
 ** boolean values, and also arrays of any dimensionality and type.
 ** Multidimensional arrays are represented as arrays of arrays.
 **
 ** The code is excluded from the report, since they convey no
 ** semantic information.
 *"

public import OpenModelica.Compiler.Exp;

public import OpenModelica.Compiler.Absyn;

public 
type ValueList = list<Value>;

type Exp_IdentList = list<Exp.Ident>;

uniontype Value
  record INTEGER
    Integer x1;
  end INTEGER;
  record REAL
    Real x1;
  end REAL;
  record STRING
    String x1;
  end STRING;
  record BOOL
    Boolean x1;
  end BOOL;
  record ENUM
    String x1;
  end ENUM;
  record ARRAY
    ValueList x1;
  end ARRAY;
  record TUPLE
    ValueList x1;
  end TUPLE;
  record RECORD
    Absyn.Path x1;
    ValueList x2 " record name ";
    Exp_IdentList x3 " orderd set of values   comp names for each value";
  end RECORD;
  record CODE
    Absyn.Code x1 " A record consist of value * Ident pairs ";
  end CODE;
end Value;

public function type_convert
  input Exp.Type in_type1;
  input Exp.Type in_type2;
  input ValueList in_valuelist3;
  output ValueList out_valuelist;
algorithm 
  out_valuelist:=
  matchcontinue (in_type1,in_type2,in_valuelist3)
    local
      ValueList vallst,vrest,vallst2,vals;
      Real rval,r;
      Exp.Type from,to;
      Integer i,ival;
    case (_,_,{}) then {}; 
    case ((from = Exp.INT()),(to = Exp.REAL()),INTEGER(i) :: vrest)
      equation 
        vallst = type_convert(from, to, vrest);
        rval = int_real(i); then REAL(rval) :: vallst;
    case ((from = Exp.REAL()),(to = Exp.INT()),REAL(r) :: vrest)
      equation 
        vallst = type_convert(from, to, vrest);
        ival = real_int(r); then INTEGER(ival) :: vallst;
    case (from,to,ARRAY(vals) :: vrest)
      equation 
        vallst = type_convert(from, to, vals);
        vallst2 = type_convert(from, to, vrest); then ARRAY(vallst) :: vallst2;
  end matchcontinue;
end type_convert;

public function is_array
  input Value in_value;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_value)
    case (INTEGER(_)) then false; 
    case (REAL(_)) then false; 
    case (STRING(_)) then false; 
    case (BOOL(_)) then false; 
    case (TUPLE(_)) then false; 
    case (ARRAY(_)) then true; 
  end matchcontinue;
end is_array;

public function unparse_values
  input ValueList in_valuelist;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_valuelist)
    local
      String s1,s2,s3,str;
      Value v;
      ValueList vallst;
    case (v :: vallst)
      equation 
        s1 = unparse_description({v});
        s2 = unparse_value_numbers({v});
        s3 = unparse_values(vallst);
        str = Util.string_append_list({s1,s2,"\n",s3}); then str;
    case ({}) then ""; 
  end matchcontinue;
end unparse_values;

protected function unparse_value_numbers
  input ValueList in_valuelist;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_valuelist)
    local
      String s1,s2,res,istr,sval;
      ValueList lst,xs;
      Integer i;
    case (TUPLE(lst) :: xs)
      equation 
        s1 = unparse_value_numbers(lst);
        s2 = unparse_value_numbers(xs);
        res = string_append(s1, s2); then res;
    case (ARRAY(lst) :: xs)
      equation 
        s1 = unparse_value_numbers(lst);
        s2 = unparse_value_numbers(xs);
        res = string_append(s1, s2); then res;
    case (INTEGER(i) :: xs)
      equation 
        s1 = unparse_value_numbers(xs);
        istr = int_string(i);
        s2 = string_append(istr, " ");
        res = string_append(s2, s1); then res;
    case (REAL(i) :: xs)
      local Real i;
      equation 
        s1 = unparse_value_numbers(xs);
        istr = real_string(i);
        s2 = string_append(istr, " ");
        res = string_append(s2, s1); then res;
    case (STRING(sval) :: xs)
      equation 
        s1 = unparse_value_numbers(xs);
        s2 = string_append(sval, " ");
        res = string_append(s2, s1); then res;
    case ({}) then ""; 
  end matchcontinue;
end unparse_value_numbers;

protected function unparse_description
  input ValueList in_valuelist;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_valuelist)
    local
      String s1,str,slenstr,sval,s2,s4;
      ValueList xs,vallst;
      Integer slen;
    case (INTEGER(_) :: xs)
      equation 
        s1 = unparse_description(xs);
        str = string_append("# i!\n", s1); then str;
    case (REAL(_) :: xs)
      equation 
        s1 = unparse_description(xs);
        str = string_append("# r!\n", s1); then str;
    case (STRING(sval) :: xs)
      equation 
        s1 = unparse_description(xs);
        slen = string_length(sval);
        slenstr = int_string(slen);
        str = Util.string_append_list({"# s[ 1 ",slenstr,"\n"}); then str;
    case (ARRAY(vallst) :: xs)
      equation 
        s1 = unparse_description(xs);
        s2 = unparse_array_description(vallst);
        s4 = string_append(s2, s1);
        str = string_append(s4, " \n"); then str;
    case ({}) then ""; 
  end matchcontinue;
end unparse_description;

protected function unparse_array_description
  input ValueList lst;
  output String str;
protected 
  String pt,s1,s2,s3,s4,s5,s6;
  Integer i1;
algorithm 
  pt := unparse_prim_type(lst);
  s1 := string_append("# ", pt);
  s2 := string_append(s1, "[");
  i1 := unparse_num_dims(lst);
  s3 := int_string(i1);
  s4 := string_append(s2, s3);
  s5 := string_append(s4, " ");
  s6 := unparse_dim_sizes(lst);
  str := string_append(s5, s6);
end unparse_array_description;

protected function unparse_prim_type
  input ValueList in_valuelist;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_valuelist)
    local
      String res;
      ValueList elts;
    case (ARRAY(elts) :: _)
      equation 
        res = unparse_prim_type(elts); then res;
    case (INTEGER(_) :: _) then "i"; 
    case (REAL(_) :: _) then "r"; 
    case (STRING(_) :: _) then "s"; 
    case (BOOL(_) :: _) then "b"; 
    case (_) then "error"; 
  end matchcontinue;
end unparse_prim_type;

protected function unparse_num_dims
  input ValueList in_valuelist;
  output Integer out_integer;
algorithm 
  out_integer:=
  matchcontinue (in_valuelist)
    local
      Integer i1;
      ValueList vals;
    case (ARRAY(vals) :: _)
      equation 
        i1 = unparse_num_dims(vals); then i1 + 1;
    case (_) then 1; 
  end matchcontinue;
end unparse_num_dims;

protected function unparse_dim_sizes
  input ValueList in_valuelist;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_valuelist)
    local
      Integer i1,len;
      String s1,s2,s3,res;
      ValueList lst,vals;
    case ((lst = ARRAY(vals) :: _))
      equation 
        i1 = list_length(lst);
        s1 = int_string(i1);
        s2 = string_append(s1, " ");
        s3 = unparse_dim_sizes(vals);
        res = string_append(s2, s3); then res;
    case (lst)
      equation 
        len = list_length(lst);
        res = int_string(len); then res;
  end matchcontinue;
end unparse_dim_sizes;

public function write_to_file_as_args
  input ValueList vallst;
  input String filename;
protected 
  String str;
algorithm 
  str := unparse_values(vallst);
  System.write_file(filename, str);
end write_to_file_as_args;

public function add_elementwise_arrayelt
  input ValueList in_valuelist1;
  input ValueList in_valuelist2;
  output ValueList out_valuelist;
algorithm 
  out_valuelist:=
  matchcontinue (in_valuelist1,in_valuelist2)
    local
      ValueList reslst,res2,v1lst,rest1,v2lst,rest2;
      Integer res,v1,v2;
    case (ARRAY(v1lst) :: rest1,ARRAY(v2lst) :: rest2)
      equation 
        reslst = add_elementwise_arrayelt(v1lst, v2lst);
        res2 = add_elementwise_arrayelt(rest1, rest2); then ARRAY(reslst) :: res2;
    case (INTEGER(v1) :: rest1,INTEGER(v2) :: rest2)
      equation 
        res = v1 + v2;
        res2 = add_elementwise_arrayelt(rest1, rest2); then INTEGER(res) :: res2;
    case (REAL(v1) :: rest1,REAL(v2) :: rest2)
      local Real res,v1,v2;
      equation 
        res = v1 +. v2;
        res2 = add_elementwise_arrayelt(rest1, rest2); then REAL(res) :: res2;
    case (STRING(v1) :: rest1,STRING(v2) :: rest2)
      local String res,v1,v2;
      equation 
        res = string_append(v1, v2);
        res2 = add_elementwise_arrayelt(rest1, rest2) " Addition of strings is string concatenation "; then STRING(res) :: res2;
    case ({},{}) then {}; 
  end matchcontinue;
end add_elementwise_arrayelt;

public function sub_elementwise_arrayelt
  input ValueList in_valuelist1;
  input ValueList in_valuelist2;
  output ValueList out_valuelist;
algorithm 
  out_valuelist:=
  matchcontinue (in_valuelist1,in_valuelist2)
    local
      ValueList reslst,res2,v1lst,rest1,v2lst,rest2;
      Integer res,v1,v2;
    case (ARRAY(v1lst) :: rest1,ARRAY(v2lst) :: rest2)
      equation 
        reslst = sub_elementwise_arrayelt(v1lst, v2lst);
        res2 = sub_elementwise_arrayelt(rest1, rest2); then ARRAY(reslst) :: res2;
    case (INTEGER(v1) :: rest1,INTEGER(v2) :: rest2)
      equation 
        res = v1 - v2;
        res2 = sub_elementwise_arrayelt(rest1, rest2); then INTEGER(res) :: res2;
    case (REAL(v1) :: rest1,REAL(v2) :: rest2)
      local Real res,v1,v2;
      equation 
        res = v1 -. v2;
        res2 = sub_elementwise_arrayelt(rest1, rest2); then REAL(res) :: res2;
    case ({},{}) then {}; 
  end matchcontinue;
end sub_elementwise_arrayelt;

public function sum_arrayelt
  input ValueList in_valuelist;
  output Value out_value;
algorithm 
  out_value:=
  matchcontinue (in_valuelist)
    local
      Integer v1,v3,v2;
      ValueList xs,arr;
    case ({INTEGER(v1)}) then INTEGER(v1); 
    case ({REAL(v1)})
      local Real v1; then REAL(v1);
    case ({ARRAY(v1)})
      local ValueList v1; then ARRAY(v1);
    case (INTEGER(v2) :: xs)
      equation 
        INTEGER(v1) = sum_arrayelt(xs);
        v3 = v1 + v2; then INTEGER(v3);
    case (REAL(v2) :: xs)
      local Real v1,v3,v2;
      equation 
        REAL(v1) = sum_arrayelt(xs);
        v3 = v1 +. v2; then REAL(v3);
    case ((arr = ARRAY(v2) :: _))
      local ValueList v1,v3,v2;
      equation 
        ARRAY(v1) = sum_arrayelt(arr);
        v3 = add_elementwise_arrayelt(v1, v2); then ARRAY(v3);
  end matchcontinue;
end sum_arrayelt;

public function mult_scalar_arrayelt
  input Value in_value;
  input ValueList in_valuelist;
  output ValueList out_valuelist;
algorithm 
  out_valuelist:=
  matchcontinue (in_value,in_valuelist)
    local
      ValueList r1,r2,vals,rest;
      Value sval;
      Integer v1,v2;
      Real v2_1,v1_1;
    case (sval,ARRAY(vals) :: rest)
      equation 
        r1 = mult_scalar_arrayelt(sval, vals);
        r2 = mult_scalar_arrayelt(sval, rest); then ARRAY(r1) :: r2;
    case ((sval = INTEGER(v1)),INTEGER(v2) :: rest)
      local Integer r1;
      equation 
        r1 = v1*v2;
        r2 = mult_scalar_arrayelt(sval, rest); then INTEGER(r1) :: r2;
    case ((sval = REAL(v1)),INTEGER(v2) :: rest)
      local Real r1,v1;
      equation 
        v2_1 = int_real(v2);
        r1 = v1*.v2_1;
        r2 = mult_scalar_arrayelt(sval, rest); then REAL(r1) :: r2;
    case ((sval = INTEGER(v1)),REAL(v2) :: rest)
      local Real r1,v2;
      equation 
        v1_1 = int_real(v1);
        r1 = v1_1*.v2;
        r2 = mult_scalar_arrayelt(sval, rest); then REAL(r1) :: r2;
    case ((sval = REAL(v1)),REAL(v2) :: rest)
      local Real r1,v1,v2;
      equation 
        r1 = v1*.v2;
        r2 = mult_scalar_arrayelt(sval, rest); then REAL(r1) :: r2;
    case (_,{}) then {}; 
  end matchcontinue;
end mult_scalar_arrayelt;

public function mult_scalar_product
  input ValueList in_valuelist1;
  input ValueList in_valuelist2;
  output Value out_value;
algorithm 
  out_value:=
  matchcontinue (in_valuelist1,in_valuelist2)
    local
      Integer r1,r2,res,v1,v2,len,len2;
      ValueList v1lst,v2lst,vres,rest,vlst,col,mat_1,vals,mat,lst1,lst2;
      Value sres,v;
      String lenstr,len2str;
    case (INTEGER(v1) :: (v1lst = _ :: _),INTEGER(v2) :: (v2lst = _ :: _))
      equation 
        r1 = v1*v2;
        INTEGER(r2) = mult_scalar_product(v1lst, v2lst);
        res = r1 + r2; then INTEGER(res);
    case ({INTEGER(v1)},{INTEGER(v2)})
      equation 
        res = v1*v2; then INTEGER(res);
    case (REAL(v1) :: (v1lst = _ :: _),REAL(v2) :: (v2lst = _ :: _))
      local Real r1,r2,res,v1,v2;
      equation 
        r1 = v1*.v2;
        REAL(r2) = mult_scalar_product(v1lst, v2lst);
        res = r1 +. r2; then REAL(res);
    case ({REAL(v1)},{REAL(v2)})
      local Real res,v1,v2;
      equation 
        res = v1*.v2; then REAL(res);
    case (ARRAY(v2lst) :: rest,(vlst = INTEGER(_) :: _))
      equation 
        sres = mult_scalar_product(v2lst, vlst);
        ARRAY(vres) = mult_scalar_product(rest, vlst); then ARRAY(sres :: vres);
    case ({},INTEGER(_) :: _) then ARRAY({}); 
    case (ARRAY(v2lst) :: rest,(vlst = REAL(_) :: _))
      equation 
        sres = mult_scalar_product(v2lst, vlst);
        ARRAY(vres) = mult_scalar_product(rest, vlst); then ARRAY(sres :: vres);
    case ({},REAL(_) :: _) then ARRAY({}); 
    case ((vlst = INTEGER(_) :: _),(mat = ARRAY(_ :: _ :: _) :: _))
      equation 
        (ARRAY(col),mat_1) = matrix_strip_first_column(mat);
        v = mult_scalar_product(vlst, col);
        ARRAY(vals) = mult_scalar_product(vlst, mat_1); then ARRAY(v :: vals);
    case ((vlst = INTEGER(_) :: _),(mat = ARRAY({_}) :: _))
      local Integer v;
      equation 
        (ARRAY(col),mat_1) = matrix_strip_first_column(mat);
        INTEGER(v) = mult_scalar_product(vlst, col); then ARRAY({INTEGER(v)});
    case ((vlst = REAL(_) :: _),(mat = ARRAY(_ :: _ :: _) :: _))
      equation 
        (ARRAY(col),mat_1) = matrix_strip_first_column(mat);
        v = mult_scalar_product(vlst, col);
        ARRAY(vals) = mult_scalar_product(vlst, mat_1); then ARRAY(v :: vals);
    case ((vlst = REAL(_) :: _),(mat = ARRAY({_}) :: _))
      local Real v;
      equation 
        (ARRAY(col),mat_1) = matrix_strip_first_column(mat);
        REAL(v) = mult_scalar_product(vlst, col); then ARRAY({REAL(v)});
    case (lst1,lst2)
      equation 
        Print.print_buf("mult_scalar_product failed\n lst1 len:");
        len = list_length(lst1);
        lenstr = int_string(len);
        Print.print_buf(lenstr);
        Print.print_buf("lst2 len:");
        len2 = list_length(lst2);
        len2str = int_string(len2);
        Print.print_buf(len2str);
        Print.print_buf("\n"); then fail();
  end matchcontinue;
end mult_scalar_product;

public function mult_matrix
  input ValueList in_valuelist1;
  input ValueList in_valuelist2;
  output ValueList out_valuelist;
algorithm 
  out_valuelist:=
  matchcontinue (in_valuelist1,in_valuelist2)
    local
      Value res1;
      ValueList res2,m1,v1lst,rest1,m2;
    case ((m1 = ARRAY(v1lst) :: rest1),(m2 = ARRAY(_) :: _))
      equation 
        res1 = mult_scalar_product(v1lst, m2);
        res2 = mult_matrix(rest1, m2); then res1 :: res2;
    case ({},_) then {}; 
  end matchcontinue;
end mult_matrix;

public function div_arrayelt_scalar
  input Value in_value;
  input ValueList in_valuelist;
  output ValueList out_valuelist;
algorithm 
  out_valuelist:=
  matchcontinue (in_value,in_valuelist)
    local
      Value sval;
      Real v1,v2_1,v1_1;
      Integer v2;
      ValueList r1,r2,vals,rest;
    case ((sval = REAL(v1)),_)
      equation 
        equality(v1 = 0.0);
        Print.print_error_buf("# Error, division by zero.\n"); then fail();
    case ((sval = INTEGER(v1)),_)
      local Integer v1;
      equation 
        equality(v1 = 0);
        Print.print_error_buf("# Error, division by zero.\n"); then fail();
    case (sval,ARRAY(vals) :: rest)
      equation 
        r1 = div_arrayelt_scalar(sval, vals);
        r2 = div_arrayelt_scalar(sval, rest); then ARRAY(r1) :: r2;
    case ((sval = INTEGER(v1)),INTEGER(v2) :: rest)
      local Integer r1,v1;
      equation 
        r1 = v2/v1;
        r2 = div_arrayelt_scalar(sval, rest); then INTEGER(r1) :: r2;
    case ((sval = REAL(v1)),INTEGER(v2) :: rest)
      local Real r1;
      equation 
        v2_1 = int_real(v2);
        r1 = v2_1/.v1;
        r2 = div_arrayelt_scalar(sval, rest); then REAL(r1) :: r2;
    case ((sval = INTEGER(v1)),REAL(v2) :: rest)
      local
        Real r1,v2;
        Integer v1;
      equation 
        v1_1 = int_real(v1);
        r1 = v2/.v1_1;
        r2 = div_arrayelt_scalar(sval, rest); then REAL(r1) :: r2;
    case ((sval = REAL(v1)),REAL(v2) :: rest)
      local Real r1,v2;
      equation 
        r1 = v2/.v1;
        r2 = div_arrayelt_scalar(sval, rest); then REAL(r1) :: r2;
    case (_,{}) then {}; 
  end matchcontinue;
end div_arrayelt_scalar;

protected function matrix_strip_first_column "* This function takes a Value list representing a matrix and strips the first column of the matrix, i.e. for each sub list it removes the first element. Returning both the stripped column and the resulting matrix.*"
  input ValueList in_valuelist;
  output Value out_value;
  output ValueList out_valuelist;
algorithm 
  (out_value,out_valuelist):=
  matchcontinue (in_valuelist)
    local
      ValueList resl,resl2,vrest,rest;
      Value v1;
    case (ARRAY(v1 :: vrest) :: rest)
      equation 
        (ARRAY(resl),resl2) = matrix_strip_first_column(rest); then (ARRAY(v1 :: resl), ARRAY(vrest) :: resl2);
    case ({}) then (ARRAY({}),{}); 
  end matchcontinue;
end matrix_strip_first_column;

public function intlist_to_value
  input IntegerList in_integerlist;
  output Value out_value;
protected 
  type IntegerList = list<Integer>;
algorithm 
  out_value:=
  matchcontinue (in_integerlist)
    local
      ValueList res;
      Integer i;
      IntegerList lst;
    case ({}) then ARRAY({}); 
    case (i :: lst)
      equation 
        ARRAY(res) = intlist_to_value(lst); then ARRAY(INTEGER(i) :: res);
  end matchcontinue;
end intlist_to_value;

protected import OpenModelica.Compiler.Print;

protected import OpenModelica.Compiler.System;

protected import OpenModelica.Compiler.Util;

protected import OpenModelica.Compiler.Dump;

public function val_string "!ignorecode
  * function: val_string
 **
 ** This function returns a textual representation of a value.
 *"
  input Value in_value;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_value)
    local
      String s,s_1,s_2,res,res_1;
      Integer n;
      Real x;
      ValueList vs;
      Value r;
      Absyn.Code c;
    case INTEGER(n)
      equation 
        s = int_string(n); then s;
    case REAL(x)
      equation 
        s = real_string(x); then s;
    case STRING(s)
      equation 
        s_1 = Util.string_append_list({"\"",s,"\""}); then s_1;
    case BOOL(false) then "false"; 
    case BOOL(true) then "true"; 
    case ARRAY(vs)
      equation 
        s = val_list_string(vs);
        s_1 = string_append("{", s);
        s_2 = string_append(s_1, "}"); then s_2;
    case TUPLE(vs)
      equation 
        s = val_list_string(vs);
        s_1 = string_append("(", s);
        s_2 = string_append(s_1, ")"); then s_2;
    case ((r = RECORD(_,_,_)))
      equation 
        s = val_record_string(r);
        res = Util.string_append_list({"record\n",s,"end record"}); then res;
    case (CODE(c))
      equation 
        res = Dump.print_code_str(c);
        res_1 = Util.string_append_list({"Code(",res,")"}); then res_1;
    case _
      equation 
        Print.print_buf("- val_string failed\n"); then fail();
  end matchcontinue;
end val_string;

protected function val_record_string "* function: val_record_string
 **
 ** This function returns a textual representation of a record,
 separating each value with a comma.
 *"
  input Value in_value;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_value)
    local
      type StringList = list<String>;
      Absyn.Path cname;
      String s1,s2,res,id;
      Value x;
      ValueList xs;
      StringList ids;
    case (RECORD(cname,{},{})) then ""; 
    case (RECORD(cname,x :: (xs = _ :: _),id :: (ids = _ :: _)))
      equation 
        s1 = val_string(x);
        s2 = val_record_string(RECORD(cname,xs,ids));
        res = Util.string_append_list({id," = ",s1,",\n",s2}); then res;
    case (RECORD(cname,x :: xs,id :: ids))
      equation 
        s1 = val_string(x);
        s2 = val_record_string(RECORD(cname,xs,ids));
        res = Util.string_append_list({"    ",id," = ",s1,"\n",s2}); then res;
  end matchcontinue;
end val_record_string;

protected function val_list_string "* function: val_list_string
 **
 ** This function returns a textual representation of a list of
 ** values, separating each value with a comman.
 *"
  input ValueList in_valuelist;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_valuelist)
    local
      String s,s_1,s_2,s_3;
      Value v;
      ValueList vs;
    case {} then ""; 
    case {v}
      equation 
        s = val_string(v); then s;
    case v :: vs
      equation 
        s = val_string(v);
        s_1 = val_list_string(vs);
        s_2 = string_append(s, ",");
        s_3 = string_append(s_2, s_1); then s_3;
    case _
      equation 
        Print.print_buf("- val_list_string failed\n"); then fail();
  end matchcontinue;
end val_list_string;

public function write_ptolemyplot_dataset
  input String in_string1;
  input Value in_value2;
  input StringList in_stringlist3;
  input String in_string4;
  output Integer out_integer;
protected 
  type StringList = list<String>;
algorithm 
  out_integer:=
  matchcontinue (in_string1,in_value2,in_stringlist3,in_string4)
    local
      String datasets,str,filename,timevar,message;
      Value time;
      ValueList rest;
      StringList varnames;
    case (filename,ARRAY(time :: rest),timevar :: varnames,message)
      equation 
        datasets = unparse_ptolemy_values(time, rest, varnames);
        str = Util.string_append_list(
          {"#Ptolemy Plot generated by OpenModelica\n","TitleText: ",
          message,"\n",datasets});
        System.write_file(filename, str); then 0;
  end matchcontinue;
end write_ptolemyplot_dataset;

protected function unparse_ptolemy_values
  input Value in_value;
  input ValueList in_valuelist;
  input StringList in_stringlist;
  output String out_string;
protected 
  type StringList = list<String>;
algorithm 
  out_string:=
  matchcontinue (in_value,in_valuelist,in_stringlist)
    local
      String str,str2,res,v1;
      Value time,s1;
      ValueList xs;
      StringList vs;
    case (_,{},_) then ""; 
    case (time,s1 :: xs,v1 :: vs)
      equation 
        str = unparse_ptolemy_set(time, s1, v1);
        str2 = unparse_ptolemy_values(time, xs, vs);
        res = string_append(str, str2); then res;
  end matchcontinue;
end unparse_ptolemy_values;

protected function unparse_ptolemy_set
  input Value v1;
  input Value v2;
  input String varname;
  output String res;
protected 
  String str;
algorithm 
  str := unparse_ptolemy_set_2(v1, v2);
  res := Util.string_append_list({"DataSet: ",varname,"\n",str});
end unparse_ptolemy_set;

protected function unparse_ptolemy_set_2
  input Value in_value1;
  input Value in_value2;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_value1,in_value2)
    local
      String s1,s2,res,res_1;
      Value v1,v2;
      ValueList v1s,v2s;
    case (ARRAY({}),ARRAY({})) then ""; 
    case (ARRAY(v1 :: v1s),ARRAY(v2 :: v2s))
      equation 
        s1 = val_string(v1);
        s2 = val_string(v2);
        res = unparse_ptolemy_set_2(ARRAY(v1s), ARRAY(v2s));
        res_1 = Util.string_append_list({s1,",",s2,"\n",res}); then res_1;
  end matchcontinue;
end unparse_ptolemy_set_2;

public function reverse_matrix " Reverses each line and each row of a matrix.
 Implementation reverses all dimensions..."
  input Value in_value;
  output Value out_value;
algorithm 
  out_value:=
  matchcontinue (in_value)
    local
      ValueList lst_1,lst_2,lst;
      Value value;
    case (ARRAY(lst))
      equation 
        lst_1 = Util.list_map(lst, reverse_matrix);
        lst_2 = list_reverse(lst_1); then ARRAY(lst_2);
    case (value) then value; 
  end matchcontinue;
end reverse_matrix;

public function print_val "* function: print_val
 **
 ** This function prints a value.
 *"
  input Value v;
protected 
  String s;
algorithm 
  s := val_string(v);
  Print.print_buf(s);
end print_val;
end Values;

