package VarTransform "
    Copyright (c) PELAB, Dept. Computer and Information Science, Linkoping University,
    Sweden, in the following denoted PELAB/LIU.

   
    This file is part of OpenModelica, in the following denoted OSM.


    The OSM License, Version 1.1, March 4, 2005.


    1 Preface

    The aim of this license is to lay down the conditions enabling you to use, 
    modify and circulate OSM. However, PELAB/LIU remain the authors of 
    OSM and so retain property rights and the use of all ancillary rights.

    2 Definitions

    OSM is defined as all successive versions of the OSM software and their 
    documentation that have been developed by PELAB/LIU and including
    accepted contributions from other contributors according to this license.

    OSM DERIVED SOFTWARE is defined as all or part of OSM that you have 
    modified and/or translated and/or adapted.


    3 Dual License

    OSM is made available under the OSM licensing scheme, which is a dual
    licensing scheme with two options, a) and b):

    a) OSM OPEN SOURCE LICENSE:

    If you wish to write Open Source software you can use the Open Source version 
    of OSM, released under the OSM license which include GPL as its open source 
    licensing option. If you use the OSM Open Source version you must release your 
    Application using OSM including this Application's source code under the GPL as well.

    This OSM license text, and Copyright (c) PELAB/Linkoping University, must
    be present in your copy of OSM and in OSM DERIVED SOFTWARE.

    You should have received a copy of the GPL - GNU General Public License
    along with OpenModelica; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.


    b) OSM COMMERCIAL LICENSE

    If you are using OSM commercially - that is, for commercial usage or
    for creating proprietary software for sale or use in a commercial setting
    - you must purchase a commercial license of OSM from PELAB/L
  *
 ** file:	 VarTransform.rml
 ** module:      VarTransform
 ** description: VarTransform contains a Binary Tree representation of variable replacements.
 **
 ** RCS: $Id: VarTransform.rml 1788 2005-05-17 13:17:05Z haklu $
 ** 
 *
  * 
 ** This module contain a Binary tree representation of variable replacements
 ** along with some functions for performing replacements of variables in equations
*"

public import OpenModelica.Compiler.Exp;

public import OpenModelica.Compiler.DAELow;

public 
uniontype VariableReplacements
  record REPLACEMENTS
    BinTree x1;
    BinTree2 x2 " src -> dst, used for replacing   dst->src, used for lookup to make 
					  rules transitive. A destination can 
					  have several sources, therefore 
					  another BinTree ";
  end REPLACEMENTS;
end VariableReplacements;

public 
type TreeValueOption = Option<TreeValue>;

type BinTreeOption = Option<BinTree>;

uniontype BinTree
  record TREENODE
    TreeValueOption x1;
    BinTreeOption x2 " Value ";
    BinTreeOption x3 " left subtree   right subtree ";
  end TREENODE;
end BinTree;

public 
type TreeValue2Option = Option<TreeValue2>;

type BinTree2Option = Option<BinTree2>;

uniontype BinTree2
  record TREENODE2
    TreeValue2Option x1;
    BinTree2Option x2 " Value ";
    BinTree2Option x3 " left subtree   right subtree ";
  end TREENODE2;
end BinTree2;

public 
uniontype TreeValue " Each node in the binary tree can have a value associated with it."
  record TREEVALUE
    Key x1 " Key ";
    Value x2 " Value ";
  end TREEVALUE;
end TreeValue;

public 
uniontype TreeValue2
  record TREEVALUE2
    Key x1 " Key ";
    Value2 x2 " Value ";
  end TREEVALUE2;
end TreeValue2;

public 
type Key = Exp.ComponentRef " Key ";

public 
type Value = Exp.ComponentRef;

public 
type Value2 = list<Exp.ComponentRef>;

protected import OpenModelica.Compiler.System;

protected import OpenModelica.Compiler.Util;

protected import OpenModelica.Compiler.Algorithm;

public function apply_replacements
  input VariableReplacements in_variablereplacements1;
  input Exp.ComponentRef in_componentref2;
  input Exp.ComponentRef in_componentref3;
  output Exp.ComponentRef out_componentref1;
  output Exp.ComponentRef out_componentref2;
algorithm 
  (out_componentref1,out_componentref2):=
  matchcontinue (in_variablereplacements1,in_componentref2,in_componentref3)
    local
      Exp.ComponentRef cr1_1,cr2_1,cr1,cr2;
      VariableReplacements repl;
    case (repl,cr1,cr2)
      equation 
        (Exp.CREF(cr1_1,_)) = replace_exp(Exp.CREF(cr1,Exp.REAL()), repl);
        (Exp.CREF(cr2_1,_)) = replace_exp(Exp.CREF(cr2,Exp.REAL()), repl); then (cr1_1,cr2_1);
  end matchcontinue;
end apply_replacements;

public function empty_replacements "* function: empty_replacements
 ** Returns an empty set of replacement rules
 *"
  output VariableReplacements out_variablereplacements;
algorithm 
  out_variablereplacements:=
  matchcontinue ()
    case () then REPLACEMENTS(TREENODE(NONE,NONE,NONE),TREENODE2(NONE,NONE,NONE)); 
  end matchcontinue;
end empty_replacements;

public function replace_equations
  input DAELow_EquationList in_daelow_equationlist;
  input VariableReplacements in_variablereplacements;
  output DAELow_EquationList out_daelow_equationlist;
protected 
  type DAELow_EquationList = list<DAELow.Equation>;
algorithm 
  out_daelow_equationlist:=
  matchcontinue (in_daelow_equationlist,in_variablereplacements)
    local
      Exp.Exp e1_1,e2_1,e1,e2,e_1,e;
      DAELow_EquationList es_1,es;
      VariableReplacements repl;
      DAELow.Equation a;
      Exp.ComponentRef cr;
    case ({},_) then {}; 
    case (DAELow.EQUATION(e1,e2) :: es,repl)
      equation 
        e1_1 = replace_exp(e1, repl);
        e2_1 = replace_exp(e2, repl);
        es_1 = replace_equations(es, repl); then DAELow.EQUATION(e1_1,e2_1) :: es_1;
    case ((a = DAELow.ALGORITHM(_,_,_)) :: es,repl)
      equation 
        es_1 = replace_equations(es, repl); then a :: es_1;
    case (DAELow.SOLVED_EQUATION(cr,e) :: es,repl)
      equation 
        e_1 = replace_exp(e, repl);
        es_1 = replace_equations(es, repl); then DAELow.SOLVED_EQUATION(cr,e_1) :: es_1;
    case (a :: es,repl)
      equation 
        es_1 = replace_equations(es, repl); then a :: es_1;
  end matchcontinue;
end replace_equations;

protected function replace_equations_stmts
  input Algorithm_StatementList in_algorithm_statementlist;
  input VariableReplacements in_variablereplacements;
  output Algorithm_StatementList out_algorithm_statementlist;
protected 
  type Algorithm_StatementList = list<Algorithm.Statement>;
algorithm 
  out_algorithm_statementlist:=
  matchcontinue (in_algorithm_statementlist,in_variablereplacements)
    local
      Exp.Exp e_1,e;
      Exp.ComponentRef cr_1,cr;
      Algorithm_StatementList xs_1,xs;
      Exp.Type tp;
      VariableReplacements repl;
      Algorithm.Statement x;
    case ({},_) then {}; 
    case (Algorithm.ASSIGN(tp,cr,e) :: xs,repl)
      equation 
        e_1 = replace_exp(e, repl);
        Exp.CREF(cr_1,_) = replace_exp(Exp.CREF(cr,Exp.OTHER()), repl);
        xs_1 = replace_equations_stmts(xs, repl); then Algorithm.ASSIGN(tp,cr_1,e_1) :: xs_1;
    case (x :: xs,repl)
      equation 
        print("Warning, not implemented in replace_equations_stmts\n");
        xs_1 = replace_equations_stmts(xs, repl); then x :: xs_1;
  end matchcontinue;
end replace_equations_stmts;

public function dump_replacements "* function: dump_replacements 
 ** Prints the variable replacements on form var1 -> var2
 *"
  input VariableReplacements in_variablereplacements;
algorithm 
  _:=
  matchcontinue (in_variablereplacements)
    local
      type Exp_ExpList = list<Exp.Exp>;
      type StringList = list<String>;
      Exp_ExpList srcs,dsts;
      StringList srcstrs,dststrs,dststrs_1,strs;
      String str,len_str;
      Integer len;
      BinTree bt;
    case (REPLACEMENTS(bt,_))
      equation 
        (srcs,dsts) = bintree_to_explist(bt);
        srcstrs = Util.list_map(srcs, Exp.print_exp_str);
        dststrs = Util.list_map(dsts, Exp.print_exp_str);
        dststrs_1 = Util.list_map_1(dststrs, string_append, "\n");
        strs = Util.list_thread(srcstrs, dststrs_1);
        str = Util.string_delimit_list(strs, " -> ");
        print("Replacements: (");
        len = list_length(srcs);
        len_str = int_string(len);
        print(len_str);
        print(")\n");
        print("=============\n");
        print(str);
        print("\n"); then ();
  end matchcontinue;
end dump_replacements;

public function add_replacement " source 
   dest "
  input VariableReplacements in_variablereplacements1;
  input Exp.ComponentRef in_componentref2;
  input Exp.ComponentRef in_componentref3;
  output VariableReplacements out_variablereplacements;
algorithm 
  out_variablereplacements:=
  matchcontinue (in_variablereplacements1,in_componentref2,in_componentref3)
    local
      Exp.ComponentRef olddst,src,dst,src_1,dst_1;
      VariableReplacements repl;
      BinTree bt,bt_1;
      BinTree2 invbt,invbt_1;
    case ((repl = REPLACEMENTS(bt,invbt)),src,dst) " source   dest "
      equation 
        olddst = tree_get(bt, src) "* if rule a->b exists, fail
	 *"; then fail();
    case ((repl = REPLACEMENTS(bt,invbt)),src,dst)
      equation 
        (REPLACEMENTS(bt,invbt),src_1,dst_1) = make_transitive(repl, src, dst);
        bt_1 = tree_add(bt, src_1, dst_1);
        invbt_1 = add_replacement_inv(invbt, src_1, dst_1); then REPLACEMENTS(bt_1,invbt_1);
    case (_,_,_)
      equation 
        print("-add_replacement failed\n"); then fail();
  end matchcontinue;
end add_replacement;

protected function add_replacement_inv
  input BinTree2 in_bintree21;
  input Exp.ComponentRef in_componentref2;
  input Exp.ComponentRef in_componentref3;
  output BinTree2 out_bintree2;
algorithm 
  out_bintree2:=
  matchcontinue (in_bintree21,in_componentref2,in_componentref3)
    local
      type ValueList = list<Exp.ComponentRef>;
      BinTree2 invbt_1,invbt;
      Exp.ComponentRef src,dst;
      ValueList srcs;
    case (invbt,src,dst)
      equation 
        failure(_ = tree_get2(invbt, dst)) " No previous elt for dst -> src ";
        invbt_1 = tree_add2(invbt, dst, {src}); then invbt_1;
    case (invbt,src,dst)
      equation 
        srcs = tree_get2(invbt, dst) " previous elt for dst -> src, append.. ";
        invbt_1 = tree_add2(invbt, dst, src :: srcs); then invbt_1;
  end matchcontinue;
end add_replacement_inv;

protected function make_transitive " replacement rules 
  src
  dst
   updated replacement rules 
   src"
  input VariableReplacements in_variablereplacements1;
  input Exp.ComponentRef in_componentref2;
  input Exp.ComponentRef in_componentref3;
  output VariableReplacements out_variablereplacements1;
  output Exp.ComponentRef out_componentref2;
  output Exp.ComponentRef out_componentref3;
algorithm 
  (out_variablereplacements1,out_componentref2,out_componentref3):=
  matchcontinue (in_variablereplacements1,in_componentref2,in_componentref3)
    local
      VariableReplacements repl_1,repl_2,repl;
      Exp.ComponentRef src_1,dst_1,src_2,dst_2,src,dst;
    case (repl,src,dst) " replacement rules  src dst  updated replacement rules   src  updated dst "
      equation 
        (repl_1,src_1,dst_1) = make_transitive1(repl, src, dst);
        (repl_2,src_2,dst_2) = make_transitive2(repl_1, src_1, dst_1); then (repl_2,src_2,dst_2);
  end matchcontinue;
end make_transitive;

protected function make_transitive1 " replacement rules 
  src
  dst
   updated replacement rules 
   src"
  input VariableReplacements in_variablereplacements1;
  input Exp.ComponentRef in_componentref2;
  input Exp.ComponentRef in_componentref3;
  output VariableReplacements out_variablereplacements1;
  output Exp.ComponentRef out_componentref2;
  output Exp.ComponentRef out_componentref3;
algorithm 
  (out_variablereplacements1,out_componentref2,out_componentref3):=
  matchcontinue (in_variablereplacements1,in_componentref2,in_componentref3)
    local
      type ValueList = list<Exp.ComponentRef>;
      ValueList lst;
      VariableReplacements repl_1,repl;
      BinTree bt;
      BinTree2 invbt;
      Exp.ComponentRef src,dst;
    case ((repl = REPLACEMENTS(bt,invbt)),src,dst) " replacement rules  src dst  updated replacement rules   src  updated dst "
      equation 
        lst = tree_get2(invbt, src) " old rule a->b must be updated to a->c when new rule b->c introduced";
        repl_1 = add_replacements(repl, lst, dst); then (repl_1,src,dst);
    case (repl,src,dst) then (repl,src,dst); 
  end matchcontinue;
end make_transitive1;

protected function make_transitive2 " replacement rules 
  src
  dst
   updated replacement rules 
   src"
  input VariableReplacements in_variablereplacements1;
  input Exp.ComponentRef in_componentref2;
  input Exp.ComponentRef in_componentref3;
  output VariableReplacements out_variablereplacements1;
  output Exp.ComponentRef out_componentref2;
  output Exp.ComponentRef out_componentref3;
algorithm 
  (out_variablereplacements1,out_componentref2,out_componentref3):=
  matchcontinue (in_variablereplacements1,in_componentref2,in_componentref3)
    local
      Exp.ComponentRef newdst,src_1,dst_1,src,dst;
      VariableReplacements repl_1,repl;
      BinTree bt;
      BinTree2 invbt;
    case ((repl = REPLACEMENTS(bt,invbt)),src,dst) " replacement rules  src dst  updated replacement rules   src  updated dst "
      equation 
        newdst = tree_get(bt, dst) " If dst has own replacement rule, change to its destintation and recurse ";
        (repl_1,src_1,dst_1) = make_transitive2(repl, src, newdst); then (repl_1,src_1,dst_1);
    case (repl,src,dst) then (repl,src,dst); 
  end matchcontinue;
end make_transitive2;

protected function add_replacements
  input VariableReplacements in_variablereplacements;
  input Exp_ComponentRefList in_exp_componentreflist;
  input Exp.ComponentRef in_componentref;
  output VariableReplacements out_variablereplacements;
protected 
  type Exp_ComponentRefList = list<Exp.ComponentRef>;
algorithm 
  out_variablereplacements:=
  matchcontinue (in_variablereplacements,in_exp_componentreflist,in_componentref)
    local
      type ValueList = list<Exp.ComponentRef>;
      VariableReplacements repl,repl_1,repl_2;
      Exp.ComponentRef src,dst;
      ValueList srcs;
    case (repl,{},_) then repl; 
    case (repl,src :: srcs,dst)
      equation 
        repl_1 = add_replacement(repl, src, dst);
        repl_2 = add_replacements(repl_1, srcs, dst); then repl_2;
    case (_,_,_)
      equation 
        print("add_replacements failed\n"); then fail();
  end matchcontinue;
end add_replacements;

public function get_replacement "* function: get_replacement
 ** Retrives a replacement variable given a set of replacement rules and a source
 ** variable.
 *"
  input VariableReplacements in_variablereplacements;
  input Exp.ComponentRef in_componentref;
  output Exp.ComponentRef out_componentref;
algorithm 
  out_componentref:=
  matchcontinue (in_variablereplacements,in_componentref)
    local
      Exp.ComponentRef dst,src;
      BinTree bt;
      BinTree2 invbt;
    case (REPLACEMENTS(bt,invbt),src)
      equation 
        dst = tree_get(bt, src); then dst;
  end matchcontinue;
end get_replacement;

public function replace_exp "* function: replace_exp
 ** Takes a set of replacement rules and an expression
 ** and replaces all variables in the expression using 
 ** the replacement rules.
 *"
  input Exp.Exp in_exp;
  input VariableReplacements in_variablereplacements;
  output Exp.Exp out_exp;
algorithm 
  out_exp:=
  matchcontinue (in_exp,in_variablereplacements)
    local
      type Exp_ExpList = list<Exp.Exp>;
      type TupleExp_ExpBoolean = tuple<Exp.Exp,Boolean>;
      type TupleExp_ExpBooleanList = list<TupleExp_ExpBoolean>;
      type TupleExp_ExpBooleanListList = list<TupleExp_ExpBooleanList>;
      Exp.ComponentRef cr_1,cr;
      Exp.Type t,tp,a;
      VariableReplacements repl;
      Exp.Exp e1_1,e2_1,e1,e2,e3_1,e3,e_1,r_1,e,r;
      Exp.Operator op;
      Exp_ExpList expl_1,expl;
      Absyn.Path path,p;
      Boolean c;
      Integer b,i;
      String id;
    case (Exp.CREF(cr,t),repl)
      equation 
        cr_1 = get_replacement(repl, cr); then Exp.CREF(cr_1,t);
    case (Exp.BINARY(e1,op,e2),repl)
      equation 
        e1_1 = replace_exp(e1, repl);
        e2_1 = replace_exp(e2, repl); then Exp.BINARY(e1_1,op,e2_1);
    case (Exp.LBINARY(e1,op,e2),repl)
      equation 
        e1_1 = replace_exp(e1, repl);
        e2_1 = replace_exp(e2, repl); then Exp.LBINARY(e1_1,op,e2_1);
    case (Exp.UNARY(op,e1),repl)
      equation 
        e1_1 = replace_exp(e1, repl); then Exp.UNARY(op,e1_1);
    case (Exp.LUNARY(op,e1),repl)
      equation 
        e1_1 = replace_exp(e1, repl); then Exp.LUNARY(op,e1_1);
    case (Exp.RELATION(e1,op,e2),repl)
      equation 
        e1_1 = replace_exp(e1, repl);
        e2_1 = replace_exp(e2, repl); then Exp.RELATION(e1_1,op,e2_1);
    case (Exp.IFEXP(e1,e2,e3),repl)
      equation 
        e1_1 = replace_exp(e1, repl);
        e2_1 = replace_exp(e2, repl);
        e3_1 = replace_exp(e3, repl); then Exp.IFEXP(e1_1,e2_1,e3_1);
    case (Exp.CALL(path,expl,t,c),repl)
      local Boolean t;
      equation 
        expl_1 = Util.list_map_1(expl, replace_exp, repl); then Exp.CALL(path,expl_1,t,c);
    case (Exp.ARRAY(tp,c,expl),repl)
      equation 
        expl_1 = Util.list_map_1(expl, replace_exp, repl); then Exp.ARRAY(tp,c,expl_1);
    case (Exp.MATRIX(a,b,c),repl)
      local TupleExp_ExpBooleanListList c;
      equation 
        print("replace_exp for matrix not impl."); then Exp.MATRIX(a,b,c);
    case (Exp.RANGE(tp,e1,NONE,e2),repl)
      equation 
        e1_1 = replace_exp(e1, repl);
        e2_1 = replace_exp(e2, repl); then Exp.RANGE(tp,e1_1,NONE,e2_1);
    case (Exp.RANGE(tp,e1,SOME(e3),e2),repl)
      equation 
        e1_1 = replace_exp(e1, repl);
        e2_1 = replace_exp(e2, repl);
        e3_1 = replace_exp(e3, repl); then Exp.RANGE(tp,e1_1,SOME(e3_1),e2_1);
    case (Exp.TUPLE(expl),repl)
      equation 
        expl_1 = Util.list_map_1(expl, replace_exp, repl); then Exp.TUPLE(expl_1);
    case (Exp.CAST(tp,e1),repl)
      equation 
        e1_1 = replace_exp(e1, repl); then Exp.CAST(tp,e1_1);
    case (Exp.ASUB(e1,i),repl)
      equation 
        e1_1 = replace_exp(e1, repl); then Exp.ASUB(e1_1,i);
    case (Exp.SIZE(e1,NONE),repl)
      equation 
        e1_1 = replace_exp(e1, repl); then Exp.SIZE(e1_1,NONE);
    case (Exp.SIZE(e1,SOME(e2)),repl)
      equation 
        e1_1 = replace_exp(e1, repl);
        e2_1 = replace_exp(e2, repl); then Exp.SIZE(e1_1,SOME(e2_1));
    case (Exp.CODE(a,b),repl)
      local
        Absyn.Code a;
        Exp.Type b;
      equation 
        print("replace_exp on CODE not impl.\n"); then Exp.CODE(a,b);
    case (Exp.REDUCTION(p,e,id,r),repl)
      equation 
        e_1 = replace_exp(e, repl);
        r_1 = replace_exp(r, repl); then Exp.REDUCTION(p,e_1,id,r_1);
    case (e,repl) then e; 
  end matchcontinue;
end replace_exp;

protected function bintree_to_explist "* function: bintree_to_list
 ** This function takes a BinTree and transform it into a list
 ** representation, i.e. two lists of keys and values
 *"
  input BinTree in_bintree;
  output Exp_ExpList out_exp_explist1;
  output Exp_ExpList out_exp_explist2;
protected 
  type Exp_ExpList = list<Exp.Exp>;
algorithm 
  (out_exp_explist1,out_exp_explist2):=
  matchcontinue (in_bintree)
    local
      Exp_ExpList klst,vlst;
      BinTree bt;
    case (bt)
      equation 
        (klst,vlst) = bintree_to_explist2(bt, {}, {}); then (klst,vlst);
  end matchcontinue;
end bintree_to_explist;

protected function bintree_to_explist2
  input BinTree in_bintree1;
  input Exp_ExpList in_exp_explist2;
  input Exp_ExpList in_exp_explist3;
  output Exp_ExpList out_exp_explist1;
  output Exp_ExpList out_exp_explist2;
protected 
  type Exp_ExpList = list<Exp.Exp>;
algorithm 
  (out_exp_explist1,out_exp_explist2):=
  matchcontinue (in_bintree1,in_exp_explist2,in_exp_explist3)
    local
      Exp_ExpList klst,vlst;
      Exp.ComponentRef key,value;
      BinTreeOption left,right;
    case (TREENODE(NONE,NONE,NONE),klst,vlst) then (klst,vlst); 
    case (TREENODE(SOME(TREEVALUE(key,value)),left,right),klst,vlst)
      equation 
        (klst,vlst) = bintree_to_explist_opt(left, klst, vlst);
        (klst,vlst) = bintree_to_explist_opt(right, klst, vlst); then (Exp.CREF(key,Exp.REAL()) :: klst,Exp.CREF(value,Exp.REAL()) :: vlst);
    case (TREENODE(NONE,left,right),klst,vlst)
      equation 
        (klst,vlst) = bintree_to_explist_opt(left, klst, vlst);
        (klst,vlst) = bintree_to_explist_opt(left, klst, vlst); then (klst,vlst);
  end matchcontinue;
end bintree_to_explist2;

protected function bintree_to_explist_opt
  input BinTreeOption in_bintreeoption1;
  input Exp_ExpList in_exp_explist2;
  input Exp_ExpList in_exp_explist3;
  output Exp_ExpList out_exp_explist1;
  output Exp_ExpList out_exp_explist2;
protected 
  type Exp_ExpList = list<Exp.Exp>;
algorithm 
  (out_exp_explist1,out_exp_explist2):=
  matchcontinue (in_bintreeoption1,in_exp_explist2,in_exp_explist3)
    local
      Exp_ExpList klst,vlst;
      BinTree bt;
    case (NONE,klst,vlst) then (klst,vlst); 
    case (SOME(bt),klst,vlst)
      equation 
        (klst,vlst) = bintree_to_explist2(bt, klst, vlst); then (klst,vlst);
  end matchcontinue;
end bintree_to_explist_opt;

protected function tree_get "* function: tree_get
 ** Copied from generic implementation. Changed that no hashrelation is passed
 ** since a string can not be uniquely mapped to an int. Therefore we need to compare two strings
 ** to get a unique ordering.
*"
  input BinTree in_bintree;
  input Key in_key;
  output Value out_value;
algorithm 
  out_value:=
  matchcontinue (in_bintree,in_key)
    local
      String rkeystr,keystr;
      Exp.ComponentRef rkey,rval,key,res;
      BinTreeOption left,right;
      Integer cmpval;
    case (TREENODE(SOME(TREEVALUE(rkey,rval)),left,right),key)
      equation 
        rkeystr = Exp.print_component_ref_str(rkey);
        keystr = Exp.print_component_ref_str(key);
        0 = System.strcmp(rkeystr, keystr); then rval;
    case (TREENODE(SOME(TREEVALUE(rkey,rval)),left,SOME(right)),key)
      local BinTree right;
      equation 
        keystr = Exp.print_component_ref_str(key) " Search to the right";
        rkeystr = Exp.print_component_ref_str(rkey);
        cmpval = System.strcmp(rkeystr, keystr);
        (cmpval > 0) = true;
        res = tree_get(right, key); then res;
    case (TREENODE(SOME(TREEVALUE(rkey,rval)),SOME(left),right),key)
      local BinTree left;
      equation 
        keystr = Exp.print_component_ref_str(key) " Search to the left";
        rkeystr = Exp.print_component_ref_str(rkey);
        cmpval = System.strcmp(rkeystr, keystr);
        (cmpval > 0) = false;
        res = tree_get(left, key); then res;
  end matchcontinue;
end tree_get;

protected function tree_add "* function: tree_add
 ** Copied from generic implementation. Changed that no hashrelation is passed
 ** since a string (ComponentRef) can not be uniquely mapped to an int. Therefore we need to compare two strings
 ** to get a unique ordering.
*"
  input BinTree in_bintree;
  input Key in_key;
  input Value in_value;
  output BinTree out_bintree;
algorithm 
  out_bintree:=
  matchcontinue (in_bintree,in_key,in_value)
    local
      Exp.ComponentRef key,value,rkey,rval;
      String rkeystr,keystr;
      BinTreeOption left,right;
      Integer cmpval;
      BinTree t_1,t,right_1,left_1;
    case (TREENODE(NONE,NONE,NONE),key,value) then TREENODE(SOME(TREEVALUE(key,value)),NONE,NONE); 
    case (TREENODE(SOME(TREEVALUE(rkey,rval)),left,right),key,value)
      equation 
        rkeystr = Exp.print_component_ref_str(rkey) " Replace this node ";
        keystr = Exp.print_component_ref_str(key);
        0 = System.strcmp(rkeystr, keystr); then TREENODE(SOME(TREEVALUE(rkey,value)),left,right);
    case (TREENODE(SOME(TREEVALUE(rkey,rval)),left,(right = SOME(t))),key,value)
      equation 
        keystr = Exp.print_component_ref_str(key) " Insert to right subtree";
        rkeystr = Exp.print_component_ref_str(rkey);
        cmpval = System.strcmp(rkeystr, keystr);
        (cmpval > 0) = true;
        t_1 = tree_add(t, key, value); then TREENODE(SOME(TREEVALUE(rkey,rval)),left,SOME(t_1));
    case (TREENODE(SOME(TREEVALUE(rkey,rval)),left,(right = NONE)),key,value)
      equation 
        keystr = Exp.print_component_ref_str(key) " Insert to right node";
        rkeystr = Exp.print_component_ref_str(rkey);
        cmpval = System.strcmp(rkeystr, keystr);
        (cmpval > 0) = true;
        right_1 = tree_add(TREENODE(NONE,NONE,NONE), key, value); then TREENODE(SOME(TREEVALUE(rkey,rval)),left,SOME(right_1));
    case (TREENODE(SOME(TREEVALUE(rkey,rval)),(left = SOME(t)),right),key,value)
      equation 
        keystr = Exp.print_component_ref_str(key) " Insert to left subtree";
        rkeystr = Exp.print_component_ref_str(rkey);
        cmpval = System.strcmp(rkeystr, keystr);
        (cmpval > 0) = false;
        t_1 = tree_add(t, key, value); then TREENODE(SOME(TREEVALUE(rkey,rval)),SOME(t_1),right);
    case (TREENODE(SOME(TREEVALUE(rkey,rval)),(left = NONE),right),key,value)
      equation 
        keystr = Exp.print_component_ref_str(key) " Insert to left node";
        rkeystr = Exp.print_component_ref_str(rkey);
        cmpval = System.strcmp(rkeystr, keystr);
        (cmpval > 0) = false;
        left_1 = tree_add(TREENODE(NONE,NONE,NONE), key, value); then TREENODE(SOME(TREEVALUE(rkey,rval)),SOME(left_1),right);
    case (_,_,_)
      equation 
        print("tree_add failed\n"); then fail();
  end matchcontinue;
end tree_add;

protected function tree_get2 "* function: tree_get2
 ** Copied from generic implementation. Changed that no hashrelation is passed
 ** since a string can not be uniquely mapped to an int. Therefore we need to compare two strings
 ** to get a unique ordering.
*"
  input BinTree2 in_bintree2;
  input Key in_key;
  output Value2 out_value2;
algorithm 
  out_value2:=
  matchcontinue (in_bintree2,in_key)
    local
      type ValueList = list<Exp.ComponentRef>;
      String rkeystr,keystr;
      Exp.ComponentRef rkey,key;
      ValueList rval,res;
      BinTree2Option left,right;
      Integer cmpval;
    case (TREENODE2(SOME(TREEVALUE2(rkey,rval)),left,right),key)
      equation 
        rkeystr = Exp.print_component_ref_str(rkey);
        keystr = Exp.print_component_ref_str(key);
        0 = System.strcmp(rkeystr, keystr); then rval;
    case (TREENODE2(SOME(TREEVALUE2(rkey,rval)),left,SOME(right)),key)
      local BinTree2 right;
      equation 
        keystr = Exp.print_component_ref_str(key) " Search to the right";
        rkeystr = Exp.print_component_ref_str(rkey);
        cmpval = System.strcmp(rkeystr, keystr);
        (cmpval > 0) = true;
        res = tree_get2(right, key); then res;
    case (TREENODE2(SOME(TREEVALUE2(rkey,rval)),SOME(left),right),key)
      local BinTree2 left;
      equation 
        keystr = Exp.print_component_ref_str(key) " Search to the left";
        rkeystr = Exp.print_component_ref_str(rkey);
        cmpval = System.strcmp(rkeystr, keystr);
        (cmpval > 0) = false;
        res = tree_get2(left, key); then res;
  end matchcontinue;
end tree_get2;

protected function tree_add2 "* function: tree_add2
 ** Copied from generic implementation. Changed that no hashrelation is passed
 ** since a string (ComponentRef) can not be uniquely mapped to an int. Therefore we need to compare two strings
 ** to get a unique ordering.
*"
  input BinTree2 in_bintree2;
  input Key in_key;
  input Value2 in_value2;
  output BinTree2 out_bintree2;
algorithm 
  out_bintree2:=
  matchcontinue (in_bintree2,in_key,in_value2)
    local
      type ValueList = list<Exp.ComponentRef>;
      Exp.ComponentRef key,rkey;
      ValueList value,rval;
      String rkeystr,keystr;
      BinTree2Option left,right;
      Integer cmpval;
      BinTree2 t_1,t,right_1,left_1;
    case (TREENODE2(NONE,NONE,NONE),key,value) then TREENODE2(SOME(TREEVALUE2(key,value)),NONE,NONE); 
    case (TREENODE2(SOME(TREEVALUE2(rkey,rval)),left,right),key,value)
      equation 
        rkeystr = Exp.print_component_ref_str(rkey) " Replace this node ";
        keystr = Exp.print_component_ref_str(key);
        0 = System.strcmp(rkeystr, keystr); then TREENODE2(SOME(TREEVALUE2(rkey,value)),left,right);
    case (TREENODE2(SOME(TREEVALUE2(rkey,rval)),left,(right = SOME(t))),key,value)
      equation 
        keystr = Exp.print_component_ref_str(key) " Insert to right subtree";
        rkeystr = Exp.print_component_ref_str(rkey);
        cmpval = System.strcmp(rkeystr, keystr);
        (cmpval > 0) = true;
        t_1 = tree_add2(t, key, value); then TREENODE2(SOME(TREEVALUE2(rkey,rval)),left,SOME(t_1));
    case (TREENODE2(SOME(TREEVALUE2(rkey,rval)),left,(right = NONE)),key,value)
      equation 
        keystr = Exp.print_component_ref_str(key) " Insert to right node";
        rkeystr = Exp.print_component_ref_str(rkey);
        cmpval = System.strcmp(rkeystr, keystr);
        (cmpval > 0) = true;
        right_1 = tree_add2(TREENODE2(NONE,NONE,NONE), key, value); then TREENODE2(SOME(TREEVALUE2(rkey,rval)),left,SOME(right_1));
    case (TREENODE2(SOME(TREEVALUE2(rkey,rval)),(left = SOME(t)),right),key,value)
      equation 
        keystr = Exp.print_component_ref_str(key) " Insert to left subtree";
        rkeystr = Exp.print_component_ref_str(rkey);
        cmpval = System.strcmp(rkeystr, keystr);
        (cmpval > 0) = false;
        t_1 = tree_add2(t, key, value); then TREENODE2(SOME(TREEVALUE2(rkey,rval)),SOME(t_1),right);
    case (TREENODE2(SOME(TREEVALUE2(rkey,rval)),(left = NONE),right),key,value)
      equation 
        keystr = Exp.print_component_ref_str(key) " Insert to left node";
        rkeystr = Exp.print_component_ref_str(rkey);
        cmpval = System.strcmp(rkeystr, keystr);
        (cmpval > 0) = false;
        left_1 = tree_add2(TREENODE2(NONE,NONE,NONE), key, value); then TREENODE2(SOME(TREEVALUE2(rkey,rval)),SOME(left_1),right);
    case (_,_,_)
      equation 
        print("tree_add2 failed\n"); then fail();
  end matchcontinue;
end tree_add2;
end VarTransform;

