package Codegen "
    Copyright (c) PELAB, Dept. Computer and Information Science, Linkoping University,
    Sweden, in the following denoted PELAB/LIU.

   
    This file is part of OpenModelica, in the following denoted OSM.


    The OSM License, Version 1.1, March 4, 2005.


    1 Preface

    The aim of this license is to lay down the conditions enabling you to use, 
    modify and circulate OSM. However, PELAB/LIU remain the authors of 
    OSM and so retain property rights and the use of all ancillary rights.

    2 Definitions

    OSM is defined as all successive versions of the OSM software and their 
    documentation that have been developed by PELAB/LIU and including
    accepted contributions from other contributors according to this license.

    OSM DERIVED SOFTWARE is defined as all or part of OSM that you have 
    modified and/or translated and/or adapted.


    3 Dual License

    OSM is made available under the OSM licensing scheme, which is a dual
    licensing scheme with two options, a) and b):

    a) OSM OPEN SOURCE LICENSE:

    If you wish to write Open Source software you can use the Open Source version 
    of OSM, released under the OSM license which include GPL as its open source 
    licensing option. If you use the OSM Open Source version you must release your 
    Application using OSM including this Application's source code under the GPL as well.

    This OSM license text, and Copyright (c) PELAB/Linkoping University, must
    be present in your copy of OSM and in OSM DERIVED SOFTWARE.

    You should have received a copy of the GPL - GNU General Public License
    along with OpenModelica; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.


    b) OSM COMMERCIAL LICENSE

    If you are using OSM commercially - that is, for commercial usage or
    for creating proprietary software for sale or use in a commercial setting
    - you must purchase a commercial license of OSM from PELAB/L
  *
 ** file:	 Codegen.rml
 ** module:      Codegen
 ** description: Generate C code from DAE (Flat Modelica) for Modelica 
 ** functions. This code is compiled and linked to the simulation code or when
 ** functions are called from the interactive environment.
 **
 ** Input: DAE
 ** Output: -   (generated code through Print module)
 ** Uses: Print Inst ModUtil Util
 **
 **
 ** RCS: $Id: Codegen.rml 1733 2005-04-27 06:47:45Z haklu $
 **
 *
   ------------------------------------------------------------------------- "

public import OpenModelica.Compiler.DAE;

public import OpenModelica.Compiler.Print;

public 
type Ident = String;

public 
type ReturnType = String;

public 
type FunctionName = String;

public 
type ArgumentDeclaration = String;

public 
type VariableDeclaration = String;

public 
type InitStatement = String;

public 
type Statement = String;

public 
type CleanupStatement = String;

public 
type ReturnTypeStruct = list<String>;

public 
type ArgumentDeclarationList = list<ArgumentDeclaration>;

type VariableDeclarationList = list<VariableDeclaration>;

type InitStatementList = list<InitStatement>;

type StatementList = list<Statement>;

type CleanupStatementList = list<CleanupStatement>;

uniontype CFunction
  record CFUNCTION
    ReturnType x1;
    FunctionName x2;
    ReturnTypeStruct x3;
    ArgumentDeclarationList x4;
    VariableDeclarationList x5;
    InitStatementList x6;
    StatementList x7;
    CleanupStatementList x8;
  end CFUNCTION;
  record CEXTFUNCTION
    ReturnType x1;
    FunctionName x2;
    ReturnTypeStruct x3;
    ArgumentDeclarationList x4;
  end CEXTFUNCTION;
end CFunction;

protected import OpenModelica.Compiler.Dump;

protected import OpenModelica.Compiler.Debug;

protected import OpenModelica.Compiler.Absyn;

protected import OpenModelica.Compiler.Algorithm;

protected import OpenModelica.Compiler.ClassInf;

protected import OpenModelica.Compiler.Exp;

protected import OpenModelica.Compiler.ModUtil;

protected import OpenModelica.Compiler.Types;

protected import OpenModelica.Compiler.Util;

protected import OpenModelica.Compiler.Inst;

protected constant CFunction c_empty_function=CFUNCTION("","",{},{},{},{},{},{}) " ------------------------------------------------------------------------- ";

protected function c_make_function
  input ReturnType in_returntype;
  input FunctionName in_functionname;
  input ReturnTypeStruct in_returntypestruct;
  input ArgumentDeclarationList in_argumentdeclarationlist;
  output CFunction out_cfunction;
algorithm 
  out_cfunction:=
  matchcontinue (in_returntype,in_functionname,in_returntypestruct,in_argumentdeclarationlist)
    local
      CleanupStatement rt,fn;
      CleanupStatementList rts,ads;
    case (rt,fn,rts,ads) then CFUNCTION(rt,fn,rts,ads,{},{},{},{}); 
  end matchcontinue;
end c_make_function;

protected function c_make_function_decl
  input ReturnType in_returntype;
  input FunctionName in_functionname;
  input ReturnTypeStruct in_returntypestruct;
  input ArgumentDeclarationList in_argumentdeclarationlist;
  output CFunction out_cfunction;
algorithm 
  out_cfunction:=
  matchcontinue (in_returntype,in_functionname,in_returntypestruct,in_argumentdeclarationlist)
    local
      CleanupStatement rt,fn;
      CleanupStatementList rts,ads;
    case (rt,fn,rts,ads) then CEXTFUNCTION(rt,fn,rts,ads); 
  end matchcontinue;
end c_make_function_decl;

protected function c_add_variables
  input CFunction in_cfunction;
  input VariableDeclarationList in_variabledeclarationlist;
  output CFunction out_cfunction;
algorithm 
  out_cfunction:=
  matchcontinue (in_cfunction,in_variabledeclarationlist)
    local
      CleanupStatementList vd_1,rts,ads,vd,is,st,cl,nvd;
      CleanupStatement rt,fn;
    case (CFUNCTION(rt,fn,rts,ads,vd,is,st,cl),nvd)
      equation 
        vd_1 = list_append(vd, nvd); then CFUNCTION(rt,fn,rts,ads,vd_1,is,st,cl);
  end matchcontinue;
end c_add_variables;

protected function c_add_inits
  input CFunction in_cfunction;
  input InitStatementList in_initstatementlist;
  output CFunction out_cfunction;
algorithm 
  out_cfunction:=
  matchcontinue (in_cfunction,in_initstatementlist)
    local
      CleanupStatementList is_1,rts,ads,vd,is,st,cl,nis;
      CleanupStatement rt,fn;
    case (CFUNCTION(rt,fn,rts,ads,vd,is,st,cl),nis)
      equation 
        is_1 = list_append(is, nis); then CFUNCTION(rt,fn,rts,ads,vd,is_1,st,cl);
  end matchcontinue;
end c_add_inits;

protected function c_add_statements
  input CFunction in_cfunction;
  input StatementList in_statementlist;
  output CFunction out_cfunction;
algorithm 
  out_cfunction:=
  matchcontinue (in_cfunction,in_statementlist)
    local
      CleanupStatementList st_1,rts,ads,vd,is,st,cl,nst;
      CleanupStatement rt,fn;
    case (CFUNCTION(rt,fn,rts,ads,vd,is,st,cl),nst)
      equation 
        st_1 = list_append(st, nst); then CFUNCTION(rt,fn,rts,ads,vd,is,st_1,cl);
  end matchcontinue;
end c_add_statements;

protected function c_add_cleanups
  input CFunction in_cfunction;
  input CleanupStatementList in_cleanupstatementlist;
  output CFunction out_cfunction;
algorithm 
  out_cfunction:=
  matchcontinue (in_cfunction,in_cleanupstatementlist)
    local
      CleanupStatementList cl_1,rts,ads,vd,is,st,cl,ncl;
      CleanupStatement rt,fn;
    case (CFUNCTION(rt,fn,rts,ads,vd,is,st,cl),ncl)
      equation 
        cl_1 = list_append(cl, ncl); then CFUNCTION(rt,fn,rts,ads,vd,is,st,cl_1);
  end matchcontinue;
end c_add_cleanups;

protected function c_merge_fns
  input CFunctionList in_cfunctionlist;
  output CFunction out_cfunction;
protected 
  type CFunctionList = list<CFunction>;
algorithm 
  out_cfunction:=
  matchcontinue (in_cfunctionlist)
    local
      CFunction cfn2,cfn,cfn1;
      CFunctionList r;
    case {} then c_empty_function; 
    case cfn1 :: r
      equation 
        cfn2 = c_merge_fns(r);
        cfn = c_merge_fn(cfn1, cfn2); then cfn;
  end matchcontinue;
end c_merge_fns;

protected function c_merge_fn
  input CFunction in_cfunction1;
  input CFunction in_cfunction2;
  output CFunction out_cfunction;
algorithm 
  out_cfunction:=
  matchcontinue (in_cfunction1,in_cfunction2)
    local
      CleanupStatementList vd,is,st,cl,rts,ad,vd1,is1,st1,cl1,vd2,is2,st2,cl2;
      CleanupStatement rt,fn;
    case (CFUNCTION(rt,fn,rts,ad,vd1,is1,st1,cl1),CFUNCTION(_,_,_,_,vd2,is2,st2,cl2))
      equation 
        vd = list_append(vd1, vd2);
        is = list_append(is1, is2);
        st = list_append(st1, st2);
        cl = list_append(cl1, cl2); then CFUNCTION(rt,fn,rts,ad,vd,is,st,cl);
  end matchcontinue;
end c_merge_fn;

protected function c_move_statements_to_inits
  input CFunction in_cfunction;
  output CFunction out_cfunction;
algorithm 
  out_cfunction:=
  matchcontinue (in_cfunction)
    local
      CleanupStatementList is_1,rts,ad,vd,is,st,cl;
      CleanupStatement rt,fn;
    case CFUNCTION(rt,fn,rts,ad,vd,is,st,cl)
      equation 
        is_1 = list_append(is, st); then CFUNCTION(rt,fn,rts,ad,vd,is_1,{},cl);
  end matchcontinue;
end c_move_statements_to_inits;

protected function c_print_functions
  input CFunctionList in_cfunctionlist;
protected 
  type CFunctionList = list<CFunction>;
algorithm 
  _:=
  matchcontinue (in_cfunctionlist)
    local
      CFunction f;
      CFunctionList r;
    case {} then (); 
    case f :: r
      equation 
        c_print_function(f);
        c_print_functions(r); then ();
  end matchcontinue;
end c_print_functions;

protected function c_print_function
  input CFunction in_cfunction;
algorithm 
  _:=
  matchcontinue (in_cfunction)
    local
      CleanupStatement args_str,stmt_str,rt,fn;
      Integer i0,i2,i3,i4,i5,i6,i7;
      CleanupStatementList rts,ad,vd,is,st,cl,ads;
    case CFUNCTION(rt,fn,rts,ad,vd,is,st,cl)
      equation 
        args_str = Util.string_delimit_list(ad, ", ");
        stmt_str = Util.string_append_list({rt," ",fn,"(",args_str,") {"});
        i0 = 0;
        i2 = c_print_indented(stmt_str, i0) "	c_print_indented_list (rts,i0) => i1 & Print.print_buf \"\n\" & ";
        Print.print_buf("\n") "	c_print_indented_list (rts,i0) => i1 & Print.print_buf \"\n\" & ";
        i3 = c_print_indented_list(vd, i2);
        Print.print_buf("\n");
        i4 = c_print_indented_list(is, i3);
        Print.print_buf("\n");
        i5 = c_print_indented_list(st, i4);
        Print.print_buf("\n");
        i6 = c_print_indented_list(cl, i5);
        Print.print_buf("\n");
        i7 = c_print_indented("}", i6);
        Print.print_buf("\n"); then ();
    case CEXTFUNCTION(rt,fn,rts,ads)
      equation 
        args_str = Util.string_delimit_list(ads, ", ");
        stmt_str = Util.string_append_list({"extern ",rt," ",fn,"(",args_str,");\n"});
        i0 = 0;
        i2 = c_print_indented(stmt_str, i0) "	c_print_indented_list (rts,i0) => i1 & Print.print_buf \"\n\" & ";
        Print.print_buf("\n") "	c_print_indented_list (rts,i0) => i1 & Print.print_buf \"\n\" & "; then ();
    case _
      equation 
        Print.print_buf("# c_print_function_failed\n"); then ();
  end matchcontinue;
end c_print_function;

protected function c_print_function_headers
  input CFunctionList in_cfunctionlist;
protected 
  type CFunctionList = list<CFunction>;
algorithm 
  _:=
  matchcontinue (in_cfunctionlist)
    local
      CFunction f;
      CFunctionList r;
    case {} then (); 
    case f :: r
      equation 
        c_print_function_header(f);
        c_print_function_headers(r); then ();
  end matchcontinue;
end c_print_function_headers;

protected function c_print_function_header
  input CFunction in_cfunction;
algorithm 
  _:=
  matchcontinue (in_cfunction)
    local
      CleanupStatement args_str,stmt_str,rt,fn;
      Integer i0,i1,i2;
      CleanupStatementList rts,ad,vd,is,st,cl,ads;
    case CFUNCTION(rt,fn,rts,ad,vd,is,st,cl)
      equation 
        args_str = Util.string_delimit_list(ad, ", ");
        stmt_str = Util.string_append_list({rt," ",fn,"(",args_str,");"});
        i0 = 0;
        i1 = c_print_indented_list(rts, i0);
        Print.print_buf("\n");
        i2 = c_print_indented(stmt_str, i1);
        Print.print_buf("\n"); then ();
    case CEXTFUNCTION(rt,fn,rts,ads)
      equation 
        args_str = Util.string_delimit_list(ads, ", ");
        stmt_str = Util.string_append_list({"extern ",rt," ",fn,"(",args_str,");\n"});
        i0 = 0;
        i1 = c_print_indented_list(rts, i0);
        Print.print_buf("\n");
        i2 = c_print_indented(stmt_str, i1);
        Print.print_buf("\n"); then ();
    case _
      equation 
        Print.print_buf("# c_print_function_header failed\n"); then ();
  end matchcontinue;
end c_print_function_header;

protected function c_print_indented_list
  input StringList in_stringlist;
  input Integer in_integer;
  output Integer out_integer;
protected 
  type StringList = list<String>;
algorithm 
  out_integer:=
  matchcontinue (in_stringlist,in_integer)
    local
      Integer i,i_1,i_2;
      CleanupStatement f;
      CleanupStatementList r;
    case ({},i) then i; 
    case (f :: r,i)
      equation 
        i_1 = c_print_indented(f, i);
        Print.print_buf("\n");
        i_2 = c_print_indented_list(r, i_1); then i_2;
  end matchcontinue;
end c_print_indented_list;

protected function c_print_indented
  input String str;
  input Integer i;
  output Integer i_1;
protected 
  type charList = list<char>;
  charList strl;
  Integer i_1,it;
algorithm 
  strl := string_list(str);
  i_1 := c_next_level(strl, i);
  it := c_this_level(strl, i);
  c_print_indent(it);
  Print.print_buf(str);
end c_print_indented;

protected function c_next_level
  input charList in_charlist;
  input Integer in_integer;
  output Integer out_integer;
protected 
  type charList = list<char>;
algorithm 
  out_integer:=
  matchcontinue (in_charlist,in_integer)
    local
      Integer i,i_1,i_2;
      char f;
      charList r;
    case ({},i) then i; 
    case (f :: r,i) " { "
      equation 
        "{" = list_string({f});
        i_1 = i + 2;
        i_2 = c_next_level(r, i_1); then i_2;
    case (f :: r,i) " } "
      equation 
        "}" = list_string({f});
        i_1 = i - 2;
        i_2 = c_next_level(r, i_1); then i_2;
    case (_ :: r,i)
      equation 
        i_1 = c_next_level(r, i); then i_1;
  end matchcontinue;
end c_next_level;

protected function c_this_level
  input charList in_charlist;
  input Integer in_integer;
  output Integer out_integer;
protected 
  type charList = list<char>;
algorithm 
  out_integer:=
  matchcontinue (in_charlist,in_integer)
    local
      char f;
      Integer i;
    case (f :: _,_)
      equation 
        "#" = list_string({f}); then 0;
    case (f :: _,i)
      equation 
        "}" = list_string({f}); then i - 2;
    case (_,i) then i; 
  end matchcontinue;
end c_this_level;

protected function c_print_indent
  input Integer in_integer;
algorithm 
  _:=
  matchcontinue (in_integer)
    local Integer i_1,i;
    case 0 then (); 
    case i
      equation 
        Print.print_buf(" ");
        i_1 = i - 1;
        c_print_indent(i_1); then ();
  end matchcontinue;
end c_print_indent;

public function generate_functions " ------------------------------------------------------------------------- 
  
 generate_functions
 generate_functions_elist
 generate_functions_elist2
 generate_params_type
 generate_function
 generate_result_struct
 generate_return_defs
 generate_return_decls
 generate_return_decl
 is_array
 is_first_in_array
 subs_is_one
 dae_exp_type
 dae_type_str
 dae_short_type_str
 exp_short_type_str
 exp_type_str
 generate_type
 generate_return_type
 generate_array_type
 generate_array_return_type
 print_int
 print_star
 generate_tuple_type
 generate_simple_type
 array_type_string
 generate_function_name
 generate_function_arg
 generate_function_body_tuple
 generate_alloc_outvars
 generate_alloc_outvar
 prefix_cr
 generate_algorithms
 generate_algorithms2
 generate_algorithm
 generate_algorithm_statements
 generate_algorithm_statement
 generate_range_expressions
 generate_else
 generate_vars
 generate_var
 is_var_q
 generate_var_q
 generate_var_q2
 generate_result_vars
 generate_result_var
 generate_equations
 generate_exp
 get_array_dim
 generate_expressions
 generate_expression
 generate_binary
 generate_temp_decl
 generate_scalar_lhs_cref
 generate_rhs_cref
 subs_to_scalar
 generate_scalar_rhs_cref
 generate_array_rhs_cref
 generate_index_spec
 generate_indices_array
 generate_indices
 generate_index_array
 generate_index
 indent_strings
 ident_cstr
 comp_ref_cstr
 generate_lbinary
 generate_lunary
 generate_relation
 generate_matrix
 generate_read_call_write
 invar_names
 generate_read
 generate_write
 is_rcw_output
 is_rcw_input


   ------------------------------------------------------------------------- "
  input DAE.DAElist in_daelist;
algorithm 
  _:=
  matchcontinue (in_daelist)
    local
      type DAE_ElementList = list<DAE.Element>;
      DAE.DAElist dae;
      DAE_ElementList elist;
    case ((dae = DAE.DAE(elist)))
      equation 
        generate_function_headers(dae);
        generate_function_bodies(dae); then ();
    case _
      equation 
        Print.print_buf("# generate_functions failed\n"); then fail();
  end matchcontinue;
end generate_functions;

public function generate_function_bodies
  input DAE.DAElist in_daelist;
algorithm 
  _:=
  matchcontinue (in_daelist)
    local
      type CFunctionList = list<CFunction>;
      type DAE_ElementList = list<DAE.Element>;
      CFunctionList cfns;
      DAE_ElementList elist;
    case DAE.DAE(elist)
      equation 
        Debug.fprintln("cgtr", "generate_function_bodies");
        cfns = generate_functions_elist(elist);
        Print.print_buf("\n/* body part */\n");
        c_print_functions(cfns);
        Print.print_buf("\n"); then ();
    case _
      equation 
        Print.print_buf("# generate_function_bodies failed\n"); then fail();
  end matchcontinue;
end generate_function_bodies;

public function generate_function_headers
  input DAE.DAElist in_daelist;
algorithm 
  _:=
  matchcontinue (in_daelist)
    local
      type CFunctionList = list<CFunction>;
      type DAE_ElementList = list<DAE.Element>;
      CFunctionList cfns;
      DAE_ElementList elist;
    case DAE.DAE(elist)
      equation 
        Debug.fprintln("cgtr", "generate_function_headers");
        cfns = generate_functions_elist(elist);
        Print.print_buf("/* header part */\n");
        Print.print_buf("#include \"modelica.h\"\n");
        c_print_function_headers(cfns) " FIXME: needed? ";
        Print.print_buf("\n") " FIXME: needed? "; then ();
    case _
      equation 
        Print.print_buf("# generate_function_headers failed\n"); then fail();
  end matchcontinue;
end generate_function_headers;

protected function generate_functions_elist
  input DAE_ElementList els;
  output CFunctionList cfns;
protected 
  type DAE_ElementList = list<DAE.Element>;
  type CFunctionList = list<CFunction>;
  DAE_ElementList fns;
algorithm 
  Debug.fprintln("cgtr", "generate_functions_elist");
  Debug.fprintln("cgtrdumpdae", "Dumping DAE:");
  Debug.fcall("cgtrdumpdae", DAE.dump2, DAE.DAE(els));
  fns := DAE.get_matching(els, DAE.is_function);
  cfns := generate_functions_elist2(fns);
end generate_functions_elist;

protected function generate_functions_elist2
  input DAE_ElementList in_dae_elementlist;
  output CFunctionList out_cfunctionlist;
protected 
  type DAE_ElementList = list<DAE.Element>;
  type CFunctionList = list<CFunction>;
algorithm 
  out_cfunctionlist:=
  matchcontinue (in_dae_elementlist)
    local
      CFunctionList cfns1,cfns2,cfns;
      DAE.Element f;
      DAE_ElementList rest;
    case {}
      equation 
        Debug.fprintln("cgtr", "generate_functions_elist2"); then {};
    case f :: rest
      equation 
        Debug.fprintln("cgtr", "generate_functions_elist2");
        cfns1 = generate_function(f);
        cfns2 = generate_functions_elist2(rest);
        cfns = list_append(cfns1, cfns2); then cfns;
  end matchcontinue;
end generate_functions_elist2;

protected function generate_params_type
  input Ident n;
  output String s;
algorithm 
  s := string_append(n, "_params");
end generate_params_type;

protected function generate_function
  input DAE.Element in_element;
  output CFunctionList out_cfunctionlist;
protected 
  type CFunctionList = list<CFunction>;
algorithm 
  out_cfunctionlist:=
  matchcontinue (in_element)
    local
      type DAE_ElementList = list<DAE.Element>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleCleanupStatementTupleTypes_TTypeAbsyn_PathOption = tuple<CleanupStatement,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleCleanupStatementTupleTypes_TTypeAbsyn_PathOptionList = list<TupleCleanupStatementTupleTypes_TTypeAbsyn_PathOption>;
      type DAE_ExtArgList = list<DAE.ExtArg>;
      CleanupStatement fn_name_str,retstr,extfnname,lang,retstructtype,extfnname_1,n;
      DAE_ElementList outvars,invars,dae,bivars,orgdae,daelist;
      CleanupStatementList struct_strs,arg_strs;
      CFunction head_cfn,body_cfn,cfn,rcw_fn,func_decl;
      Absyn.Path fpath;
      TupleCleanupStatementTupleTypes_TTypeAbsyn_PathOptionList args;
      TupleTypes_TTypeAbsyn_PathOption restype;
      DAE_ExtArgList extargs;
      DAE.ExtArg extretarg;
      DAE.ExternalDecl extdecl;
      CFunctionList cfns;
      DAE.Element comp;
    case DAE.FUNCTION(fpath,DAE.DAE(dae),(Types.T_FUNCTION(args,restype),_))
      equation 
        fn_name_str = generate_function_name(fpath);
        Debug.fprintl("cgtr", {"generating function ",fn_name_str,"\n"});
        Debug.fprintln("cgtrdumpdae3", "Dumping DAE:");
        Debug.fcall("cgtrdumpdae3", DAE.dump2, DAE.DAE(dae));
        outvars = DAE.get_output_vars(dae);
        invars = DAE.get_input_vars(dae);
        struct_strs = generate_result_struct(outvars, fpath);
        retstr = generate_return_type(fpath);
        arg_strs = Util.list_map(args, generate_function_arg);
        head_cfn = c_make_function(retstr, fn_name_str, struct_strs, arg_strs);
        body_cfn = generate_function_body_tuple(fpath, dae, restype);
        cfn = c_merge_fn(head_cfn, body_cfn);
        rcw_fn = generate_read_call_write(fn_name_str, outvars, retstr, invars); then {cfn,rcw_fn};
    case DAE.EXTFUNCTION(fpath,DAE.DAE(orgdae),(Types.T_FUNCTION(args,restype),_),extdecl)
      equation 
        fn_name_str = generate_function_name(fpath);
        Debug.fprintl("cgtr", {"generating external function ",fn_name_str,"\n"});
        DAE.EXTERNALDECL(extfnname,extargs,extretarg,lang) = extdecl;
        Debug.fprintln("cgtrdumpdae1", "Dumping DAE:");
        Debug.fcall("cgtrdumpdae1", DAE.dump2, DAE.DAE(orgdae));
        dae = Inst.init_vars_modelica_output(orgdae);
        Debug.fprintln("cgtrdumpdae2", "Dumping DAE:");
        Debug.fcall("cgtrdumpdae2", DAE.dump2, DAE.DAE(dae));
        outvars = DAE.get_output_vars(dae);
        invars = DAE.get_input_vars(dae);
        bivars = DAE.get_bidir_vars(dae);
        struct_strs = generate_result_struct(outvars, fpath);
        retstructtype = generate_return_type(fpath);
        retstr = generate_ext_return_type(extretarg);
        extfnname_1 = generate_ext_function_name(extfnname, lang);
        arg_strs = generate_ext_function_args(extargs, lang);
        func_decl = c_make_function_decl(retstr, extfnname_1, struct_strs, arg_strs);
        rcw_fn = generate_read_call_write_external(fn_name_str, outvars, retstructtype, invars, extdecl, 
          bivars); then {func_decl,rcw_fn};
    case DAE.COMP(n,DAE.DAE(daelist))
      equation 
        cfns = generate_functions_elist(daelist); then cfns;
    case comp
      equation 
        Print.print_buf("# generate_function failed\n");
        DAE.dump2(DAE.DAE({comp})); then fail();
  end matchcontinue;
end generate_function;

protected function generate_ext_function_name
  input String in_string1;
  input String in_string2;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_string1,in_string2)
    local CleanupStatement name,name_1;
    case (name,"C") then name; 
    case (name,"FORTRAN 77")
      equation 
        name_1 = string_append(name, "_"); then name_1;
    case (_,_)
      equation 
        Print.print_error_buf("# Unknown language in external declaration\n"); then fail();
  end matchcontinue;
end generate_ext_function_name;

protected function generate_result_struct
  input DAE_ElementList outvars;
  input Absyn.Path fpath;
  output StringList strs;
protected 
  type DAE_ElementList = list<DAE.Element>;
  type StringList = list<String>;
  CleanupStatement ptname,first_row,last_row;
  CleanupStatementList var_strs,var_names,defs,var_strs_1;
algorithm 
  ptname := generate_return_type(fpath);
  (var_strs,var_names) := generate_return_decls(outvars);
  defs := generate_return_defs(ptname, var_names, 1);
  var_strs_1 := indent_strings(var_strs);
  first_row := Util.string_append_list({"typedef struct ",ptname,"_s"});
  last_row := Util.string_append_list({"} ",ptname,";"});
  strs := Util.list_flatten({defs,{first_row,"{"},var_strs_1,{last_row}});
end generate_result_struct;

protected function generate_return_defs
  input String in_string;
  input StringList in_stringlist;
  input Integer in_integer;
  output StringList out_stringlist;
protected 
  type StringList = list<String>;
algorithm 
  out_stringlist:=
  matchcontinue (in_string,in_stringlist,in_integer)
    local
      CleanupStatement i_str,f_1,tn,f;
      Integer i_1,i;
      CleanupStatementList r_1,r;
    case (_,{},_) then {}; 
    case (tn,f :: r,i)
      equation 
        i_str = int_string(i);
        f_1 = Util.string_append_list({"#define ",tn,"_",i_str," ",f});
        i_1 = i + 1;
        r_1 = generate_return_defs(tn, r, i_1); then f_1 :: r_1;
  end matchcontinue;
end generate_return_defs;

protected function generate_return_decls
  input DAE_ElementList in_dae_elementlist;
  output StringList out_stringlist1;
  output StringList out_stringlist2;
protected 
  type DAE_ElementList = list<DAE.Element>;
  type StringList = list<String>;
algorithm 
  (out_stringlist1,out_stringlist2):=
  matchcontinue (in_dae_elementlist)
    local
      CleanupStatementList rs,rd;
      DAE.Element first;
      DAE_ElementList rest;
      CleanupStatement fs,fd;
    case {} then ({},{}); 
    case first :: rest
      equation 
        ("",_) = generate_return_decl(first);
        (rs,rd) = generate_return_decls(rest); then (rs,rd);
    case first :: rest
      equation 
        (fs,fd) = generate_return_decl(first);
        (rs,rd) = generate_return_decls(rest); then (fs :: rs,fd :: rd);
  end matchcontinue;
end generate_return_decls;

protected function tmpprintinit
  input Exp_ExpOption in_exp_expoption;
  output String out_string;
protected 
  type Exp_ExpOption = Option<Exp.Exp>;
algorithm 
  out_string:=
  matchcontinue (in_exp_expoption)
    local
      CleanupStatement str,str1;
      Exp.Exp e;
    case NONE then ""; 
    case SOME(e)
      equation 
        str = Exp.print_exp_str(e);
        str1 = Util.string_append_list({" /* ",str," */"}); then str1;
  end matchcontinue;
end tmpprintinit;

protected function generate_return_decl
  input DAE.Element in_element;
  output String out_string1;
  output String out_string2;
algorithm 
  (out_string1,out_string2):=
  matchcontinue (in_element)
    local
      type Exp_ExpOption = Option<Exp.Exp>;
      type Exp_SubscriptList = list<Exp.Subscript>;
      type Absyn_PathList = list<Absyn.Path>;
      type DAE_VariableAttributesOption = Option<DAE.VariableAttributes>;
      type Absyn_CommentOption = Option<Absyn.Comment>;
      Boolean is_a;
      CleanupStatement typ_str,id_str,dims_str,decl_str_1,expstr,decl_str;
      CleanupStatementList dim_strs;
      DAE.Element var;
      Exp.ComponentRef id;
      DAE.Type typ;
      Exp_ExpOption initopt,start;
      Exp_SubscriptList inst_dims;
      DAE.Flow flow_;
      Absyn_PathList class_;
      DAE_VariableAttributesOption dae_var_attr;
      Absyn_CommentOption comment;
    case ((var = DAE.VAR(id,DAE.VARIABLE(),DAE.OUTPUT(),typ,initopt,inst_dims,start,flow_,class_,dae_var_attr,comment))) " changed DAE.VAR(..., NONE, inst_dims) to ..._, inst_dims) because removed second   rule "
      equation 
        is_a = is_array(var);
        typ_str = dae_type_str(typ, is_a);
        (id_str,_) = comp_ref_cstr(id);
        dim_strs = Util.list_map(inst_dims, dim_string);
        dims_str = Util.string_delimit_list(dim_strs, ", ");
        decl_str_1 = Util.string_append_list({typ_str," ",id_str,";"," /* [",dims_str,"] */"});
        expstr = tmpprintinit(initopt);
        decl_str = string_append(decl_str_1, expstr); then (decl_str,id_str);
    case (_) then ("",""); 
  end matchcontinue;
end generate_return_decl;

protected function is_array
  input DAE.Element in_element;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_element)
    local
      type Exp_ExpOption = Option<Exp.Exp>;
      type Absyn_PathList = list<Absyn.Path>;
      type DAE_VariableAttributesOption = Option<DAE.VariableAttributes>;
      type Absyn_CommentOption = Option<Absyn.Comment>;
      DAE.Element el;
      Exp.ComponentRef cr;
      DAE.VarKind vk;
      DAE.VarDirection vd;
      DAE.Type ty;
      Exp_ExpOption st;
      DAE.Flow fl;
      Absyn_PathList cl;
      DAE_VariableAttributesOption dae_var_attr;
      Absyn_CommentOption comment;
    case ((el = DAE.VAR(cr,vk,vd,ty,_,{},st,fl,cl,dae_var_attr,comment))) "
  axiom	is_array DAE.VAR(cr,vk,vd,ty,_,{},st,fl,cl) => false
  axiom	is_array DAE.VAR(cr,vk,vd,ty,_,_::_,st,fl,cl) => true
"
      equation 
        Debug.fcall("isarrdb", DAE.dump2, DAE.DAE({el})); then false;
    case ((el = DAE.VAR(cr,vk,vd,ty,_,_ :: _,st,fl,cl,dae_var_attr,comment)))
      equation 
        Debug.fcall("isarrdb", DAE.dump2, DAE.DAE({el})); then true;
    case el
      equation 
        Print.print_buf("#-- is_array failed\n");
        Print.print_buf("#-- Not a var?");
        DAE.dump2(DAE.DAE({el})); then fail();
  end matchcontinue;
end is_array;

protected function is_array_or_string
  input DAE.Element in_element;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_element)
    local
      type Exp_ExpOption = Option<Exp.Exp>;
      type Absyn_PathList = list<Absyn.Path>;
      type DAE_VariableAttributesOption = Option<DAE.VariableAttributes>;
      type Absyn_CommentOption = Option<Absyn.Comment>;
      DAE.Element var,el;
      Exp.ComponentRef cr;
      DAE.VarKind vk;
      DAE.VarDirection vd;
      Exp_ExpOption st;
      DAE.Flow fl;
      Absyn_PathList cl;
      DAE_VariableAttributesOption dae_var_attr;
      Absyn_CommentOption comment;
      DAE.Type ty;
    case ((var = DAE.VAR(cr,vk,vd,DAE.STRING(),_,_,st,fl,cl,dae_var_attr,comment)))
      equation 
        true = is_array(var);
        Print.print_error_buf("# array of strings is probably not supported yet\n"); then fail();
    case ((var = DAE.VAR(cr,vk,vd,ty,_,_,st,fl,cl,dae_var_attr,comment)))
      equation 
        true = is_array(var); then true;
    case ((var = DAE.VAR(cr,vk,vd,DAE.STRING(),_,_,st,fl,cl,dae_var_attr,comment))) then true; 
    case el
      equation 
        Print.print_buf("# is_array_or_string failed\n");
        DAE.dump2(DAE.DAE({el})); then fail();
  end matchcontinue;
end is_array_or_string;

protected function dae_exp_type "
relation is_first_in_array : Exp.ComponentRef => bool =

  axiom	is_first_in_array Exp.CREF_IDENT(_,{}) => true

  rule	subs_is_one subs => b
	---------------------
	is_first_in_array Exp.CREF_IDENT(_,subs) => b

  rule	is_first_in_array cref => b
	---------------------------
	is_first_in_array Exp.CREF_QUAL(_,{},cref) => b

  rule	subs_is_one subs => b1 &
	is_first_in_array cref => b2 &
	bool_and(b1,b2) => b
	------------------------------
	is_first_in_array Exp.CREF_QUAL(_,subs,cref) => b
end

relation subs_is_one : Exp.Subscript list => bool =

  axiom subs_is_one {} => true

  rule	subs_is_one r => b
	------------------
	subs_is_one Exp.INDEX(Exp.ICONST(1))::r => b

  axiom	subs_is_one _::_ => false

end
"
  input DAE.Type in_type;
  output Exp.Type out_type;
algorithm 
  out_type:=
  matchcontinue (in_type)
    case DAE.INT() then Exp.INT; 
    case DAE.REAL() then Exp.REAL; 
    case DAE.STRING() then Exp.STRING; 
    case DAE.BOOL() then Exp.BOOL; 
    case DAE.ENUM() then Exp.ENUM; 
    case _ then Exp.OTHER; 
  end matchcontinue;
end dae_exp_type;

protected function dae_type_str
  input DAE.Type t;
  input Boolean a;
  output String str;
protected 
  Exp.Type t_1;
algorithm 
  t_1 := dae_exp_type(t);
  str := exp_type_str(t_1, a);
end dae_type_str;

protected function dae_short_type_str
  input DAE.Type t;
  output String str;
protected 
  Exp.Type t_1;
algorithm 
  t_1 := dae_exp_type(t);
  str := exp_short_type_str(t_1);
end dae_short_type_str;

protected function exp_short_type_str
  input Exp.Type in_type;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_type)
    case Exp.INT then "integer"; 
    case Exp.REAL then "real"; 
    case Exp.STRING then "string"; 
    case Exp.BOOL then "boolean"; 
    case Exp.OTHER then "OTHER"; 
    case Exp.ENUM then "ENUM_NOT_IMPLEMENTED"; 
  end matchcontinue;
end exp_short_type_str;

protected function exp_type_str
  input Exp.Type in_type;
  input Boolean in_boolean;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_type,in_boolean)
    local
      CleanupStatement tstr,str;
      Exp.Type t;
    case (t,false)
      equation 
        tstr = exp_short_type_str(t);
        str = string_append("modelica_", tstr); then str;
    case (t,true)
      equation 
        tstr = exp_short_type_str(t);
        str = string_append(tstr, "_array"); then str;
  end matchcontinue;
end exp_type_str;

protected function generate_type
  input Types.Type in_type;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_type)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleTypes_TTypeAbsyn_PathOptionList = list<TupleTypes_TTypeAbsyn_PathOption>;
      type IntegerList = list<Integer>;
      CleanupStatement ty_str;
      TupleTypes_TTypeAbsyn_PathOptionList tys;
      TupleTypes_TTypeAbsyn_PathOption arrayty,ty;
      IntegerList dims;
    case ((Types.T_TUPLE(tys),_))
      equation 
        Debug.fprintln("cgtr", "generate_type");
        ty_str = generate_tuple_type(tys); then ty_str;
    case ((tys = (Types.T_ARRAY(_,_),_)))
      local TupleTypes_TTypeAbsyn_PathOption tys;
      equation 
        Debug.fprintln("cgtr", "generate_type");
        (arrayty,dims) = Types.flatten_array_type(tys);
        ty_str = generate_array_type(arrayty, dims); then ty_str;
    case ((Types.T_INTEGER(_),_)) then "modelica_integer"; 
    case ((Types.T_REAL(_),_)) then "modelica_real"; 
    case ((Types.T_STRING(_),_)) then "modelica_string"; 
    case ((Types.T_BOOL(_),_)) then "modelica_boolean"; 
    case ty
      equation 
        Print.print_buf("#-- generate_type failed: ");
        Types.print_type(ty);
        Print.print_buf("\n"); then fail();
  end matchcontinue;
end generate_type;

protected function generate_type_external
  input Types.Type in_type;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_type)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      CleanupStatement str;
      Types.ArrayDim dim;
      TupleTypes_TTypeAbsyn_PathOption ty;
    case ((Types.T_INTEGER(_),_)) then "int"; 
    case ((Types.T_REAL(_),_)) then "double"; 
    case ((Types.T_STRING(_),_)) then "const char*"; 
    case ((Types.T_BOOL(_),_)) then "int"; 
    case ((Types.T_ARRAY(dim,ty),_))
      equation 
        str = generate_type_external(ty); then str;
    case ty
      equation 
        Print.print_buf("#-- generate_type_external failed: ");
        Types.print_type(ty);
        Print.print_buf("\n"); then fail();
  end matchcontinue;
end generate_type_external;

protected function generate_type_internal
  input Types.Type in_type;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_type)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      CleanupStatement tystr,str;
      TupleTypes_TTypeAbsyn_PathOption ty,elty;
    case ty " for the basic types, only those must match "
      equation 
        true = Types.basic_type(ty);
        tystr = generate_type_internal_namepart(ty);
        str = string_append("modelica_", tystr); then str;
    case ty
      equation 
        true = Types.is_array(ty);
        elty = Types.array_element_type(ty);
        tystr = generate_type_internal(elty);
        str = string_append(tystr, "_array"); then str;
    case ty
      equation 
        Print.print_buf("#-- generate_type_internal failed: ");
        Types.print_type(ty);
        Print.print_buf("\n"); then fail();
  end matchcontinue;
end generate_type_internal;

protected function generate_type_internal_namepart " for the basic types, only those must match "
  input Types.Type in_type;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_type)
    case ((Types.T_INTEGER(_),_)) then "integer"; 
    case ((Types.T_REAL(_),_)) then "real"; 
    case ((Types.T_STRING(_),_)) then "string"; 
    case ((Types.T_BOOL(_),_)) then "boolean"; 
    case ((Types.T_ENUM,_)) then "T_ENUM_NOT_IMPLEMENTED"; 
  end matchcontinue;
end generate_type_internal_namepart;

protected function generate_return_type
  input Absyn.Path fpath;
  output String res;
protected 
  CleanupStatement fstr;
algorithm 
  fstr := generate_function_name(fpath);
  res := string_append(fstr, "_rettype");
end generate_return_type;

protected function generate_array_type
  input Types.Type ty;
  input IntegerList dims;
  output String str;
protected 
  type IntegerList = list<Integer>;
algorithm 
  str := array_type_string(ty);
end generate_array_type;

protected function generate_array_return_type
  input Types.Type ty;
  input IntegerList dims;
  output String ty_str;
protected 
  type IntegerList = list<Integer>;
algorithm 
  ty_str := array_type_string(ty);
end generate_array_return_type;

protected function print_int
  input Integer i;
protected 
  CleanupStatement str;
algorithm 
  str := int_string(i);
  Print.print_buf(str);
end print_int;

protected function print_star
  input Integer i;
algorithm 
  Print.print_buf("*");
end print_star;

protected function generate_tuple_type
  input Types_TypeList in_types_typelist;
  output String out_string;
protected 
  type Types_TypeList = list<Types.Type>;
algorithm 
  out_string:=
  matchcontinue (in_types_typelist)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleTypes_TTypeAbsyn_PathOptionList = list<TupleTypes_TTypeAbsyn_PathOption>;
      CleanupStatement str,str_1,str_2,str_3;
      TupleTypes_TTypeAbsyn_PathOption ty;
      TupleTypes_TTypeAbsyn_PathOptionList tys;
    case {ty}
      equation 
        Debug.fprintln("cgtr", "generate_tuple_type_1");
        str = generate_simple_type(ty); then str;
    case (ty :: tys)
      equation 
        Debug.fprintln("cgtr", "generate_tuple_type_2");
        str = generate_simple_type(ty);
        str_1 = generate_tuple_type(tys);
        str_2 = string_append(str, str_1);
        str_3 = string_append("struct ", str_2); then str_3;
  end matchcontinue;
end generate_tuple_type;

protected function generate_simple_type
  input Types.Type in_type;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_type)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      CleanupStatement n_1,n_2,n,t_str;
      TupleTypes_TTypeAbsyn_PathOption t_1,t,ty;
    case ((Types.T_INTEGER(_),_)) then "modelica_integer"; 
    case ((Types.T_REAL(_),_)) then "modelica_real"; 
    case ((Types.T_STRING(_),_)) then "modelica_string"; 
    case ((Types.T_BOOL(_),_)) then "modelica_boolean"; 
    case ((Types.T_COMPLEX(ClassInf.RECORD(n),_),_))
      equation 
        n_1 = string_append("const ", n);
        n_2 = string_append(n_1, "&"); then n_2;
    case ((t = (Types.T_ARRAY(_,_),_)))
      equation 
        t_1 = Types.array_element_type(t);
        t_str = array_type_string(t_1); then t_str;
    case ty
      equation 
        Print.print_buf("#--generate_simple_type failed ");
        Types.print_type(ty);
        Print.print_buf("\n"); then fail();
  end matchcontinue;
end generate_simple_type;

protected function array_type_string
  input Types.Type in_type;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_type)
    case ((Types.T_INTEGER(_),_)) then "integer_array"; 
    case ((Types.T_REAL(_),_)) then "real_array"; 
    case ((Types.T_STRING(_),_)) then "string_array"; 
    case ((Types.T_BOOL(_),_)) then "boolean_array"; 
  end matchcontinue;
end array_type_string;

protected function generate_function_name
  input Absyn.Path fpath;
  output String fstr;
algorithm 
  fstr := ModUtil.path_string2(fpath, "_");
end generate_function_name;

protected function generate_ext_function_args " input string is language, e.g. \"C\" or \"FORTRAN 77\" "
  input DAE_ExtArgList in_dae_extarglist;
  input String in_string;
  output StringList out_stringlist;
protected 
  type DAE_ExtArgList = list<DAE.ExtArg>;
  type StringList = list<String>;
algorithm 
  out_stringlist:=
  matchcontinue (in_dae_extarglist,in_string)
    local
      CleanupStatementList arg_strs;
      DAE_ExtArgList extargs;
      CleanupStatement lang;
    case (extargs,"C")
      equation 
        arg_strs = Util.list_map(extargs, generate_ext_function_arg); then arg_strs;
    case (extargs,"FORTRAN 77")
      equation 
        arg_strs = Util.list_map(extargs, generate_ext_function_arg_f77); then arg_strs;
    case (_,lang)
      equation 
        Print.print_buf("#-- Unknown external language declaration\n");
        Print.print_buf(lang); then fail();
  end matchcontinue;
end generate_ext_function_args;

protected function generate_function_arg
  input Types.FuncArg in_funcarg;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_funcarg)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      CleanupStatement str,str_1,str_2,name;
      TupleTypes_TTypeAbsyn_PathOption ty;
    case ((name,ty))
      equation 
        str = generate_tuple_type({ty});
        str_1 = string_append(str, " ");
        str_2 = string_append(str_1, name); then str_2;
  end matchcontinue;
end generate_function_arg;

protected function generate_ext_arg_type " FIXME : shouldn't it be \"const <type> *\" instead of
   <type> const *\"
   changed this already.
"
  input Types.Attributes in_attributes;
  input Types.Type in_type;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_attributes,in_type)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      CleanupStatement str,resstr,tystr;
      TupleTypes_TTypeAbsyn_PathOption ty;
    case (Types.ATTR(_,_,_,Absyn.INPUT),ty)
      equation 
        false = Types.is_array(ty);
        str = generate_type_external(ty); then str;
    case (Types.ATTR(_,_,_,Absyn.INPUT),ty)
      equation 
        true = Types.is_array(ty);
        str = generate_type_external(ty);
        resstr = Util.string_append_list({"const ",str," *"}); then resstr;
    case (Types.ATTR(_,_,_,Absyn.OUTPUT),ty)
      equation 
        tystr = generate_type_external(ty);
        str = string_append(tystr, "*"); then str;
    case (Types.ATTR(_,_,_,Absyn.BIDIR),ty)
      equation 
        tystr = generate_type_external(ty);
        str = string_append(tystr, "*"); then str;
    case (_,ty)
      equation 
        str = generate_type_external(ty); then str;
    case (_,_)
      equation 
        Print.print_buf("#-- generate_ext_arg_type failed\n"); then fail();
  end matchcontinue;
end generate_ext_arg_type;

protected function generate_ext_function_arg
  input DAE.ExtArg in_extarg;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_extarg)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      CleanupStatement tystr,name,res;
      Exp.ComponentRef cref;
      Types.Attributes attr;
      TupleTypes_TTypeAbsyn_PathOption ty;
      Exp.Exp exp;
    case DAE.EXTARG(cref,attr,ty) " LS: Why different names, it's only the declaration anyway   commenting out  
  rule	Types.is_array ty => false &
	generate_ext_arg_type (attr,ty) => tystr &
	var_name_external cref => name &
	Util.string_append_list({tystr, \" \", name}) => res
	-----------------------------------
	generate_ext_function_arg DAE.EXTARG(cref, attr, ty) => res
"
      equation 
        tystr = generate_ext_arg_type(attr, ty) " Types.is_array ty => true & ";
        (name,_) = comp_ref_cstr(cref);
        res = Util.string_append_list({tystr," ",name}); then res;
    case DAE.EXTARGEXP(exp,ty)
      equation 
        res = generate_type_external(ty); then res;
    case DAE.EXTARGSIZE(_,_,_,_) then "size_t"; 
    case (_)
      equation 
        Print.print_buf("#-- generate_ext_function_arg failed\n"); then fail();
  end matchcontinue;
end generate_ext_function_arg;

protected function generate_ext_arg_type_f77
  input Types.Attributes in_attributes;
  input Types.Type in_type;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_attributes,in_type)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      CleanupStatement str,resstr,tystr;
      TupleTypes_TTypeAbsyn_PathOption ty;
      Types.Attributes attr;
    case (Types.ATTR(_,_,_,Absyn.INPUT),ty)
      equation 
        str = generate_type_external(ty);
        resstr = Util.string_append_list({"const ",str," *"}); then resstr;
    case (Types.ATTR(_,_,_,Absyn.OUTPUT),ty)
      equation 
        tystr = generate_type_external(ty);
        str = string_append(tystr, "*"); then str;
    case ((attr = Types.ATTR(_,_,_,Absyn.BIDIR)),ty)
      equation 
        str = generate_ext_arg_type(attr, ty); then str;
    case ((attr = Types.ATTR(_,_,_,Absyn.BIDIR)),ty)
      equation 
        str = generate_ext_arg_type(attr, ty); then str;
    case (attr,ty)
      equation 
        str = generate_ext_arg_type(attr, ty); then str;
    case (_,_)
      equation 
        Print.print_buf("#-- generate_ext_arg_type_f77 failed\n"); then fail();
  end matchcontinue;
end generate_ext_arg_type_f77;

protected function generate_ext_function_arg_f77
  input DAE.ExtArg in_extarg;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_extarg)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      CleanupStatement tystr,name,res;
      Exp.ComponentRef cref;
      Types.Attributes attr;
      TupleTypes_TTypeAbsyn_PathOption ty;
      DAE.ExtArg arg;
    case DAE.EXTARG(cref,attr,ty)
      equation 
        tystr = generate_ext_arg_type_f77(attr, ty);
        (name,_) = comp_ref_cstr(cref);
        res = Util.string_append_list({tystr," ",name}); then res;
    case ((arg = DAE.EXTARGEXP(_,_)))
      equation 
        res = generate_ext_function_arg(arg); then res;
    case DAE.EXTARGSIZE(_,_,_,_) then "int const *"; 
    case (_)
      equation 
        Print.print_buf("#-- generate_ext_function_arg failed\n"); then fail();
  end matchcontinue;
end generate_ext_function_arg_f77;

protected function generate_ext_return_type "
relation generate_ext_arg_dims : (Ident, int list) => string =

  rule	generate_ext_arg_dims_2 (name,dims,1) => str	
	-----------------------------------------
       	generate_ext_arg_dims (name,dims) => str

end

relation generate_ext_arg_dims_2 : (Ident, int list, int) => string =

  rule	int_string i => dimno &
	Util.string_append_list ({\"size_t \", name, \"_dim\", dimno}) => str
	------------------------------------------
	generate_ext_arg_dims_2 (name, {v}, i) => str

  rule	int_add(i,1) => ipp &
	generate_ext_arg_dims_2 (name, rest, ipp) => reststr &
	int_string i => dimno &
	Util.string_append_list({\"size_t\", name, \"_dim\", dimno, \", \", reststr}) => str
	--------------------------------------------------------------------
	generate_ext_arg_dims_2 (name, v::rest, i) => str

end
"
  input DAE.ExtArg in_extarg;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_extarg)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      CleanupStatement res;
      Exp.ComponentRef cref;
      Types.Attributes attr;
      TupleTypes_TTypeAbsyn_PathOption ty;
    case DAE.EXTARG(cref,attr,ty)
      equation 
        res = generate_type_external(ty); then res;
    case DAE.NOEXTARG() then "void"; 
    case _
      equation 
        Print.print_buf("#-- generate_ext_return_type failed\n"); then fail();
  end matchcontinue;
end generate_ext_return_type;

protected function generate_ext_return_type_f77
  input DAE.ExtArg arg;
  output String str;
algorithm 
  str := generate_ext_return_type(arg);
end generate_ext_return_type_f77;

protected function generate_function_body_tuple
  input Absyn.Path fpath;
  input DAE_ElementList dae;
  input Types.Type restype;
  output CFunction cfn;
protected 
  type DAE_ElementList = list<DAE.Element>;
  Integer tnr,tnr_ret_1,tnr_ret,tnr_mem,tnr_var,tnr_alg,tnr_res;
  CleanupStatement ret_type_str,ret_decl,ret_var,ret_stmt,mem_decl,mem_var,mem_stmt1,mem_stmt2;
  DAE_ElementList outvars;
  CFunction out_fn,mem_fn_1,mem_fn,var_fn,alg_fn,res_var_fn,cfn_1,cfn_2,cfn_3;
algorithm 
  Debug.fprintln("cgtr", "generate_function_body_tuple");
  tnr := 1;
  ret_type_str := generate_return_type(fpath);
  (ret_decl,ret_var,tnr_ret_1) := generate_temp_decl(ret_type_str, tnr);
  ret_stmt := Util.string_append_list({"return ",ret_var,";"});
  outvars := DAE.get_output_vars(dae);
  (out_fn,tnr_ret) := generate_alloc_outvars(outvars, ret_decl, ret_var, tnr_ret_1);
  (mem_decl,mem_var,tnr_mem) := generate_temp_decl("state", tnr_ret);
  mem_stmt1 := Util.string_append_list({mem_var," = get_memory_state();"});
  mem_stmt2 := Util.string_append_list({"restore_memory_state(",mem_var,");"});
  mem_fn_1 := c_add_variables(out_fn, {mem_decl});
  mem_fn := c_add_inits(mem_fn_1, {mem_stmt1});
  (var_fn,tnr_var) := generate_vars(dae, is_var_q, tnr_mem);
  (alg_fn,tnr_alg) := generate_algorithms(dae, tnr_var);
  (res_var_fn,tnr_res) := generate_result_vars(dae, ret_var, tnr_alg);
  cfn_1 := c_merge_fn(mem_fn, var_fn);
  cfn_2 := c_merge_fn(cfn_1, alg_fn);
  cfn_3 := c_merge_fn(cfn_2, res_var_fn);
  cfn := c_add_cleanups(cfn_3, {mem_stmt2,ret_stmt});
end generate_function_body_tuple;

protected function generate_alloc_outvars
  input DAE_ElementList in_dae_elementlist1;
  input String in_string2;
  input String in_string3;
  input Integer in_integer4;
  output CFunction out_cfunction;
  output Integer out_integer;
protected 
  type DAE_ElementList = list<DAE.Element>;
algorithm 
  (out_cfunction,out_integer):=
  matchcontinue (in_dae_elementlist1,in_string2,in_string3,in_integer4)
    local
      type Exp_ExpOption = Option<Exp.Exp>;
      type Exp_SubscriptList = list<Exp.Subscript>;
      type Absyn_PathList = list<Absyn.Path>;
      type DAE_VariableAttributesOption = Option<DAE.VariableAttributes>;
      type Absyn_CommentOption = Option<Absyn.Comment>;
      CleanupStatement rv,rd;
      Integer tnr,tnr1,tnr2;
      CFunction cfn,cfn1,cfn2;
      DAE.Element var;
      Exp.ComponentRef cr;
      DAE.VarKind vk;
      DAE.VarDirection vd;
      DAE.Type t;
      Exp_ExpOption e,start;
      Exp_SubscriptList id;
      DAE.Flow flow_;
      Absyn_PathList class_;
      DAE_VariableAttributesOption dae_var_attr;
      Absyn_CommentOption comment;
      DAE_ElementList r;
    case ({},"",rv,tnr) then (c_empty_function,tnr); 
    case ({},rd,rv,tnr)
      equation 
        cfn = c_add_variables(c_empty_function, {rd}); then (cfn,tnr);
    case ((var = DAE.VAR(cr,vk,vd,t,e,id,start,flow_,class_,dae_var_attr,comment)) :: r,rd,rv,tnr)
      equation 
        (cfn1,tnr1) = generate_alloc_outvar(var, rv, tnr);
        (cfn2,tnr2) = generate_alloc_outvars(r, rd, rv, tnr1);
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,tnr2);
    case (_ :: r,rd,rv,tnr)
      equation 
        (cfn2,tnr2) = generate_alloc_outvars(r, rd, rv, tnr); then (cfn2,tnr2);
  end matchcontinue;
end generate_alloc_outvars;

protected function generate_alloc_outvar
  input DAE.Element in_element;
  input String in_string;
  input Integer in_integer;
  output CFunction out_cfunction;
  output Integer out_integer;
algorithm 
  (out_cfunction,out_integer):=
  matchcontinue (in_element,in_string,in_integer)
    local
      type Exp_ExpOption = Option<Exp.Exp>;
      type Exp_SubscriptList = list<Exp.Subscript>;
      type Absyn_PathList = list<Absyn.Path>;
      type DAE_VariableAttributesOption = Option<DAE.VariableAttributes>;
      type Absyn_CommentOption = Option<Absyn.Comment>;
      Boolean is_a,emptypre;
      CleanupStatement typ_str,cref_str1,cref_str2,cref_str,ndims_str,dims_str,alloc_str,prefix;
      CFunction cfn1,cfn1_1,cfn_1,cfn;
      CleanupStatementList dim_strs;
      Integer tnr1,ndims,tnr;
      DAE.Element var;
      Exp.ComponentRef id;
      DAE.VarKind vk;
      DAE.VarDirection vd;
      DAE.Type typ;
      Exp_ExpOption e,start;
      Exp_SubscriptList inst_dims;
      DAE.Flow flow_;
      Absyn_PathList class_;
      DAE_VariableAttributesOption dae_var_attr;
      Absyn_CommentOption comment;
    case ((var = DAE.VAR(id,vk,vd,typ,e,inst_dims,start,flow_,class_,dae_var_attr,comment)),prefix,tnr)
      equation 
        is_a = is_array(var);
        typ_str = dae_type_str(typ, is_a);
        (cref_str1,_) = comp_ref_cstr(id);
        cref_str2 = Util.string_append_list({prefix,".",cref_str1});
        emptypre = Util.is_empty_string(prefix);
        cref_str = Util.if_(emptypre, cref_str1, cref_str2);
        (cfn1,dim_strs,tnr1) = generate_size_subscripts(cref_str, inst_dims, tnr);
        cfn1_1 = c_move_statements_to_inits(cfn1);
        ndims = list_length(dim_strs);
        ndims_str = int_string(ndims);
        dims_str = Util.string_delimit_list(dim_strs, ", ");
        alloc_str = Util.string_append_list(
          {"alloc_",typ_str,"(&",cref_str,", ",ndims_str,", ",
          dims_str,");"});
        cfn_1 = c_add_inits(cfn1_1, {alloc_str});
        cfn = Util.if_(is_a, cfn_1, cfn1_1); then (cfn,tnr1);
    case (e,_,tnr)
      local DAE.Element e;
      equation 
        failure(DAE.is_var(e)); then (c_empty_function,tnr);
  end matchcontinue;
end generate_alloc_outvar;

protected function generate_alloc_outvars_ext
  input DAE_ElementList in_dae_elementlist;
  input String in_string;
  input Integer in_integer;
  input DAE.ExternalDecl in_externaldecl;
  output CFunction out_cfunction;
  output Integer out_integer;
protected 
  type DAE_ElementList = list<DAE.Element>;
algorithm 
  (out_cfunction,out_integer):=
  matchcontinue (in_dae_elementlist,in_string,in_integer,in_externaldecl)
    local
      type Exp_ExpOption = Option<Exp.Exp>;
      type Exp_SubscriptList = list<Exp.Subscript>;
      type Absyn_PathList = list<Absyn.Path>;
      type DAE_VariableAttributesOption = Option<DAE.VariableAttributes>;
      type Absyn_CommentOption = Option<Absyn.Comment>;
      CleanupStatement rv;
      Integer tnr,tnr1,tnr2;
      DAE.ExternalDecl extdecl;
      CFunction cfn1,cfn2,cfn;
      DAE.Element var;
      Exp.ComponentRef cr;
      DAE.VarKind vk;
      DAE.VarDirection vd;
      DAE.Type t;
      Exp_ExpOption e,start;
      Exp_SubscriptList id;
      DAE.Flow flow_;
      Absyn_PathList class_;
      DAE_VariableAttributesOption dae_var_attr;
      Absyn_CommentOption comment;
      DAE_ElementList r;
    case ({},rv,tnr,extdecl) then (c_empty_function,tnr); 
    case ((var = DAE.VAR(cr,vk,vd,t,e,id,start,flow_,class_,dae_var_attr,comment)) :: r,rv,tnr,extdecl)
      equation 
        DAE.EXTERNALDECL(_,_,_,"C") = extdecl;
        (cfn1,tnr1) = generate_alloc_outvar(var, rv, tnr);
        (cfn2,tnr2) = generate_alloc_outvars_ext(r, rv, tnr1, extdecl);
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,tnr2);
    case ((var = DAE.VAR(cr,vk,vd,t,e,id,start,flow_,class_,dae_var_attr,comment)) :: r,rv,tnr,extdecl)
      equation 
        DAE.EXTERNALDECL(_,_,_,"FORTRAN 77") = extdecl;
        (cfn1,tnr1) = generate_alloc_outvar_f77(var, rv, tnr);
        (cfn2,tnr2) = generate_alloc_outvars_ext(r, rv, tnr1, extdecl);
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,tnr2);
    case (_ :: r,rv,tnr,extdecl)
      equation 
        (cfn2,tnr2) = generate_alloc_outvars_ext(r, rv, tnr, extdecl); then (cfn2,tnr2);
  end matchcontinue;
end generate_alloc_outvars_ext;

protected function generate_alloc_outvar_f77
  input DAE.Element in_element;
  input String in_string;
  input Integer in_integer;
  output CFunction out_cfunction;
  output Integer out_integer;
algorithm 
  (out_cfunction,out_integer):=
  matchcontinue (in_element,in_string,in_integer)
    local
      type Exp_ExpOption = Option<Exp.Exp>;
      type Exp_SubscriptList = list<Exp.Subscript>;
      type Absyn_PathList = list<Absyn.Path>;
      type DAE_VariableAttributesOption = Option<DAE.VariableAttributes>;
      type Absyn_CommentOption = Option<Absyn.Comment>;
      Boolean is_a,emptypre;
      CleanupStatement typ_str,cref_str1,cref_str2,cref_str,ndims_str,dims_str,alloc_str,prefix;
      CFunction cfn1,cfn1_1,cfn_1,cfn;
      CleanupStatementList dim_strs;
      Integer tnr1,ndims,tnr;
      DAE.Element var;
      Exp.ComponentRef id;
      DAE.VarKind vk;
      DAE.VarDirection vd;
      DAE.Type typ;
      Exp_ExpOption e,start;
      Exp_SubscriptList inst_dims;
      DAE.Flow flow_;
      Absyn_PathList class_;
      DAE_VariableAttributesOption dae_var_attr;
      Absyn_CommentOption comment;
    case ((var = DAE.VAR(id,vk,vd,typ,e,inst_dims,start,flow_,class_,dae_var_attr,comment)),prefix,tnr)
      equation 
        is_a = is_array(var);
        typ_str = dae_type_str(typ, is_a);
        emptypre = Util.string_equal(prefix, "");
        (cref_str1,_) = comp_ref_cstr(id);
        cref_str2 = Util.string_append_list({prefix,".",cref_str1});
        cref_str = Util.if_(emptypre, cref_str1, cref_str2);
        (cfn1,dim_strs,tnr1) = generate_size_subscripts(cref_str, inst_dims, tnr) "	list_reverse inst_dims => inst_dims' & ";
        cfn1_1 = c_move_statements_to_inits(cfn1);
        ndims = list_length(dim_strs);
        ndims_str = int_string(ndims);
        dims_str = Util.string_delimit_list(dim_strs, ", ");
        alloc_str = Util.string_append_list(
          {"alloc_",typ_str,"(&",cref_str,", ",ndims_str,", ",
          dims_str,");"});
        cfn_1 = c_add_inits(cfn1_1, {alloc_str});
        cfn = Util.if_(is_a, cfn_1, cfn1_1); then (cfn,tnr1);
    case (e,_,tnr)
      local DAE.Element e;
      equation 
        failure(DAE.is_var(e)); then (c_empty_function,tnr);
  end matchcontinue;
end generate_alloc_outvar_f77;

protected function generate_size_subscripts
  input String in_string;
  input Exp_SubscriptList in_exp_subscriptlist;
  input Integer in_integer;
  output CFunction out_cfunction;
  output StringList out_stringlist;
  output Integer out_integer;
protected 
  type Exp_SubscriptList = list<Exp.Subscript>;
  type StringList = list<String>;
algorithm 
  (out_cfunction,out_stringlist,out_integer):=
  matchcontinue (in_string,in_exp_subscriptlist,in_integer)
    local
      Integer tnr,tnr1,tnr2;
      CFunction cfn1,cfn2,cfn;
      CleanupStatement var1,id,s;
      CleanupStatementList vars2;
      Exp.Exp e;
      Exp_SubscriptList r,subs;
    case (_,{},tnr) then (c_empty_function,{},tnr); 
    case (id,Exp.INDEX(e) :: r,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e, tnr);
        (cfn2,vars2,tnr2) = generate_size_subscripts(id, r, tnr1);
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var1 :: vars2,tnr2);
    case (id,subs,_) "
  rule	generate_temp_decl(\"WHOLEDIM\",tnr) => (decl,var1,tnr1) &
	c_add_variables (c_empty_function, {decl}) => cfn1 &
	generate_size_subscripts (r,tnr1) => (cfn2,vars2,tnr2) &
	c_merge_fn(cfn1,cfn2) => cfn
	----------------------------
	generate_size_subscripts (Exp.WHOLEDIM::r,tnr)
	  => (cfn,var1::vars2,tnr2)
"
      equation 
        Print.print_buf("# generate_size_subscripts failed\n");
        s = Util.string_append_list({" - ",id,"["});
        Print.print_buf(s);
        Dump.print_list(subs, Exp.print_subscript, ", ");
        Print.print_buf("]\n"); then fail();
  end matchcontinue;
end generate_size_subscripts;

protected function generate_alloc_array_f77
  input String in_string;
  input Types.Type in_type;
  output CFunction out_cfunction;
algorithm 
  out_cfunction:=
  matchcontinue (in_string,in_type)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type IntegerList = list<Integer>;
      type Exp_SubscriptList = list<Exp.Subscript>;
      TupleTypes_TTypeAbsyn_PathOption elty,ty;
      IntegerList dims;
      Exp_SubscriptList dimsubs;
      Integer tnr,tnr1,ndims;
      CFunction cfn1,cfn1_1,cfn;
      CleanupStatementList dim_strs;
      CleanupStatement typ_str,ndims_str,dims_str,alloc_str,crefstr;
    case (crefstr,ty)
      equation 
        true = Types.is_array(ty);
        (elty,dims) = Types.flatten_array_type(ty);
        dimsubs = Exp.int_subscripts(dims);
        tnr = tick();
        (cfn1,dim_strs,tnr1) = generate_size_subscripts(crefstr, dimsubs, tnr);
        cfn1_1 = c_move_statements_to_inits(cfn1);
        typ_str = generate_type(ty);
        ndims = list_length(dim_strs);
        ndims_str = int_string(ndims);
        dims_str = Util.string_delimit_list(dim_strs, ", ");
        alloc_str = Util.string_append_list(
          {"alloc_",typ_str,"(&",crefstr,", ",ndims_str,", ",dims_str,
          ");"});
        cfn = c_add_inits(cfn1_1, {alloc_str}); then cfn;
    case (_,_)
      equation 
        Print.print_buf("#-- generate_alloc_array_f77 failed\n"); then fail();
  end matchcontinue;
end generate_alloc_array_f77;

protected function prefix_cr
  input String in_string;
  input Exp.ComponentRef in_componentref;
  output Exp.ComponentRef out_componentref;
algorithm 
  out_componentref:=
  matchcontinue (in_string,in_componentref)
    local
      CleanupStatement prf;
      Exp.ComponentRef cref;
    case (prf,cref) then Exp.CREF_QUAL(prf,{},cref); 
  end matchcontinue;
end prefix_cr;

protected function generate_algorithms
  input DAE_ElementList in_dae_elementlist;
  input Integer in_integer;
  output CFunction out_cfunction;
  output Integer out_integer;
protected 
  type DAE_ElementList = list<DAE.Element>;
algorithm 
  (out_cfunction,out_integer):=
  matchcontinue (in_dae_elementlist,in_integer)
    local
      DAE_ElementList algs,els;
      CFunction cfn;
      Integer tnr_1,tnr;
    case (els,tnr)
      equation 
        algs = DAE.get_matching(els, DAE.is_algorithm);
        (cfn,tnr_1) = generate_algorithms2(algs, tnr); then (cfn,tnr_1);
  end matchcontinue;
end generate_algorithms;

protected function generate_algorithms2
  input DAE_ElementList in_dae_elementlist;
  input Integer in_integer;
  output CFunction out_cfunction;
  output Integer out_integer;
protected 
  type DAE_ElementList = list<DAE.Element>;
algorithm 
  (out_cfunction,out_integer):=
  matchcontinue (in_dae_elementlist,in_integer)
    local
      Integer tnr,tnr1,tnr2;
      CFunction cfn1,cfn2,cfn;
      DAE.Element first;
      DAE_ElementList rest;
    case ({},tnr) then (c_empty_function,tnr); 
    case (first :: rest,tnr)
      equation 
        (cfn1,tnr1) = generate_algorithm(first, tnr);
        (cfn2,tnr2) = generate_algorithms2(rest, tnr1);
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,tnr2);
  end matchcontinue;
end generate_algorithms2;

public function generate_algorithm
  input DAE.Element in_element;
  input Integer in_integer;
  output CFunction out_cfunction;
  output Integer out_integer;
algorithm 
  (out_cfunction,out_integer):=
  matchcontinue (in_element,in_integer)
    local
      type Algorithm_StatementList = list<Algorithm.Statement>;
      CFunction cfn;
      Integer tnr_1,tnr;
      Algorithm_StatementList stmts;
    case (DAE.ALGORITHM(Algorithm.ALGORITHM(stmts)),tnr)
      equation 
        (cfn,tnr_1) = generate_algorithm_statements(stmts, tnr); then (cfn,tnr_1);
    case (_,_)
      equation 
        Print.print_buf("# generate_algorithm failed\n"); then fail();
  end matchcontinue;
end generate_algorithm;

protected function generate_algorithm_statements
  input Algorithm_StatementList in_algorithm_statementlist;
  input Integer in_integer;
  output CFunction out_cfunction;
  output Integer out_integer;
protected 
  type Algorithm_StatementList = list<Algorithm.Statement>;
algorithm 
  (out_cfunction,out_integer):=
  matchcontinue (in_algorithm_statementlist,in_integer)
    local
      Integer tnr,tnr1,tnr2;
      CFunction cfn1,cfn2,cfn;
      Algorithm.Statement f;
      Algorithm_StatementList r;
    case ({},tnr) then (c_empty_function,tnr); 
    case (f :: r,tnr)
      equation 
        (cfn1,tnr1) = generate_algorithm_statement(f, tnr);
        (cfn2,tnr2) = generate_algorithm_statements(r, tnr1);
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,tnr2);
  end matchcontinue;
end generate_algorithm_statements;

protected function generate_algorithm_statement
  input Algorithm.Statement in_statement;
  input Integer in_integer;
  output CFunction out_cfunction;
  output Integer out_integer;
algorithm 
  (out_cfunction,out_integer):=
  matchcontinue (in_statement,in_integer)
    local
      type Exp_SubscriptList = list<Exp.Subscript>;
      type Algorithm_StatementList = list<Algorithm.Statement>;
      CFunction cfn1,cfn2,cfn_1,cfn,cfn2_1,cfn1_1,cfn3,cfn4,cfn3_1,cfn3_2,cfn4_1;
      CleanupStatement var1,var2,stmt,cref_str,type_str,if_begin,sdecl,svar,dvar,ident_type_str,short_type,rdecl1,rvar1,rdecl2,rvar2,rdecl3,rvar3,e1var,e2var,e3var,r_stmt,for_begin,def_beg1,mem_begin,mem_end,for_end,def_end1,i,tdecl,tvar,idecl,ivar,array_type_str,evar,stmt_array,stmt_scalar,crit_stmt;
      Integer tnr1,tnr2,tnr,tnr3,tnr_1,tnr2_2,tnr2_3,tnr4,tnr_2;
      Exp.Type typ,t;
      Exp.ComponentRef cref;
      Exp.Exp exp,e;
      Exp_SubscriptList subs;
      Algorithm_StatementList then_,stmts;
      Algorithm.Else else_;
      Boolean a;
    case (Algorithm.ASSIGN(typ,cref,exp),tnr)
      equation 
        Debug.fprintln("cgas", "generate_algorithm_statement");
        (cfn1,var1,tnr1) = generate_expression(exp, tnr);
        (cfn2,var2,tnr2) = generate_scalar_lhs_cref(typ, cref, tnr1);
        stmt = Util.string_append_list({var2," = ",var1,";"});
        cfn_1 = c_merge_fn(cfn1, cfn2);
        cfn = c_add_statements(cfn_1, {stmt}); then (cfn,tnr2);
    case (Algorithm.ASSIGN_ARR(typ,cref,exp),tnr)
      equation 
        (cref_str,{}) = comp_ref_cstr(cref);
        (cfn1,var1,tnr1) = generate_expression(exp, tnr);
        type_str = exp_type_str(typ, true);
        stmt = Util.string_append_list({"copy_",type_str,"_data(&",var1,", &",cref_str,");"});
        cfn2 = c_add_statements(cfn1, {stmt}); then (cfn2,tnr1);
    case (Algorithm.ASSIGN_ARR(typ,cref,exp),tnr)
      equation 
        (cref_str, subs = _ :: _) = comp_ref_cstr(cref);
        (cfn1,var1,tnr1) = generate_expression(exp, tnr);
        (cfn2,var2,tnr2) = generate_index_spec(subs, tnr1);
        type_str = exp_type_str(typ, true);
        stmt = Util.string_append_list(
          {"indexed_assign_",type_str,"(&",var1,", &",cref_str,", &",
          var2,");"});
        cfn2_1 = c_add_statements(cfn2, {stmt});
        cfn = c_merge_fn(cfn1, cfn2_1); then (cfn,tnr2);
    case (Algorithm.IF(e,then_,else_),tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e, tnr);
        if_begin = Util.string_append_list({"if (",var1,") {"});
        cfn1_1 = c_add_statements(cfn1, {if_begin});
        (cfn2,tnr2) = generate_algorithm_statements(then_, tnr1);
        cfn2_1 = c_add_statements(cfn2, {"}"});
        (cfn3,tnr3) = generate_else(else_, tnr2);
        cfn = c_merge_fns({cfn1_1,cfn2_1,cfn3}); then (cfn,tnr3);
    case (Algorithm.FOR(t,a,i,e,stmts),tnr)
      equation 
        true = Exp.is_range(e);
        (sdecl,svar,tnr_1) = generate_temp_decl("state", tnr);
        (_,dvar,tnr1) = generate_temp_decl("", tnr_1);
        ident_type_str = exp_type_str(t, a);
        short_type = exp_short_type_str(t);
        (rdecl1,rvar1,tnr2_2) = generate_temp_decl(ident_type_str, tnr1);
        (rdecl2,rvar2,tnr2_3) = generate_temp_decl(ident_type_str, tnr2_2);
        (rdecl3,rvar3,tnr2) = generate_temp_decl(ident_type_str, tnr2_3);
        (cfn3,e1var,e2var,e3var,tnr3) = generate_range_expressions(e, tnr2);
        r_stmt = Util.string_append_list(
          {rvar1," = ",e1var,"; ",rvar2," = ",e2var,"; ",rvar3," = ",
          e3var,";"});
        for_begin = Util.string_append_list(
          {"for (",i," = ",rvar1,"; ","in_range_",short_type,"(",i,
          ", ",rvar1,", ",rvar3,"); ",i," += ",rvar2,") {"});
        def_beg1 = Util.string_append_list({"{\n  ",ident_type_str," ",i,";\n"});
        mem_begin = Util.string_append_list({svar," = get_memory_state();"});
        (cfn4,tnr4) = generate_algorithm_statements(stmts, tnr3);
        mem_end = Util.string_append_list({"restore_memory_state(",svar,");"});
        for_end = "}";
        def_end1 = "} /* end for*/\n";
        cfn3_1 = c_add_variables(cfn3, {sdecl,rdecl1,rdecl2,rdecl3});
        cfn3_2 = c_add_statements(cfn3_1, {r_stmt,def_beg1,for_begin,mem_begin});
        cfn4_1 = c_add_statements(cfn4, {mem_end,for_end,def_end1});
        cfn = c_merge_fns({cfn3_2,cfn4_1}); then (cfn,tnr4);
    case (Algorithm.FOR(t,a,i,e,stmts),tnr)
      equation 
        (sdecl,svar,tnr_1) = generate_temp_decl("state", tnr);
        (_,dvar,tnr_2) = generate_temp_decl("", tnr_1);
        (tdecl,tvar,tnr1) = generate_temp_decl("int", tnr_2);
        ident_type_str = exp_type_str(t, a);
        (idecl,ivar,tnr2) = generate_temp_decl(ident_type_str, tnr1);
        array_type_str = exp_type_str(t, true);
        (cfn3,evar,tnr3) = generate_expression(e, tnr2);
        for_begin = Util.string_append_list(
          {"for (",tvar," = 0; ",tvar," < size_of_dimension_",
          array_type_str,"(",evar,", 1); ","++",tvar,") {"});
        def_beg1 = Util.string_append_list({"{\n  ",ident_type_str," ",i,";\n"});
        mem_begin = Util.string_append_list({svar," = get_memory_state();"});
        stmt_array = Util.string_append_list(
          {"simple_index_alloc_",ident_type_str,"1(&",evar,", ",tvar,
          ", &",ivar,"));"});
        stmt_scalar = Util.string_append_list(
          {i," = *(",array_type_str,"_element_addr1(&",evar,", 1, ",
          tvar,"));"});
        stmt = Util.if_(a, stmt_array, stmt_scalar) " Use fast implementation for 1 dim ";
        (cfn4,tnr4) = generate_algorithm_statements(stmts, tnr3);
        mem_end = Util.string_append_list({"restore_memory_state(",svar,");"});
        for_end = "}";
        def_end1 = "} /* end for*/\n";
        cfn3_1 = c_add_variables(cfn3, {sdecl,tdecl,idecl});
        cfn3_2 = c_add_statements(cfn3_1, {def_beg1,for_begin,mem_begin,stmt});
        cfn4_1 = c_add_statements(cfn4, {mem_end,for_end,def_end1});
        cfn = c_merge_fns({cfn3_2,cfn4_1}); then (cfn,tnr4);
    case (Algorithm.WHILE(e,stmts),tnr)
      equation 
        cfn1 = c_add_statements(c_empty_function, {"while (1) {"});
        (cfn2,var2,tnr2) = generate_expression(e, tnr);
        crit_stmt = Util.string_append_list({"if (!",var2,") break;"});
        cfn2_1 = c_add_statements(cfn2, {crit_stmt});
        (cfn3,tnr3) = generate_algorithm_statements(stmts, tnr2);
        cfn3_1 = c_add_statements(cfn3, {"}"});
        cfn = c_merge_fns({cfn1,cfn2_1,cfn3_1}); then (cfn,tnr3);
    case (Algorithm.WHEN(_,_),_)
      equation 
        Print.print_buf("# when statement not implemented\n"); then fail();
    case (_,_)
      equation 
        Print.print_buf("# generate_algorithm_statement failed\n"); then fail();
  end matchcontinue;
end generate_algorithm_statement;

protected function generate_range_expressions
  input Exp.Exp in_exp;
  input Integer in_integer;
  output CFunction out_cfunction1;
  output String out_string2;
  output String out_string3;
  output String out_string4;
  output Integer out_integer5;
algorithm 
  (out_cfunction1,out_string2,out_string3,out_string4,out_integer5):=
  matchcontinue (in_exp,in_integer)
    local
      CFunction cfn1,cfn3,cfn,cfn2;
      CleanupStatement var1,var2,var3;
      Integer tnr1,tnr3,tnr,tnr2;
      Exp.Type t;
      Exp.Exp e1,e3,e2;
    case (Exp.RANGE(t,e1,NONE,e3),tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        var2 = "(1)";
        (cfn3,var3,tnr3) = generate_expression(e3, tnr1);
        cfn = c_merge_fn(cfn1, cfn3); then (cfn,var1,var2,var3,tnr3);
    case (Exp.RANGE(t,e1,SOME(e2),e3),tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        (cfn3,var3,tnr3) = generate_expression(e3, tnr2);
        cfn = c_merge_fns({cfn1,cfn2,cfn3}); then (cfn,var1,var2,var3,tnr3);
    case (_,_)
      equation 
        Print.print_buf("# generate_range_expressions failed\n"); then fail();
  end matchcontinue;
end generate_range_expressions;

protected function generate_else
  input Algorithm.Else in_else;
  input Integer in_integer;
  output CFunction out_cfunction;
  output Integer out_integer;
algorithm 
  (out_cfunction,out_integer):=
  matchcontinue (in_else,in_integer)
    local
      type Algorithm_StatementList = list<Algorithm.Statement>;
      Integer tnr,tnr2,tnr3,tnr4;
      CFunction cfn1,cfn2,cfn2_1,cfn3,cfn3_1,cfn4,cfn4_1,cfn;
      CleanupStatement var2,if_begin;
      Exp.Exp e;
      Algorithm_StatementList stmts;
      Algorithm.Else else_;
    case (Algorithm.NOELSE,tnr) then (c_empty_function,tnr); 
    case (Algorithm.ELSEIF(e,stmts,else_),tnr)
      equation 
        cfn1 = c_add_statements(c_empty_function, {"else {"});
        (cfn2,var2,tnr2) = generate_expression(e, tnr);
        if_begin = Util.string_append_list({"if (",var2,") {"});
        cfn2_1 = c_add_statements(cfn2, {if_begin});
        (cfn3,tnr3) = generate_algorithm_statements(stmts, tnr2);
        cfn3_1 = c_add_statements(cfn3, {"}"});
        (cfn4,tnr4) = generate_else(else_, tnr3);
        cfn4_1 = c_add_statements(cfn4, {"}"});
        cfn = c_merge_fns({cfn1,cfn2_1,cfn3_1,cfn4_1}); then (cfn,tnr4);
    case (Algorithm.ELSE(stmts),tnr)
      equation 
        cfn1 = c_add_statements(c_empty_function, {"else {"});
        (cfn2,tnr2) = generate_algorithm_statements(stmts, tnr);
        cfn2_1 = c_add_statements(cfn2, {"}"});
        cfn = c_merge_fn(cfn1, cfn2_1); then (cfn,tnr);
    case (_,_)
      equation 
        Print.print_buf("# generate_else failed\n"); then fail();
  end matchcontinue;
end generate_else;

protected function generate_vars
  input DAE_ElementList in_dae_elementlist;
  input FuncType1 in_functype1;
  input Integer in_integer;
  output CFunction out_cfunction;
  output Integer out_integer;
protected 
  type DAE_ElementList = list<DAE.Element>;
  function FuncType1
    input DAE.Element in_element;
  end FuncType1;
algorithm 
  (out_cfunction,out_integer):=
  matchcontinue (in_dae_elementlist,in_functype1,in_integer)
    local
      Integer tnr,tnr1,tnr2;
      CFunction cfn1,cfn2,cfn;
      DAE.Element first;
      DAE_ElementList rest;
      FuncType1 verify;
    case ({},_,tnr) then (c_empty_function,tnr); 
    case (first :: rest,verify,tnr)
      equation 
        verify(first);
        (cfn1,tnr1) = generate_var(first, tnr);
        (cfn2,tnr2) = generate_vars(rest, verify, tnr1);
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,tnr2);
    case (first :: rest,verify,tnr)
      equation 
        failure(verify(first));
        (cfn,tnr2) = generate_vars(rest, verify, tnr); then (cfn,tnr2);
    case (_,_,_)
      equation 
        Print.print_buf("# generate_vars failed\n"); then fail();
  end matchcontinue;
end generate_vars;

protected function generate_var_decls
  input DAE_ElementList in_dae_elementlist;
  input FuncType1 in_functype1;
  input Integer in_integer;
  output CFunction out_cfunction;
  output Integer out_integer;
protected 
  type DAE_ElementList = list<DAE.Element>;
  function FuncType1
    input DAE.Element in_element;
  end FuncType1;
algorithm 
  (out_cfunction,out_integer):=
  matchcontinue (in_dae_elementlist,in_functype1,in_integer)
    local
      Integer tnr,tnr1,tnr2;
      CFunction cfn1,cfn2,cfn;
      DAE.Element first;
      DAE_ElementList rest;
      FuncType1 verify;
    case ({},_,tnr) then (c_empty_function,tnr); 
    case (first :: rest,verify,tnr)
      equation 
        verify(first);
        (cfn1,tnr1) = generate_var_decl(first, tnr);
        (cfn2,tnr2) = generate_var_decls(rest, verify, tnr1);
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,tnr2);
    case (first :: rest,verify,tnr)
      equation 
        failure(verify(first));
        (cfn,tnr2) = generate_var_decls(rest, verify, tnr); then (cfn,tnr2);
    case (_,_,_)
      equation 
        Print.print_buf("# generate_var_decls failed\n"); then fail();
  end matchcontinue;
end generate_var_decls;

protected function generate_var_inits
  input DAE_ElementList in_dae_elementlist;
  input FuncType1 in_functype1;
  input Integer in_integer;
  input String in_string;
  output CFunction out_cfunction;
  output Integer out_integer;
protected 
  type DAE_ElementList = list<DAE.Element>;
  function FuncType1
    input DAE.Element in_element;
  end FuncType1;
algorithm 
  (out_cfunction,out_integer):=
  matchcontinue (in_dae_elementlist,in_functype1,in_integer,in_string)
    local
      Integer tnr,tnr1,tnr2;
      CFunction cfn1,cfn2,cfn;
      DAE.Element first;
      DAE_ElementList rest;
      FuncType1 verify;
      CleanupStatement pre;
    case ({},_,tnr,_) then (c_empty_function,tnr); 
    case (first :: rest,verify,tnr,pre)
      equation 
        verify(first);
        (cfn1,tnr1) = generate_var_init(first, tnr, pre);
        (cfn2,tnr2) = generate_var_inits(rest, verify, tnr1, pre);
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,tnr2);
    case (first :: rest,verify,tnr,pre)
      equation 
        failure(verify(first));
        (cfn,tnr2) = generate_var_inits(rest, verify, tnr, pre); then (cfn,tnr2);
    case (_,_,_,_)
      equation 
        Print.print_buf("# generate_var_inits failed\n"); then fail();
  end matchcontinue;
end generate_var_inits;

protected function generate_var "
relation generate_var_init : (DAE.Element, int) => (CFunction, int) =

  rule	is_array var => is_a &
	dae_type_str (typ, is_a) => typ_str &
	comp_ref_cstr id => (id_str,_) &
	Exp.print_exp_str e => expstr & 
	Util.string_append_list ({\"copy_\", typ_str,
				  \"_data(\", id_str, \",\", expstr, \");\"
				  }) => init_str &
	c_add_statements(c_empty_function,{init_str}) => cfn
	-----------
	generate_var_init    (var as DAE.VAR(id, 
					     DAE.VARIABLE, 
					     DAE.OUTPUT, 
					     typ, 
					     SOME(e),
					     _,
					     _,start,flow,class,dae_var_attr,comment),tnr)
	  => (cfn, tnr)

  axiom	generate_var_init (_, tnr) => (c_empty_function, tnr)

end
"
  input DAE.Element in_element;
  input Integer in_integer;
  output CFunction out_cfunction;
  output Integer out_integer;
algorithm 
  (out_cfunction,out_integer):=
  matchcontinue (in_element,in_integer)
    local
      type Exp_SubscriptList = list<Exp.Subscript>;
      type Exp_ExpOption = Option<Exp.Exp>;
      type Absyn_PathList = list<Absyn.Path>;
      type DAE_VariableAttributesOption = Option<DAE.VariableAttributes>;
      type Absyn_CommentOption = Option<Absyn.Comment>;
      Boolean is_a;
      CleanupStatement typ_str,cref_str,dimvars_str,dims_str,dim_comment,dim_comment_1,ndims_str,decl_str,alloc_str,init_stmt;
      CFunction cfn1_1,cfn1,cfn_1,cfn_2,cfn;
      CleanupStatementList vars1,dim_strs;
      Integer tnr1,ndims,tnr;
      DAE.Element var;
      Exp.ComponentRef id;
      DAE.VarKind vk;
      DAE.VarDirection vd;
      DAE.Type typ;
      Exp_SubscriptList inst_dims;
      Exp_ExpOption start;
      DAE.Flow flow_;
      Absyn_PathList class_;
      DAE_VariableAttributesOption dae_var_attr;
      Absyn_CommentOption comment;
      Exp.Exp e;
    case ((var = DAE.VAR(id,vk,vd,typ,NONE,inst_dims,start,flow_,class_,dae_var_attr,comment)),tnr) "
 parameters are put in the params struct

  rule	generate_var_q2 (vd) &
	indent 2 &
	DAE.dump_type typ &
	Exp.print_component_ref id &
	Print.print_buf \" = 0;  /* Warning: without value, set to zero */\n\"
	-----------
	generate_var(DAE.VAR(id, DAE.PARAM, vd, typ, NONE,_,class,dae_var_attr,comment))
 
  rule	is_first_in_array id => false
	-----------------------------
	generate_var(DAE.VAR(id,_,_,_,_,_,_,_,_,_),tnr) => (c_empty_function,tnr)
"
      equation 
        is_a = is_array(var);
        typ_str = dae_type_str(typ, is_a);
        (cref_str,_) = comp_ref_cstr(id);
        (cfn1_1,vars1,tnr1) = generate_size_subscripts(cref_str, inst_dims, tnr);
        cfn1 = c_move_statements_to_inits(cfn1_1);
        dimvars_str = Util.string_delimit_list(vars1, ", ");
        dim_strs = Util.list_map(inst_dims, dim_string);
        dims_str = Util.string_delimit_list(dim_strs, ", ");
        dim_comment = Util.string_append_list({" /* [",dims_str,"] */"});
        dim_comment_1 = Util.if_(is_a, dim_comment, "");
        ndims = list_length(dim_strs);
        ndims_str = int_string(ndims);
        decl_str = Util.string_append_list({typ_str," ",cref_str,";",dim_comment_1});
        alloc_str = Util.string_append_list(
          {"alloc_",typ_str,"(&",cref_str,", ",ndims_str,", ",
          dimvars_str,");"});
        cfn_1 = c_add_variables(cfn1, {decl_str});
        cfn_2 = c_add_inits(cfn_1, {alloc_str});
        cfn = Util.if_(is_a, cfn_2, cfn_1); then (cfn,tnr1);
    case ((var = DAE.VAR(id,vk,vd,typ,SOME(e),inst_dims,start,flow_,class_,dae_var_attr,comment)),tnr)
      equation 
        is_a = is_array(var);
        typ_str = dae_type_str(typ, is_a);
        (cref_str,_) = comp_ref_cstr(id);
        (cfn1_1,vars1,tnr1) = generate_size_subscripts(cref_str, inst_dims, tnr);
        cfn1 = c_move_statements_to_inits(cfn1_1);
        dimvars_str = Util.string_delimit_list(vars1, ", ");
        dim_strs = Util.list_map(inst_dims, dim_string);
        dims_str = Util.string_delimit_list(dim_strs, ", ");
        dim_comment = Util.string_append_list({" /* [",dims_str,"] */"});
        dim_comment_1 = Util.if_(is_a, dim_comment, "");
        ndims = list_length(dim_strs);
        ndims_str = int_string(ndims);
        decl_str = Util.string_append_list({typ_str," ",cref_str,";",dim_comment_1});
        alloc_str = Util.string_append_list(
          {"alloc_",typ_str,"(&",cref_str,", ",ndims_str,", ",
          dimvars_str,");"});
        cfn_1 = c_add_variables(cfn1, {decl_str});
        cfn_2 = c_add_inits(cfn_1, {alloc_str});
        cfn = Util.if_(is_a, cfn_2, cfn_1);
        Print.print_buf("# default value not implemented yet: ");
        Exp.print_exp(e);
        Print.print_buf("\n"); then (cfn,tnr1);
    case ((var = DAE.VAR(id,vk,vd,typ,SOME(e),inst_dims,start,flow_,class_,dae_var_attr,comment)),tnr)
      local CleanupStatement var;
      equation 
        is_a = is_array(var);
        typ_str = dae_type_str(typ, is_a);
        (cref_str,_) = comp_ref_cstr(id);
        decl_str = Util.string_append_list({typ_str," ",cref_str,";"});
        (cfn,var,tnr1) = generate_expression(e, tnr);
        cfn_1 = c_add_variables(cfn, {decl_str});
        init_stmt = Util.string_append_list({cref_str," = ",var,";"});
        cfn_2 = c_add_inits(cfn_1, {init_stmt});
        Print.print_buf("# default value not implemented yet: ");
        Exp.print_exp(e);
        Print.print_buf("\n"); then (cfn_2,tnr1);
    case (e,_)
      local DAE.Element e;
      equation 
        Print.print_buf("# generate_var failed\n  ");
        DAE.dump_debug_element(e);
        Print.print_buf("\n"); then fail();
  end matchcontinue;
end generate_var;

protected function generate_var_decl
  input DAE.Element in_element;
  input Integer in_integer;
  output CFunction out_cfunction;
  output Integer out_integer;
algorithm 
  (out_cfunction,out_integer):=
  matchcontinue (in_element,in_integer)
    local
      type Exp_SubscriptList = list<Exp.Subscript>;
      type Exp_ExpOption = Option<Exp.Exp>;
      type Absyn_PathList = list<Absyn.Path>;
      type DAE_VariableAttributesOption = Option<DAE.VariableAttributes>;
      type Absyn_CommentOption = Option<Absyn.Comment>;
      Boolean is_a;
      CleanupStatement typ_str,cref_str,dims_str,dim_comment,dim_comment_1,ndims_str,decl_str;
      CleanupStatementList dim_strs;
      Integer ndims,tnr,tnr1;
      CFunction cfn;
      DAE.Element var;
      Exp.ComponentRef id;
      DAE.VarKind vk;
      DAE.VarDirection vd;
      DAE.Type typ;
      Exp_SubscriptList inst_dims;
      Exp_ExpOption start;
      DAE.Flow flow_;
      Absyn_PathList class_;
      DAE_VariableAttributesOption dae_var_attr;
      Absyn_CommentOption comment;
      Exp.Exp e;
    case ((var = DAE.VAR(id,vk,vd,typ,NONE,inst_dims,start,flow_,class_,dae_var_attr,comment)),tnr) "
  rule	is_first_in_array id => false
	-----------------------------
	generate_var_decl(DAE.VAR(id,_,_,_,_,_,_,_,_,_),tnr) => (c_empty_function,tnr)
"
      equation 
        is_a = is_array(var);
        typ_str = dae_type_str(typ, is_a);
        (cref_str,_) = comp_ref_cstr(id);
        dim_strs = Util.list_map(inst_dims, dim_string);
        dims_str = Util.string_delimit_list(dim_strs, ", ");
        dim_comment = Util.string_append_list({" /* [",dims_str,"] */"});
        dim_comment_1 = Util.if_(is_a, dim_comment, "");
        ndims = list_length(dim_strs);
        ndims_str = int_string(ndims);
        decl_str = Util.string_append_list({typ_str," ",cref_str,";",dim_comment_1});
        cfn = c_add_variables(c_empty_function, {decl_str}); then (cfn,tnr);
    case ((var = DAE.VAR(id,vk,vd,typ,SOME(e),inst_dims,start,flow_,class_,dae_var_attr,comment)),tnr)
      equation 
        (cfn,tnr1) = generate_var_decl(
          DAE.VAR(id,vk,vd,typ,NONE,inst_dims,start,flow_,class_,
          dae_var_attr,comment), tnr); then (cfn,tnr1);
    case (e,_)
      local DAE.Element e;
      equation 
        Print.print_buf("# generate_var_decl failed\n  ");
        DAE.dump_debug_element(e);
        Print.print_buf("\n"); then fail();
  end matchcontinue;
end generate_var_decl;

protected function generate_var_init
  input DAE.Element in_element;
  input Integer in_integer;
  input String in_string;
  output CFunction out_cfunction;
  output Integer out_integer;
algorithm 
  (out_cfunction,out_integer):=
  matchcontinue (in_element,in_integer,in_string)
    local
      type Exp_SubscriptList = list<Exp.Subscript>;
      type Exp_ExpOption = Option<Exp.Exp>;
      type Absyn_PathList = list<Absyn.Path>;
      type DAE_VariableAttributesOption = Option<DAE.VariableAttributes>;
      type Absyn_CommentOption = Option<Absyn.Comment>;
      DAE.Element var;
      Exp.ComponentRef id,id_1,idstr;
      DAE.VarKind vk;
      DAE.VarDirection vd;
      DAE.Type typ;
      Exp_SubscriptList inst_dims;
      Exp_ExpOption start;
      DAE.Flow flow_;
      Absyn_PathList class_;
      DAE_VariableAttributesOption dae_var_attr;
      Absyn_CommentOption comment;
      Integer tnr,tnr1;
      CleanupStatement pre;
      Boolean is_a,emptyprep;
      Exp.Type exptype;
      Algorithm.Statement scalarassign,arrayassign,assign;
      CFunction cfn;
      Exp.Exp e;
    case ((var = DAE.VAR(id,vk,vd,typ,NONE,inst_dims,start,flow_,class_,dae_var_attr,comment)),tnr,pre) then (c_empty_function,tnr); 
    case ((var = DAE.VAR(id,vk,vd,typ,SOME(e),inst_dims,start,flow_,class_,dae_var_attr,comment)),tnr,pre)
      equation 
        is_a = is_array(var);
        emptyprep = Util.string_equal(pre, "");
        id_1 = Exp.prepend_string_cref("out.", id);
        idstr = Util.if_(emptyprep, id, id_1);
        exptype = dae_exp_type(typ);
        scalarassign = Algorithm.ASSIGN(exptype,idstr,e);
        arrayassign = Algorithm.ASSIGN_ARR(exptype,idstr,e);
        assign = Util.if_(is_a, arrayassign, scalarassign);
        (cfn,tnr1) = generate_algorithm_statement(assign, tnr); then (cfn,tnr1);
    case (e,_,_)
      local DAE.Element e;
      equation 
        Print.print_buf("# generate_var_init failed\n  ");
        DAE.dump_debug_element(e);
        Print.print_buf("\n"); then fail();
  end matchcontinue;
end generate_var_init;

protected function dim_string
  input Exp.Subscript in_subscript;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_subscript)
    local
      CleanupStatement str;
      Integer i;
      Exp.Subscript e;
    case Exp.INDEX(Exp.ICONST(i))
      equation 
        str = int_string(i); then str;
    case e
      equation 
        str = Exp.print_subscript_str(e); then ":";
  end matchcontinue;
end dim_string;

protected function is_var_q
  input DAE.Element in_element;
algorithm 
  _:=
  matchcontinue (in_element)
    local
      Exp.ComponentRef id;
      DAE.VarKind vk;
      DAE.VarDirection vd;
    case DAE.VAR(id,vk,vd,_,_,_,_,_,_,_,_)
      equation 
        generate_var_q(vk);
        generate_var_q2(vd); then ();
  end matchcontinue;
end is_var_q;

protected function generate_var_q
  input DAE.VarKind in_varkind;
algorithm 
  _:=
  matchcontinue (in_varkind)
    case DAE.VARIABLE() then ();  "  axiom	generate_var_q DAE.PARAM "
    case DAE.CONST() then (); 
  end matchcontinue;
end generate_var_q;

protected function generate_var_q2
  input DAE.VarDirection in_vardirection;
algorithm 
  _:=
  matchcontinue (in_vardirection)
    case DAE.OUTPUT() then (); 
    case DAE.BIDIR() then (); 
  end matchcontinue;
end generate_var_q2;

protected function generate_result_vars
  input DAE_ElementList in_dae_elementlist;
  input String in_string;
  input Integer in_integer;
  output CFunction out_cfunction;
  output Integer out_integer;
protected 
  type DAE_ElementList = list<DAE.Element>;
algorithm 
  (out_cfunction,out_integer):=
  matchcontinue (in_dae_elementlist,in_string,in_integer)
    local
      Integer tnr,tnr1,tnr2;
      CFunction cfn1,cfn2,cfn;
      DAE.Element first;
      DAE_ElementList rest;
      CleanupStatement varname;
    case ({},_,tnr) then (c_empty_function,tnr); 
    case (first :: rest,varname,tnr)
      equation 
        (cfn1,tnr1) = generate_result_var(first, varname, tnr);
        (cfn2,tnr2) = generate_result_vars(rest, varname, tnr1);
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,tnr2);
  end matchcontinue;
end generate_result_vars;

protected function generate_result_var
  input DAE.Element in_element;
  input String in_string;
  input Integer in_integer;
  output CFunction out_cfunction;
  output Integer out_integer;
algorithm 
  (out_cfunction,out_integer):=
  matchcontinue (in_element,in_string,in_integer)
    local
      CleanupStatement cref_str1,cref_str2,stmt,varname,typ_str;
      CFunction cfn;
      DAE.Element var;
      Exp.ComponentRef id;
      DAE.Type typ;
      Integer tnr;
    case ((var = DAE.VAR(id,DAE.VARIABLE(),DAE.OUTPUT(),typ,_,_,_,_,_,_,_)),varname,tnr)
      equation 
        false = is_array(var);
        (cref_str1,_) = comp_ref_cstr(id);
        (cref_str2,_) = comp_ref_cstr(id);
        stmt = Util.string_append_list({varname,".",cref_str1," = ",cref_str2,";"});
        cfn = c_add_cleanups(c_empty_function, {stmt}); then (cfn,tnr);
    case ((var = DAE.VAR(id,DAE.VARIABLE(),DAE.OUTPUT(),typ,_,_,_,_,_,_,_)),varname,tnr)
      equation 
        true = is_array(var);
        typ_str = dae_type_str(typ, true);
        (cref_str1,_) = comp_ref_cstr(id);
        (cref_str2,_) = comp_ref_cstr(id);
        stmt = Util.string_append_list(
          {"copy_",typ_str,"_data(&",cref_str1,", &",varname,".",
          cref_str2,");"});
        cfn = c_add_cleanups(c_empty_function, {stmt}); then (cfn,tnr);
    case ((var = DAE.VAR(id,DAE.VARIABLE(),DAE.OUTPUT(),typ,_,_,_,_,_,_,_)),varname,tnr)
      equation 
        true = is_array(var); then (c_empty_function,tnr);
    case (_,_,tnr) then (c_empty_function,tnr); 
  end matchcontinue;
end generate_result_var;

protected function generate_equations " LS: ATT! These can only be assignments for the moment, 
 since no sorting or solving exists "
  input DAE_ElementList in_dae_elementlist;
protected 
  type DAE_ElementList = list<DAE.Element>;
algorithm 
  _:=
  matchcontinue (in_dae_elementlist)
    local
      Exp.Exp e1,e2;
      DAE_ElementList rest;
    case {} then (); 
    case DAE.EQUATION(e1,e2) :: rest
      equation 
        Exp.print_exp(e1);
        Print.print_buf(" = ");
        Exp.print_exp(e2);
        Print.print_buf(";\n");
        generate_equations(rest); then ();
    case _ :: rest
      equation 
        generate_equations(rest); then ();
  end matchcontinue;
end generate_equations;

protected function generate_exp
  input Exp.Exp in_exp;
algorithm 
  _:=
  matchcontinue (in_exp)
    local
      type Exp_ExpList = list<Exp.Exp>;
      Exp_ExpList es;
      Exp.Exp e;
    case Exp.ARRAY(_,_,es)
      equation 
        Print.print_buf("{");
        Exp.print_list(es, generate_exp, ", ");
        Print.print_buf("}"); then ();
    case e
      equation 
        Exp.print_exp(e); then ();
  end matchcontinue;
end generate_exp;

protected function get_array_dim
  input Types.Type in_type;
  output Integer out_integer;
algorithm 
  out_integer:=
  matchcontinue (in_type)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      Integer ad;
      TupleTypes_TTypeAbsyn_PathOption ty2;
    case ((Types.T_ARRAY(Types.DIM(SOME(ad)),ty2),_)) then ad;  "* LS: Fail if not array, or if there is no dim value *"
  end matchcontinue;
end get_array_dim;

protected function generate_expressions
  input Exp_ExpList in_exp_explist;
  input Integer in_integer;
  output CFunction out_cfunction;
  output StringList out_stringlist;
  output Integer out_integer;
protected 
  type Exp_ExpList = list<Exp.Exp>;
  type StringList = list<String>;
algorithm 
  (out_cfunction,out_stringlist,out_integer):=
  matchcontinue (in_exp_explist,in_integer)
    local
      Integer tnr,tnr1,tnr2;
      CFunction cfn1,cfn2,cfn;
      CleanupStatement var1;
      CleanupStatementList vars2;
      Exp.Exp f;
      Exp_ExpList r;
    case ({},tnr) then (c_empty_function,{},tnr); 
    case (f :: r,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(f, tnr);
        (cfn2,vars2,tnr2) = generate_expressions(r, tnr1);
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var1 :: vars2,tnr2);
  end matchcontinue;
end generate_expressions;

protected function generate_expression "
 generate_expression

 returns
   CFunction | the generated code
   string    | expression result variable name, or c expression
   int       | next temporary number 
"
  input Exp.Exp in_exp;
  input Integer in_integer;
  output CFunction out_cfunction;
  output String out_string;
  output Integer out_integer;
algorithm 
  (out_cfunction,out_string,out_integer):=
  matchcontinue (in_exp,in_integer)
    local
      type Exp_ExpList = list<Exp.Exp>;
      type TupleExp_ExpBoolean = tuple<Exp.Exp,Boolean>;
      type TupleExp_ExpBooleanList = list<TupleExp_ExpBoolean>;
      type TupleExp_ExpBooleanListList = list<TupleExp_ExpBooleanList>;
      CleanupStatement istr,rstr,sstr,s,var,var1,decl,tvar,b_stmt,if_begin,var2,var3,ret_type,fn_name,tdecl,args_str,stmt,var_not_bi,typestr,nvars_str,array_type_str,short_type_str,scalar,scalar_ref,scalar_delimit,type_string,var_1;
      Integer i,tnr,tnr_1,tnr1,tnr1_1,tnr2,tnr3,nvars,maxn,tnr4;
      Real r;
      Boolean b,builtin,a;
      CFunction cfn,cfn1,cfn1_2,cfn1_1,cfn2,cfn2_1,cfn3,cfn3_1,cfn4,cfn_1;
      Exp.ComponentRef cref,cr;
      Exp.Type t,ty;
      Exp.Exp e1,e2,e,then_,else_,crexp,dim,e3;
      Exp.Operator op;
      CleanupStatementList vars1;
      Absyn.Path fn;
      Exp_ExpList args,elist;
      TupleExp_ExpBooleanListList ell;
    case (Exp.ICONST(i),tnr)
      equation 
        istr = int_string(i); then (c_empty_function,istr,tnr);
    case (Exp.RCONST(r),tnr)
      equation 
        rstr = real_string(r); then (c_empty_function,rstr,tnr);
    case (Exp.SCONST(s),tnr)
      equation 
        sstr = Util.string_append_list({"\"",s,"\""}); then (c_empty_function,sstr,tnr);
    case (Exp.BCONST(b),tnr)
      equation 
        var = Util.if_(b, "(1)", "(0)"); then (c_empty_function,var,tnr);
    case (Exp.CREF(cref,t),tnr)
      equation 
        (cfn,var,tnr_1) = generate_rhs_cref(cref, t, tnr); then (cfn,var,tnr_1);
    case (Exp.BINARY(e1,op,e2),tnr)
      equation 
        (cfn,var,tnr_1) = generate_binary(e1, op, e2, tnr); then (cfn,var,tnr_1);
    case (Exp.UNARY(op,e),tnr)
      equation 
        (cfn,var,tnr_1) = generate_unary(op, e, tnr); then (cfn,var,tnr_1);
    case (Exp.LBINARY(e1,op,e2),tnr)
      equation 
        (cfn,var,tnr_1) = generate_lbinary(e1, op, e2, tnr); then (cfn,var,tnr_1);
    case (Exp.LUNARY(op,e),tnr)
      equation 
        (cfn,var,tnr_1) = generate_lunary(op, e, tnr); then (cfn,var,tnr_1);
    case (Exp.RELATION(e1,op,e2),tnr)
      equation 
        (cfn,var,tnr_1) = generate_relation(e1, op, e2, tnr); then (cfn,var,tnr_1);
    case (Exp.IFEXP(e,then_,else_),tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e, tnr);
        (decl,tvar,tnr1_1) = generate_temp_decl("modelica_boolean", tnr1);
        b_stmt = Util.string_append_list({tvar," = ",var1,";"});
        if_begin = Util.string_append_list({"if (",tvar,") {"});
        cfn1_2 = c_add_statements(cfn1, {b_stmt,if_begin});
        cfn1_1 = c_add_variables(cfn1_2, {decl});
        (cfn2,var2,tnr2) = generate_expression(then_, tnr1_1);
        cfn2_1 = c_add_statements(cfn2, {"}","else {"});
        (cfn3,var3,tnr3) = generate_expression(else_, tnr2);
        cfn3_1 = c_add_statements(cfn3, {"}"});
        cfn = c_merge_fns({cfn1_1,cfn2_1,cfn3_1});
        var = Util.string_append_list({"((",tvar,")?",var2,":",var3,")"}); then (cfn,var,tnr3);
    case (Exp.CALL(fn,args,false,builtin),tnr)
      equation 
        (cfn1,vars1,tnr1) = generate_expressions(args, tnr);
        ret_type = generate_return_type(fn);
        fn_name = generate_function_name(fn);
        (tdecl,tvar,tnr2) = generate_temp_decl(ret_type, tnr1);
        cfn2 = c_add_variables(cfn1, {tdecl});
        args_str = Util.string_delimit_list(vars1, ", ");
        stmt = Util.string_append_list({tvar," = ",fn_name,"(",args_str,");"});
        cfn = c_add_statements(cfn2, {stmt});
        var_not_bi = Util.string_append_list({tvar,".",ret_type,"_1"});
        var = Util.if_(builtin, tvar, var_not_bi); then (cfn,var,tnr2);
    case (Exp.CALL(fn,args,true,builtin),tnr)
      equation 
        (cfn1,vars1,tnr1) = generate_expressions(args, tnr);
        ret_type = generate_return_type(fn);
        fn_name = generate_function_name(fn);
        (tdecl,tvar,tnr2) = generate_temp_decl(ret_type, tnr1);
        cfn2 = c_add_variables(cfn1, {tdecl});
        args_str = Util.string_delimit_list(vars1, ", ");
        stmt = Util.string_append_list({tvar," = ",fn_name,"(",args_str,");"});
        cfn = c_add_statements(cfn2, {stmt}); then (cfn,tvar,tnr2);
    case (Exp.SIZE((crexp = Exp.CREF(cr,ty)),SOME(dim)),tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(crexp, tnr);
        (tdecl,tvar,tnr2) = generate_temp_decl("size_t", tnr1);
        cfn2 = c_add_variables(cfn1, {tdecl});
        typestr = exp_type_str(ty, true);
        (cfn3,var2,tnr3) = generate_expression(dim, tnr2);
        stmt = Util.string_append_list(
          {tvar," = size_of_dimension_",typestr,"(",var1,",",var2,");"});
        cfn4 = c_merge_fn(cfn2, cfn3);
        cfn = c_add_statements(cfn4, {stmt}); then (cfn,tvar,tnr2);
    case (Exp.SIZE(cr,NONE),tnr)
      local Exp.Exp cr;
      equation 
        Print.print_buf("#-- Codegen.generate_expression: size(X) not implemented"); then fail();
    case (Exp.ARRAY(t,a,elist),tnr)
      equation 
        (cfn1,vars1,tnr1) = generate_expressions(elist, tnr);
        nvars = list_length(vars1);
        nvars_str = int_string(nvars);
        array_type_str = exp_type_str(t, true);
        short_type_str = exp_short_type_str(t);
        (tdecl,tvar,tnr2) = generate_temp_decl(array_type_str, tnr1);
        scalar = Util.if_(a, "", "scalar_");
        scalar_ref = Util.if_(a, "&", "");
        scalar_delimit = string_append(", ", scalar_ref);
        args_str = Util.string_delimit_list(vars1, scalar_delimit);
        stmt = Util.string_append_list(
          {"array_",scalar,array_type_str,"(&",tvar,", ",nvars_str,
          ", ",scalar_ref,args_str,");"});
        cfn_1 = c_add_variables(cfn1, {tdecl});
        cfn = c_add_statements(cfn_1, {stmt}); then (cfn,tvar,tnr2);
    case (Exp.MATRIX(t,maxn,ell),tnr)
      equation 
        (cfn,var,tnr_1) = generate_matrix(t, maxn, ell, tnr); then (cfn,var,tnr_1);
    case (Exp.RANGE(t,e1,NONE,e2),tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        type_string = exp_type_str(t, true);
        (tdecl,tvar,tnr3) = generate_temp_decl(type_string, tnr2);
        stmt = Util.string_append_list(
          {"range_alloc_",type_string,"(",var1,", ",var2,", 1, &",
          tvar,");"});
        cfn1_1 = c_add_variables(cfn1, {tdecl});
        cfn2_1 = c_add_statements(cfn2, {stmt});
        cfn = c_merge_fns({cfn1_1,cfn2_1}); then (cfn,tvar,tnr3);
    case (Exp.RANGE(t,e1,SOME(e2),e3),tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        (cfn3,var3,tnr3) = generate_expression(e3, tnr1);
        type_string = exp_type_str(t, true);
        (tdecl,tvar,tnr4) = generate_temp_decl(type_string, tnr3);
        stmt = Util.string_append_list(
          {"range_alloc_",type_string,"(",var1,", ",var3,", ",var2,
          ", &",tvar,");"});
        cfn1_1 = c_add_variables(cfn1, {tdecl});
        cfn2_1 = c_add_statements(cfn2, {stmt});
        cfn = c_merge_fns({cfn1_1,cfn2_1}); then (cfn,tvar,tnr4);
    case (Exp.TUPLE(_),_)
      equation 
        Print.print_buf("# Codegen.generate_expression: tuple not implemented\n"); then fail();
    case (Exp.CAST(Exp.INT,e),tnr)
      equation 
        (cfn,var,tnr_1) = generate_expression(e, tnr);
        var_1 = Util.string_append_list({"((modelica_int)",var,")"}); then (cfn,var_1,tnr_1);
    case (Exp.CAST(Exp.REAL,e),tnr)
      equation 
        (cfn,var,tnr_1) = generate_expression(e, tnr);
        var_1 = Util.string_append_list({"((modelica_real)",var,")"}); then (cfn,var_1,tnr_1);
    case (Exp.ASUB(_,_),_)
      equation 
        Print.print_buf("# Codegen.generate_expression: asub not implemented\n"); then fail();
    case (e,_) then fail(); 
  end matchcontinue;
end generate_expression;

protected function generate_unary
  input Exp.Operator in_operator;
  input Exp.Exp in_exp;
  input Integer in_integer;
  output CFunction out_cfunction;
  output String out_string;
  output Integer out_integer;
algorithm 
  (out_cfunction,out_string,out_integer):=
  matchcontinue (in_operator,in_exp,in_integer)
    local
      CFunction cfn;
      CleanupStatement var,var_1;
      Integer tnr_1,tnr;
      Exp.Exp e;
    case (Exp.UPLUS(Exp.REAL),e,tnr)
      equation 
        (cfn,var,tnr_1) = generate_expression(e, tnr); then (cfn,var,tnr_1);
    case (Exp.UPLUS(Exp.INT),e,tnr)
      equation 
        (cfn,var,tnr_1) = generate_expression(e, tnr); then (cfn,var,tnr_1);
    case (Exp.UMINUS(Exp.REAL),e,tnr)
      equation 
        (cfn,var,tnr_1) = generate_expression(e, tnr);
        var_1 = Util.string_append_list({"(-",var,")"}); then (cfn,var_1,tnr_1);
    case (Exp.UMINUS(Exp.INT),e,tnr)
      equation 
        (cfn,var,tnr_1) = generate_expression(e, tnr);
        var_1 = Util.string_append_list({"(-",var,")"}); then (cfn,var_1,tnr_1);
    case (Exp.UPLUS_ARR(Exp.REAL),e,tnr)
      equation 
        (cfn,var,tnr_1) = generate_expression(e, tnr); then (cfn,var,tnr_1);
    case (Exp.UPLUS_ARR(Exp.INT),e,tnr)
      equation 
        (cfn,var,tnr_1) = generate_expression(e, tnr); then (cfn,var,tnr_1);
    case (Exp.UMINUS_ARR(_),_,_)
      equation 
        Print.print_buf("# unary minus for arrays not implemented\n"); then fail();
  end matchcontinue;
end generate_unary;

protected function generate_binary
  input Exp.Exp in_exp1;
  input Exp.Operator in_operator2;
  input Exp.Exp in_exp3;
  input Integer in_integer4;
  output CFunction out_cfunction;
  output String out_string;
  output Integer out_integer;
algorithm 
  (out_cfunction,out_string,out_integer):=
  matchcontinue (in_exp1,in_operator2,in_exp3,in_integer4)
    local
      CFunction cfn1,cfn2,cfn,cfn_1,cfn_2;
      CleanupStatement var1,var2,var,decl,stmt;
      Integer tnr1,tnr2,tnr,tnr3;
      Exp.Exp e1,e2;
    case (e1,Exp.ADD(_),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        var = Util.string_append_list({"(",var1," + ",var2,")"});
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var,tnr2);
    case (e1,Exp.SUB(_),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        var = Util.string_append_list({"(",var1," - ",var2,")"});
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var,tnr2);
    case (e1,Exp.MUL(_),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        var = Util.string_append_list({"(",var1," * ",var2,")"});
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var,tnr2);
    case (e1,Exp.DIV(_),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        var = Util.string_append_list({"(",var1," / ",var2,")"});
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var,tnr2);
    case (e1,Exp.POW(_),e2,tnr)
      equation 
        Print.print_buf("# pow not implemented yet"); then fail();
    case (_,Exp.UMINUS(_),_,_)
      equation 
        Print.print_buf("# Unary minus in binary expression (internal error)"); then fail();
    case (_,Exp.UPLUS(_),_,_)
      equation 
        Print.print_buf("# Unary plus in binary expression (internal error)"); then fail();
    case (e1,Exp.ADD_ARR(Exp.REAL),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        (decl,var,tnr3) = generate_temp_decl("real_array", tnr2);
        stmt = Util.string_append_list({"add_alloc_real_array(&",var1,", &",var2,", &",var,");"});
        cfn_1 = c_merge_fn(cfn1, cfn2);
        cfn_2 = c_add_variables(cfn_1, {decl});
        cfn = c_add_statements(cfn_2, {stmt}); then (cfn,var,tnr3);
    case (e1,Exp.ADD_ARR(Exp.INT),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        (decl,var,tnr3) = generate_temp_decl("integer_array", tnr2);
        stmt = Util.string_append_list(
          {"add_alloc_integer_array(&",var1,", &",var2,", &",var,");"});
        cfn_1 = c_merge_fn(cfn1, cfn2);
        cfn_2 = c_add_variables(cfn_1, {decl});
        cfn = c_add_statements(cfn_2, {stmt}); then (cfn,var,tnr3);
    case (e1,Exp.SUB_ARR(Exp.REAL),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        (decl,var,tnr3) = generate_temp_decl("real_array", tnr2);
        stmt = Util.string_append_list({"sub_alloc_real_array(&",var1,", &",var2,", &",var,");"});
        cfn_1 = c_merge_fn(cfn1, cfn2);
        cfn_2 = c_add_variables(cfn_1, {decl});
        cfn = c_add_statements(cfn_2, {stmt}); then (cfn,var,tnr3);
    case (e1,Exp.SUB_ARR(Exp.INT),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        (decl,var,tnr3) = generate_temp_decl("integer_array", tnr2);
        stmt = Util.string_append_list(
          {"sub_alloc_integer_array(&",var1,", &",var2,", &",var,");"});
        cfn_1 = c_merge_fn(cfn1, cfn2);
        cfn_2 = c_add_variables(cfn_1, {decl});
        cfn = c_add_statements(cfn_2, {stmt}); then (cfn,var,tnr3);
    case (e1,Exp.MUL_SCALAR_ARRAY(Exp.REAL),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        (decl,var,tnr3) = generate_temp_decl("real_array", tnr2);
        stmt = Util.string_append_list(
          {"mul_alloc_scalar_real_array(",var1,", &",var2,", &",var,
          ");"});
        cfn_1 = c_merge_fn(cfn1, cfn2);
        cfn_2 = c_add_variables(cfn_1, {decl});
        cfn = c_add_statements(cfn_2, {stmt}); then (cfn,var,tnr3);
    case (e1,Exp.MUL_SCALAR_ARRAY(Exp.INT),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        (decl,var,tnr3) = generate_temp_decl("integer_array", tnr2);
        stmt = Util.string_append_list(
          {"mul_alloc_scalar_integer_array(",var1,", &",var2,", &",
          var,");"});
        cfn_1 = c_merge_fn(cfn1, cfn2);
        cfn_2 = c_add_variables(cfn_1, {decl});
        cfn = c_add_statements(cfn_2, {stmt}); then (cfn,var,tnr3);
    case (e1,Exp.MUL_ARRAY_SCALAR(Exp.REAL),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        (decl,var,tnr3) = generate_temp_decl("real_array", tnr2);
        stmt = Util.string_append_list(
          {"mul_alloc_real_array_scalar(&",var1,", ",var2,", &",var,
          ");"});
        cfn_1 = c_merge_fn(cfn1, cfn2);
        cfn_2 = c_add_variables(cfn_1, {decl});
        cfn = c_add_statements(cfn_2, {stmt}); then (cfn,var,tnr3);
    case (e1,Exp.MUL_ARRAY_SCALAR(Exp.INT),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        (decl,var,tnr3) = generate_temp_decl("integer_array", tnr2);
        stmt = Util.string_append_list(
          {"mul_alloc_integer_array_scalar(&",var1,", ",var2,", &",
          var,");"});
        cfn_1 = c_merge_fn(cfn1, cfn2);
        cfn_2 = c_add_variables(cfn_1, {decl});
        cfn = c_add_statements(cfn_2, {stmt}); then (cfn,var,tnr3);
    case (e1,Exp.MUL_SCALAR_PRODUCT(Exp.REAL),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        var = Util.string_append_list({"mul_real_scalar_product(&",var1,", &",var2,")"});
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var,tnr2);
    case (e1,Exp.MUL_SCALAR_PRODUCT(Exp.INT),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        var = Util.string_append_list({"mul_integer_scalar_product(&",var1,", &",var2,")"});
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var,tnr2);
    case (e1,Exp.MUL_MATRIX_PRODUCT(Exp.REAL),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        (decl,var,tnr3) = generate_temp_decl("real_array", tnr2);
        stmt = Util.string_append_list(
          {"mul_alloc_real_matrix_product_smart(&",var1,", &",var2,
          ", &",var,");"});
        cfn_1 = c_merge_fn(cfn1, cfn2);
        cfn_2 = c_add_variables(cfn_1, {decl});
        cfn = c_add_statements(cfn_2, {stmt}); then (cfn,var,tnr3);
    case (e1,Exp.MUL_MATRIX_PRODUCT(Exp.INT),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        (decl,var,tnr3) = generate_temp_decl("integer_array", tnr2);
        stmt = Util.string_append_list(
          {"mul_alloc_integer_matrix_product_smart(&",var1,", &",var2,
          ", &",var,");"});
        cfn_1 = c_merge_fn(cfn1, cfn2);
        cfn_2 = c_add_variables(cfn_1, {decl});
        cfn = c_add_statements(cfn_2, {stmt}); then (cfn,var,tnr3);
    case (e1,Exp.DIV_ARRAY_SCALAR(Exp.REAL),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        (decl,var,tnr3) = generate_temp_decl("real_array", tnr2);
        stmt = Util.string_append_list(
          {"div_alloc_real_array_scalar(&",var1,", ",var2,", &",var,
          ");"});
        cfn_1 = c_merge_fn(cfn1, cfn2);
        cfn_2 = c_add_variables(cfn_1, {decl});
        cfn = c_add_statements(cfn_2, {stmt}); then (cfn,var,tnr3);
    case (e1,Exp.DIV_ARRAY_SCALAR(Exp.INT),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        (decl,var,tnr3) = generate_temp_decl("integer_array", tnr2);
        stmt = Util.string_append_list(
          {"div_alloc_integer_array_scalar(&",var1,", ",var2,", &",
          var,");"});
        cfn_1 = c_merge_fn(cfn1, cfn2);
        cfn_2 = c_add_variables(cfn_1, {decl});
        cfn = c_add_statements(cfn_2, {stmt}); then (cfn,var,tnr3);
    case (_,Exp.DIV_ARRAY_SCALAR(_),_,_)
      equation 
        Print.print_buf("# div_array_scalar FAILING BECAUSE IT SUX\n"); then fail();
    case (_,Exp.POW_ARR(_),_,_)
      equation 
        Print.print_buf("# pow_array not implemented\n"); then fail();
    case (_,Exp.DIV_ARRAY_SCALAR(_),_,_)
      equation 
        Print.print_buf("# div_array_scalar not implemented\n"); then fail();
    case (_,_,_,_)
      equation 
        Print.print_buf("# generate_binary failed\n"); then fail();
  end matchcontinue;
end generate_binary;

protected function generate_temp_decl
  input String in_string;
  input Integer in_integer;
  output String out_string1;
  output String out_string2;
  output Integer out_integer3;
algorithm 
  (out_string1,out_string2,out_integer3):=
  matchcontinue (in_string,in_integer)
    local
      CleanupStatement tnr_str,tmp_name,t_1,t;
      Integer tnr_1,tnr;
    case (t,tnr)
      equation 
        tnr_str = int_string(tnr);
        tnr_1 = tnr + 1;
        tmp_name = string_append("tmp", tnr_str);
        t_1 = Util.string_append_list({t," ",tmp_name,";"}); then (t_1,tmp_name,tnr_1);
  end matchcontinue;
end generate_temp_decl;

protected function generate_scalar_lhs_cref
  input Exp.Type in_type;
  input Exp.ComponentRef in_componentref;
  input Integer in_integer;
  output CFunction out_cfunction;
  output String out_string;
  output Integer out_integer;
algorithm 
  (out_cfunction,out_string,out_integer):=
  matchcontinue (in_type,in_componentref,in_integer)
    local
      type Exp_SubscriptList = list<Exp.Subscript>;
      CleanupStatement cref_str,var,ndims_str,idxs_str,type_str,cref1,id;
      Exp.Type t;
      Exp.ComponentRef cref;
      Integer tnr,tnr_1,tnr1,ndims;
      Exp_SubscriptList subs,idx;
      CFunction cfn,cfn1;
      CleanupStatementList idxs1;
    case (t,cref,tnr)
      equation 
        (cref_str,{}) = comp_ref_cstr(cref); then (c_empty_function,cref_str,tnr);
    case (t,cref,tnr)
      equation 
        (cref_str,subs) = comp_ref_cstr(cref);
        (cfn,var,tnr_1) = generate_scalar_rhs_cref(cref_str, t, subs, tnr); then (cfn,var,tnr_1);
    case (t,Exp.CREF_IDENT(id,idx),tnr) " two special cases rules for 1 and 2 dimensions for faster code (no vararg) "
      equation 
        Debug.fprintln("gcge", "generating cref ccode");
        (cfn1,idxs1,tnr1) = generate_indices(idx, tnr);
        1 = list_length(idxs1);
        ndims_str = int_string(1) " ndims == 1";
        idxs_str = Util.string_delimit_list(idxs1, ", ");
        type_str = exp_type_str(t, true);
        cref1 = Util.string_append_list(
          {"(*",type_str,"_element_addr1(&",id,", ",ndims_str,", ",
          idxs_str,"))"}); then (cfn1,cref1,tnr1);
    case (t,Exp.CREF_IDENT(id,idx),tnr)
      equation 
        Debug.fprintln("gcge", "generating cref ccode");
        (cfn1,idxs1,tnr1) = generate_indices(idx, tnr);
        2 = list_length(idxs1);
        ndims_str = int_string(2) " ndims == 2 ";
        idxs_str = Util.string_delimit_list(idxs1, ", ");
        type_str = exp_type_str(t, true);
        cref1 = Util.string_append_list(
          {"(*",type_str,"_element_addr2(&",id,", ",ndims_str,", ",
          idxs_str,"))"}); then (cfn1,cref1,tnr1);
    case (t,Exp.CREF_IDENT(id,idx),tnr)
      equation 
        Debug.fprintln("gcge", "generating cref ccode");
        (cfn1,idxs1,tnr1) = generate_indices(idx, tnr);
        ndims = list_length(idxs1);
        ndims_str = int_string(ndims);
        idxs_str = Util.string_delimit_list(idxs1, ", ");
        type_str = exp_type_str(t, true);
        cref1 = Util.string_append_list(
          {"(*",type_str,"_element_addr(&",id,", ",ndims_str,", ",
          idxs_str,"))"}); then (cfn1,cref1,tnr1);
    case (_,_,_)
      equation 
        Print.print_buf("# generate_scalar_lhs_cref failed\n"); then fail();
  end matchcontinue;
end generate_scalar_lhs_cref;

protected function generate_rhs_cref
  input Exp.ComponentRef in_componentref;
  input Exp.Type in_type;
  input Integer in_integer;
  output CFunction out_cfunction;
  output String out_string;
  output Integer out_integer;
algorithm 
  (out_cfunction,out_string,out_integer):=
  matchcontinue (in_componentref,in_type,in_integer)
    local
      type Exp_SubscriptList = list<Exp.Subscript>;
      CleanupStatement cref_str,var;
      Exp.ComponentRef cref;
      Exp.Type crt;
      Integer tnr,tnr_1;
      Exp_SubscriptList subs;
      CFunction cfn;
    case (cref,crt,tnr)
      equation 
        (cref_str,{}) = comp_ref_cstr(cref); then (c_empty_function,cref_str,tnr);
    case (cref,crt,tnr)
      equation 
        (cref_str,subs) = comp_ref_cstr(cref);
        true = subs_to_scalar(subs);
        (cfn,var,tnr_1) = generate_scalar_rhs_cref(cref_str, crt, subs, tnr); then (cfn,var,tnr_1);
    case (cref,crt,tnr)
      equation 
        (cref_str,subs) = comp_ref_cstr(cref);
        false = subs_to_scalar(subs);
        (cfn,var,tnr_1) = generate_array_rhs_cref(cref_str, crt, subs, tnr); then (cfn,var,tnr_1);
  end matchcontinue;
end generate_rhs_cref;

protected function subs_to_scalar
  input Exp_SubscriptList in_exp_subscriptlist;
  output Boolean out_boolean;
protected 
  type Exp_SubscriptList = list<Exp.Subscript>;
algorithm 
  out_boolean:=
  matchcontinue (in_exp_subscriptlist)
    local
      Boolean b;
      Exp_SubscriptList r;
    case {} then true; 
    case Exp.SLICE(_) :: _ then false; 
    case Exp.WHOLEDIM :: _ then false; 
    case Exp.INDEX(_) :: r
      equation 
        b = subs_to_scalar(r); then b;
  end matchcontinue;
end subs_to_scalar;

protected function generate_scalar_rhs_cref
  input String in_string;
  input Exp.Type in_type;
  input Exp_SubscriptList in_exp_subscriptlist;
  input Integer in_integer;
  output CFunction out_cfunction;
  output String out_string;
  output Integer out_integer;
protected 
  type Exp_SubscriptList = list<Exp.Subscript>;
algorithm 
  (out_cfunction,out_string,out_integer):=
  matchcontinue (in_string,in_type,in_exp_subscriptlist,in_integer)
    local
      CFunction cfn1;
      CleanupStatementList idxs1;
      Integer tnr1,tnr,ndims;
      CleanupStatement ndims_str,idxs_str,array_type_str,cref1,cref_str;
      Exp.Type crt;
      Exp_SubscriptList subs;
    case (cref_str,crt,subs,tnr) " Two special rules for faster code when ndims == 1 or 2 "
      equation 
        (cfn1,idxs1,tnr1) = generate_indices(subs, tnr);
        1 = list_length(idxs1);
        ndims_str = int_string(1) " ndims == 1";
        idxs_str = Util.string_delimit_list(idxs1, ", ");
        array_type_str = exp_type_str(crt, true);
        cref1 = Util.string_append_list(
          {"(*",array_type_str,"_element_addr1(&",cref_str,", ",
          ndims_str,", ",idxs_str,"))"}); then (cfn1,cref1,tnr1);
    case (cref_str,crt,subs,tnr)
      equation 
        (cfn1,idxs1,tnr1) = generate_indices(subs, tnr);
        2 = list_length(idxs1);
        ndims_str = int_string(2) " ndims == 2";
        idxs_str = Util.string_delimit_list(idxs1, ", ");
        array_type_str = exp_type_str(crt, true);
        cref1 = Util.string_append_list(
          {"(*",array_type_str,"_element_addr2(&",cref_str,", ",
          ndims_str,", ",idxs_str,"))"}); then (cfn1,cref1,tnr1);
    case (cref_str,crt,subs,tnr)
      equation 
        (cfn1,idxs1,tnr1) = generate_indices(subs, tnr);
        ndims = list_length(idxs1);
        ndims_str = int_string(ndims);
        idxs_str = Util.string_delimit_list(idxs1, ", ");
        array_type_str = exp_type_str(crt, true);
        cref1 = Util.string_append_list(
          {"(*",array_type_str,"_element_addr(&",cref_str,", ",
          ndims_str,", ",idxs_str,"))"}); then (cfn1,cref1,tnr1);
  end matchcontinue;
end generate_scalar_rhs_cref;

protected function generate_array_rhs_cref
  input String in_string;
  input Exp.Type in_type;
  input Exp_SubscriptList in_exp_subscriptlist;
  input Integer in_integer;
  output CFunction out_cfunction;
  output String out_string;
  output Integer out_integer;
protected 
  type Exp_SubscriptList = list<Exp.Subscript>;
algorithm 
  (out_cfunction,out_string,out_integer):=
  matchcontinue (in_string,in_type,in_exp_subscriptlist,in_integer)
    local
      CFunction cfn1,cfn_1,cfn;
      CleanupStatement spec1,array_type_str,decl,temp,stmt,cref_str;
      Integer tnr1,tnr2,tnr;
      Exp.Type crt;
      Exp_SubscriptList subs;
    case (cref_str,crt,subs,tnr)
      equation 
        (cfn1,spec1,tnr1) = generate_index_spec(subs, tnr);
        array_type_str = exp_type_str(crt, true);
        (decl,temp,tnr2) = generate_temp_decl(array_type_str, tnr1);
        stmt = Util.string_append_list(
          {"index_alloc_",array_type_str,"(&",cref_str,", &",spec1,
          ", &",temp,");"});
        cfn_1 = c_add_variables(cfn1, {decl});
        cfn = c_add_statements(cfn_1, {stmt}); then (cfn,temp,tnr2);
  end matchcontinue;
end generate_array_rhs_cref;

protected function generate_index_spec
  input Exp_SubscriptList in_exp_subscriptlist;
  input Integer in_integer;
  output CFunction out_cfunction;
  output String out_string;
  output Integer out_integer;
protected 
  type Exp_SubscriptList = list<Exp.Subscript>;
algorithm 
  (out_cfunction,out_string,out_integer):=
  matchcontinue (in_exp_subscriptlist,in_integer)
    local
      CFunction cfn1,cfn_1,cfn;
      CleanupStatementList idxs1,idxsizes,idxs_1;
      Integer tnr1,tnr2,nridx,tnr;
      CleanupStatement decl,spec,nridx_str,idxs_str,stmt;
      Exp_SubscriptList subs;
    case (subs,tnr)
      equation 
        (cfn1,idxs1,idxsizes,tnr1) = generate_indices_array(subs, tnr);
        (decl,spec,tnr2) = generate_temp_decl("index_spec_t", tnr1);
        nridx = list_length(idxs1);
        nridx_str = int_string(nridx);
        idxs_1 = Util.list_thread(idxsizes, idxs1);
        idxs_str = Util.string_delimit_list(idxs_1, ", ");
        stmt = Util.string_append_list(
          {"create_index_spec(&",spec,", ",nridx_str,", ",idxs_str,
          ");"});
        cfn_1 = c_add_variables(cfn1, {decl});
        cfn = c_add_statements(cfn_1, {stmt}); then (cfn,spec,tnr2);
    case (_,_)
      equation 
        Print.print_buf("# generate_index_spec failed\n"); then fail();
  end matchcontinue;
end generate_index_spec;

protected function generate_indices_array
  input Exp_SubscriptList in_exp_subscriptlist;
  input Integer in_integer;
  output CFunction out_cfunction1;
  output StringList out_stringlist2;
  output StringList out_stringlist3;
  output Integer out_integer4;
protected 
  type Exp_SubscriptList = list<Exp.Subscript>;
  type StringList = list<String>;
algorithm 
  (out_cfunction1,out_stringlist2,out_stringlist3,out_integer4):=
  matchcontinue (in_exp_subscriptlist,in_integer)
    local
      Integer tnr,tnr1,tnr2;
      CFunction cfn1,cfn2,cfn;
      CleanupStatement idx1,idxsize1;
      CleanupStatementList idxs2,idxsizes2,idxs,idxsizes;
      Exp.Subscript f;
      Exp_SubscriptList r;
    case ({},tnr) then (c_empty_function,{},{},tnr); 
    case (f :: r,tnr)
      equation 
        (cfn1,idx1,idxsize1,tnr1) = generate_index_array(f, tnr);
        (cfn2,idxs2,idxsizes2,tnr2) = generate_indices_array(r, tnr1);
        cfn = c_merge_fn(cfn1, cfn2);
        idxs = idx1 :: idxs2;
        idxsizes = idxsize1 :: idxsizes2; then (cfn,idxs,idxsizes,tnr2);
    case (_,_)
      equation 
        Print.print_buf("# generate_indices_array failed\n"); then fail();
  end matchcontinue;
end generate_indices_array;

protected function generate_indices
  input Exp_SubscriptList in_exp_subscriptlist;
  input Integer in_integer;
  output CFunction out_cfunction;
  output StringList out_stringlist;
  output Integer out_integer;
protected 
  type Exp_SubscriptList = list<Exp.Subscript>;
  type StringList = list<String>;
algorithm 
  (out_cfunction,out_stringlist,out_integer):=
  matchcontinue (in_exp_subscriptlist,in_integer)
    local
      Integer tnr,tnr1,tnr2;
      CFunction cfn1,cfn2,cfn;
      CleanupStatement idx1;
      CleanupStatementList idxs2,idxs;
      Exp.Subscript f;
      Exp_SubscriptList r;
    case ({},tnr) then (c_empty_function,{},tnr); 
    case (f :: r,tnr)
      equation 
        (cfn1,idx1,tnr1) = generate_index(f, tnr);
        (cfn2,idxs2,tnr2) = generate_indices(r, tnr1);
        cfn = c_merge_fn(cfn1, cfn2);
        idxs = idx1 :: idxs2; then (cfn,idxs,tnr2);
    case (_,_)
      equation 
        Print.print_buf("# generate_indices failed\n"); then fail();
  end matchcontinue;
end generate_indices;

protected function generate_index_array
  input Exp.Subscript in_subscript;
  input Integer in_integer;
  output CFunction out_cfunction1;
  output String out_string2;
  output String out_string3;
  output Integer out_integer4;
algorithm 
  (out_cfunction1,out_string2,out_string3,out_integer4):=
  matchcontinue (in_subscript,in_integer)
    local
      CFunction cfn;
      CleanupStatement var1,idx,idxsize;
      Integer tnr1,tnr;
      Exp.Exp e;
    case (Exp.INDEX(e),tnr)
      equation 
        (cfn,var1,tnr1) = generate_expression(e, tnr);
        idx = Util.string_append_list({"make_index_array(1, ",var1,")"});
        idxsize = "(0)"; then (cfn,idx,idxsize,tnr1);
    case (Exp.WHOLEDIM,tnr)
      equation 
        idx = "(0)";
        idxsize = "(1)"; then (c_empty_function,idx,idxsize,tnr);
    case (_,_)
      equation 
        Print.print_buf("# generate_index_array failed\n"); then fail();
  end matchcontinue;
end generate_index_array;

protected function generate_index
  input Exp.Subscript in_subscript;
  input Integer in_integer;
  output CFunction out_cfunction;
  output String out_string;
  output Integer out_integer;
algorithm 
  (out_cfunction,out_string,out_integer):=
  matchcontinue (in_subscript,in_integer)
    local
      CFunction cfn;
      CleanupStatement var1;
      Integer tnr1,tnr;
      Exp.Exp e;
    case (Exp.INDEX(e),tnr)
      equation 
        (cfn,var1,tnr1) = generate_expression(e, tnr); then (cfn,var1,tnr1);
    case (_,_)
      equation 
        Print.print_buf("# generate_index failed\n"); then fail();
  end matchcontinue;
end generate_index;

protected function indent_strings
  input StringList in_stringlist;
  output StringList out_stringlist;
protected 
  type StringList = list<String>;
algorithm 
  out_stringlist:=
  matchcontinue (in_stringlist)
    local
      CleanupStatement f_1,f;
      CleanupStatementList r_1,r;
    case {} then {}; 
    case f :: r
      equation 
        f_1 = string_append("  ", f);
        r_1 = indent_strings(r); then f_1 :: r_1;
  end matchcontinue;
end indent_strings;

protected function ident_cstr
  input String in_string;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_string)
    local CleanupStatement s;
    case s then s; 
  end matchcontinue;
end ident_cstr;

protected function comp_ref_cstr
  input Exp.ComponentRef in_componentref;
  output String out_string;
  output Exp_SubscriptList out_exp_subscriptlist;
protected 
  type Exp_SubscriptList = list<Exp.Subscript>;
algorithm 
  (out_string,out_exp_subscriptlist):=
  matchcontinue (in_componentref)
    local
      CleanupStatement id_str,id,cref_str,cref_str_1;
      Exp_SubscriptList subs,cref_subs,subs_1;
      Exp.ComponentRef cref;
    case Exp.CREF_IDENT(id,subs)
      equation 
        id_str = ident_cstr(id); then (id_str,subs);
    case Exp.CREF_QUAL(id,subs,cref)
      equation 
        id_str = ident_cstr(id);
        (cref_str,cref_subs) = comp_ref_cstr(cref);
        cref_str_1 = Util.string_append_list({id,"__",cref_str});
        subs_1 = Util.list_flatten({subs,cref_subs}); then (cref_str_1,subs_1);
  end matchcontinue;
end comp_ref_cstr;

protected function generate_lbinary
  input Exp.Exp in_exp1;
  input Exp.Operator in_operator2;
  input Exp.Exp in_exp3;
  input Integer in_integer4;
  output CFunction out_cfunction;
  output String out_string;
  output Integer out_integer;
algorithm 
  (out_cfunction,out_string,out_integer):=
  matchcontinue (in_exp1,in_operator2,in_exp3,in_integer4)
    local
      CFunction cfn1,cfn2,cfn;
      CleanupStatement var1,var2,var;
      Integer tnr1,tnr2,tnr;
      Exp.Exp e1,e2;
    case (e1,Exp.AND,e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        var = Util.string_append_list({"(",var1," && ",var2,")"});
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var,tnr2);
    case (e1,Exp.OR,e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        var = Util.string_append_list({"(",var1," || ",var2,")"});
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var,tnr2);
    case (_,_,_,_)
      equation 
        Print.print_buf("# generate_lbinary failed\n"); then fail();
  end matchcontinue;
end generate_lbinary;

protected function generate_lunary
  input Exp.Operator in_operator;
  input Exp.Exp in_exp;
  input Integer in_integer;
  output CFunction out_cfunction;
  output String out_string;
  output Integer out_integer;
algorithm 
  (out_cfunction,out_string,out_integer):=
  matchcontinue (in_operator,in_exp,in_integer)
    local
      CFunction cfn1;
      CleanupStatement var1,var;
      Integer tnr1,tnr;
      Exp.Exp e;
    case (Exp.NOT,e,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e, tnr);
        var = Util.string_append_list({"(!",var1,")"}); then (cfn1,var,tnr1);
    case (_,_,_)
      equation 
        Print.print_buf("# generate_lbinary failed\n"); then fail();
  end matchcontinue;
end generate_lunary;

protected function generate_relation
  input Exp.Exp in_exp1;
  input Exp.Operator in_operator2;
  input Exp.Exp in_exp3;
  input Integer in_integer4;
  output CFunction out_cfunction;
  output String out_string;
  output Integer out_integer;
algorithm 
  (out_cfunction,out_string,out_integer):=
  matchcontinue (in_exp1,in_operator2,in_exp3,in_integer4)
    local
      CFunction cfn1,cfn2,cfn;
      CleanupStatement var1,var2,var;
      Integer tnr1,tnr2,tnr;
      Exp.Exp e1,e2;
    case (e1,Exp.LESS(Exp.BOOL),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        var = Util.string_append_list({"(!",var1," && ",var2,")"});
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var,tnr2);
    case (e1,Exp.LESS(Exp.STRING),e2,tnr)
      equation 
        Print.print_buf("# string comparison not supported\n"); then fail();
    case (e1,Exp.LESS(Exp.INT),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        var = Util.string_append_list({"(",var1," < ",var2,")"});
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var,tnr2);
    case (e1,Exp.LESS(Exp.REAL),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        var = Util.string_append_list({"(",var1," < ",var2,")"});
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var,tnr2);
    case (e1,Exp.GREATER(Exp.BOOL),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        var = Util.string_append_list({"(",var1," && !",var2,")"});
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var,tnr2);
    case (e1,Exp.GREATER(Exp.STRING),e2,tnr)
      equation 
        Print.print_buf("# string comparison not supported\n"); then fail();
    case (e1,Exp.GREATER(Exp.INT),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        var = Util.string_append_list({"(",var1," > ",var2,")"});
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var,tnr2);
    case (e1,Exp.GREATER(Exp.REAL),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        var = Util.string_append_list({"(",var1," > ",var2,")"});
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var,tnr2);
    case (e1,Exp.LESSEQ(Exp.BOOL),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        var = Util.string_append_list({"(!",var1," || ",var2,")"});
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var,tnr2);
    case (e1,Exp.LESSEQ(Exp.STRING),e2,tnr)
      equation 
        Print.print_buf("# string comparison not supported\n"); then fail();
    case (e1,Exp.LESSEQ(Exp.INT),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        var = Util.string_append_list({"(",var1," <= ",var2,")"});
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var,tnr2);
    case (e1,Exp.LESSEQ(Exp.REAL),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        var = Util.string_append_list({"(",var1," <= ",var2,")"});
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var,tnr2);
    case (e1,Exp.GREATEREQ(Exp.BOOL),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        var = Util.string_append_list({"(",var1," || !",var2,")"});
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var,tnr2);
    case (e1,Exp.GREATEREQ(Exp.STRING),e2,tnr)
      equation 
        Print.print_buf("# string comparison not supported\n"); then fail();
    case (e1,Exp.GREATEREQ(Exp.INT),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        var = Util.string_append_list({"(",var1," >= ",var2,")"});
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var,tnr2);
    case (e1,Exp.GREATEREQ(Exp.REAL),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        var = Util.string_append_list({"(",var1," >= ",var2,")"});
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var,tnr2);
    case (e1,Exp.EQUAL(Exp.BOOL),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        var = Util.string_append_list({"((!",var1," && !",var2,") || (",var1," && ",var2,"))"});
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var,tnr2);
    case (e1,Exp.EQUAL(Exp.STRING),e2,tnr)
      equation 
        Print.print_buf("# string comparison not supported\n"); then fail();
    case (e1,Exp.EQUAL(Exp.INT),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        var = Util.string_append_list({"(",var1," == ",var2,")"});
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var,tnr2);
    case (e1,Exp.EQUAL(Exp.REAL),e2,tnr)
      equation 
        Print.print_buf("# Reals can't be compared with ==\n"); then fail();
    case (e1,Exp.NEQUAL(Exp.BOOL),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        var = Util.string_append_list({"((!",var1," && ",var2,") || (",var1," && !",var2,"))"});
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var,tnr2);
    case (e1,Exp.NEQUAL(Exp.STRING),e2,tnr)
      equation 
        Print.print_buf("# string comparison not supported\n"); then fail();
    case (e1,Exp.NEQUAL(Exp.INT),e2,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e1, tnr);
        (cfn2,var2,tnr2) = generate_expression(e2, tnr1);
        var = Util.string_append_list({"(",var1," != ",var2,")"});
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var,tnr2);
    case (e1,Exp.NEQUAL(Exp.REAL),e2,tnr)
      equation 
        Print.print_buf("# Reals can't be compared with <>\n"); then fail();
    case (_,_,_,_)
      equation 
        Print.print_buf("# generate_relation failed\n"); then fail();
  end matchcontinue;
end generate_relation;

protected function generate_matrix
  input Exp.Type in_type1;
  input Integer in_integer2;
  input TupleExp_ExpBooleanListList in_tupleexp_expbooleanlistlist3;
  input Integer in_integer4;
  output CFunction out_cfunction;
  output String out_string;
  output Integer out_integer;
protected 
  type TupleExp_ExpBoolean = tuple<Exp.Exp,Boolean>;
  type TupleExp_ExpBooleanList = list<TupleExp_ExpBoolean>;
  type TupleExp_ExpBooleanListList = list<TupleExp_ExpBooleanList>;
algorithm 
  (out_cfunction,out_string,out_integer):=
  matchcontinue (in_type1,in_integer2,in_tupleexp_expbooleanlistlist3,in_integer4)
    local
      type CleanupStatementListList = list<CleanupStatementList>;
      CFunction cfn1,cfn2,cfn_1,cfn_2,cfn;
      CleanupStatementListList vars1;
      Integer tnr1,tnr2,n,tnr3,maxn,tnr;
      CleanupStatementList vars2;
      CleanupStatement array_type_str,args_str,n_str,tdecl,tvar,stmt;
      Exp.Type typ;
      TupleExp_ExpBooleanListList exps;
    case (typ,maxn,exps,tnr)
      equation 
        (cfn1,vars1,tnr1) = generate_matrix_expressions(typ, exps, maxn, tnr);
        (cfn2,vars2,tnr2) = concatenate_matrix_rows(typ, vars1, tnr1);
        array_type_str = exp_type_str(typ, true);
        args_str = Util.string_delimit_list(vars2, ", &");
        n = list_length(vars2);
        n_str = int_string(n);
        (tdecl,tvar,tnr3) = generate_temp_decl(array_type_str, tnr2);
        stmt = Util.string_append_list(
          {"cat_alloc_",array_type_str,"(1, &",tvar,", ",n_str,", &",
          args_str,");"});
        cfn_1 = c_add_variables(cfn2, {tdecl});
        cfn_2 = c_add_statements(cfn_1, {stmt});
        cfn = c_merge_fn(cfn1, cfn_2) "

	 Generate code for every expression and
	 promote it to maxn dimensions
	 for every row create cat(2,rowvar1,....)
	 for every column create cat(1,row1,....)

	 "; then (cfn,tvar,tnr3);
  end matchcontinue;
end generate_matrix;

protected function concatenate_matrix_rows
  input Exp.Type in_type;
  input StringListList in_stringlistlist;
  input Integer in_integer;
  output CFunction out_cfunction;
  output StringList out_stringlist;
  output Integer out_integer;
protected 
  type StringList = list<String>;
  type StringListList = list<StringList>;
algorithm 
  (out_cfunction,out_stringlist,out_integer):=
  matchcontinue (in_type,in_stringlistlist,in_integer)
    local
      type CleanupStatementListList = list<CleanupStatementList>;
      Integer tnr,tnr1,tnr2;
      CFunction cfn1,cfn2,cfn;
      CleanupStatement var1;
      CleanupStatementList vars2,f;
      Exp.Type typ;
      CleanupStatementListList r;
    case (_,{},tnr) then (c_empty_function,{},tnr); 
    case (typ,f :: r,tnr)
      equation 
        (cfn1,var1,tnr1) = concatenate_matrix_row(typ, f, tnr);
        (cfn2,vars2,tnr2) = concatenate_matrix_rows(typ, r, tnr1);
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var1 :: vars2,tnr2);
  end matchcontinue;
end concatenate_matrix_rows;

protected function concatenate_matrix_row
  input Exp.Type in_type;
  input StringList in_stringlist;
  input Integer in_integer;
  output CFunction out_cfunction;
  output String out_string;
  output Integer out_integer;
protected 
  type StringList = list<String>;
algorithm 
  (out_cfunction,out_string,out_integer):=
  matchcontinue (in_type,in_stringlist,in_integer)
    local
      CleanupStatement array_type_str,args_str,n_str,tdecl,tvar,stmt;
      Integer n,tnr1,tnr;
      CFunction cfn_1,cfn;
      Exp.Type typ;
      CleanupStatementList vars;
    case (typ,vars,tnr)
      equation 
        array_type_str = exp_type_str(typ, true);
        args_str = Util.string_delimit_list(vars, ", &");
        n = list_length(vars);
        n_str = int_string(n);
        (tdecl,tvar,tnr1) = generate_temp_decl(array_type_str, tnr);
        stmt = Util.string_append_list(
          {"cat_alloc_",array_type_str,"(2, &",tvar,", ",n_str,", &",
          args_str,");"});
        cfn_1 = c_add_variables(c_empty_function, {tdecl});
        cfn = c_add_statements(cfn_1, {stmt}); then (cfn,tvar,tnr1);
  end matchcontinue;
end concatenate_matrix_row;

protected function generate_matrix_expressions
  input Exp.Type in_type1;
  input TupleExp_ExpBooleanListList in_tupleexp_expbooleanlistlist2;
  input Integer in_integer3;
  input Integer in_integer4;
  output CFunction out_cfunction;
  output StringListList out_stringlistlist;
  output Integer out_integer;
protected 
  type TupleExp_ExpBoolean = tuple<Exp.Exp,Boolean>;
  type TupleExp_ExpBooleanList = list<TupleExp_ExpBoolean>;
  type TupleExp_ExpBooleanListList = list<TupleExp_ExpBooleanList>;
  type StringList = list<String>;
  type StringListList = list<StringList>;
algorithm 
  (out_cfunction,out_stringlistlist,out_integer):=
  matchcontinue (in_type1,in_tupleexp_expbooleanlistlist2,in_integer3,in_integer4)
    local
      type CleanupStatementListList = list<CleanupStatementList>;
      Integer tnr,tnr1,tnr2,maxn;
      CFunction cfn1,cfn2,cfn;
      CleanupStatementList vars1;
      CleanupStatementListList vars2;
      Exp.Type typ;
      TupleExp_ExpBooleanList fr;
      TupleExp_ExpBooleanListList rr;
    case (_,{},_,tnr) then (c_empty_function,{},tnr); 
    case (typ,fr :: rr,maxn,tnr)
      equation 
        (cfn1,vars1,tnr1) = generate_matrix_expr_row(typ, fr, maxn, tnr);
        (cfn2,vars2,tnr2) = generate_matrix_expressions(typ, rr, maxn, tnr1);
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,vars1 :: vars2,tnr2);
  end matchcontinue;
end generate_matrix_expressions;

protected function generate_matrix_expr_row
  input Exp.Type in_type1;
  input TupleExp_ExpBooleanList in_tupleexp_expbooleanlist2;
  input Integer in_integer3;
  input Integer in_integer4;
  output CFunction out_cfunction;
  output StringList out_stringlist;
  output Integer out_integer;
protected 
  type TupleExp_ExpBoolean = tuple<Exp.Exp,Boolean>;
  type TupleExp_ExpBooleanList = list<TupleExp_ExpBoolean>;
  type StringList = list<String>;
algorithm 
  (out_cfunction,out_stringlist,out_integer):=
  matchcontinue (in_type1,in_tupleexp_expbooleanlist2,in_integer3,in_integer4)
    local
      Integer tnr,tnr1,tnr2,maxn;
      CFunction cfn1,cfn2,cfn;
      CleanupStatement var1;
      CleanupStatementList vars2;
      Exp.Type t;
      TupleExp_ExpBoolean f;
      TupleExp_ExpBooleanList r;
    case (_,{},_,tnr) then (c_empty_function,{},tnr); 
    case (t,f :: r,maxn,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_matrix_expression(t, f, maxn, tnr);
        (cfn2,vars2,tnr2) = generate_matrix_expr_row(t, r, maxn, tnr1);
        cfn = c_merge_fn(cfn1, cfn2); then (cfn,var1 :: vars2,tnr2);
  end matchcontinue;
end generate_matrix_expr_row;

protected function generate_matrix_expression
  input Exp.Type in_type1;
  input TupleExp_ExpBoolean in_tupleexp_expboolean2;
  input Integer in_integer3;
  input Integer in_integer4;
  output CFunction out_cfunction;
  output String out_string;
  output Integer out_integer;
protected 
  type TupleExp_ExpBoolean = tuple<Exp.Exp,Boolean>;
algorithm 
  (out_cfunction,out_string,out_integer):=
  matchcontinue (in_type1,in_tupleexp_expboolean2,in_integer3,in_integer4)
    local
      CFunction cfn1,cfn_1,cfn;
      CleanupStatement var1,array_type_str,maxn_str,tdecl,tvar,scalar,sc_ref,stmt;
      Integer tnr1,tnr2,maxn,tnr;
      Exp.Type t;
      Exp.Exp e;
      Boolean b;
    case (t,(e,b),maxn,tnr)
      equation 
        (cfn1,var1,tnr1) = generate_expression(e, tnr);
        array_type_str = exp_type_str(t, true);
        maxn_str = int_string(maxn);
        (tdecl,tvar,tnr2) = generate_temp_decl(array_type_str, tnr1);
        scalar = Util.if_(b, "", "scalar_");
        sc_ref = Util.if_(b, "&", "");
        stmt = Util.string_append_list(
          {"promote_",scalar,array_type_str,"(",sc_ref,var1,", ",
          maxn_str,", &",tvar,");"});
        cfn_1 = c_add_variables(cfn1, {tdecl});
        cfn = c_add_statements(cfn_1, {stmt}); then (cfn,tvar,tnr2);
  end matchcontinue;
end generate_matrix_expression;

protected function generate_read_call_write
  input String fnname;
  input DAE_ElementList outvars;
  input String retstr;
  input DAE_ElementList invars;
  output CFunction cfn;
protected 
  type DAE_ElementList = list<DAE.Element>;
  CleanupStatement rcw_fnname,out_decl,in_args,fn_call;
  CFunction cfn1,cfn1_1,cfn31,cfn32,cfn3,cfn3_1,cfn4,cfn4_1,cfn5,cfn5_1;
  Integer tnr21,tnr2;
  CleanupStatementList in_names;
algorithm 
  Debug.fprintln("cgtr", "generate_read_call_write");
  rcw_fnname := string_append(fnname, "_read_call_write");
  cfn1 := c_make_function("int", rcw_fnname, {}, 
          {"char const* in_filename","char const* out_filename"});
  out_decl := Util.string_append_list({retstr," out;"});
  cfn1_1 := c_add_inits(cfn1, {"PRE_VARIABLES",out_decl});
  (cfn31,tnr21) := generate_var_decls(invars, is_rcw_input, 1) "generate_vars(outvars,is_rcw_output,1) => (cfn2,tnr1) &";
  (cfn32,tnr2) := generate_var_inits(invars, is_rcw_input, tnr21, "");
  cfn3 := c_merge_fns({cfn31,cfn32});
  cfn3_1 := c_add_inits(cfn3, {"PRE_OPEN_INFILE"});
  in_names := invar_names(invars);
  in_args := Util.string_delimit_list(in_names, ", ");
  cfn4 := generate_read(invars);
  fn_call := Util.string_append_list({"out = ",fnname,"(",in_args,");"});
  cfn4_1 := c_add_statements(cfn4, {"PRE_READ_DONE",fn_call,"PRE_OPEN_OUTFILE"});
  cfn5 := generate_write(outvars);
  cfn5_1 := c_add_statements(cfn5, {"PRE_WRITE_DONE","return 0;"});
  cfn := c_merge_fns({cfn1_1,cfn3_1,cfn4_1,cfn5_1});
end generate_read_call_write;

protected function generate_read_call_write_external
  input String in_string1;
  input DAE_ElementList in_dae_elementlist2;
  input String in_string3;
  input DAE_ElementList in_dae_elementlist4;
  input DAE.ExternalDecl in_externaldecl5;
  input DAE_ElementList in_dae_elementlist6;
  output CFunction out_cfunction;
protected 
  type DAE_ElementList = list<DAE.Element>;
algorithm 
  out_cfunction:=
  matchcontinue (in_string1,in_dae_elementlist2,in_string3,in_dae_elementlist4,in_externaldecl5,in_dae_elementlist6)
    local
      Integer tnr,tnr_ret,tnr_bialloc_1,tnr_bialloc,tnr_mem,tnr_invars1,tnr_invars,tnr_bivars1,tnr_bivars,tnr_extcall;
      CleanupStatement rcw_fnname,out_decl,mem_decl,mem_var,get_mem_stmt,rest_mem_stmt,fnname,retstr;
      CFunction cfn1,cfn1_1,allocstmts_1,allocstmts,biallocstmts,cfnoutinit,cfnoutbialloc,mem_fn_1,mem_fn_2,mem_fn,cfn31,cfn32,cfn33,cfn34,cfn3,cfn3_1,readinvars,readdone,extcall,cfn4_1,cfn5,cfn5_1,cfn_1,cfn;
      DAE_ElementList vars_1,vars,outvars,invars,bivars;
      DAE.ExternalDecl extdecl;
    case (fnname,outvars,retstr,invars,extdecl,bivars)
      equation 
        Debug.fprintln("cgtr", "generate_read_call_write_external");
        tnr = 1;
        rcw_fnname = string_append(fnname, "_read_call_write");
        cfn1 = c_make_function("int", rcw_fnname, {}, 
          {"char const* in_filename","char const* out_filename"});
        out_decl = Util.string_append_list({retstr," out;"});
        cfn1_1 = c_add_inits(cfn1, {"PRE_VARIABLES"});
        (allocstmts_1,tnr_ret) = generate_alloc_outvars_ext(outvars, "out", tnr, extdecl) "generate_vars(outvars,is_rcw_output,1) => (cfn2,tnr1) &";
        allocstmts = c_add_variables(allocstmts_1, {out_decl});
        (biallocstmts,tnr_bialloc_1) = generate_alloc_outvars_ext(bivars, "", tnr_ret, extdecl);
        (cfnoutinit,tnr_bialloc) = generate_var_inits(outvars, is_rcw_output, tnr_bialloc_1, "out");
        cfnoutbialloc = c_merge_fns({allocstmts,biallocstmts,cfnoutinit});
        (mem_decl,mem_var,tnr_mem) = generate_temp_decl("state", tnr_bialloc);
        get_mem_stmt = Util.string_append_list({mem_var," = get_memory_state();"});
        rest_mem_stmt = Util.string_append_list({"restore_memory_state(",mem_var,");"});
        mem_fn_1 = c_add_variables(c_empty_function, {mem_decl});
        mem_fn_2 = c_add_inits(mem_fn_1, {get_mem_stmt});
        mem_fn = c_merge_fns({mem_fn_2,cfnoutbialloc});
        (cfn31,tnr_invars1) = generate_var_decls(invars, is_rcw_input, tnr_mem);
        (cfn32,tnr_invars) = generate_var_inits(invars, is_rcw_input, tnr_invars1, "");
        (cfn33,tnr_bivars1) = generate_var_decls(bivars, is_rcw_bidir, tnr_invars);
        (cfn34,tnr_bivars) = generate_var_inits(bivars, is_rcw_bidir, tnr_bivars1, "");
        cfn3 = c_merge_fns({cfn31,cfn32,cfn33,cfn34});
        cfn3_1 = c_add_inits(cfn3, {"PRE_OPEN_INFILE"});
        readinvars = generate_read(invars);
        readdone = c_add_inits(readinvars, {"PRE_READ_DONE"});
        vars_1 = list_append(invars, outvars);
        vars = list_append(vars_1, bivars);
        (extcall,tnr_extcall) = generate_ext_call(vars, extdecl, tnr_bivars) "
	generate_var_inits(outvars, tnr_bivars) => (cfnvarinits1, tnr_varinits1) &
	generate_var_inits(bivars, tnr_varinits1) => (cfnvarinits2, tnr_varinits2) &

	c_merge_fns({cfnvarinits1,cfnvarinits2}) => cfnvarinits &

	generate_ext_call (vars, extdecl, tnr_varinits2) => (extcall, tnr_extcall) &
";
        cfn4_1 = c_add_statements(extcall, {"PRE_OPEN_OUTFILE"});
        cfn5 = generate_write(outvars);
        cfn5_1 = c_add_statements(cfn5, {"PRE_WRITE_DONE"});
        cfn_1 = c_merge_fns({cfn1_1,cfn3_1,readdone,mem_fn,cfn4_1,cfn5_1}) "	c_merge_fns({cfn1',cfn3',readdone,mem_fn,biallocstmts,cfnvarinits,cfn4',cfn5'}) => cfn' & ";
        cfn = c_add_cleanups(cfn_1, {rest_mem_stmt,"return 0;"}); then cfn;
    case (_,_,_,_,_,_)
      equation 
        Print.print_buf("#-- generate_read_call_write_external failed\n"); then fail();
  end matchcontinue;
end generate_read_call_write_external;

protected function generate_ext_call
  input DAE_ElementList in_dae_elementlist;
  input DAE.ExternalDecl in_externaldecl;
  input Integer in_integer;
  output CFunction out_cfunction;
  output Integer out_integer;
protected 
  type DAE_ElementList = list<DAE.Element>;
algorithm 
  (out_cfunction,out_integer):=
  matchcontinue (in_dae_elementlist,in_externaldecl,in_integer)
    local
      type DAE_ExtArgList = list<DAE.ExtArg>;
      CleanupStatement extdeclstr,n,lang;
      CFunction argdecls,fcall,argcopies,extcall;
      DAE_ExtArgList arglist_1,outbiarglist,arglist;
      Integer tnr_1,tnr_2,tnr;
      DAE_ElementList vars;
      DAE.ExternalDecl extdecl;
      DAE.ExtArg retarg;
    case (vars,(extdecl = DAE.EXTERNALDECL(n,arglist,retarg,lang)),tnr)
      equation 
        Debug.fcall("cgtrdumpdaeextcall", DAE.dump2, DAE.DAE(vars));
        extdeclstr = DAE.dump_ext_decl_str(extdecl);
        Debug.fprintln("cgtrdumpdaeextcall", extdeclstr);
        (argdecls,arglist_1,tnr_1) = generate_extcall_vardecls(vars, arglist, retarg, lang, tnr);
        fcall = generate_ext_call_fcall(n, arglist_1, retarg, lang);
        outbiarglist = Util.list_matching(arglist_1, is_extarg_output_or_bidir);
        (argcopies,tnr_2) = generate_extcall_varcopy(outbiarglist, retarg, lang, tnr_1);
        extcall = c_merge_fns({argdecls,fcall,argcopies}); then (extcall,tnr_2);
    case (_,_,_)
      equation 
        Print.print_buf("#-- generate_ext_call failed\n"); then fail();
  end matchcontinue;
end generate_ext_call;

protected function generate_extcall_vardecls
  input DAE_ElementList in_dae_elementlist;
  input DAE_ExtArgList in_dae_extarglist;
  input DAE.ExtArg in_extarg;
  input String in_string;
  input Integer in_integer;
  output CFunction out_cfunction;
  output DAE_ExtArgList out_dae_extarglist;
  output Integer out_integer;
protected 
  type DAE_ElementList = list<DAE.Element>;
  type DAE_ExtArgList = list<DAE.ExtArg>;
algorithm 
  (out_cfunction,out_dae_extarglist,out_integer):=
  matchcontinue (in_dae_elementlist,in_dae_extarglist,in_extarg,in_string,in_integer)
    local
      CFunction decls,copydecls,res;
      DAE_ElementList vars;
      DAE_ExtArgList args,args_1;
      DAE.ExtArg retarg;
      Integer tnr,tnr_1,tnr_2;
    case (vars,args,retarg,"C",tnr)
      equation 
        (decls) = generate_extcall_vardecls2(args, retarg); then (decls,args,tnr);
    case (vars,args,retarg,"FORTRAN 77",tnr)
      equation 
        (copydecls,tnr_1) = generate_extcall_copydecls_f77(vars, tnr);
        (decls,args_1,tnr_2) = generate_extcall_vardecls2_f77(args, retarg, tnr_1);
        res = c_merge_fn(copydecls, decls); then (res,args_1,tnr_2);
    case (_,_,_,_,_)
      equation 
        Print.print_buf("#-- generate_extcall_vardecls failed\n"); then fail();
  end matchcontinue;
end generate_extcall_vardecls;

protected function generate_extcall_copydecls_f77
  input DAE_ElementList in_dae_elementlist;
  input Integer in_integer;
  output CFunction out_cfunction;
  output Integer out_integer;
protected 
  type DAE_ElementList = list<DAE.Element>;
algorithm 
  (out_cfunction,out_integer):=
  matchcontinue (in_dae_elementlist,in_integer)
    local
      type Exp_ExpOption = Option<Exp.Exp>;
      type Exp_SubscriptList = list<Exp.Subscript>;
      Integer tnr,tnr_1,tnr_3;
      Exp.ComponentRef cref,cref_1;
      DAE.VarKind vk;
      DAE.VarDirection vd;
      DAE.Type ty;
      Exp_ExpOption value,start;
      Exp_SubscriptList dims,dims_1;
      DAE.Element extvar,var;
      CFunction fn,restfn,resfn;
      DAE_ElementList rest;
    case ({},tnr) then (c_empty_function,tnr); 
    case (var :: rest,tnr)
      equation 
        DAE.VAR(cref,vk,vd,ty,value,dims,start,_,_,_,_) = var;
        true = is_array(var);
        cref_1 = var_name_external_cref(cref);
        dims_1 = list_reverse(dims);
        extvar = DAE.VAR(cref_1,vk,vd,ty,value,dims_1,NONE,DAE.NON_FLOW(),{},NONE,
          NONE);
        (fn,tnr_1) = generate_var_decl(extvar, tnr);
        (restfn,tnr_3) = generate_extcall_copydecls_f77(rest, tnr_1);
        resfn = c_merge_fn(fn, restfn); then (resfn,tnr_3);
    case (var :: rest,tnr)
      equation 
        Debug.fprint("cgtr", "#--Ignoring: ");
        Debug.fcall("cgtr", DAE.dump2, DAE.DAE({var}));
        Debug.fprintln("cgtr", "");
        (fn,tnr_1) = generate_extcall_copydecls_f77(rest, tnr); then (fn,tnr_1);
  end matchcontinue;
end generate_extcall_copydecls_f77;

protected function generate_extcall_vardecls2
  input DAE_ExtArgList in_dae_extarglist;
  input DAE.ExtArg in_extarg;
  output CFunction out_cfunction;
protected 
  type DAE_ExtArgList = list<DAE.ExtArg>;
algorithm 
  out_cfunction:=
  matchcontinue (in_dae_extarglist,in_extarg)
    local
      CFunction retdecl,decl,decls,res;
      DAE.ExtArg retarg,var;
      DAE_ExtArgList rest;
    case ({},DAE.NOEXTARG()) then c_empty_function; 
    case ({},retarg)
      equation 
        retdecl = generate_extcall_vardecl(retarg); then retdecl;
    case (var :: rest,retarg)
      equation 
        decl = generate_extcall_vardecl(var);
        decls = generate_extcall_vardecls2(rest, retarg);
        res = c_merge_fn(decl, decls); then res;
    case (_,_)
      equation 
        Print.print_buf("#-- generate_extcall_vardecls2 failed\n"); then fail();
  end matchcontinue;
end generate_extcall_vardecls2;

protected function generate_vardecl_func
  input String in_string1;
  input String in_string2;
  input StringOption in_stringoption3;
  output CFunction out_cfunction;
protected 
  type StringOption = Option<String>;
algorithm 
  out_cfunction:=
  matchcontinue (in_string1,in_string2,in_stringoption3)
    local
      CleanupStatement str,str3,tystr,name,expr;
      CFunction f1,res;
    case (tystr,name,SOME(expr))
      equation 
        str = Util.string_append_list({tystr," ",name,";"});
        f1 = c_add_variables(c_empty_function, {str});
        str3 = Util.string_append_list({name," = (",tystr,")",expr,";"});
        res = c_add_statements(f1, {str3}); then res;
    case (tystr,name,NONE)
      equation 
        str = Util.string_append_list({tystr," ",name,";"});
        res = c_add_variables(c_empty_function, {str}); then res;
  end matchcontinue;
end generate_vardecl_func;

protected function generate_extcall_vardecl
  input DAE.ExtArg in_extarg;
  output CFunction out_cfunction;
algorithm 
  out_cfunction:=
  matchcontinue (in_extarg)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      Exp.ComponentRef cref,cr;
      TupleTypes_TTypeAbsyn_PathOption ty;
      CleanupStatement tystr,name,orgname;
      CFunction res;
      DAE.ExtArg arg;
      Types.Attributes attr;
      Exp.Exp exp;
    case arg " INPUT NON-ARRAY "
      equation 
        DAE.EXTARG(cref,Types.ATTR(_,_,_,Absyn.INPUT),ty) = arg;
        false = Types.is_array(ty);
        false = Types.is_string(ty);
        tystr = generate_type_external(ty);
        name = var_name_external(cref);
        (orgname,_) = comp_ref_cstr(cref);
        res = generate_vardecl_func(tystr, name, SOME(orgname)); then res;
    case arg " INPUT NON-ARRAY STRING   do nothing   INPUT ARRAY "
      equation 
        DAE.EXTARG(cref,Types.ATTR(_,_,_,Absyn.INPUT),ty) = arg;
        true = Types.is_array(ty); then c_empty_function;
    case arg " OUTPUT NON-ARRAY "
      equation 
        DAE.EXTARG(cref,Types.ATTR(_,_,_,Absyn.OUTPUT),ty) = arg;
        false = Types.is_array(ty);
        tystr = generate_type_external(ty);
        name = var_name_external(cref);
        res = generate_vardecl_func(tystr, name, NONE); then res;
    case arg " OUTPUT ARRAY "
      equation 
        DAE.EXTARG(cref,Types.ATTR(_,_,_,Absyn.OUTPUT),ty) = arg;
        true = Types.is_array(ty); then c_empty_function;
    case DAE.EXTARG(cr,attr,ty) then c_empty_function; 
    case DAE.EXTARGEXP(exp,ty) then c_empty_function; 
    case DAE.EXTARGSIZE(_,_,_,_) then c_empty_function; 
    case _
      equation 
        Print.print_buf("#-- generate_extcall_vardecl failed\n"); then fail();
  end matchcontinue;
end generate_extcall_vardecl;

protected function generate_extcall_vardecls2_f77
  input DAE_ExtArgList in_dae_extarglist;
  input DAE.ExtArg in_extarg;
  input Integer in_integer;
  output CFunction out_cfunction;
  output DAE_ExtArgList out_dae_extarglist;
  output Integer out_integer;
protected 
  type DAE_ExtArgList = list<DAE.ExtArg>;
algorithm 
  (out_cfunction,out_dae_extarglist,out_integer):=
  matchcontinue (in_dae_extarglist,in_extarg,in_integer)
    local
      Integer tnr,tnr_1,tnr_2;
      CFunction retdecl,decl,decls,res;
      DAE.ExtArg retarg,var_1,var;
      DAE_ExtArgList varr,rest;
    case ({},DAE.NOEXTARG(),tnr) then (c_empty_function,{},tnr); 
    case ({},retarg,tnr)
      equation 
        (retdecl,_,tnr_1) = generate_extcall_vardecl_f77(retarg, tnr); then (retdecl,{},tnr_1);
    case (var :: rest,retarg,tnr)
      equation 
        (decl,var_1,tnr_1) = generate_extcall_vardecl_f77(var, tnr);
        (decls,varr,tnr_2) = generate_extcall_vardecls2_f77(rest, retarg, tnr_1);
        res = c_merge_fn(decl, decls); then (res,var_1 :: varr,tnr_2);
    case (_,_,_)
      equation 
        Print.print_buf("#-- generate_extcall_vardecls2_f77 failed\n"); then fail();
  end matchcontinue;
end generate_extcall_vardecls2_f77;

protected function generate_c_to_f77_converter
  input Types.Type in_type;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_type)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      TupleTypes_TTypeAbsyn_PathOption elty,ty;
      CleanupStatement eltystr,str;
    case ((ty = (Types.T_ARRAY(_,_),_)))
      equation 
        elty = Types.array_element_type(ty);
        eltystr = generate_type_internal_namepart(elty);
        str = Util.string_append_list({"convert_alloc_",eltystr,"_array_to_f77"}); then str;
    case ty
      equation 
        Print.print_buf("#-- generate_c_to_f77_converter failed\n");
        Print.print_buf("#-- Not an array?\n");
        Types.print_type(ty); then fail();
  end matchcontinue;
end generate_c_to_f77_converter;

protected function generate_f77_to_c_converter
  input Types.Type in_type;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_type)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      TupleTypes_TTypeAbsyn_PathOption elty,ty;
      CleanupStatement eltystr,str;
    case ((ty = (Types.T_ARRAY(_,_),_)))
      equation 
        elty = Types.array_element_type(ty);
        eltystr = generate_type_internal_namepart(elty);
        str = Util.string_append_list({"convert_alloc_",eltystr,"_array_from_f77"}); then str;
    case _
      equation 
        Print.print_buf("#-- generate_f77_to_c_converter failed\n");
        Print.print_buf("#-- Not an array?\n"); then fail();
  end matchcontinue;
end generate_f77_to_c_converter;

protected function is_output_or_bidir
  input Types.Attributes attr;
  output Boolean res;
protected 
  Boolean outvar,bivar;
algorithm 
  outvar := Types.is_output_attr(attr);
  bivar := Types.is_bidir_attr(attr);
  res := bool_or(outvar, bivar);
end is_output_or_bidir;

protected function generate_extcall_vardecl_f77
  input DAE.ExtArg in_extarg;
  input Integer in_integer;
  output CFunction out_cfunction;
  output DAE.ExtArg out_extarg;
  output Integer out_integer;
algorithm 
  (out_cfunction,out_extarg,out_integer):=
  matchcontinue (in_extarg,in_integer)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      Exp.ComponentRef cref,cr,tmpcref;
      Types.Attributes attr;
      TupleTypes_TTypeAbsyn_PathOption ty;
      CFunction res,decl;
      DAE.ExtArg arg,extarg,newarg;
      Integer tnr,tnr_1;
      CleanupStatement tystr,name,orgname,converter,initstr,tmpname_1,tnrstr,tmpstr,callstr,declstr,argstr;
      Exp.Exp exp,dim;
    case (arg,tnr) " INPUT NON-ARRAY "
      equation 
        DAE.EXTARG(cref,attr,ty) = arg;
        Debug.fprintln("cgtr", "generate_extcall_vardecl_f77_1");
        true = Types.is_input_attr(attr);
        false = Types.is_array(ty);
        res = generate_extcall_vardecl(arg); then (res,arg,tnr);
    case (extarg,tnr) " INPUT ARRAY "
      equation 
        DAE.EXTARG(cref,attr,ty) = extarg;
        Debug.fprintln("cgtr", "generate_extcall_vardecl_f77_2");
        true = Types.is_input_attr(attr);
        true = Types.is_array(ty);
        tystr = generate_type(ty);
        name = var_name_external(cref);
        (orgname,_) = comp_ref_cstr(cref);
        converter = generate_c_to_f77_converter(ty);
        initstr = Util.string_append_list({converter,"(&",orgname,", &",name,");"});
        res = c_add_statements(c_empty_function, {initstr}); then (res,extarg,tnr);
    case (arg,tnr) " OUTPUT NON-ARRAY "
      equation 
        DAE.EXTARG(cref,attr,ty) = arg;
        Debug.fprintln("cgtr", "generate_extcall_vardecl_f77_3");
        true = Types.is_output_attr(attr);
        false = Types.is_array(ty);
        res = generate_extcall_vardecl(arg); then (res,arg,tnr);
    case (extarg,tnr) " OUTPUT ARRAY "
      equation 
        DAE.EXTARG(cref,attr,ty) = extarg;
        Debug.fprintln("cgtr", "generate_extcall_vardecl_f77_4");
        true = Types.is_array(ty);
        true = Types.is_output_attr(attr);
        tystr = generate_type(ty);
        name = var_name_external(cref);
        (orgname,_) = comp_ref_cstr(cref);
        converter = generate_c_to_f77_converter(ty);
        initstr = Util.string_append_list({converter,"(&out.",orgname,", &",name,");"});
        res = c_add_statements(c_empty_function, {initstr}); then (res,extarg,tnr);
    case (arg,tnr) " INPUT/OUTPUT ARRAY "
      equation 
        DAE.EXTARG(cref,attr,ty) = arg;
        true = Types.is_array(ty);
        Debug.fprintln("cgtr", "generate_extcall_vardecl_f77_41");
        tystr = generate_type(ty);
        name = var_name_external(cref);
        (orgname,_) = comp_ref_cstr(cref);
        converter = generate_c_to_f77_converter(ty);
        initstr = Util.string_append_list({converter,"(&",orgname,", &",name,");"});
        res = c_add_statements(c_empty_function, {initstr}); then (res,arg,tnr);
    case (arg,tnr) " INPUT/OUTPUT NON-ARRAY "
      equation 
        DAE.EXTARG(cref,attr,ty) = arg;
        false = Types.is_array(ty);
        Debug.fprintln("cgtr", "generate_extcall_vardecl_f77_41");
        res = generate_extcall_vardecl(arg); then (res,arg,tnr);
    case (arg,tnr)
      equation 
        DAE.EXTARGEXP(exp,ty) = arg;
        res = generate_extcall_vardecl(arg); then (res,arg,tnr);
    case (arg,tnr) " SIZE "
      equation 
        DAE.EXTARGSIZE(cr,attr,ty,dim) = arg;
        Debug.fprintln("cgtr", "generate_extcall_vardecl_f77_5");
        tmpname_1 = var_name_array(cr, attr);
        tnrstr = int_string(tnr);
        tnr_1 = tnr + 1;
        tmpstr = Util.string_append_list({tmpname_1,"_size_",tnrstr});
        tmpcref = Exp.CREF_IDENT(tmpstr,{});
        callstr = generate_ext_array_size_call(arg);
        declstr = Util.string_append_list({"int ",tmpstr,";"});
        decl = c_add_variables(c_empty_function, {declstr});
        callstr = Util.string_append_list({tmpstr," = ",callstr,";"});
        res = c_add_statements(decl, {callstr});
        newarg = DAE.EXTARGSIZE(tmpcref,attr,ty,dim); then (res,newarg,tnr_1);
    case (arg,_)
      equation 
        Print.print_buf("#-- generate_extcall_vardecl_f77 failed\n#-- ");
        argstr = DAE.dump_ext_arg_str(arg);
        Print.print_buf(argstr);
        Print.print_buf("\n"); then fail();
  end matchcontinue;
end generate_extcall_vardecl_f77;

protected function generate_ext_call_fcall " input string is language "
  input Ident in_ident;
  input DAE_ExtArgList in_dae_extarglist;
  input DAE.ExtArg in_extarg;
  input String in_string;
  output CFunction out_cfunction;
protected 
  type DAE_ExtArgList = list<DAE.ExtArg>;
algorithm 
  out_cfunction:=
  matchcontinue (in_ident,in_dae_extarglist,in_extarg,in_string)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      CleanupStatement fcall2,str,fnname,lang,crstr;
      CFunction res;
      DAE_ExtArgList args;
      Exp.ComponentRef cr;
      TupleTypes_TTypeAbsyn_PathOption ty;
    case (fnname,args,DAE.NOEXTARG(),lang) " call without return value "
      equation 
        fcall2 = generate_ext_call_fcall2(fnname, args, lang);
        str = string_append(fcall2, ";");
        res = c_add_statements(c_empty_function, {str}); then res;
    case (fnname,args,DAE.EXTARG(cr,_,ty),lang) " return value assignment, shouldn't happen for arrays "
      equation 
        false = Types.is_array(ty);
        fcall2 = generate_ext_call_fcall2(fnname, args, lang);
        crstr = var_name_external(cr);
        str = Util.string_append_list({crstr," = ",fcall2,";"});
        res = c_add_statements(c_empty_function, {str}); then res;
    case (_,_,_,_)
      equation 
        Print.print_buf("#-- generate_ext_call_fcall failed\n"); then fail();
  end matchcontinue;
end generate_ext_call_fcall;

protected function generate_ext_call_fcall2
  input Ident in_ident;
  input DAE_ExtArgList in_dae_extarglist;
  input String in_string;
  output String out_string;
protected 
  type DAE_ExtArgList = list<DAE.ExtArg>;
algorithm 
  out_string:=
  matchcontinue (in_ident,in_dae_extarglist,in_string)
    local
      CleanupStatementList strlist;
      CleanupStatement str,res,n;
      DAE_ExtArgList args;
    case (n,args,"C")
      equation 
        strlist = Util.list_map(args, generate_ext_call_fcall_arg);
        str = Util.string_delimit_list(strlist, ", ");
        res = Util.string_append_list({n,"(",str,")"}); then res;
    case (n,args,"FORTRAN 77")
      equation 
        strlist = Util.list_map(args, generate_ext_call_fcall_arg_f77);
        str = Util.string_delimit_list(strlist, ", ");
        res = Util.string_append_list({n,"_(",str,")"}); then res;
    case (_,_,_)
      equation 
        Print.print_buf("#-- generate_ext_call_fcall2 failed\n"); then fail();
  end matchcontinue;
end generate_ext_call_fcall2;

protected function generate_ext_call_fcall_arg " HERE!!! is_array AND is_string
   string has also .data
"
  input DAE.ExtArg in_extarg;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_extarg)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      Exp.ComponentRef cref;
      Types.Attributes attr;
      TupleTypes_TTypeAbsyn_PathOption ty;
      CleanupStatement res,name,str;
      DAE.ExtArg arg;
      Exp.Exp exp;
    case arg " INPUT NON-ARRAY NON-STRING "
      equation 
        DAE.EXTARG(cref,attr,ty) = arg;
        true = Types.is_input_attr(attr);
        false = Types.is_array(ty);
        false = Types.is_string(ty);
        res = var_name_external(cref); then res;
    case arg " OUTPUT NON-ARRAY "
      equation 
        DAE.EXTARG(cref,attr,ty) = arg;
        false = Types.is_array(ty);
        true = Types.is_output_attr(attr);
        name = var_name_external(cref);
        res = string_append("&", name); then res;
    case arg " INPUT/OUTPUT ARRAY "
      equation 
        DAE.EXTARG(cref,attr,ty) = arg;
        true = Types.is_array(ty);
        name = var_name_array(cref, attr);
        res = string_append(name, ".data"); then res;
    case arg " INPUT/OUTPUT STRING "
      equation 
        DAE.EXTARG(cref,attr,ty) = arg;
        true = Types.is_string(ty);
        (name,_) = comp_ref_cstr(cref);
        res = string_append(name, ".data"); then res;
    case arg " INPUT/OUTPUT NON-ARRAY "
      equation 
        DAE.EXTARG(cref,attr,ty) = arg;
        false = Types.is_array(ty);
        (name,_) = comp_ref_cstr(cref);
        res = string_append("&", name); then res;
    case arg
      equation 
        DAE.EXTARGEXP(exp,ty) = arg;
        (_,res,_) = generate_expression(exp, 1); then res;
    case ((arg = DAE.EXTARGSIZE(_,_,_,_))) "
  rule	var_name_external cref => name &
	string_append(\"&\", name) => res 
	-----------------------------
	generate_ext_call_fcall_arg DAE.EXTARG(cref, Types.ATTR(_,_,_,Absyn.BIDIR), ty) => res
  SIZE "
      equation 
        str = generate_array_size_call(arg); then str;
    case arg
      equation 
        Print.print_buf("#-- generate_ext_call_fcall_arg failed\n");
        str = DAE.dump_ext_arg_str(arg);
        Print.print_buf(str);
        Print.print_buf("\n"); then fail();
  end matchcontinue;
end generate_ext_call_fcall_arg;

protected function generate_ext_call_fcall_arg_f77
  input DAE.ExtArg in_extarg;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_extarg)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      Exp.ComponentRef cref;
      Types.Attributes attr;
      TupleTypes_TTypeAbsyn_PathOption ty;
      CleanupStatement name,res,str;
      DAE.ExtArg arg;
      Exp.Exp exp,dim;
    case arg " INPUT NON-ARRAY "
      equation 
        DAE.EXTARG(cref,attr,ty) = arg;
        true = Types.is_input_attr(attr);
        false = Types.is_array(ty);
        name = var_name_external(cref);
        res = string_append("&", name); then res;
    case arg " OUTPUT NON-ARRAY "
      equation 
        DAE.EXTARG(cref,attr,ty) = arg;
        true = Types.is_output_attr(attr);
        false = Types.is_array(ty);
        name = var_name_external(cref);
        res = string_append("&", name); then res;
    case arg " INPUT ARRAY "
      equation 
        DAE.EXTARG(cref,attr,ty) = arg;
        true = Types.is_input_attr(attr);
        true = Types.is_array(ty);
        name = var_name_external(cref);
        res = string_append(name, ".data"); then res;
    case arg " OUTPUT ARRAY "
      equation 
        DAE.EXTARG(cref,attr,ty) = arg;
        true = Types.is_output_attr(attr);
        true = Types.is_array(ty);
        name = var_name_external(cref);
        res = string_append(name, ".data"); then res;
    case arg " INPUT/OUTPUT ARRAY "
      equation 
        DAE.EXTARG(cref,attr,ty) = arg;
        true = Types.is_array(ty);
        name = var_name_external(cref);
        res = string_append(name, ".data"); then res;
    case arg " INPUT/OUTPUT NON-ARRAY "
      equation 
        DAE.EXTARG(cref,attr,ty) = arg;
        false = Types.is_array(ty);
        (name,_) = comp_ref_cstr(cref);
        res = string_append("&", name); then res;
    case arg
      equation 
        DAE.EXTARGEXP(exp,ty) = arg;
        res = generate_ext_call_fcall_arg(arg); then res;
    case DAE.EXTARGSIZE(cref,attr,ty,dim) " SIZE "
      equation 
        (name,_) = comp_ref_cstr(cref);
        res = string_append("&", name); then res;
    case arg
      equation 
        Print.print_buf("#-- generate_ext_call_fcall_arg_f77 failed\n#-- ");
        str = DAE.dump_ext_arg_str(arg);
        Print.print_buf(str);
        Print.print_buf("\n"); then fail();
  end matchcontinue;
end generate_ext_call_fcall_arg_f77;

protected function generate_array_size_call
  input DAE.ExtArg in_extarg;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_extarg)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      CleanupStatement crstr,dimstr,str;
      Exp.ComponentRef cr;
      Types.Attributes attr;
      TupleTypes_TTypeAbsyn_PathOption ty;
      Exp.Exp dim;
    case DAE.EXTARGSIZE(cr,attr,ty,dim)
      equation 
        ((Types.T_INTEGER(_),_)) = Types.array_element_type(ty);
        crstr = var_name_array(cr, attr);
        dimstr = Exp.print_exp_str(dim);
        str = Util.string_append_list({"size_of_dimension_integer_array(",crstr,", ",dimstr,")"}); then str;
    case DAE.EXTARGSIZE(cr,attr,ty,dim)
      equation 
        ((Types.T_REAL(_),_)) = Types.array_element_type(ty);
        crstr = var_name_array(cr, attr);
        dimstr = Exp.print_exp_str(dim);
        str = Util.string_append_list({"size_of_dimension_real_array(",crstr,", ",dimstr,")"}); then str;
    case _
      equation 
        Print.print_buf(
          "#-- generate_array_size_call failed\n#-- Not a DAE.EXTARGSIZE?\n"); then fail();
  end matchcontinue;
end generate_array_size_call;

protected function generate_ext_array_size_call
  input DAE.ExtArg in_extarg;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_extarg)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      CleanupStatement crstr,dimstr,str;
      Exp.ComponentRef cr;
      Types.Attributes attr;
      TupleTypes_TTypeAbsyn_PathOption ty;
      Exp.Exp dim;
    case DAE.EXTARGSIZE(cr,attr,ty,dim)
      equation 
        ((Types.T_INTEGER(_),_)) = Types.array_element_type(ty);
        crstr = var_name_array(cr, attr);
        dimstr = Exp.print_exp_str(dim);
        str = Util.string_append_list({"size_of_dimension_integer_array(",crstr,", ",dimstr,")"}); then str;
    case DAE.EXTARGSIZE(cr,attr,ty,dim)
      equation 
        ((Types.T_REAL(_),_)) = Types.array_element_type(ty);
        crstr = var_name_array(cr, attr);
        dimstr = Exp.print_exp_str(dim);
        str = Util.string_append_list({"size_of_dimension_real_array(",crstr,", ",dimstr,")"}); then str;
    case _
      equation 
        Print.print_buf(
          "#-- generate_array_size_call failed\n#-- Not a DAE.EXTARGSIZE?\n"); then fail();
  end matchcontinue;
end generate_ext_array_size_call;

protected function is_extarg_output
  input DAE.ExtArg in_extarg;
algorithm 
  _:=
  matchcontinue (in_extarg)
    case DAE.EXTARG(_,Types.ATTR(_,_,_,Absyn.OUTPUT),_) then (); 
  end matchcontinue;
end is_extarg_output;

protected function is_extarg_bidir
  input DAE.ExtArg in_extarg;
algorithm 
  _:=
  matchcontinue (in_extarg)
    case DAE.EXTARG(_,Types.ATTR(_,_,_,Absyn.BIDIR),_) then (); 
  end matchcontinue;
end is_extarg_bidir;

protected function is_extarg_output_or_bidir
  input DAE.ExtArg in_extarg;
algorithm 
  _:=
  matchcontinue (in_extarg)
    local DAE.ExtArg arg;
    case arg
      equation 
        is_extarg_output(arg); then ();
    case arg
      equation 
        is_extarg_bidir(arg); then ();
  end matchcontinue;
end is_extarg_output_or_bidir;

protected function generate_extcall_varcopy
  input DAE_ExtArgList in_dae_extarglist;
  input DAE.ExtArg in_extarg;
  input String in_string;
  input Integer in_integer;
  output CFunction out_cfunction;
  output Integer out_integer;
protected 
  type DAE_ExtArgList = list<DAE.ExtArg>;
algorithm 
  (out_cfunction,out_integer):=
  matchcontinue (in_dae_extarglist,in_extarg,in_string,in_integer)
    local
      Integer tnr,tnr_1;
      CFunction retcopy,vc,vcr,res;
      DAE.ExtArg retarg,var;
      CleanupStatement lang;
      DAE_ExtArgList rest;
    case ({},DAE.NOEXTARG(),_,tnr) then (c_empty_function,tnr); 
    case ({},retarg,lang,tnr)
      equation 
        is_extarg_output(retarg);
        retcopy = generate_extcall_varcopy_single(retarg); then (retcopy,tnr);
    case ({},retarg,lang,tnr)
      equation 
        failure(is_extarg_output(retarg)); then (c_empty_function,tnr);
    case (var :: rest,retarg,(lang = "C"),tnr) " extarg list is already filtered and contains only outputs "
      equation 
        vc = generate_extcall_varcopy_single(var);
        (vcr,tnr_1) = generate_extcall_varcopy(rest, retarg, lang, tnr);
        res = c_merge_fn(vc, vcr); then (res,tnr_1);
    case (var :: rest,retarg,(lang = "FORTRAN 77"),tnr)
      equation 
        vc = generate_extcall_varcopy_single_f77(var);
        (vcr,tnr_1) = generate_extcall_varcopy(rest, retarg, lang, tnr);
        res = c_merge_fn(vc, vcr); then (res,tnr_1);
    case (_,_,_,_)
      equation 
        Print.print_buf("#-- generate_extcall_varcopy failed\n"); then fail();
  end matchcontinue;
end generate_extcall_varcopy;

protected function generate_extcall_varcopy_single
  input DAE.ExtArg in_extarg;
  output CFunction out_cfunction;
algorithm 
  out_cfunction:=
  matchcontinue (in_extarg)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      CleanupStatement name,orgname,typcast,str;
      CFunction res;
      Exp.ComponentRef cref;
      Types.Attributes attr;
      TupleTypes_TTypeAbsyn_PathOption ty;
    case DAE.EXTARG(cref,attr,ty)
      equation 
        false = Types.is_array(ty);
        true = Types.is_output_attr(attr);
        name = var_name_external(cref);
        (orgname,_) = comp_ref_cstr(cref);
        typcast = generate_type(ty);
        str = Util.string_append_list({"out.",orgname," = (",typcast,")",name,";"});
        res = c_add_statements(c_empty_function, {str}); then res;
    case DAE.EXTARG(cref,attr,ty)
      equation 
        true = Types.is_array(ty);
        true = Types.is_output_attr(attr); then c_empty_function;
    case DAE.EXTARG(cref,attr,ty) then c_empty_function; 
    case _
      equation 
        Print.print_buf("#-- generate_extcall_varcopy_single failed\n"); then fail();
  end matchcontinue;
end generate_extcall_varcopy_single;

protected function generate_extcall_varcopy_single_f77
  input DAE.ExtArg in_extarg;
  output CFunction out_cfunction;
algorithm 
  out_cfunction:=
  matchcontinue (in_extarg)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      Exp.ComponentRef cref;
      Types.Attributes attr;
      TupleTypes_TTypeAbsyn_PathOption ty;
      CleanupStatement name,orgname,converter,str,typcast,tystr;
      CFunction res;
      DAE.ExtArg extarg;
    case extarg " INPUT ARRAY "
      equation 
        DAE.EXTARG(cref,attr,ty) = extarg;
        true = Types.is_array(ty);
        true = Types.is_input_attr(attr);
        name = var_name_external(cref);
        (orgname,_) = comp_ref_cstr(cref);
        converter = generate_f77_to_c_converter(ty);
        str = Util.string_append_list({converter,"(&",name,", &",orgname,");"});
        res = c_add_statements(c_empty_function, {str}); then res;
    case DAE.EXTARG(cref,attr,ty) " OUTPUT NON-ARRAY "
      equation 
        false = Types.is_array(ty);
        true = Types.is_output_attr(attr);
        name = var_name_external(cref);
        (orgname,_) = comp_ref_cstr(cref);
        typcast = generate_type(ty);
        str = Util.string_append_list({"out.",orgname," = (",typcast,")",name,";"});
        res = c_add_statements(c_empty_function, {str}); then res;
    case extarg " OUTPUT ARRAY "
      equation 
        DAE.EXTARG(cref,attr,ty) = extarg;
        true = Types.is_array(ty);
        true = Types.is_output_attr(attr);
        tystr = generate_type(ty);
        name = var_name_external(cref);
        (orgname,_) = comp_ref_cstr(cref);
        converter = generate_f77_to_c_converter(ty);
        str = Util.string_append_list({converter,"(&",name,", &out.",orgname,");"});
        res = c_add_statements(c_empty_function, {str}); then res;
    case extarg " BIDIR ARRAY "
      equation 
        DAE.EXTARG(cref,attr,ty) = extarg;
        true = Types.is_array(ty);
        true = Types.is_bidir_attr(attr);
        tystr = generate_type(ty);
        name = var_name_external(cref);
        (orgname,_) = comp_ref_cstr(cref);
        converter = generate_f77_to_c_converter(ty);
        str = Util.string_append_list({converter,"(&",name,", &",orgname,");"});
        res = c_add_statements(c_empty_function, {str}); then res;
    case DAE.EXTARG(cref,attr,ty) then c_empty_function; 
    case _
      equation 
        Print.print_buf("#-- generate_extcall_varcopy_single_f77 failed\n"); then fail();
  end matchcontinue;
end generate_extcall_varcopy_single_f77;

protected function invar_names
  input DAE_ElementList in_dae_elementlist;
  output StringList out_stringlist;
protected 
  type DAE_ElementList = list<DAE.Element>;
  type StringList = list<String>;
algorithm 
  out_stringlist:=
  matchcontinue (in_dae_elementlist)
    local
      CleanupStatement cref_str;
      CleanupStatementList r_1,cfn;
      Exp.ComponentRef id;
      DAE.VarKind vk;
      DAE.Type t;
      DAE_ElementList r;
    case {} then {}; 
    case DAE.VAR(id,vk,DAE.INPUT(),t,_,_,_,_,_,_,_) :: r
      equation 
        (cref_str,_) = comp_ref_cstr(id);
        r_1 = invar_names(r); then cref_str :: r_1;
    case _ :: r
      equation 
        cfn = invar_names(r); then cfn;
  end matchcontinue;
end invar_names;

protected function is_input_or_output_var
  input DAE.Element in_element;
algorithm 
  _:=
  matchcontinue (in_element)
    case DAE.VAR(_,_,DAE.INPUT(),_,_,_,_,_,_,_,_) then (); 
    case DAE.VAR(_,_,DAE.OUTPUT(),_,_,_,_,_,_,_,_) then (); 
  end matchcontinue;
end is_input_or_output_var;

protected function var_name_external
  input Exp.ComponentRef cref;
  output String str;
protected 
  Exp.ComponentRef cref_1;
algorithm 
  cref_1 := var_name_external_cref(cref);
  (str,_) := comp_ref_cstr(cref_1);
end var_name_external;

protected function var_name_external_cref
  input Exp.ComponentRef cref;
  output Exp.ComponentRef cref_1;
protected 
  Exp.ComponentRef cref_1;
algorithm 
  cref_1 := suffix_cref(cref, "_ext");
end var_name_external_cref;

protected function suffix_cref
  input Exp.ComponentRef in_componentref;
  input String in_string;
  output Exp.ComponentRef out_componentref;
algorithm 
  out_componentref:=
  matchcontinue (in_componentref,in_string)
    local
      type Exp_SubscriptList = list<Exp.Subscript>;
      CleanupStatement id_1,id,str;
      Exp_SubscriptList subs;
      Exp.ComponentRef cref_1,cref;
    case (Exp.CREF_IDENT(id,subs),str)
      equation 
        id_1 = string_append(id, str); then Exp.CREF_IDENT(id_1,subs);
    case (Exp.CREF_QUAL(id,subs,cref),str)
      equation 
        cref_1 = suffix_cref(cref, str); then Exp.CREF_QUAL(id,subs,cref_1);
  end matchcontinue;
end suffix_cref;

protected function var_name_array
  input Exp.ComponentRef in_componentref;
  input Types.Attributes in_attributes;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_componentref,in_attributes)
    local
      CleanupStatement str,cref_str;
      Exp.ComponentRef cref;
      Types.Attributes attr;
    case (cref,attr) " INPUT "
      equation 
        (str,_) = comp_ref_cstr(cref);
        true = Types.is_input_attr(attr); then str;
    case (cref,attr) " OUTPUT "
      equation 
        (cref_str,_) = comp_ref_cstr(cref);
        true = Types.is_output_attr(attr);
        str = string_append("out.", cref_str); then str;
    case (cref,attr) " INPUT/OUTPUT "
      equation 
        (str,_) = comp_ref_cstr(cref); then str;
  end matchcontinue;
end var_name_array;

protected function var_arg_names_external
  input DAE_ElementList in_dae_elementlist;
  output StringList out_stringlist;
protected 
  type DAE_ElementList = list<DAE.Element>;
  type StringList = list<String>;
algorithm 
  out_stringlist:=
  matchcontinue (in_dae_elementlist)
    local
      CleanupStatement cref_str,cref_str2;
      CleanupStatementList r_1,cfn;
      Exp.ComponentRef id;
      DAE.VarKind vk;
      DAE.Type t;
      DAE_ElementList r;
    case {} then {}; 
    case DAE.VAR(id,vk,DAE.INPUT(),t,_,_,_,_,_,_,_) :: r
      equation 
        cref_str = var_name_external(id);
        r_1 = var_arg_names_external(r); then cref_str :: r_1;
    case DAE.VAR(id,vk,DAE.OUTPUT(),t,_,_,_,_,_,_,_) :: r
      equation 
        cref_str = var_name_external(id);
        cref_str2 = string_append("&", cref_str);
        r_1 = var_arg_names_external(r); then cref_str2 :: r_1;
    case _ :: r
      equation 
        cfn = var_arg_names_external(r); then cfn;
  end matchcontinue;
end var_arg_names_external;

protected function generate_read
  input DAE_ElementList in_dae_elementlist;
  output CFunction out_cfunction;
protected 
  type DAE_ElementList = list<DAE.Element>;
algorithm 
  out_cfunction:=
  matchcontinue (in_dae_elementlist)
    local
      CleanupStatement cref_str,type_string,stmt;
      CFunction cfn1,cfn2,cfn;
      Exp.ComponentRef id;
      DAE.VarKind vk;
      DAE.Type t;
      DAE_ElementList r;
    case {} then c_empty_function; 
    case DAE.VAR(id,vk,DAE.INPUT(),t,_,{},_,_,_,_,_) :: r
      equation 
        (cref_str,_) = comp_ref_cstr(id);
        type_string = dae_type_str(t, false);
        stmt = Util.string_append_list(
          {"if(read_",type_string,"(in_file, &",cref_str,
          ")) return 1;"});
        cfn1 = c_add_inits(c_empty_function, {stmt});
        cfn2 = generate_read(r);
        cfn = c_merge_fn(cfn1, cfn2); then cfn;
    case DAE.VAR(id,vk,DAE.INPUT(),t,_,_ :: _,_,_,_,_,_) :: r
      equation 
        (cref_str,_) = comp_ref_cstr(id);
        type_string = dae_type_str(t, true);
        stmt = Util.string_append_list(
          {"if(read_",type_string,"(in_file, &",cref_str,
          ")) return 1;"});
        cfn1 = c_add_inits(c_empty_function, {stmt});
        cfn2 = generate_read(r);
        cfn = c_merge_fn(cfn1, cfn2); then cfn;
    case _ :: r
      equation 
        cfn = generate_read(r); then cfn;
  end matchcontinue;
end generate_read;

protected function generate_write
  input DAE_ElementList in_dae_elementlist;
  output CFunction out_cfunction;
protected 
  type DAE_ElementList = list<DAE.Element>;
algorithm 
  out_cfunction:=
  matchcontinue (in_dae_elementlist)
    local
      CleanupStatement cref_str,type_string,stmt;
      CFunction cfn1,cfn2,cfn;
      Exp.ComponentRef id;
      DAE.VarKind vk;
      DAE.Type t;
      DAE_ElementList r;
    case {} then c_empty_function; 
    case DAE.VAR(id,vk,DAE.OUTPUT(),t,_,{},_,_,_,_,_) :: r
      equation 
        (cref_str,_) = comp_ref_cstr(id);
        type_string = dae_type_str(t, false);
        stmt = Util.string_append_list({"write_",type_string,"(out_file, &out.",cref_str,");"});
        cfn1 = c_add_statements(c_empty_function, {stmt});
        cfn2 = generate_write(r);
        cfn = c_merge_fn(cfn1, cfn2); then cfn;
    case DAE.VAR(id,vk,DAE.OUTPUT(),t,_,_ :: _,_,_,_,_,_) :: r
      equation 
        (cref_str,_) = comp_ref_cstr(id);
        type_string = dae_type_str(t, true);
        stmt = Util.string_append_list({"write_",type_string,"(out_file, &out.",cref_str,");"});
        cfn1 = c_add_statements(c_empty_function, {stmt});
        cfn2 = generate_write(r);
        cfn = c_merge_fn(cfn1, cfn2); then cfn;
    case _ :: r
      equation 
        cfn = generate_write(r); then cfn;
  end matchcontinue;
end generate_write;

protected function is_rcw_output
  input DAE.Element e;
algorithm 
  DAE.is_var(e);
  DAE.is_output_var(e);
end is_rcw_output;

protected function is_rcw_input
  input DAE.Element e;
algorithm 
  DAE.is_var(e);
  DAE.is_input_var(e);
end is_rcw_input;

protected function is_rcw_bidir
  input DAE.Element e;
algorithm 
  DAE.is_var(e);
  DAE.is_bidir_var(e);
end is_rcw_bidir;

protected function new_ident
  input String str;
  output Exp.ComponentRef out_componentref;
protected 
  Integer i;
  CleanupStatement is,s;
algorithm 
  i := tick();
  is := int_string(i);
  s := string_append(str, is);
  out_componentref := Exp.CREF_IDENT(s,{});
end new_ident;
end Codegen;

