package DAELow "
    Copyright (c) PELAB, Dept. Computer and Information Science, Linkoping University,
    Sweden, in the following denoted PELAB/LIU.

   
    This file is part of OpenModelica, in the following denoted OSM.


    The OSM License, Version 1.1, March 4, 2005.


    1 Preface

    The aim of this license is to lay down the conditions enabling you to use, 
    modify and circulate OSM. However, PELAB/LIU remain the authors of 
    OSM and so retain property rights and the use of all ancillary rights.

    2 Definitions

    OSM is defined as all successive versions of the OSM software and their 
    documentation that have been developed by PELAB/LIU and including
    accepted contributions from other contributors according to this license.

    OSM DERIVED SOFTWARE is defined as all or part of OSM that you have 
    modified and/or translated and/or adapted.


    3 Dual License

    OSM is made available under the OSM licensing scheme, which is a dual
    licensing scheme with two options, a) and b):

    a) OSM OPEN SOURCE LICENSE:

    If you wish to write Open Source software you can use the Open Source version 
    of OSM, released under the OSM license which include GPL as its open source 
    licensing option. If you use the OSM Open Source version you must release your 
    Application using OSM including this Application's source code under the GPL as well.

    This OSM license text, and Copyright (c) PELAB/Linkoping University, must
    be present in your copy of OSM and in OSM DERIVED SOFTWARE.

    You should have received a copy of the GPL - GNU General Public License
    along with OpenModelica; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.


    b) OSM COMMERCIAL LICENSE

    If you are using OSM commercially - that is, for commercial usage or
    for creating proprietary software for sale or use in a commercial setting
    - you must purchase a commercial license of OSM from PELAB/L
  *
 ** file:	 DAELow.rml
 ** module:      DAELow
 ** description: DAELow a lower form of DAE including sparse matrises for
 ** BLT decomposition, etc.
 **
 ** RCS: $Id: DAELow.rml 1803 2005-06-09 11:33:52Z x02lucpo $
 ** 
 *
  * 
 ** This module is a lowered form of a DAE including equations 
 ** and simple equations in
 ** two separate lists. The variables are split into known variables
 ** parameters and constants, and unknown variables, 
 ** states and algebraic variables.
 ** The module includes the BLT sorting algorithm which sorts the
 ** equations into blocks, and the index reduction algorithm using
 ** dummy derivatives for solving higher index problems.
 ** It also includes the tarjan algorithm to detect strong components
 ** in the BLT sorting.
 *"

public import OpenModelica.Compiler.DAE;

public import OpenModelica.Compiler.Exp;

public import OpenModelica.Compiler.Values;

public import OpenModelica.Compiler.Absyn;

public import OpenModelica.Compiler.Algorithm;

public 
uniontype VarKind
  record VARIABLE
  end VARIABLE;
  record STATE
  end STATE;
  record DUMMY_DER
  end DUMMY_DER;
  record DUMMY_STATE
  end DUMMY_STATE;
  record DISCRETE
  end DISCRETE;
  record PARAM
  end PARAM;
  record CONST
  end CONST;
end VarKind;

public 
type Exp_ExpOption = Option<Exp.Exp>;

type Values_ValueOption = Option<Values.Value>;

type Absyn_PathList = list<Absyn.Path>;

type DAE_VariableAttributesOption = Option<DAE.VariableAttributes>;

type Absyn_CommentOption = Option<Absyn.Comment>;

uniontype Var
  record VAR
    Exp.ComponentRef x1;
    VarKind x2 " variable name ";
    DAE.VarDirection x3 " Kind of variable";
    DAE.Type x4 " input, output or bidirectional";
    Exp_ExpOption x5 " builtin type or enumeration ";
    Values_ValueOption x6 " Binding expression e.g. for parameters";
    DAE.InstDims x7 " binding value for parameters";
    DAE.StartValue x8 " array dimensions on nonexpanded var ";
    Integer x9 " value of start attribute ";
    String x10 " index in impl. vector ";
    Absyn_PathList x11 " original variable name ";
    DAE_VariableAttributesOption x12 " classname variable belongs to ";
    Absyn_CommentOption x13 " values on builtin attributes  this contains the comment and annotation from Absyn ";
  end VAR;
end Var;

public 
uniontype StateSelect
  record NEVER
  end NEVER;
  record AVOID
  end AVOID;
  record DEFAULT
  end DEFAULT;
  record PREFER
  end PREFER;
  record ALWAYS
  end ALWAYS;
end StateSelect;

public 
type Exp_ExpList = list<Exp.Exp>;

uniontype Equation
  record EQUATION
    Exp.Exp x1;
    Exp.Exp x2;
  end EQUATION;
  record SOLVED_EQUATION
    Exp.ComponentRef x1;
    Exp.Exp x2;
  end SOLVED_EQUATION;
  record RESIDUAL_EQUATION
    Exp.Exp x1 " not present from front end ";
  end RESIDUAL_EQUATION;
  record ALGORITHM
    Integer x1 " index in algorithms, 0..n-1";
    Exp_ExpList x2;
    Exp_ExpList x3 "inputs CREF or der(CREF) outputs CREF or der(CREF)";
  end ALGORITHM;
  record WHEN_EQUATION
    WhenEquation x1;
  end WHEN_EQUATION;
end Equation;

public 
uniontype WhenEquation
  record WHEN_EQ
    Integer x1;
    Exp.ComponentRef x2 " index in when clauses ";
    Exp.Exp x3 " Left hand side of equation   Right hand side of equation ";
  end WHEN_EQ;
  record REINIT
    Integer x1;
    Exp.ComponentRef x2 " index in when clauses ";
    Exp.Exp x3 " state variable to reinit   value after reinit ";
  end REINIT;
end WhenEquation;

public 
uniontype WhenClause
  record WHEN_CLAUSE
    Exp.Exp x1 " condition ";
  end WHEN_CLAUSE;
end WhenClause;

public 
type IntegerList = list<Integer>;

uniontype ZeroCrossing
  record ZERO_CROSSING
    Exp.Exp x1;
    IntegerList x2 " function ";
    IntegerList x3 " list of equations where the function occurs   list of when clauses where the function occurs ";
  end ZERO_CROSSING;
end ZeroCrossing;

public 
type WhenClauseList = list<WhenClause>;

type ZeroCrossingList = list<ZeroCrossing>;

uniontype EventInfo
  record EVENT_INFO
    WhenClauseList x1;
    ZeroCrossingList x2 " List of when clauses. The WhenEquation datatype refer to this list by position ";
  end EVENT_INFO;
end EventInfo;

public 
type Algorithm_AlgorithmArray = Algorithm.Algorithm[:];

uniontype DAELow " THE LOWERED DAE consist of variables and equations. The variables are split into 
 * two lists, one for unknown variables states and algebraic and one for known variables
 * constants and parameters.
 * The equations are also split into two lists, one with simple equations, a=b, a-b=0, etc., that 
  * are removed from  the set of equations to speed up calculations.
 "
  record DAELOW
    Variables x1 " ordered Variables, only states and alg. vars";
    Variables x2 " Known variables, i.e. constants and parameters";
    EquationArray x3 " ordered Equations ";
    EquationArray x4 " Removed equations a=b";
    EquationArray x5 " Initial equations ";
    Algorithm_AlgorithmArray x6 " Algorithms ";
    EventInfo x7;
  end DAELOW;
end DAELow;

public 
type CrefIndexList = list<CrefIndex>;

type CrefIndexListArray = CrefIndexList[:];

uniontype Variables
  record VARIABLES
    CrefIndexListArray x1 " HashTB, cref->indx";
    VariableArray x2 " Array of variables ";
    Integer x3 " bucket size ";
    Integer x4 " no. of vars ";
  end VARIABLES;
end Variables;

public 
uniontype CrefIndex
  record CREFINDEX
    Exp.ComponentRef x1;
    Integer x2;
  end CREFINDEX;
end CrefIndex;

public 
type VarOption = Option<Var>;

type VarOptionArray = VarOption[:];

uniontype VariableArray " array of Equations are expandable, to amortize the cost of adding 
 ** equations in a more efficient manner
 "
  record VARIABLE_ARRAY
    Integer x1;
    Integer x2 " no. elements ";
    VarOptionArray x3 " array size ";
  end VARIABLE_ARRAY;
end VariableArray;

public 
type EquationOption = Option<Equation>;

type EquationOptionArray = EquationOption[:];

uniontype EquationArray
  record EQUATION_ARRAY
    Integer x1;
    Integer x2 " no. elements ";
    EquationOptionArray x3 " array size ";
  end EQUATION_ARRAY;
end EquationArray;

public 
type IntegerArray = Integer[:];

uniontype Assignments "* Assignments of variables to equations and vice versa are implemented by a
  ** expandable array to amortize addition of array elements more efficient "
  record ASSIGNMENTS
    Integer x1;
    Integer x2 " actual size ";
    IntegerArray x3 " allocaded size >= actual size   array of indices ";
  end ASSIGNMENTS;
end Assignments;

public 
type TreeValueOption = Option<TreeValue>;

type BinTreeOption = Option<BinTree>;

uniontype BinTree "* Generic Binary tree implementation "
  record TREENODE
    TreeValueOption x1;
    BinTreeOption x2 " Value ";
    BinTreeOption x3 " left subtree   right subtree ";
  end TREENODE;
end BinTree;

public 
uniontype TreeValue " Each node in the binary tree can have a value associated with it."
  record TREEVALUE
    Key x1 " Key ";
    Value x2 " Value ";
  end TREEVALUE;
end TreeValue;

public 
type Key = Exp.ComponentRef " Key ";

public 
type Value = Integer;

public 
type IntegerList = list<Integer>;

type IncidenceMatrix = IntegerList[:];

public 
type IncidenceMatrixT = IncidenceMatrix " an array of lists of variable indexes (1..n), 
					one for each equation. For state variables
					the index is negated, since when sorting to 
					retrieve underlying ODE, the states are considered 
					known. The system is solved for the derivatives and
					the algebraic variables: \dot{x},y.
					But the states must be located when debugging and
					when a state is transformed to an algebraic varialbe
					in index reduction using dummy derivatives.
					";

protected import OpenModelica.Compiler.Util " End Module 
 Implementation ";

protected import OpenModelica.Compiler.DAEEXT;

protected import OpenModelica.Compiler.Print;

protected import OpenModelica.Compiler.Derive;

protected import OpenModelica.Compiler.Debug;

protected import OpenModelica.Compiler.Env;

protected import OpenModelica.Compiler.Builtin;

protected import OpenModelica.Compiler.Ceval;

protected import OpenModelica.Compiler.Types;

protected import OpenModelica.Compiler.SCode;

protected import OpenModelica.Compiler.Dump;

protected import OpenModelica.Compiler.System;

protected import OpenModelica.Compiler.VarTransform;

protected import OpenModelica.Compiler.RTOpts;

protected constant BinTree empty_bintree=TREENODE(NONE,NONE,NONE);

public function lower "* function: lower
 ** This function translates a DAE, which is the result from instantiating a 
 ** class, into a more precise form, called DAELow defined in this module.
 ** The DAELow representation splits the DAE into equations and variables
 ** and further divides variables into known and unknown variables and the 
 ** equations into simple and nonsimple equations.
 ** The variables are inserted into a hash table. This gives a lookup cost of 
 ** O(1) for finding a variable. The equations are put in an expandable
 ** array. Where adding a new equation can be done in O(1) time if space 
 ** is available.
 *"
  input DAE.DAElist lst;
  output DAELow out_daelow;
protected 
  type EquationList = list<Equation>;
  type Algorithm_AlgorithmList = list<Algorithm.Algorithm>;
  BinTree s;
  Variables vars,knvars,vars_1;
  EquationList eqns,reqns,ieqns,algeqns,eqns_1;
  Algorithm_AlgorithmList algs;
  WhenClauseList whenclauses,whenclauses_1;
  ZeroCrossingList zero_crossings;
  EquationArray eqnarr,reqnarr,ieqnarr;
  Algorithm_AlgorithmArray algarr;
algorithm 
  s := states(lst, empty_bintree);
  (vars,knvars,eqns,reqns,ieqns,algs,whenclauses) := lower2(lst, s, {});
  whenclauses_1 := list_reverse(whenclauses);
  algeqns := lower_algorithms(vars, algs);
  eqns := list_append(algeqns, eqns);
  (vars,knvars,eqns,reqns) := remove_simple_equations(vars, knvars, eqns, reqns, s);
  vars_1 := detect_implicit_discrete(vars, eqns);
  eqns_1 := sort_eqn(eqns);
  (zero_crossings) := find_zero_crossings(vars_1, eqns_1, whenclauses_1);
  eqnarr := list_equation(eqns_1) "	Util.list_map(zero_crossings,dump_zc_str) => zcstr &
	Util.string_append_list zcstr => zcstr' &
	print zcstr' &
";
  reqnarr := list_equation(reqns);
  ieqnarr := list_equation(ieqns);
  algarr := list_array(algs);
  out_daelow := DAELOW(vars_1,knvars,eqnarr,reqnarr,ieqnarr,algarr,
          EVENT_INFO(whenclauses_1,zero_crossings));
end lower;

protected function dump_zc_str "* function: dump_zc_str
 ** Dumps a zerocrossing into a string, for debugging purposes.
 *"
  input ZeroCrossing in_zerocrossing;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_zerocrossing)
    local
      type StringList = list<String>;
      type ValueList = list<Value>;
      StringList eq_s_list,wc_s_list;
      String eq_s,wc_s,str,str2;
      Exp.Exp e;
      ValueList eq,wc;
    case ZERO_CROSSING(e,eq,wc)
      equation 
        eq_s_list = Util.list_map(eq, int_string);
        eq_s = Util.string_delimit_list(eq_s_list, ",");
        wc_s_list = Util.list_map(wc, int_string);
        wc_s = Util.string_delimit_list(wc_s_list, ",");
        str = Exp.print_exp_str(e);
        str2 = Util.string_append_list(
          {str," in equations [",eq_s,"] and when conditions [",wc_s,
          "]\n"}); then str2;
  end matchcontinue;
end dump_zc_str;

protected function merge_zero_crossings "* function: merge_zero_crossings
 ** Takes a list of zero crossings and if more than one have identical
 ** function expressions they are merged into one zerocrossing.
 ** In trhe resulting list all zerocrossing have uniq function expressions.
 *"
  input ZeroCrossingList in_zerocrossinglist;
  output ZeroCrossingList out_zerocrossinglist;
algorithm 
  out_zerocrossinglist:=
  matchcontinue (in_zerocrossinglist)
    local
      ZeroCrossing zc,same_1;
      ZeroCrossingList samezc,diff,diff_1,xs;
    case {} then {}; 
    case {zc} then {zc}; 
    case zc :: xs
      equation 
        samezc = Util.list_select_1(xs, zc, same_zero_crossing);
        diff = Util.list_select_1(xs, zc, different_zero_crossing);
        diff_1 = merge_zero_crossings(diff);
        same_1 = Util.list_fold(samezc, merge_zero_crossing, zc); then same_1 :: diff_1;
  end matchcontinue;
end merge_zero_crossings;

protected function merge_zero_crossing "* function: merge_zero_crossing
 ** Merges two zero crossings into one by makeing the union of the lists of 
 ** equaions and when clauses they appear in.
 *"
  input ZeroCrossing in_zerocrossing1;
  input ZeroCrossing in_zerocrossing2;
  output ZeroCrossing out_zerocrossing;
algorithm 
  out_zerocrossing:=
  matchcontinue (in_zerocrossing1,in_zerocrossing2)
    local
      type ValueList = list<Value>;
      ValueList eq,zc,eq1,wc1,eq2,wc2;
      Exp.Exp e1,e2;
    case (ZERO_CROSSING(e1,eq1,wc1),ZERO_CROSSING(e2,eq2,wc2))
      equation 
        eq = Util.list_union(eq1, eq2);
        zc = Util.list_union(wc1, wc2); then ZERO_CROSSING(e1,eq,zc);
  end matchcontinue;
end merge_zero_crossing;

protected function same_zero_crossing "* function: same_zero_crossing
 ** Returns true if both zero crossings have the same function expression
 *"
  input ZeroCrossing in_zerocrossing1;
  input ZeroCrossing in_zerocrossing2;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_zerocrossing1,in_zerocrossing2)
    local
      Boolean res;
      Exp.Exp e1,e2;
    case (ZERO_CROSSING(e1,_,_),ZERO_CROSSING(e2,_,_))
      equation 
        res = Exp.exp_equal(e1, e2); then res;
  end matchcontinue;
end same_zero_crossing;

protected function different_zero_crossing "* function: different_zero_crossing
 ** Return true if the realation expressions differ.
 *"
  input ZeroCrossing zc1;
  input ZeroCrossing zc2;
  output Boolean res_1;
protected 
  Boolean res,res_1;
algorithm 
  res := same_zero_crossing(zc1, zc2);
  res_1 := bool_not(res);
end different_zero_crossing;

protected function find_zero_crossings
  input Variables vars;
  input EquationList eq;
  input WhenClauseList wc;
  output ZeroCrossingList res_1;
protected 
  type EquationList = list<Equation>;
  ZeroCrossingList res,res_1;
algorithm 
  res := find_zero_crossings2(vars, eq, 1, wc, 1);
  res_1 := merge_zero_crossings(res);
end find_zero_crossings;

protected function find_zero_crossings2
  input Variables in_variables1;
  input EquationList in_equationlist2;
  input Integer in_integer3;
  input WhenClauseList in_whenclauselist4;
  input Integer in_integer5;
  output ZeroCrossingList out_zerocrossinglist;
protected 
  type EquationList = list<Equation>;
algorithm 
  out_zerocrossinglist:=
  matchcontinue (in_variables1,in_equationlist2,in_integer3,in_whenclauselist4,in_integer5)
    local
      Variables v;
      Exp_ExpList rellst1,rellst2,rel;
      ZeroCrossingList zc1,zc2,zc3,zc4,res,res1,res2;
      Value eq_count_1,eq_count,wc_count_1,wc_count;
      Equation e;
      Exp.Exp e1,e2;
      EquationList xs,el;
      WhenClause wc;
    case (v,{},_,{},_) then {}; 
    case (v,(e = EQUATION(e1,e2)) :: xs,eq_count,{},_)
      equation 
        rellst1 = Exp.get_relations(e1);
        zc1 = make_zero_crossings(rellst1, {eq_count}, {});
        rellst2 = Exp.get_relations(e2);
        zc2 = make_zero_crossings(rellst2, {eq_count}, {});
        eq_count_1 = eq_count + 1;
        zc3 = find_zero_crossings2(v, xs, eq_count_1, {}, 0);
        zc4 = list_append(zc1, zc2);
        res = list_append(zc3, zc4); then res;
    case (v,e :: xs,eq_count,{},_)
      equation 
        eq_count_1 = eq_count + 1;
        (res) = find_zero_crossings2(v, xs, eq_count_1, {}, 0); then res;
    case (v,el,eq_count,(wc = WHEN_CLAUSE(e)) :: xs,wc_count)
      local
        Exp.Exp e;
        WhenClauseList xs;
      equation 
        wc_count_1 = wc_count + 1;
        (res1) = find_zero_crossings2(v, el, eq_count, xs, wc_count_1);
        rel = Exp.get_relations(e);
        res2 = make_zero_crossings(rel, {}, {wc_count});
        res = list_append(res1, res2); then res;
  end matchcontinue;
end find_zero_crossings2;

protected function make_zero_crossing "* function: make_zero_crossing
 ** Constructs a ZeroCrossing from an expression and lists of equation indecies
 ** and when clause indecies.
 *"
  input Exp.Exp in_exp1;
  input IntegerList in_integerlist2;
  input IntegerList in_integerlist3;
  output ZeroCrossing out_zerocrossing;
algorithm 
  out_zerocrossing:=
  matchcontinue (in_exp1,in_integerlist2,in_integerlist3)
    local
      type ValueList = list<Value>;
      Exp.Exp e;
      ValueList eq_ind,wc_ind;
    case (e,eq_ind,wc_ind) then ZERO_CROSSING(e,eq_ind,wc_ind); 
  end matchcontinue;
end make_zero_crossing;

protected function make_zero_crossings "* function: make_zero_crossings
 ** Constructs a list of ZeroCrossings from a list expressions and lists of 
 ** equation indecies and when clause indecies.
 ** Each Zerocrossing gets the same lists of indecies.
 *"
  input Exp_ExpList in_exp_explist1;
  input IntegerList in_integerlist2;
  input IntegerList in_integerlist3;
  output ZeroCrossingList out_zerocrossinglist;
algorithm 
  out_zerocrossinglist:=
  matchcontinue (in_exp_explist1,in_integerlist2,in_integerlist3)
    local
      type ValueList = list<Value>;
      ZeroCrossing res;
      ZeroCrossingList resx;
      Exp.Exp e;
      Exp_ExpList xs;
      ValueList eq_ind,wc_ind;
    case ({},_,_) then {}; 
    case (e :: xs,eq_ind,wc_ind)
      equation 
        res = make_zero_crossing(e, eq_ind, wc_ind);
        resx = make_zero_crossings(xs, eq_ind, wc_ind); then res :: resx;
  end matchcontinue;
end make_zero_crossings;

protected function detect_implicit_discrete "* function: detect_implicit_discrete
 ** This function updates the variable kind to discrete for variables set 
 ** in when equations.
 *"
  input Variables in_variables;
  input EquationList in_equationlist;
  output Variables out_variables;
protected 
  type EquationList = list<Equation>;
algorithm 
  out_variables:=
  matchcontinue (in_variables,in_equationlist)
    local
      type Exp_SubscriptList = list<Exp.Subscript>;
      Variables v,v_1,v_2;
      Exp.ComponentRef cr;
      DAE.VarDirection dir;
      DAE.Type vartype;
      Exp_ExpOption bind,start;
      Values_ValueOption value;
      Exp_SubscriptList dims;
      Value ind;
      String orig;
      Absyn_PathList clname;
      DAE_VariableAttributesOption attr;
      Absyn_CommentOption comment;
      EquationList xs;
    case (v,{}) then v; 
    case (v,WHEN_EQUATION(WHEN_EQ(_,cr,_)) :: xs)
      equation 
        (VAR(cr,_,dir,vartype,bind,value,dims,start,ind,orig,clname,attr,comment),_) = get_var(cr, v);
        v_1 = add_var(
          VAR(cr,DISCRETE(),dir,vartype,bind,value,dims,start,ind,orig,
          clname,attr,comment), v);
        v_2 = detect_implicit_discrete(v_1, xs); then v_2;
    case (v,_ :: xs)
      equation 
        v_1 = detect_implicit_discrete(v, xs); then v_1;
  end matchcontinue;
end detect_implicit_discrete;

protected function sort_eqn "* function: sort_eqn 
 ** This function sorts the equation. It puts first the algebraic eqns
 ** and last the differentiated eqns
 **
 **
 *"
  input EquationList in_equationlist;
  output EquationList out_equationlist;
protected 
  type EquationList = list<Equation>;
algorithm 
  out_equationlist:=
  matchcontinue (in_equationlist)
    local EquationList algEqns,diffEqns,res,eqns;
    case (eqns)
      equation 
        (algEqns,diffEqns) = extract_algebraic_and_differential_eqn(eqns);
        res = list_append(algEqns, diffEqns) "        print \"ALGEBRAIC EQUATIONS\n\" &
        dump_eqns(algEqns) &
        print \"\n\nDIFFERENTIATED EQUATIONS\n\" &
        dump_eqns(diffEqns) &"; then res;
    case (eqns)
      equation 
        print("sort_eqn failed \n"); then fail();
  end matchcontinue;
end sort_eqn;

protected function extract_algebraic_and_differential_eqn
  input EquationList in_equationlist;
  output EquationList out_equationlist1;
  output EquationList out_equationlist2;
protected 
  type EquationList = list<Equation>;
algorithm 
  (out_equationlist1,out_equationlist2):=
  matchcontinue (in_equationlist)
    local
      EquationList resAlgEqn,resDiffEqn,rest;
      Equation eqn,alg;
      Exp.Exp exp1,exp2;
    case ({}) then ({},{}); 
    case ((eqn = EQUATION(exp1,exp2)) :: rest)
      equation 
        true = is_algebraic(exp1);
        true = is_algebraic(exp2);
        (resAlgEqn,resDiffEqn) = extract_algebraic_and_differential_eqn(rest); then (eqn :: resAlgEqn,resDiffEqn);
    case ((eqn = EQUATION(exp1,exp2)) :: rest)
      equation 
        (resAlgEqn,resDiffEqn) = extract_algebraic_and_differential_eqn(rest); then (resAlgEqn,eqn :: resDiffEqn);
    case (alg :: rest)
      equation 
        (resAlgEqn,resDiffEqn) = extract_algebraic_and_differential_eqn(rest) " Put algorithms in algebraic equations "; then (alg :: resAlgEqn,resDiffEqn);
  end matchcontinue;
end extract_algebraic_and_differential_eqn;

public function generate_state_partition " assignments1 
   assignments2 
   dynamic blocks 
   output blocks "
  input IntegerListList in_integerlistlist1;
  input DAELow in_daelow2;
  input IntegerVector in_integervector3;
  input IntegerVector in_integervector4;
  input IncidenceMatrix in_incidencematrix5;
  input IncidenceMatrixT in_incidencematrixt6;
  output IntegerListList out_integerlistlist1;
  output IntegerListList out_integerlistlist2;
protected 
  type IntegerListList = list<IntegerList>;
  type IntegerVector = Integer[:];
algorithm 
  (out_integerlistlist1,out_integerlistlist2):=
  matchcontinue (in_integerlistlist1,in_daelow2,in_integervector3,in_integervector4,in_incidencematrix5,in_incidencematrixt6)
    local
      type ValueArray = Value[:];
      type ValueList = list<Value>;
      type ValueListList = list<ValueList>;
      type ValueVector = Value[:];
      type ValueListArray = ValueList[:];
      Value size;
      ValueArray arr,arr_1;
      ValueListList blt_states,blt_no_states,blt;
      DAELow dae;
      Variables v,kv;
      EquationArray e,se,ie;
      Algorithm_AlgorithmArray al;
      ValueVector ass1,ass2;
      ValueListArray m,mt;
    case (blt,(dae = DAELOW(v,kv,e,se,ie,al,_)),ass1,ass2,m,mt) " blocks   assignments1   assignments2   dynamic blocks   output blocks "
      equation 
        size = array_length(ass1) "equation_size(e) => size &";
        arr = fill(0, size);
        arr_1 = mark_state_equations(dae, arr, m, mt, ass1, ass2);
        (blt_states,blt_no_states) = split_blocks(blt, arr); then (blt_states,blt_no_states);
    case (_,_,_,_,_,_)
      equation 
        print("-generate_state_partition failed\n"); then fail();
  end matchcontinue;
end generate_state_partition;

protected function split_blocks " dynamic 
   output "
  input IntegerListList in_integerlistlist;
  input IntegerArray in_integerarray;
  output IntegerListList out_integerlistlist1;
  output IntegerListList out_integerlistlist2;
protected 
  type IntegerListList = list<IntegerList>;
algorithm 
  (out_integerlistlist1,out_integerlistlist2):=
  matchcontinue (in_integerlistlist,in_integerarray)
    local
      type ValueList = list<Value>;
      type ValueListList = list<ValueList>;
      type ValueArray = Value[:];
      ValueListList states,output_,blocks;
      ValueList block_;
      ValueArray arr;
    case ({},_) then ({},{}); 
    case (block_ :: blocks,arr)
      equation 
        true = block_is_dynamic(block_, arr) " block is dynamic, belong in dynamic section ";
        (states,output_) = split_blocks(blocks, arr); then (block_ :: states,output_);
    case (block_ :: blocks,arr)
      equation 
        (states,output_) = split_blocks(blocks, arr) " block is not dynamic, belong in output section "; then (states,block_ :: output_);
  end matchcontinue;
end split_blocks;

protected function block_is_dynamic " function block_is_dynamic
 ** Return true if the block contains a variable that is marked
 *"
  input IntegerList in_integerlist;
  input IntegerArray in_integerarray;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_integerlist,in_integerarray)
    local
      type ValueList = list<Value>;
      type ValueArray = Value[:];
      Value x_1,x,mark_value;
      Boolean res;
      ValueList xs;
      ValueArray arr;
    case ({},_) then false; 
    case (x :: xs,arr)
      equation 
        x_1 = x - 1;
        0 = array_nth(arr, x_1);
        res = block_is_dynamic(xs, arr); then res;
    case (x :: xs,arr)
      equation 
        x_1 = x - 1;
        mark_value = arr[x_1 + 1];
        (mark_value <> 0) = true; then true;
  end matchcontinue;
end block_is_dynamic;

protected function mark_state_equations " marks 
   assignments 
   assignments 
   marks "
  input DAELow in_daelow1;
  input IntegerArray in_integerarray2;
  input IncidenceMatrix in_incidencematrix3;
  input IncidenceMatrixT in_incidencematrixt4;
  input IntegerVector in_integervector5;
  input IntegerVector in_integervector6;
  output IntegerArray out_integerarray;
protected 
  type IntegerVector = Integer[:];
algorithm 
  out_integerarray:=
  matchcontinue (in_daelow1,in_integerarray2,in_incidencematrix3,in_incidencematrixt4,in_integervector5,in_integervector6)
    local
      type VarList = list<Var>;
      type ValueArray = Value[:];
      type ValueList = list<Value>;
      type ValueListArray = ValueList[:];
      type ValueVector = Value[:];
      VarList v_lst,statevar_lst;
      DAELow dae;
      ValueArray arr_1,arr;
      ValueListArray m,mt;
      ValueVector a1,a2;
      Variables v,kn;
      EquationArray e,se,ie;
      Algorithm_AlgorithmArray alg;
    case ((dae = DAELOW(v,kn,e,se,ie,alg,_)),arr,m,mt,a1,a2) " marks   assignments   assignments   marks "
      equation 
        v_lst = var_list(v);
        statevar_lst = Util.list_select(v_lst, is_state_var);
        ((dae,arr_1,m,mt,a1,a2)) = Util.list_fold(statevar_lst, mark_state_equation, (dae,arr,m,mt,a1,a2)); then arr_1;
    case (_,_,_,_,_,_)
      equation 
        print("-mark_state_equations failed\n"); then fail();
  end matchcontinue;
end mark_state_equations;

protected function mark_state_equation
  input Var in_var;
  input TupleDAELowIntegerArrayIncidenceMatrixIncidenceMatrixTIntegerVectorIntegerVector in_tupledaelowintegerarrayincidencematrixincidencematrixtintegervectorintegervector;
  output TupleDAELowIntegerArrayIncidenceMatrixIncidenceMatrixTIntegerVectorIntegerVector out_tupledaelowintegerarrayincidencematrixincidencematrixtintegervectorintegervector;
protected 
  type IntegerVector = Integer[:];
  type TupleDAELowIntegerArrayIncidenceMatrixIncidenceMatrixTIntegerVectorIntegerVector = tuple<DAELow,IntegerArray,IncidenceMatrix,IncidenceMatrixT,IntegerVector,IntegerVector>;
algorithm 
  out_tupledaelowintegerarrayincidencematrixincidencematrixtintegervectorintegervector:=
  matchcontinue (in_var,in_tupledaelowintegerarrayincidencematrixincidencematrixtintegervectorintegervector)
    local
      type ValueArray = Value[:];
      type ValueList = list<Value>;
      type ValueListArray = ValueList[:];
      type ValueVector = Value[:];
      Var v;
      Value v_indx,v_indx_1,eqn;
      ValueArray arr_1,arr;
      ValueListArray m,mt;
      ValueVector a1,a2;
      Exp.ComponentRef cr;
      DAELow dae;
      Variables vars;
      String s,str;
    case (VAR(cr,_,_,_,_,_,_,_,_,_,_,_,_),((dae = DAELOW(vars,_,_,_,_,_,_)),arr,m,mt,a1,a2))
      equation 
        (v,v_indx) = get_var(cr, vars);
        v_indx_1 = v_indx - 1;
        eqn = a1[v_indx_1 + 1];
        ((arr_1,m,mt,a1,a2)) = mark_state_equation2(eqn, (arr,m,mt,a1,a2)); then ((dae,arr_1,m,mt,a1,a2));
    case (VAR(cr,_,_,_,_,_,_,_,_,_,_,_,_),((dae = DAELOW(vars,_,_,_,_,_,_)),arr,m,mt,a1,a2))
      equation 
        failure((_,_) = get_var(cr, vars));
        print("mark_state_equation var ");
        s = Exp.print_component_ref_str(cr);
        print(s);
        print("not found\n"); then fail();
    case (VAR(cr,_,_,_,_,_,_,_,_,_,_,_,_),((dae = DAELOW(vars,_,_,_,_,_,_)),arr,m,mt,a1,a2))
      equation 
        (_,v_indx) = get_var(cr, vars);
        v_indx_1 = v_indx - 1;
        failure(eqn = a1[v_indx_1 + 1]);
        print("mark_state_equation index =");
        str = int_string(v_indx);
        print(str);
        print(", failed\n"); then fail();
  end matchcontinue;
end mark_state_equation;

protected function mark_state_equation2 " marks "
  input Integer in_integer;
  input TupleIntegerArrayIncidenceMatrixIncidenceMatrixTIntegerVectorIntegerVector in_tupleintegerarrayincidencematrixincidencematrixtintegervectorintegervector;
  output TupleIntegerArrayIncidenceMatrixIncidenceMatrixTIntegerVectorIntegerVector out_tupleintegerarrayincidencematrixincidencematrixtintegervectorintegervector;
protected 
  type IntegerVector = Integer[:];
  type TupleIntegerArrayIncidenceMatrixIncidenceMatrixTIntegerVectorIntegerVector = tuple<IntegerArray,IncidenceMatrix,IncidenceMatrixT,IntegerVector,IntegerVector>;
algorithm 
  out_tupleintegerarrayincidencematrixincidencematrixtintegervectorintegervector:=
  matchcontinue (in_integer,in_tupleintegerarrayincidencematrixincidencematrixtintegervectorintegervector)
    local
      type ValueArray = Value[:];
      type ValueList = list<Value>;
      type ValueListArray = ValueList[:];
      type ValueVector = Value[:];
      Value eqn_1,eqn,mark_value,len;
      ValueArray marks_1,marks_2,marks;
      ValueList inv_reachable,inv_reachable_1;
      ValueListArray m,mt;
      ValueVector a1,a2;
      String eqnstr,lens,ms;
    case (eqn,(marks,m,mt,a1,a2))
      equation 
        eqn_1 = eqn - 1 " Mark an unmarked node/equation ";
        0 = array_nth(marks, eqn_1);
        marks_1 = array_update(marks, eqn_1 + 1, 1);
        inv_reachable = inv_reachable_nodes(eqn, m, mt, a1, a2);
        inv_reachable_1 = remove_negative(inv_reachable);
        ((marks_2,m,mt,a1,a2)) = Util.list_fold(inv_reachable_1, mark_state_equation2, (marks_1,m,mt,a1,a2)); 
        then ((marks_2,m,mt,a1,a2));
    case (eqn,(marks,m,mt,a1,a2))
      equation 
        eqn_1 = eqn - 1 " Node allready marked. ";
        mark_value = marks[eqn_1 + 1];
        (mark_value <> 0) = true; then ((marks,m,mt,a1,a2));
    case (eqn,(marks,m,mt,a1,a2))
      equation 
        print("mark_state_equation2 failed, eqn:");
        eqnstr = int_string(eqn);
        print(eqnstr);
        print("array length =");
        len = array_length(marks);
        lens = int_string(len);
        print(lens);
        print("\n");
        eqn_1 = eqn - 1;
        mark_value = marks[eqn_1 + 1];
        ms = int_string(mark_value);
        print("mark_value:");
        print(ms);
        print("\n"); then fail();
  end matchcontinue;
end mark_state_equation2;

protected function inv_reachable_nodes
  input Integer in_integer1;
  input IncidenceMatrix in_incidencematrix2;
  input IncidenceMatrixT in_incidencematrixt3;
  input IntegerVector in_integervector4;
  input IntegerVector in_integervector5;
  output IntegerList out_integerlist;
protected 
  type IntegerVector = Integer[:];
algorithm 
  out_integerlist:=
  matchcontinue (in_integer1,in_incidencematrix2,in_incidencematrixt3,in_integervector4,in_integervector5)
    local
      type ValueList = list<Value>;
      type ValueListArray = ValueList[:];
      type ValueVector = Value[:];
      Value eqn_1,e,eqn;
      ValueList var_lst,var_lst_1,lst;
      ValueListArray m,mt;
      ValueVector a1,a2;
      String eqn_str;
    case (e,m,mt,a1,a2)
      equation 
        eqn_1 = e - 1;
        var_lst = m[eqn_1 + 1];
        var_lst_1 = remove_negative(var_lst);
        lst = inv_reachable_nodes2(var_lst_1, a1); then lst;
    case (eqn,_,_,_,_)
      equation 
        print("-inv_reachable_nodes failed, eqn:");
        eqn_str = int_string(eqn);
        print(eqn_str);
        print("\n"); then fail();
  end matchcontinue;
end inv_reachable_nodes;

protected function inv_reachable_nodes2 " assignments1"
  input IntegerList in_integerlist;
  input IntegerVector in_integervector;
  output IntegerList out_integerlist;
protected 
  type IntegerVector = Integer[:];
algorithm 
  out_integerlist:=
  matchcontinue (in_integerlist,in_integervector)
    local
      type ValueList = list<Value>;
      type ValueVector = Value[:];
      ValueList eqns,vs;
      Value v_1,eqn,v;
      ValueVector a1;
    case ({},_) then {}; 
    case (v :: vs,a1)
      equation 
        eqns = inv_reachable_nodes2(vs, a1);
        v_1 = v - 1;
        eqn = a1[v_1 + 1] " Which equation is variable solved in? "; then eqn :: eqns;
    case (_,_)
      equation 
        print("-inv_reachable_nodes2 failed\n"); then fail();
  end matchcontinue;
end inv_reachable_nodes2;

public function is_state_var "* function: is_state_var
 ** Returns true for state variables, false otherwise.
*"
  input Var in_var;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_var)
    case (VAR(_,STATE(),_,_,_,_,_,_,_,_,_,_,_)) then true; 
    case (_) then false; 
  end matchcontinue;
end is_state_var;

public function is_non_state "* function: is_non_state 
 ** this equation checks if the the varkind is state of variable
 ** used both in build_equation and generate_compute_state
 *"
  input VarKind in_varkind;
algorithm 
  _:=
  matchcontinue (in_varkind)
    case (VARIABLE()) then (); 
    case (DUMMY_DER()) then (); 
    case (DUMMY_STATE()) then (); 
    case (DISCRETE()) then (); 
  end matchcontinue;
end is_non_state;

public function is_discrete "* function: is_discrete
 ** this equation checks if the the varkind is discrete,
 ** used both in build_equation and generate_compute_state
 *"
  input VarKind in_varkind;
algorithm 
  _:=
  matchcontinue (in_varkind)
    case (DISCRETE()) then (); 
  end matchcontinue;
end is_discrete;

public function dump "* function: dump
 ** This function dumps the DAELow representaton to stdout.
 *"
  input DAELow in_daelow;
algorithm 
  _:=
  matchcontinue (in_daelow)
    local
      type VarList = list<Var>;
      type EquationList = list<Equation>;
      VarList vars,knvars;
      Value varlen,eqnlen;
      String varlen_str,eqnlen_str;
      EquationList eqnsl,reqnsl,ieqnsl;
      Variables vars1,vars2;
      EquationArray eqns,reqns,ieqns;
      Algorithm_AlgorithmArray algs;
      EventInfo eventinfo;
    case (DAELOW(vars1,vars2,eqns,reqns,ieqns,algs,eventinfo))
      equation 
        print("Variables (");
        vars = var_list(vars1);
        varlen = list_length(vars);
        varlen_str = int_string(varlen);
        print(varlen_str);
        print(")\n");
        print("=========\n");
        dump_vars(vars);
        print("\n");
        print("Known Variables (constants) (");
        knvars = var_list(vars2);
        varlen = list_length(knvars);
        varlen_str = int_string(varlen);
        print(varlen_str);
        print(")\n");
        print("=============================\n");
        dump_vars(knvars);
        print("\nEquations (");
        eqnsl = equation_list(eqns);
        eqnlen = list_length(eqnsl);
        eqnlen_str = int_string(eqnlen);
        print(eqnlen_str);
        print(")\n");
        print("=========\n");
        dump_eqns(eqnsl);
        print("Simple Equations (");
        reqnsl = equation_list(reqns);
        eqnlen = list_length(reqnsl);
        eqnlen_str = int_string(eqnlen);
        print(eqnlen_str);
        print(")\n");
        print("=========\n");
        dump_eqns(reqnsl);
        print("Initial Equations (");
        ieqnsl = equation_list(ieqns);
        eqnlen = list_length(ieqnsl);
        eqnlen_str = int_string(eqnlen);
        print(eqnlen_str);
        print(")\n");
        print("=========\n");
        dump_eqns(ieqnsl); then ();
  end matchcontinue;
end dump;

public function var_list "* function: var_list
 ** Takes Variables and returns a list of 'Var', useful for e.g. dumping.
 *"
  input Variables in_variables;
  output VarList out_varlist;
protected 
  type VarList = list<Var>;
algorithm 
  out_varlist:=
  matchcontinue (in_variables)
    local
      VarList varlst;
      VariableArray vararr;
    case (VARIABLES(_,vararr,_,_))
      equation 
        varlst = vararray_list(vararr); then varlst;
  end matchcontinue;
end var_list;

public function list_var "* function: list_var
 ** author: PA
 ** Takes Var list and creates a Variables structure, see also var_list.
 *"
  input VarList in_varlist;
  output Variables out_variables;
protected 
  type VarList = list<Var>;
algorithm 
  out_variables:=
  matchcontinue (in_varlist)
    local
      Variables res,vars,vars_1;
      Var v;
      VarList vs;
    case ({})
      equation 
        res = empty_vars(); then res;
    case (v :: vs)
      equation 
        vars = list_var(vs);
        vars_1 = add_var(v, vars); then vars_1;
  end matchcontinue;
end list_var;

public function var_cref "* function: var_cref
 ** author: PA
 **
 ** extracts the ComponentRef of a variable.
 *"
  input Var in_var;
  output Exp.ComponentRef out_componentref;
algorithm 
  out_componentref:=
  matchcontinue (in_var)
    local Exp.ComponentRef cr;
    case (VAR(cr,_,_,_,_,_,_,_,_,_,_,_,_)) then cr; 
  end matchcontinue;
end var_cref;

public function var_nominal "* function: var_nominal
 ** author: PA
 **
 ** Extacts the nominal attribute of a variable. If the variable has no 
 ** nominal value, the function fails.
 *"
  input Var in_var;
  output Real out_real;
algorithm 
  out_real:=
  matchcontinue (in_var)
    local Real nominal;
    case (VAR(_,_,_,_,_,_,_,_,_,_,_,SOME(DAE.VAR_ATTR_REAL(_,_,_,_,_,_,SOME(nominal),_)),_)) then nominal; 
  end matchcontinue;
end var_nominal;

public function vararray_list "* function: vararray_list
 ** Transforms a VariableArray to a Var list
 *"
  input VariableArray in_variablearray;
  output VarList out_varlist;
protected 
  type VarList = list<Var>;
algorithm 
  out_varlist:=
  matchcontinue (in_variablearray)
    local
      VarOptionArray arr;
      Var elt;
      Value lastpos,n,size;
      VarList lst;
    case (VARIABLE_ARRAY(0,_,arr)) then {}; 
    case (VARIABLE_ARRAY(1,_,arr))
      equation 
        SOME(elt) = array_nth(arr, 0); then {elt};
    case (VARIABLE_ARRAY(n,size,arr))
      equation 
        lastpos = n - 1;
        lst = vararray_list2(arr, 0, lastpos); then lst;
  end matchcontinue;
end vararray_list;

protected function vararray_list2 "* function: vararray_list2
 ** Helper function to vararray_list
*"
  input VarOptionArray in_varoptionarray1;
  input Integer in_integer2;
  input Integer in_integer3;
  output VarList out_varlist;
protected 
  type VarList = list<Var>;
algorithm 
  out_varlist:=
  matchcontinue (in_varoptionarray1,in_integer2,in_integer3)
    local
      Var v;
      VarOptionArray arr;
      Value pos,lastpos,pos_1;
      VarList res;
    case (arr,pos,lastpos)
      equation 
        (pos == lastpos) = true;
        SOME(v) = array_nth(arr, pos); then {v};
    case (arr,pos,lastpos)
      equation 
        pos_1 = pos + 1;
        SOME(v) = array_nth(arr, pos);
        res = vararray_list2(arr, pos_1, lastpos); then v :: res;
  end matchcontinue;
end vararray_list2;

public function dump_jacobian_str "* function: dump_jacobian_str
 ** Dumps the sparse jacobian.
 ** Uses the variables to determine size of Jacobian matrix.
 *"
  input TupleIntegerIntegerEquationList eqns;
  output String res_1;
protected 
  type TupleIntegerIntegerEquation = tuple<Integer,Integer,Equation>;
  type TupleIntegerIntegerEquationList = list<TupleIntegerIntegerEquation>;
  type StringList = list<String>;
  StringList res;
  String res_1;
algorithm 
  res := dump_jacobian_str2(eqns);
  res_1 := Util.string_delimit_list(res, ", ");
end dump_jacobian_str;

protected function dump_jacobian_str2 "* function: dump_jacobian_str2
 ** Helper function to dump_jacobian_str
*"
  input TupleIntegerIntegerEquationList in_tupleintegerintegerequationlist;
  output StringList out_stringlist;
protected 
  type TupleIntegerIntegerEquation = tuple<Integer,Integer,Equation>;
  type TupleIntegerIntegerEquationList = list<TupleIntegerIntegerEquation>;
  type StringList = list<String>;
algorithm 
  out_stringlist:=
  matchcontinue (in_tupleintegerintegerequationlist)
    local
      type TupleValueValueEquation = tuple<Value,Value,Equation>;
      type TupleValueValueEquationList = list<TupleValueValueEquation>;
      String estr,rowstr,colstr,str;
      StringList strs;
      Value row,col;
      Exp.Exp e;
      TupleValueValueEquationList eqns;
    case ({}) then {}; 
    case ((row,col,RESIDUAL_EQUATION(e)) :: eqns)
      equation 
        estr = Exp.print_exp_str(e);
        rowstr = int_string(row);
        colstr = int_string(col);
        str = Util.string_append_list({"{",rowstr,",",colstr,"}:",estr});
        strs = dump_jacobian_str2(eqns); then str :: strs;
  end matchcontinue;
end dump_jacobian_str2;

public function dump_eqns "* function: dump_eqns
 ** Helper function to dump.
 *"
  input EquationList eqns;
protected 
  type EquationList = list<Equation>;
algorithm 
  dump_eqns2(eqns, 1);
end dump_eqns;

protected function dump_eqns2
  input EquationList in_equationlist;
  input Integer in_integer;
protected 
  type EquationList = list<Equation>;
algorithm 
  _:=
  matchcontinue (in_equationlist,in_integer)
    local
      String es,is;
      Value index_1,index;
      Equation eqn;
      EquationList eqns;
    case ({},_) then (); 
    case (eqn :: eqns,index)
      equation 
        es = equation_str(eqn);
        is = int_string(index);
        print(is);
        print(" : ");
        print(es);
        print("\n");
        index_1 = index + 1;
        dump_eqns2(eqns, index_1); then ();
  end matchcontinue;
end dump_eqns2;

protected function equation_str "* function: equation_str
 ** Helper function to e.g. dump.
 *"
  input Equation in_equation;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_equation)
    local
      String s1,s2,res,is;
      Exp.Exp e1,e2,e;
      Exp.ComponentRef cr;
      Value i;
    case (EQUATION(e1,e2))
      equation 
        s1 = Exp.print_exp_str(e1);
        s2 = Exp.print_exp_str(e2);
        res = Util.string_append_list({s1," = ",s2}); then res;
    case (SOLVED_EQUATION(cr,e2))
      equation 
        s1 = Exp.print_component_ref_str(cr);
        s2 = Exp.print_exp_str(e2);
        res = Util.string_append_list({s1," := ",s2}); then res;
    case (WHEN_EQUATION(WHEN_EQ(i,cr,e2)))
      equation 
        s1 = Exp.print_component_ref_str(cr);
        s2 = Exp.print_exp_str(e2);
        is = int_string(i);
        res = Util.string_append_list({s1," := ",s2,"when clause no:",is,"\n"}); then res;
    case (RESIDUAL_EQUATION(e))
      equation 
        s1 = Exp.print_exp_str(e);
        res = Util.string_append_list({s1,"= 0"}); then res;
    case (ALGORITHM(i,_,_))
      equation 
        is = int_string(i);
        res = Util.string_append_list({"Algorithm no: ",is,"\n"}); then res;
  end matchcontinue;
end equation_str;

protected function remove_simple_equations " known vars 
   eqns 
   simple eqns "
  input Variables in_variables1;
  input Variables in_variables2;
  input EquationList in_equationlist3;
  input EquationList in_equationlist4;
  input BinTree in_bintree5;
  output Variables out_variables1;
  output Variables out_variables2;
  output EquationList out_equationlist3;
  output EquationList out_equationlist4;
protected 
  type EquationList = list<Equation>;
algorithm 
  (out_variables1,out_variables2,out_equationlist3,out_equationlist4):=
  matchcontinue (in_variables1,in_variables2,in_equationlist3,in_equationlist4,in_bintree5)
    local
      VarTransform.VariableReplacements repl,vartransf;
      EquationList eqns_1,seqns,eqns_2,seqns_1,eqns,reqns;
      BinTree movedvars_1,states;
      Variables vars_1,knvars_1,vars,knvars;
    case (vars,knvars,eqns,reqns,states) " vars   known vars   eqns   simple eqns "
      equation 
        repl = VarTransform.empty_replacements();
        (eqns_1,seqns,movedvars_1,vartransf) = remove_simple_equations2(eqns, vars, knvars, empty_bintree, states, repl);
        eqns_2 = VarTransform.replace_equations(eqns_1, vartransf) " 	VarTransform.dump_replacements(vartransf) => () & ";
        seqns_1 = VarTransform.replace_equations(seqns, vartransf);
        (vars_1,knvars_1) = move_variables(vars, knvars, movedvars_1); then (vars_1,knvars_1,eqns_2,seqns_1);
    case (_,_,_,_,_)
      equation 
        print("-remove_simple_equations failed\n"); then fail();
  end matchcontinue;
end remove_simple_equations;

protected function remove_simple_equations2 " variables 
   known variables 
   moved vars 
   states 
   variable replacements
   diff equations and non-simple equations  
   simple equations
   moved variables 
    list of replacements "
  input EquationList in_equationlist1;
  input Variables in_variables2;
  input Variables in_variables3;
  input BinTree in_bintree4;
  input BinTree in_bintree5;
  input VarTransform.VariableReplacements in_variablereplacements6;
  output EquationList out_equationlist1;
  output EquationList out_equationlist2;
  output BinTree out_bintree3;
  output VarTransform.VariableReplacements out_variablereplacements4;
protected 
  type EquationList = list<Equation>;
algorithm 
  (out_equationlist1,out_equationlist2,out_bintree3,out_variablereplacements4):=
  matchcontinue (in_equationlist1,in_variables2,in_variables3,in_bintree4,in_bintree5,in_variablereplacements6)
    local
      Variables vars,knvars;
      BinTree mvars,states,mvars_1,mvars_2;
      VarTransform.VariableReplacements repl,repl_1,repl_2;
      Exp.ComponentRef cr1,cr2;
      EquationList eqns_1,seqns_1,eqns;
      Equation e;
      Exp.Type t;
    case ({},vars,knvars,mvars,states,repl) then ({},{},mvars,repl); 
    case (e :: eqns,vars,knvars,mvars,states,repl)
      equation 
        (cr1,cr2) = simple_equation(e) " Both are states, i.e. x1 = x2 can not be removed ";
        (cr1,cr2) = VarTransform.apply_replacements(repl, cr1, cr2);
        _ = tree_get(states, cr1);
        _ = tree_get(states, cr2);
        (eqns_1,seqns_1,mvars_1,repl) = remove_simple_equations2(eqns, vars, knvars, mvars, states, repl); then (e :: eqns_1,seqns_1,mvars_1,repl);
    case (e :: eqns,vars,knvars,mvars,states,repl)
      equation 
        (cr1,cr2) = simple_equation(e) " cr1 is state and cr2 not, remove cr2";
        false = is_var_on_top_level_and_output(cr2, vars) " is the variable to be removed is output the do not remove it ";
        t = typeof_equation(e);
        (cr1,cr2) = VarTransform.apply_replacements(repl, cr1, cr2);
        failure(_ = VarTransform.get_replacement(repl, cr2));
        _ = tree_get(states, cr1);
        repl_1 = VarTransform.add_replacement(repl, cr2, cr1);
        mvars_1 = tree_add(mvars, cr2, 0);
        (eqns_1,seqns_1,mvars_2,repl_2) = remove_simple_equations2(eqns, vars, knvars, mvars_1, states, repl_1); then (eqns_1,SOLVED_EQUATION(cr2,Exp.CREF(cr1,t)) :: seqns_1,mvars_2,repl_2);
    case (e :: eqns,vars,knvars,mvars,states,repl)
      equation 
        (cr2,cr1) = simple_equation(e) " as above but swapped args: cr1 is state and cr2 not, remove cr2";
        false = is_var_on_top_level_and_output(cr2, vars) " is the variable to be removed is output the do not remove it ";
        t = typeof_equation(e);
        (cr1,cr2) = VarTransform.apply_replacements(repl, cr1, cr2);
        failure(_ = VarTransform.get_replacement(repl, cr2));
        _ = tree_get(states, cr1);
        repl_1 = VarTransform.add_replacement(repl, cr2, cr1);
        mvars_1 = tree_add(mvars, cr2, 0);
        (eqns_1,seqns_1,mvars_2,repl_2) = remove_simple_equations2(eqns, vars, knvars, mvars_1, states, repl_1); then (eqns_1,SOLVED_EQUATION(cr2,Exp.CREF(cr1,t)) :: seqns_1,mvars_2,repl_2);
    case (e :: eqns,vars,knvars,mvars,states,repl)
      equation 
        (cr1,cr2) = simple_equation(e) " cr1 variable, cr2 constant, remove cr1 ";
        t = typeof_equation(e);
        false = is_var_on_top_level_and_output(cr1, vars) " is the variable to be removed is output the do not remove it ";
        (cr1,cr2) = VarTransform.apply_replacements(repl, cr1, cr2);
        failure(_ = VarTransform.get_replacement(repl, cr1));
        is_variable(cr1, vars, knvars);
        failure(is_variable(cr2, vars, knvars));
        repl_1 = VarTransform.add_replacement(repl, cr1, cr2);
        mvars_1 = tree_add(mvars, cr1, 0);
        (eqns_1,seqns_1,mvars_2,repl_2) = remove_simple_equations2(eqns, vars, knvars, mvars_1, states, repl_1); then (eqns_1,SOLVED_EQUATION(cr1,Exp.CREF(cr2,t)) :: seqns_1,mvars_2,repl_2);
    case (e :: eqns,vars,knvars,mvars,states,repl)
      equation 
        (cr2,cr1) = simple_equation(e) " same as above but swapped args: remove cr1 ";
        t = typeof_equation(e);
        false = is_var_on_top_level_and_output(cr1, vars) " is the variable to be removed is output the do not remove it ";
        (cr1,cr2) = VarTransform.apply_replacements(repl, cr1, cr2);
        failure(_ = VarTransform.get_replacement(repl, cr1));
        is_variable(cr1, vars, knvars);
        failure(is_variable(cr2, vars, knvars));
        repl_1 = VarTransform.add_replacement(repl, cr1, cr2);
        mvars_1 = tree_add(mvars, cr1, 0);
        (eqns_1,seqns_1,mvars_2,repl_2) = remove_simple_equations2(eqns, vars, knvars, mvars_1, states, repl_1); then (eqns_1,SOLVED_EQUATION(cr1,Exp.CREF(cr2,t)) :: seqns_1,mvars_2,repl_2);
    case (e :: eqns,vars,knvars,mvars,states,repl)
      equation 
        (cr1,cr2) = simple_equation(e) " None are states, but  variables, remove cr1 ";
        t = typeof_equation(e);
        false = is_var_on_top_level_and_output(cr1, vars) " is the variable to be removed is output the do not remove it ";
        (cr1,cr2) = VarTransform.apply_replacements(repl, cr1, cr2);
        failure(_ = VarTransform.get_replacement(repl, cr1));
        failure(_ = tree_get(states, cr1));
        failure(_ = tree_get(states, cr2));
        is_variable(cr1, vars, knvars);
        is_variable(cr2, vars, knvars);
        repl_1 = VarTransform.add_replacement(repl, cr1, cr2);
        mvars_1 = tree_add(mvars, cr1, 0);
        (eqns_1,seqns_1,mvars_2,repl_2) = remove_simple_equations2(eqns, vars, knvars, mvars_1, states, repl_1); then (eqns_1,SOLVED_EQUATION(cr1,Exp.CREF(cr2,t)) :: seqns_1,mvars_2,repl_2);
    case (e :: eqns,vars,knvars,mvars,states,repl)
      equation 
        (cr1,cr2) = simple_equation(e) " same as above but swapped args, remove cr1 ";
        t = typeof_equation(e);
        false = is_var_on_top_level_and_output(cr1, vars) " is the variable to be removed is output the do not remove it ";
        (cr1,cr2) = VarTransform.apply_replacements(repl, cr1, cr2);
        failure(_ = VarTransform.get_replacement(repl, cr1));
        failure(_ = tree_get(states, cr1));
        failure(_ = tree_get(states, cr2));
        is_variable(cr1, vars, knvars);
        is_variable(cr2, vars, knvars);
        repl_1 = VarTransform.add_replacement(repl, cr1, cr2);
        mvars_1 = tree_add(mvars, cr1, 0);
        (eqns_1,seqns_1,mvars_2,repl_2) = remove_simple_equations2(eqns, vars, knvars, mvars_1, states, repl_1); then (eqns_1,SOLVED_EQUATION(cr1,Exp.CREF(cr2,t)) :: seqns_1,mvars_2,repl_2);
    case (e :: eqns,vars,knvars,mvars,states,repl)
      equation 
        (eqns_1,seqns_1,mvars_1,repl_1) = remove_simple_equations2(eqns, vars, knvars, mvars, states, repl) " Not a simple variable, check rest"; then (e :: eqns_1,seqns_1,mvars_1,repl_1);
  end matchcontinue;
end remove_simple_equations2;

protected function is_var_on_top_level_and_output "* rela-tion is_var_on_top_level_and_output
 ** this rela-tion checks if the provided cr is from a var that is on top model
 ** and has the DAE.VarDirection = OUTPUT
 ** The check for top-model is done by spliting the old name at '.' and 
 ** check if the list-lenght is 1
 *"
  input Exp.ComponentRef in_componentref;
  input Variables in_variables;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_componentref,in_variables)
    local
      type StringList = list<String>;
      Exp.ComponentRef cr;
      VarKind kind;
      String old_name;
      StringList cr_str_lst;
      Variables vars;
    case (cr,vars)
      equation 
        (VAR(cr,kind,DAE.OUTPUT(),_,_,_,_,_,_,old_name,_,_,_),_) = get_var(cr, vars);
        cr_str_lst = Util.string_split_at_char(old_name, #".");
        1 = list_length(cr_str_lst); then true;
    case (_,_) then false; 
  end matchcontinue;
end is_var_on_top_level_and_output;

protected function typeof_equation "* function: typeof_equation
 ** Returns the Exp.Type of an equation
*"
  input Equation in_equation;
  output Exp.Type out_type;
algorithm 
  out_type:=
  matchcontinue (in_equation)
    local
      Exp.Type t;
      Exp.Exp e;
    case (EQUATION(e,_))
      equation 
        t = Exp.typeof(e); then t;
    case (SOLVED_EQUATION(_,e))
      equation 
        t = Exp.typeof(e); then t;
    case (WHEN_EQUATION(WHEN_EQ(_,_,e)))
      equation 
        t = Exp.typeof(e); then t;
  end matchcontinue;
end typeof_equation;

protected function move_variables " known 
   vars to move from first7 to second
   updated alg+state vars"
  input Variables in_variables1;
  input Variables in_variables2;
  input BinTree in_bintree3;
  output Variables out_variables1;
  output Variables out_variables2;
algorithm 
  (out_variables1,out_variables2):=
  matchcontinue (in_variables1,in_variables2,in_bintree3)
    local
      type VarList = list<Var>;
      VarList lst1,lst2,lst1_1,lst2_1;
      Variables v1,v2,vars,knvars,vars1,vars2;
      BinTree mvars;
    case (vars1,vars2,mvars) " alg+state   known   vars to move from first7 to second  updated alg+state vars  updated known vars"
      equation 
        lst1 = var_list(vars1);
        lst2 = var_list(vars2);
        (lst1_1,lst2_1) = move_variables2(lst1, lst2, mvars);
        v1 = empty_vars();
        v2 = empty_vars();
        vars = add_vars(lst1_1, v1);
        knvars = add_vars(lst2_1, v2); then (vars,knvars);
  end matchcontinue;
end move_variables;

protected function move_variables2 " alg+state vars as list 
   known vars as list 
   move-variables as BinTree 
   updated alg+state vars as list"
  input VarList in_varlist1;
  input VarList in_varlist2;
  input BinTree in_bintree3;
  output VarList out_varlist1;
  output VarList out_varlist2;
protected 
  type VarList = list<Var>;
algorithm 
  (out_varlist1,out_varlist2):=
  matchcontinue (in_varlist1,in_varlist2,in_bintree3)
    local
      VarList knvars,vs_1,knvars_1,vs;
      Var v;
      Exp.ComponentRef cr;
      BinTree mvars;
    case ({},knvars,_) then ({},knvars); 
    case ((v = VAR(cr,_,_,_,_,_,_,_,_,_,_,_,_)) :: vs,knvars,mvars)
      equation 
        _ = tree_get(mvars, cr) " alg var moved to known vars";
        (vs_1,knvars_1) = move_variables2(vs, knvars, mvars); then (vs_1,v :: knvars_1);
    case ((v = VAR(cr,_,_,_,_,_,_,_,_,_,_,_,_)) :: vs,knvars,mvars)
      equation 
        failure(_ = tree_get(mvars, cr)) " alg var not moved to known vars";
        (vs_1,knvars_1) = move_variables2(vs, knvars, mvars); then (v :: vs_1,knvars_1);
  end matchcontinue;
end move_variables2;

protected function is_variable "* function: is_variable
 ** This function takes a Exp.ComponentRef and two Variables. It searches the two sets of variables and 
 ** succed if the variable is STATE or VARIABLE. Otherwise it fails.
 *"
  input Exp.ComponentRef in_componentref1;
  input Variables in_variables2;
  input Variables in_variables3;
algorithm 
  _:=
  matchcontinue (in_componentref1,in_variables2,in_variables3)
    local
      Exp.ComponentRef cr;
      Variables vars,knvars;
    case (cr,vars,_)
      equation 
        (VAR(_,VARIABLE(),_,_,_,_,_,_,_,_,_,_,_),_) = get_var(cr, vars); then ();
    case (cr,vars,_)
      equation 
        (VAR(_,STATE(),_,_,_,_,_,_,_,_,_,_,_),_) = get_var(cr, vars); then ();
    case (cr,_,knvars)
      equation 
        (VAR(_,VARIABLE(),_,_,_,_,_,_,_,_,_,_,_),_) = get_var(cr, knvars); then ();
  end matchcontinue;
end is_variable;

public function simple_equation "* function: simple_equation
 ** This function investigates if an Equation is simple, i.e. on the form
 ** a = b or a-b=0 
 ** These functions can be removed from the set of equations.
 *"
  input Equation in_equation;
  output Exp.ComponentRef out_componentref1;
  output Exp.ComponentRef out_componentref2;
algorithm 
  (out_componentref1,out_componentref2):=
  matchcontinue (in_equation)
    local
      Exp.ComponentRef cr1,cr2;
      Exp.Exp e;
    case EQUATION(Exp.CREF(cr1,_),Exp.CREF(cr2,_)) then (cr1,cr2); 
    case (EQUATION(Exp.BINARY(Exp.CREF(cr1,_),Exp.SUB(_),Exp.CREF(cr2,_)),e))
      equation 
        true = Exp.is_zero(e); then (cr1,cr2);
    case (EQUATION(e,Exp.BINARY(Exp.CREF(cr1,_),Exp.SUB(_),Exp.CREF(cr2,_))))
      equation 
        true = Exp.is_zero(e); then (cr1,cr2);
  end matchcontinue;
end simple_equation;

protected function remove_variable_named "* function: remove_variable_named
 ** Removes a varaible from the 'Variables' set given a ComponentRef name.
 ** The removed variable is returned, such that is can be used elsewhere.
 *"
  input Variables in_variables;
  input Exp.ComponentRef in_componentref;
  output Variables out_variables;
  output Var out_var;
algorithm 
  (out_variables,out_var):=
  matchcontinue (in_variables,in_componentref)
    local
      String str;
      Variables vars,vars_1;
      Exp.ComponentRef cr;
      Var var;
    case (vars,cr)
      equation 
        failure((_,_) = get_var(cr, vars));
        print("-remove_variable_named failed. variable ");
        str = Exp.print_component_ref_str(cr);
        print(str);
        print(" not found.\n"); then fail();
    case (vars,cr)
      equation 
        (var,_) = get_var(cr, vars);
        vars_1 = delete_var(vars, cr); then (vars_1,var);
    case (_,_)
      equation 
        print("-remove_variable_named failed\n"); then fail();
  end matchcontinue;
end remove_variable_named;

public function dump_vars "* function: dump_vars
 ** Helper function to dump.
 *"
  input VarList vars;
protected 
  type VarList = list<Var>;
algorithm 
  dump_vars2(vars, 1);
end dump_vars;

protected function dump_vars2
  input VarList in_varlist;
  input Integer in_integer;
protected 
  type VarList = list<Var>;
algorithm 
  _:=
  matchcontinue (in_varlist,in_integer)
    local
      type StringList = list<String>;
      String varnostr,dirstr,str,path_str,comment_str,s,old_name;
      StringList paths_lst,path_strs;
      Value varno_1,varno;
      Exp.ComponentRef cr;
      VarKind kind;
      DAE.VarDirection dir;
      Exp.Exp e;
      Absyn_PathList paths;
      DAE_VariableAttributesOption dae_var_attr;
      Absyn_CommentOption comment;
      VarList xs;
    case ({},_) then (); 
    case (VAR(cr,kind,dir,_,SOME(e),_,_,_,_,old_name,paths,dae_var_attr,comment) :: xs,varno)
      equation 
        varnostr = int_string(varno);
        print(varnostr);
        print(": ");
        dirstr = DAE.dump_direction_str(dir);
        print(dirstr);
        print(" ");
        str = Exp.print_component_ref_str(cr);
        print(str);
        print(":");
        dump_kind(kind);
        paths_lst = Util.list_map(paths, Absyn.path_string);
        path_str = Util.string_delimit_list(paths_lst, ", ");
        comment_str = Dump.unparse_comment_option(comment);
        print("= ");
        s = Exp.print_exp_str(e);
        print(s);
        print(" ");
        print(path_str);
        print("  ");
        print(comment_str);
        print(" former: ");
        print(old_name);
        varno_1 = varno + 1;
        print(",\n");
        DAE.dump_variable_attributes(dae_var_attr);
        dump_vars2(xs, varno_1); then ();
    case (VAR(cr,kind,dir,_,NONE,_,_,_,_,old_name,paths,dae_var_attr,comment) :: xs,varno)
      equation 
        varnostr = int_string(varno);
        print(varnostr);
        print(": ");
        dirstr = DAE.dump_direction_str(dir);
        print(dirstr);
        print(" ");
        str = Exp.print_component_ref_str(cr);
        path_strs = Util.list_map(paths, Absyn.path_string);
        path_str = Util.string_delimit_list(path_strs, ", ");
        comment_str = Dump.unparse_comment_option(comment);
        print(str);
        print(":");
        dump_kind(kind);
        print(" ");
        print(path_str);
        print(" former: ");
        print(old_name);
        DAE.dump_variable_attributes(dae_var_attr);
        print("  ");
        print(comment_str);
        print(",\n");
        varno_1 = varno + 1;
        dump_vars2(xs, varno_1); then ();
  end matchcontinue;
end dump_vars2;

protected function dump_kind "* function: dump_kind
 ** Helper function to dump.
 *"
  input VarKind in_varkind;
algorithm 
  _:=
  matchcontinue (in_varkind)
    case VARIABLE()
      equation 
        print("VARIABLE"); then ();
    case STATE()
      equation 
        print("STATE"); then ();
    case DUMMY_DER()
      equation 
        print("DUMMY_DER"); then ();
    case DUMMY_STATE()
      equation 
        print("DUMMY_STATE"); then ();
    case DISCRETE()
      equation 
        print("DISCRETE"); then ();
    case PARAM()
      equation 
        print("PARAM"); then ();
    case CONST()
      equation 
        print("CONST"); then ();
  end matchcontinue;
end dump_kind;

public function states "* function: states
 ** Returns a BinTree of all states in the DAE.
 ** This function is used by the lower function.
 *"
  input DAE.DAElist in_daelist;
  input BinTree in_bintree;
  output BinTree out_bintree;
algorithm 
  out_bintree:=
  matchcontinue (in_daelist,in_bintree)
    local
      type DAE_ElementList = list<DAE.Element>;
      BinTree bt;
      Exp.Exp e1,e2;
      DAE_ElementList xs;
      DAE.DAElist dae;
    case (DAE.DAE({}),bt) then bt; 
    case (DAE.DAE(DAE.EQUATION(e1,e2) :: xs),bt)
      equation 
        bt = states(DAE.DAE(xs), bt);
        bt = states_exp(e1, bt);
        bt = states_exp(e2, bt); then bt;
    case (DAE.DAE(DAE.COMP(_,dae) :: xs),bt)
      equation 
        bt = states(dae, bt);
        bt = states(DAE.DAE(xs), bt); then bt;
    case (DAE.DAE(_ :: xs),bt)
      equation 
        bt = states(DAE.DAE(xs), bt); then bt;
  end matchcontinue;
end states;

protected function states_exp "* function: states_exp
 ** Helper function to states.
 *"
  input Exp.Exp in_exp;
  input BinTree in_bintree;
  output BinTree out_bintree;
algorithm 
  out_bintree:=
  matchcontinue (in_exp,in_bintree)
    local
      BinTree bt;
      Exp.Exp e1,e2,e,e3;
      Exp.ComponentRef cr_1,cr;
      Exp_ExpList expl;
    case (Exp.BINARY(e1,_,e2),bt)
      equation 
        bt = states_exp(e1, bt);
        bt = states_exp(e2, bt); then bt;
    case (Exp.UNARY(_,e),bt)
      equation 
        bt = states_exp(e, bt); then bt;
    case (Exp.LBINARY(e1,_,e2),bt)
      equation 
        bt = states_exp(e1, bt);
        bt = states_exp(e2, bt); then bt;
    case (Exp.LUNARY(_,e),bt)
      equation 
        bt = states_exp(e, bt); then bt;
    case (Exp.RELATION(e1,_,e2),bt)
      equation 
        bt = states_exp(e1, bt);
        bt = states_exp(e2, bt); then bt;
    case (Exp.IFEXP(e1,e2,e3),bt)
      equation 
        bt = states_exp(e1, bt);
        bt = states_exp(e2, bt);
        bt = states_exp(e3, bt); then bt;
    case (Exp.CALL(Absyn.IDENT("der"),{Exp.CREF(cr,_)},_,_),bt)
      equation 
        cr_1 = Exp.stringify_component_ref(cr) " value irrelevant, give zero ";
        bt = tree_add(bt, cr_1, 0); then bt;
    case (Exp.CALL(_,expl,_,_),bt)
      equation 
        bt = Util.list_fold(expl, states_exp, bt); then bt;
    case (Exp.ARRAY(_,_,expl),bt)
      equation 
        bt = Util.list_fold(expl, states_exp, bt); then bt;
    case (Exp.MATRIX(_,_,_),bt)
      equation 
        Print.print_buf("states_exp on MATRIX not impl.\n");
        print("states_exp on MATRIX not impl.\n"); then bt;
    case (Exp.TUPLE(expl),bt)
      equation 
        bt = Util.list_fold(expl, states_exp, bt); then bt;
    case (Exp.CAST(_,e),bt)
      equation 
        bt = states_exp(e, bt); then bt;
    case (Exp.ASUB(e,_),bt)
      equation 
        bt = states_exp(e, bt); then bt;
    case (Exp.REDUCTION(_,e1,_,e2),bt)
      equation 
        bt = states_exp(e1, bt);
        bt = states_exp(e2, bt); then bt;
    case (_,bt) then bt; 
  end matchcontinue;
end states_exp;

protected function lower_when_eqn " when-clause index 
   when-clause index"
  input DAE.Element in_element;
  input Integer in_integer;
  input WhenClauseList in_whenclauselist;
  output EquationList out_equationlist;
  output Variables out_variables;
  output Integer out_integer;
  output WhenClauseList out_whenclauselist;
protected 
  type EquationList = list<Equation>;
algorithm 
  (out_equationlist,out_variables,out_integer,out_whenclauselist):=
  matchcontinue (in_element,in_integer,in_whenclauselist)
    local
      type DAE_ElementList = list<DAE.Element>;
      Variables vars;
      Value i_1,i;
      EquationList res;
      Exp.Exp cond;
      DAE_ElementList eqnl;
      WhenClauseList whenList;
    case (DAE.WHEN_EQUATION(cond,eqnl,NONE),i,whenList) " when-clause index   when-clause index  No elsewhen yet "
      equation 
        vars = empty_vars();
        i_1 = i + 1;
        res = lower_when_eqn2(eqnl, i); then (res,vars,i_1,WHEN_CLAUSE(cond) :: whenList);
  end matchcontinue;
end lower_when_eqn;

protected function lower_when_eqn2
  input DAE_ElementList in_dae_elementlist;
  input Integer in_integer;
  output EquationList out_equationlist;
protected 
  type DAE_ElementList = list<DAE.Element>;
  type EquationList = list<Equation>;
algorithm 
  out_equationlist:=
  matchcontinue (in_dae_elementlist,in_integer)
    local
      EquationList eqnl;
      Exp.Exp e_1,e_2,cre,e;
      Exp.ComponentRef cr_1,cr;
      DAE_ElementList xs;
      Value i;
    case ({},_) then {}; 
    case (DAE.EQUATION((cre = Exp.CREF(cr,_)),e) :: xs,i)
      equation 
        eqnl = lower_when_eqn2(xs, i);
        e_1 = Exp.simplify(e);
        e_2 = Exp.stringify_crefs(e_1);
        cr_1 = Exp.stringify_component_ref(cr); then WHEN_EQUATION(WHEN_EQ(i,cr_1,e_2)) :: eqnl;
  end matchcontinue;
end lower_when_eqn2;

protected function lower2
  input DAE.DAElist in_daelist;
  input BinTree in_bintree;
  input WhenClauseList in_whenclauselist;
  output Variables out_variables1;
  output Variables out_variables2;
  output EquationList out_equationlist3;
  output EquationList out_equationlist4;
  output EquationList out_equationlist5;
  output Algorithm_AlgorithmList out_algorithm_algorithmlist6;
  output WhenClauseList out_whenclauselist7;
protected 
  type EquationList = list<Equation>;
  type Algorithm_AlgorithmList = list<Algorithm.Algorithm>;
algorithm 
  (out_variables1,out_variables2,out_equationlist3,out_equationlist4,out_equationlist5,out_algorithm_algorithmlist6,out_whenclauselist7):=
  matchcontinue (in_daelist,in_bintree,in_whenclauselist)
    local
      type DAE_ElementList = list<DAE.Element>;
      Variables v1,v2,vars,knvars,vars_1,knvars_1,vars1,vars2,knvars1,knvars2,kv;
      WhenClauseList whenclauses,whenclauses_1,whenclauses_2;
      EquationList eqns,reqns,ieqns,eqns1,eqns2,reqns1,ieqns1,reqns2,ieqns2,re,ie;
      Algorithm_AlgorithmList algs,algs1,algs2,al;
      Var v_1;
      DAE.Element v,e;
      DAE_ElementList xs;
      BinTree states;
      Equation e_1;
      Exp.Exp e1,e2,c;
      Value count,count_1;
      Algorithm.Algorithm a;
      DAE.DAElist dae;
    case (DAE.DAE({}),_,whenclauses) " states "
      equation 
        v1 = empty_vars();
        v2 = empty_vars(); then (v1,v2,{},{},{},{},whenclauses);
    case (DAE.DAE((v = DAE.VAR(_,_,_,_,_,_,_,_,_,_,_)) :: xs),states,whenclauses)
      equation 
        (vars,knvars,eqns,reqns,ieqns,algs,whenclauses_1) = lower2(DAE.DAE(xs), states, whenclauses);
        v_1 = lower_var(v, states);
        vars_1 = add_var(v_1, vars); then (vars_1,knvars,eqns,reqns,ieqns,algs,whenclauses_1);
    case (DAE.DAE((v = DAE.VAR(_,_,_,_,_,_,_,_,_,_,_)) :: xs),states,whenclauses)
      equation 
        (vars,knvars,eqns,reqns,ieqns,algs,whenclauses_1) = lower2(DAE.DAE(xs), states, whenclauses);
        v_1 = lower_known_var(v) " in previous rule, lower_var failed.  ";
        knvars_1 = add_var(v_1, knvars); then (vars,knvars_1,eqns,reqns,ieqns,algs,whenclauses_1);
    case (DAE.DAE((e = DAE.EQUATION(e1,e2)) :: xs),states,whenclauses)
      equation 
        (vars,knvars,eqns,reqns,ieqns,algs,whenclauses_1) = lower2(DAE.DAE(xs), states, whenclauses);
        e_1 = lower_eqn(e); then (vars,knvars,e_1 :: eqns,reqns,ieqns,algs,whenclauses_1);
    case (DAE.DAE((e = DAE.WHEN_EQUATION(c,eqns,NONE)) :: xs),states,whenclauses)
      equation 
        (vars1,knvars,eqns1,reqns,ieqns,algs,whenclauses_1) = lower2(DAE.DAE(xs), states, whenclauses);
        count = list_length(whenclauses_1);
        (eqns2,vars2,count_1,whenclauses_2) = lower_when_eqn(e, count, whenclauses_1);
        vars = merge_vars(vars1, vars2);
        eqns = list_append(eqns1, eqns2); then (vars,knvars,eqns,reqns,ieqns,algs,whenclauses_2);
    case (DAE.DAE((e = DAE.INITIALEQUATION(e1,e2)) :: xs),states,whenclauses)
      equation 
        (vars,knvars,eqns,reqns,ieqns,algs,whenclauses_1) = lower2(DAE.DAE(xs), states, whenclauses);
        e_1 = lower_eqn(e); then (vars,knvars,eqns,reqns,e_1 :: ieqns,algs,whenclauses_1);
    case (DAE.DAE(DAE.ALGORITHM(a) :: xs),states,whenclauses)
      equation 
        (vars,knvars,eqns,reqns,ieqns,algs,whenclauses_1) = lower2(DAE.DAE(xs), states, whenclauses); then (vars,knvars,eqns,reqns,ieqns,a :: algs,whenclauses_1);
    case (DAE.DAE(DAE.COMP(_,dae) :: xs),states,whenclauses)
      equation 
        (vars1,knvars1,eqns1,reqns1,ieqns1,algs1,whenclauses_1) = lower2(dae, states, whenclauses);
        (vars2,knvars2,eqns2,reqns2,ieqns2,algs2,whenclauses_2) = lower2(DAE.DAE(xs), states, whenclauses_1);
        vars = merge_vars(vars1, vars2);
        eqns = list_append(eqns1, eqns2);
        ieqns = list_append(ieqns1, ieqns2);
        reqns = list_append(reqns1, reqns2);
        knvars = merge_vars(knvars1, knvars2);
        algs = list_append(algs1, algs2); then (vars,knvars,eqns,reqns,ieqns,algs,whenclauses_2);
    case (DAE.DAE(_ :: xs),states,whenclauses)
      local
        Variables v;
        EquationList e;
      equation 
        (v,kv,e,re,ie,al,whenclauses_1) = lower2(DAE.DAE(xs), states, whenclauses); then (v,kv,e,re,ie,al,whenclauses_1);
    case (_,_,_)
      equation 
        print("-lower2 failed\n"); then fail();
  end matchcontinue;
end lower2;

protected function lower_algorithms
  input Variables vars;
  input Algorithm_AlgorithmList algs;
  output EquationList eqns;
protected 
  type Algorithm_AlgorithmList = list<Algorithm.Algorithm>;
  type EquationList = list<Equation>;
algorithm 
  (eqns,_) := lower_algorithms2(vars, algs, 0);
end lower_algorithms;

protected function lower_algorithms2 " algindex
   updated algindex"
  input Variables in_variables;
  input Algorithm_AlgorithmList in_algorithm_algorithmlist;
  input Integer in_integer;
  output EquationList out_equationlist;
  output Integer out_integer;
protected 
  type Algorithm_AlgorithmList = list<Algorithm.Algorithm>;
  type EquationList = list<Equation>;
algorithm 
  (out_equationlist,out_integer):=
  matchcontinue (in_variables,in_algorithm_algorithmlist,in_integer)
    local
      Variables vars;
      Value aindx;
      EquationList eqns,eqns2,res;
      Algorithm.Algorithm a;
      Algorithm_AlgorithmList algs;
    case (vars,{},aindx) then ({},aindx); 
    case (vars,a :: algs,aindx)
      equation 
        eqns = lower_algorithm(vars, a, aindx);
        aindx = aindx + 1;
        (eqns2,aindx) = lower_algorithms2(vars, algs, aindx);
        res = list_append(eqns, eqns2); then (res,aindx);
  end matchcontinue;
end lower_algorithms2;

protected function lower_algorithm "algindx"
  input Variables vars;
  input Algorithm.Algorithm a;
  input Integer aindx;
  output EquationList lst;
protected 
  type EquationList = list<Equation>;
  Exp_ExpList inputs,outputs;
  Value numnodes;
algorithm 
  (inputs,outputs) := lower_algorithm_inputs_outputs(vars, a);
  numnodes := list_length(outputs);
  lst := lower_algorithm2(inputs, outputs, numnodes, aindx);
end lower_algorithm;

protected function lower_algorithm2 " outputs 
   numnodes
   aindx "
  input Exp_ExpList in_exp_explist1;
  input Exp_ExpList in_exp_explist2;
  input Integer in_integer3;
  input Integer in_integer4;
  output EquationList out_equationlist;
protected 
  type EquationList = list<Equation>;
algorithm 
  out_equationlist:=
  matchcontinue (in_exp_explist1,in_exp_explist2,in_integer3,in_integer4)
    local
      Value numnodes_1,numnodes,aindx;
      EquationList res;
      Exp_ExpList inputs,outputs;
    case (_,_,0,_) then {}; 
    case (inputs,outputs,numnodes,aindx)
      equation 
        numnodes_1 = numnodes - 1;
        res = lower_algorithm2(inputs, outputs, numnodes_1, aindx); then ALGORITHM(aindx,inputs,outputs) :: res;
  end matchcontinue;
end lower_algorithm2;

protected function lower_algorithm_inputs_outputs
  input Variables in_variables;
  input Algorithm.Algorithm in_algorithm;
  output Exp_ExpList out_exp_explist1;
  output Exp_ExpList out_exp_explist2;
algorithm 
  (out_exp_explist1,out_exp_explist2):=
  matchcontinue (in_variables,in_algorithm)
    local
      type Algorithm_StatementList = list<Algorithm.Statement>;
      Exp_ExpList inputs1,outputs1,inputs2,outputs2,inputs,outputs;
      Variables vars;
      Algorithm.Statement s;
      Algorithm_StatementList ss;
    case (_,Algorithm.ALGORITHM({})) then ({},{}); 
    case (vars,Algorithm.ALGORITHM(s :: ss))
      equation 
        (inputs1,outputs1) = lower_statement_inputs_outputs(vars, s);
        (inputs2,outputs2) = lower_algorithm_inputs_outputs(vars, Algorithm.ALGORITHM(ss));
        inputs = Util.list_union_p(inputs1, inputs2, Exp.exp_equal);
        outputs = Util.list_union_p(outputs1, outputs2, Exp.exp_equal); then (inputs,outputs);
  end matchcontinue;
end lower_algorithm_inputs_outputs;

protected function lower_statement_inputs_outputs " inputs, CREF or der(CREF) 
   outputs, CREF or der(CREF)"
  input Variables in_variables;
  input Algorithm.Statement in_statement;
  output Exp_ExpList out_exp_explist1;
  output Exp_ExpList out_exp_explist2;
algorithm 
  (out_exp_explist1,out_exp_explist2):=
  matchcontinue (in_variables,in_statement)
    local
      Exp_ExpList inputs;
      Variables vars;
      Exp.Type tp;
      Exp.ComponentRef cr;
      Exp.Exp e;
    case (vars,Algorithm.ASSIGN(tp,cr,e)) " vars   inputs, CREF or der(CREF)   outputs, CREF or der(CREF)"
      equation 
        inputs = states_and_vars_exp(e, vars); then (inputs,{Exp.CREF(cr,tp)});
    case (_,_)
      equation 
        print("lower_statement_input_outputs finished yet\n"); then fail();
  end matchcontinue;
end lower_statement_inputs_outputs;

protected function states_and_vars_exp " vars "
  input Exp.Exp in_exp;
  input Variables in_variables;
  output Exp_ExpList out_exp_explist;
algorithm 
  out_exp_explist:=
  matchcontinue (in_exp,in_variables)
    local
      type Exp_ExpListList = list<Exp_ExpList>;
      Exp.Exp e,e1,e2,e3;
      Exp.ComponentRef cr;
      Exp.Type tp;
      Variables vars;
      Exp_ExpList s1,s2,res,s3,expl;
      Value p;
      Exp_ExpListList lst;
    case ((e = Exp.CREF(cr,tp)),vars) " vars "
      equation 
        (_,_) = get_var(cr, vars); then {e};
    case (Exp.BINARY(e1,_,e2),vars)
      equation 
        s1 = states_and_vars_exp(e1, vars);
        s2 = states_and_vars_exp(e2, vars);
        res = Util.list_union_p(s1, s2, Exp.exp_equal); then res;
    case (Exp.UNARY(_,e),vars)
      equation 
        res = states_and_vars_exp(e, vars); then res;
    case (Exp.LBINARY(e1,_,e2),vars)
      equation 
        s1 = states_and_vars_exp(e1, vars);
        s2 = states_and_vars_exp(e2, vars);
        res = Util.list_union_p(s1, s2, Exp.exp_equal); then res;
    case (Exp.LUNARY(_,e),vars)
      equation 
        res = states_and_vars_exp(e, vars); then res;
    case (Exp.RELATION(e1,_,e2),vars)
      equation 
        s1 = states_and_vars_exp(e1, vars);
        s2 = states_and_vars_exp(e2, vars);
        res = list_append(s1, s2); then res;
    case (Exp.IFEXP(e1,e2,e3),vars)
      equation 
        s1 = states_and_vars_exp(e1, vars);
        s2 = states_and_vars_exp(e2, vars);
        s3 = states_and_vars_exp(e3, vars);
        res = Util.list_list_union_p({s1,s2,s3}, Exp.exp_equal); then res;
    case ((e = Exp.CALL(Absyn.IDENT("der"),{Exp.CREF(cr,_)},_,_)),vars)
      equation 
        (VAR(_,STATE(),_,_,_,_,_,_,_,_,_,_,_),_) = get_var(cr, vars); then {e};
    case (Exp.CALL(Absyn.IDENT("der"),{Exp.CREF(cr,_)},_,_),vars)
      equation 
        (_,p) = get_var(cr, vars); then {};
    case (Exp.CALL(_,expl,_,_),vars)
      equation 
        lst = Util.list_map_1(expl, states_and_vars_exp, vars);
        res = Util.list_list_union_p(lst, Exp.exp_equal); then res;
    case (Exp.ARRAY(_,_,expl),vars)
      equation 
        print("not impl. yet"); then {};
    case (Exp.MATRIX(_,_,_),vars)
      equation 
        Print.print_buf("states_and_vars_exp on MATRIX not impl.\n");
        print("states_and_vars_exp on MATRIX not impl.\n"); then {};
    case (Exp.TUPLE(expl),vars)
      equation 
        print("not impl. yet."); then {};
    case (Exp.CAST(_,e),vars)
      equation 
        res = states_and_vars_exp(e, vars); then res;
    case (Exp.ASUB(e,_),vars)
      equation 
        res = states_and_vars_exp(e, vars); then res;
    case (Exp.REDUCTION(_,e1,_,e2),vars)
      equation 
        s1 = states_and_vars_exp(e1, vars);
        s2 = states_and_vars_exp(e2, vars);
        res = Util.list_union_p(s1, s2, Exp.exp_equal); then res;
    case (_,_) then {}; 
  end matchcontinue;
end states_and_vars_exp;

protected function lower_eqn "* function: lower_eqn
 ** Helper function to lower2.
 ** Transforma a DAE.Element to Equation.
 *"
  input DAE.Element in_element;
  output Equation out_equation;
algorithm 
  out_equation:=
  matchcontinue (in_element)
    local Exp.Exp e1_1,e2_1,e1_2,e2_2,e1,e2;
    case (DAE.EQUATION(e1,e2))
      equation 
        e1_1 = Exp.simplify(e1);
        e2_1 = Exp.simplify(e2);
        e1_2 = Exp.stringify_crefs(e1_1);
        e2_2 = Exp.stringify_crefs(e2_1); then EQUATION(e1_2,e2_2);
    case (DAE.INITIALEQUATION(e1,e2))
      equation 
        e1_1 = Exp.simplify(e1);
        e2_1 = Exp.simplify(e2);
        e1_2 = Exp.stringify_crefs(e1_1);
        e2_2 = Exp.stringify_crefs(e2_1); then EQUATION(e1_2,e2_2);
  end matchcontinue;
end lower_eqn;

protected function lower_var "* function: lower_var
 ** Transforms a DAE variable to DAELOW variable.
 ** Includes changing the ComponentRef name to a simpler form 'a'.'b'.'c' becomes
 ** 'a.b.c' (as CREF_IDENT(\"a.b.c\",{}) )
 *"
  input DAE.Element in_element;
  input BinTree in_bintree;
  output Var out_var;
algorithm 
  out_var:=
  matchcontinue (in_element,in_bintree)
    local
      type Exp_SubscriptList = list<Exp.Subscript>;
      String origname;
      Exp.ComponentRef newname,name;
      VarKind kind_1;
      Exp_ExpOption bind_1,start_1,bind,start;
      DAE.VarKind kind;
      DAE.VarDirection dir;
      DAE.Type tp;
      Exp_SubscriptList dims;
      Absyn_PathList class_;
      DAE_VariableAttributesOption dae_var_attr;
      Absyn_CommentOption comment;
      BinTree states;
    case (DAE.VAR(name,kind,dir,tp,bind,dims,start,_,class_,dae_var_attr,comment),states) " states   rule	print \"-lower_var failed\n\" 
	------------------------
	lower_var (_,_) => fail"
      equation 
        origname = Exp.print_component_ref_str(name);
        newname = Exp.CREF_IDENT(origname,{});
        kind_1 = lower_varkind(kind, tp, newname, states);
        bind_1 = lower_binding(bind);
        start_1 = lower_binding(start); then VAR(newname,kind_1,dir,tp,bind_1,NONE,dims,start_1,-1,origname,
          class_,dae_var_attr,comment);
  end matchcontinue;
end lower_var;

protected function lower_binding "* function: lower_binding
 ** Helper function to lower_var
 *"
  input Exp_ExpOption in_exp_expoption;
  output Exp_ExpOption out_exp_expoption;
algorithm 
  out_exp_expoption:=
  matchcontinue (in_exp_expoption)
    local Exp.Exp e_1,e;
    case NONE then NONE; 
    case (SOME(e))
      equation 
        e_1 = Exp.stringify_crefs(e); then SOME(e);
  end matchcontinue;
end lower_binding;

protected function lower_known_var "* function: lower_known_var
 ** Helper function to lower2
 *"
  input DAE.Element in_element;
  output Var out_var;
algorithm 
  out_var:=
  matchcontinue (in_element)
    local
      type Exp_SubscriptList = list<Exp.Subscript>;
      VarKind kind_1;
      String origname;
      Exp_ExpOption bind_1,start_1,bind,start;
      Exp.ComponentRef name;
      DAE.VarKind kind;
      DAE.VarDirection dir;
      DAE.Type tp;
      Exp_SubscriptList dims;
      Absyn_PathList class_;
      DAE_VariableAttributesOption dae_var_attr;
      Absyn_CommentOption comment;
    case (DAE.VAR(name,kind,dir,tp,bind,dims,start,_,class_,dae_var_attr,comment))
      equation 
        kind_1 = lower_known_varkind(kind);
        origname = Exp.print_component_ref_str(name);
        bind_1 = lower_binding(bind);
        start_1 = lower_binding(start); then VAR(Exp.CREF_IDENT(origname,{}),kind_1,dir,tp,bind_1,NONE,dims,
          start_1,-1,origname,class_,dae_var_attr,comment);
  end matchcontinue;
end lower_known_var;

protected function lower_varkind
  input DAE.VarKind in_varkind;
  input DAE.Type in_type;
  input Exp.ComponentRef in_componentref;
  input BinTree in_bintree;
  output VarKind out_varkind;
algorithm 
  out_varkind:=
  matchcontinue (in_varkind,in_type,in_componentref,in_bintree)
    local
      Exp.ComponentRef v;
      BinTree states;
    case (DAE.VARIABLE(),_,v,states) " states   Fails for not states that are not algebraic variables, e.g.
	 parameters and constants "
      equation 
        _ = tree_get(states, v); then STATE();
    case (DAE.VARIABLE(),DAE.BOOL(),_,_) then DISCRETE(); 
    case (DAE.DISCRETE(),DAE.BOOL(),_,_) then DISCRETE(); 
    case (DAE.VARIABLE(),DAE.INT(),_,_) then DISCRETE(); 
    case (DAE.DISCRETE(),DAE.INT(),_,_) then DISCRETE(); 
    case (DAE.VARIABLE(),_,_,_) then VARIABLE(); 
    case (DAE.DISCRETE(),_,_,_) then DISCRETE(); 
  end matchcontinue;
end lower_varkind;

protected function lower_known_varkind "* function: lower_known_varkind
 ** Helper function to lower_known_var.
 *"
  input DAE.VarKind in_varkind;
  output VarKind out_varkind;
algorithm 
  out_varkind:=
  matchcontinue (in_varkind)
    case (DAE.PARAM()) then PARAM(); 
    case (DAE.CONST()) then CONST(); 
  end matchcontinue;
end lower_known_varkind;

public function incidence_matrix
  input DAELow in_daelow;
  output IncidenceMatrix out_incidencematrix;
algorithm 
  out_incidencematrix:=
  matchcontinue (in_daelow)
    local
      type EquationList = list<Equation>;
      type ValueList = list<Value>;
      type ValueListList = list<ValueList>;
      type ValueListArray = ValueList[:];
      EquationList eqnsl;
      ValueListList lstlst;
      ValueListArray arr;
      Variables vars,knvars;
      EquationArray eqns,reqns;
    case (DAELOW(vars,knvars,eqns,reqns,_,_,_))
      equation 
        eqnsl = equation_list(eqns);
        lstlst = incidence_matrix2(vars, eqnsl);
        arr = list_array(lstlst); then arr;
    case (_)
      equation 
        print("incidence_matrix failed\n"); then fail();
  end matchcontinue;
end incidence_matrix;

protected function incidence_matrix2 "* function: incidence_matrix2
 ** Helper function to incidence_matrix
 ** Calculates the incidence matrix as a list of list of integers
 *"
  input Variables in_variables;
  input EquationList in_equationlist;
  output IntegerListList out_integerlistlist;
protected 
  type EquationList = list<Equation>;
  type IntegerListList = list<IntegerList>;
algorithm 
  out_integerlistlist:=
  matchcontinue (in_variables,in_equationlist)
    local
      type ValueList = list<Value>;
      type ValueListList = list<ValueList>;
      ValueListList lst;
      ValueList row;
      Variables vars;
      Equation e;
      EquationList eqns;
    case (_,{}) then {}; 
    case (vars,e :: eqns)
      equation 
        lst = incidence_matrix2(vars, eqns);
        row = incidence_row(vars, e); then row :: lst;
    case (_,_)
      equation 
        print("incidence_matrix2 failed\n"); then fail();
  end matchcontinue;
end incidence_matrix2;

protected function incidence_row
  input Variables in_variables;
  input Equation in_equation;
  output IntegerList out_integerlist;
algorithm 
  out_integerlist:=
  matchcontinue (in_variables,in_equation)
    local
      type ValueList = list<Value>;
      type ValueListList = list<ValueList>;
      ValueList lst1,lst2,res,res_1;
      Variables vars;
      Exp.Exp e1,e2;
      Exp.ComponentRef cr;
      WhenEquation we;
      Value indx;
      Exp_ExpList inputs,outputs;
    case (vars,EQUATION(e1,e2))
      equation 
        lst1 = incidence_row_exp(e1, vars) "Inefficient to for each cref in equation traverse the varlist  Should use a map or a bintree";
        lst2 = incidence_row_exp(e2, vars);
        res = list_append(lst1, lst2); then res;
    case (vars,WHEN_EQUATION(we))
      equation 
        (cr,e2) = get_when_equation_expr(we) "Inefficient to for each cref in equation traverse the varlist  Should use a map or a bintree";
        e1 = Exp.CREF(cr,Exp.OTHER());
        lst1 = incidence_row_exp(e1, vars);
        lst2 = incidence_row_exp(e2, vars);
        res = list_append(lst1, lst2); then res;
    case (vars,ALGORITHM(indx,inputs,outputs))
      local ValueListList lst1,lst2,res;
      equation 
        lst1 = Util.list_map_1(inputs, incidence_row_exp, vars) "* For now assume that algorithm will be solvable for correct variables.
	 ** I.e. find all variables in algorithm and add to lst.
	 ** If algorithm later on needs to be inverted, i.e. solved for different
	 ** variables than calculated, a non linear solver or analysis of algorithm itself
	 ** needs to be implemented. 
	 *";
        lst2 = Util.list_map_1(outputs, incidence_row_exp, vars);
        res = list_append(lst1, lst2);
        res_1 = Util.list_flatten(res); then res_1;
    case (vars,_)
      equation 
        print("-incidence_row failed\n"); then fail();
  end matchcontinue;
end incidence_row;

protected function incidence_row_stmts "* function: incidence_row_stmts
 ** Helper function to incidence_row, investigates statements for
 ** variables, returning variable indexes.
 *"
  input Algorithm_StatementList in_algorithm_statementlist;
  input Variables in_variables;
  output IntegerList out_integerlist;
protected 
  type Algorithm_StatementList = list<Algorithm.Statement>;
algorithm 
  out_integerlist:=
  matchcontinue (in_algorithm_statementlist,in_variables)
    local
      type ValueList = list<Value>;
      type ValueListList = list<ValueList>;
      ValueList lst1,lst2,lst3,res,lst3_1;
      Exp.Type tp;
      Exp.ComponentRef cr;
      Exp.Exp e;
      Algorithm_StatementList rest,stmts;
      Variables vars;
      Exp_ExpList expl;
      Algorithm.Else else_;
    case ({},_) then {}; 
    case (Algorithm.ASSIGN(tp,cr,e) :: rest,vars)
      equation 
        lst1 = incidence_row_stmts(rest, vars);
        lst2 = incidence_row_exp(e, vars);
        lst3 = incidence_row_exp(Exp.CREF(cr,Exp.OTHER()), vars);
        res = Util.list_flatten({lst1,lst2,lst3}); then res;
    case (Algorithm.TUPLE_ASSIGN(tp,expl,e) :: rest,vars)
      local ValueListList lst3;
      equation 
        lst1 = incidence_row_stmts(rest, vars);
        lst2 = incidence_row_exp(e, vars);
        lst3 = Util.list_map_1(expl, incidence_row_exp, vars);
        lst3_1 = Util.list_flatten(lst3);
        res = Util.list_flatten({lst1,lst2,lst3_1}); then res;
    case (Algorithm.ASSIGN_ARR(tp,cr,e) :: rest,vars)
      equation 
        lst1 = incidence_row_stmts(rest, vars);
        lst2 = incidence_row_exp(e, vars);
        lst3 = incidence_row_exp(Exp.CREF(cr,Exp.OTHER()), vars);
        res = Util.list_flatten({lst1,lst2,lst3}); then res;
    case (Algorithm.IF(e,stmts,else_) :: rest,vars)
      equation 
        print("incidence_row_stmts on IF not implemented\n"); then {};
    case (Algorithm.FOR(_,_,_,_,_) :: rest,vars)
      equation 
        print("incidence_row_stmts on FOR not implemented\n"); then {};
    case (Algorithm.WHILE(_,_) :: rest,vars)
      equation 
        print("incidence_row_stmts on WHILE not implemented\n"); then {};
    case (Algorithm.WHEN(e,_) :: rest,vars)
      equation 
        print("incidence_row_stmts on WHEN not implemented\n"); then {};
    case (Algorithm.ASSERT(_,_) :: rest,vars)
      equation 
        print("incidence_row_stmts on ASSERT not implemented\n"); then {};
  end matchcontinue;
end incidence_row_stmts;

protected function incidence_row_exp
  input Exp.Exp in_exp;
  input Variables in_variables;
  output IntegerList out_integerlist;
algorithm 
  out_integerlist:=
  matchcontinue (in_exp,in_variables)
    local
      type ValueList = list<Value>;
      type ValueListList = list<ValueList>;
      Value p,p_1;
      Exp.ComponentRef cr;
      Variables vars;
      ValueList s1,s2,res,s3;
      Exp.Exp e1,e2,e,e3;
      ValueListList lst;
      Exp_ExpList expl;
    case (Exp.CREF(cr,_),vars)
      equation 
        (VAR(_,STATE(),_,_,_,_,_,_,_,_,_,_,_),p) = get_var(cr, vars) " If variable x is a state, der(x) is a variable in incidence matrix,
	 x is inserted as negative value, since it is needed by debugging and index
	 reduction using dummy derivatives";
        p_1 = 0 - p; then {p_1};
    case (Exp.CREF(cr,_),vars)
      equation 
        (VAR(_,VARIABLE(),_,_,_,_,_,_,_,_,_,_,_),p) = get_var(cr, vars); then {p};
    case (Exp.CREF(cr,_),vars)
      equation 
        (VAR(_,DISCRETE(),_,_,_,_,_,_,_,_,_,_,_),p) = get_var(cr, vars); then {p};
    case (Exp.CREF(cr,_),vars)
      equation 
        (VAR(_,DUMMY_DER(),_,_,_,_,_,_,_,_,_,_,_),p) = get_var(cr, vars); then {p};
    case (Exp.CREF(cr,_),vars)
      equation 
        (VAR(_,DUMMY_STATE(),_,_,_,_,_,_,_,_,_,_,_),p) = get_var(cr, vars); then {p};
    case (Exp.BINARY(e1,_,e2),vars)
      equation 
        s1 = incidence_row_exp(e1, vars);
        s2 = incidence_row_exp(e2, vars);
        res = list_append(s1, s2); then res;
    case (Exp.UNARY(_,e),vars)
      equation 
        res = incidence_row_exp(e, vars); then res;
    case (Exp.LBINARY(e1,_,e2),vars)
      equation 
        s1 = incidence_row_exp(e1, vars);
        s2 = incidence_row_exp(e2, vars);
        res = list_append(s1, s2); then res;
    case (Exp.LUNARY(_,e),vars)
      equation 
        res = incidence_row_exp(e, vars); then res;
    case (Exp.RELATION(e1,_,e2),vars)
      equation 
        s1 = incidence_row_exp(e1, vars);
        s2 = incidence_row_exp(e2, vars);
        res = list_append(s1, s2); then res;
    case (Exp.IFEXP(e1,e2,e3),vars)
      equation 
        s1 = incidence_row_exp(e1, vars);
        s2 = incidence_row_exp(e2, vars);
        s3 = incidence_row_exp(e3, vars);
        res = Util.list_flatten({s1,s2,s3}); then res;
    case (Exp.CALL(Absyn.IDENT("der"),{Exp.CREF(cr,_)},_,_),vars)
      equation 
        (VAR(_,STATE(),_,_,_,_,_,_,_,_,_,_,_),p) = get_var(cr, vars); then {p};
    case (Exp.CALL(Absyn.IDENT("der"),{Exp.CREF(cr,_)},_,_),vars)
      equation 
        (_,p) = get_var(cr, vars); then {};
    case (Exp.CALL(_,expl,_,_),vars)
      equation 
        lst = Util.list_map_1(expl, incidence_row_exp, vars);
        res = Util.list_flatten(lst); then res;
    case (Exp.ARRAY(_,_,expl),vars)
      equation 
        print("not impl. yet"); then {};
    case (Exp.MATRIX(_,_,_),vars)
      equation 
        Print.print_buf("incidence_row_exp on MATRIX not impl.\n");
        print("incidence_row_exp on MATRIX not impl.\n"); then {};
    case (Exp.TUPLE(expl),vars)
      equation 
        print("not impl. yet."); then {};
    case (Exp.CAST(_,e),vars)
      equation 
        res = incidence_row_exp(e, vars); then res;
    case (Exp.ASUB(e,_),vars)
      equation 
        res = incidence_row_exp(e, vars); then res;
    case (Exp.REDUCTION(_,e1,_,e2),vars)
      equation 
        s1 = incidence_row_exp(e1, vars);
        s2 = incidence_row_exp(e2, vars);
        res = list_append(s1, s2); then res;
    case (_,_) then {}; 
  end matchcontinue;
end incidence_row_exp;

protected function empty_vars "*relation empty_vars
 ** Returns a Variable datastructure that is empty.
 ** Using the bucketsize 10000 and array size 1000.
 *"
  output Variables out_variables;
protected 
  type VarOptionList = list<VarOption>;
  CrefIndexListArray arr;
  VarOptionList lst;
  VarOptionArray emptyarr;
algorithm 
  arr := fill({}, 10000);
  lst := Util.list_fill(NONE, 1000);
  emptyarr := list_array(lst);
  out_variables := VARIABLES(arr,VARIABLE_ARRAY(0,1000,emptyarr),10000,0);
end empty_vars;

protected function merge_vars "* function: merge_vars
 ** Takes two sets of Variables and merges them. The variables of the first argument takes 
 ** precedence over the second set, i.e. if a variable name exists in both sets, the variable
 ** definition from the first set is used.
 *"
  input Variables in_variables1;
  input Variables in_variables2;
  output Variables out_variables;
algorithm 
  out_variables:=
  matchcontinue (in_variables1,in_variables2)
    local
      type VarList = list<Var>;
      VarList varlst;
      Variables vars1_1,vars1,vars2;
    case (vars1,vars2)
      equation 
        varlst = var_list(vars2);
        vars1_1 = Util.list_fold(varlst, add_var, vars1); then vars1_1;
    case (_,_)
      equation 
        print("-merge_vars failed\n"); then fail();
  end matchcontinue;
end merge_vars;

protected function add_var "* function: add_var
 ** Add a variable to Variables.
 ** If the variable allready exists, the function updates the variable.
 *"
  input Var in_var;
  input Variables in_variables;
  output Variables out_variables;
algorithm 
  out_variables:=
  matchcontinue (in_var,in_variables)
    local
      Value hval,indx,newpos,n_1,bsize,n,indx_1;
      VariableArray varr_1,varr;
      CrefIndexList indexes;
      CrefIndexListArray hashvec_1,hashvec;
      Var v,newv;
      Exp.ComponentRef cr;
      Variables vars;
    case ((v = VAR(cr,_,_,_,_,_,_,_,_,_,_,_,_)),(vars = VARIABLES(hashvec,varr,bsize,n)))
      equation 
        failure((_,_) = get_var(cr, vars)) " adding when not existing previously ";
        hval = hash_component_ref(cr);
        indx = int_mod(hval, bsize);
        newpos = vararray_length(varr);
        varr_1 = vararray_add(varr, v);
        indexes = hashvec[indx + 1];
        hashvec_1 = array_update(hashvec, indx + 1, CREFINDEX(cr,newpos) :: indexes);
        n_1 = vararray_length(varr_1); then VARIABLES(hashvec_1,varr_1,bsize,n_1);
    case ((newv = VAR(cr,_,_,_,_,_,_,_,_,_,_,_,_)),(vars = VARIABLES(hashvec,varr,bsize,n)))
      equation 
        (v,indx) = get_var(cr, vars) " adding when allready present => Updating value ";
        indx_1 = indx - 1;
        varr_1 = vararray_setnth(varr, indx_1, newv); then VARIABLES(hashvec,varr_1,bsize,n);
    case (_,_)
      equation 
        print("-add_var failed\n"); then fail();
  end matchcontinue;
end add_var;

public function vararray_length "* function: vararray_length
 ** Returns the number of variable in the VariableArray
 *"
  input VariableArray in_variablearray;
  output Integer out_integer;
algorithm 
  out_integer:=
  matchcontinue (in_variablearray)
    local Value n;
    case (VARIABLE_ARRAY(n,_,_)) then n; 
  end matchcontinue;
end vararray_length;

public function vararray_add "* function: vararray_add
 ** Adds a variable last to the VariableArray, increasing array size
 ** if no space left by factor 1.4
*"
  input VariableArray in_variablearray;
  input Var in_var;
  output VariableArray out_variablearray;
algorithm 
  out_variablearray:=
  matchcontinue (in_variablearray,in_var)
    local
      Value n_1,n,size,expandsize,newsize;
      VarOptionArray arr_1,arr,arr_2;
      Var v;
      Real rsize,rexpandsize;
    case (VARIABLE_ARRAY(n,size,arr),v)
      equation 
        (n < size) = true " Have space to add array elt. ";
        n_1 = n + 1;
        arr_1 = array_update(arr, n + 1, SOME(v)); then VARIABLE_ARRAY(n_1,size,arr_1);
    case (VARIABLE_ARRAY(n,size,arr),v)
      equation 
        (n < size) = false " Do NOT have splace to add array elt. Expand with factor 1.4 ";
        rsize = int_real(size);
        rexpandsize = rsize*.0.4;
        expandsize = real_int(rexpandsize);
        newsize = expandsize + size;
        arr_1 = Util.array_expand(expandsize, arr, NONE);
        n_1 = n + 1;
        arr_2 = array_update(arr_1, n + 1, SOME(v)); then VARIABLE_ARRAY(n_1,newsize,arr_2);
    case (_,_)
      equation 
        print("-vararray_add failed\n"); then fail();
  end matchcontinue;
end vararray_add;

public function vararray_setnth " n 
   v "
  input VariableArray in_variablearray;
  input Integer in_integer;
  input Var in_var;
  output VariableArray out_variablearray;
algorithm 
  out_variablearray:=
  matchcontinue (in_variablearray,in_integer,in_var)
    local
      VarOptionArray arr_1,arr;
      Value n,size,pos;
      Var v;
    case (VARIABLE_ARRAY(n,size,arr),pos,v) " n   v "
      equation 
        (pos < size) = true;
        arr_1 = array_update(arr, pos + 1, SOME(v)); then VARIABLE_ARRAY(n,size,arr_1);
    case (_,_,_)
      equation 
        print("-vararray_setnth failed\n"); then fail();
  end matchcontinue;
end vararray_setnth;

public function vararray_nth " n "
  input VariableArray in_variablearray;
  input Integer in_integer;
  output Var out_var;
algorithm 
  out_var:=
  matchcontinue (in_variablearray,in_integer)
    local
      Var v;
      Value n,pos,len;
      VarOptionArray arr;
      String ps,lens,ns;
    case (VARIABLE_ARRAY(n,_,arr),pos) " n "
      equation 
        (pos < n) = true;
        SOME(v) = array_nth(arr, pos); then v;
    case (VARIABLE_ARRAY(n,_,arr),pos)
      equation 
        (pos < n) = true;
        NONE = arr[pos + 1];
        print("vararray_nth has NONE!!!\n"); then fail();
    case ((arr = VARIABLE_ARRAY(n,_,_)),pos)
      local VariableArray arr;
      equation 
        print("-vararray_nth , pos : ");
        ps = int_string(pos);
        print(ps);
        print("\n  array_length:");
        len = vararray_length(arr);
        lens = int_string(len);
        print(lens);
        print("\n");
        ns = int_string(n);
        print("n :");
        print(ns);
        print("\n"); then fail();
  end matchcontinue;
end vararray_nth;

protected function replace_var "* function: replace_var
 ** Takes a 'Var' list and a 'Var' and replaces the var with the 
 ** same ComponentRef in Var list with Var
*"
  input VarList in_varlist;
  input Var in_var;
  output VarList out_varlist;
protected 
  type VarList = list<Var>;
algorithm 
  out_varlist:=
  matchcontinue (in_varlist,in_var)
    local
      Exp.ComponentRef cr1,cr2;
      VarList vs,vs_1;
      Var v,repl;
    case ({},_) then {}; 
    case (VAR(cr1,_,_,_,_,_,_,_,_,_,_,_,_) :: vs,(v = VAR(cr2,_,_,_,_,_,_,_,_,_,_,_,_)))
      equation 
        true = Exp.cref_equal(cr1, cr2); then v :: vs;
    case (v :: vs,(repl = VAR(cr2,_,_,_,_,_,_,_,_,_,_,_,_)))
      equation 
        vs_1 = replace_var(vs, repl); then v :: vs_1;
  end matchcontinue;
end replace_var;

protected function hash_component_ref "* function: hash_component_ref
 ** Calculates a hash value for Exp.ComponentRef
 *"
  input Exp.ComponentRef cr;
  output Integer res;
protected 
  String crstr;
algorithm 
  crstr := Exp.print_component_ref_str(cr);
  res := hash_string(crstr);
end hash_component_ref;

protected function hash_string "* function: hash_string
 ** Calculates a hash value of a string
 *"
  input String str;
  output Integer res;
algorithm 
  res := System.hash(str) "string_list(str) => charlst &
	hash_chars(charlst) => res";
end hash_string;

protected function hash_chars "* function: hash_chars
 ** Calculates a hash value for a list of chars
 *"
  input charList in_charlist;
  output Integer out_integer;
protected 
  type charList = list<char>;
algorithm 
  out_integer:=
  matchcontinue (in_charlist)
    local
      Value c2,c1;
      char c;
      charList cs;
    case ({}) then 0; 
    case (c :: cs)
      equation 
        c2 = char_int(c);
        c1 = hash_chars(cs); then c1 + c2;
  end matchcontinue;
end hash_chars;

public function get_var_at "* function: get_var_at
 ** Return variable at a given position, enumerated from 1..n
 *"
  input Variables in_variables;
  input Integer in_integer;
  output Var out_var;
algorithm 
  out_var:=
  matchcontinue (in_variables,in_integer)
    local
      Value pos,n;
      Var v;
      VariableArray vararr;
    case (VARIABLES(_,vararr,_,_),n)
      equation 
        pos = n - 1;
        v = vararray_nth(vararr, pos); then v;
  end matchcontinue;
end get_var_at;

public function get_var "* function: get_var
 ** Return a variable and its index in the vector.
 ** The index is enumerated from 1..n 
 *"
  input Exp.ComponentRef in_componentref;
  input Variables in_variables;
  output Var out_var;
  output Integer out_integer;
algorithm 
  (out_var,out_integer):=
  matchcontinue (in_componentref,in_variables)
    local
      Value hval,hashindx,indx,indx_1,bsize,n;
      CrefIndexList indexes;
      Var v;
      Exp.ComponentRef cr2,cr;
      CrefIndexListArray hashvec;
      VariableArray varr;
      String str;
    case (cr,VARIABLES(hashvec,varr,bsize,n))
      equation 
        hval = hash_component_ref(cr);
        hashindx = int_mod(hval, bsize);
        indexes = hashvec[hashindx + 1];
        indx = get_var2(cr, indexes);
        (v = VAR(cr2,_,_,_,_,_,_,_,_,_,_,_,_)) = vararray_nth(varr, indx);
        true = Exp.cref_equal(cr, cr2);
        indx_1 = indx + 1; then (v,indx_1);
    case (cr,VARIABLES(hashvec,varr,bsize,n))
      equation 
        hval = hash_component_ref(cr);
        hashindx = int_mod(hval, bsize);
        indexes = hashvec[hashindx + 1];
        indx = get_var2(cr, indexes);
        failure((_) = vararray_nth(varr, indx));
        print("get var failed because vararray_nth failed, cr:");
        str = Exp.print_component_ref_str(cr);
        print(str);
        print("\n"); then fail();
  end matchcontinue;
end get_var;

public function exists_var "* function: exists_var
 ** Return true if a variable exists in th vector
 *"
  input Exp.ComponentRef in_componentref;
  input Variables in_variables;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_componentref,in_variables)
    local
      Value hval,hashindx,indx,bsize,n;
      CrefIndexList indexes;
      Var v;
      Exp.ComponentRef cr2,cr;
      CrefIndexListArray hashvec;
      VariableArray varr;
      String str;
    case (cr,VARIABLES(hashvec,varr,bsize,n))
      equation 
        hval = hash_component_ref(cr);
        hashindx = int_mod(hval, bsize);
        indexes = hashvec[hashindx + 1];
        indx = get_var2(cr, indexes);
        (v = VAR(cr2,_,_,_,_,_,_,_,_,_,_,_,_)) = vararray_nth(varr, indx);
        true = Exp.cref_equal(cr, cr2); then true;
    case (cr,VARIABLES(hashvec,varr,bsize,n))
      equation 
        hval = hash_component_ref(cr);
        hashindx = int_mod(hval, bsize);
        indexes = hashvec[hashindx + 1];
        indx = get_var2(cr, indexes);
        failure((_) = vararray_nth(varr, indx));
        print("could not found variable, cr:");
        str = Exp.print_component_ref_str(cr);
        print(str);
        print("\n"); then false;
  end matchcontinue;
end exists_var;

protected function get_var2 "* function: get_var2
 ** Helper function to get_var
 *"
  input Exp.ComponentRef in_componentref;
  input CrefIndexList in_crefindexlist;
  output Integer out_integer;
algorithm 
  out_integer:=
  matchcontinue (in_componentref,in_crefindexlist)
    local
      Exp.ComponentRef cr,cr2;
      Value v,res;
      CrefIndexList vs;
    case (cr,CREFINDEX(cr2,v) :: _)
      equation 
        true = Exp.cref_equal(cr, cr2); then v;
    case (cr,v :: vs)
      local CrefIndex v;
      equation 
        res = get_var2(cr, vs); then res;
  end matchcontinue;
end get_var2;

protected function delete_var "* function: delete_var
 ** Deletes a variable from 'Variables'. This is an expensive operation since we need to create a new 
 ** binary tree with new indexes as well as a new compacted vector of variables.
 *"
  input Variables in_variables;
  input Exp.ComponentRef in_componentref;
  output Variables out_variables;
algorithm 
  out_variables:=
  matchcontinue (in_variables,in_componentref)
    local
      type VarList = list<Var>;
      VarList varlst,varlst_1;
      Variables newvars,newvars_1;
      CrefIndexListArray hashvec;
      VariableArray varr;
      Value bsize,n;
      Exp.ComponentRef cr;
    case (VARIABLES(hashvec,varr,bsize,n),cr)
      equation 
        varlst = vararray_list(varr);
        varlst_1 = delete_var2(cr, varlst);
        newvars = empty_vars();
        newvars_1 = add_vars(varlst_1, newvars); then newvars_1;
  end matchcontinue;
end delete_var;

protected function delete_var2 "* function: delete_var2
 ** Helper function to delete_var.
 ** Deletes the var named 'Exp.ComponentRef' from the 'Var' list.
 *"
  input Exp.ComponentRef in_componentref;
  input VarList in_varlist;
  output VarList out_varlist;
protected 
  type VarList = list<Var>;
algorithm 
  out_varlist:=
  matchcontinue (in_componentref,in_varlist)
    local
      Exp.ComponentRef cr1,cr2;
      VarList vs,vs_1;
      Var v;
    case (_,{}) then {}; 
    case (cr1,VAR(cr2,_,_,_,_,_,_,_,_,_,_,_,_) :: vs)
      equation 
        true = Exp.cref_equal(cr1, cr2); then vs;
    case (cr1,v :: vs)
      equation 
        vs_1 = delete_var2(cr1, vs); then v :: vs_1;
  end matchcontinue;
end delete_var2;

public function transpose_matrix "* function: transpose_matrix
 ** Calculates the transpose of the incidence matrix, i.e. which equations 
 ** each variable is present in.
 *"
  input IncidenceMatrix m;
  output IncidenceMatrixT mt;
protected 
  type ValueList = list<Value>;
  type ValueListList = list<ValueList>;
  ValueListList mlst,mtlst;
algorithm 
  mlst := array_list(m);
  mtlst := transpose_matrix2(mlst);
  mt := list_array(mtlst);
end transpose_matrix;

protected function transpose_matrix2
  input IntegerListList in_integerlistlist;
  output IntegerListList out_integerlistlist;
protected 
  type IntegerListList = list<IntegerList>;
algorithm 
  out_integerlistlist:=
  matchcontinue (in_integerlistlist)
    local
      type ValueList = list<Value>;
      type ValueListList = list<ValueList>;
      Value neq;
      ValueListList mt,m;
    case (m)
      equation 
        neq = list_length(m);
        mt = transpose_matrix3(m, neq, 0, {}); then mt;
    case (_)
      equation 
        print("#transpose_matrix2 failed\n"); then fail();
  end matchcontinue;
end transpose_matrix2;

protected function transpose_matrix3
  input IntegerListList in_integerlistlist1;
  input Integer in_integer2;
  input Integer in_integer3;
  input IntegerListList in_integerlistlist4;
  output IntegerListList out_integerlistlist;
protected 
  type IntegerListList = list<IntegerList>;
algorithm 
  out_integerlistlist:=
  matchcontinue (in_integerlistlist1,in_integer2,in_integer3,in_integerlistlist4)
    local
      type ValueList = list<Value>;
      type ValueListList = list<ValueList>;
      Value neq_1,eqno_1,neq,eqno;
      ValueListList mt_1,m,mt;
      ValueList row;
    case (_,0,_,_) then {}; 
    case (m,neq,eqno,mt)
      equation 
        neq_1 = neq - 1;
        eqno_1 = eqno + 1;
        mt_1 = transpose_matrix3(m, neq_1, eqno_1, mt);
        row = transpose_row(m, eqno_1, 1); then row :: mt_1;
  end matchcontinue;
end transpose_matrix3;

public function abs_incidence_matrix "* function abs_incidence_matrix
 ** author: PA
 ** 
 ** Applies absolute value to all entries in the incidence matrix.
 ** This can be used when e.g. der(x) and x are considered the same variable.
 *"
  input IncidenceMatrix m;
  output IncidenceMatrix res;
protected 
  type ValueList = list<Value>;
  type ValueListList = list<ValueList>;
  ValueListList lst,lst_1;
algorithm 
  lst := array_list(m);
  lst_1 := Util.list_list_map(lst, int_abs);
  res := list_array(lst_1);
end abs_incidence_matrix;

protected function transpose_row " function: transpose_row
 ** Helper function to transpose_matrix2.
 ** Input: IncidenceMatrix (eqn => var)
 ** Input: row number (variable)
 ** Input: iterator (start with one)
*"
  input IntegerListList in_integerlistlist1;
  input Integer in_integer2;
  input Integer in_integer3;
  output IntegerList out_integerlist;
protected 
  type IntegerListList = list<IntegerList>;
algorithm 
  out_integerlist:=
  matchcontinue (in_integerlistlist1,in_integer2,in_integer3)
    local
      type ValueList = list<Value>;
      type ValueListList = list<ValueList>;
      Value eqn_1,varno,eqn,varno_1,eqnneg;
      ValueList res,m;
      ValueListList ms;
    case ({},_,_) then {}; 
    case (m :: ms,varno,eqn)
      equation 
        true = list_member(varno, m);
        eqn_1 = eqn + 1;
        res = transpose_row(ms, varno, eqn_1); then eqn :: res;
    case (m :: ms,varno,eqn)
      equation 
        varno_1 = 0 - varno " Negative index present, state variable.  list_member(varno,m) => false &";
        true = list_member(varno_1, m);
        eqnneg = 0 - eqn;
        eqn_1 = eqn + 1;
        res = transpose_row(ms, varno, eqn_1); then eqnneg :: res;
    case (m :: ms,varno,eqn)
      equation 
        eqn_1 = eqn + 1 " not present at all ";
        res = transpose_row(ms, varno, eqn_1); then res;
    case (_,_,_)
      equation 
        print("-transpose_row failed\n"); then fail();
  end matchcontinue;
end transpose_row;

public function dump_incidence_matrix "* function: dump_incidence_matrix
 ** Prints the incidence matrix on stdout.
 *"
  input IncidenceMatrix m;
protected 
  type ValueList = list<Value>;
  type ValueListList = list<ValueList>;
  Value mlen;
  String mlen_str;
  ValueListList m_1;
algorithm 
  print("Incidence Matrix (row == equation)\n");
  print("====================================\n");
  mlen := array_length(m);
  mlen_str := int_string(mlen);
  print("number of rows: ");
  print(mlen_str);
  print("\n");
  m_1 := array_list(m);
  dump_incidence_matrix2(m_1);
end dump_incidence_matrix;

public function dump_incidence_matrix_t "* function: dump_incidence_matrix_t
 ** Prints the transposed incidence matrix on stdout.
 *"
  input IncidenceMatrix m;
protected 
  type ValueList = list<Value>;
  type ValueListList = list<ValueList>;
  Value mlen;
  String mlen_str;
  ValueListList m_1;
algorithm 
  print("Transpose Incidence Matrix (row == var)\n");
  print("=====================================\n");
  mlen := array_length(m);
  mlen_str := int_string(mlen);
  print("number of rows: ");
  print(mlen_str);
  print("\n");
  m_1 := array_list(m);
  dump_incidence_matrix2(m_1);
end dump_incidence_matrix_t;

protected function dump_incidence_matrix2 "* function: dump_incidence_matrix2
 ** Helper function to dump_incidence_matrix (+_t).
 *"
  input IntegerListList in_integerlistlist;
protected 
  type IntegerListList = list<IntegerList>;
algorithm 
  _:=
  matchcontinue (in_integerlistlist)
    local
      type ValueList = list<Value>;
      type ValueListList = list<ValueList>;
      ValueList row;
      ValueListList rows;
    case ({}) then (); 
    case (row :: rows)
      equation 
        dump_incidence_row(row);
        dump_incidence_matrix2(rows); then ();
  end matchcontinue;
end dump_incidence_matrix2;

protected function dump_incidence_row "* function: dump_incidence_row
 ** Helper function to dump_incidence_matrix2.
 *"
  input IntegerList in_integerlist;
algorithm 
  _:=
  matchcontinue (in_integerlist)
    local
      type ValueList = list<Value>;
      String s;
      Value x;
      ValueList xs;
    case {}
      equation 
        print("\n"); then ();
    case (x :: xs)
      equation 
        s = int_string(x);
        print(s);
        print(" ");
        dump_incidence_row(xs); then ();
  end matchcontinue;
end dump_incidence_row;

public function dump_matching "* function: dump_matching
 ** prints the matching information on stdout.
 *"
  input IntegerVector v;
protected 
  type IntegerVector = Integer[:];
  Value len;
  String len_str;
algorithm 
  print("Matching\n");
  print("========\n");
  len := array_length(v);
  len_str := int_string(len);
  print(len_str);
  print(" variables and equations\n");
  dump_matching2(v, 0);
end dump_matching;

protected function dump_matching2 "* function: dump_matching2
 ** Helper function to dump_matching.
 *"
  input IntegerVector in_integervector;
  input Integer in_integer;
protected 
  type IntegerVector = Integer[:];
algorithm 
  _:=
  matchcontinue (in_integervector,in_integer)
    local
      type ValueVector = Value[:];
      Value len,i_1,eqn,i;
      String s,s2;
      ValueVector v;
    case (v,i)
      equation 
        len = array_length(v);
        i_1 = i + 1;
        (len == i_1) = true;
        s = int_string(i_1);
        eqn = v[i + 1];
        s2 = int_string(eqn);
        print("var ");
        print(s);
        print(" is solved in eqn ");
        print(s2);
        print("\n"); then ();
    case (v,i)
      equation 
        len = array_length(v);
        i_1 = i + 1;
        (len == i_1) = false;
        s = int_string(i_1);
        eqn = v[i + 1];
        s2 = int_string(eqn);
        print("var ");
        print(s);
        print(" is solved in eqn ");
        print(s2);
        print("\n");
        dump_matching2(v, i_1); then ();
  end matchcontinue;
end dump_matching2;

public function matching_algorithm " vector of equation indices
   vector of variable indices"
  input DAELow in_daelow;
  input IncidenceMatrix in_incidencematrix;
  input IncidenceMatrixT in_incidencematrixt;
  output IntegerVector out_integervector1;
  output IntegerVector out_integervector2;
  output DAELow out_daelow3;
  output IncidenceMatrix out_incidencematrix4;
  output IncidenceMatrixT out_incidencematrixt5;
protected 
  type IntegerVector = Integer[:];
algorithm 
  (out_integervector1,out_integervector2,out_daelow3,out_incidencematrix4,out_incidencematrixt5):=
  matchcontinue (in_daelow,in_incidencematrix,in_incidencematrixt)
    local
      type ValueList = list<Value>;
      type ValueListArray = ValueList[:];
      type ValueVector = Value[:];
      Value nvars,neqns,memsize;
      Assignments assign1,assign2,ass1,ass2;
      DAELow dae;
      ValueListArray m,mt;
      ValueVector vec1,vec2,v1,v2;
    case (dae,m,mt) " vector of equation indices  vector of variable indices"
      equation 
        check_matching(dae);
        nvars = array_length(m);
        neqns = array_length(mt);
        (nvars > 0) = true;
        (neqns > 0) = true;
        memsize = nvars + nvars;
        assign1 = assignments_create(nvars, memsize, 0);
        assign2 = assignments_create(nvars, memsize, 0);
        (ass1,ass2,dae,m,mt) = matching_algorithm2(dae, m, mt, nvars, neqns, 1, assign1, assign2);
        vec1 = assignments_vector(ass1);
        vec2 = assignments_vector(ass2); then (vec1,vec2,dae,m,mt);
    case (dae,m,mt)
      equation 
        Print.print_error_buf("#Error, matching failed\n");
        v1 = list_array({});
        v2 = list_array({}); then (v1,v2,dae,m,mt);
  end matchcontinue;
end matching_algorithm;

protected function check_matching "* function: check_matching
 ** Checks that the matching is correct, i.e. that the number of variables
 ** is the same as the number of equations. If not, the function fails and
 ** prints an error message.
 *"
  input DAELow in_daelow;
algorithm 
  _:=
  matchcontinue (in_daelow)
    local
      Value esize,vars_size;
      EquationArray eqns;
    case (DAELOW(VARIABLES(_,_,_,vars_size),_,eqns,_,_,_,_))
      equation 
        esize = equation_size(eqns);
        (esize == vars_size) = true; then ();
    case (DAELOW(VARIABLES(_,_,_,vars_size),_,eqns,_,_,_,_))
      equation 
        esize = equation_size(eqns);
        (esize < vars_size) = true;
        Print.print_error_buf("#Error, too few equations. Underdetermined system\n");
        print("#Error, to few equations\n"); then fail();
    case (DAELOW(VARIABLES(_,_,_,vars_size),_,eqns,_,_,_,_))
      equation 
        esize = equation_size(eqns);
        (esize > vars_size) = true;
        Print.print_error_buf("#Error, to many equations. Overdetermined system.\n");
        print("#Error, too many equations."); then fail();
    case (_)
      equation 
        print("check_matching failed\n"); then fail();
  end matchcontinue;
end check_matching;

protected function assignments_vector "* function: assignments_vector
** Converts Assignments to vector of int elements
 *"
  input Assignments in_assignments;
  output IntegerVector out_integervector;
protected 
  type IntegerVector = Integer[:];
algorithm 
  out_integervector:=
  matchcontinue (in_assignments)
    local
      type ValueArray = Value[:];
      type ValueVector = Value[:];
      ValueArray newarr,newarr_1,arr;
      ValueVector vec;
      Value size;
    case (ASSIGNMENTS(size,_,arr))
      equation 
        newarr = fill(0, size);
        newarr_1 = Util.array_n_copy(arr, newarr, size);
        vec = array_copy(newarr_1); then vec;
    case (_)
      equation 
        print("-assignments_vector failed\n"); then fail();
  end matchcontinue;
end assignments_vector;

protected function assignments_create
  input Integer n;
  input Integer memsize;
  input Integer v;
  output Assignments out_assignments;
protected 
  type ValueList = list<Value>;
  type ValueArray = Value[:];
  ValueList lst;
  ValueArray arr;
algorithm 
  lst := Util.list_fill(0, memsize);
  arr := list_array(lst) "	array_create(memsize,v) => arr &
	";
  out_assignments := ASSIGNMENTS(n,memsize,arr);
end assignments_create;

protected function assignments_setnth
  input Assignments in_assignments1;
  input Integer in_integer2;
  input Integer in_integer3;
  output Assignments out_assignments;
algorithm 
  out_assignments:=
  matchcontinue (in_assignments1,in_integer2,in_integer3)
    local
      type ValueArray = Value[:];
      ValueArray arr;
      Value s,ms,n,v;
    case (ASSIGNMENTS(s,ms,arr),n,v) "n value"
      equation 
        arr = array_update(arr, n + 1, v); then ASSIGNMENTS(s,ms,arr);
    case (_,_,_)
      equation 
        print("-assignments_setnth failed\n"); then fail();
  end matchcontinue;
end assignments_setnth;

protected function assignments_expand "* function: assignments_expand
 ** Expands the assignments array with n values, initialized with zero.
 *"
  input Assignments in_assignments;
  input Integer in_integer;
  output Assignments out_assignments;
algorithm 
  out_assignments:=
  matchcontinue (in_assignments,in_integer)
    local
      Assignments ass,ass_1,ass_2;
      Value n_1,n;
    case (ass,0) then ass; 
    case (ass,n)
      equation 
        ass_1 = assignments_add(ass, 0);
        n_1 = n - 1;
        ass_2 = assignments_expand(ass_1, n_1); then ass_2;
  end matchcontinue;
end assignments_expand;

protected function assignments_add
  input Assignments in_assignments;
  input Integer in_integer;
  output Assignments out_assignments;
algorithm 
  out_assignments:=
  matchcontinue (in_assignments,in_integer)
    local
      type ValueArray = Value[:];
      Real msr,msr_1;
      Value ms_1,s_1,ms_2,s,ms,v;
      ValueArray arr_1,arr_2,arr;
    case (ASSIGNMENTS(s,ms,arr),v) "value"
      equation 
        (s == ms) = true " Out of bounds, increase and copy. ";
        msr = int_real(ms);
        msr_1 = msr*.0.4;
        ms_1 = real_int(msr_1);
        s_1 = s + 1;
        ms_2 = ms_1 + ms;
        arr_1 = Util.array_expand(ms_1, arr, 0);
        arr_2 = array_update(arr_1, s + 1, v); then ASSIGNMENTS(s_1,ms_2,arr_2);
    case (ASSIGNMENTS(s,ms,arr),v)
      equation 
        arr_1 = array_update(arr, s + 1, v) " space available, increase size and insert element.";
        s_1 = s + 1; then ASSIGNMENTS(s_1,ms,arr_1);
    case (ASSIGNMENTS(s,ms,arr),v)
      equation 
        print("-assignments_add failed\n"); then fail();
  end matchcontinue;
end assignments_add;

protected function matching_algorithm2 " number of vars
   number of eqns
   current var
  assignments, array of eqn indices
  assignments, array of var indices
   assignments, array of equation indices
   assignments, list of variable indices "
  input DAELow in_daelow1;
  input IncidenceMatrix in_incidencematrix2;
  input IncidenceMatrixT in_incidencematrixt3;
  input Integer in_integer4;
  input Integer in_integer5;
  input Integer in_integer6;
  input Assignments in_assignments7;
  input Assignments in_assignments8;
  output Assignments out_assignments1;
  output Assignments out_assignments2;
  output DAELow out_daelow3;
  output IncidenceMatrix out_incidencematrix4;
  output IncidenceMatrixT out_incidencematrixt5;
algorithm 
  (out_assignments1,out_assignments2,out_daelow3,out_incidencematrix4,out_incidencematrixt5):=
  matchcontinue (in_daelow1,in_incidencematrix2,in_incidencematrixt3,in_integer4,in_integer5,in_integer6,in_assignments7,in_assignments8)
    local
      type ValueList = list<Value>;
      type ValueListArray = ValueList[:];
      Assignments ass1_1,ass2_1,ass1,ass2,ass1_2,ass2_2;
      DAELow dae;
      ValueListArray m,mt;
      Value nv,nf,i,i_1,nv_1,nkv,nf_1,nvd;
      EquationArray eqns;
      String is;
    case (dae,m,mt,nv,nf,i,ass1,ass2) " number of vars  number of eqns  current var assignments, array of eqn indices assignments, array of var indices  assignments, array of equation indices  assignments, list of variable indices "
      equation 
        (nv == i) = true;
        DAEEXT.init_marks(nv, nf);
        (ass1_1,ass2_1) = path_found(m, mt, i, ass1, ass2) " eMark(i)=vMark(i)=false "; then (ass1_1,ass2_1,dae,m,mt);
    case (dae,m,mt,nv,nf,i,ass1,ass2)
      equation 
        i_1 = i + 1;
        DAEEXT.init_marks(nv, nf);
        (ass1_1,ass2_1) = path_found(m, mt, i, ass1, ass2) " eMark(i)=vMark(i)=false ";
        (ass1_2,ass2_2,dae,m,mt) = matching_algorithm2(dae, m, mt, nv, nf, i_1, ass1_1, ass2_1); then (ass1_2,ass2_2,dae,m,mt);
    case (dae,m,mt,nv,nf,i,ass1,ass2)
      equation 
        (dae = DAELOW(VARIABLES(_,_,_,nv_1),VARIABLES(_,_,_,nkv),eqns,_,_,_,_),m,mt) = reduce_index_dummy_der(dae, m, mt, nv, nf, i) " path_found failed, Try index reduction using dummy derivatives.
	 When a constraint exist between states and index reduction is needed
	 the dummy derivative will select one of the states as a dummy state
	 (and the derivative of that state as a dummy derivative).
	 For instance, u1=u2 is a constraint between states. Choose u1 as dummy state
	 and der(u1) as dummy derivative, named der_u1. The differentiated function 
	 then becomes: der_u1 = der(u2).
	 In the dummy derivative method this equation is added and the original equation
	 u1=u2 is kept. This is not the case for the original pantilides algorithm, where
	 the original equation is removed from the system.
	 ";
        nf_1 = equation_size(eqns) " and try again, restarting. This could be optimized later. It should not
	   be necessary to restart the matching, according to Bernard Bachmann. Instead one 
	   could continue the matching as usual. This was tested (2004-11-22) and it does not 
	   work to continue without restarting.
	   For instance the Influenca model \"../testsuite/mofiles/Influenca.mo\" does not work if
	   not restarting.
	   2004-12-29 PA. This was a bug, assignment lists needed to be expanded with the size
	   of the system in order to work. SO: Matching is not needed to be restarted from 
	   scratch.
	   ";
        nvd = nv_1 - nv;
        ass1_1 = assignments_expand(ass1, nvd);
        ass2_1 = assignments_expand(ass2, nvd);
        (ass1_2,ass2_2,dae,m,mt) = matching_algorithm2(dae, m, mt, nv_1, nf_1, i, ass1_1, ass2_1); then (ass1_2,ass2_2,dae,m,mt);
    case (dae,m,mt,nv,nf,i,ass1,ass2)
      equation 
        print("Error, model singular. i=") " When index reduction also fails, the model is structurally 
	 singular. *";
        is = int_string(i);
        print(is);
        print("\n");
        DAEEXT.dump_marked_equations(nf);
        DAEEXT.dump_marked_variables(nv); then fail();
  end matchcontinue;
end matching_algorithm2;

protected function reduce_index_dummy_der " number of vars
   number of eqns
   i "
  input DAELow in_daelow1;
  input IncidenceMatrix in_incidencematrix2;
  input IncidenceMatrixT in_incidencematrixt3;
  input Integer in_integer4;
  input Integer in_integer5;
  input Integer in_integer6;
  output DAELow out_daelow;
  output IncidenceMatrix out_incidencematrix;
  output IncidenceMatrixT out_incidencematrixt;
algorithm 
  (out_daelow,out_incidencematrix,out_incidencematrixt):=
  matchcontinue (in_daelow1,in_incidencematrix2,in_incidencematrixt3,in_integer4,in_integer5,in_integer6)
    local
      type ValueList = list<Value>;
      type KeyList = list<Exp.ComponentRef>;
      type ValueListArray = ValueList[:];
      type StringList = list<String>;
      ValueList eqns,stateindx,deqns,reqns,changedeqns;
      KeyList states;
      DAELow dae;
      ValueListArray m,mt;
      Value nv,nf,stateno,i;
      Exp.ComponentRef state,dummy_der;
      StringList es;
      String es_1;
    case (dae,m,mt,nv,nf,i) " number of vars  number of eqns  i "
      equation 
        eqns = DAEEXT.get_marked_eqns();
        (states,stateindx) = states_in_eqns(eqns, dae, m, mt) "	Util.list_map(eqns,int_string) => es &
	Util.string_delimit_list(es,\", \") => es' &
	print \"marked equations \" & print es' & print \"\n\" &  Collect the states in the equations that are singular, i.e. composing
	 a constraint between states.";
        (dae,m,mt,nv,nf,deqns) = differentiate_eqns(dae, m, mt, nv, nf, eqns);
        (state,stateno) = select_dummy_state(states, stateindx, dae, m, mt);
        (dummy_der,dae) = new_dummy_var(state, dae) "		Exp.print_component_ref_str state => statestr &
	 print \"Choosen dummy state: \" & print statestr & print \"\n\" &";
        reqns = eqns_for_var_with_states(mt, stateno);
        changedeqns = Util.list_union_p(deqns, reqns, int_eq);
        (dae,m,mt) = replace_dummy_der(state, dummy_der, dae, m, mt, changedeqns) " We need to change variables in the differentiated equations and in the 
	  equations having the dummy derivative ";
        dae = make_algebraic(dae, state);
        (m,mt) = update_incidence_matrix(dae, m, mt, changedeqns); then (dae,m,mt);
    case (dae,m,mt,nv,nf,i)
      equation 
        eqns = DAEEXT.get_marked_eqns();
        es = Util.list_map(eqns, int_string);
        es_1 = Util.string_delimit_list(es, ", ");
        ({},_) = states_in_eqns(eqns, dae, m, mt);
        print_equations(eqns, dae); then fail();
    case (_,_,_,_,_,_)
      equation 
        print("-reduce_index_dummy_der failed\n"); then fail();
  end matchcontinue;
end reduce_index_dummy_der;

protected function update_incidence_matrix " list of equations to update"
  input DAELow in_daelow;
  input IncidenceMatrix in_incidencematrix;
  input IncidenceMatrixT in_incidencematrixt;
  input IntegerList in_integerlist;
  output IncidenceMatrix out_incidencematrix;
  output IncidenceMatrixT out_incidencematrixt;
algorithm 
  (out_incidencematrix,out_incidencematrixt):=
  matchcontinue (in_daelow,in_incidencematrix,in_incidencematrixt,in_integerlist)
    local
      type ValueList = list<Value>;
      type ValueListArray = ValueList[:];
      type ValueListList = list<ValueList>;
      ValueListArray m_1,mt_1,m,mt;
      ValueListList changedvars;
      ValueList changedvars_1,eqns;
      DAELow dae;
    case (dae,m,mt,eqns) " list of equations to update"
      equation 
        (m_1,changedvars) = update_incidence_matrix2(dae, m, eqns);
        changedvars_1 = Util.list_flatten(changedvars);
        mt_1 = update_transposed_matrix(changedvars_1, m_1, mt); then (m_1,mt_1);
    case (dae,m,mt,eqns)
      equation 
        print("update_incidence_matrix failed\n"); then fail();
  end matchcontinue;
end update_incidence_matrix;

protected function update_incidence_matrix2 " list of equations to update 
   changed vars "
  input DAELow in_daelow;
  input IncidenceMatrix in_incidencematrix;
  input IntegerList in_integerlist;
  output IncidenceMatrix out_incidencematrix;
  output IntegerListList out_integerlistlist;
protected 
  type IntegerListList = list<IntegerList>;
algorithm 
  (out_incidencematrix,out_integerlistlist):=
  matchcontinue (in_daelow,in_incidencematrix,in_integerlist)
    local
      type ValueList = list<Value>;
      type ValueListArray = ValueList[:];
      type ValueListList = list<ValueList>;
      DAELow dae;
      ValueListArray m,m_1,m_2;
      Value e_1,e;
      Equation eqn;
      ValueList row,changedvars1,eqns;
      ValueListList changedvars2;
      Variables vars,knvars;
      EquationArray daeeqns,daeseqns;
    case (dae,m,{}) then (m,{{}}); 
    case ((dae = DAELOW(vars,knvars,daeeqns,daeseqns,_,_,_)),m,e :: eqns)
      equation 
        e_1 = e - 1;
        eqn = equation_nth(daeeqns, e_1);
        row = incidence_row(vars, eqn);
        m_1 = Util.array_replaceat_with_fill(row, e_1, m, {});
        changedvars1 = vars_in_eqn(m_1, e);
        (m_2,changedvars2) = update_incidence_matrix2(dae, m_1, eqns); then (m_2,changedvars1 :: changedvars2);
    case (_,_,_)
      equation 
        print("-update_incididence_matrix2 failed\n"); then fail();
  end matchcontinue;
end update_incidence_matrix2;

protected function update_transposed_matrix
  input IntegerList in_integerlist;
  input IncidenceMatrix in_incidencematrix;
  input IncidenceMatrixT in_incidencematrixt;
  output IncidenceMatrixT out_incidencematrixt;
algorithm 
  out_incidencematrixt:=
  matchcontinue (in_integerlist,in_incidencematrix,in_incidencematrixt)
    local
      type ValueList = list<Value>;
      type ValueListArray = ValueList[:];
      type ValueListList = list<ValueList>;
      ValueListArray m,mt,mt_1,mt_2;
      ValueListList mlst;
      ValueList row_1,vars;
      Value v_1,v;
    case ({},m,mt) then mt; 
    case (v :: vars,m,mt)
      equation 
        mlst = array_list(m);
        row_1 = transpose_row(mlst, v, 1);
        v_1 = v - 1;
        mt_1 = Util.array_replaceat_with_fill(row_1, v_1, mt, {});
        mt_2 = update_transposed_matrix(vars, m, mt_1); then mt_2;
    case (_,_,_)
      equation 
        print("-update_transposed_matrix failed\n"); then fail();
  end matchcontinue;
end update_transposed_matrix;

protected function make_algebraic " state "
  input DAELow in_daelow;
  input Exp.ComponentRef in_componentref;
  output DAELow out_daelow;
algorithm 
  out_daelow:=
  matchcontinue (in_daelow,in_componentref)
    local
      type Exp_SubscriptList = list<Exp.Subscript>;
      Exp.ComponentRef cr;
      VarKind kind;
      DAE.VarDirection d;
      DAE.Type t;
      Exp_ExpOption b,start;
      Values_ValueOption value;
      Exp_SubscriptList dim;
      Value idx,indx;
      String name;
      Absyn_PathList class_;
      DAE_VariableAttributesOption dae_var_attr;
      Absyn_CommentOption comment;
      Variables vars_1,vars,kv;
      EquationArray e,se,ie;
      Algorithm_AlgorithmArray al;
      EventInfo wc;
    case (DAELOW(vars,kv,e,se,ie,al,wc),cr) " state "
      equation 
        (VAR(cr,kind,d,t,b,value,dim,start,idx,name,class_,dae_var_attr,comment),indx) = get_var(cr, vars);
        vars_1 = add_var(
          VAR(cr,DUMMY_STATE(),d,t,b,value,dim,start,idx,name,class_,
          dae_var_attr,comment), vars); then DAELOW(vars_1,kv,e,se,ie,al,wc);
    case (_,_)
      equation 
        print("-make_algebraic failed\n"); then fail();
  end matchcontinue;
end make_algebraic;

protected function replace_dummy_der " dummy der name 
   equations "
  input Exp.ComponentRef in_componentref1;
  input Exp.ComponentRef in_componentref2;
  input DAELow in_daelow3;
  input IncidenceMatrix in_incidencematrix4;
  input IncidenceMatrixT in_incidencematrixt5;
  input IntegerList in_integerlist6;
  output DAELow out_daelow;
  output IncidenceMatrix out_incidencematrix;
  output IncidenceMatrixT out_incidencematrixt;
algorithm 
  (out_daelow,out_incidencematrix,out_incidencematrixt):=
  matchcontinue (in_componentref1,in_componentref2,in_daelow3,in_incidencematrix4,in_incidencematrixt5,in_integerlist6)
    local
      type ValueList = list<Value>;
      type ValueListArray = ValueList[:];
      Exp.ComponentRef state,dummy,dummyder;
      DAELow dae;
      ValueListArray m,mt;
      Value e_1,e;
      Equation eqn,eqn_1;
      Variables v_1,v,kv;
      EquationArray eqns_1,eqns,seqns,ie;
      Algorithm_AlgorithmArray al;
      EventInfo wc;
      ValueList rest;
    case (state,dummy,dae,m,mt,{}) then (dae,m,mt); 
    case (state,dummyder,DAELOW(v,kv,eqns,seqns,ie,al,wc),m,mt,e :: rest)
      equation 
        e_1 = e - 1;
        eqn = equation_nth(eqns, e_1);
        eqn_1 = replace_dummy_der2(state, dummyder, eqn);
        (eqn_1,v_1) = replace_dummy_der_others(eqn_1, v);
        eqns_1 = equation_setnth(eqns, e_1, eqn_1) "incidence_row(v'',eqn') => row' &
	 Util.list_replaceat(row',e',m) => m' &
	 transpose_matrix(m') => mt' &";
        (dae,m,mt) = replace_dummy_der(state, dummyder, DAELOW(v_1,kv,eqns_1,seqns,ie,al,wc), m, 
          mt, rest); then (dae,m,mt);
    case (_,_,_,_,_,_)
      equation 
        print("-replace_dummy_der failed\n"); then fail();
  end matchcontinue;
end replace_dummy_der;

protected function replace_dummy_der2
  input Exp.ComponentRef in_componentref1;
  input Exp.ComponentRef in_componentref2;
  input Equation in_equation3;
  output Equation out_equation;
algorithm 
  out_equation:=
  matchcontinue (in_componentref1,in_componentref2,in_equation3)
    local
      Exp.Exp dercall,e1_1,e2_1,e1,e2;
      Exp.ComponentRef st,dummyder,cr;
      Equation res;
      Value i;
    case (st,dummyder,EQUATION(e1,e2))
      equation 
        dercall = Exp.CALL(Absyn.IDENT("der"),{Exp.CREF(st,Exp.REAL())},false,true);
        (e1_1,_) = Exp.replace_exp(e1, dercall, Exp.CREF(dummyder,Exp.REAL()));
        (e2_1,_) = Exp.replace_exp(e2, dercall, Exp.CREF(dummyder,Exp.REAL())); then EQUATION(e1_1,e2_1);
    case (st,dummyder,WHEN_EQUATION(WHEN_EQ(i,cr,e1)))
      equation 
        dercall = Exp.CALL(Absyn.IDENT("der"),{Exp.CREF(st,Exp.REAL())},false,true);
        (e1_1,_) = Exp.replace_exp(e1, dercall, Exp.CREF(dummyder,Exp.REAL()));
        res = WHEN_EQUATION(WHEN_EQ(i,cr,e1_1)); then res;
    case (_,_,_)
      equation 
        print("-replace_dummy_der2 failed\n"); then fail();
  end matchcontinue;
end replace_dummy_der2;

protected function replace_dummy_der_others "* function: replace_dummy_der_others
 ** Helper function to reduce_index_dummy_der.
 ** This function replaces
 ** 1. der(der_s)  with der2_s (Where der_s is a dummy state)
 ** 2. der(der(v)) with der2_v (where v is a state)
 ** 3. der(v)  for alg. var v with der_v 
 ** in the 'Equation' given as arguments. To do this it needs the 'Variables' 
 ** also passed as argument to the function to e.g. determine if a variable
 ** is a dummy variable, etc.
 *"
  input Equation in_equation;
  input Variables in_variables;
  output Equation out_equation;
  output Variables out_variables;
algorithm 
  (out_equation,out_variables):=
  matchcontinue (in_equation,in_variables)
    local
      Exp.Exp e1_1,e2_1,e1,e2;
      Variables vars_1,vars_2,vars;
      Value i;
      Exp.ComponentRef cr;
    case (EQUATION(e1,e2),vars)
      equation 
        (e1_1,vars_1) = replace_dummy_der_others_exp(e1, vars);
        (e2_1,vars_2) = replace_dummy_der_others_exp(e2, vars_1); then (EQUATION(e1_1,e2_1),vars_2);
    case (WHEN_EQUATION(WHEN_EQ(i,cr,e2)),vars)
      equation 
        (e2_1,vars_1) = replace_dummy_der_others_exp(e2, vars); then (WHEN_EQUATION(WHEN_EQ(i,cr,e2_1)),vars_1);
    case (_,_)
      equation 
        print("-replace_dummy_der_others failed\n"); then fail();
  end matchcontinue;
end replace_dummy_der_others;

protected function replace_dummy_der_others_exp "* function: replace_dummy_der_others_exp
 ** Helper function for replace_dummy_der_others
 *"
  input Exp.Exp in_exp;
  input Variables in_variables;
  output Exp.Exp out_exp;
  output Variables out_variables;
algorithm 
  (out_exp,out_variables):=
  matchcontinue (in_exp,in_variables)
    local
      type Exp_SubscriptList = list<Exp.Subscript>;
      Exp.Exp e,e1_1,e2_1,e1,e2,e3_1,e3;
      Variables vars,vars1,vars2,vars3,vars_1;
      Exp.Operator op;
      DAE.VarDirection a;
      DAE.Type b;
      Exp_ExpOption c,f;
      Values_ValueOption d;
      Value g;
      String h;
      Absyn_PathList i;
      DAE_VariableAttributesOption dae_var_attr;
      Absyn_CommentOption comment;
      Exp.ComponentRef dummyder,dummyder_1,cr;
    case ((e = Exp.ICONST(_)),vars) then (e,vars); 
    case ((e = Exp.RCONST(_)),vars) then (e,vars); 
    case ((e = Exp.SCONST(_)),vars) then (e,vars); 
    case ((e = Exp.BCONST(_)),vars) then (e,vars); 
    case ((e = Exp.CREF(_,_)),vars) then (e,vars); 
    case (Exp.BINARY(e1,op,e2),vars)
      equation 
        (e1_1,vars1) = replace_dummy_der_others_exp(e1, vars);
        (e2_1,vars2) = replace_dummy_der_others_exp(e2, vars1); then (Exp.BINARY(e1_1,op,e2_1),vars2);
    case (Exp.LBINARY(e1,op,e2),vars)
      equation 
        (e1_1,vars1) = replace_dummy_der_others_exp(e1, vars);
        (e2_1,vars2) = replace_dummy_der_others_exp(e2, vars1); then (Exp.LBINARY(e1_1,op,e2_1),vars2);
    case (Exp.UNARY(op,e1),vars)
      equation 
        (e1_1,vars1) = replace_dummy_der_others_exp(e1, vars); then (Exp.UNARY(op,e1_1),vars1);
    case (Exp.LUNARY(op,e1),vars)
      equation 
        (e1_1,vars1) = replace_dummy_der_others_exp(e1, vars); then (Exp.LUNARY(op,e1_1),vars1);
    case (Exp.RELATION(e1,op,e2),vars)
      equation 
        (e1_1,vars1) = replace_dummy_der_others_exp(e1, vars);
        (e2_1,vars2) = replace_dummy_der_others_exp(e2, vars1); then (Exp.RELATION(e1_1,op,e2_1),vars2);
    case (Exp.IFEXP(e1,e2,e3),vars)
      equation 
        (e1_1,vars1) = replace_dummy_der_others_exp(e1, vars);
        (e2_1,vars2) = replace_dummy_der_others_exp(e2, vars1);
        (e3_1,vars3) = replace_dummy_der_others_exp(e3, vars2); then (Exp.IFEXP(e1_1,e2_1,e3_1),vars3);
    case (Exp.CALL(Absyn.IDENT("der"),{Exp.CALL(Absyn.IDENT("der"),{Exp.CREF(cr,_)},_,_)},_,_),vars)
      local Exp_SubscriptList e;
      equation 
        (VAR(_,STATE(),a,b,c,d,e,f,g,h,i,dae_var_attr,comment),_) = get_var(cr, vars) " der(der(s)) s is state => der_der_s ";
        dummyder = create_dummy_var(cr);
        dummyder_1 = create_dummy_var(dummyder);
        vars_1 = add_var(
          VAR(dummyder_1,DUMMY_DER(),a,b,NONE,NONE,e,NONE,0,"dummyder",i,
          dae_var_attr,comment), vars); then (Exp.CREF(dummyder_1,Exp.REAL()),vars_1);
    case (Exp.CALL(Absyn.IDENT("der"),{Exp.CREF(cr,_)},_,_),vars)
      local Exp_SubscriptList e;
      equation 
        (VAR(_,DUMMY_DER(),a,b,c,d,e,f,g,h,i,dae_var_attr,comment),_) = get_var(cr, vars) " der(der_s)) der_s is dummy var => der_der_s ";
        dummyder = create_dummy_var(cr);
        vars_1 = add_var(
          VAR(dummyder,DUMMY_DER(),a,b,NONE,NONE,e,NONE,0,"dummyder",i,
          dae_var_attr,comment), vars); then (Exp.CREF(dummyder,Exp.REAL()),vars_1);
    case (Exp.CALL(Absyn.IDENT("der"),{Exp.CREF(cr,_)},_,_),vars)
      local Exp_SubscriptList e;
      equation 
        (VAR(_,VARIABLE(),a,b,c,d,e,f,g,h,i,dae_var_attr,comment),_) = get_var(cr, vars) " der(v) v is alg var => der_v ";
        dummyder = create_dummy_var(cr);
        vars_1 = add_var(
          VAR(dummyder,DUMMY_DER(),a,b,NONE,NONE,e,NONE,0,"dummyder",i,
          dae_var_attr,comment), vars); then (Exp.CREF(dummyder,Exp.REAL()),vars_1);
    case (e,vars) then (e,vars); 
  end matchcontinue;
end replace_dummy_der_others_exp;

protected function var_equal
  input Var in_var1;
  input Var in_var2;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_var1,in_var2)
    local
      Boolean res;
      Exp.ComponentRef cr1,cr2;
    case (VAR(cr1,_,_,_,_,_,_,_,_,_,_,_,_),VAR(cr2,_,_,_,_,_,_,_,_,_,_,_,_))
      equation 
        res = Exp.cref_equal(cr1, cr2); then res;
  end matchcontinue;
end var_equal;

protected function new_dummy_var "* function: new_dummy_var
 ** This function creates a new variable named \"der_\"+<varname> and 
 ** adds it to the dae.
 *"
  input Exp.ComponentRef in_componentref;
  input DAELow in_daelow;
  output Exp.ComponentRef out_componentref;
  output DAELow out_daelow;
algorithm 
  (out_componentref,out_daelow):=
  matchcontinue (in_componentref,in_daelow)
    local
      type Exp_SubscriptList = list<Exp.Subscript>;
      VarKind kind;
      DAE.VarDirection dir;
      DAE.Type tp;
      Exp_ExpOption bind,start;
      Values_ValueOption value;
      Exp_SubscriptList dim;
      Value idx;
      String name;
      Absyn_PathList class_;
      DAE_VariableAttributesOption dae_var_attr;
      Absyn_CommentOption comment;
      Exp.ComponentRef dummyvar,var;
      Variables vars_1,vars,kv;
      EquationArray eqns,seqns,ie;
      Algorithm_AlgorithmArray al;
      EventInfo wc;
    case (var,DAELOW(vars,kv,eqns,seqns,ie,al,wc))
      equation 
        (VAR(_,kind,dir,tp,bind,value,dim,start,idx,name,class_,dae_var_attr,comment),_) = get_var(var, vars);
        dummyvar = create_dummy_var(var);
        vars_1 = add_var(
          VAR(dummyvar,DUMMY_DER(),dir,tp,NONE,NONE,dim,NONE,0,
          "dummyvar",class_,dae_var_attr,comment), vars); then (dummyvar,DAELOW(vars_1,kv,eqns,seqns,ie,al,wc));
    case (_,_)
      equation 
        print("-new_dummy_var failed\n"); then fail();
  end matchcontinue;
end new_dummy_var;

protected function create_dummy_var "* function: create_dummy_var
 ** Creates a new variable name by adding \"der_\" before it.
 ** Helper function to new_dummy_var.
 *"
  input Exp.ComponentRef in_componentref;
  output Exp.ComponentRef out_componentref;
algorithm 
  out_componentref:=
  matchcontinue (in_componentref)
    local
      type Exp_SubscriptList = list<Exp.Subscript>;
      String id_1,id;
      Exp_SubscriptList subs;
      Exp.ComponentRef cr_1,cr;
    case (Exp.CREF_IDENT(id,subs))
      equation 
        id_1 = string_append("der_", id); then Exp.CREF_IDENT(id_1,subs);
    case (Exp.CREF_QUAL(id,subs,cr))
      equation 
        cr_1 = create_dummy_var(cr); then Exp.CREF_QUAL(id,subs,cr_1);
  end matchcontinue;
end create_dummy_var;

protected function select_dummy_state " variable numbers "
  input Exp_ComponentRefList in_exp_componentreflist;
  input IntegerList in_integerlist;
  input DAELow in_daelow;
  input IncidenceMatrix in_incidencematrix;
  input IncidenceMatrixT in_incidencematrixt;
  output Exp.ComponentRef out_componentref;
  output Integer out_integer;
protected 
  type Exp_ComponentRefList = list<Exp.ComponentRef>;
algorithm 
  (out_componentref,out_integer):=
  matchcontinue (in_exp_componentreflist,in_integerlist,in_daelow,in_incidencematrix,in_incidencematrixt)
    local
      Exp.ComponentRef s;
      Value sn;
    case (s :: _,sn :: _,_,_,_) then (s,sn); 
    case ({},_,_,_,_)
      equation 
        print("Error, no state to select\n"); then fail();
  end matchcontinue;
end select_dummy_state;

protected function states_in_eqns " name for each state 
   number for each state "
  input IntegerList in_integerlist;
  input DAELow in_daelow;
  input IncidenceMatrix in_incidencematrix;
  input IncidenceMatrixT in_incidencematrixt;
  output Exp_ComponentRefList out_exp_componentreflist;
  output IntegerList out_integerlist;
protected 
  type Exp_ComponentRefList = list<Exp.ComponentRef>;
algorithm 
  (out_exp_componentreflist,out_integerlist):=
  matchcontinue (in_integerlist,in_daelow,in_incidencematrix,in_incidencematrixt)
    local
      type KeyList = list<Exp.ComponentRef>;
      type ValueList = list<Value>;
      type VarList = list<Var>;
      type ValueListArray = ValueList[:];
      KeyList res1,res11,res1_1;
      ValueList res2,vars2,res22,res2_1,rest;
      Value e_1,e;
      Equation eqn;
      VarList varlst;
      Variables vars,kv;
      EquationArray eqns,seqns,ie;
      Algorithm_AlgorithmArray al;
      EventInfo wc;
      ValueListArray m,mt;
    case ({},_,_,_) then ({},{}); 
    case (e :: rest,DAELOW(vars,kv,eqns,seqns,ie,al,wc),m,mt)
      equation 
        (res1,res2) = states_in_eqns(rest, DAELOW(vars,kv,eqns,seqns,ie,al,wc), m, mt);
        e_1 = e - 1;
        eqn = equation_nth(eqns, e_1);
        vars2 = states_in_eqn(eqn, vars);
        varlst = var_list(vars);
        (res11,res22) = states_in_vars(varlst, vars2);
        res1_1 = list_append(res11, res1);
        res2_1 = list_append(res22, res2); then (res1_1,res2_1);
    case (_,_,_,_)
      equation 
        print("-states_in_eqns failed\n"); then fail();
  end matchcontinue;
end states_in_eqns;

protected function states_in_vars " names of the states 
   number for each state "
  input VarList in_varlist;
  input IntegerList in_integerlist;
  output Exp_ComponentRefList out_exp_componentreflist;
  output IntegerList out_integerlist;
protected 
  type VarList = list<Var>;
  type Exp_ComponentRefList = list<Exp.ComponentRef>;
algorithm 
  (out_exp_componentreflist,out_integerlist):=
  matchcontinue (in_varlist,in_integerlist)
    local
      type KeyList = list<Exp.ComponentRef>;
      type ValueList = list<Value>;
      VarList vars;
      Value v_1,v;
      Exp.ComponentRef cr;
      KeyList res1;
      ValueList res2,rest;
    case (vars,{}) then ({},{}); 
    case (vars,v :: rest)
      equation 
        v_1 = v - 1;
        VAR(cr,_,_,_,_,_,_,_,_,_,_,_,_) = list_nth(vars, v_1);
        (res1,res2) = states_in_vars(vars, rest); then (cr :: res1,v :: res2);
    case (vars,v :: rest)
      equation 
        (res1,res2) = states_in_vars(vars, rest); then (res1,res2);
  end matchcontinue;
end states_in_vars;

protected function differentiate_eqns " number of vars
   number of eqns
   equations 
   number of vars
   number of eqns
   differentiated equations "
  input DAELow in_daelow1;
  input IncidenceMatrix in_incidencematrix2;
  input IncidenceMatrixT in_incidencematrixt3;
  input Integer in_integer4;
  input Integer in_integer5;
  input IntegerList in_integerlist6;
  output DAELow out_daelow1;
  output IncidenceMatrix out_incidencematrix2;
  output IncidenceMatrixT out_incidencematrixt3;
  output Integer out_integer4;
  output Integer out_integer5;
  output IntegerList out_integerlist6;
algorithm 
  (out_daelow1,out_incidencematrix2,out_incidencematrixt3,out_integer4,out_integer5,out_integerlist6):=
  matchcontinue (in_daelow1,in_incidencematrix2,in_incidencematrixt3,in_integer4,in_integer5,in_integerlist6)
    local
      type ValueList = list<Value>;
      type ValueListArray = ValueList[:];
      DAELow dae;
      ValueListArray m,mt;
      Value nv,nf,e_1,leneqns,e;
      Equation eqn,eqn_1;
      String str;
      EquationArray eqns_1,eqns,seqns,ie;
      ValueList reqns,es;
      Variables v,kv;
      Algorithm_AlgorithmArray al;
      EventInfo wc;
    case (dae,m,mt,nv,nf,{}) then (dae,m,mt,nv,nf,{}); 
    case ((dae = DAELOW(v,kv,eqns,seqns,ie,al,wc)),m,mt,nv,nf,e :: es)
      equation 
        e_1 = e - 1;
        eqn = equation_nth(eqns, e_1);
        eqn_1 = Derive.differentiate_equation_time(eqn, v);
        Debug.fprint("bltdump", "High index problem, differentiated equation: ");
        str = equation_str(eqn) "	print \"differentiated equation \" &";
        Debug.fprint("bltdump", str) "	print str & print \"\n\" &";
        Debug.fprint("bltdump", " to ") "	print str & print \"\n\" &";
        str = equation_str(eqn_1);
        Debug.fprint("bltdump", str) "	print \" to \" & print str &  print \"\n\" &";
        Debug.fprint("bltdump", "\n") "	print \" to \" & print str &  print \"\n\" &";
        eqns_1 = equation_add(eqns, eqn_1);
        leneqns = equation_size(eqns_1);
        (dae,m,mt,nv,nf,reqns) = differentiate_eqns(DAELOW(v,kv,eqns_1,seqns,ie,al,wc), m, mt, nv, nf, es) " length gives index of new equation "; then (dae,m,mt,nv,nf,leneqns :: e :: reqns);
    case (_,_,_,_,_,_)
      equation 
        print("-differentiate_eqns failed\n"); then fail();
  end matchcontinue;
end differentiate_eqns;

public function equation_add
  input EquationArray in_equationarray;
  input Equation in_equation;
  output EquationArray out_equationarray;
algorithm 
  out_equationarray:=
  matchcontinue (in_equationarray,in_equation)
    local
      Value n_1,n,size,expandsize,newsize;
      EquationOptionArray arr_1,arr,arr_2;
      Equation e;
      Real rsize,rexpandsize;
      EquationArray eqn;
    case (EQUATION_ARRAY(n,size,arr),e)
      equation 
        (n < size) = true " Have space to add array elt. ";
        n_1 = n + 1;
        arr_1 = array_update(arr, n + 1, SOME(e)); then EQUATION_ARRAY(n_1,size,arr_1);
    case (EQUATION_ARRAY(n,size,arr),e)
      equation 
        (n < size) = false " Do NOT Have space to add array elt. Expand array with factor 1.4 ";
        rsize = int_real(size);
        rexpandsize = rsize*.0.4;
        expandsize = real_int(rexpandsize);
        newsize = expandsize + size;
        arr_1 = Util.array_expand(expandsize, arr, NONE);
        n_1 = n + 1;
        arr_2 = array_update(arr_1, n + 1, SOME(e)); then EQUATION_ARRAY(n_1,newsize,arr_2);
    case (eqn,e)
      equation 
        print("-equation_add failed\n"); then fail();
  end matchcontinue;
end equation_add;

public function equation_list "* function: equation_list
 ** Transform the expandable Equation array to a list of Equations.
 *"
  input EquationArray in_equationarray;
  output EquationList out_equationlist;
protected 
  type EquationList = list<Equation>;
algorithm 
  out_equationlist:=
  matchcontinue (in_equationarray)
    local
      EquationOptionArray arr;
      Equation elt;
      Value lastpos,n,size;
      EquationList lst;
    case (EQUATION_ARRAY(0,_,arr)) then {}; 
    case (EQUATION_ARRAY(1,_,arr))
      equation 
        SOME(elt) = array_nth(arr, 0); then {elt};
    case (EQUATION_ARRAY(n,size,arr))
      equation 
        lastpos = n - 1;
        lst = equation_list2(arr, 0, lastpos); then lst;
    case (_)
      equation 
        print("equation_list failed\n"); then fail();
  end matchcontinue;
end equation_list;

public function list_equation "* function: list_equation
 ** Transform the a list of Equations into an expandable Equation array.
 *"
  input EquationList lst;
  output EquationArray out_equationarray;
protected 
  type EquationList = list<Equation>;
  type EquationOptionList = list<EquationOption>;
  Value len,size;
  Real rlen,rlen_1;
  EquationOptionArray optarr,eqnarr,newarr;
  EquationOptionList eqn_optlst;
algorithm 
  len := list_length(lst);
  rlen := int_real(len);
  rlen_1 := rlen*.1.4;
  size := real_int(rlen_1);
  optarr := fill(NONE, size);
  eqn_optlst := Util.list_map(lst, Util.make_option);
  eqnarr := list_array(eqn_optlst);
  newarr := Util.array_copy(eqnarr, optarr);
  out_equationarray := EQUATION_ARRAY(len,size,newarr);
end list_equation;

protected function equation_list2 "pos
  lastpos"
  input EquationOptionArray in_equationoptionarray1;
  input Integer in_integer2;
  input Integer in_integer3;
  output EquationList out_equationlist;
protected 
  type EquationList = list<Equation>;
algorithm 
  out_equationlist:=
  matchcontinue (in_equationoptionarray1,in_integer2,in_integer3)
    local
      Equation e;
      EquationOptionArray arr;
      Value pos,lastpos,pos_1;
      EquationList res;
    case (arr,pos,lastpos) "pos lastpos"
      equation 
        (pos == lastpos) = true;
        SOME(e) = array_nth(arr, pos); then {e};
    case (arr,pos,lastpos)
      equation 
        pos_1 = pos + 1;
        SOME(e) = array_nth(arr, pos);
        res = equation_list2(arr, pos_1, lastpos); then e :: res;
  end matchcontinue;
end equation_list2;

protected function equation_size "* function: equation_size
 ** Returns the number of equations
 *"
  input EquationArray in_equationarray;
  output Integer out_integer;
algorithm 
  out_integer:=
  matchcontinue (in_equationarray)
    local Value n;
    case (EQUATION_ARRAY(n,_,_)) then n; 
  end matchcontinue;
end equation_size;

protected function variable_size "* function: variable_size
 ** Returns the number of variables
 *"
  input Variables in_variables;
  output Integer out_integer;
algorithm 
  out_integer:=
  matchcontinue (in_variables)
    local Value n;
    case (VARIABLES(_,_,_,n)) then n; 
  end matchcontinue;
end variable_size;

public function equation_nth "n"
  input EquationArray in_equationarray;
  input Integer in_integer;
  output Equation out_equation;
algorithm 
  out_equation:=
  matchcontinue (in_equationarray,in_integer)
    local
      Equation e;
      Value n,pos;
      EquationOptionArray arr;
    case (EQUATION_ARRAY(n,_,arr),pos) "n"
      equation 
        (pos < n) = true;
        SOME(e) = array_nth(arr, pos); then e;
    case (_,_)
      equation 
        print("equation_nth failed\n"); then fail();
  end matchcontinue;
end equation_nth;

public function equation_setnth "* function: equation_setnth
 ** Sets the nth array element of an EquationArray.
 *"
  input EquationArray in_equationarray;
  input Integer in_integer;
  input Equation in_equation;
  output EquationArray out_equationarray;
algorithm 
  out_equationarray:=
  matchcontinue (in_equationarray,in_integer,in_equation)
    local
      EquationOptionArray arr_1,arr;
      Value n,size,pos;
      Equation eqn;
    case (EQUATION_ARRAY(n,size,arr),pos,eqn)
      equation 
        arr_1 = array_update(arr, pos + 1, SOME(eqn)); then EQUATION_ARRAY(n,size,arr_1);
  end matchcontinue;
end equation_setnth;

protected function add_marked_vars " number of vars
   number of eqns
   marked vars 
   number of vars
   number of eqns"
  input DAELow in_daelow1;
  input IncidenceMatrix in_incidencematrix2;
  input IncidenceMatrixT in_incidencematrixt3;
  input Integer in_integer4;
  input Integer in_integer5;
  input IntegerList in_integerlist6;
  output DAELow out_daelow1;
  output IncidenceMatrix out_incidencematrix2;
  output IncidenceMatrixT out_incidencematrixt3;
  output Integer out_integer4;
  output Integer out_integer5;
algorithm 
  (out_daelow1,out_incidencematrix2,out_incidencematrixt3,out_integer4,out_integer5):=
  matchcontinue (in_daelow1,in_incidencematrix2,in_incidencematrixt3,in_integer4,in_integer5,in_integerlist6)
    local
      type ValueList = list<Value>;
      type ValueListArray = ValueList[:];
      DAELow dae;
      ValueListArray m,mt,nt;
      Value nv,nf,nv_1,v;
      ValueList vs;
    case (dae,m,mt,nv,nf,{}) then (dae,m,mt,nv,nf); 
    case (dae,m,nt,nv,nf,v :: vs)
      equation 
        nv_1 = nv + 1 " TODO remove variable from dae and m,mt and add der{variable} instead";
        DAEEXT.set_v(v, nv_1);
        (dae,m,mt,nv,nf) = add_marked_vars(dae, m, nt, nv_1, nf, vs); then (dae,m,mt,nv,nf);
  end matchcontinue;
end add_marked_vars;

protected function path_found
  input IncidenceMatrix in_incidencematrix1;
  input IncidenceMatrixT in_incidencematrixt2;
  input Integer in_integer3;
  input Assignments in_assignments4;
  input Assignments in_assignments5;
  output Assignments out_assignments1;
  output Assignments out_assignments2;
algorithm 
  (out_assignments1,out_assignments2):=
  matchcontinue (in_incidencematrix1,in_incidencematrixt2,in_integer3,in_assignments4,in_assignments5)
    local
      type ValueList = list<Value>;
      type ValueListArray = ValueList[:];
      Assignments ass1_1,ass2_1,ass1,ass2;
      ValueListArray m,mt;
      Value i;
    case (m,mt,i,ass1,ass2) " equation "
      equation 
        DAEEXT.e_mark(i) "Sideeffect";
        (ass1_1,ass2_1) = assign_one_in_eqn(m, mt, i, ass1, ass2); then (ass1_1,ass2_1);
    case (m,mt,i,ass1,ass2)
      equation 
        (ass1_1,ass2_1) = forall_unmarked_vars_in_eqn(m, mt, i, ass1, ass2); then (ass1_1,ass2_1);
  end matchcontinue;
end path_found;

protected function assign_one_in_eqn
  input IncidenceMatrix in_incidencematrix1;
  input IncidenceMatrixT in_incidencematrixt2;
  input Integer in_integer3;
  input Assignments in_assignments4;
  input Assignments in_assignments5;
  output Assignments out_assignments1;
  output Assignments out_assignments2;
algorithm 
  (out_assignments1,out_assignments2):=
  matchcontinue (in_incidencematrix1,in_incidencematrixt2,in_integer3,in_assignments4,in_assignments5)
    local
      type ValueList = list<Value>;
      type ValueListArray = ValueList[:];
      ValueList vars;
      Assignments ass1_1,ass2_1,ass1,ass2;
      ValueListArray m,mt;
      Value i;
    case (m,mt,i,ass1,ass2)
      equation 
        vars = vars_in_eqn(m, i);
        (ass1_1,ass2_1) = assign_first_unassigned(i, vars, ass1, ass2); then (ass1_1,ass2_1);
  end matchcontinue;
end assign_one_in_eqn;

protected function states_in_eqn "* function: states_in_eqn
 ** Helper function to states_in_eqns
 *"
  input Equation eqn;
  input Variables vars;
  output IntegerList res;
protected 
  Variables vars_1;
algorithm 
  vars_1 := states_as_algebraic_vars(vars);
  res := incidence_row(vars_1, eqn);
end states_in_eqn;

protected function states_as_algebraic_vars "* function: states_as_algebraic_vars
 ** Return the subset of variables consisting of all states, but changed varkind to variable.
 *"
  input Variables vars;
  output Variables v1_1;
protected 
  type VarList = list<Var>;
  VarList varlst,varlst_1;
  Variables v1,v1_1;
algorithm 
  varlst := var_list(vars) " Creates a new set of Variables from a Var list ";
  varlst_1 := states_as_algebraic_vars2(varlst);
  v1 := empty_vars();
  v1_1 := add_vars(varlst_1, v1);
end states_as_algebraic_vars;

protected function states_as_algebraic_vars2 "* function: states_as_algebraic_vars2
 ** helper function to states_as_algebraic_vars
*"
  input VarList in_varlist;
  output VarList out_varlist;
protected 
  type VarList = list<Var>;
algorithm 
  out_varlist:=
  matchcontinue (in_varlist)
    local
      type Exp_SubscriptList = list<Exp.Subscript>;
      VarList res,vs;
      Exp.ComponentRef cr;
      DAE.VarDirection a;
      DAE.Type b;
      Exp_ExpOption c,f;
      Values_ValueOption d;
      Exp_SubscriptList e;
      Value g;
      String h;
      Absyn_PathList i;
      DAE_VariableAttributesOption dae_var_attr;
      Absyn_CommentOption comment;
    case {} then {}; 
    case (VAR(cr,STATE(),a,b,c,d,e,f,g,h,i,dae_var_attr,comment) :: vs)
      equation 
        res = states_as_algebraic_vars2(vs) " states treated as algebraic variables "; then VAR(cr,VARIABLE(),a,b,c,d,e,f,g,h,i,dae_var_attr,comment) :: res;
    case (VAR(cr,_,a,b,c,d,e,f,g,h,i,dae_var_attr,comment) :: vs)
      equation 
        res = states_as_algebraic_vars2(vs) " other variables treated as known "; then VAR(cr,CONST(),a,b,c,d,e,f,g,h,i,dae_var_attr,comment) :: res;
    case (_ :: vs)
      equation 
        res = states_as_algebraic_vars2(vs); then res;
  end matchcontinue;
end states_as_algebraic_vars2;

public function vars_in_eqn " variables "
  input IncidenceMatrix in_incidencematrix;
  input Integer in_integer;
  output IntegerList out_integerlist;
algorithm 
  out_integerlist:=
  matchcontinue (in_incidencematrix,in_integer)
    local
      type ValueList = list<Value>;
      type ValueListArray = ValueList[:];
      Value n_1,n,indx;
      ValueList res,res_1;
      ValueListArray m;
      String s;
    case (m,n) " equation  variables "
      equation 
        n_1 = n - 1;
        res = m[n_1 + 1];
        res_1 = remove_negative(res); then res_1;
    case (_,indx)
      equation 
        print("vars_in_eqn failed, indx=");
        s = int_string(indx);
        print(s);
        print("\n"); then fail();
  end matchcontinue;
end vars_in_eqn;

protected function remove_negative "* function: remove_negative
 ** Removes all negative integers.
*"
  input IntegerList lst;
  output IntegerList lst_1;
protected 
  type ValueList = list<Value>;
  ValueList lst_1;
algorithm 
  lst_1 := Util.list_select(lst, Util.int_positive);
end remove_negative;

protected function eqns_for_var " equations "
  input IncidenceMatrixT in_incidencematrixt;
  input Integer in_integer;
  output IntegerList out_integerlist;
algorithm 
  out_integerlist:=
  matchcontinue (in_incidencematrixt,in_integer)
    local
      type ValueList = list<Value>;
      type ValueListArray = ValueList[:];
      Value n_1,n,indx;
      ValueList res,res_1;
      ValueListArray mt;
      String s;
    case (mt,n) " variable   equations "
      equation 
        n_1 = n - 1;
        res = mt[n_1 + 1];
        res_1 = remove_negative(res); then res_1;
    case (_,indx)
      equation 
        print("eqns_for_var failed, indx=");
        s = int_string(indx);
        print(s);
        print("\n"); then fail();
  end matchcontinue;
end eqns_for_var;

protected function eqns_for_var_with_states " equations "
  input IncidenceMatrixT in_incidencematrixt;
  input Integer in_integer;
  output IntegerList out_integerlist;
algorithm 
  out_integerlist:=
  matchcontinue (in_incidencematrixt,in_integer)
    local
      type ValueList = list<Value>;
      type ValueListArray = ValueList[:];
      Value n_1,n,indx;
      ValueList res,res_1;
      ValueListArray mt;
      String s;
    case (mt,n) " variable   equations "
      equation 
        n_1 = n - 1;
        res = mt[n_1 + 1];
        res_1 = Util.list_map(res, int_abs); then res_1;
    case (_,indx)
      equation 
        print("eqns_for_var failed, indx=");
        s = int_string(indx);
        print(s);
        print("\n"); then fail();
  end matchcontinue;
end eqns_for_var_with_states;

protected function assign_first_unassigned " variables
   ass1 
   ass2 
   ass1 
   ass2 "
  input Integer in_integer1;
  input IntegerList in_integerlist2;
  input Assignments in_assignments3;
  input Assignments in_assignments4;
  output Assignments out_assignments1;
  output Assignments out_assignments2;
algorithm 
  (out_assignments1,out_assignments2):=
  matchcontinue (in_integer1,in_integerlist2,in_assignments3,in_assignments4)
    local
      type ValueList = list<Value>;
      Assignments ass1_1,ass2_1,ass1,ass2;
      Value i,v;
      ValueList vs;
    case (i,v :: vs,ass1,ass2) " equation  variables  ass1   ass2   ass1   ass2 "
      equation 
        0 = get_assigned(v, ass1, ass2);
        (ass1_1,ass2_1) = assign(v, i, ass1, ass2); then (ass1_1,ass2_1);
    case (i,v :: vs,ass1,ass2)
      equation 
        (ass1_1,ass2_1) = assign_first_unassigned(i, vs, ass1, ass2); then (ass1_1,ass2_1);
  end matchcontinue;
end assign_first_unassigned;

protected function get_assigned " ass1 
   ass2 
   equation "
  input Integer in_integer1;
  input Assignments in_assignments2;
  input Assignments in_assignments3;
  output Integer out_integer;
algorithm 
  out_integer:=
  matchcontinue (in_integer1,in_assignments2,in_assignments3)
    local
      type ValueArray = Value[:];
      Value v_1,v;
      ValueArray m;
    case (v,ASSIGNMENTS(_,_,m),_) " variable   ass1   ass2   equation "
      equation 
        v_1 = v - 1; then m[v_1 + 1];
  end matchcontinue;
end get_assigned;

protected function assign " equation 
   ass1 
   ass2 
   updated ass1 
   updated ass2 "
  input Integer in_integer1;
  input Integer in_integer2;
  input Assignments in_assignments3;
  input Assignments in_assignments4;
  output Assignments out_assignments1;
  output Assignments out_assignments2;
algorithm 
  (out_assignments1,out_assignments2):=
  matchcontinue (in_integer1,in_integer2,in_assignments3,in_assignments4)
    local
      Value v_1,e_1,v,e;
      Assignments ass1_1,ass2_1,ass1,ass2;
    case (v,e,ass1,ass2) " variable   equation   ass1   ass2   updated ass1   updated ass2 "
      equation 
        v_1 = v - 1 "print \"assign \" & int_string v => vs &
	int_string e => es & print vs & print \" to eqn \" &
	print es & print \"\n\" &";
        e_1 = e - 1;
        ass1_1 = assignments_setnth(ass1, v_1, e);
        ass2_1 = assignments_setnth(ass2, e_1, v); then (ass1_1,ass2_1);
  end matchcontinue;
end assign;

protected function forall_unmarked_vars_in_eqn " ass1
  ass2"
  input IncidenceMatrix in_incidencematrix1;
  input IncidenceMatrixT in_incidencematrixt2;
  input Integer in_integer3;
  input Assignments in_assignments4;
  input Assignments in_assignments5;
  output Assignments out_assignments1;
  output Assignments out_assignments2;
algorithm 
  (out_assignments1,out_assignments2):=
  matchcontinue (in_incidencematrix1,in_incidencematrixt2,in_integer3,in_assignments4,in_assignments5)
    local
      type ValueList = list<Value>;
      type ValueListArray = ValueList[:];
      ValueList vars,vars_1;
      Assignments ass1_1,ass2_1,ass1,ass2;
      ValueListArray m,mt;
      Value i;
    case (m,mt,i,ass1,ass2) " ass1 ass2"
      equation 
        vars = vars_in_eqn(m, i);
        vars_1 = Util.list_matching(vars, is_not_v_marked);
        (ass1_1,ass2_1) = forall_unmarked_vars_in_eqn_body(m, mt, i, vars_1, ass1, ass2); then (ass1_1,ass2_1);
  end matchcontinue;
end forall_unmarked_vars_in_eqn;

protected function is_not_v_marked "* function: is_not_v_marked
 ** This function succeds for variables that are not marked.
*"
  input Integer i;
algorithm 
  false := DAEEXT.get_v_mark(i);
end is_not_v_marked;

protected function forall_unmarked_vars_in_eqn_body " var list"
  input IncidenceMatrix in_incidencematrix1;
  input IncidenceMatrixT in_incidencematrixt2;
  input Integer in_integer3;
  input IntegerList in_integerlist4;
  input Assignments in_assignments5;
  input Assignments in_assignments6;
  output Assignments out_assignments1;
  output Assignments out_assignments2;
algorithm 
  (out_assignments1,out_assignments2):=
  matchcontinue (in_incidencematrix1,in_incidencematrixt2,in_integer3,in_integerlist4,in_assignments5,in_assignments6)
    local
      type ValueList = list<Value>;
      type ValueListArray = ValueList[:];
      Value assarg,i,v;
      Assignments ass1_1,ass2_1,ass1_2,ass2_2,ass1,ass2;
      ValueListArray m,mt;
      ValueList vars,vs;
    case (m,mt,i,(vars = v :: vs),ass1,ass2) " var list"
      equation 
        DAEEXT.v_mark(v);
        assarg = get_assigned(v, ass1, ass2);
        (ass1_1,ass2_1) = path_found(m, mt, assarg, ass1, ass2);
        (ass1_2,ass2_2) = assign(v, i, ass1_1, ass2_1); then (ass1_2,ass2_2);
    case (m,mt,i,(vars = v :: vs),ass1,ass2)
      equation 
        DAEEXT.v_mark(v);
        (ass1_1,ass2_1) = forall_unmarked_vars_in_eqn_body(m, mt, i, vs, ass1, ass2); then (ass1_1,ass2_1);
  end matchcontinue;
end forall_unmarked_vars_in_eqn_body;

public function strong_components " list of components "
  input IncidenceMatrix in_incidencematrix1;
  input IncidenceMatrixT in_incidencematrixt2;
  input IntegerVector in_integervector3;
  input IntegerVector in_integervector4;
  output IntegerListList out_integerlistlist;
protected 
  type IntegerVector = Integer[:];
  type IntegerListList = list<IntegerList>;
algorithm 
  out_integerlistlist:=
  matchcontinue (in_incidencematrix1,in_incidencematrixt2,in_integervector3,in_integervector4)
    local
      type ValueList = list<Value>;
      type ValueListList = list<ValueList>;
      type ValueListArray = ValueList[:];
      type ValueVector = Value[:];
      Value n,i;
      ValueList stack;
      ValueListList comps;
      ValueListArray m,mt;
      ValueVector ass1,ass2;
    case (m,mt,ass1,ass2) " list of components "
      equation 
        n = array_length(m);
        DAEEXT.init_lowlink(n);
        DAEEXT.init_number(n);
        (i,stack,comps) = strong_connect_main(m, mt, ass1, ass2, n, 0, 1, {}, {}); then comps;
    case (_,_,_,_)
      equation 
        print("strong_components failed\n"); then fail();
  end matchcontinue;
end strong_components;

protected function strong_connect_main " Assignment 
   Assignment 
   n - number of equations 
   i 
   w 
   stack 
   components 
   i 
   stack 
   components "
  input IncidenceMatrix in_incidencematrix1;
  input IncidenceMatrixT in_incidencematrixt2;
  input IntegerVector in_integervector3;
  input IntegerVector in_integervector4;
  input Integer in_integer5;
  input Integer in_integer6;
  input Integer in_integer7;
  input IntegerList in_integerlist8;
  input IntegerListList in_integerlistlist9;
  output Integer out_integer;
  output IntegerList out_integerlist;
  output IntegerListList out_integerlistlist;
protected 
  type IntegerVector = Integer[:];
  type IntegerListList = list<IntegerList>;
algorithm 
  (out_integer,out_integerlist,out_integerlistlist):=
  matchcontinue (in_incidencematrix1,in_incidencematrixt2,in_integervector3,in_integervector4,in_integer5,in_integer6,in_integer7,in_integerlist8,in_integerlistlist9)
    local
      type ValueList = list<Value>;
      type ValueListArray = ValueList[:];
      type ValueVector = Value[:];
      type ValueListList = list<ValueList>;
      ValueListArray m,mt;
      ValueVector a1,a2;
      Value n,i,w,w_1,num;
      ValueList stack,stack_1,stack_2;
      ValueListList comp,comps;
    case (m,mt,a1,a2,n,i,w,stack,comp) " Assignment   Assignment   n - number of equations   i   w   stack   components   i   stack   components "
      equation 
        (w > n) = true; then (i,stack,comp);
    case (m,mt,a1,a2,n,i,w,stack,comps)
      equation 
        0 = DAEEXT.get_number(w);
        (i,stack_1,comps) = strong_connect(m, mt, a1, a2, i, w, stack, comps);
        w_1 = w + 1;
        (i,stack_2,comps) = strong_connect_main(m, mt, a1, a2, n, i, w_1, stack_1, comps); then (i,stack_2,comps);
    case (m,mt,a1,a2,n,i,w,stack,comps)
      equation 
        num = DAEEXT.get_number(w);
        (num == 0) = false;
        w_1 = w + 1;
        (i,stack_1,comps) = strong_connect_main(m, mt, a1, a2, n, i, w_1, stack, comps); then (i,stack_1,comps);
  end matchcontinue;
end strong_connect_main;

protected function strong_connect " i 
   v 
   stack 
   components 
   i 
   stack 
   int list list"
  input IncidenceMatrix in_incidencematrix1;
  input IncidenceMatrixT in_incidencematrixt2;
  input IntegerVector in_integervector3;
  input IntegerVector in_integervector4;
  input Integer in_integer5;
  input Integer in_integer6;
  input IntegerList in_integerlist7;
  input IntegerListList in_integerlistlist8;
  output Integer out_integer;
  output IntegerList out_integerlist;
  output IntegerListList out_integerlistlist;
protected 
  type IntegerVector = Integer[:];
  type IntegerListList = list<IntegerList>;
algorithm 
  (out_integer,out_integerlist,out_integerlistlist):=
  matchcontinue (in_incidencematrix1,in_incidencematrixt2,in_integervector3,in_integervector4,in_integer5,in_integer6,in_integerlist7,in_integerlistlist8)
    local
      type ValueList = list<Value>;
      type ValueListList = list<ValueList>;
      type ValueListArray = ValueList[:];
      type ValueVector = Value[:];
      Value i_1,i,v;
      ValueList stack_1,eqns,stack_2,stack_3,comp,stack;
      ValueListList comps_1,comps_2,comps;
      ValueListArray m,mt;
      ValueVector a1,a2;
    case (m,mt,a1,a2,i,v,stack,comps) " i   v   stack   components   i   stack   int list list"
      equation 
        i_1 = i + 1;
        DAEEXT.set_number(v, i_1) "	set_number(number,v,i') => number' &
	set_lowlink(lowlink,v,i') => lowlink' &";
        DAEEXT.set_lowlink(v, i_1);
        stack_1 = v :: stack;
        eqns = reachable_nodes(v, m, mt, a1, a2);
        (i_1,stack_2,comps_1) = iterate_reachable_nodes(eqns, m, mt, a1, a2, i_1, v, stack_1, comps);
        (i_1,stack_3,comp) = check_root(m, mt, a1, a2, i_1, v, stack_2);
        comps_2 = cons_if_nonempty(comp, comps_1); then (i_1,stack_3,comps_2);
    case (_,_,_,_,_,_,_,_)
      equation 
        print("-strong_connect failed\n"); then fail();
  end matchcontinue;
end strong_connect;

protected function cons_if_nonempty "* function: cons_if_nonempty
 ** Small helper function to avoid empty sublists.
 ** Consider moving to Util?
 *"
  input IntegerList in_integerlist;
  input IntegerListList in_integerlistlist;
  output IntegerListList out_integerlistlist;
protected 
  type IntegerListList = list<IntegerList>;
algorithm 
  out_integerlistlist:=
  matchcontinue (in_integerlist,in_integerlistlist)
    local
      type ValueList = list<Value>;
      type ValueListList = list<ValueList>;
      ValueListList lst;
      ValueList e;
    case ({},lst) then lst; 
    case (e,lst) then e :: lst; 
  end matchcontinue;
end cons_if_nonempty;

protected function reachable_nodes
  input Integer in_integer1;
  input IncidenceMatrix in_incidencematrix2;
  input IncidenceMatrixT in_incidencematrixt3;
  input IntegerVector in_integervector4;
  input IntegerVector in_integervector5;
  output IntegerList out_integerlist;
protected 
  type IntegerVector = Integer[:];
algorithm 
  out_integerlist:=
  matchcontinue (in_integer1,in_incidencematrix2,in_incidencematrixt3,in_integervector4,in_integervector5)
    local
      type ValueList = list<Value>;
      type ValueListArray = ValueList[:];
      type ValueVector = Value[:];
      Value eqn_1,var,var_1,pos,eqn;
      ValueList reachable,reachable_1,reachable_2;
      ValueListArray m,mt;
      ValueVector a1,a2;
      String eqnstr;
    case (eqn,m,mt,a1,a2)
      equation 
        eqn_1 = eqn - 1;
        var = a2[eqn_1 + 1];
        var_1 = var - 1;
        reachable = mt[var_1 + 1] " Got the variable that is solved in the equation ";
        reachable_1 = remove_negative(reachable) " in which other equations is this variable present ?";
        pos = Util.list_position(eqn, reachable_1) " .. except this one ";
        reachable_2 = list_delete(reachable_1, pos); then reachable_2;
    case (eqn,_,_,_,_)
      equation 
        print("-reachable_nodes failed, eqn: ");
        eqnstr = int_string(eqn);
        print(eqnstr);
        print("\n"); then fail();
  end matchcontinue;
end reachable_nodes;

protected function iterate_reachable_nodes " i 
   v 
   stack 
   components 
   i 
   stack 
   components "
  input IntegerList in_integerlist1;
  input IncidenceMatrix in_incidencematrix2;
  input IncidenceMatrixT in_incidencematrixt3;
  input IntegerVector in_integervector4;
  input IntegerVector in_integervector5;
  input Integer in_integer6;
  input Integer in_integer7;
  input IntegerList in_integerlist8;
  input IntegerListList in_integerlistlist9;
  output Integer out_integer;
  output IntegerList out_integerlist;
  output IntegerListList out_integerlistlist;
protected 
  type IntegerVector = Integer[:];
  type IntegerListList = list<IntegerList>;
algorithm 
  (out_integer,out_integerlist,out_integerlistlist):=
  matchcontinue (in_integerlist1,in_incidencematrix2,in_incidencematrixt3,in_integervector4,in_integervector5,in_integer6,in_integer7,in_integerlist8,in_integerlistlist9)
    local
      type ValueList = list<Value>;
      type ValueListList = list<ValueList>;
      type ValueListArray = ValueList[:];
      type ValueVector = Value[:];
      Value i,lv,lw,minv,w,v,nw,nv,lowlinkw;
      ValueList stack,ws;
      ValueListList comps_1,comps_2,comps;
      ValueListArray m,mt;
      ValueVector a1,a2;
    case (w :: ws,m,mt,a1,a2,i,v,stack,comps) " i   v   stack   components   i   stack   components "
      equation 
        0 = DAEEXT.get_number(w);
        (i,stack,comps_1) = strong_connect(m, mt, a1, a2, i, w, stack, comps);
        lv = DAEEXT.get_lowlink(v);
        lw = DAEEXT.get_lowlink(w);
        minv = int_min(lv, lw);
        DAEEXT.set_lowlink(v, minv);
        (i,stack,comps_2) = iterate_reachable_nodes(ws, m, mt, a1, a2, i, v, stack, comps_1); then (i,stack,comps_2);
    case (w :: ws,m,mt,a1,a2,i,v,stack,comps)
      equation 
        nw = DAEEXT.get_number(w);
        nv = DAEEXT.get_number(v);
        (nw < nv) = true;
        true = list_member(w, stack);
        lowlinkw = DAEEXT.get_lowlink(w);
        minv = int_min(nw, lowlinkw);
        DAEEXT.set_lowlink(v, minv);
        (i,stack,comps_1) = iterate_reachable_nodes(ws, m, mt, a1, a2, i, v, stack, comps); then (i,stack,comps_1);
    case (w :: ws,m,mt,a1,a2,i,v,stack,comps)
      equation 
        (i,stack,comps_1) = iterate_reachable_nodes(ws, m, mt, a1, a2, i, v, stack, comps); then (i,stack,comps_1);
    case ({},m,mt,a1,a2,i,v,stack,comps) then (i,stack,comps); 
  end matchcontinue;
end iterate_reachable_nodes;

protected function dump_list "* function: dump_list
 ** Helper function to dump.
 *"
  input IntegerList l;
  input String str;
protected 
  type StringList = list<String>;
  StringList s;
  String sl;
algorithm 
  s := Util.list_map(l, int_string);
  sl := Util.string_delimit_list(s, ", ");
  print(str);
  print(sl);
  print("\n");
end dump_list;

protected function check_root " i 
   v 
   stack 
   i 
   stack 
   comps "
  input IncidenceMatrix in_incidencematrix1;
  input IncidenceMatrixT in_incidencematrixt2;
  input IntegerVector in_integervector3;
  input IntegerVector in_integervector4;
  input Integer in_integer5;
  input Integer in_integer6;
  input IntegerList in_integerlist7;
  output Integer out_integer1;
  output IntegerList out_integerlist2;
  output IntegerList out_integerlist3;
protected 
  type IntegerVector = Integer[:];
algorithm 
  (out_integer1,out_integerlist2,out_integerlist3):=
  matchcontinue (in_incidencematrix1,in_incidencematrixt2,in_integervector3,in_integervector4,in_integer5,in_integer6,in_integerlist7)
    local
      type ValueList = list<Value>;
      type ValueListArray = ValueList[:];
      type ValueVector = Value[:];
      Value lv,nv,i,v;
      ValueList stack_1,comps,stack;
      ValueListArray m,mt;
      ValueVector a1,a2;
    case (m,mt,a1,a2,i,v,stack) " i   v   stack   i   stack   comps "
      equation 
        lv = DAEEXT.get_lowlink(v);
        nv = DAEEXT.get_number(v);
        (lv == nv) = true;
        (i,stack_1,comps) = check_stack(m, mt, a1, a2, i, v, stack, {}); then (i,stack_1,comps);
    case (m,mt,a1,a2,i,v,stack) then (i,stack,{}); 
  end matchcontinue;
end check_root;

protected function check_stack " i 
   v 
   stack 
   component list
   i 
   stack 
   comps "
  input IncidenceMatrix in_incidencematrix1;
  input IncidenceMatrixT in_incidencematrixt2;
  input IntegerVector in_integervector3;
  input IntegerVector in_integervector4;
  input Integer in_integer5;
  input Integer in_integer6;
  input IntegerList in_integerlist7;
  input IntegerList in_integerlist8;
  output Integer out_integer1;
  output IntegerList out_integerlist2;
  output IntegerList out_integerlist3;
protected 
  type IntegerVector = Integer[:];
algorithm 
  (out_integer1,out_integerlist2,out_integerlist3):=
  matchcontinue (in_incidencematrix1,in_incidencematrixt2,in_integervector3,in_integervector4,in_integer5,in_integer6,in_integerlist7,in_integerlist8)
    local
      type ValueList = list<Value>;
      type ValueListArray = ValueList[:];
      type ValueVector = Value[:];
      Value topn,vn,i,v,top;
      ValueList stack_1,comp_1,rest,comp,stack;
      ValueListArray m,mt;
      ValueVector a1,a2;
    case (m,mt,a1,a2,i,v,top :: rest,comp) " i   v   stack   component list  i   stack   comps "
      equation 
        topn = DAEEXT.get_number(top);
        vn = DAEEXT.get_number(v);
        (topn >= vn) = true;
        (i,stack_1,comp_1) = check_stack(m, mt, a1, a2, i, v, rest, comp); then (i,stack_1,top :: comp_1);
    case (m,mt,a1,a2,i,v,stack,comp) then (i,stack,comp); 
  end matchcontinue;
end check_stack;

public function dump_components "* function: dump_components
 ** Prints the blocks of the BLT sorting on stdout.
 *"
  input IntegerListList l;
protected 
  type IntegerListList = list<IntegerList>;
algorithm 
  print("Blocks\n");
  print("=======\n");
  dump_components2(l, 1);
end dump_components;

protected function dump_components2 "* function: dump_components2
 ** Helper function to dump_components.
 *"
  input IntegerListList in_integerlistlist;
  input Integer in_integer;
protected 
  type IntegerListList = list<IntegerList>;
algorithm 
  _:=
  matchcontinue (in_integerlistlist,in_integer)
    local
      type StringList = list<String>;
      type ValueList = list<Value>;
      type ValueListList = list<ValueList>;
      Value ni,i_1,i;
      StringList ls;
      String s;
      ValueList l;
      ValueListList lst;
    case ({},_) then (); 
    case (l :: lst,i)
      equation 
        ni = DAEEXT.get_lowlink(i);
        print("{");
        ls = Util.list_map(l, int_string);
        s = Util.string_delimit_list(ls, ", ");
        print(s);
        print("}\n");
        i_1 = i + 1;
        dump_components2(lst, i_1); then ();
  end matchcontinue;
end dump_components2;

public function translate_dae "* function: translate_dae
 ** Translates the dae so variables are indexed into different arrays:
 ** - xd for derivatives
 ** - x for states
 ** - dummy_der for dummy derivatives
 ** - dummy for dummy states
 ** - y for algebraic variables 
 ** - p for parameters
 ** 
 **  The equations are updated with the new variable names.
 *"
  input DAELow in_daelow;
  output DAELow out_daelow;
algorithm 
  out_daelow:=
  matchcontinue (in_daelow)
    local
      type VarList = list<Var>;
      type EquationList = list<Equation>;
      VarList varlst,knvarlst,varlst_1,knvarlst_1,totvars,varlst_2,knvarlst_2;
      EquationList eqnsl,eqnsl_1;
      Exp_ExpList s,t;
      Algorithm_AlgorithmArray al_1,al;
      WhenClauseList wc_1,wc;
      Variables vars_1,knvars_1,vars_2,knvars_2,vars,knvars;
      EquationArray eqns_1,eqns,seqns,ie;
      ZeroCrossingList zc;
    case (DAELOW(vars,knvars,eqns,seqns,ie,al,EVENT_INFO(wc,zc)))
      equation 
        varlst = var_list(vars);
        knvarlst = var_list(knvars);
        varlst_1 = calculate_indexes(varlst);
        knvarlst_1 = calculate_indexes(knvarlst);
        totvars = list_append(varlst_1, knvarlst_1);
        eqnsl = equation_list(eqns);
        (s,t) = variable_replacements(totvars, eqnsl);
        eqnsl_1 = replace_variables(eqnsl, s, t);
        al_1 = replace_variables_in_alg(al, s, t);
        wc_1 = replace_variables_in_when_clauses(wc, s, t);
        varlst_2 = transform_variables(varlst_1, s, t);
        knvarlst_2 = transform_variables(knvarlst_1, s, t);
        vars_1 = empty_vars();
        knvars_1 = empty_vars();
        vars_2 = add_vars(varlst_2, vars_1);
        knvars_2 = add_vars(knvarlst_2, knvars_1);
        eqns_1 = list_equation(eqnsl_1); then DAELOW(vars_2,knvars_2,eqns_1,seqns,ie,al_1,EVENT_INFO(wc_1,zc));
  end matchcontinue;
end translate_dae;

protected function replace_variables_in_when_clauses " source list 
   target list "
  input WhenClauseList in_whenclauselist1;
  input Exp_ExpList in_exp_explist2;
  input Exp_ExpList in_exp_explist3;
  output WhenClauseList out_whenclauselist;
algorithm 
  out_whenclauselist:=
  matchcontinue (in_whenclauselist1,in_exp_explist2,in_exp_explist3)
    local
      WhenClause wc_1,wc;
      WhenClauseList wcx_1,wcx;
      Exp_ExpList s,t;
    case ({},_,_) then {}; 
    case (wc :: wcx,s,t)
      equation 
        wc_1 = replace_variables_in_when_clause(wc, s, t);
        wcx_1 = replace_variables_in_when_clauses(wcx, s, t); then wc_1 :: wcx_1;
  end matchcontinue;
end replace_variables_in_when_clauses;

protected function replace_variables_in_when_clause " source list 
   target list "
  input WhenClause in_whenclause1;
  input Exp_ExpList in_exp_explist2;
  input Exp_ExpList in_exp_explist3;
  output WhenClause out_whenclause;
algorithm 
  out_whenclause:=
  matchcontinue (in_whenclause1,in_exp_explist2,in_exp_explist3)
    local
      Exp.Exp e_1,e;
      Exp_ExpList s,t;
    case (WHEN_CLAUSE(e),s,t) " source list   target list "
      equation 
        (e_1,_) = Exp.replace_exp_list(e, s, t); then WHEN_CLAUSE(e_1);
  end matchcontinue;
end replace_variables_in_when_clause;

protected function replace_variables_in_alg " source list 
   target list "
  input Algorithm_AlgorithmArray in_algorithm_algorithmarray1;
  input Exp_ExpList in_exp_explist2;
  input Exp_ExpList in_exp_explist3;
  output Algorithm_AlgorithmArray out_algorithm_algorithmarray;
algorithm 
  out_algorithm_algorithmarray:=
  matchcontinue (in_algorithm_algorithmarray1,in_exp_explist2,in_exp_explist3)
    local
      type Algorithm_AlgorithmList = list<Algorithm.Algorithm>;
      Algorithm_AlgorithmList alglst,alglst_1;
      Algorithm_AlgorithmArray algarr_1,algarr;
      Exp_ExpList s,t;
    case (algarr,s,t) " source list   target list "
      equation 
        alglst = array_list(algarr);
        alglst_1 = replace_variables_in_alg2(alglst, s, t);
        algarr_1 = list_array(alglst_1); then algarr_1;
    case (_,_,_)
      equation 
        print("-replace_variables_in_alg failed\n"); then fail();
  end matchcontinue;
end replace_variables_in_alg;

protected function replace_variables_in_alg2 " source list 
   target list "
  input Algorithm_AlgorithmList in_algorithm_algorithmlist1;
  input Exp_ExpList in_exp_explist2;
  input Exp_ExpList in_exp_explist3;
  output Algorithm_AlgorithmList out_algorithm_algorithmlist;
protected 
  type Algorithm_AlgorithmList = list<Algorithm.Algorithm>;
algorithm 
  out_algorithm_algorithmlist:=
  matchcontinue (in_algorithm_algorithmlist1,in_exp_explist2,in_exp_explist3)
    local
      type Algorithm_StatementList = list<Algorithm.Statement>;
      Algorithm_AlgorithmList algs_1,algs;
      Algorithm_StatementList stmts_1,stmts;
      Exp_ExpList s,t;
    case ({},_,_) then {}; 
    case (Algorithm.ALGORITHM(stmts) :: algs,s,t)
      equation 
        algs_1 = replace_variables_in_alg2(algs, s, t);
        stmts_1 = replace_variables_in_stmts(stmts, s, t); then Algorithm.ALGORITHM(stmts_1) :: algs_1;
  end matchcontinue;
end replace_variables_in_alg2;

protected function replace_variables_in_stmts " source list 
   target list "
  input Algorithm_StatementList in_algorithm_statementlist1;
  input Exp_ExpList in_exp_explist2;
  input Exp_ExpList in_exp_explist3;
  output Algorithm_StatementList out_algorithm_statementlist;
protected 
  type Algorithm_StatementList = list<Algorithm.Statement>;
algorithm 
  out_algorithm_statementlist:=
  matchcontinue (in_algorithm_statementlist1,in_exp_explist2,in_exp_explist3)
    local
      Algorithm.Statement stmt_1,stmt;
      Algorithm_StatementList stmts_1,stmts;
      Exp_ExpList s,t;
    case ({},_,_) then {}; 
    case (stmt :: stmts,s,t)
      equation 
        stmt_1 = replace_variables_in_stmt(stmt, s, t);
        stmts_1 = replace_variables_in_stmts(stmts, s, t); then stmt_1 :: stmts_1;
  end matchcontinue;
end replace_variables_in_stmts;

protected function replace_variables_in_stmt " source list 
   target list "
  input Algorithm.Statement in_statement1;
  input Exp_ExpList in_exp_explist2;
  input Exp_ExpList in_exp_explist3;
  output Algorithm.Statement out_statement;
algorithm 
  out_statement:=
  matchcontinue (in_statement1,in_exp_explist2,in_exp_explist3)
    local
      type ValueList = list<Value>;
      type Algorithm_StatementList = list<Algorithm.Statement>;
      Exp.Exp e_1,e,exp_1,exp,e1_1,e2_1,e1,e2;
      Exp.ComponentRef cr_1,cr;
      Exp.Type tp;
      Exp_ExpList s,t,expl_1,expl;
      ValueList cnt;
      Algorithm_StatementList stmts_1,stmts;
      Algorithm.Else else_branch_1,else_branch;
      Boolean b;
      String id;
      Algorithm.Statement a;
    case (Algorithm.ASSIGN(tp,cr,e),s,t) " source list   target list "
      equation 
        (e_1,_) = Exp.replace_exp_list(e, s, t);
        (Exp.CREF(cr_1,_),_) = Exp.replace_exp_list(Exp.CREF(cr,Exp.OTHER()), s, t); then Algorithm.ASSIGN(tp,cr_1,e_1);
    case (Algorithm.TUPLE_ASSIGN(tp,expl,exp),s,t)
      equation 
        expl_1 = Util.list_map_2_2(expl, Exp.replace_exp_list, s, t);
        (expl_1,cnt) = Util.split_tuple2_list(expl_1);
        (exp_1,_) = Exp.replace_exp_list(exp, s, t); then Algorithm.TUPLE_ASSIGN(tp,expl_1,exp_1);
    case (Algorithm.ASSIGN_ARR(tp,cr,e),s,t)
      equation 
        (e_1,_) = Exp.replace_exp_list(e, s, t); then Algorithm.ASSIGN_ARR(tp,cr,e);
    case (Algorithm.IF(e,stmts,else_branch),s,t)
      equation 
        (e_1,_) = Exp.replace_exp_list(e, s, t);
        stmts_1 = replace_variables_in_stmts(stmts, s, t);
        else_branch_1 = replace_variables_in_else_branch(else_branch, s, t); then Algorithm.IF(e_1,stmts_1,else_branch_1);
    case (Algorithm.FOR(tp,b,id,e,stmts),s,t)
      equation 
        (e_1,_) = Exp.replace_exp_list(e, s, t);
        stmts_1 = replace_variables_in_stmts(stmts, s, t); then Algorithm.FOR(tp,b,id,e_1,stmts_1);
    case (Algorithm.WHILE(e,stmts),s,t)
      equation 
        (e_1,_) = Exp.replace_exp_list(e, s, t);
        stmts_1 = replace_variables_in_stmts(stmts, s, t); then Algorithm.WHILE(e,stmts);
    case (Algorithm.ASSERT(e1,e2),s,t)
      equation 
        (e1_1,_) = Exp.replace_exp_list(e1, s, t);
        (e2_1,_) = Exp.replace_exp_list(e2, s, t); then Algorithm.ASSERT(e1_1,e2_1);
    case (a,_,_)
      equation 
        print("Warning, fallthrough in replace_variables_in_stmts\n"); then a;
  end matchcontinue;
end replace_variables_in_stmt;

protected function replace_variables_in_else_branch " source list 
   target list "
  input Algorithm.Else in_else1;
  input Exp_ExpList in_exp_explist2;
  input Exp_ExpList in_exp_explist3;
  output Algorithm.Else out_else;
algorithm 
  out_else:=
  matchcontinue (in_else1,in_exp_explist2,in_exp_explist3)
    local
      type Algorithm_StatementList = list<Algorithm.Statement>;
      Algorithm.Else else_branch_1,else_branch;
      Exp.Exp e_1,e;
      Algorithm_StatementList stmts_1,stmts;
      Exp_ExpList s,t;
    case (Algorithm.NOELSE(),_,_) then Algorithm.NOELSE(); 
    case (Algorithm.ELSEIF(e,stmts,else_branch),s,t)
      equation 
        else_branch_1 = replace_variables_in_else_branch(else_branch, s, t);
        (e_1,_) = Exp.replace_exp_list(e, s, t);
        stmts_1 = replace_variables_in_stmts(stmts, s, t); then Algorithm.ELSEIF(e_1,stmts_1,else_branch_1);
    case (Algorithm.ELSE(stmts),s,t)
      equation 
        stmts_1 = replace_variables_in_stmts(stmts, s, t); then Algorithm.ELSE(stmts_1);
  end matchcontinue;
end replace_variables_in_else_branch;

protected function add_vars "* function: add_vars
 ** Adds a list of 'Var' to 'Variables'
 *"
  input VarList varlst;
  input Variables vars;
  output Variables vars_1;
protected 
  type VarList = list<Var>;
  Variables vars_1;
algorithm 
  vars_1 := Util.list_fold(varlst, add_var, vars);
end add_vars;

public function calculate_jacobian
  input Variables in_variables;
  input EquationArray in_equationarray;
  input IncidenceMatrix in_incidencematrix;
  input IncidenceMatrixT in_incidencematrixt;
  output TupleIntegerIntegerEquationList out_tupleintegerintegerequationlist;
protected 
  type TupleIntegerIntegerEquation = tuple<Integer,Integer,Equation>;
  type TupleIntegerIntegerEquationList = list<TupleIntegerIntegerEquation>;
algorithm 
  out_tupleintegerintegerequationlist:=
  matchcontinue (in_variables,in_equationarray,in_incidencematrix,in_incidencematrixt)
    local
      type EquationList = list<Equation>;
      type TupleValueValueEquation = tuple<Value,Value,Equation>;
      type TupleValueValueEquationList = list<TupleValueValueEquation>;
      type ValueList = list<Value>;
      type ValueListArray = ValueList[:];
      EquationList eqn_lst,eqn_lst_1;
      TupleValueValueEquationList jac,jac_1;
      Variables vars;
      EquationArray eqns;
      ValueListArray m,mt;
    case (vars,eqns,m,mt)
      equation 
        eqn_lst = equation_list(eqns);
        eqn_lst_1 = Util.list_map(eqn_lst, equation_to_residual_form);
        jac = calculate_jacobian_rows(eqn_lst_1, vars, m, mt);
        jac_1 = list_reverse(jac); then jac_1;
    case (_,_,_,_)
      equation 
        print("calculate_jacobian failed\n"); then fail();
  end matchcontinue;
end calculate_jacobian;

protected function calculate_jacobian_rows
  input EquationList eqns;
  input Variables vars;
  input IncidenceMatrix m;
  input IncidenceMatrixT mt;
  output TupleIntegerIntegerEquationList res;
protected 
  type EquationList = list<Equation>;
  type TupleIntegerIntegerEquation = tuple<Integer,Integer,Equation>;
  type TupleIntegerIntegerEquationList = list<TupleIntegerIntegerEquation>;
algorithm 
  res := calculate_jacobian_rows2(eqns, vars, m, mt, 1);
end calculate_jacobian_rows;

protected function calculate_jacobian_rows2
  input EquationList in_equationlist;
  input Variables in_variables;
  input IncidenceMatrix in_incidencematrix;
  input IncidenceMatrixT in_incidencematrixt;
  input Integer in_integer;
  output TupleIntegerIntegerEquationList out_tupleintegerintegerequationlist;
protected 
  type EquationList = list<Equation>;
  type TupleIntegerIntegerEquation = tuple<Integer,Integer,Equation>;
  type TupleIntegerIntegerEquationList = list<TupleIntegerIntegerEquation>;
algorithm 
  out_tupleintegerintegerequationlist:=
  matchcontinue (in_equationlist,in_variables,in_incidencematrix,in_incidencematrixt,in_integer)
    local
      type TupleValueValueEquation = tuple<Value,Value,Equation>;
      type TupleValueValueEquationList = list<TupleValueValueEquation>;
      type ValueList = list<Value>;
      type ValueListArray = ValueList[:];
      Value eqn_indx_1,eqn_indx;
      TupleValueValueEquationList l1,l2,res;
      Equation eqn;
      EquationList eqns;
      Variables vars;
      ValueListArray m,mt;
    case ({},_,_,_,_) then {}; 
    case (eqn :: eqns,vars,m,mt,eqn_indx)
      equation 
        eqn_indx_1 = eqn_indx + 1;
        l1 = calculate_jacobian_rows2(eqns, vars, m, mt, eqn_indx_1);
        l2 = calculate_jacobian_row(eqn, vars, m, mt, eqn_indx);
        res = list_append(l1, l2); then res;
  end matchcontinue;
end calculate_jacobian_rows2;

protected function calculate_jacobian_row " eqn index "
  input Equation in_equation;
  input Variables in_variables;
  input IncidenceMatrix in_incidencematrix;
  input IncidenceMatrixT in_incidencematrixt;
  input Integer in_integer;
  output TupleIntegerIntegerEquationList out_tupleintegerintegerequationlist;
protected 
  type TupleIntegerIntegerEquation = tuple<Integer,Integer,Equation>;
  type TupleIntegerIntegerEquationList = list<TupleIntegerIntegerEquation>;
algorithm 
  out_tupleintegerintegerequationlist:=
  matchcontinue (in_equation,in_variables,in_incidencematrix,in_incidencematrixt,in_integer)
    local
      type ValueList = list<Value>;
      type TupleValueValueEquation = tuple<Value,Value,Equation>;
      type TupleValueValueEquationList = list<TupleValueValueEquation>;
      type ValueListArray = ValueList[:];
      ValueList var_indxs,var_indxs_1,var_indxs_2;
      TupleValueValueEquationList eqns;
      Exp.Exp e;
      Variables vars;
      ValueListArray m,mt;
      Value eqn_indx;
    case (RESIDUAL_EQUATION(e),vars,m,mt,eqn_indx)
      equation 
        var_indxs = vars_in_eqn(m, eqn_indx);
        var_indxs_1 = Util.list_union_p(var_indxs, {}, int_eq) " Remove duplicates and get in correct order: acsending index ";
        var_indxs_2 = list_reverse(var_indxs_1);
        eqns = calculate_jacobian_row2(e, vars, eqn_indx, var_indxs_2); then eqns;
    case (_,_,_,_,_)
      equation 
        print("calculate_jacobian_row failed\n"); then fail();
  end matchcontinue;
end calculate_jacobian_row;

protected function make_residual_eqn "* function: make_residual_eqn
 ** Transforms an expression into a residual equation
 *"
  input Exp.Exp in_exp;
  output Equation out_equation;
algorithm 
  out_equation:=
  matchcontinue (in_exp)
    local Exp.Exp e;
    case (e) then RESIDUAL_EQUATION(e); 
  end matchcontinue;
end make_residual_eqn;

protected function calculate_jacobian_row2 " equation index 
   var indexes "
  input Exp.Exp in_exp;
  input Variables in_variables;
  input Integer in_integer;
  input IntegerList in_integerlist;
  output TupleIntegerIntegerEquationList out_tupleintegerintegerequationlist;
protected 
  type TupleIntegerIntegerEquation = tuple<Integer,Integer,Equation>;
  type TupleIntegerIntegerEquationList = list<TupleIntegerIntegerEquation>;
algorithm 
  out_tupleintegerintegerequationlist:=
  matchcontinue (in_exp,in_variables,in_integer,in_integerlist)
    local
      type TupleValueValueEquation = tuple<Value,Value,Equation>;
      type TupleValueValueEquationList = list<TupleValueValueEquation>;
      type ValueList = list<Value>;
      Exp.Exp e,e_1,e_2;
      Var v;
      Exp.ComponentRef cr;
      TupleValueValueEquationList es;
      Variables vars;
      Value eqn_indx,vindx;
      ValueList vindxs;
    case (e,_,_,{}) then {}; 
    case (e,vars,eqn_indx,vindx :: vindxs)
      equation 
        v = get_var_at(vars, vindx);
        cr = var_cref(v);
        e_1 = Derive.differentiate_exp(e, cr);
        e_2 = Exp.simplify(e_1);
        es = calculate_jacobian_row2(e, vars, eqn_indx, vindxs); then (eqn_indx,vindx,RESIDUAL_EQUATION(e_2)) :: es;
  end matchcontinue;
end calculate_jacobian_row2;

public function residual_exp "* function: residual_exp
** This function extracts the residual expression from a residual equation
 *"
  input Equation in_equation;
  output Exp.Exp out_exp;
algorithm 
  out_exp:=
  matchcontinue (in_equation)
    local Exp.Exp e;
    case (RESIDUAL_EQUATION(e)) then e; 
  end matchcontinue;
end residual_exp;

public function to_residual_form "* function: to_residual_form
 ** author: PA
 **
 ** This function transforms a daelow to residualform on the equations.
 *"
  input DAELow in_daelow;
  output DAELow out_daelow;
algorithm 
  out_daelow:=
  matchcontinue (in_daelow)
    local
      type EquationList = list<Equation>;
      EquationList eqn_lst,eqn_lst2;
      EquationArray eqns2,eqns,seqns,ieqns;
      Variables vars,knvars;
      Algorithm_AlgorithmArray ialg;
      EventInfo wc;
    case (DAELOW(vars,knvars,eqns,seqns,ieqns,ialg,wc))
      equation 
        eqn_lst = equation_list(eqns);
        eqn_lst2 = Util.list_map(eqn_lst, equation_to_residual_form);
        eqns2 = list_equation(eqn_lst2); then DAELOW(vars,knvars,eqns2,seqns,ieqns,ialg,wc);
  end matchcontinue;
end to_residual_form;

public function equation_to_residual_form "* function: equation_to_residual_form
 ** This function transforms an equation to its residual form.
 ** For instance, a=b is transformed to a-b=0 
 *"
  input Equation in_equation;
  output Equation out_equation;
algorithm 
  out_equation:=
  matchcontinue (in_equation)
    local
      Exp.Exp e,e1,e2,exp;
      Exp.ComponentRef cr;
    case (EQUATION(e1,e2))
      equation 
        e = Exp.simplify(Exp.BINARY(e1,Exp.SUB(Exp.REAL()),e2)); then RESIDUAL_EQUATION(e);
    case (SOLVED_EQUATION(cr,exp))
      equation 
        e = Exp.simplify(Exp.BINARY(Exp.CREF(cr,Exp.REAL()),Exp.SUB(Exp.REAL()),exp)); then RESIDUAL_EQUATION(e);
    case ((e = RESIDUAL_EQUATION(_)))
      local Equation e; then e;
    case ((e = ALGORITHM(_,_,_)))
      local Equation e; then e;
    case (_)
      equation 
        print("equation_to_residual_form failed\n"); then fail();
  end matchcontinue;
end equation_to_residual_form;

public function calculate_sizes " ny 
   np "
  input DAELow in_daelow;
  output Integer out_integer1;
  output Integer out_integer2;
  output Integer out_integer3;
algorithm 
  (out_integer1,out_integer2,out_integer3):=
  matchcontinue (in_daelow)
    local
      type VarList = list<Var>;
      VarList varlst,knvarlst;
      Value np,nx,ny;
      Variables vars,knvars;
    case (DAELOW(vars,knvars,_,_,_,_,_)) " nx   ny   np "
      equation 
        varlst = var_list(vars);
        knvarlst = var_list(knvars);
        np = calculate_param_sizes(knvarlst);
        (nx,ny) = calculate_var_sizes(varlst, 0, 0); then (nx,ny,np);
  end matchcontinue;
end calculate_sizes;

protected function calculate_param_sizes "* function: calculate_param_sizes
 ** Helper function to calculate_sizes
 *"
  input VarList in_varlist;
  output Integer out_integer;
protected 
  type VarList = list<Var>;
algorithm 
  out_integer:=
  matchcontinue (in_varlist)
    local
      Value s1;
      VarList vs;
    case ({}) then 0; 
    case (VAR(_,PARAM(),_,_,_,_,_,_,_,_,_,_,_) :: vs)
      equation 
        s1 = calculate_param_sizes(vs); then s1 + 1;
    case (VAR(_,_,_,_,_,_,_,_,_,_,_,_,_) :: vs)
      equation 
        s1 = calculate_param_sizes(vs); then s1;
  end matchcontinue;
end calculate_param_sizes;

protected function calculate_var_sizes "* function: calculate_var_sizes
 ** Helper function to calculate_sizes
 *"
  input VarList in_varlist1;
  input Integer in_integer2;
  input Integer in_integer3;
  output Integer out_integer1;
  output Integer out_integer2;
protected 
  type VarList = list<Var>;
algorithm 
  (out_integer1,out_integer2):=
  matchcontinue (in_varlist1,in_integer2,in_integer3)
    local
      Value nx,ny,ny_1,nx_2,ny_2,nx_1;
      VarList vs;
    case ({},nx,ny) then (nx,ny); 
    case (VAR(_,VARIABLE(),_,_,_,_,_,_,_,_,_,_,_) :: vs,nx,ny)
      equation 
        ny_1 = ny + 1;
        (nx_2,ny_2) = calculate_var_sizes(vs, nx, ny_1); then (nx_2,ny_2);
    case (VAR(_,DISCRETE(),_,_,_,_,_,_,_,_,_,_,_) :: vs,nx,ny)
      equation 
        ny_1 = ny + 1;
        (nx_2,ny_2) = calculate_var_sizes(vs, nx, ny_1); then (nx_2,ny_2);
    case (VAR(_,STATE(),_,_,_,_,_,_,_,_,_,_,_) :: vs,nx,ny)
      equation 
        nx_1 = nx + 1;
        (nx_2,ny_2) = calculate_var_sizes(vs, nx_1, ny); then (nx_2,ny_2);
    case (VAR(_,DUMMY_STATE(),_,_,_,_,_,_,_,_,_,_,_) :: vs,nx,ny)
      equation 
        ny_1 = ny + 1;
        (nx_2,ny_2) = calculate_var_sizes(vs, nx, ny_1); then (nx_2,ny_2);
    case (VAR(_,DUMMY_DER(),_,_,_,_,_,_,_,_,_,_,_) :: vs,nx,ny)
      equation 
        ny_1 = ny + 1;
        (nx_2,ny_2) = calculate_var_sizes(vs, nx, ny_1); then (nx_2,ny_2);
    case (_,_,_)
      equation 
        print("-calculate_var_sizes failed\n"); then fail();
  end matchcontinue;
end calculate_var_sizes;

protected function replace_variables " source list 
   target list "
  input EquationList in_equationlist1;
  input Exp_ExpList in_exp_explist2;
  input Exp_ExpList in_exp_explist3;
  output EquationList out_equationlist;
protected 
  type EquationList = list<Equation>;
algorithm 
  out_equationlist:=
  matchcontinue (in_equationlist1,in_exp_explist2,in_exp_explist3)
    local
      Exp.Exp e1_1,e2_1,e1,e2;
      EquationList es_1,es;
      Exp_ExpList s,t,inputs,outputs;
      Exp.ComponentRef cr_1,cr;
      Value i,indx;
    case ({},_,_) then {}; 
    case (EQUATION(e1,e2) :: es,s,t)
      equation 
        (e1_1,_) = Exp.replace_exp_list(e1, s, t);
        (e2_1,_) = Exp.replace_exp_list(e2, s, t);
        es_1 = replace_variables(es, s, t); then EQUATION(e1_1,e2_1) :: es_1;
    case (WHEN_EQUATION(WHEN_EQ(i,cr,e2)) :: es,s,t)
      equation 
        e1 = Exp.CREF(cr,Exp.OTHER());
        (e1_1 = Exp.CREF(cr_1,_),_) = Exp.replace_exp_list(e1, s, t);
        (e2_1,_) = Exp.replace_exp_list(e2, s, t);
        es_1 = replace_variables(es, s, t); then WHEN_EQUATION(WHEN_EQ(i,cr_1,e2_1)) :: es_1;
    case (ALGORITHM(indx,inputs,outputs) :: es,s,t)
      equation 
        es_1 = replace_variables(es, s, t) " TODO: we need to replace variables in algorithms too,
	 * or disallow removal of simple equations that can be found
	 * in algorithms 
	 *"; then ALGORITHM(indx,inputs,outputs) :: es_1;
    case (_,_,_)
      equation 
        print("-replace_variables failed\n"); then fail();
  end matchcontinue;
end replace_variables;

public function calculate_values "* function: calculate_values
 ** This function calculates the values from the parameter binding expressions.
 ** This is performed by building an environment and adding all the parameters and constants
 ** to it and then calling ceval to retreive the constant values of each parameter or constant.
 ** NOTE: This depends on the DAELow having the indexed forms of component references, since the 
 ** environmen requires simple names for each variable.
 **
*"
  input DAELow in_daelow;
  output DAELow out_daelow;
algorithm 
  out_daelow:=
  matchcontinue (in_daelow)
    local
      type Env_FrameList = list<Env.Frame>;
      type VarList = list<Var>;
      Env_FrameList env,env_1;
      VarList knvarlst,knvarlst_1;
      Variables knvars,knvars_1,vars;
      EquationArray eqns,seqns,ie;
      Algorithm_AlgorithmArray al;
      EventInfo wc;
    case (DAELOW(vars,knvars,eqns,seqns,ie,al,wc))
      equation 
        env = Builtin.initial_env();
        knvarlst = var_list(knvars);
        env_1 = add_variables_to_env(knvarlst, env);
        knvarlst_1 = update_variables(knvarlst, env_1);
        knvars = empty_vars();
        knvars_1 = add_vars(knvarlst_1, knvars); then DAELOW(vars,knvars_1,eqns,seqns,ie,al,wc);
  end matchcontinue;
end calculate_values;

protected function add_variables_to_env "* function: add_variables_to_env
 ** Helper function to calculate_values
 *"
  input VarList in_varlist;
  input Env.Env in_env;
  output Env.Env out_env;
protected 
  type VarList = list<Var>;
algorithm 
  out_env:=
  matchcontinue (in_varlist,in_env)
    local
      type Env_FrameList = list<Env.Frame>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Exp_SubscriptList = list<Exp.Subscript>;
      Env_FrameList env,env_1,env_2;
      TupleTypes_TTypeAbsyn_PathOption t_1;
      String crn,h;
      VarKind a;
      DAE.VarDirection b;
      DAE.Type t;
      Exp.Exp e;
      Exp_SubscriptList d;
      Exp_ExpOption f;
      Value g;
      Absyn_PathList i;
      DAE_VariableAttributesOption dae_var_attr;
      Absyn_CommentOption comment;
      VarList rest;
      Exp.ComponentRef cr;
    case ({},env) then env; 
    case (VAR(Exp.CREF_IDENT(crn,_),a,b,t,SOME(e),_,d,f,g,h,i,dae_var_attr,comment) :: rest,env)
      equation 
        t_1 = DAE.generate_dae_type(t) " Some of the attributes added to env here does not matter, defaults are used ";
        env_1 = Env.extend_frame_v(env, 
          Types.VAR(crn,Types.ATTR(false,SCode.RW(),SCode.CONST(),Absyn.BIDIR()),
          false,t_1,Types.EQBOUND(e,true)), NONE, false);
        env_2 = add_variables_to_env(rest, env_1); then env_2;
    case (VAR(Exp.CREF_IDENT(crn,_),a,b,t,NONE,_,d,f,g,h,i,dae_var_attr,comment) :: rest,env)
      equation 
        t_1 = DAE.generate_dae_type(t) " Some of the attributes added to env here does not matter, defaults are used ";
        env_1 = Env.extend_frame_v(env, 
          Types.VAR(crn,Types.ATTR(false,SCode.RW(),SCode.CONST(),Absyn.BIDIR()),
          false,t_1,Types.UNBOUND()), NONE, false);
        env_2 = add_variables_to_env(rest, env_1); then env_2;
    case (VAR((cr = Exp.CREF_QUAL(_,_,_)),_,_,_,_,_,_,_,_,_,_,_,_) :: rest,env)
      equation 
        Print.print_buf("Warning, skipping a variable qualified:");
        Exp.print_component_ref(cr);
        env_1 = add_variables_to_env(rest, env); then env_1;
    case (_ :: rest,env)
      equation 
        Print.print_buf("Warning, skipping a variable :");
        env_1 = add_variables_to_env(rest, env); then env_1;
  end matchcontinue;
end add_variables_to_env;

protected function update_variables "* function: update_variables
 ** Helper function to calculate_values
 *"
  input VarList in_varlist;
  input Env.Env in_env;
  output VarList out_varlist;
protected 
  type VarList = list<Var>;
algorithm 
  out_varlist:=
  matchcontinue (in_varlist,in_env)
    local
      type Exp_SubscriptList = list<Exp.Subscript>;
      type Env_FrameList = list<Env.Frame>;
      VarList rest_1,rest;
      Values.Value v;
      Exp.ComponentRef cr;
      VarKind a;
      DAE.VarDirection b;
      DAE.Type c;
      Exp.Exp e;
      Exp_SubscriptList d;
      Exp_ExpOption f;
      Value g;
      String h;
      Absyn_PathList i;
      DAE_VariableAttributesOption dae_var_attr;
      Absyn_CommentOption comment;
      Env_FrameList env;
    case ({},_) then {}; 
    case (VAR(cr,a,b,c,SOME(e),_,d,f,g,h,i,dae_var_attr,comment) :: rest,env)
      equation 
        rest_1 = update_variables(rest, env);
        (v,_) = Ceval.ceval(env, e, false, NONE, NONE); then VAR(cr,a,b,c,SOME(e),SOME(v),d,f,g,h,i,dae_var_attr,comment) :: rest_1;
    case (VAR(cr,a,b,c,SOME(e),v,d,f,g,h,i,dae_var_attr,comment) :: rest,env)
      local Values_ValueOption v;
      equation 
        rest_1 = update_variables(rest, env);
        failure((_,_) = Ceval.ceval(env, e, false, NONE, NONE));
        Print.print_buf("Warning, ceval failed for parameter: ");
        Exp.print_component_ref(cr);
        Print.print_buf("\n"); then VAR(cr,a,b,c,SOME(e),v,d,f,g,h,i,dae_var_attr,comment) :: rest_1;
    case (VAR(cr,a,b,c,NONE,v,d,f,g,h,i,dae_var_attr,comment) :: rest,env)
      local Values_ValueOption v;
      equation 
        rest_1 = update_variables(rest, env); then VAR(cr,a,b,c,NONE,v,d,f,g,h,i,dae_var_attr,comment) :: rest_1;
  end matchcontinue;
end update_variables;

protected function variable_replacements
  input VarList in_varlist;
  input EquationList in_equationlist;
  output Exp_ExpList out_exp_explist1;
  output Exp_ExpList out_exp_explist2;
protected 
  type VarList = list<Var>;
  type EquationList = list<Equation>;
algorithm 
  (out_exp_explist1,out_exp_explist2):=
  matchcontinue (in_varlist,in_equationlist)
    local
      type KeyList = list<Exp.ComponentRef>;
      BinTree bt;
      KeyList states;
      Exp_ExpList s1,t1,s2,t2,s,t;
      VarList vars;
      EquationList eqns;
    case (vars,eqns)
      equation 
        bt = states_eqns(eqns, empty_bintree);
        (states,_) = bintree_to_list(bt);
        (s1,t1) = derivative_replacements(states, vars);
        (s2,t2) = alg_variable_replacements(vars);
        s = list_append(s1, s2);
        t = list_append(t1, t2); then (s,t);
    case (vars,eqns)
      equation 
        print("-variable_replacements failed\n"); then fail();
  end matchcontinue;
end variable_replacements;

protected function alg_variable_replacements "* function: alg_variable_replacements
 ** Build replacement \"rules\" for the variables, eg. states, 
 ** algebraic variables, parameters, etc. 
 *"
  input VarList in_varlist;
  output Exp_ExpList out_exp_explist1;
  output Exp_ExpList out_exp_explist2;
protected 
  type VarList = list<Var>;
algorithm 
  (out_exp_explist1,out_exp_explist2):=
  matchcontinue (in_varlist)
    local
      Exp_ExpList s1,t1;
      String indxs,newid;
      Exp.ComponentRef cr;
      Value indx;
      DAE_VariableAttributesOption dae_var_attr;
      Absyn_CommentOption comment;
      VarList vs;
      VarKind STRUCTPARAM;
    case ({}) then ({},{}); 
    case (VAR(cr,VARIABLE(),_,_,_,_,_,_,indx,_,_,dae_var_attr,comment) :: vs)
      equation 
        (s1,t1) = alg_variable_replacements(vs);
        indxs = int_string(indx);
        newid = Util.string_append_list({"y[",indxs,"]"}); then (Exp.CREF(cr,Exp.REAL()) :: s1,Exp.CREF(Exp.CREF_IDENT(newid,{}),Exp.REAL()) :: t1);
    case (VAR(cr,STATE(),_,_,_,_,_,_,indx,_,_,dae_var_attr,comment) :: vs)
      equation 
        (s1,t1) = alg_variable_replacements(vs);
        indxs = int_string(indx);
        newid = Util.string_append_list({"x[",indxs,"]"}); then (Exp.CREF(cr,Exp.REAL()) :: s1,Exp.CREF(Exp.CREF_IDENT(newid,{}),Exp.REAL()) :: t1);
    case (VAR(cr,DUMMY_DER(),_,_,_,_,_,_,indx,_,_,dae_var_attr,comment) :: vs)
      equation 
        (s1,t1) = alg_variable_replacements(vs);
        indxs = int_string(indx);
        newid = Util.string_append_list({"y[",indxs,"]"}); then (Exp.CREF(cr,Exp.REAL()) :: s1,Exp.CREF(Exp.CREF_IDENT(newid,{}),Exp.REAL()) :: t1);
    case (VAR(cr,DUMMY_STATE(),_,_,_,_,_,_,indx,_,_,dae_var_attr,comment) :: vs)
      equation 
        (s1,t1) = alg_variable_replacements(vs);
        indxs = int_string(indx);
        newid = Util.string_append_list({"y[",indxs,"]"}); then (Exp.CREF(cr,Exp.REAL()) :: s1,Exp.CREF(Exp.CREF_IDENT(newid,{}),Exp.REAL()) :: t1);
    case (VAR(cr,DISCRETE(),_,_,_,_,_,_,indx,_,_,dae_var_attr,comment) :: vs)
      equation 
        (s1,t1) = alg_variable_replacements(vs);
        indxs = int_string(indx);
        newid = Util.string_append_list({"y[",indxs,"]"}); then (Exp.CREF(cr,Exp.REAL()) :: s1,Exp.CREF(Exp.CREF_IDENT(newid,{}),Exp.REAL()) :: t1);
    case (VAR(cr,PARAM(),_,_,_,_,_,_,indx,_,_,dae_var_attr,comment) :: vs)
      equation 
        (s1,t1) = alg_variable_replacements(vs);
        indxs = int_string(indx);
        newid = Util.string_append_list({"p[",indxs,"]"}); then (Exp.CREF(cr,Exp.REAL()) :: s1,Exp.CREF(Exp.CREF_IDENT(newid,{}),Exp.REAL()) :: t1);
    case (VAR(cr,STRUCTPARAM,_,_,_,_,_,_,indx,_,_,dae_var_attr,comment) :: vs)
      equation 
        (s1,t1) = alg_variable_replacements(vs);
        indxs = int_string(indx);
        newid = Util.string_append_list({"p[",indxs,"]"}); then (Exp.CREF(cr,Exp.REAL()) :: s1,Exp.CREF(Exp.CREF_IDENT(newid,{}),Exp.REAL()) :: t1);
    case (VAR(cr,CONST(),_,_,_,_,_,_,indx,_,_,dae_var_attr,comment) :: vs)
      equation 
        (s1,t1) = alg_variable_replacements(vs);
        indxs = int_string(indx);
        newid = Util.string_append_list({"p[",indxs,"]"}); then (Exp.CREF(cr,Exp.REAL()) :: s1,Exp.CREF(Exp.CREF_IDENT(newid,{}),Exp.REAL()) :: t1);
    case (_)
      equation 
        print("-alg_variable_replacements failed\n"); then fail();
  end matchcontinue;
end alg_variable_replacements;

protected function states_eqns "* function: states_eqns
 ** Takes a list of equations and an (empty) BinTree and 
 ** fills the tree with the state variables present in the equations
 *"
  input EquationList in_equationlist;
  input BinTree in_bintree;
  output BinTree out_bintree;
protected 
  type EquationList = list<Equation>;
algorithm 
  out_bintree:=
  matchcontinue (in_equationlist,in_bintree)
    local
      BinTree bt;
      Exp.Exp e1,e2;
      EquationList es;
      Value indx;
      Exp_ExpList expl1,expl2;
    case ({},bt) then bt; 
    case (EQUATION(e1,e2) :: es,bt)
      equation 
        bt = states_eqns(es, bt);
        bt = states_exp(e1, bt);
        bt = states_exp(e2, bt); then bt;
    case (ALGORITHM(indx,expl1,expl2) :: es,bt)
      equation 
        bt = Util.list_fold(expl1, states_exp, bt);
        bt = Util.list_fold(expl2, states_exp, bt);
        bt = states_eqns(es, bt); then bt;
    case (WHEN_EQUATION(_) :: es,bt)
      equation 
        bt = states_eqns(es, bt); then bt;
    case (_,_)
      equation 
        print("-states_eqns failed\n"); then fail();
  end matchcontinue;
end states_eqns;

protected function derivative_replacements
  input Exp_ComponentRefList in_exp_componentreflist;
  input VarList in_varlist;
  output Exp_ExpList out_exp_explist1;
  output Exp_ExpList out_exp_explist2;
protected 
  type Exp_ComponentRefList = list<Exp.ComponentRef>;
  type VarList = list<Var>;
algorithm 
  (out_exp_explist1,out_exp_explist2):=
  matchcontinue (in_exp_componentreflist,in_varlist)
    local
      type KeyList = list<Exp.ComponentRef>;
      Exp_ExpList s1,t1;
      Value indx;
      String indxs,newid;
      Exp.ComponentRef s;
      KeyList ss;
      VarList vars;
    case ({},_) then ({},{}); 
    case (s :: ss,vars)
      equation 
        (s1,t1) = derivative_replacements(ss, vars);
        indx = get_index(s, vars);
        indxs = int_string(indx);
        newid = Util.string_append_list({"xd[",indxs,"]"}); then (Exp.CALL(Absyn.IDENT("der"),{Exp.CREF(s,Exp.REAL())},false,true) :: s1,Exp.CREF(Exp.CREF_IDENT(newid,{}),Exp.REAL()) :: t1);
    case (_,_)
      equation 
        print("-derivative_replacements failed\n"); then fail();
  end matchcontinue;
end derivative_replacements;

protected function get_index "* function: get_index
 ** Helper function to derivative_replacements 
 *"
  input Exp.ComponentRef in_componentref;
  input VarList in_varlist;
  output Integer out_integer;
protected 
  type VarList = list<Var>;
algorithm 
  out_integer:=
  matchcontinue (in_componentref,in_varlist)
    local
      Exp.ComponentRef cr1,cr2;
      Value indx;
      DAE_VariableAttributesOption dae_var_attr;
      Absyn_CommentOption comment;
      VarList vs;
    case (cr1,VAR(cr2,_,_,_,_,_,_,_,indx,_,_,dae_var_attr,comment) :: _)
      equation 
        true = Exp.cref_equal(cr1, cr2); then indx;
    case (cr1,_ :: vs)
      equation 
        indx = get_index(cr1, vs); then indx;
  end matchcontinue;
end get_index;

protected function calculate_indexes "* function: calculate_indexes
 ** Helper function to translate_dae. Calculates the indexes for each variable
 ** in one of the arrays. x, xd, y.
 *"
  input VarList in_varlist;
  output VarList out_varlist;
protected 
  type VarList = list<Var>;
algorithm 
  out_varlist:=
  matchcontinue (in_varlist)
    local VarList vars_1,vars;
    case (vars)
      equation 
        vars_1 = calculate_indexes2(vars, 0, 0, 0, 0, 0); then vars_1;
    case (_)
      equation 
        print("-calculate_indexes failed\n"); then fail();
  end matchcontinue;
end calculate_indexes;

protected function transform_variables "* function: transform_variables
 ** Helper function to translate_dae
*"
  input VarList in_varlist1;
  input Exp_ExpList in_exp_explist2;
  input Exp_ExpList in_exp_explist3;
  output VarList out_varlist;
protected 
  type VarList = list<Var>;
algorithm 
  out_varlist:=
  matchcontinue (in_varlist1,in_exp_explist2,in_exp_explist3)
    local
      type Exp_SubscriptList = list<Exp.Subscript>;
      VarList vs_1,vs;
      Exp.ComponentRef cr_1,cr;
      String name;
      Exp.Exp e_1,e;
      VarKind kind;
      DAE.VarDirection a;
      DAE.Type b;
      Values_ValueOption c;
      Exp_SubscriptList d;
      Exp_ExpOption f;
      Value i;
      Absyn_PathList j;
      DAE_VariableAttributesOption dae_var_attr;
      Absyn_CommentOption comment;
      Exp_ExpList s,t;
    case ({},_,_) then {}; 
    case (VAR(cr,kind,a,b,SOME(e),c,d,f,i,_,j,dae_var_attr,comment) :: vs,s,t)
      equation 
        vs_1 = transform_variables(vs, s, t);
        cr_1 = transform_variable(i, kind);
        name = Exp.print_component_ref_str(cr);
        (e_1,_) = Exp.replace_exp_list(e, s, t); then VAR(cr_1,kind,a,b,SOME(e_1),c,d,f,i,name,j,dae_var_attr,comment) :: vs_1;
    case (VAR(cr,kind,a,b,NONE,c,d,f,i,_,j,dae_var_attr,comment) :: vs,s,t)
      equation 
        vs_1 = transform_variables(vs, s, t);
        cr_1 = transform_variable(i, kind);
        name = Exp.print_component_ref_str(cr); then VAR(cr_1,kind,a,b,NONE,c,d,f,i,name,j,dae_var_attr,comment) :: vs_1;
  end matchcontinue;
end transform_variables;

protected function transform_variable "* function: transform_variable
 ** Helper function to transform_variables
*"
  input Integer in_integer;
  input VarKind in_varkind;
  output Exp.ComponentRef out_componentref;
algorithm 
  out_componentref:=
  matchcontinue (in_integer,in_varkind)
    local
      String is,id;
      Value i;
    case (i,VARIABLE())
      equation 
        is = int_string(i);
        id = Util.string_append_list({"y[",is,"]"}); then Exp.CREF_IDENT(id,{});
    case (i,STATE())
      equation 
        is = int_string(i);
        id = Util.string_append_list({"x[",is,"]"}); then Exp.CREF_IDENT(id,{});
    case (i,DUMMY_DER())
      equation 
        is = int_string(i);
        id = Util.string_append_list({"y[",is,"]"}); then Exp.CREF_IDENT(id,{});
    case (i,DUMMY_STATE())
      equation 
        is = int_string(i);
        id = Util.string_append_list({"y[",is,"]"}); then Exp.CREF_IDENT(id,{});
    case (i,DISCRETE())
      equation 
        is = int_string(i);
        id = Util.string_append_list({"y[",is,"]"}); then Exp.CREF_IDENT(id,{});
    case (i,PARAM())
      equation 
        is = int_string(i);
        id = Util.string_append_list({"p[",is,"]"}); then Exp.CREF_IDENT(id,{});
    case (i,CONST())
      equation 
        is = int_string(i);
        id = Util.string_append_list({"p[",is,"]"}); then Exp.CREF_IDENT(id,{});
  end matchcontinue;
end transform_variable;

protected function calculate_indexes2 "* function: calculate_indexes2
** Helper function to calculate_indexes
 *"
  input VarList in_varlist1;
  input Integer in_integer2;
  input Integer in_integer3;
  input Integer in_integer4;
  input Integer in_integer5;
  input Integer in_integer6;
  output VarList out_varlist;
protected 
  type VarList = list<Var>;
algorithm 
  out_varlist:=
  matchcontinue (in_varlist1,in_integer2,in_integer3,in_integer4,in_integer5,in_integer6)
    local
      type Exp_SubscriptList = list<Exp.Subscript>;
      Value y_1,x,xd,y,p,dummy,x_1,p_1;
      VarList vars_1,vs;
      Exp.ComponentRef cr;
      DAE.VarDirection d;
      DAE.Type tp;
      Exp_ExpOption b,start;
      Values_ValueOption value;
      Exp_SubscriptList dim;
      String name;
      Absyn_PathList cl;
      DAE_VariableAttributesOption dae_var_attr;
      Absyn_CommentOption comment;
    case ({},_,_,_,_,_) then {}; 
    case (VAR(cr,VARIABLE(),d,tp,b,value,dim,start,_,name,cl,dae_var_attr,comment) :: vs,x,xd,y,p,dummy)
      equation 
        y_1 = y + 1;
        vars_1 = calculate_indexes2(vs, x, xd, y_1, p, dummy); then VAR(cr,VARIABLE(),d,tp,b,value,dim,start,y,name,cl,
          dae_var_attr,comment) :: vars_1;
    case (VAR(cr,STATE(),d,tp,b,value,dim,start,_,name,cl,dae_var_attr,comment) :: vs,x,xd,y,p,dummy)
      equation 
        x_1 = x + 1;
        vars_1 = calculate_indexes2(vs, x_1, xd, y, p, dummy); then VAR(cr,STATE(),d,tp,b,value,dim,start,x,name,cl,dae_var_attr,
          comment) :: vars_1;
    case (VAR(cr,DUMMY_DER(),d,tp,b,value,dim,start,_,name,cl,dae_var_attr,comment) :: vs,x,xd,y,p,dummy)
      equation 
        y_1 = y + 1 " Dummy derivatives become algebraic variables ";
        vars_1 = calculate_indexes2(vs, x, xd, y_1, p, dummy); then VAR(cr,DUMMY_DER(),d,tp,b,value,dim,start,y,name,cl,
          dae_var_attr,comment) :: vars_1;
    case (VAR(cr,DUMMY_STATE(),d,tp,b,value,dim,start,_,name,cl,dae_var_attr,comment) :: vs,x,xd,y,p,dummy)
      equation 
        y_1 = y + 1 " Dummy state become algebraic variables ";
        vars_1 = calculate_indexes2(vs, x, xd, y_1, p, dummy); then VAR(cr,DUMMY_STATE(),d,tp,b,value,dim,start,y,name,cl,
          dae_var_attr,comment) :: vars_1;
    case (VAR(cr,DISCRETE(),d,tp,b,value,dim,start,_,name,cl,dae_var_attr,comment) :: vs,x,xd,y,p,dummy)
      equation 
        y_1 = y + 1;
        vars_1 = calculate_indexes2(vs, x, xd, y_1, p, dummy); then VAR(cr,DISCRETE(),d,tp,b,value,dim,start,y,name,cl,
          dae_var_attr,comment) :: vars_1;
    case (VAR(cr,PARAM(),d,tp,b,value,dim,start,_,name,cl,dae_var_attr,comment) :: vs,x,xd,y,p,dummy)
      equation 
        p_1 = p + 1;
        vars_1 = calculate_indexes2(vs, x, xd, y, p_1, dummy); then VAR(cr,PARAM(),d,tp,b,value,dim,start,p,name,cl,dae_var_attr,
          comment) :: vars_1;
    case (VAR(cr,CONST(),d,tp,b,value,dim,start,_,name,cl,dae_var_attr,comment) :: vs,x,xd,y,p,dummy)
      equation 
        p_1 = p + 1;
        vars_1 = calculate_indexes2(vs, x, xd, y, p_1, dummy); then VAR(cr,CONST(),d,tp,b,value,dim,start,p,name,cl,dae_var_attr,
          comment) :: vars_1;
  end matchcontinue;
end calculate_indexes2;

protected function print_equations "* function: print_equations
 ** Helper function to dump
 *"
  input IntegerList in_integerlist;
  input DAELow in_daelow;
algorithm 
  _:=
  matchcontinue (in_integerlist,in_daelow)
    local
      type ValueList = list<Value>;
      Value n;
      ValueList rest;
      DAELow dae;
    case ({},_) then (); 
    case (n :: rest,dae)
      equation 
        print_equations(rest, dae);
        print_equation_no(n, dae); then ();
  end matchcontinue;
end print_equations;

protected function print_equation_no "* function: print_equation_no
 ** Helper function to print_equations
 *"
  input Integer in_integer;
  input DAELow in_daelow;
algorithm 
  _:=
  matchcontinue (in_integer,in_daelow)
    local
      Value eqno_1,eqno;
      Equation eq;
      EquationArray eqns;
    case (eqno,DAELOW(_,_,eqns,_,_,_,_))
      equation 
        eqno_1 = eqno - 1;
        eq = equation_nth(eqns, eqno_1);
        print_equation(eq); then ();
  end matchcontinue;
end print_equation_no;

protected function print_equation "* function: print_equation
 ** Helper function to print_equations
 *"
  input Equation in_equation;
algorithm 
  _:=
  matchcontinue (in_equation)
    local
      String s1,s2,res;
      Exp.Exp e1,e2;
      Exp.ComponentRef cr;
      WhenEquation w;
    case (EQUATION(e1,e2))
      equation 
        s1 = Exp.print_exp_str(e1);
        s2 = Exp.print_exp_str(e2);
        res = Util.string_append_list({s1," = ",s2,"\n"});
        print(res); then ();
    case (WHEN_EQUATION(w))
      equation 
        (cr,e2) = get_when_equation_expr(w);
        s1 = Exp.print_component_ref_str(cr);
        s2 = Exp.print_exp_str(e2);
        res = Util.string_append_list({s1," =  ",s2,"\n"});
        print(res); then ();
  end matchcontinue;
end print_equation;

protected function tree_get "* function: tree_get
 ** Copied from generic implementation. Changed that no hashrelation is passed
 ** since a string can not be uniquely mapped to an int. Therefore we need to compare two strings
 ** to get a unique ordering.
*"
  input BinTree bt;
  input Key key;
  output Value v;
protected 
  String keystr;
algorithm 
  keystr := Exp.print_component_ref_str(key);
  v := tree_get2(bt, keystr);
end tree_get;

protected function tree_get2 "* function: tree_get2
 ** Helper function to tree_get
 *"
  input BinTree in_bintree;
  input String in_string;
  output Value out_value;
algorithm 
  out_value:=
  matchcontinue (in_bintree,in_string)
    local
      String rkeystr,keystr;
      Exp.ComponentRef rkey;
      Value rval,cmpval,res;
      BinTreeOption left,right;
    case (TREENODE(SOME(TREEVALUE(rkey,rval)),left,right),keystr)
      equation 
        rkeystr = Exp.print_component_ref_str(rkey);
        0 = System.strcmp(rkeystr, keystr); then rval;
    case (TREENODE(SOME(TREEVALUE(rkey,rval)),left,SOME(right)),keystr)
      local BinTree right;
      equation 
        rkeystr = Exp.print_component_ref_str(rkey) " Search to the right";
        cmpval = System.strcmp(rkeystr, keystr);
        (cmpval > 0) = true;
        res = tree_get2(right, keystr); then res;
    case (TREENODE(SOME(TREEVALUE(rkey,rval)),SOME(left),right),keystr)
      local BinTree left;
      equation 
        rkeystr = Exp.print_component_ref_str(rkey) " Search to the left";
        cmpval = System.strcmp(rkeystr, keystr);
        (cmpval > 0) = false;
        res = tree_get2(left, keystr); then res;
  end matchcontinue;
end tree_get2;

protected function tree_add "* function: tree_add
 ** Copied from generic implementation. Changed that no hashrelation is passed
 ** since a string (ComponentRef) can not be uniquely mapped to an int. Therefore we need to compare two strings
 ** to get a unique ordering.
*"
  input BinTree in_bintree;
  input Key in_key;
  input Value in_value;
  output BinTree out_bintree;
algorithm 
  out_bintree:=
  matchcontinue (in_bintree,in_key,in_value)
    local
      Exp.ComponentRef key,rkey;
      Value value,rval,cmpval;
      String rkeystr,keystr;
      BinTreeOption left,right;
      BinTree t_1,t,right_1,left_1;
    case (TREENODE(NONE,NONE,NONE),key,value) then TREENODE(SOME(TREEVALUE(key,value)),NONE,NONE); 
    case (TREENODE(SOME(TREEVALUE(rkey,rval)),left,right),key,value)
      equation 
        rkeystr = Exp.print_component_ref_str(rkey) " Replace this node ";
        keystr = Exp.print_component_ref_str(key);
        0 = System.strcmp(rkeystr, keystr); then TREENODE(SOME(TREEVALUE(rkey,value)),left,right);
    case (TREENODE(SOME(TREEVALUE(rkey,rval)),left,(right = SOME(t))),key,value)
      equation 
        keystr = Exp.print_component_ref_str(key) " Insert to right subtree";
        rkeystr = Exp.print_component_ref_str(rkey);
        cmpval = System.strcmp(rkeystr, keystr);
        (cmpval > 0) = true;
        t_1 = tree_add(t, key, value); then TREENODE(SOME(TREEVALUE(rkey,rval)),left,SOME(t_1));
    case (TREENODE(SOME(TREEVALUE(rkey,rval)),left,(right = NONE)),key,value)
      equation 
        keystr = Exp.print_component_ref_str(key) " Insert to right node";
        rkeystr = Exp.print_component_ref_str(rkey);
        cmpval = System.strcmp(rkeystr, keystr);
        (cmpval > 0) = true;
        right_1 = tree_add(TREENODE(NONE,NONE,NONE), key, value); then TREENODE(SOME(TREEVALUE(rkey,rval)),left,SOME(right_1));
    case (TREENODE(SOME(TREEVALUE(rkey,rval)),(left = SOME(t)),right),key,value)
      equation 
        keystr = Exp.print_component_ref_str(key) " Insert to left subtree";
        rkeystr = Exp.print_component_ref_str(rkey);
        cmpval = System.strcmp(rkeystr, keystr);
        (cmpval > 0) = false;
        t_1 = tree_add(t, key, value); then TREENODE(SOME(TREEVALUE(rkey,rval)),SOME(t_1),right);
    case (TREENODE(SOME(TREEVALUE(rkey,rval)),(left = NONE),right),key,value)
      equation 
        keystr = Exp.print_component_ref_str(key) " Insert to left node";
        rkeystr = Exp.print_component_ref_str(rkey);
        cmpval = System.strcmp(rkeystr, keystr);
        (cmpval > 0) = false;
        left_1 = tree_add(TREENODE(NONE,NONE,NONE), key, value); then TREENODE(SOME(TREEVALUE(rkey,rval)),SOME(left_1),right);
    case (_,_,_)
      equation 
        print("tree_add failed\n"); then fail();
  end matchcontinue;
end tree_add;

protected function tree_delete "* function: tree_delete
 ** This function deletes an entry from the BinTree.
 *"
  input BinTree in_bintree;
  input Key in_key;
  output BinTree out_bintree;
algorithm 
  out_bintree:=
  matchcontinue (in_bintree,in_key)
    local
      BinTree bt,right_1,right,t_1,t;
      Exp.ComponentRef key,rkey;
      String rkeystr,keystr;
      TreeValue rightmost;
      BinTreeOption optright_1,left,lleft,lright,topt_1;
      Value rval,cmpval;
      TreeValueOption leftval;
    case ((bt = TREENODE(NONE,NONE,NONE)),key) then bt; 
    case (TREENODE(SOME(TREEVALUE(rkey,rval)),left,SOME(right)),key)
      equation 
        rkeystr = Exp.print_component_ref_str(rkey) " delete this node, when existing right node ";
        keystr = Exp.print_component_ref_str(key);
        0 = System.strcmp(rkeystr, keystr);
        (rightmost,right_1) = tree_delete_rightmost_value(right);
        optright_1 = tree_prune_empty_nodes(right_1); then TREENODE(SOME(rightmost),left,optright_1);
    case (TREENODE(SOME(TREEVALUE(rkey,rval)),SOME(TREENODE(leftval,lleft,lright)),NONE),key)
      equation 
        rkeystr = Exp.print_component_ref_str(rkey) " delete this node, when no right node, but left node ";
        keystr = Exp.print_component_ref_str(key);
        0 = System.strcmp(rkeystr, keystr); then TREENODE(leftval,lleft,lright);
    case (TREENODE(SOME(TREEVALUE(rkey,rval)),NONE,NONE),key)
      equation 
        rkeystr = Exp.print_component_ref_str(rkey) " delete this node, when no left or right node ";
        keystr = Exp.print_component_ref_str(key);
        0 = System.strcmp(rkeystr, keystr); then TREENODE(NONE,NONE,NONE);
    case (TREENODE(SOME(TREEVALUE(rkey,rval)),left,(right = SOME(t))),key)
      local BinTreeOption right;
      equation 
        keystr = Exp.print_component_ref_str(key) " delete in right subtree";
        rkeystr = Exp.print_component_ref_str(rkey);
        cmpval = System.strcmp(rkeystr, keystr);
        (cmpval > 0) = true;
        t_1 = tree_delete(t, key);
        topt_1 = tree_prune_empty_nodes(t_1); then TREENODE(SOME(TREEVALUE(rkey,rval)),left,topt_1);
    case (TREENODE(SOME(TREEVALUE(rkey,rval)),(left = SOME(t)),right),key)
      local BinTreeOption right;
      equation 
        keystr = Exp.print_component_ref_str(key) " delete in left subtree";
        rkeystr = Exp.print_component_ref_str(rkey);
        cmpval = System.strcmp(rkeystr, keystr);
        (cmpval > 0) = false;
        t_1 = tree_delete(t, key);
        topt_1 = tree_prune_empty_nodes(t_1); then TREENODE(SOME(TREEVALUE(rkey,rval)),topt_1,right);
    case (_,_)
      equation 
        print("tree_delete failed\n"); then fail();
  end matchcontinue;
end tree_delete;

protected function tree_delete_rightmost_value " deleted value"
  input BinTree in_bintree;
  output TreeValue out_treevalue;
  output BinTree out_bintree;
algorithm 
  (out_treevalue,out_bintree):=
  matchcontinue (in_bintree)
    local
      TreeValue treevalue,value;
      BinTree left,right_1,right,bt;
      BinTreeOption rightopt_1;
      TreeValueOption treeval;
    case (TREENODE(SOME(treevalue),NONE,NONE)) then (treevalue,TREENODE(NONE,NONE,NONE)); 
    case (TREENODE(SOME(treevalue),SOME(left),NONE)) then (treevalue,left); 
    case (TREENODE(treeval,left,SOME(right)))
      local BinTreeOption left;
      equation 
        (value,right_1) = tree_delete_rightmost_value(right);
        rightopt_1 = tree_prune_empty_nodes(right_1); then (value,TREENODE(treeval,left,rightopt_1));
    case (TREENODE(SOME(treeval),NONE,SOME(right)))
      local TreeValue treeval;
      equation 
        failure((_,_) = tree_delete_rightmost_value(right));
        print("right value was empty , left NONE\n"); then (treeval,TREENODE(NONE,NONE,NONE));
    case (bt)
      equation 
        print("-tree_delete_rightmost_value failed\n"); then fail();
  end matchcontinue;
end tree_delete_rightmost_value;

protected function tree_prune_empty_nodes "* function: tree_prune_emtpy_nodes
 ** This function is a helper function to tree_delete
 ** It is used to delete empty nodes of the BinTree representation, that might be introduced
 ** when deleting nodes.
 *"
  input BinTree in_bintree;
  output BinTreeOption out_bintreeoption;
algorithm 
  out_bintreeoption:=
  matchcontinue (in_bintree)
    local BinTree bt;
    case TREENODE(NONE,NONE,NONE) then NONE; 
    case bt then SOME(bt); 
  end matchcontinue;
end tree_prune_empty_nodes;

protected function bintree_to_list "* function: bintree_to_list
 ** This function takes a BinTree and transform it into a list
 ** representation, i.e. two lists of keys and values
 *"
  input BinTree in_bintree;
  output KeyList out_keylist;
  output ValueList out_valuelist;
protected 
  type KeyList = list<Key>;
  type ValueList = list<Value>;
algorithm 
  (out_keylist,out_valuelist):=
  matchcontinue (in_bintree)
    local
      KeyList klst;
      ValueList vlst;
      BinTree bt;
    case (bt)
      equation 
        (klst,vlst) = bintree_to_list2(bt, {}, {}); then (klst,vlst);
    case (_)
      equation 
        print("-bintree_to_list failed\n"); then fail();
  end matchcontinue;
end bintree_to_list;

protected function bintree_to_list2
  input BinTree in_bintree;
  input KeyList in_keylist;
  input ValueList in_valuelist;
  output KeyList out_keylist;
  output ValueList out_valuelist;
protected 
  type KeyList = list<Key>;
  type ValueList = list<Value>;
algorithm 
  (out_keylist,out_valuelist):=
  matchcontinue (in_bintree,in_keylist,in_valuelist)
    local
      KeyList klst;
      ValueList vlst;
      Exp.ComponentRef key;
      Value value;
      BinTreeOption left,right;
    case (TREENODE(NONE,NONE,NONE),klst,vlst) then (klst,vlst); 
    case (TREENODE(SOME(TREEVALUE(key,value)),left,right),klst,vlst)
      equation 
        (klst,vlst) = bintree_to_list_opt(left, klst, vlst);
        (klst,vlst) = bintree_to_list_opt(right, klst, vlst); then (key :: klst,value :: vlst);
    case (TREENODE(NONE,left,right),klst,vlst)
      equation 
        (klst,vlst) = bintree_to_list_opt(left, klst, vlst);
        (klst,vlst) = bintree_to_list_opt(left, klst, vlst); then (klst,vlst);
  end matchcontinue;
end bintree_to_list2;

protected function bintree_to_list_opt
  input BinTreeOption in_bintreeoption;
  input KeyList in_keylist;
  input ValueList in_valuelist;
  output KeyList out_keylist;
  output ValueList out_valuelist;
protected 
  type KeyList = list<Key>;
  type ValueList = list<Value>;
algorithm 
  (out_keylist,out_valuelist):=
  matchcontinue (in_bintreeoption,in_keylist,in_valuelist)
    local
      KeyList klst;
      ValueList vlst;
      BinTree bt;
    case (NONE,klst,vlst) then (klst,vlst); 
    case (SOME(bt),klst,vlst)
      equation 
        (klst,vlst) = bintree_to_list2(bt, klst, vlst); then (klst,vlst);
  end matchcontinue;
end bintree_to_list_opt;

protected function print_vars_statistics "* function: print_vars_statistics
 ** Prints statistics on variables, currently depth of BinTree, etc.
 *"
  input Variables in_variables1;
  input Variables in_variables2;
algorithm 
  _:=
  matchcontinue (in_variables1,in_variables2)
    local
      String lenstr,bstr;
      VariableArray v1,v2;
      Value bsize1,n1,bsize2,n2;
    case (VARIABLES(_,v1,bsize1,n1),VARIABLES(_,v2,bsize2,n2))
      equation 
        print("Variable Statistics\n");
        print("===================\n");
        print("Number of variables: ");
        lenstr = int_string(n1);
        print(lenstr);
        print("\n");
        print("Bucket size for variables: ");
        bstr = int_string(bsize1);
        print(bstr);
        print("\n");
        print("Number of known variables: ");
        lenstr = int_string(n2);
        print(lenstr);
        print("\n");
        print("Bucket size for known variables: ");
        bstr = int_string(bsize1);
        print(bstr);
        print("\n"); then ();
  end matchcontinue;
end print_vars_statistics;

protected function bintree_depth "* function: bintree_depth
 ** This function calculates the depth of the Binary Tree given
 ** as input. It can be used for debugging purposes to investigate
 ** how balanced binary trees are.
 *"
  input BinTree in_bintree;
  output Integer out_integer;
algorithm 
  out_integer:=
  matchcontinue (in_bintree)
    local
      Value ld,rd,res;
      BinTree left,right;
    case (TREENODE(_,NONE,NONE)) then 1; 
    case (TREENODE(_,SOME(left),SOME(right)))
      equation 
        ld = bintree_depth(left);
        rd = bintree_depth(right);
        res = int_max(ld, rd); then res + 1;
    case (TREENODE(_,SOME(left),NONE))
      equation 
        ld = bintree_depth(left); then ld;
    case (TREENODE(_,NONE,SOME(right)))
      equation 
        rd = bintree_depth(right); then rd;
  end matchcontinue;
end bintree_depth;

protected function is_algebraic "* function: is_algebraic
 ** This function returns true if an expression is purely algebraic, i.e. not
 ** containing any derivatives
 ** Otherwise it returns false.
 *"
  input Exp.Exp in_exp;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_exp)
    local
      type TupleExp_ExpBoolean = tuple<Exp.Exp,Boolean>;
      type TupleExp_ExpBooleanList = list<TupleExp_ExpBoolean>;
      type TupleExp_ExpBooleanListList = list<TupleExp_ExpBooleanList>;
      Value x,ival,i;
      String s,id;
      Exp.ComponentRef c;
      Exp.Exp e1,e2,e21,e22,e,t,f,start,stop,step,cr,dim,exp,iterexp;
      Exp.Operator op;
      Exp.Type ty,ty2,REAL;
      Exp_ExpList args,es;
      Absyn.Path fcn;
    case (Exp.END()) then true; 
    case (Exp.ICONST(x)) then true; 
    case (Exp.RCONST(x))
      local Real x; then true;
    case (Exp.SCONST(s)) then true; 
    case (Exp.BCONST(false)) then true; 
    case (Exp.BCONST(true)) then true; 
    case (Exp.CREF(c,_)) then true; 
    case (Exp.BINARY(e1,(op = Exp.SUB(ty)),(e2 = Exp.BINARY(e21,Exp.SUB(ty2),e22))))
      equation 
        true = is_algebraic(e1);
        true = is_algebraic(e2); then true;
    case (Exp.BINARY(e1,op,e2))
      equation 
        true = is_algebraic(e1);
        true = is_algebraic(e2); then true;
    case (Exp.UNARY(op,e))
      equation 
        true = is_algebraic(e); then true;
    case (Exp.LBINARY(e1,op,e2))
      equation 
        true = is_algebraic(e1);
        true = is_algebraic(e2); then true;
    case (Exp.LUNARY(op,e))
      equation 
        true = is_algebraic(e); then true;
    case (Exp.RELATION(e1,op,e2))
      equation 
        true = is_algebraic(e1);
        true = is_algebraic(e2); then true;
    case (Exp.IFEXP(c,t,f))
      local Exp.Exp c;
      equation 
        true = is_algebraic(c);
        true = is_algebraic(t);
        true = is_algebraic(f); then true;
    case (Exp.CALL(Absyn.IDENT("der"),args,_,_)) then false; 
    case (Exp.CALL(fcn,args,_,_)) then true; 
    case (Exp.ARRAY(_,_,es)) then true; 
    case (Exp.TUPLE(es)) then true; 
    case (Exp.MATRIX(_,_,es))
      local TupleExp_ExpBooleanListList es; then true;
    case (Exp.RANGE(_,start,NONE,stop))
      equation 
        true = is_algebraic(start);
        true = is_algebraic(stop); then true;
    case (Exp.RANGE(_,start,SOME(step),stop))
      equation 
        true = is_algebraic(start);
        true = is_algebraic(step);
        true = is_algebraic(stop); then true;
    case (Exp.CAST(REAL,Exp.ICONST(ival))) then true; 
    case (Exp.CAST(REAL,Exp.UNARY(Exp.UMINUS(_),Exp.ICONST(ival)))) then true; 
    case (Exp.CAST(Exp.REAL(),e)) then true; 
    case (Exp.CAST(Exp.REAL(),e)) then true; 
    case (Exp.ASUB(e,i))
      equation 
        true = is_algebraic(e); then true;
    case (Exp.SIZE(cr,SOME(dim))) then true; 
    case (Exp.SIZE(cr,NONE)) then true; 
    case (Exp.REDUCTION(fcn,exp,id,iterexp)) then true; 
    case (_) then true; 
  end matchcontinue;
end is_algebraic;

public function is_var_known "* function: is_var_known
 ** Returns true if the the variable is present in the variable list.
 ** This is done by traversing the list, searching for a matching variable 
 ** name.
 *"
  input VarList in_varlist;
  input Exp.ComponentRef in_componentref;
  output Boolean out_boolean;
protected 
  type VarList = list<Var>;
algorithm 
  out_boolean:=
  matchcontinue (in_varlist,in_componentref)
    local
      Exp.ComponentRef var_name,cr;
      Var variable;
      Value indx;
      String origname;
      DAE_VariableAttributesOption dae_var_attr;
      Absyn_CommentOption comment;
      VarList rest;
      Boolean res;
    case ({},var_name) then false; 
    case ((variable = VAR(cr,_,_,_,_,_,_,_,indx,origname,_,dae_var_attr,comment)) :: rest,var_name)
      equation 
        true = Exp.cref_equal(cr, var_name); then true;
    case ((variable = VAR(cr,_,_,_,_,_,_,_,indx,origname,_,dae_var_attr,comment)) :: rest,var_name)
      equation 
        res = is_var_known(rest, var_name); then res;
  end matchcontinue;
end is_var_known;

public function get_all_exps "* function: get_all_exps
 ** 
 ** This function goes through the DAELow structure and finds all the
 ** expressions and returns them in a list
 *"
  input DAELow in_daelow;
  output Exp_ExpList out_exp_explist;
algorithm 
  out_exp_explist:=
  matchcontinue (in_daelow)
    local
      type Algorithm_AlgorithmList = list<Algorithm.Algorithm>;
      type Exp_ExpListList = list<Exp_ExpList>;
      Exp_ExpList exps1,exps2,exps3,exps4,exps5,exps;
      Algorithm_AlgorithmList alglst;
      Exp_ExpListList explist6,explist;
      Variables vars1,vars2;
      EquationArray eqns,reqns,ieqns;
      Algorithm_AlgorithmArray algs;
    case (DAELOW(vars1,vars2,eqns,reqns,ieqns,algs,_))
      equation 
        exps1 = get_all_exps_vars(vars1);
        exps2 = get_all_exps_vars(vars2);
        exps3 = get_all_exps_eqns(eqns);
        exps4 = get_all_exps_eqns(reqns);
        exps5 = get_all_exps_eqns(ieqns);
        alglst = array_list(algs);
        explist6 = Util.list_map(alglst, Algorithm.get_all_exps);
        explist = list_append({exps1,exps2,exps3,exps4,exps5}, explist6);
        exps = Util.list_flatten(explist); then exps;
  end matchcontinue;
end get_all_exps;

protected function get_all_exps_vars "* function: get_all_exps_vars
 ** 
 ** Helper to get_all_exps. Goes through the Variables type
 *"
  input Variables in_variables;
  output Exp_ExpList out_exp_explist;
algorithm 
  out_exp_explist:=
  matchcontinue (in_variables)
    local
      type VarList = list<Var>;
      VarList vars;
      Exp_ExpList exps;
      CrefIndexListArray crefindex;
      VariableArray vararray;
      Value bsize,nvars;
    case VARIABLES(crefindex,vararray,bsize,nvars) " We can ignore crefs, they don't contains real expressions "
      equation 
        vars = vararray_list(vararray);
        exps = Util.list_map(vars, get_all_exps_var);
        exps = Util.list_flatten(exps); then exps;
  end matchcontinue;
end get_all_exps_vars;

protected function get_all_exps_var "* function: get_all_exps_var
 ** 
 ** Helper to get_all_exps_vars. Get all exps from a  Var.
 ** Exp.OTHER is used as type for componentref. Not important here.
 ** We only use the exp list for finding function calls
 *"
  input Var in_var;
  output Exp_ExpList out_exp_explist;
algorithm 
  out_exp_explist:=
  matchcontinue (in_var)
    local
      type Exp_SubscriptList = list<Exp.Subscript>;
      Exp_ExpList e1,e2,e3,exps;
      Exp.ComponentRef cref;
      VarKind vk;
      DAE.VarDirection vd;
      DAE.Type ty;
      Exp_ExpOption bndexp,startvalexp;
      Values_ValueOption bndval;
      Exp_SubscriptList instdims;
      Value ind;
      String orgname;
      Absyn_PathList clsnames;
      DAE_VariableAttributesOption dae_var_attr;
      Absyn_CommentOption comment;
    case VAR(cref,vk,vd,ty,bndexp,bndval,instdims,startvalexp,ind,orgname,clsnames,dae_var_attr,comment)
      equation 
        e1 = Util.option_to_list(bndexp);
        e2 = Util.option_to_list(startvalexp);
        e3 = Util.list_map(instdims, get_all_exps_subscript);
        e3 = Util.list_flatten(e3);
        exps = Util.list_flatten({e1,e2,e3,{Exp.CREF(cref,Exp.OTHER())}}); then exps;
  end matchcontinue;
end get_all_exps_var;

protected function get_all_exps_subscript "* function: get_all_exps_subscript
 ** 
 ** Get all exps from a Subscript 
 *"
  input Exp.Subscript in_subscript;
  output Exp_ExpList out_exp_explist;
algorithm 
  out_exp_explist:=
  matchcontinue (in_subscript)
    local Exp.Exp e;
    case Exp.WHOLEDIM() then {}; 
    case Exp.SLICE(e) then {e}; 
    case Exp.INDEX(e) then {e}; 
  end matchcontinue;
end get_all_exps_subscript;

protected function get_all_exps_eqns "* function: get_all_exps_eqns
 ** 
 ** Helper to get_all_exps. Goes through the EquationArray type
 *"
  input EquationArray in_equationarray;
  output Exp_ExpList out_exp_explist;
algorithm 
  out_exp_explist:=
  matchcontinue (in_equationarray)
    local
      type EquationList = list<Equation>;
      EquationList eqns;
      Exp_ExpList exps;
      EquationArray eqnarray;
    case ((eqnarray = EQUATION_ARRAY(_,_,_)))
      equation 
        eqns = equation_list(eqnarray);
        exps = Util.list_map(eqns, get_all_exps_eqn);
        exps = Util.list_flatten(exps); then exps;
  end matchcontinue;
end get_all_exps_eqns;

protected function get_all_exps_eqn "* function: get_all_exps_eqn
 ** 
 ** Helper to get_all_exps_eqns. Get all exps from an Equation.
 *"
  input Equation in_equation;
  output Exp_ExpList out_exp_explist;
algorithm 
  out_exp_explist:=
  matchcontinue (in_equation)
    local
      Exp.Exp e1,e2,e;
      Exp.ComponentRef cr;
      Exp_ExpList exps;
      Value ind;
    case EQUATION(e1,e2) then {e1,e2}; 
    case SOLVED_EQUATION(cr,e) then {Exp.CREF(cr,Exp.OTHER()),e}; 
    case WHEN_EQUATION(WHEN_EQ(_,cr,e)) then {Exp.CREF(cr,Exp.OTHER()),e}; 
    case ALGORITHM(ind,e1,e2)
      local Exp_ExpList e1,e2;
      equation 
        exps = list_append(e1, e2); then exps;
  end matchcontinue;
end get_all_exps_eqn;

public function is_param "* function: is_param
 **
 ** Return true if variable is a parameter.
 *"
  input Var in_var;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_var)
    case (VAR(_,PARAM(),_,_,_,_,_,_,_,_,_,_,_)) then true; 
    case (_) then false; 
  end matchcontinue;
end is_param;

public function is_output "* function: is_output
 **
 ** Return true if variable is declared as output. Note that the output
 ** attribute sticks with a variable even if it is originating from a sub
 ** component, which is not the case for Dymola.
 *"
  input Var in_var;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_var)
    case (VAR(_,_,DAE.OUTPUT(),_,_,_,_,_,_,_,_,_,_)) then true; 
    case (_) then false; 
  end matchcontinue;
end is_output;

public function is_input "* function: is_input
 **
 ** Returns true if variable is declared as input.
 ** See also is_ouput above
*"
  input Var in_var;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_var)
    case (VAR(_,_,DAE.INPUT(),_,_,_,_,_,_,_,_,_,_)) then true; 
    case (_) then false; 
  end matchcontinue;
end is_input;

public function get_when_equation_expr "* function: get_when_equation_expr
 **
 ** Get the left and right hand parts from an equation appearing in a when clause
 *"
  input WhenEquation in_whenequation;
  output Exp.ComponentRef out_componentref;
  output Exp.Exp out_exp;
algorithm 
  (out_componentref,out_exp):=
  matchcontinue (in_whenequation)
    local
      Exp.ComponentRef cr;
      Exp.Exp e;
    case (WHEN_EQ(_,cr,e)) then (cr,e); 
  end matchcontinue;
end get_when_equation_expr;
end DAELow;

