package Interactive "
    Copyright (c) PELAB, Dept. Computer and Information Science, Linkoping University,
    Sweden, in the following denoted PELAB/LIU.

   
    This file is part of OpenModelica, in the following denoted OSM.


    The OSM License, Version 1.1, March 4, 2005.


    1 Preface

    The aim of this license is to lay down the conditions enabling you to use, 
    modify and circulate OSM. However, PELAB/LIU remain the authors of 
    OSM and so retain property rights and the use of all ancillary rights.

    2 Definitions

    OSM is defined as all successive versions of the OSM software and their 
    documentation that have been developed by PELAB/LIU and including
    accepted contributions from other contributors according to this license.

    OSM DERIVED SOFTWARE is defined as all or part of OSM that you have 
    modified and/or translated and/or adapted.


    3 Dual License

    OSM is made available under the OSM licensing scheme, which is a dual
    licensing scheme with two options, a) and b):

    a) OSM OPEN SOURCE LICENSE:

    If you wish to write Open Source software you can use the Open Source version 
    of OSM, released under the OSM license which include GPL as its open source 
    licensing option. If you use the OSM Open Source version you must release your 
    Application using OSM including this Application's source code under the GPL as well.

    This OSM license text, and Copyright (c) PELAB/Linkoping University, must
    be present in your copy of OSM and in OSM DERIVED SOFTWARE.

    You should have received a copy of the GPL - GNU General Public License
    along with OpenModelica; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.


    b) OSM COMMERCIAL LICENSE

    If you are using OSM commercially - that is, for commercial usage or
    for creating proprietary software for sale or use in a commercial setting
    - you must purchase a commercial license of OSM from PELAB/L
   $Id: Interactive.rml 1801 2005-06-03 06:46:30Z x02lucpo $ 
  *
 ** This module contain functionality for model management, expression evaluation, etc. 
 ** in the interactive environment.
 ** The module defines a symboltable used in the interactive environment containing:
 ** - Modelica models (described using Absyn AST)
 ** - Variable bindings
 ** - Compiled functions (so they do not need to be recompiled)
 ** - Instantiated classes (that can be reused, not impl. yet)
 ** - Modelica models in SCode form (to speed up instantiation. not impl. yet)
 *"

public import OpenModelica.Compiler.Absyn;

public import OpenModelica.Compiler.SCode;

public import OpenModelica.Compiler.DAE;

public import OpenModelica.Compiler.Types;

public import OpenModelica.Compiler.Values;

public import OpenModelica.Compiler.Env;

public 
uniontype InteractiveStmt
  record IALG
    Absyn.AlgorithmItem x1;
  end IALG;
  record IEXP
    Absyn.Exp x1;
  end IEXP;
end InteractiveStmt;

public 
type InteractiveStmtList = list<InteractiveStmt>;

uniontype InteractiveStmts
  record ISTMTS
    InteractiveStmtList x1;
    Boolean x2 " true if output result == no semicolon";
  end ISTMTS;
end InteractiveStmts;

public 
type DAE_ElementList = list<DAE.Element>;

uniontype InstantiatedClass
  record INSTCLASS
    Absyn.Path x1;
    DAE_ElementList x2 " The F.Q.name of the inst:ed class  The list of DAE elements ";
    Env.Env x3;
  end INSTCLASS;
end InstantiatedClass;

public 
uniontype InteractiveVariable
  record IVAR
    Absyn.Ident x1;
    Values.Value x2 " The variable identifier ";
    Types.Type x3 " The expression containing the value   The type of the expression ";
  end IVAR;
end InteractiveVariable;

public 
type InstantiatedClassList = list<InstantiatedClass>;

type InteractiveVariableList = list<InteractiveVariable>;

type TupleAbsyn_PathTypes_Type = tuple<Absyn.Path,Types.Type>;

type TupleAbsyn_PathTypes_TypeList = list<TupleAbsyn_PathTypes_Type>;

uniontype InteractiveSymbolTable
  record SYMBOLTABLE
    Absyn.Program x1;
    SCode.Program x2 "The ast";
    InstantiatedClassList x3 " The exploded ast ";
    InteractiveVariableList x4 " List of instantiated classes";
    TupleAbsyn_PathTypes_TypeList x5 "List of variables with values  List of compiled functions, F.Q name + type";
  end SYMBOLTABLE;
end InteractiveSymbolTable;

protected import OpenModelica.Compiler.Dump;

protected import OpenModelica.Compiler.Debug;

protected import OpenModelica.Compiler.RTOpts;

protected import OpenModelica.Compiler.Util;

protected import OpenModelica.Compiler.Parser;

protected import OpenModelica.Compiler.Prefix;

protected import OpenModelica.Compiler.Mod;

protected import OpenModelica.Compiler.Lookup;

protected import OpenModelica.Compiler.ClassInf;

protected import OpenModelica.Compiler.Exp;

protected import OpenModelica.Compiler.Inst;

protected import OpenModelica.Compiler.Static;

protected import OpenModelica.Compiler.ModUtil;

protected import OpenModelica.Compiler.Codegen;

protected import OpenModelica.Compiler.Print;

protected import OpenModelica.Compiler.System;

protected import OpenModelica.Compiler.ClassLoader;

protected import OpenModelica.Compiler.Ceval;

protected constant String Placement_records="record Transformation Real x=0.0; Real y=0.0; Real scale=1.0; Real aspectRatio=1.0; Boolean flipHorizontal=false; Boolean flipVertical=false; Real rotation=0.0; end Transformation; record Placement Boolean visible=true; Transformation transformation; Transformation iconTransformation; end Placement;";

protected constant String Line_records="type LinePattern=enumeration(None,Solid,Dash,Dot,DashDot,DashDotDot); type Arrow=enumeration(None,Open,Filled,Half); record Line  Boolean visible=true;  Real points[:,2]; Integer color[3]={0,0,0}; LinePattern pattern = LinePattern.Solid; Real thickness=0.25; Arrow arrow[2]={Arrow.None,Arrow.None};  Real arrowSize = 3.0;  Boolean smooth=false; end Line;";

protected constant String Icon_records="record GraphicItem Boolean visible=true; end GraphicItem; record CoordinateSystem Real extent[2,2]; end CoordinateSystem; record Icon CoordinateSystem coordinateSystem(extent ={{-10.0, -10.0}, {10.0, 10.0}}); /*GraphicItem graphics[:];*/ end Icon;";

protected constant String Diagram_records="record GraphicItem Boolean visible=true; end GraphicItem; record CoordinateSystem Real extent[2,2]; end CoordinateSystem; record Diagram CoordinateSystem coordinateSystem(extent ={{-100.0, -100.0}, {100.0, 100.0}}); /*GraphicItem graphics[:];*/ end Diagram;";

protected constant String Graphics_records="type LinePattern=enumeration(None,Solid,Dash,Dot,DashDot,DashDotDot); type Arrow=enumeration(None,Open,Filled,Half); type FillPattern=enumeration(None,Solid,Horizontal,Vertical,Cross,Forward,Backward,CrossDiag,HorizontalCylinder,VerticalCylinder,Sphere); type BorderPattern=enumeration(None,Raised,Sunken,Engraved); type TextStyle=enumeration(Bold,Italic,Underline); record Line  Boolean visible=true;  Real points[:,2]; Integer color[3]={0,0,0}; LinePattern pattern = LinePattern.Solid; Real thickness=0.25; Arrow arrow[2]={Arrow.None,Arrow.None};  Real arrowSize = 3.0;  Boolean smooth=false; end Line; record Polygon Boolean visible=true;Integer lineColor[3]={0,0,0};Integer fillColor[3]={0,0,0}; LinePattern pattern=LinePattern.Solid; FillPattern fillPattern=FillPattern.None; Real lineThickness=0.25; Real points[:,2]; Boolean smooth=false; end Polygon; record Rectangle Boolean visible=true;Integer lineColor[3]={0,0,0};Integer fillColor[3]={0,0,0}; LinePattern pattern=LinePattern.Solid; FillPattern fillPattern=FillPattern.None; Real lineThickness=0.25; BorderPattern borderPattern=BorderPattern.None; Real extent[2,2]; Real radius=0.0; end Rectangle; record Ellipse Boolean visible=true;Integer lineColor[3]={0,0,0};Integer fillColor[3]={0,0,0}; LinePattern pattern=LinePattern.Solid; FillPattern fillPattern=FillPattern.None; Real lineThickness=0.25; Real extent[2,2]; end Ellipse; record Text Boolean visible=true;Integer lineColor[3]={0,0,0};Integer fillColor[3]={0,0,0}; LinePattern pattern=LinePattern.Solid; FillPattern fillPattern=FillPattern.None; Real lineThickness=0.25; Real extent[2,2]; String textString; Real fontSize = 0.0; String fontName=\"\"; TextStyle textStyle[:]; end Text; record Bitmap Boolean visible=true; Real extent[2,2]; String fileName=\"\"; String imageSource=\"\"; end Bitmap;" " This variable contains all records needed for the graphical primitives, such as Line, Ellipse, etc. ";

public function evaluate
  input InteractiveStmts in_interactivestmts;
  input InteractiveSymbolTable in_interactivesymboltable;
  input Boolean in_boolean;
  output String out_string;
  output InteractiveSymbolTable out_interactivesymboltable;
algorithm 
  (out_string,out_interactivesymboltable):=
  matchcontinue (in_interactivestmts,in_interactivesymboltable,in_boolean)
    local
      String res,res2,res_1;
      InteractiveSymbolTable newst,st,newst_1;
      InteractiveStmt x;
      Boolean verbose,sc;
      InteractiveStmtList xs;
    case (ISTMTS({x},true),st,false)
      equation 
        (res,newst) = evaluate2(ISTMTS({x},true), st); then ("",newst);
    case (ISTMTS({x},true),st,true)
      equation 
        (res,newst) = evaluate2(ISTMTS({x},false), st) " Verbose, semicolon off "; then (res,newst);
    case (ISTMTS({x},false),st,verbose)
      equation 
        (res,newst) = evaluate2(ISTMTS({x},false), st); then (res,newst);
    case (ISTMTS(x :: xs,sc),st,false)
      equation 
        (res,newst) = evaluate2(ISTMTS({x},sc), st);
        (res,newst_1) = evaluate(ISTMTS(xs,sc), newst, false); then (res,newst_1);
    case (ISTMTS(x :: xs,sc),st,true)
      equation 
        (res,newst) = evaluate2(ISTMTS({x},sc), st);
        (res2,newst_1) = evaluate(ISTMTS(xs,sc), newst, true);
        res_1 = Util.string_append_list({res,res2}); then (res_1,newst_1);
  end matchcontinue;
end evaluate;

protected function evaluate2
  input InteractiveStmts in_interactivestmts;
  input InteractiveSymbolTable in_interactivesymboltable;
  output String out_string;
  output InteractiveSymbolTable out_interactivesymboltable;
algorithm 
  (out_string,out_interactivesymboltable):=
  matchcontinue (in_interactivestmts,in_interactivesymboltable)
    local
      String vars,str,str_1,s1;
      InteractiveSymbolTable st,newst;
      InteractiveStmts stmts;
      InteractiveStmt exp;
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("listVariables",_),Absyn.FUNCTIONARGS({},{})))},_),(st = SYMBOLTABLE(_,_,_,vars,_)))
      equation 
        vars = get_variable_names(vars);
        str = string_append(vars, "\n"); then (str,st);
    case ((stmts = ISTMTS({IEXP(Absyn.CALL(_,_))},_)),st)
      equation 
        (str,newst) = evaluate_graphical_api(stmts, st);
        str_1 = string_append(str, "\n"); then (str_1,newst);
    case ((stmts = ISTMTS({exp},_)),st)
      equation 
        (s1,newst) = evaluate_expr(exp, st);
        str = string_append(s1, "\n"); then (str,newst);
    case (_,st) then ("Not implemented yet\n",st); 
  end matchcontinue;
end evaluate2;

protected function evaluate_expr
  input InteractiveStmt in_interactivestmt;
  input InteractiveSymbolTable in_interactivesymboltable;
  output String out_string;
  output InteractiveSymbolTable out_interactivesymboltable;
algorithm 
  (out_string,out_interactivesymboltable):=
  matchcontinue (in_interactivestmt,in_interactivesymboltable)
    local
      type Env_FrameList = list<Env.Frame>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleTypes_TTypeAbsyn_PathOptionList = list<TupleTypes_TTypeAbsyn_PathOption>;
      type StringList = list<String>;
      type Values_ValueList = list<Values.Value>;
      type Absyn_ExpList = list<Absyn.Exp>;
      Env_FrameList env;
      Exp.Exp sexp,econd,msg_1,srexp;
      Types.Properties prop,rprop;
      InteractiveSymbolTable st_1,st_2,st,newst;
      Values.Value value;
      String str,ident,s1;
      Absyn.Exp exp,cond,msg,rexp;
      Absyn.Program p;
      TupleTypes_TTypeAbsyn_PathOption t;
      TupleTypes_TTypeAbsyn_PathOptionList types;
      StringList idents;
      Values_ValueList values;
      Absyn_ExpList crefexps;
      InteractiveStmt stmt;
    case (IEXP(exp),(st = SYMBOLTABLE(p,_,_,_,_)))
      equation 
        env = build_env_from_symboltable(st);
        (sexp,prop,SOME(st_1)) = Static.elab_exp(env, exp, true, SOME(st));
        (value,SOME(st_2)) = Ceval.ceval(env, sexp, true, SOME(st_1), NONE);
        str = Values.val_string(value); then (str,st_2);
    case (IEXP(exp),(st = SYMBOLTABLE(p,_,_,_,_)))
      equation 
        env = build_env_from_symboltable(st);
        failure((sexp,prop,SOME(st_1)) = Static.elab_exp(env, exp, true, SOME(st))); then fail();
    case (IEXP(exp),(st = SYMBOLTABLE(p,_,_,_,_)))
      equation 
        env = build_env_from_symboltable(st);
        (sexp,prop,SOME(st_1)) = Static.elab_exp(env, exp, true, SOME(st));
        failure((_,_) = Ceval.ceval(env, sexp, true, SOME(st_1), NONE)); then fail();
    case (IALG(Absyn.ALGORITHMITEM(Absyn.ALG_NORETCALL(Absyn.CREF_IDENT("assert",_),Absyn.FUNCTIONARGS({cond,msg},_)),_)),(st = SYMBOLTABLE(p,_,_,_,_)))
      equation 
        env = build_env_from_symboltable(st);
        (econd,prop,SOME(st_1)) = Static.elab_exp(env, cond, true, SOME(st));
        (Values.BOOL(true),SOME(st_2)) = Ceval.ceval(env, econd, true, SOME(st_1), NONE); then ("",st_2);
    case (IALG(Absyn.ALGORITHMITEM(Absyn.ALG_NORETCALL(Absyn.CREF_IDENT("assert",_),Absyn.FUNCTIONARGS({cond,msg},_)),_)),(st = SYMBOLTABLE(p,_,_,_,_)))
      equation 
        env = build_env_from_symboltable(st);
        (msg_1,prop,SOME(st_1)) = Static.elab_exp(env, msg, true, SOME(st));
        (Values.STRING(str),SOME(st_2)) = Ceval.ceval(env, msg_1, true, SOME(st_1), NONE); then (str,st_2);
    case (IALG(Absyn.ALGORITHMITEM(Absyn.ALG_ASSIGN(Absyn.CREF_IDENT(ident,{}),exp),_)),(st = SYMBOLTABLE(p,_,_,_,_)))
      equation 
        env = build_env_from_symboltable(st);
        (sexp,Types.PROP(t,_),SOME(st_1)) = Static.elab_exp(env, exp, true, SOME(st));
        (value,SOME(st_2)) = Ceval.ceval(env, sexp, true, SOME(st_1), NONE);
        str = Values.val_string(value);
        newst = add_var_to_symboltable(ident, value, t, st_2); then (str,newst);
    case (IALG(Absyn.ALGORITHMITEM(Absyn.ALG_TUPLE_ASSIGN(Absyn.TUPLE(crefexps),rexp),_)),(st = SYMBOLTABLE(p,_,_,_,_))) " Since expressions cannot be tuples an empty string is returned "
      equation 
        env = build_env_from_symboltable(st);
        (srexp,rprop,SOME(st_1)) = Static.elab_exp(env, rexp, true, SOME(st));
        ((Types.T_TUPLE(types),_)) = Types.get_prop_type(rprop);
        idents = Util.list_map(crefexps, get_ident_from_crefexp);
        (Values.TUPLE(values),SOME(st_2)) = Ceval.ceval(env, srexp, true, SOME(st_1), NONE);
        newst = add_vars_to_symboltable(idents, values, types, st_2); then ("",newst);
    case (stmt,st)
      equation 
        s1 = Print.get_error_string();
        Print.clear_error_buf();
        str = string_append("Error evaluating expr.\n", s1);
        Debug.fprint("failtrace", "#-- evaluate_expr failed:");
        Debug.fcall("failtrace", Dump.dump_istmt, ISTMTS({stmt},true));
        Debug.fprintln("failtrace", "\n"); then (str,st);
  end matchcontinue;
end evaluate_expr;

protected function get_ident_from_crefexp
  input Absyn.Exp in_exp;
  output Absyn.Ident out_ident;
algorithm 
  out_ident:=
  matchcontinue (in_exp)
    local
      String id,str;
      Absyn.Exp exp;
    case Absyn.CREF(Absyn.CREF_IDENT(id,_)) then id; 
    case exp
      equation 
        Print.print_error_buf("# Not a component reference in tuple assignment:\n# ");
        str = Dump.print_exp_str(exp);
        Print.print_error_buf(str);
        Print.print_error_buf("\n"); then fail();
  end matchcontinue;
end get_ident_from_crefexp;

protected function get_variable_names
  input InteractiveVariableList vars;
  output String res;
protected 
  type StringList = list<String>;
  StringList strlst;
  String str;
algorithm 
  strlst := get_variable_list_str(vars);
  str := Util.string_delimit_list(strlst, ", ");
  res := Util.string_append_list({"{",str,"}"});
end get_variable_names;

protected function get_variable_list_str
  input InteractiveVariableList in_interactivevariablelist;
  output StringList out_stringlist;
protected 
  type StringList = list<String>;
algorithm 
  out_stringlist:=
  matchcontinue (in_interactivevariablelist)
    local
      StringList res;
      String p;
      InteractiveVariableList vs;
    case ({}) then {}; 
    case (IVAR(p,_,_) :: vs)
      equation 
        res = get_variable_list_str(vs); then p :: res;
  end matchcontinue;
end get_variable_list_str;

public function get_type_of_variable
  input Absyn.Ident in_ident;
  input InteractiveVariableList in_interactivevariablelist;
  output Types.Type out_type;
algorithm 
  out_type:=
  matchcontinue (in_ident,in_interactivevariablelist)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      String id,varid;
      TupleTypes_TTypeAbsyn_PathOption tp;
      InteractiveVariableList rest;
    case (id,{}) then fail(); 
    case (varid,IVAR(id,_,tp) :: rest)
      equation 
        equality(varid = id); then tp;
    case (varid,IVAR(id,_,_) :: rest)
      equation 
        failure(equality(varid = id));
        tp = get_type_of_variable(varid, rest); then tp;
  end matchcontinue;
end get_type_of_variable;

protected function add_vars_to_symboltable
  input Absyn_IdentList in_absyn_identlist;
  input Values_ValueList in_values_valuelist;
  input Types_TypeList in_types_typelist;
  input InteractiveSymbolTable in_interactivesymboltable;
  output InteractiveSymbolTable out_interactivesymboltable;
protected 
  type Absyn_IdentList = list<Absyn.Ident>;
  type Values_ValueList = list<Values.Value>;
  type Types_TypeList = list<Types.Type>;
algorithm 
  out_interactivesymboltable:=
  matchcontinue (in_absyn_identlist,in_values_valuelist,in_types_typelist,in_interactivesymboltable)
    local
      type StringList = list<String>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleTypes_TTypeAbsyn_PathOptionList = list<TupleTypes_TTypeAbsyn_PathOption>;
      InteractiveSymbolTable st,st_1,st_2;
      String id;
      StringList idrest;
      Values.Value v;
      Values_ValueList vrest;
      TupleTypes_TTypeAbsyn_PathOption t;
      TupleTypes_TTypeAbsyn_PathOptionList trest;
    case ({},_,_,st) then st; 
    case (id :: idrest,v :: vrest,t :: trest,st)
      equation 
        st_1 = add_var_to_symboltable(id, v, t, st);
        st_2 = add_vars_to_symboltable(idrest, vrest, trest, st_1); then st_2;
  end matchcontinue;
end add_vars_to_symboltable;

public function add_var_to_symboltable
  input Absyn.Ident in_ident;
  input Values.Value in_value;
  input Types.Type in_type;
  input InteractiveSymbolTable in_interactivesymboltable;
  output InteractiveSymbolTable out_interactivesymboltable;
algorithm 
  out_interactivesymboltable:=
  matchcontinue (in_ident,in_value,in_type,in_interactivesymboltable)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type SCode_ClassList = list<SCode.Class>;
      type TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOption = tuple<Absyn.Path,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOptionList = list<TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOption>;
      InteractiveVariableList vars_1,vars;
      String ident;
      Values.Value v;
      TupleTypes_TTypeAbsyn_PathOption t;
      Absyn.Program p;
      SCode_ClassList sp;
      InstantiatedClassList id;
      TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOptionList cf;
    case (ident,v,t,SYMBOLTABLE(p,sp,id,vars,cf))
      equation 
        vars_1 = add_var_to_varlist(ident, v, t, vars); then SYMBOLTABLE(p,sp,id,vars_1,cf);
  end matchcontinue;
end add_var_to_symboltable;

protected function add_var_to_varlist
  input Absyn.Ident in_ident;
  input Values.Value in_value;
  input Types.Type in_type;
  input InteractiveVariableList in_interactivevariablelist;
  output InteractiveVariableList out_interactivevariablelist;
algorithm 
  out_interactivevariablelist:=
  matchcontinue (in_ident,in_value,in_type,in_interactivevariablelist)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      String ident,id2;
      Values.Value v,val2;
      TupleTypes_TTypeAbsyn_PathOption t,t2;
      InteractiveVariableList rest,rest_1;
    case (ident,v,t,IVAR(id2,_,_) :: rest)
      equation 
        equality(ident = id2); then IVAR(ident,v,t) :: rest;
    case (ident,v,t,IVAR(id2,val2,t2) :: rest)
      equation 
        failure(equality(ident = id2));
        rest_1 = add_var_to_varlist(ident, v, t, rest); then IVAR(id2,val2,t2) :: rest_1;
    case (ident,v,t,{}) then {IVAR(ident,v,t)}; 
  end matchcontinue;
end add_var_to_varlist;

public function build_env_from_symboltable "* function: build_env_from_symboltable
 ** author: PA
 ** 
 ** Builds an environment from a symboltable by adding all interactive variables
 ** and their bindings to the environment.
 *"
  input InteractiveSymbolTable in_interactivesymboltable;
  output Env.Env out_env;
algorithm 
  out_env:=
  matchcontinue (in_interactivesymboltable)
    local
      type SCode_ClassList = list<SCode.Class>;
      type Env_FrameList = list<Env.Frame>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOption = tuple<Absyn.Path,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOptionList = list<TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOption>;
      SCode_ClassList p_1,sp;
      Env_FrameList env,env_1;
      Absyn.Program p;
      InstantiatedClassList ic;
      InteractiveVariableList vars;
      TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOptionList cf;
    case (SYMBOLTABLE(p,sp,ic,vars,cf))
      equation 
        p_1 = SCode.elaborate(p);
        env = Inst.make_env_from_program(p_1, Absyn.IDENT(""));
        env_1 = add_vars_to_env(vars, env); then env_1;
  end matchcontinue;
end build_env_from_symboltable;

protected function add_vars_to_env
  input InteractiveVariableList in_interactivevariablelist;
  input Env.Env in_env;
  output Env.Env out_env;
algorithm 
  out_env:=
  matchcontinue (in_interactivevariablelist,in_env)
    local
      type Env_FrameList = list<Env.Frame>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      Env_FrameList env_1,env_2,env;
      String id;
      Values.Value v;
      TupleTypes_TTypeAbsyn_PathOption tp;
      InteractiveVariableList rest;
    case (IVAR(id,v,tp) :: rest,env)
      equation 
        (_,_,_) = Lookup.lookup_var(env, Exp.CREF_IDENT(id,{}));
        env_1 = Env.update_frame_v(env, 
          Types.VAR(id,Types.ATTR(false,SCode.RW(),SCode.VAR(),Absyn.BIDIR()),
          false,tp,Types.VALBOUND(v)), false);
        env_2 = add_vars_to_env(rest, env_1); then env_2;
    case (IVAR(id,v,tp) :: rest,env)
      equation 
        failure((_,_,_) = Lookup.lookup_var(env, Exp.CREF_IDENT(id,{})));
        env_1 = Env.extend_frame_v(env, 
          Types.VAR(id,Types.ATTR(false,SCode.RW(),SCode.VAR(),Absyn.BIDIR()),
          false,tp,Types.VALBOUND(v)), NONE, false);
        env_2 = add_vars_to_env(rest, env_1); then env_2;
    case ({},env) then env; 
  end matchcontinue;
end add_vars_to_env;

protected function evaluate_graphical_api
  input InteractiveStmts in_interactivestmts;
  input InteractiveSymbolTable in_interactivesymboltable;
  output String out_string;
  output InteractiveSymbolTable out_interactivesymboltable;
algorithm 
  (out_string,out_interactivesymboltable):=
  matchcontinue (in_interactivestmts,in_interactivesymboltable)
    local
      type SCode_ClassList = list<SCode.Class>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOption = tuple<Absyn.Path,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOptionList = list<TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOption>;
      type Absyn_NamedArgList = list<Absyn.NamedArg>;
      String resstr,filename,ident,name,top_names_str,str,resstr_1,res,s1;
      Absyn.ComponentRef class_,subident,cr,path,tp,model_,c1,c2,old_cname,new_cname;
      InteractiveSymbolTable st,newst;
      Absyn.Program p,newp,p_1,p1,newp_1;
      SCode_ClassList s,s_1;
      InstantiatedClassList ic;
      InteractiveVariableList iv;
      TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOptionList cf;
      Absyn.Modification mod;
      Absyn.Exp exp;
      Absyn.Path path_1,wpath;
      Integer rest,count,n;
      Absyn_NamedArgList nargs;
      Boolean b1,b2;
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getSourceFile",_),Absyn.FUNCTIONARGS({Absyn.CREF(class_)},{})))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        resstr = get_source_file(class_, p); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("setSourceFile",_),Absyn.FUNCTIONARGS({Absyn.CREF(class_),Absyn.STRING(filename)},{})))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        (resstr,newp) = set_source_file(class_, filename, p); then (resstr,SYMBOLTABLE(newp,s,ic,iv,cf));
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("setExtendsModifierValue",_),Absyn.FUNCTIONARGS({Absyn.CREF(class_),Absyn.CREF(Absyn.CREF_QUAL(ident,_,subident)),Absyn.CODE(Absyn.C_MODIFICATION(mod))},{})))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        (newp,resstr) = set_extends_modifier_value(class_, Absyn.CREF_IDENT(ident,{}), subident, mod, p); then (resstr,SYMBOLTABLE(newp,s,ic,iv,cf));
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getExtendsModifierNames",_),Absyn.FUNCTIONARGS({Absyn.CREF(class_),Absyn.CREF(ident)},{})))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      local Absyn.ComponentRef ident;
      equation 
        resstr = get_extends_modifier_names(class_, ident, p); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getExtendsModifierValue",_),Absyn.FUNCTIONARGS({Absyn.CREF(class_),Absyn.CREF(Absyn.CREF_QUAL(ident,_,subident))},{})))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        resstr = get_extends_modifier_value(class_, Absyn.CREF_IDENT(ident,{}), subident, p); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getComponentModifierNames",_),Absyn.FUNCTIONARGS({Absyn.CREF(class_),Absyn.CREF(ident)},{})))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      local Absyn.ComponentRef ident;
      equation 
        resstr = get_component_modifier_names(class_, ident, p); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getComponentModifierValue",_),Absyn.FUNCTIONARGS({Absyn.CREF(class_),Absyn.CREF(Absyn.CREF_QUAL(ident,_,subident))},{})))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        resstr = get_component_modifier_value(class_, Absyn.CREF_IDENT(ident,{}), subident, p); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getComponentModifierValue",_),Absyn.FUNCTIONARGS({Absyn.CREF(class_),Absyn.CREF(Absyn.CREF_IDENT(ident,_))},{})))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        resstr = get_parameter_value(class_, Absyn.CREF_IDENT(ident,{}), p); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("setComponentModifierValue",_),Absyn.FUNCTIONARGS({Absyn.CREF(class_),Absyn.CREF((ident = Absyn.CREF_IDENT(_,_))),Absyn.CODE(Absyn.C_MODIFICATION(Absyn.CLASSMOD(_,SOME(exp))))},{})))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      local Absyn.ComponentRef ident;
      equation 
        (p_1,resstr) = set_parameter_value(class_, ident, exp, p) " expressions "; then (resstr,SYMBOLTABLE(p_1,s,ic,iv,cf));
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("setComponentModifierValue",_),Absyn.FUNCTIONARGS({Absyn.CREF(class_),Absyn.CREF((ident = Absyn.CREF_IDENT(_,_))),Absyn.CODE(Absyn.C_MODIFICATION((mod = Absyn.CLASSMOD({},NONE))))},{})))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      local Absyn.ComponentRef ident;
      equation 
        (p_1,resstr) = set_component_modifier(class_, ident, Absyn.CREF_IDENT("",{}), mod, p) " special case for clearing modifier "; then (resstr,SYMBOLTABLE(p_1,s,ic,iv,cf));
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("setComponentModifierValue",_),Absyn.FUNCTIONARGS({Absyn.CREF(class_),Absyn.CREF(Absyn.CREF_QUAL(ident,_,subident)),Absyn.CODE(Absyn.C_MODIFICATION(mod))},{})))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        (p_1,resstr) = set_component_modifier(class_, Absyn.CREF_IDENT(ident,{}), subident, mod, p); then (resstr,SYMBOLTABLE(p_1,s,ic,iv,cf));
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getParameterValue",_),Absyn.FUNCTIONARGS({Absyn.CREF(class_),Absyn.CREF(ident)},{})))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      local Absyn.ComponentRef ident;
      equation 
        resstr = get_parameter_value(class_, ident, p); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("setParameterValue",_),Absyn.FUNCTIONARGS({Absyn.CREF(class_),Absyn.CREF(ident),exp},{})))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      local Absyn.ComponentRef ident;
      equation 
        (p_1,resstr) = set_parameter_value(class_, ident, exp, p); then (resstr,SYMBOLTABLE(p_1,s,ic,iv,cf));
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getParameterNames",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr)},{})))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        resstr = get_parameter_names(cr, p); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("createModel",_),Absyn.FUNCTIONARGS({Absyn.CREF(Absyn.CREF_IDENT(name,_))},{})))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        newp = update_program(
          Absyn.PROGRAM(
          {
          Absyn.CLASS(name,false,false,false,Absyn.R_MODEL(),
          Absyn.PARTS({Absyn.PUBLIC({})},NONE),"NoFile")},Absyn.TOP()), p);
        newst = SYMBOLTABLE(newp,s,ic,iv,cf); then ("true",newst);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("createModel",_),Absyn.FUNCTIONARGS({Absyn.CREF((path = Absyn.CREF_QUAL(_,_,_)))},{})))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        path_1 = Absyn.cref_to_path(path);
        name = Absyn.path_last_ident(path_1);
        wpath = Absyn.strip_last(path_1);
        newp = update_program(
          Absyn.PROGRAM(
          {
          Absyn.CLASS(name,false,false,false,Absyn.R_MODEL(),
          Absyn.PARTS({Absyn.PUBLIC({})},NONE),"NoFile")},Absyn.WITHIN(wpath)), p);
        newst = SYMBOLTABLE(newp,s,ic,iv,cf); then ("true",newst);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("newModel",_),Absyn.FUNCTIONARGS({Absyn.CREF(Absyn.CREF_IDENT(name,_)),Absyn.CREF(cr)},{})))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      local Absyn.Path path;
      equation 
        path = Absyn.cref_to_path(cr);
        newp = update_program(
          Absyn.PROGRAM(
          {
          Absyn.CLASS(name,false,false,false,Absyn.R_MODEL(),
          Absyn.PARTS({Absyn.PUBLIC({})},NONE),"NoFile")},Absyn.WITHIN(path)), p);
        newst = SYMBOLTABLE(newp,s,ic,iv,cf);
        resstr = string_append(name, "\n"); then ("true",newst);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("loadFileInteractive",_),Absyn.FUNCTIONARGS({Absyn.STRING(name)},{})))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        0 = System.regular_file_exist(name);
        p1 = Parser.parse(name);
        newp = update_program(p1, p);
        top_names_str = get_top_classnames(p1); then (top_names_str,SYMBOLTABLE(newp,s,ic,iv,cf));
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("loadFileInteractive",_),Absyn.FUNCTIONARGS({Absyn.STRING(name)},{})))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf))) "* it the rule above have failed then check if file exists *  without this omc crashes "
      equation 
        rest = System.regular_file_exist(name);
        (rest > 0) = true; then ("error",st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("loadFileInteractive",_),Absyn.FUNCTIONARGS({Absyn.STRING(name)},{})))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf))) "check if the parse went wrong"
      equation 
        failure(p1 = Parser.parse(name)); then ("error",st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("deleteClass",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr)},{})))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        (resstr,newp) = delete_class(cr, p); then (resstr,SYMBOLTABLE(newp,s,ic,iv,cf));
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("addComponent",_),Absyn.FUNCTIONARGS({Absyn.CREF(Absyn.CREF_IDENT(name,_)),Absyn.CREF(tp),Absyn.CREF(model_)},nargs)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        (newp,resstr) = add_component(name, tp, model_, nargs, p);
        str = Print.get_string();
        resstr_1 = string_append(resstr, str); then ("true",SYMBOLTABLE(newp,s,ic,iv,cf));
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("updateComponent",_),Absyn.FUNCTIONARGS({Absyn.CREF(Absyn.CREF_IDENT(name,_)),Absyn.CREF(tp),Absyn.CREF(model_)},nargs)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        (newp,res) = update_component(name, tp, model_, nargs, p) "delete_component(name,model,p) => (newp,resstr) &
	add_component(name,tp,model,nargs,newp) => (newp2,resstr)"; then (res,SYMBOLTABLE(newp,s,ic,iv,cf));
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("deleteComponent",_),Absyn.FUNCTIONARGS({Absyn.CREF(Absyn.CREF_IDENT(name,_)),Absyn.CREF(model_)},nargs)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        (newp,resstr) = delete_component(name, model_, p);
        str = Print.get_string();
        resstr_1 = string_append(resstr, str); then ("true",SYMBOLTABLE(newp,s,ic,iv,cf));
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("addClassAnnotation",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr)},nargs)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        newp = add_class_annotation(cr, nargs, p);
        newst = SYMBOLTABLE(newp,s,ic,iv,cf); then ("true",newst);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getComponentCount",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        count = get_component_count(cr, p);
        resstr = int_string(count); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getNthComponent",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr),Absyn.INTEGER(n)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        resstr = get_nth_component(cr, p, n); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getComponents",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        resstr = get_components(cr, p); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getComponentAnnotations",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        resstr = get_component_annotations(cr, p); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getNthComponentAnnotation",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr),Absyn.INTEGER(n)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        resstr = get_nth_component_annotation(cr, p, n); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getNthComponentModification",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr),Absyn.INTEGER(n)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        resstr = get_nth_component_modification(cr, p, n); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getInheritanceCount",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        count = get_inheritance_count(cr, p);
        resstr = int_string(count); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getNthInheritedClass",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr),Absyn.INTEGER(n)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        resstr = get_nth_inherited_class(cr, n, p); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getConnectionCount",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        resstr = get_connection_count(cr, p); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getNthConnection",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr),Absyn.INTEGER(n)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        resstr = get_nth_connection(cr, p, n); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("addConnection",_),Absyn.FUNCTIONARGS({Absyn.CREF(c1),Absyn.CREF(c2),Absyn.CREF(cr)},nargs)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        (resstr,newp) = add_connection(cr, c1, c2, nargs, p); then (resstr,SYMBOLTABLE(newp,s,ic,iv,cf));
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("deleteConnection",_),Absyn.FUNCTIONARGS({Absyn.CREF(c1),Absyn.CREF(c2),Absyn.CREF(cr)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        (resstr,newp) = delete_connection(cr, c1, c2, p); then (resstr,SYMBOLTABLE(newp,s,ic,iv,cf));
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("updateConnection",_),Absyn.FUNCTIONARGS({Absyn.CREF(c1),Absyn.CREF(c2),Absyn.CREF(cr)},nargs)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        (s1,newp) = delete_connection(cr, c1, c2, p);
        (resstr,newp_1) = add_connection(cr, c1, c2, nargs, newp); then (resstr,SYMBOLTABLE(newp_1,s,ic,iv,cf));
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getNthConnectionAnnotation",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr),Absyn.INTEGER(n)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        resstr = get_nth_connection_annotation(cr, p, n); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getConnectorCount",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        resstr = get_connector_count(cr, p); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getNthConnector",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr),Absyn.INTEGER(n)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        resstr = get_nth_connector(cr, p, n); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getNthConnectorIconAnnotation",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr),Absyn.INTEGER(n)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        resstr = get_nth_connector_icon_annotation(cr, p, n); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getIconAnnotation",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      local Absyn.Path path;
      equation 
        path = Absyn.cref_to_path(cr);
        resstr = get_icon_annotation(path, p); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getDiagramAnnotation",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      local Absyn.Path path;
      equation 
        path = Absyn.cref_to_path(cr);
        resstr = get_diagram_annotation(path, p); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getPackages",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      local Absyn.Path path;
      equation 
        path = Absyn.cref_to_path(cr);
        resstr = get_packages_in_path(path, p); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getPackages",_),Absyn.FUNCTIONARGS({},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        s1 = get_top_packages(p);
        resstr = string_append(s1, "\n"); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getClassNames",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      local Absyn.Path path;
      equation 
        path = Absyn.cref_to_path(cr);
        resstr = get_classnames_in_path(path, p); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("getClassNames",_),Absyn.FUNCTIONARGS({},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        resstr = get_top_classnames(p); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("isPrimitive",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        b1 = is_primitive(cr, p);
        resstr = Util.bool_string(b1); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("isType",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        b1 = is_type(cr, p);
        resstr = Util.bool_string(b1); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("isConnector",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        b1 = is_connector(cr, p);
        resstr = Util.bool_string(b1); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("isModel",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        b1 = is_model(cr, p);
        resstr = Util.bool_string(b1); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("isRecord",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        b1 = is_record(cr, p);
        resstr = Util.bool_string(b1); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("isBlock",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        b1 = is_block(cr, p);
        resstr = Util.bool_string(b1); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("isFunction",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        b1 = is_function(cr, p);
        resstr = Util.bool_string(b1); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("isPackage",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        b1 = is_package(cr, p);
        resstr = Util.bool_string(b1); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("isClass",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        b1 = is_class(cr, p);
        resstr = Util.bool_string(b1); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("isParameter",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr),Absyn.CREF(class_)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        b1 = is_parameter(cr, class_, p);
        resstr = Util.bool_string(b1); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("isProtected",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr),Absyn.CREF(class_)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        b1 = is_protected(cr, class_, p);
        resstr = Util.bool_string(b1); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("isConstant",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr),Absyn.CREF(class_)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        b1 = is_constant(cr, class_, p);
        resstr = Util.bool_string(b1); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("existClass",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        b1 = exist_class(cr, p);
        resstr = Util.bool_string(b1); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("existModel",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      local Boolean res;
      equation 
        b1 = exist_class(cr, p);
        b2 = is_model(cr, p);
        res = bool_and(b1, b2);
        resstr = Util.bool_string(res); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("existPackage",_),Absyn.FUNCTIONARGS({Absyn.CREF(cr)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      local Boolean res;
      equation 
        b1 = exist_class(cr, p);
        b2 = is_package(cr, p);
        res = bool_and(b1, b2);
        resstr = Util.bool_string(res); then (resstr,st);
    case (ISTMTS({IEXP(Absyn.CALL(Absyn.CREF_IDENT("renameClass",_),Absyn.FUNCTIONARGS({Absyn.CREF(old_cname),Absyn.CREF(new_cname)},_)))},_),(st = SYMBOLTABLE(p,s,ic,iv,cf)))
      equation 
        p_1 = rename_class(p, old_cname, new_cname) " For now, renaming a class clears all caches...   Substantial analysis required to find out what to keep in cache and 
	  what must be thrown out ";
        s_1 = SCode.elaborate(p_1); then ("OK",SYMBOLTABLE(p_1,s_1,{},{},{}));
  end matchcontinue;
end evaluate_graphical_api;

protected function is_parameter_element "* function: is_parameter_element
 ** Returns true if Element is a component of variability parameter, false otherwise.
*"
  input Absyn.Element in_element;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_element)
    case (Absyn.ELEMENT(_,_,_,_,Absyn.COMPONENTS(Absyn.ATTR(_,Absyn.PARAM(),_,_),_,_),_,_,_)) then true; 
    case (_) then false; 
  end matchcontinue;
end is_parameter_element;

protected function get_parameter_names
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_componentref,in_program)
    local
      type Absyn_ElementList = list<Absyn.Element>;
      type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
      type Absyn_ComponentItemListList = list<Absyn_ComponentItemList>;
      type StringList = list<String>;
      Absyn.Path p_class;
      Absyn.Class cdef;
      Absyn_ElementList comps,comps_1;
      Absyn_ComponentItemListList compelts;
      Absyn_ComponentItemList compelts_1;
      StringList names;
      String res,res_1;
      Absyn.ComponentRef class_;
      Absyn.Program p;
    case (class_,p) " class "
      equation 
        p_class = Absyn.cref_to_path(class_);
        cdef = get_pathed_class_in_program(p_class, p);
        comps = get_components_in_class(cdef);
        comps_1 = Util.list_select(comps, is_parameter_element);
        compelts = Util.list_map(comps_1, get_componentitems_in_element);
        compelts_1 = Util.list_flatten(compelts);
        names = Util.list_map(compelts_1, get_componentitem_name);
        res = Util.string_delimit_list(names, ", ");
        res_1 = Util.string_append_list({"{",res,"}"}); then res_1;
    case (_,_) then "Error"; 
  end matchcontinue;
end get_parameter_names;

protected function get_class_env "* function: get_class_env
 ** Retrieves the environment of the class, including the frame of the class
 ** itself by partially instantiating it.
*"
  input Absyn.Program p;
  input Absyn.Path p_class;
  output Env.Env env_2;
protected 
  type SCode_ClassList = list<SCode.Class>;
  type Env_FrameList = list<Env.Frame>;
  SCode_ClassList p_1;
  Env_FrameList env,env_1,env2,env_2;
  SCode.Class cl;
  String id;
  Boolean encflag;
  SCode.Restriction restr;
  ClassInf.State ci_state;
algorithm 
  p_1 := SCode.elaborate(p);
  env := Inst.make_env_from_program(p_1, Absyn.IDENT(""));
  (cl = SCode.CLASS(id,_,encflag,restr,_),env_1) := Lookup.lookup_class(env, p_class, false);
  env2 := Env.open_scope(env_1, encflag, SOME(id));
  ci_state := ClassInf.start(restr, id);
  (env_2,_) := Inst.partial_inst_class_in(env2, Types.NOMOD(), Prefix.NOPRE, {}, ci_state, cl, 
          false, {});
end get_class_env;

protected function get_source_file
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_componentref,in_program)
    local
      Absyn.Path p_class;
      Absyn.Class cdef;
      String filename;
      Absyn.ComponentRef class_;
      Absyn.Program p;
    case (class_,p) " class "
      equation 
        p_class = Absyn.cref_to_path(class_);
        cdef = get_pathed_class_in_program(p_class, p);
        filename = Absyn.class_filename(cdef); then filename;
    case (_,_) then "Error"; 
  end matchcontinue;
end get_source_file;

protected function set_source_file " filename "
  input Absyn.ComponentRef in_componentref;
  input String in_string;
  input Absyn.Program in_program;
  output String out_string;
  output Absyn.Program out_program;
algorithm 
  (out_string,out_program):=
  matchcontinue (in_componentref,in_string,in_program)
    local
      Absyn.Path p_class;
      Absyn.Class cdef,cdef_1;
      Absyn.Within within_;
      Absyn.Program newp,p;
      Absyn.ComponentRef class_;
      String filename;
    case (class_,filename,p) " class   filename "
      equation 
        p_class = Absyn.cref_to_path(class_);
        cdef = get_pathed_class_in_program(p_class, p);
        within_ = build_within(p_class);
        cdef_1 = Absyn.set_class_filename(cdef, filename);
        newp = update_program(Absyn.PROGRAM({cdef_1},within_), p); then ("Ok",newp);
    case (_,_,p) then ("Error",p); 
  end matchcontinue;
end set_source_file;

protected function set_extends_modifier_value " inherit class
   subident"
  input Absyn.ComponentRef in_componentref1;
  input Absyn.ComponentRef in_componentref2;
  input Absyn.ComponentRef in_componentref3;
  input Absyn.Modification in_modification4;
  input Absyn.Program in_program5;
  output Absyn.Program out_program;
  output String out_string;
algorithm 
  (out_program,out_string):=
  matchcontinue (in_componentref1,in_componentref2,in_componentref3,in_modification4,in_program5)
    local
      type Env_FrameList = list<Env.Frame>;
      Absyn.Path p_class,inherit_class;
      Absyn.Within within_;
      Absyn.Class cdef,cdef_1;
      Env_FrameList env;
      Absyn.Program newp,p;
      Absyn.ComponentRef class_,inheritclass,subident;
      Absyn.Modification mod;
    case (class_,inheritclass,subident,mod,p) " class   inherit class  subident"
      equation 
        p_class = Absyn.cref_to_path(class_);
        inherit_class = Absyn.cref_to_path(inheritclass);
        within_ = build_within(p_class);
        cdef = get_pathed_class_in_program(p_class, p);
        env = get_class_env(p, p_class);
        cdef_1 = set_extends_submodifier_in_class(cdef, inherit_class, subident, mod, env);
        newp = update_program(Absyn.PROGRAM({cdef_1},within_), p); then (newp,"Ok");
    case (_,_,_,_,p) then (p,"Error"); 
  end matchcontinue;
end set_extends_modifier_value;

protected function set_extends_submodifier_in_class " inherit_name
   submodifier"
  input Absyn.Class in_class;
  input Absyn.Path in_path;
  input Absyn.ComponentRef in_componentref;
  input Absyn.Modification in_modification;
  input Env.Env in_env;
  output Absyn.Class out_class;
algorithm 
  out_class:=
  matchcontinue (in_class,in_path,in_componentref,in_modification,in_env)
    local
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      type StringOption = Option<String>;
      type Env_FrameList = list<Env.Frame>;
      Absyn_ClassPartList parts_1,parts;
      String id,file;
      Boolean p,f,e;
      Absyn.Restriction r;
      StringOption cmt;
      Absyn.Path inherit_name;
      Absyn.ComponentRef submod;
      Absyn.Modification mod;
      Env_FrameList env;
    case (Absyn.CLASS(id,p,f,e,r,Absyn.PARTS(parts,cmt),file),inherit_name,submod,mod,env) " inherit_name  submodifier"
      equation 
        parts_1 = set_extends_submodifier_in_classparts(parts, inherit_name, submod, mod, env); then Absyn.CLASS(id,p,f,e,r,Absyn.PARTS(parts_1,cmt),file);
  end matchcontinue;
end set_extends_submodifier_in_class;

protected function set_extends_submodifier_in_classparts " inherit_name
   submodifier"
  input Absyn_ClassPartList in_absyn_classpartlist;
  input Absyn.Path in_path;
  input Absyn.ComponentRef in_componentref;
  input Absyn.Modification in_modification;
  input Env.Env in_env;
  output Absyn_ClassPartList out_absyn_classpartlist;
protected 
  type Absyn_ClassPartList = list<Absyn.ClassPart>;
algorithm 
  out_absyn_classpartlist:=
  matchcontinue (in_absyn_classpartlist,in_path,in_componentref,in_modification,in_env)
    local
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      type Env_FrameList = list<Env.Frame>;
      Absyn_ClassPartList res,rest;
      Absyn_ElementItemList elts_1,elts;
      Absyn.Path inherit;
      Absyn.ComponentRef submod;
      Absyn.Modification mod;
      Env_FrameList env;
      Absyn.ClassPart elt;
    case ({},_,_,_,_) then {}; 
    case (Absyn.PUBLIC(elts) :: rest,inherit,submod,mod,env)
      equation 
        res = set_extends_submodifier_in_classparts(rest, inherit, submod, mod, env);
        elts_1 = set_extends_submodifier_in_elementitems(elts, inherit, submod, mod, env); then Absyn.PUBLIC(elts_1) :: res;
    case (Absyn.PROTECTED(elts) :: rest,inherit,submod,mod,env)
      equation 
        res = set_extends_submodifier_in_classparts(rest, inherit, submod, mod, env);
        elts_1 = set_extends_submodifier_in_elementitems(elts, inherit, submod, mod, env); then Absyn.PROTECTED(elts_1) :: res;
    case (elt :: rest,inherit,submod,mod,env)
      equation 
        res = set_extends_submodifier_in_classparts(rest, inherit, submod, mod, env); then elt :: res;
  end matchcontinue;
end set_extends_submodifier_in_classparts;

protected function set_extends_submodifier_in_elementitems "inherit_name
  submodifier"
  input Absyn_ElementItemList in_absyn_elementitemlist;
  input Absyn.Path in_path;
  input Absyn.ComponentRef in_componentref;
  input Absyn.Modification in_modification;
  input Env.Env in_env;
  output Absyn_ElementItemList out_absyn_elementitemlist;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
algorithm 
  out_absyn_elementitemlist:=
  matchcontinue (in_absyn_elementitemlist,in_path,in_componentref,in_modification,in_env)
    local
      type Env_FrameList = list<Env.Frame>;
      Absyn_ElementItemList res,rest;
      Absyn.Element elt_1,elt;
      Absyn.Path inherit;
      Absyn.ComponentRef submod;
      Absyn.Modification mod;
      Env_FrameList env;
    case ({},_,_,_,_) then {}; 
    case (Absyn.ELEMENTITEM(elt) :: rest,inherit,submod,mod,env)
      equation 
        res = set_extends_submodifier_in_elementitems(rest, inherit, submod, mod, env);
        elt_1 = set_extends_submodifier_in_element(elt, inherit, submod, mod, env); then Absyn.ELEMENTITEM(elt_1) :: res;
    case (elt :: rest,inherit,submod,mod,env)
      local Absyn.ElementItem elt;
      equation 
        res = set_extends_submodifier_in_elementitems(rest, inherit, submod, mod, env); then elt :: res;
  end matchcontinue;
end set_extends_submodifier_in_elementitems;

protected function set_extends_submodifier_in_element "inherit_name
  submodifier"
  input Absyn.Element in_element;
  input Absyn.Path in_path;
  input Absyn.ComponentRef in_componentref;
  input Absyn.Modification in_modification;
  input Env.Env in_env;
  output Absyn.Element out_element;
algorithm 
  out_element:=
  matchcontinue (in_element,in_path,in_componentref,in_modification,in_env)
    local
      type Absyn_ElementArgList = list<Absyn.ElementArg>;
      type Absyn_ElementSpecOption = Option<Absyn.ElementSpec>;
      type Env_FrameList = list<Env.Frame>;
      Boolean f,r;
      Absyn.InnerOuter i;
      String n,s;
      Absyn.Path path,inherit,path_1;
      Absyn_ElementArgList eargs,eargs_1;
      Integer l;
      Absyn_ElementSpecOption constr;
      Absyn.ComponentRef submod;
      Env_FrameList env;
      Absyn.Modification mod;
      Absyn.Element elt;
    case (Absyn.ELEMENT(f,r,i,n,Absyn.EXTENDS(path,eargs),s,l,constr),inherit,submod,Absyn.CLASSMOD({},NONE),env) then Absyn.ELEMENT(f,r,i,n,Absyn.EXTENDS(path,{}),s,l,constr);  "inherit_name submodifier"
    case (Absyn.ELEMENT(f,r,i,n,Absyn.EXTENDS(path,eargs),s,l,constr),inherit,submod,mod,env)
      equation 
        path_1 = Inst.make_fully_qualified(env, path);
        true = ModUtil.path_equal(inherit, path_1);
        eargs_1 = set_submodifier_in_elementargs(eargs, submod, mod); then Absyn.ELEMENT(f,r,i,n,Absyn.EXTENDS(path,eargs_1),s,l,constr);
    case (elt,_,_,_,_) then elt; 
  end matchcontinue;
end set_extends_submodifier_in_element;

protected function get_extends_modifier_value " ident
   subident"
  input Absyn.ComponentRef in_componentref1;
  input Absyn.ComponentRef in_componentref2;
  input Absyn.ComponentRef in_componentref3;
  input Absyn.Program in_program4;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_componentref1,in_componentref2,in_componentref3,in_program4)
    local
      type Env_FrameList = list<Env.Frame>;
      type Absyn_ElementSpecList = list<Absyn.ElementSpec>;
      type Absyn_ElementArgList = list<Absyn.ElementArg>;
      Absyn.Path p_class,name,extpath;
      Absyn.Class cdef;
      Env_FrameList env;
      Absyn_ElementSpecList exts,exts_1;
      Absyn_ElementArgList extmod;
      Absyn.Modification mod;
      String res;
      Absyn.ComponentRef class_,inherit_name,subident;
      Absyn.Program p;
    case (class_,inherit_name,subident,p) "class   ident  subident"
      equation 
        p_class = Absyn.cref_to_path(class_);
        name = Absyn.cref_to_path(inherit_name);
        cdef = get_pathed_class_in_program(p_class, p);
        env = get_class_env(p, p_class);
        exts = get_extends_elementspec_in_class(cdef);
        exts_1 = Util.list_map_1(exts, make_extends_fully_qualified, env);
        {Absyn.EXTENDS(extpath,extmod)} = Util.list_select_1(exts_1, name, extends_elementspec_named);
        mod = get_modification_value(extmod, subident);
        res = Dump.unparse_modification_str(mod); then res;
    case (_,_,_,_) then "Error"; 
  end matchcontinue;
end get_extends_modifier_value;

protected function make_extends_fully_qualified
  input Absyn.ElementSpec in_elementspec;
  input Env.Env in_env;
  output Absyn.ElementSpec out_elementspec;
algorithm 
  out_elementspec:=
  matchcontinue (in_elementspec,in_env)
    local
      type Absyn_ElementArgList = list<Absyn.ElementArg>;
      type Env_FrameList = list<Env.Frame>;
      Absyn.Path path_1,path;
      Absyn_ElementArgList earg;
      Env_FrameList env;
    case (Absyn.EXTENDS(path,earg),env)
      equation 
        path_1 = Inst.make_fully_qualified(env, path); then Absyn.EXTENDS(path_1,earg);
  end matchcontinue;
end make_extends_fully_qualified;

protected function get_extends_modifier_names " inherited class "
  input Absyn.ComponentRef in_componentref1;
  input Absyn.ComponentRef in_componentref2;
  input Absyn.Program in_program3;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_componentref1,in_componentref2,in_program3)
    local
      type Absyn_ElementSpecList = list<Absyn.ElementSpec>;
      type Env_FrameList = list<Env.Frame>;
      type Absyn_ElementArgList = list<Absyn.ElementArg>;
      type StringList = list<String>;
      Absyn.Path p_class,name,extpath;
      Absyn.Class cdef;
      Absyn_ElementSpecList exts,exts_1;
      Env_FrameList env;
      Absyn_ElementArgList extmod;
      StringList res;
      String res_1,res_2;
      Absyn.ComponentRef class_,inherit_name;
      Absyn.Program p;
    case (class_,inherit_name,p) " class   inherited class "
      equation 
        p_class = Absyn.cref_to_path(class_);
        name = Absyn.cref_to_path(inherit_name);
        cdef = get_pathed_class_in_program(p_class, p);
        exts = get_extends_elementspec_in_class(cdef);
        env = get_class_env(p, p_class);
        exts_1 = Util.list_map_1(exts, make_extends_fully_qualified, env);
        {Absyn.EXTENDS(extpath,extmod)} = Util.list_select_1(exts_1, name, extends_elementspec_named);
        res = get_modification_names(extmod);
        res_1 = Util.string_delimit_list(res, ", ");
        res_2 = Util.string_append_list({"{",res_1,"}"}); then res_2;
    case (_,_,_) then "Error"; 
  end matchcontinue;
end get_extends_modifier_names;

protected function extends_elementspec_named
  input Absyn.ElementSpec in_elementspec;
  input Absyn.Path in_path;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_elementspec,in_path)
    local
      Boolean res;
      Absyn.Path extpath,path;
    case (Absyn.EXTENDS(extpath,_),path)
      equation 
        res = ModUtil.path_equal(path, extpath); then res;
  end matchcontinue;
end extends_elementspec_named;

protected function extends_name "* function extends_name
 ** Return the class name of an EXTENDS element spec.
 *"
  input Absyn.ElementSpec in_elementspec;
  output Absyn.Path out_path;
algorithm 
  out_path:=
  matchcontinue (in_elementspec)
    local Absyn.Path path;
    case (Absyn.EXTENDS(path,_)) then path; 
  end matchcontinue;
end extends_name;

protected function get_extends_elementspec_in_class
  input Absyn.Class in_class;
  output Absyn_ElementSpecList out_absyn_elementspeclist;
protected 
  type Absyn_ElementSpecList = list<Absyn.ElementSpec>;
algorithm 
  out_absyn_elementspeclist:=
  matchcontinue (in_class)
    local
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      Absyn_ElementSpecList ext;
      Absyn_ClassPartList parts;
    case (Absyn.CLASS(_,_,_,_,_,Absyn.PARTS(parts,_),_))
      equation 
        ext = get_extends_elementspec_in_classparts(parts); then ext;
    case (_) then {}; 
  end matchcontinue;
end get_extends_elementspec_in_class;

protected function get_extends_elementspec_in_classparts
  input Absyn_ClassPartList in_absyn_classpartlist;
  output Absyn_ElementSpecList out_absyn_elementspeclist;
protected 
  type Absyn_ClassPartList = list<Absyn.ClassPart>;
  type Absyn_ElementSpecList = list<Absyn.ElementSpec>;
algorithm 
  out_absyn_elementspeclist:=
  matchcontinue (in_absyn_classpartlist)
    local
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      Absyn_ElementSpecList lst1,lst2,res;
      Absyn_ElementItemList elts;
      Absyn_ClassPartList rest;
      Absyn.ClassPart elt;
    case ({}) then {}; 
    case (Absyn.PUBLIC(elts) :: rest)
      equation 
        lst1 = get_extends_elementspec_in_classparts(rest);
        lst2 = get_extends_elementspec_in_elementitems(elts);
        res = list_append(lst1, lst2); then res;
    case (Absyn.PROTECTED(elts) :: rest)
      equation 
        lst1 = get_extends_elementspec_in_classparts(rest);
        lst2 = get_extends_elementspec_in_elementitems(elts);
        res = list_append(lst1, lst2); then res;
    case (elt :: rest)
      equation 
        res = get_extends_elementspec_in_classparts(rest); then res;
  end matchcontinue;
end get_extends_elementspec_in_classparts;

protected function get_extends_elementspec_in_elementitems
  input Absyn_ElementItemList in_absyn_elementitemlist;
  output Absyn_ElementSpecList out_absyn_elementspeclist;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
  type Absyn_ElementSpecList = list<Absyn.ElementSpec>;
algorithm 
  out_absyn_elementspeclist:=
  matchcontinue (in_absyn_elementitemlist)
    local
      Absyn.ElementSpec elt;
      Absyn_ElementSpecList res;
      Absyn_ElementItemList rest;
    case ({}) then {}; 
    case (Absyn.ELEMENTITEM(elt) :: rest)
      equation 
        elt = get_extends_elementspec_in_element(elt) " Bug in RML. If the two premisses below are in swapped order
	 * the compiler enters infinite loop (but no stack overflow) ";
        res = get_extends_elementspec_in_elementitems(rest); then elt :: res;
    case (_ :: rest)
      equation 
        res = get_extends_elementspec_in_elementitems(rest); then res;
  end matchcontinue;
end get_extends_elementspec_in_elementitems;

protected function get_extends_elementspec_in_element
  input Absyn.Element in_element;
  output Absyn.ElementSpec out_elementspec;
algorithm 
  out_elementspec:=
  matchcontinue (in_element)
    local Absyn.ElementSpec ext;
    case (Absyn.ELEMENT(_,_,_,_,(ext = Absyn.EXTENDS(_,_)),_,_,_)) then ext; 
  end matchcontinue;
end get_extends_elementspec_in_element;

protected function set_component_modifier " variable name 
   submodifier name"
  input Absyn.ComponentRef in_componentref1;
  input Absyn.ComponentRef in_componentref2;
  input Absyn.ComponentRef in_componentref3;
  input Absyn.Modification in_modification4;
  input Absyn.Program in_program5;
  output Absyn.Program out_program;
  output String out_string;
algorithm 
  (out_program,out_string):=
  matchcontinue (in_componentref1,in_componentref2,in_componentref3,in_modification4,in_program5)
    local
      Absyn.Path p_class;
      Absyn.Within within_;
      Absyn.Class cdef,cdef_1;
      Absyn.Program newp,p;
      Absyn.ComponentRef class_,subident;
      String varname;
      Absyn.Modification mod;
    case (class_,Absyn.CREF_IDENT(varname,_),subident,mod,p) " class   variable name   submodifier name"
      equation 
        p_class = Absyn.cref_to_path(class_);
        within_ = build_within(p_class);
        cdef = get_pathed_class_in_program(p_class, p);
        cdef_1 = set_component_submodifier_in_class(cdef, varname, subident, mod);
        newp = update_program(Absyn.PROGRAM({cdef_1},within_), p); then (newp,"Ok");
    case (_,_,_,_,p) then (p,"Error"); 
  end matchcontinue;
end set_component_modifier;

protected function set_component_submodifier_in_class " component name
  subvariable path"
  input Absyn.Class in_class;
  input Absyn.Ident in_ident;
  input Absyn.ComponentRef in_componentref;
  input Absyn.Modification in_modification;
  output Absyn.Class out_class;
algorithm 
  out_class:=
  matchcontinue (in_class,in_ident,in_componentref,in_modification)
    local
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      type StringOption = Option<String>;
      Absyn_ClassPartList parts_1,parts;
      String id,file,varname;
      Boolean p,f,e;
      Absyn.Restriction r;
      StringOption cmt;
      Absyn.ComponentRef submodident;
      Absyn.Modification mod;
    case (Absyn.CLASS(id,p,f,e,r,Absyn.PARTS(parts,cmt),file),varname,submodident,mod) " component name subvariable path"
      equation 
        parts_1 = set_component_submodifier_in_classparts(parts, varname, submodident, mod); then Absyn.CLASS(id,p,f,e,r,Absyn.PARTS(parts_1,cmt),file);
  end matchcontinue;
end set_component_submodifier_in_class;

protected function set_component_submodifier_in_classparts " component name
   subvariable path"
  input Absyn_ClassPartList in_absyn_classpartlist;
  input Absyn.Ident in_ident;
  input Absyn.ComponentRef in_componentref;
  input Absyn.Modification in_modification;
  output Absyn_ClassPartList out_absyn_classpartlist;
protected 
  type Absyn_ClassPartList = list<Absyn.ClassPart>;
algorithm 
  out_absyn_classpartlist:=
  matchcontinue (in_absyn_classpartlist,in_ident,in_componentref,in_modification)
    local
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      Absyn_ClassPartList res,rest;
      Absyn_ElementItemList elts_1,elts;
      String varname;
      Absyn.ComponentRef submodident;
      Absyn.Modification mod;
      Absyn.ClassPart elt;
    case ({},_,_,_) then {}; 
    case (Absyn.PUBLIC(elts) :: rest,varname,submodident,mod)
      equation 
        res = set_component_submodifier_in_classparts(rest, varname, submodident, mod);
        elts_1 = set_component_submodifier_in_elementitems(elts, varname, submodident, mod); then Absyn.PUBLIC(elts_1) :: res;
    case (Absyn.PROTECTED(elts) :: rest,varname,submodident,mod)
      equation 
        res = set_component_submodifier_in_classparts(rest, varname, submodident, mod);
        elts_1 = set_component_submodifier_in_elementitems(elts, varname, submodident, mod); then Absyn.PROTECTED(elts_1) :: res;
    case (elt :: rest,varname,submodident,mod)
      equation 
        res = set_component_submodifier_in_classparts(rest, varname, submodident, mod); then elt :: res;
  end matchcontinue;
end set_component_submodifier_in_classparts;

protected function set_component_submodifier_in_elementitems " component name
   subvariable path"
  input Absyn_ElementItemList in_absyn_elementitemlist;
  input Absyn.Ident in_ident;
  input Absyn.ComponentRef in_componentref;
  input Absyn.Modification in_modification;
  output Absyn_ElementItemList out_absyn_elementitemlist;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
algorithm 
  out_absyn_elementitemlist:=
  matchcontinue (in_absyn_elementitemlist,in_ident,in_componentref,in_modification)
    local
      Absyn_ElementItemList res,rest;
      Absyn.Element elt_1,elt;
      String varname;
      Absyn.ComponentRef submodident;
      Absyn.Modification mod;
    case ({},_,_,_) then {}; 
    case (Absyn.ELEMENTITEM(elt) :: rest,varname,submodident,mod)
      equation 
        res = set_component_submodifier_in_elementitems(rest, varname, submodident, mod);
        elt_1 = set_component_submodifier_in_element(elt, varname, submodident, mod); then Absyn.ELEMENTITEM(elt_1) :: res;
    case (elt :: rest,varname,submodident,mod)
      local Absyn.ElementItem elt;
      equation 
        res = set_component_submodifier_in_elementitems(rest, varname, submodident, mod); then elt :: res;
  end matchcontinue;
end set_component_submodifier_in_elementitems;

protected function set_component_submodifier_in_element " component name
   submodifier path"
  input Absyn.Element in_element;
  input Absyn.Ident in_ident;
  input Absyn.ComponentRef in_componentref;
  input Absyn.Modification in_modification;
  output Absyn.Element out_element;
algorithm 
  out_element:=
  matchcontinue (in_element,in_ident,in_componentref,in_modification)
    local
      type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
      type Absyn_ElementSpecOption = Option<Absyn.ElementSpec>;
      Absyn_ComponentItemList compitems_1,compitems;
      Boolean f,r;
      Absyn.InnerOuter i;
      String n,s,varname;
      Absyn.ElementAttributes attr;
      Absyn.Path tp;
      Integer l;
      Absyn_ElementSpecOption constr;
      Absyn.ComponentRef submodident;
      Absyn.Modification mod;
      Absyn.Element elt;
    case (Absyn.ELEMENT(f,r,i,n,Absyn.COMPONENTS(attr,tp,compitems),s,l,constr),varname,submodident,mod) " component name  submodifier path"
      equation 
        compitems_1 = set_component_submodifier_in_compitems(compitems, varname, submodident, mod); then Absyn.ELEMENT(f,r,i,n,Absyn.COMPONENTS(attr,tp,compitems_1),s,l,constr);
    case (elt,_,_,_) then elt; 
  end matchcontinue;
end set_component_submodifier_in_element;

protected function set_component_submodifier_in_compitems " component name
   submodifier path"
  input Absyn_ComponentItemList in_absyn_componentitemlist;
  input Absyn.Ident in_ident;
  input Absyn.ComponentRef in_componentref;
  input Absyn.Modification in_modification;
  output Absyn_ComponentItemList out_absyn_componentitemlist;
protected 
  type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
algorithm 
  out_absyn_componentitemlist:=
  matchcontinue (in_absyn_componentitemlist,in_ident,in_componentref,in_modification)
    local
      type Absyn_SubscriptList = list<Absyn.Subscript>;
      type Absyn_CommentOption = Option<Absyn.Comment>;
      type Absyn_ElementArgList = list<Absyn.ElementArg>;
      type Absyn_ExpOption = Option<Absyn.Exp>;
      String id,varname;
      Absyn_SubscriptList dim;
      Absyn_CommentOption cmt;
      Absyn_ComponentItemList rest,res;
      Absyn.ComponentRef submodpath,submod,submodident;
      Absyn_ElementArgList args_1,args;
      Absyn_ExpOption expopt;
      Absyn.Modification mod;
      Absyn.ComponentItem comp;
    case ({},_,_,_) then {}; 
    case (Absyn.COMPONENTITEM(Absyn.COMPONENT(id,dim,_),cmt) :: rest,varname,submodpath,Absyn.CLASSMOD({},NONE))
      equation 
        equality(varname = id) " for \"empty\" modifications"; then Absyn.COMPONENTITEM(Absyn.COMPONENT(id,dim,NONE),cmt) :: rest;
    case (Absyn.COMPONENTITEM(Absyn.COMPONENT(id,dim,SOME(Absyn.CLASSMOD(args,expopt))),cmt) :: rest,varname,submodpath,mod)
      equation 
        equality(varname = id);
        args_1 = set_submodifier_in_elementargs(args, submodpath, mod); then Absyn.COMPONENTITEM(
          Absyn.COMPONENT(id,dim,SOME(Absyn.CLASSMOD(args_1,expopt))),cmt) :: rest;
    case (Absyn.COMPONENTITEM(Absyn.COMPONENT(id,dim,NONE),cmt) :: rest,varname,submod,mod)
      equation 
        equality(varname = id); then Absyn.COMPONENTITEM(
          Absyn.COMPONENT(id,dim,
          SOME(
          Absyn.CLASSMOD(
          {
          Absyn.MODIFICATION(false,Absyn.NON_EACH(),submod,SOME(mod),NONE)},NONE))),cmt) :: rest;
    case (comp :: rest,varname,submodident,mod)
      equation 
        res = set_component_submodifier_in_compitems(rest, varname, submodident, mod); then comp :: res;
    case (_,_,_,_)
      equation 
        print("-set_component_submodifier_in_compitems failed\n"); then fail();
  end matchcontinue;
end set_component_submodifier_in_compitems;

protected function set_submodifier_in_elementargs " subcomponent name"
  input Absyn_ElementArgList in_absyn_elementarglist;
  input Absyn.ComponentRef in_componentref;
  input Absyn.Modification in_modification;
  output Absyn_ElementArgList out_absyn_elementarglist;
protected 
  type Absyn_ElementArgList = list<Absyn.ElementArg>;
algorithm 
  out_absyn_elementarglist:=
  matchcontinue (in_absyn_elementarglist,in_componentref,in_modification)
    local
      type Absyn_SubscriptList = list<Absyn.Subscript>;
      type StringOption = Option<String>;
      type Absyn_ExpOption = Option<Absyn.Exp>;
      Absyn.ComponentRef cref,cr1,cr2,cr;
      Absyn.Modification mod;
      Boolean f;
      Absyn.Each each_;
      String name,submodident,name1,name2;
      Absyn_SubscriptList idx;
      StringOption cmt;
      Absyn_ElementArgList rest,args_1,args,res;
      Absyn_ExpOption exp;
      Absyn.ElementArg m;
    case ({},cref,mod) then {
          Absyn.MODIFICATION(false,Absyn.NON_EACH(),cref,SOME(mod),NONE)}; 
    case (Absyn.MODIFICATION(f,each_,Absyn.CREF_IDENT(name,idx),_,cmt) :: rest,Absyn.CREF_IDENT(submodident,_),mod)
      equation 
        equality(name = submodident); then Absyn.MODIFICATION(f,each_,Absyn.CREF_IDENT(name,idx),SOME(mod),cmt) :: rest;
    case (Absyn.MODIFICATION(f,each_,(cr1 = Absyn.CREF_QUAL(_,_,_)),_,cmt) :: rest,cr2,mod)
      equation 
        true = Absyn.cref_equal(cr1, cr2); then Absyn.MODIFICATION(f,each_,cr1,SOME(mod),cmt) :: rest;
    case (Absyn.MODIFICATION(f,each_,Absyn.CREF_QUAL(name1,_,_),_,cmt) :: rest,Absyn.CREF_IDENT(name2,idx),mod)
      equation 
        equality(name1 = name2); then Absyn.MODIFICATION(f,each_,Absyn.CREF_IDENT(name2,idx),SOME(mod),cmt) :: rest;
    case (Absyn.MODIFICATION(f,each_,(cr = Absyn.CREF_IDENT(name2,_)),SOME(Absyn.CLASSMOD(args,exp)),cmt) :: rest,Absyn.CREF_QUAL(name1,_,cr1),mod)
      equation 
        equality(name1 = name2);
        args_1 = set_submodifier_in_elementargs(args, cr1, mod); then Absyn.MODIFICATION(f,each_,cr,SOME(Absyn.CLASSMOD(args_1,exp)),cmt) :: rest;
    case (m :: rest,submodident,mod)
      local Absyn.ComponentRef submodident;
      equation 
        res = set_submodifier_in_elementargs(rest, submodident, mod); then m :: res;
    case (_,_,_)
      equation 
        print("-set_submodifier_in_elementargs failed\n"); then fail();
  end matchcontinue;
end set_submodifier_in_elementargs;

protected function get_component_modifier_value " variable name 
   submodifier name"
  input Absyn.ComponentRef in_componentref1;
  input Absyn.ComponentRef in_componentref2;
  input Absyn.ComponentRef in_componentref3;
  input Absyn.Program in_program4;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_componentref1,in_componentref2,in_componentref3,in_program4)
    local
      type Absyn_ElementList = list<Absyn.Element>;
      type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
      type Absyn_ComponentItemListList = list<Absyn_ComponentItemList>;
      Absyn.Path p_class;
      String name,res;
      Absyn.Class cdef;
      Absyn_ElementList comps;
      Absyn_ComponentItemListList compelts;
      Absyn_ComponentItemList compelts_1;
      Absyn.Modification mod;
      Absyn.ComponentRef class_,ident,subident;
      Absyn.Program p;
    case (class_,ident,subident,p) " class   variable name   submodifier name"
      equation 
        p_class = Absyn.cref_to_path(class_);
        Absyn.IDENT(name) = Absyn.cref_to_path(ident);
        cdef = get_pathed_class_in_program(p_class, p);
        comps = get_components_in_class(cdef);
        compelts = Util.list_map(comps, get_componentitems_in_element);
        compelts_1 = Util.list_flatten(compelts);
        {Absyn.COMPONENTITEM(Absyn.COMPONENT(_,_,SOME(Absyn.CLASSMOD(mod,_))),_)} = Util.list_select_1(compelts_1, name, componentitem_named);
        mod = get_modification_value(mod, subident);
        res = Dump.unparse_modification_str(mod); then res;
    case (_,_,_,_) then "Error"; 
  end matchcontinue;
end get_component_modifier_value;

protected function get_modification_value
  input Absyn_ElementArgList in_absyn_elementarglist;
  input Absyn.ComponentRef in_componentref;
  output Absyn.Modification out_modification;
protected 
  type Absyn_ElementArgList = list<Absyn.ElementArg>;
algorithm 
  out_modification:=
  matchcontinue (in_absyn_elementarglist,in_componentref)
    local
      type StringOption = Option<String>;
      type Absyn_ExpOption = Option<Absyn.Exp>;
      Boolean f;
      Absyn.Each each_;
      Absyn.ComponentRef cr1,cr2,name;
      Absyn.Modification mod,res;
      StringOption cmt;
      Absyn_ElementArgList rest,args;
      String name1,name2;
      Absyn_ExpOption exp;
    case (Absyn.MODIFICATION(f,each_,cr1,SOME(mod),cmt) :: rest,cr2)
      equation 
        true = Absyn.cref_equal(cr1, cr2); then mod;
    case (Absyn.MODIFICATION(f,each_,Absyn.CREF_IDENT(name1,_),SOME(Absyn.CLASSMOD(args,exp)),cmt) :: rest,Absyn.CREF_QUAL(name2,_,cr2))
      equation 
        equality(name1 = name2);
        res = get_modification_value(args, cr2); then res;
    case (_ :: rest,name)
      equation 
        mod = get_modification_value(rest, name); then mod;
  end matchcontinue;
end get_modification_value;

protected function get_component_modifier_names " variable name "
  input Absyn.ComponentRef in_componentref1;
  input Absyn.ComponentRef in_componentref2;
  input Absyn.Program in_program3;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_componentref1,in_componentref2,in_program3)
    local
      type Absyn_ElementList = list<Absyn.Element>;
      type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
      type Absyn_ComponentItemListList = list<Absyn_ComponentItemList>;
      type Absyn_ElementArgList = list<Absyn.ElementArg>;
      type StringList = list<String>;
      Absyn.Path p_class;
      String name,res_1,res_2;
      Absyn.Class cdef;
      Absyn_ElementList comps;
      Absyn_ComponentItemListList compelts;
      Absyn_ComponentItemList compelts_1;
      Absyn_ElementArgList mod;
      StringList res;
      Absyn.ComponentRef class_,ident;
      Absyn.Program p;
    case (class_,ident,p) " class   variable name "
      equation 
        p_class = Absyn.cref_to_path(class_);
        Absyn.IDENT(name) = Absyn.cref_to_path(ident);
        cdef = get_pathed_class_in_program(p_class, p);
        comps = get_components_in_class(cdef);
        compelts = Util.list_map(comps, get_componentitems_in_element);
        compelts_1 = Util.list_flatten(compelts);
        {Absyn.COMPONENTITEM(Absyn.COMPONENT(_,_,SOME(Absyn.CLASSMOD(mod,_))),_)} = Util.list_select_1(compelts_1, name, componentitem_named);
        res = get_modification_names(mod);
        res_1 = Util.string_delimit_list(res, ", ");
        res_2 = Util.string_append_list({"{",res_1,"}"}); then res_2;
    case (_,_,_) then "{}"; 
  end matchcontinue;
end get_component_modifier_names;

protected function get_modification_names
  input Absyn_ElementArgList in_absyn_elementarglist;
  output StringList out_stringlist;
protected 
  type Absyn_ElementArgList = list<Absyn.ElementArg>;
  type StringList = list<String>;
algorithm 
  out_stringlist:=
  matchcontinue (in_absyn_elementarglist)
    local
      type StringOption = Option<String>;
      StringList names,names2,names2_1,names2_2,res;
      Boolean f;
      Absyn.Each each_;
      String name;
      StringOption cmt;
      Absyn_ElementArgList rest,args;
      Absyn.ComponentRef cr;
    case ({}) then {}; 
    case (Absyn.MODIFICATION(f,each_,Absyn.CREF_IDENT(name,_),NONE,cmt) :: rest)
      equation 
        names = get_modification_names(rest); then name :: names;
    case (Absyn.MODIFICATION(f,each_,cr,SOME(Absyn.CLASSMOD({},_)),cmt) :: rest)
      equation 
        name = Dump.print_component_ref_str(cr);
        names = get_modification_names(rest); then name :: names;
    case (Absyn.MODIFICATION(f,each_,cr,SOME(Absyn.CLASSMOD(args,_)),cmt) :: rest)
      equation 
        name = Dump.print_component_ref_str(cr);
        names2 = get_modification_names(args);
        names2_1 = Util.list_map_1r(names2, string_append, ".");
        names2_2 = Util.list_map_1r(names2_1, string_append, name);
        names = get_modification_names(rest);
        res = list_append(names2_2, names); then res;
    case (_ :: rest)
      equation 
        names = get_modification_names(rest); then names;
  end matchcontinue;
end get_modification_names;

protected function get_parameter_value " variable name "
  input Absyn.ComponentRef in_componentref1;
  input Absyn.ComponentRef in_componentref2;
  input Absyn.Program in_program3;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_componentref1,in_componentref2,in_program3)
    local
      type Absyn_ElementList = list<Absyn.Element>;
      type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
      type Absyn_ComponentItemListList = list<Absyn_ComponentItemList>;
      Absyn.Path p_class;
      String name,res;
      Absyn.Class cdef;
      Absyn_ElementList comps,comps_1;
      Absyn_ComponentItemListList compelts;
      Absyn_ComponentItemList compelts_1;
      Absyn.ComponentItem compitem;
      Absyn.Exp exp;
      Absyn.ComponentRef class_;
      Absyn.Program p;
    case (class_,name,p) " class   variable name "
      equation 
        p_class = Absyn.cref_to_path(class_);
        Absyn.IDENT(name) = Absyn.cref_to_path(name);
        cdef = get_pathed_class_in_program(p_class, p);
        comps = get_components_in_class(cdef);
        comps_1 = Util.list_select(comps, is_parameter_element);
        compelts = Util.list_map(comps_1, get_componentitems_in_element);
        compelts_1 = Util.list_flatten(compelts);
        {compitem} = Util.list_select_1(compelts_1, name, componentitem_named);
        exp = get_variable_binding_in_componentitem(compitem);
        res = Dump.print_exp_str(exp); then res;
    case (_,_,_) then "Error"; 
  end matchcontinue;
end get_parameter_value;

protected function get_variable_binding_in_componentitem
  input Absyn.ComponentItem in_componentitem;
  output Absyn.Exp out_exp;
algorithm 
  out_exp:=
  matchcontinue (in_componentitem)
    local Absyn.Exp e;
    case (Absyn.COMPONENTITEM(Absyn.COMPONENT(_,_,SOME(Absyn.CLASSMOD(_,SOME(e)))),_)) then e; 
  end matchcontinue;
end get_variable_binding_in_componentitem;

protected function set_parameter_value "ident
   exp"
  input Absyn.ComponentRef in_componentref1;
  input Absyn.ComponentRef in_componentref2;
  input Absyn.Exp in_exp3;
  input Absyn.Program in_program4;
  output Absyn.Program out_program;
  output String out_string;
algorithm 
  (out_program,out_string):=
  matchcontinue (in_componentref1,in_componentref2,in_exp3,in_program4)
    local
      Absyn.Path p_class;
      String varname;
      Absyn.Within within_;
      Absyn.Class cdef,cdef_1;
      Absyn.Program newp,p;
      Absyn.ComponentRef class_,name;
      Absyn.Exp exp;
    case (class_,name,exp,p) "class ident  exp"
      equation 
        p_class = Absyn.cref_to_path(class_);
        Absyn.IDENT(varname) = Absyn.cref_to_path(name);
        within_ = build_within(p_class);
        cdef = get_pathed_class_in_program(p_class, p);
        cdef_1 = set_variable_binding_in_class(cdef, varname, exp);
        newp = update_program(Absyn.PROGRAM({cdef_1},within_), p); then (newp,"Ok");
    case (_,_,_,p) then (p,"Error"); 
  end matchcontinue;
end set_parameter_value;

protected function set_variable_binding_in_class
  input Absyn.Class in_class;
  input Absyn.Ident in_ident;
  input Absyn.Exp in_exp;
  output Absyn.Class out_class;
algorithm 
  out_class:=
  matchcontinue (in_class,in_ident,in_exp)
    local
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      type StringOption = Option<String>;
      Absyn_ClassPartList parts_1,parts;
      String id,file,id2;
      Boolean p,f,e;
      Absyn.Restriction r;
      StringOption cmt;
      Absyn.Exp exp;
    case (Absyn.CLASS(id,p,f,e,r,Absyn.PARTS(parts,cmt),file),id2,exp)
      equation 
        parts_1 = set_variable_binding_in_classparts(parts, id2, exp); then Absyn.CLASS(id,p,f,e,r,Absyn.PARTS(parts_1,cmt),file);
  end matchcontinue;
end set_variable_binding_in_class;

protected function set_variable_binding_in_classparts
  input Absyn_ClassPartList in_absyn_classpartlist;
  input Absyn.Ident in_ident;
  input Absyn.Exp in_exp;
  output Absyn_ClassPartList out_absyn_classpartlist;
protected 
  type Absyn_ClassPartList = list<Absyn.ClassPart>;
algorithm 
  out_absyn_classpartlist:=
  matchcontinue (in_absyn_classpartlist,in_ident,in_exp)
    local
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      Absyn_ClassPartList res,rest;
      Absyn_ElementItemList elts_1,elts;
      String id;
      Absyn.Exp exp;
      Absyn.ClassPart elt;
    case ({},_,_) then {}; 
    case (Absyn.PUBLIC(elts) :: rest,id,exp)
      equation 
        res = set_variable_binding_in_classparts(rest, id, exp);
        elts_1 = set_variable_binding_in_elementitems(elts, id, exp); then Absyn.PUBLIC(elts_1) :: res;
    case (Absyn.PROTECTED(elts) :: rest,id,exp)
      equation 
        res = set_variable_binding_in_classparts(rest, id, exp);
        elts_1 = set_variable_binding_in_elementitems(elts, id, exp); then Absyn.PROTECTED(elts_1) :: res;
    case (elt :: rest,id,exp)
      equation 
        res = set_variable_binding_in_classparts(rest, id, exp); then elt :: res;
  end matchcontinue;
end set_variable_binding_in_classparts;

protected function set_variable_binding_in_elementitems
  input Absyn_ElementItemList in_absyn_elementitemlist;
  input Absyn.Ident in_ident;
  input Absyn.Exp in_exp;
  output Absyn_ElementItemList out_absyn_elementitemlist;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
algorithm 
  out_absyn_elementitemlist:=
  matchcontinue (in_absyn_elementitemlist,in_ident,in_exp)
    local
      Absyn_ElementItemList res,rest;
      Absyn.Element elt_1,elt;
      String id;
      Absyn.Exp exp;
    case ({},_,_) then {}; 
    case (Absyn.ELEMENTITEM(elt) :: rest,id,exp)
      equation 
        res = set_variable_binding_in_elementitems(rest, id, exp);
        elt_1 = set_variable_binding_in_element(elt, id, exp); then Absyn.ELEMENTITEM(elt_1) :: res;
    case (elt :: rest,id,exp)
      local Absyn.ElementItem elt;
      equation 
        res = set_variable_binding_in_elementitems(rest, id, exp); then elt :: res;
  end matchcontinue;
end set_variable_binding_in_elementitems;

protected function set_variable_binding_in_element
  input Absyn.Element in_element;
  input Absyn.Ident in_ident;
  input Absyn.Exp in_exp;
  output Absyn.Element out_element;
algorithm 
  out_element:=
  matchcontinue (in_element,in_ident,in_exp)
    local
      type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
      type Absyn_ElementSpecOption = Option<Absyn.ElementSpec>;
      Absyn_ComponentItemList compitems_1,compitems;
      Boolean f,r;
      Absyn.InnerOuter i;
      String n,s,id;
      Absyn.ElementAttributes attr;
      Absyn.Path tp;
      Integer l;
      Absyn_ElementSpecOption constr;
      Absyn.Exp exp;
      Absyn.Element elt;
    case (Absyn.ELEMENT(f,r,i,n,Absyn.COMPONENTS(attr,tp,compitems),s,l,constr),id,exp)
      equation 
        compitems_1 = set_variable_binding_in_compitems(compitems, id, exp); then Absyn.ELEMENT(f,r,i,n,Absyn.COMPONENTS(attr,tp,compitems_1),s,l,constr);
    case (elt,id,exp) then elt; 
  end matchcontinue;
end set_variable_binding_in_element;

protected function set_variable_binding_in_compitems
  input Absyn_ComponentItemList in_absyn_componentitemlist;
  input Absyn.Ident in_ident;
  input Absyn.Exp in_exp;
  output Absyn_ComponentItemList out_absyn_componentitemlist;
protected 
  type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
algorithm 
  out_absyn_componentitemlist:=
  matchcontinue (in_absyn_componentitemlist,in_ident,in_exp)
    local
      type Absyn_SubscriptList = list<Absyn.Subscript>;
      type Absyn_ElementArgList = list<Absyn.ElementArg>;
      type Absyn_CommentOption = Option<Absyn.Comment>;
      String id,id2;
      Absyn_SubscriptList dim;
      Absyn_ElementArgList arg;
      Absyn_CommentOption cmt;
      Absyn_ComponentItemList rest,res;
      Absyn.Exp exp;
      Absyn.ComponentItem item;
    case ({},_,_) then {}; 
    case (Absyn.COMPONENTITEM(Absyn.COMPONENT(id,dim,SOME(Absyn.CLASSMOD(arg,_))),cmt) :: rest,id2,exp)
      equation 
        equality(id = id2); then Absyn.COMPONENTITEM(
          Absyn.COMPONENT(id,dim,SOME(Absyn.CLASSMOD(arg,SOME(exp)))),cmt) :: rest;
    case (Absyn.COMPONENTITEM(Absyn.COMPONENT(id,dim,NONE),cmt) :: rest,id2,exp)
      equation 
        equality(id = id2); then Absyn.COMPONENTITEM(Absyn.COMPONENT(id,dim,SOME(Absyn.CLASSMOD({},SOME(exp)))),
          cmt) :: rest;
    case (item :: rest,id,exp)
      equation 
        res = set_variable_binding_in_compitems(rest, id, exp); then item :: res;
  end matchcontinue;
end set_variable_binding_in_compitems;

protected function build_within "* function: build_within
 ** From a fully qualified model name, build a suitable within clause
*"
  input Absyn.Path in_path;
  output Absyn.Within out_within;
algorithm 
  out_within:=
  matchcontinue (in_path)
    local Absyn.Path w_path,path;
    case (Absyn.IDENT(_)) then Absyn.TOP(); 
    case (path)
      equation 
        w_path = Absyn.strip_last(path); then Absyn.WITHIN(w_path);
  end matchcontinue;
end build_within;

protected function componentitem_named "* function: componentitem_named
 ** Returns true if the component item has the name matching the second argument.
 *"
  input Absyn.ComponentItem in_componentitem;
  input Absyn.Ident in_ident;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_componentitem,in_ident)
    local String id,id2;
    case (Absyn.COMPONENTITEM(Absyn.COMPONENT(id,_,_),_),id2)
      equation 
        equality(id = id2); then true;
    case (_,_) then false; 
  end matchcontinue;
end componentitem_named;

protected function get_componentitem_name "*  function: get_componentitem_name
 ** Returns the name of a ComponentItem
 *"
  input Absyn.ComponentItem in_componentitem;
  output Absyn.Ident out_ident;
algorithm 
  out_ident:=
  matchcontinue (in_componentitem)
    local String id;
    case (Absyn.COMPONENTITEM(Absyn.COMPONENT(id,_,_),_)) then id; 
  end matchcontinue;
end get_componentitem_name;

protected function get_componentitems_in_element "* function: get_componentitems_in_element
 ** Retrieves the ComponentItems of a component Element.
 ** If Element is not a component, empty list is returned.
 *"
  input Absyn.Element in_element;
  output Absyn_ComponentItemList out_absyn_componentitemlist;
protected 
  type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
algorithm 
  out_absyn_componentitemlist:=
  matchcontinue (in_element)
    local Absyn_ComponentItemList l;
    case (Absyn.ELEMENT(_,_,_,_,Absyn.COMPONENTS(_,_,l),_,_,_)) then l; 
    case (_) then {}; 
  end matchcontinue;
end get_componentitems_in_element;

protected function rename_class " old class as qualified name
   new class, as identifier "
  input Absyn.Program in_program1;
  input Absyn.ComponentRef in_componentref2;
  input Absyn.ComponentRef in_componentref3;
  output Absyn.Program out_program;
algorithm 
  out_program:=
  matchcontinue (in_program1,in_componentref2,in_componentref3)
    local
      Absyn.Path old_path;
      Absyn.Program p_1,p;
      Absyn.ComponentRef old_class;
      String new_name;
    case (p,(old_class = Absyn.CREF_QUAL(_,_,_)),Absyn.CREF_IDENT(new_name,_)) " old class as qualified name  new class, as identifier "
      equation 
        old_path = Absyn.cref_to_path(old_class) " class in package ";
        ((p_1,_)) = traverse_classes(p, rename_class_visitor, (old_path,new_name), true) " traverse protected"; then p_1;
  end matchcontinue;
end rename_class;

protected function rename_class_visitor
  input TupleAbsyn_ClassTupleAbsyn_PathString in_tupleabsyn_classtupleabsyn_pathstring;
  output TupleAbsyn_ClassTupleAbsyn_PathString out_tupleabsyn_classtupleabsyn_pathstring;
protected 
  type TupleAbsyn_PathString = tuple<Absyn.Path,String>;
  type TupleAbsyn_ClassTupleAbsyn_PathString = tuple<Absyn.Class,TupleAbsyn_PathString>;
algorithm 
  out_tupleabsyn_classtupleabsyn_pathstring:=
  matchcontinue (in_tupleabsyn_classtupleabsyn_pathstring)
    local
      String id2,id,f,new_name;
      Boolean a,b,c;
      Absyn.Restriction d;
      Absyn.ClassDef e;
      Absyn.Path old_class;
      Absyn.Class class_;
      TupleAbsyn_PathString args;
    case ((Absyn.CLASS(id,a,b,c,d,e,f),(old_class,new_name)))
      equation 
        id2 = Absyn.path_last_ident(old_class);
        equality(id = id2); then ((Absyn.CLASS(new_name,a,b,c,d,e,f),(old_class,new_name)));
    case ((class_,args)) then ((class_,args)); 
  end matchcontinue;
end rename_class_visitor;

public function traverse_classes " true = traverse protected elements"
  input Absyn.Program in_program;
  input FuncType1 in_functype1;
  input Type_a in_type_a;
  input Boolean in_boolean;
  output TupleAbsyn_ProgramType_a out_tupleabsyn_programtype_a;
protected 
  function FuncType1
    input TupleAbsyn_ClassType_a in_tupleabsyn_classtype_a;
    output TupleAbsyn_ClassType_a out_tupleabsyn_classtype_a;
  protected 
    replaceable type Type_a;
    type TupleAbsyn_ClassType_a = tuple<Absyn.Class,Type_a>;
  end FuncType1;
  replaceable type Type_a;
  type TupleAbsyn_ProgramType_a = tuple<Absyn.Program,Type_a>;
algorithm 
  out_tupleabsyn_programtype_a:=
  matchcontinue (in_program,in_functype1,in_type_a,in_boolean)
    local
      type Absyn_ClassList = list<Absyn.Class>;
      Absyn_ClassList lst_1,lst;
      Type_a args_1,args;
      Absyn.Within within_;
      FuncType1 visitor;
      Boolean traverse_prot;
    case (Absyn.PROGRAM(lst,within_),visitor,args,traverse_prot)
      equation 
        ((lst_1,args_1)) = traverse_classes2(lst, visitor, args, traverse_prot); then ((Absyn.PROGRAM(lst_1,within_),args_1));
  end matchcontinue;
end traverse_classes;

protected function traverse_classes2 " true = traverse protected elements"
  input Absyn_ClassList in_absyn_classlist;
  input FuncType1 in_functype1;
  input Type_a in_type_a;
  input Boolean in_boolean;
  output TupleAbsyn_ClassListType_a out_tupleabsyn_classlisttype_a;
protected 
  type Absyn_ClassList = list<Absyn.Class>;
  function FuncType1
    input TupleAbsyn_ClassType_a in_tupleabsyn_classtype_a;
    output TupleAbsyn_ClassType_a out_tupleabsyn_classtype_a;
  protected 
    replaceable type Type_a;
    type TupleAbsyn_ClassType_a = tuple<Absyn.Class,Type_a>;
  end FuncType1;
  replaceable type Type_a;
  type TupleAbsyn_ClassListType_a = tuple<Absyn_ClassList,Type_a>;
algorithm 
  out_tupleabsyn_classlisttype_a:=
  matchcontinue (in_absyn_classlist,in_functype1,in_type_a,in_boolean)
    local
      Type_a args,args_1,args_2,args_3;
      Absyn.Class class_1,class_2,class_;
      Absyn_ClassList classes_1,classes;
      FuncType1 visitor;
      Boolean traverse_prot;
    case ({},_,args,_) then (({},args)); 
    case (class_ :: classes,visitor,args,traverse_prot)
      equation 
        ((class_1,args_1)) = visitor((class_,args));
        ((class_2,args_2)) = traverse_inner_class(class_, visitor, args_1, traverse_prot);
        ((classes_1,args_3)) = traverse_classes2(classes, visitor, args_2, traverse_prot); then ((class_2 :: classes_1, args_3));
    case (_,_,_,_)
      equation 
        print("-traverse_classes2 failed\n"); then fail();
  end matchcontinue;
end traverse_classes2;

protected function traverse_inner_class " true = traverse protected elts "
  input Absyn.Class in_class;
  input FuncType1 in_functype1;
  input Type_a in_type_a;
  input Boolean in_boolean;
  output TupleAbsyn_ClassType_a out_tupleabsyn_classtype_a;
protected 
  function FuncType1
    input TupleAbsyn_ClassType_a in_tupleabsyn_classtype_a;
    output TupleAbsyn_ClassType_a out_tupleabsyn_classtype_a;
  protected 
    replaceable type Type_a;
    type TupleAbsyn_ClassType_a = tuple<Absyn.Class,Type_a>;
  end FuncType1;
  replaceable type Type_a;
  type TupleAbsyn_ClassType_a = tuple<Absyn.Class,Type_a>;
algorithm 
  out_tupleabsyn_classtype_a:=
  matchcontinue (in_class,in_functype1,in_type_a,in_boolean)
    local
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      type StringOption = Option<String>;
      Absyn_ClassPartList parts_1,parts;
      Type_a args_1,args;
      String name,file;
      Boolean p,f,e,visit_prot;
      Absyn.Restriction r;
      StringOption str_opt;
      FuncType1 visitor;
      Absyn.Class cl;
    case (Absyn.CLASS(name,p,f,e,r,Absyn.PARTS(parts,str_opt),file),visitor,args,visit_prot)
      equation 
        ((parts_1,args_1)) = traverse_inner_class_parts(parts, visitor, args, visit_prot); then ((
          Absyn.CLASS(name,p,f,e,r,Absyn.PARTS(parts_1,str_opt),file),args_1));
    case (cl,_,args,_) then ((cl,args)); 
  end matchcontinue;
end traverse_inner_class;

protected function traverse_inner_class_parts " true = visit protected elements"
  input Absyn_ClassPartList in_absyn_classpartlist;
  input FuncType1 in_functype1;
  input Type_a in_type_a;
  input Boolean in_boolean;
  output TupleAbsyn_ClassPartListType_a out_tupleabsyn_classpartlisttype_a;
protected 
  type Absyn_ClassPartList = list<Absyn.ClassPart>;
  function FuncType1
    input TupleAbsyn_ClassType_a in_tupleabsyn_classtype_a;
    output TupleAbsyn_ClassType_a out_tupleabsyn_classtype_a;
  protected 
    replaceable type Type_a;
    type TupleAbsyn_ClassType_a = tuple<Absyn.Class,Type_a>;
  end FuncType1;
  replaceable type Type_a;
  type TupleAbsyn_ClassPartListType_a = tuple<Absyn_ClassPartList,Type_a>;
algorithm 
  out_tupleabsyn_classpartlisttype_a:=
  matchcontinue (in_absyn_classpartlist,in_functype1,in_type_a,in_boolean)
    local
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      Type_a args,args_1,args_2;
      Absyn_ElementItemList elts_1,elts;
      Absyn_ClassPartList parts_1,parts;
      FuncType1 visitor;
      Boolean visit_prot;
      Absyn.ClassPart part;
    case ({},_,args,_) then (({},args)); 
    case (Absyn.PUBLIC(elts) :: parts,visitor,args,visit_prot)
      equation 
        ((elts_1,args_1)) = traverse_inner_class_elements(elts, visitor, args, visit_prot);
        ((parts_1,args_2)) = traverse_inner_class_parts(parts, visitor, args_1, visit_prot); then ((Absyn.PUBLIC(elts_1) :: parts_1,args_2));
    case (Absyn.PROTECTED(elts) :: parts,visitor,args,true)
      equation 
        ((elts_1,args_1)) = traverse_inner_class_elements(elts, visitor, args, true);
        ((parts_1,args_2)) = traverse_inner_class_parts(parts, visitor, args_1, true); then ((Absyn.PROTECTED(elts_1) :: parts_1,args_2));
    case (part :: parts,visitor,args,true)
      equation 
        ((parts_1,args_1)) = traverse_inner_class_parts(parts, visitor, args, true); then ((part :: parts_1,args_1));
  end matchcontinue;
end traverse_inner_class_parts;

protected function traverse_inner_class_elements " visit protected elts "
  input Absyn_ElementItemList in_absyn_elementitemlist;
  input FuncType1 in_functype1;
  input Type_a in_type_a;
  input Boolean in_boolean;
  output TupleAbsyn_ElementItemListType_a out_tupleabsyn_elementitemlisttype_a;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
  function FuncType1
    input TupleAbsyn_ClassType_a in_tupleabsyn_classtype_a;
    output TupleAbsyn_ClassType_a out_tupleabsyn_classtype_a;
  protected 
    replaceable type Type_a;
    type TupleAbsyn_ClassType_a = tuple<Absyn.Class,Type_a>;
  end FuncType1;
  replaceable type Type_a;
  type TupleAbsyn_ElementItemListType_a = tuple<Absyn_ElementItemList,Type_a>;
algorithm 
  out_tupleabsyn_elementitemlisttype_a:=
  matchcontinue (in_absyn_elementitemlist,in_functype1,in_type_a,in_boolean)
    local
      type Absyn_ElementSpecOption = Option<Absyn.ElementSpec>;
      Type_a args,args_1,args_2;
      Absyn.ElementSpec elt_spec_1,elt_spec;
      Absyn_ElementItemList elts_1,elts;
      Boolean f,r,visit_prot;
      Absyn.InnerOuter io;
      String n,file;
      Integer line;
      Absyn_ElementSpecOption constr;
      FuncType1 visitor;
      Absyn.ElementItem elt;
    case ({},_,args,_) then (({},args)); 
    case (Absyn.ELEMENTITEM(Absyn.ELEMENT(f,r,io,n,elt_spec,file,line,constr)) :: elts,visitor,args,visit_prot)
      equation 
        ((elt_spec_1,args_1)) = traverse_inner_class_elementspec(elt_spec, visitor, args, visit_prot);
        ((elts_1,args_2)) = traverse_inner_class_elements(elts, visitor, args_1, visit_prot); then ((
          Absyn.ELEMENTITEM(
          Absyn.ELEMENT(f,r,io,n,elt_spec_1,file,line,constr)) :: elts_1, args_2));
    case (elt :: elts,visitor,args,visit_prot)
      equation 
        ((elts_1,args_1)) = traverse_inner_class_elements(elts, visitor, args, visit_prot); then ((elt :: elts_1,args_1));
  end matchcontinue;
end traverse_inner_class_elements;

protected function traverse_inner_class_elementspec " visit protected elts "
  input Absyn.ElementSpec in_elementspec;
  input FuncType1 in_functype1;
  input Type_a in_type_a;
  input Boolean in_boolean;
  output TupleAbsyn_ElementSpecType_a out_tupleabsyn_elementspectype_a;
protected 
  function FuncType1
    input TupleAbsyn_ClassType_a in_tupleabsyn_classtype_a;
    output TupleAbsyn_ClassType_a out_tupleabsyn_classtype_a;
  protected 
    replaceable type Type_a;
    type TupleAbsyn_ClassType_a = tuple<Absyn.Class,Type_a>;
  end FuncType1;
  replaceable type Type_a;
  type TupleAbsyn_ElementSpecType_a = tuple<Absyn.ElementSpec,Type_a>;
algorithm 
  out_tupleabsyn_elementspectype_a:=
  matchcontinue (in_elementspec,in_functype1,in_type_a,in_boolean)
    local
      Absyn.Class class_1,class_2,class_;
      Type_a args_1,args_2,args;
      Boolean repl,visit_prot;
      FuncType1 visitor;
      Absyn.ElementSpec elt_spec;
    case (Absyn.CLASSDEF(repl,class_),visitor,args,visit_prot)
      equation 
        ((class_1,args_1)) = visitor((class_,args));
        ((class_2,args_2)) = traverse_inner_class(class_1, visitor, args_1, visit_prot); then ((Absyn.CLASSDEF(repl,class_2),args_2));
    case (elt_spec,_,args,_) then ((elt_spec,args)); 
  end matchcontinue;
end traverse_inner_class_elementspec;

public function is_primitive "* function: is_primitive 
 ** This function takes a component reference and a program. 
 ** It returns the true if the refrenced type is a primitive type, otherwise it returns 
 ** false.
*"
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_componentref,in_program)
    local
      Absyn.Path path;
      Absyn.Class class_;
      Boolean res;
      Absyn.ComponentRef cr;
      Absyn.Program p;
    case (cr,p)
      equation 
        path = Absyn.cref_to_path(cr);
        class_ = get_pathed_class_in_program(path, p);
        res = is_primitive_class(class_, p); then res;
    case (Absyn.CREF_IDENT("Real",_),_) then true; 
    case (Absyn.CREF_IDENT("Integer",_),_) then true; 
    case (Absyn.CREF_IDENT("String",_),_) then true; 
    case (Absyn.CREF_IDENT("Boolean",_),_) then true; 
    case (_,_) then false; 
  end matchcontinue;
end is_primitive;

protected function delete_class "* function: delete_class
 ** This function takes a component reference and a program. 
 ** It deletes the class specified by the component reference from the given program.
 *"
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  output String out_string;
  output Absyn.Program out_program;
algorithm 
  (out_string,out_program):=
  matchcontinue (in_componentref,in_program)
    local
      type Absyn_ClassList = list<Absyn.Class>;
      Absyn.Path cpath,parentcpath,parentparentcpath;
      Absyn.Class cdef,parentcdef,parentcdef_1;
      Absyn.Program newp,p;
      Absyn.ComponentRef class_;
      Absyn_ClassList clist,clist_1;
      Absyn.Within w;
    case (class_,(p = Absyn.PROGRAM(clist,w)))
      equation 
        cpath = Absyn.cref_to_path(class_) " Class inside another class, inside another class ";
        parentcpath = Absyn.strip_last(cpath);
        parentparentcpath = Absyn.strip_last(parentcpath);
        cdef = get_pathed_class_in_program(cpath, p);
        parentcdef = get_pathed_class_in_program(parentcpath, p);
        parentcdef_1 = remove_inner_class(cdef, parentcdef);
        newp = update_program(
          Absyn.PROGRAM({parentcdef_1},Absyn.WITHIN(parentparentcpath)), p); then ("true",newp);
    case (class_,(p = Absyn.PROGRAM(clist,w)))
      equation 
        cpath = Absyn.cref_to_path(class_) " Class inside other class ";
        parentcpath = Absyn.strip_last(cpath);
        cdef = get_pathed_class_in_program(cpath, p);
        parentcdef = get_pathed_class_in_program(parentcpath, p);
        parentcdef_1 = remove_inner_class(cdef, parentcdef);
        newp = update_program(Absyn.PROGRAM({parentcdef_1},Absyn.TOP()), p); then ("true",newp);
    case (class_,(p = Absyn.PROGRAM(clist,w)))
      equation 
        cpath = Absyn.cref_to_path(class_) " Top level class ";
        cdef = get_pathed_class_in_program(cpath, p);
        clist_1 = delete_class_from_list(cdef, clist); then ("true",Absyn.PROGRAM(clist_1,w));
    case (_,p) then ("false",p); 
  end matchcontinue;
end delete_class;

protected function delete_class_from_list
  input Absyn.Class in_class;
  input Absyn_ClassList in_absyn_classlist;
  output Absyn_ClassList out_absyn_classlist;
protected 
  type Absyn_ClassList = list<Absyn.Class>;
algorithm 
  out_absyn_classlist:=
  matchcontinue (in_class,in_absyn_classlist)
    local
      String name,name2;
      Absyn_ClassList xs,res;
      Absyn.Class cdef,x;
    case (_,{}) then {}; 
    case (Absyn.CLASS(name,_,_,_,_,_,_),Absyn.CLASS(name2,_,_,_,_,_,_) :: xs)
      equation 
        equality(name = name2); then xs;
    case ((cdef = Absyn.CLASS(name,_,_,_,_,_,_)),(x = Absyn.CLASS(name2,_,_,_,_,_,_)) :: xs)
      equation 
        failure(equality(name = name2));
        res = delete_class_from_list(cdef, xs); then x :: res;
    case ((cdef = Absyn.CLASS(name,_,_,_,_,_,_)),x :: xs)
      equation 
        res = delete_class_from_list(cdef, xs); then x :: res;
  end matchcontinue;
end delete_class_from_list;

protected function is_type "* function: is_type 
 ** This function takes a component reference and a program. 
 ** It returns true if the refrenced class has the restriction \"type\", otherwise it returns 
 ** false.
*"
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_componentref,in_program)
    local
      Absyn.Path path;
      Absyn.ComponentRef cr;
      Absyn.Program p;
    case (cr,p)
      equation 
        path = Absyn.cref_to_path(cr);
        Absyn.CLASS(_,_,_,_,Absyn.R_TYPE(),_,_) = get_pathed_class_in_program(path, p); then true;
    case (cr,p) then false; 
  end matchcontinue;
end is_type;

protected function is_connector "* function: is_connector
 ** This function takes a component reference and a program. 
 ** It returns true if the refrenced class has the restriction \"connector\", otherwise it returns  
 ** false.
*"
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_componentref,in_program)
    local
      Absyn.Path path;
      Absyn.ComponentRef cr;
      Absyn.Program p;
    case (cr,p)
      equation 
        path = Absyn.cref_to_path(cr);
        Absyn.CLASS(_,_,_,_,Absyn.R_CONNECTOR(),_,_) = get_pathed_class_in_program(path, p); then true;
    case (cr,p) then false; 
  end matchcontinue;
end is_connector;

protected function is_model "* function: is_model
 ** This function takes a component reference and a program. 
 ** It returns true if the refrenced class has the restriction \"model\", otherwise it returns 
 ** false.
*"
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_componentref,in_program)
    local
      Absyn.Path path;
      Absyn.ComponentRef cr;
      Absyn.Program p;
    case (cr,p)
      equation 
        path = Absyn.cref_to_path(cr);
        Absyn.CLASS(_,_,_,_,Absyn.R_MODEL(),_,_) = get_pathed_class_in_program(path, p); then true;
    case (cr,p) then false; 
  end matchcontinue;
end is_model;

protected function is_record "* function: is_record
 ** This function takes a component reference and a program. 
 ** It returns true if the refrenced class has the restriction \"record\", otherwise it returns 
 ** false.
*"
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_componentref,in_program)
    local
      Absyn.Path path;
      Absyn.ComponentRef cr;
      Absyn.Program p;
    case (cr,p)
      equation 
        path = Absyn.cref_to_path(cr);
        Absyn.CLASS(_,_,_,_,Absyn.R_RECORD(),_,_) = get_pathed_class_in_program(path, p); then true;
    case (cr,p) then false; 
  end matchcontinue;
end is_record;

protected function is_block "* function: is_block
 ** This function takes a component reference and a program. 
 ** It returns true if the refrenced class has the restriction \"block\", otherwise it returns 
 ** false.
*"
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_componentref,in_program)
    local
      Absyn.Path path;
      Absyn.ComponentRef cr;
      Absyn.Program p;
    case (cr,p)
      equation 
        path = Absyn.cref_to_path(cr);
        Absyn.CLASS(_,_,_,_,Absyn.R_BLOCK(),_,_) = get_pathed_class_in_program(path, p); then true;
    case (cr,p) then false; 
  end matchcontinue;
end is_block;

protected function is_function "* function: is_function
 ** This function takes a component reference and a program. 
 ** It returns true if the refrenced class has the restriction \"function\", otherwise it returns
 ** false.
*"
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_componentref,in_program)
    local
      Absyn.Path path;
      Absyn.ComponentRef cr;
      Absyn.Program p;
    case (cr,p)
      equation 
        path = Absyn.cref_to_path(cr);
        Absyn.CLASS(_,_,_,_,Absyn.R_FUNCTION(),_,_) = get_pathed_class_in_program(path, p); then true;
    case (cr,p) then false; 
  end matchcontinue;
end is_function;

public function is_package "* function: is_package
 ** This function takes a component reference and a program. 
 ** It returns true if the refrenced class has the restriction \"package\", otherwise it returns 
 ** false.
*"
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_componentref,in_program)
    local
      Absyn.Path path;
      Absyn.ComponentRef cr;
      Absyn.Program p;
    case (cr,p)
      equation 
        path = Absyn.cref_to_path(cr);
        Absyn.CLASS(_,_,_,_,Absyn.R_PACKAGE(),_,_) = get_pathed_class_in_program(path, p); then true;
    case (cr,p) then false; 
  end matchcontinue;
end is_package;

protected function is_class "* function: is_class
 ** This function takes a component reference and a program. 
 ** It returns true if the refrenced class has the restriction \"class\", otherwise it returns 
 ** false.
*"
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_componentref,in_program)
    local
      Absyn.Path path;
      Absyn.ComponentRef cr;
      Absyn.Program p;
    case (cr,p)
      equation 
        path = Absyn.cref_to_path(cr);
        Absyn.CLASS(_,_,_,_,Absyn.R_CLASS(),_,_) = get_pathed_class_in_program(path, p); then true;
    case (cr,p) then false; 
  end matchcontinue;
end is_class;

protected function is_parameter "* function: is_parameter
 ** This function takes a class and a component reference and a program
 ** and returns true if the component referenced is a parameter.
*"
  input Absyn.ComponentRef in_componentref1;
  input Absyn.ComponentRef in_componentref2;
  input Absyn.Program in_program3;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_componentref1,in_componentref2,in_program3)
    local
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      Absyn.Path path;
      String i;
      Boolean p,f,e;
      Absyn.Restriction r;
      Absyn_ClassPartList parts;
      Absyn_ElementItemList publst;
      Absyn.ComponentRef cr,classname;
    case (cr,classname,p)
      equation 
        path = Absyn.cref_to_path(classname);
        Absyn.CLASS(i,p,f,e,r,Absyn.PARTS(parts,_),_) = get_pathed_class_in_program(path, p);
        publst = get_public_list(parts);
        Absyn.COMPONENTS(Absyn.ATTR(_,Absyn.PARAM(),_,_),_,_) = get_components_contains_name(cr, publst); then true;
    case (_,_,_) then false; 
  end matchcontinue;
end is_parameter;

protected function is_protected "* function: is_protected
 ** This function takes a class and a component reference and a program
 ** and returns true if the component referenced is in a protected section.
*"
  input Absyn.ComponentRef in_componentref1;
  input Absyn.ComponentRef in_componentref2;
  input Absyn.Program in_program3;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_componentref1,in_componentref2,in_program3)
    local
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      Absyn.Path path;
      String i;
      Boolean p,f,e;
      Absyn.Restriction r;
      Absyn_ClassPartList parts;
      Absyn_ElementItemList publst,protlst;
      Absyn.ComponentRef cr,classname;
    case (cr,classname,p)
      equation 
        path = Absyn.cref_to_path(classname);
        Absyn.CLASS(i,p,f,e,r,Absyn.PARTS(parts,_),_) = get_pathed_class_in_program(path, p);
        publst = get_public_list(parts);
        _ = get_components_contains_name(cr, publst); then false;
    case (cr,classname,p)
      equation 
        path = Absyn.cref_to_path(classname);
        Absyn.CLASS(i,p,f,e,r,Absyn.PARTS(parts,_),_) = get_pathed_class_in_program(path, p);
        protlst = get_protected_list(parts);
        _ = get_components_contains_name(cr, protlst); then true;
    case (_,_,_) then false; 
  end matchcontinue;
end is_protected;

protected function is_constant "* function: is_constant
 ** This function takes a class and a component reference and a program
 ** and returns true if the component referenced is a constant.
*"
  input Absyn.ComponentRef in_componentref1;
  input Absyn.ComponentRef in_componentref2;
  input Absyn.Program in_program3;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_componentref1,in_componentref2,in_program3)
    local
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      Absyn.Path path;
      String i;
      Boolean p,f,e;
      Absyn.Restriction r;
      Absyn_ClassPartList parts;
      Absyn_ElementItemList publst;
      Absyn.ComponentRef cr,classname;
    case (cr,classname,p)
      equation 
        path = Absyn.cref_to_path(classname);
        Absyn.CLASS(i,p,f,e,r,Absyn.PARTS(parts,_),_) = get_pathed_class_in_program(path, p);
        publst = get_public_list(parts);
        Absyn.COMPONENTS(Absyn.ATTR(_,Absyn.CONST(),_,_),_,_) = get_components_contains_name(cr, publst); then true;
    case (_,_,_) then false; 
  end matchcontinue;
end is_constant;

protected function get_components_contains_name
  input Absyn.ComponentRef in_componentref;
  input Absyn_ElementItemList in_absyn_elementitemlist;
  output Absyn.ElementSpec out_elementspec;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
algorithm 
  out_elementspec:=
  matchcontinue (in_componentref,in_absyn_elementitemlist)
    local
      type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
      Absyn.ComponentRef cr;
      Absyn.ElementSpec res;
      Absyn_ComponentItemList ellst;
      Absyn_ElementItemList xs;
      Absyn.ElementItem x;
    case (cr,Absyn.ELEMENTITEM(Absyn.ELEMENT(_,_,_,_,(res = Absyn.COMPONENTS(_,_,ellst)),_,_,_)) :: xs)
      equation 
        _ = get_compitem_named(cr, ellst); then res;
    case (cr,x :: xs)
      equation 
        res = get_components_contains_name(cr, xs); then res;
  end matchcontinue;
end get_components_contains_name;

protected function get_compitem_named
  input Absyn.ComponentRef in_componentref;
  input Absyn_ComponentItemList in_absyn_componentitemlist;
  output Absyn.ComponentItem out_componentitem;
protected 
  type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
algorithm 
  out_componentitem:=
  matchcontinue (in_componentref,in_absyn_componentitemlist)
    local
      String id1,id2;
      Absyn.ComponentItem x,res;
      Absyn_ComponentItemList xs;
      Absyn.ComponentRef cr;
    case (Absyn.CREF_IDENT(id1,_),(x = Absyn.COMPONENTITEM(Absyn.COMPONENT(id2,_,_),_)) :: xs)
      equation 
        equality(id1 = id2); then x;
    case (cr,x :: xs)
      equation 
        res = get_compitem_named(cr, xs); then res;
  end matchcontinue;
end get_compitem_named;

protected function exist_class "* function: exist_class
 ** This function takes a component reference and a program. 
 ** It returns true if the refrenced class exists in the symbol table, otherwise it returns
 ** false.
*"
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_componentref,in_program)
    local
      Absyn.Path path;
      Absyn.ComponentRef cr;
      Absyn.Program p;
    case (cr,p)
      equation 
        path = Absyn.cref_to_path(cr);
        _ = get_pathed_class_in_program(path, p); then true;
    case (cr,p) then false; 
  end matchcontinue;
end exist_class;

public function is_primitive_class
  input Absyn.Class in_class;
  input Absyn.Program in_program;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_class,in_program)
    local
      Absyn.Path inmodel,path;
      Absyn.Class cdef;
      Boolean res;
      String cname;
      Absyn.Program p;
    case (Absyn.CLASS(_,_,_,_,Absyn.R_PREDEFINED_INT(),_,_),_) then true; 
    case (Absyn.CLASS(_,_,_,_,Absyn.R_PREDEFINED_REAL(),_,_),_) then true; 
    case (Absyn.CLASS(_,_,_,_,Absyn.R_PREDEFINED_STRING(),_,_),_) then true; 
    case (Absyn.CLASS(_,_,_,_,Absyn.R_PREDEFINED_BOOL(),_,_),_) then true; 
    case (Absyn.CLASS(_,_,_,_,Absyn.R_TYPE(),_,_),_) then true; 
    case (Absyn.CLASS(cname,_,_,_,Absyn.R_CLASS(),Absyn.DERIVED(path,_,_,_,_),_),p)
      equation 
        inmodel = Absyn.cref_to_path(Absyn.CREF_IDENT(cname,{}));
        (cdef,_) = lookup_classdef(path, inmodel, p);
        res = is_primitive_class(cdef, p); then res;
  end matchcontinue;
end is_primitive_class;

public function remove_compiled_functions
  input Absyn.Program in_program;
  input TupleAbsyn_PathTypes_TypeList in_tupleabsyn_pathtypes_typelist;
  output TupleAbsyn_PathTypes_TypeList out_tupleabsyn_pathtypes_typelist;
algorithm 
  out_tupleabsyn_pathtypes_typelist:=
  matchcontinue (in_program,in_tupleabsyn_pathtypes_typelist)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOption = tuple<Absyn.Path,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOptionList = list<TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOption>;
      TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOptionList cfs_1,cfs;
      String id;
    case (Absyn.PROGRAM({Absyn.CLASS(id,_,_,_,Absyn.R_FUNCTION(),_,_)},_),cfs)
      equation 
        cfs_1 = remove_cf(id, cfs); then cfs_1;
    case (_,cfs) then cfs; 
  end matchcontinue;
end remove_compiled_functions;

protected function remove_cf
  input Absyn.Ident in_ident;
  input TupleAbsyn_PathTypes_TypeList in_tupleabsyn_pathtypes_typelist;
  output TupleAbsyn_PathTypes_TypeList out_tupleabsyn_pathtypes_typelist;
algorithm 
  out_tupleabsyn_pathtypes_typelist:=
  matchcontinue (in_ident,in_tupleabsyn_pathtypes_typelist)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOption = tuple<Absyn.Path,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOptionList = list<TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOption>;
      TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOptionList res,rest;
      String id1,id2;
      TupleTypes_TTypeAbsyn_PathOption t;
    case (_,{}) then {}; 
    case (id1,(Absyn.IDENT(id2),t) :: rest)
      equation 
        equality(id1 = id2);
        res = remove_cf(id1, rest); then res;
    case (id1,(Absyn.IDENT(id2),t) :: rest)
      equation 
        failure(equality(id1 = id2));
        res = remove_cf(id1, rest); then (Absyn.IDENT(id2),t) :: res;
  end matchcontinue;
end remove_cf;

public function update_program "* function: update_program
 **
 ** This function takes an old program (second argument), i.e. the old symboltable, 
 ** and a new program (first argument), i.e. a new set of classes and updates the old 
 ** program with the definitions in the new one
 *"
  input Absyn.Program in_program1;
  input Absyn.Program in_program2;
  output Absyn.Program out_program;
algorithm 
  out_program:=
  matchcontinue (in_program1,in_program2)
    local
      type Absyn_ClassList = list<Absyn.Class>;
      Absyn.Program prg,newp,oldp,pnew,p2,pnew_1,a,b;
      Absyn.Class newclass,c1,cdef,newcdef;
      String name;
      Absyn.Restriction restr;
      Boolean p,e;
      Absyn.Path w,path,modelwithin;
      Absyn_ClassList c2,c3;
      Absyn.Within w2;
      Absyn.ElementSpec elt;
    case (Absyn.PROGRAM({},_),prg) then prg; 
    case (Absyn.BEGIN_DEFINITION(Absyn.IDENT(name),restr,p,e),oldp)
      equation 
        newclass = Absyn.CLASS(name,p,false,e,restr,Absyn.PARTS({Absyn.PUBLIC({})},NONE),
          "NoFile") " For split definitions at top, when introducing new model, eg. \"package A\" ";
        newp = update_program(Absyn.PROGRAM({newclass},Absyn.TOP()), oldp); then newp;
    case (Absyn.BEGIN_DEFINITION((path = Absyn.QUALIFIED(_,_)),restr,p,e),oldp)
      equation 
        w = Absyn.strip_last(path) " For split definitions not at top,  eg. \"package A.B\" ";
        name = Absyn.path_last_ident(path);
        newclass = Absyn.CLASS(name,p,false,e,restr,Absyn.PARTS({Absyn.PUBLIC({})},NONE),
          "NoFile");
        newp = update_program(Absyn.PROGRAM({newclass},Absyn.WITHIN(w)), oldp); then newp;
    case (Absyn.PROGRAM((c1 = Absyn.CLASS(name,_,_,_,_,_,_)) :: c2,(w = Absyn.TOP())),(p2 = Absyn.PROGRAM(c3,w2)))
      local Absyn.Within w;
      equation 
        false = class_in_program(name, p2);
        pnew = update_program(Absyn.PROGRAM(c2,w), Absyn.PROGRAM(c1 :: c3,w2)); then pnew;
    case (Absyn.PROGRAM((c1 = Absyn.CLASS(name,_,_,_,_,_,_)) :: c2,(w = Absyn.TOP())),p2)
      local Absyn.Within w;
      equation 
        true = class_in_program(name, p2);
        pnew = update_program(Absyn.PROGRAM(c2,w), p2);
        pnew_1 = replace_class_in_program(c1, pnew); then pnew_1;
    case (Absyn.PROGRAM(c1 :: c2,(w = Absyn.WITHIN(path))),p2)
      local Absyn.Within w;
      equation 
        pnew = insert_class_in_program(c1, w, p2);
        pnew_1 = update_program(Absyn.PROGRAM(c2,w), pnew); then pnew_1;
    case (Absyn.COMP_DEFINITION(elt,SOME((path = Absyn.QUALIFIED(_,_)))),p)
      local Absyn.Program p;
      equation 
        cdef = get_pathed_class_in_program(path, p) " nested packages (of form A.B) ";
        modelwithin = Absyn.strip_last(path);
        newcdef = add_to_public(cdef, 
          Absyn.ELEMENTITEM(
          Absyn.ELEMENT(false,false,Absyn.UNSPECIFIED(),"",elt,"",0,NONE)));
        newp = update_program(Absyn.PROGRAM({newcdef},Absyn.WITHIN(modelwithin)), p); then newp;
    case (Absyn.COMP_DEFINITION(elt,SOME((path = Absyn.IDENT(_)))),p)
      local Absyn.Program p;
      equation 
        cdef = get_pathed_class_in_program(path, p) " top package ";
        newcdef = add_to_public(cdef, 
          Absyn.ELEMENTITEM(
          Absyn.ELEMENT(false,false,Absyn.UNSPECIFIED(),"",elt,"",0,NONE)));
        newp = update_program(Absyn.PROGRAM({newcdef},Absyn.TOP()), p); then newp;
    case (Absyn.IMPORT_DEFINITION(elt,SOME((path = Absyn.QUALIFIED(_,_)))),p)
      local Absyn.Program p;
      equation 
        cdef = get_pathed_class_in_program(path, p) " nested packages ( e.g. A.B ) ";
        modelwithin = Absyn.strip_last(path);
        newcdef = add_to_public(cdef, 
          Absyn.ELEMENTITEM(
          Absyn.ELEMENT(false,false,Absyn.UNSPECIFIED(),"",elt,"",0,NONE)));
        newp = update_program(Absyn.PROGRAM({newcdef},Absyn.WITHIN(modelwithin)), p); then newp;
    case (Absyn.IMPORT_DEFINITION(elt,SOME((path = Absyn.IDENT(_)))),p)
      local Absyn.Program p;
      equation 
        cdef = get_pathed_class_in_program(path, p) " top level package e.g. A ";
        newcdef = add_to_public(cdef, 
          Absyn.ELEMENTITEM(
          Absyn.ELEMENT(false,false,Absyn.UNSPECIFIED(),"",elt,"",0,NONE)));
        newp = update_program(Absyn.PROGRAM({newcdef},Absyn.TOP()), p); then newp;
    case (a,b)
      equation 
        Print.print_buf("Further program merging not implemented yet\n"); then b;
  end matchcontinue;
end update_program;

public function add_scope
  input Absyn.Program in_program;
  input InteractiveVariableList in_interactivevariablelist;
  output Absyn.Program out_program;
algorithm 
  out_program:=
  matchcontinue (in_program,in_interactivevariablelist)
    local
      type Absyn_ClassList = list<Absyn.Class>;
      Absyn.Path path,newpath,path2;
      Absyn_ClassList cls;
      InteractiveVariableList vars;
      Absyn.Within w;
      Absyn.Restriction restr;
      Boolean p,e;
      Absyn.ElementSpec elt;
    case (Absyn.PROGRAM(cls,Absyn.TOP()),vars)
      equation 
        Values.CODE(Absyn.C_TYPENAME(path)) = get_variable_value("scope", vars); then Absyn.PROGRAM(cls,Absyn.WITHIN(path));
    case (Absyn.PROGRAM(cls,w),vars)
      equation 
        failure(_ = get_variable_value("scope", vars)); then Absyn.PROGRAM(cls,w);
    case (Absyn.PROGRAM(cls,Absyn.WITHIN(path2)),vars)
      equation 
        Values.CODE(Absyn.C_TYPENAME(path)) = get_variable_value("scope", vars) " This should probably be forbidden. ";
        newpath = Absyn.join_paths(path, path2); then Absyn.PROGRAM(cls,Absyn.WITHIN(newpath));
    case (Absyn.BEGIN_DEFINITION(path2,restr,p,e),vars)
      equation 
        Values.CODE(Absyn.C_TYPENAME(path)) = get_variable_value("scope", vars);
        newpath = Absyn.join_paths(path, path2); then Absyn.BEGIN_DEFINITION(newpath,restr,p,e);
    case (Absyn.COMP_DEFINITION(elt,NONE),vars)
      equation 
        Values.CODE(Absyn.C_TYPENAME(path)) = get_variable_value("scope", vars); then Absyn.COMP_DEFINITION(elt,SOME(path));
    case (Absyn.IMPORT_DEFINITION(elt,NONE),vars)
      equation 
        Values.CODE(Absyn.C_TYPENAME(path)) = get_variable_value("scope", vars); then Absyn.IMPORT_DEFINITION(elt,SOME(path));
    case (p,_)
      local Absyn.Program p; then p;
  end matchcontinue;
end add_scope;

public function update_scope
  input Absyn.Program in_program;
  input InteractiveVariableList in_interactivevariablelist;
  output InteractiveVariableList out_interactivevariablelist;
algorithm 
  out_interactivevariablelist:=
  matchcontinue (in_program,in_interactivevariablelist)
    local
      Absyn.Path path,newscope,path_1;
      Values.Value newscope_1;
      InteractiveVariableList vars_1,vars;
      String id,id2,id1;
    case (Absyn.BEGIN_DEFINITION(Absyn.IDENT(id),_,_,_),vars)
      equation 
        Values.CODE(Absyn.C_TYPENAME(path)) = get_variable_value("scope", vars) " If not top scope ";
        newscope = Absyn.join_paths(path, Absyn.IDENT(id));
        newscope_1 = Values.CODE(Absyn.C_TYPENAME(newscope));
        vars_1 = add_var_to_varlist("scope", newscope_1, 
          (Types.T_COMPLEX(ClassInf.UNKNOWN("TypeName"),{}),NONE), vars); then vars_1;
    case (Absyn.BEGIN_DEFINITION(Absyn.IDENT(id),_,_,_),vars)
      equation 
        newscope_1 = Values.CODE(Absyn.C_TYPENAME(Absyn.IDENT(id))) " If top scope ";
        vars_1 = add_var_to_varlist("scope", newscope_1, 
          (Types.T_COMPLEX(ClassInf.UNKNOWN("TypeName"),{}),NONE), vars); then vars_1;
    case (Absyn.END_DEFINITION(id1),vars)
      equation 
        Values.CODE(Absyn.C_TYPENAME(path)) = get_variable_value("scope", vars) " If not top scope ";
        id2 = Absyn.path_last_ident(path);
        equality(id1 = id2);
        newscope = Absyn.strip_last(path);
        newscope_1 = Values.CODE(Absyn.C_TYPENAME(newscope));
        path_1 = Absyn.strip_last(path);
        vars_1 = add_var_to_varlist("scope", newscope_1, 
          (Types.T_COMPLEX(ClassInf.UNKNOWN("TypeName"),{}),NONE), vars); then vars_1;
    case (Absyn.END_DEFINITION(id1),vars)
      equation 
        Values.CODE(Absyn.C_TYPENAME(Absyn.IDENT(id2))) = get_variable_value("scope", vars);
        equality(id1 = id2);
        vars_1 = remove_var_from_varlist("scope", vars); then vars_1;
    case (_,vars) then vars; 
  end matchcontinue;
end update_scope;

protected function remove_var_from_varlist
  input Absyn.Ident in_ident;
  input InteractiveVariableList in_interactivevariablelist;
  output InteractiveVariableList out_interactivevariablelist;
algorithm 
  out_interactivevariablelist:=
  matchcontinue (in_ident,in_interactivevariablelist)
    local
      String id1,id2;
      InteractiveVariableList rest,rest_1;
      InteractiveVariable v;
    case (_,{}) then {}; 
    case (id1,IVAR(id2,_,_) :: rest)
      equation 
        equality(id1 = id2); then rest;
    case (id1,(v = IVAR(id2,_,_)) :: rest)
      equation 
        failure(equality(id1 = id2));
        rest_1 = remove_var_from_varlist(id1, rest); then v :: rest_1;
  end matchcontinue;
end remove_var_from_varlist;

protected function get_variable_value
  input Absyn.Ident in_ident;
  input InteractiveVariableList in_interactivevariablelist;
  output Values.Value out_value;
algorithm 
  out_value:=
  matchcontinue (in_ident,in_interactivevariablelist)
    local
      String id1,id2;
      Values.Value v;
      InteractiveVariableList rest;
    case (id1,IVAR(id2,v,_) :: _)
      equation 
        equality(id1 = id2); then v;
    case (id1,IVAR(id2,v,_) :: rest)
      equation 
        failure(equality(id1 = id2));
        v = get_variable_value(id1, rest); then v;
  end matchcontinue;
end get_variable_value;

protected function lookup_classdef "* function: lookup_classdef
 ** This function takes a Path of a class to lookup and a Path as a starting point for the lookup rules and a Program.
 ** It returns the Class definition and the complete Path to the class.
*"
  input Absyn.Path in_path1;
  input Absyn.Path in_path2;
  input Absyn.Program in_program3;
  output Absyn.Class out_class;
  output Absyn.Path out_path;
algorithm 
  (out_class,out_path):=
  matchcontinue (in_path1,in_path2,in_program3)
    local
      Absyn.Class inmodeldef,cdef;
      Absyn.Path newpath,path,inmodel,innewpath,respath,inpath;
      Absyn.Program p;
      String s1,s2;
    case (path,inmodel,p)
      equation 
        inmodeldef = get_pathed_class_in_program(inmodel, p) " Look first inside 'inmodel' ";
        cdef = get_pathed_class_in_program(path, Absyn.PROGRAM({inmodeldef},Absyn.TOP()));
        newpath = Absyn.join_paths(inmodel, path); then (cdef,newpath);
    case (path,inmodel,p) " Then look inside next level "
      equation 
        innewpath = Absyn.strip_last(inmodel);
        (cdef,respath) = lookup_classdef(path, innewpath, p); then (cdef,respath);
    case (path,_,p)
      equation 
        cdef = get_pathed_class_in_program(path, p) " Finally look in top level "; then (cdef,path);
    case (Absyn.IDENT("Real"),_,_) then (Absyn.CLASS("Real",false,false,false,Absyn.R_PREDEFINED_REAL(),
          Absyn.PARTS({},NONE),"NoFile"),Absyn.IDENT("Real")); 
    case (Absyn.IDENT("Integer"),_,_) then (Absyn.CLASS("Integer",false,false,false,Absyn.R_PREDEFINED_INT(),
          Absyn.PARTS({},NONE),"NoFile"),Absyn.IDENT("Integer")); 
    case (Absyn.IDENT("String"),_,_) then (Absyn.CLASS("String",false,false,false,Absyn.R_PREDEFINED_STRING(),
          Absyn.PARTS({},NONE),"NoFile"),Absyn.IDENT("String")); 
    case (Absyn.IDENT("Boolean"),_,_) then (Absyn.CLASS("Boolean",false,false,false,Absyn.R_PREDEFINED_BOOL(),
          Absyn.PARTS({},NONE),"NoFile"),Absyn.IDENT("Boolean")); 
    case (path,inpath,_)
      equation 
        Print.print_error_buf("#Error, lookup_classdef failed\n");
        s1 = Absyn.path_string(path);
        s2 = Absyn.path_string(inpath);
        Print.print_error_buf(s1);
        Print.print_error_buf(" in ");
        Print.print_error_buf(s2);
        Print.print_error_buf("\n"); then fail();
  end matchcontinue;
end lookup_classdef;

protected function delete_component
  input String in_string;
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  output Absyn.Program out_program;
  output String out_string;
algorithm 
  (out_program,out_string):=
  matchcontinue (in_string,in_componentref,in_program)
    local
      Absyn.Path modelpath,modelwithin;
      String name;
      Absyn.ComponentRef model_;
      Absyn.Program p,newp;
      Absyn.Class cdef,newcdef;
      Absyn.Within w;
    case (name,model_,p)
      equation 
        modelpath = Absyn.cref_to_path(model_);
        failure(_ = get_pathed_class_in_program(modelpath, p)); then (p,"false\n");
    case (name,(model_ = Absyn.CREF_QUAL(_,_,_)),(p = Absyn.PROGRAM(_,w)))
      equation 
        modelpath = Absyn.cref_to_path(model_);
        modelwithin = Absyn.strip_last(modelpath);
        cdef = get_pathed_class_in_program(modelpath, p);
        newcdef = delete_component_from_class(name, cdef);
        newp = update_program(Absyn.PROGRAM({newcdef},Absyn.WITHIN(modelwithin)), p); then (newp,"true\n");
    case (name,(model_ = Absyn.CREF_IDENT(_,_)),(p = Absyn.PROGRAM(_,w)))
      equation 
        modelpath = Absyn.cref_to_path(model_);
        cdef = get_pathed_class_in_program(modelpath, p);
        newcdef = delete_component_from_class(name, cdef);
        newp = update_program(Absyn.PROGRAM({newcdef},Absyn.TOP()), p); then (newp,"true\n");
    case (_,_,p) then (p,"false\n"); 
  end matchcontinue;
end delete_component;

protected function delete_component_from_class
  input String in_string;
  input Absyn.Class in_class;
  output Absyn.Class out_class;
algorithm 
  out_class:=
  matchcontinue (in_string,in_class)
    local
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      type StringOption = Option<String>;
      Absyn_ElementItemList publst,publst2,protlst,protlst2;
      Integer l2,l1,l1_1;
      Absyn_ClassPartList parts2,parts;
      String name,i,file;
      Boolean p,f,e;
      Absyn.Restriction r;
      StringOption cmt;
    case (name,Absyn.CLASS(i,p,f,e,r,Absyn.PARTS(parts,cmt),file)) " Search in public list "
      equation 
        publst = get_public_list(parts);
        publst2 = delete_component_from_elementitems(name, publst);
        l2 = list_length(publst2);
        l1 = list_length(publst);
        l1_1 = l1 - 1;
        equality(l1_1 = l2);
        parts2 = replace_public_list(parts, publst2); then Absyn.CLASS(i,p,f,e,r,Absyn.PARTS(parts2,cmt),file);
    case (name,Absyn.CLASS(i,p,f,e,r,Absyn.PARTS(parts,cmt),file)) " Search in protected list "
      equation 
        protlst = get_protected_list(parts);
        protlst2 = delete_component_from_elementitems(name, protlst);
        l2 = list_length(protlst2);
        l1 = list_length(protlst);
        l1_1 = l1 - 1;
        equality(l1_1 = l2);
        parts2 = replace_protected_list(parts, protlst2); then Absyn.CLASS(i,p,f,e,r,Absyn.PARTS(parts2,cmt),file);
  end matchcontinue;
end delete_component_from_class;

protected function delete_component_from_elementitems
  input String in_string;
  input Absyn_ElementItemList in_absyn_elementitemlist;
  output Absyn_ElementItemList out_absyn_elementitemlist;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
algorithm 
  out_absyn_elementitemlist:=
  matchcontinue (in_string,in_absyn_elementitemlist)
    local
      String name,name2;
      Absyn_ElementItemList xs,res;
      Absyn.ElementItem x;
    case (_,{}) then {}; 
    case (name,Absyn.ELEMENTITEM(Absyn.ELEMENT(_,_,_,_,Absyn.COMPONENTS(_,_,{Absyn.COMPONENTITEM(Absyn.COMPONENT(name2,_,_),_)}),_,_,_)) :: xs)
      equation 
        equality(name = name2); then xs;
    case (name,(x = Absyn.ELEMENTITEM(Absyn.ELEMENT(_,_,_,_,Absyn.COMPONENTS(_,_,{Absyn.COMPONENTITEM(Absyn.COMPONENT(name2,_,_),_)}),_,_,_))) :: xs)
      equation 
        failure(equality(name = name2));
        res = delete_component_from_elementitems(name, xs); then x :: res;
    case (name,x :: xs)
      equation 
        res = delete_component_from_elementitems(name, xs); then x :: res;
  end matchcontinue;
end delete_component_from_elementitems;

public function add_component "* function add_component
 ** This function takes:  arg1 - string giving the instancename, arg2 - `ComponentRef' giving the component type
 ** arg3 - ComponentRef giving the model to instantiate the component within, arg4 - `NamedArg' list of annotations 
 ** and arg5 - a Program. The result is an updated program with the component and its annotations inserted, and a string
 ** \"OK\" for success. If the insertion fails, a suitable error string is given along with the input Program.
 *"
  input String in_string1;
  input Absyn.ComponentRef in_componentref2;
  input Absyn.ComponentRef in_componentref3;
  input Absyn_NamedArgList in_absyn_namedarglist4;
  input Absyn.Program in_program5;
  output Absyn.Program out_program;
  output String out_string;
protected 
  type Absyn_NamedArgList = list<Absyn.NamedArg>;
algorithm 
  (out_program,out_string):=
  matchcontinue (in_string1,in_componentref2,in_componentref3,in_absyn_namedarglist4,in_program5)
    local
      type Absyn_CommentOption = Option<Absyn.Comment>;
      type Absyn_ModificationOption = Option<Absyn.Modification>;
      Absyn.Path modelpath,modelwithin,tppath;
      String name;
      Absyn.ComponentRef tp,model_;
      Absyn_NamedArgList nargs;
      Absyn.Program p,newp;
      Absyn.Class cdef,newcdef;
      Absyn_CommentOption annotation_;
      Absyn_ModificationOption modification;
      Absyn.Within w;
    case (name,tp,model_,nargs,p)
      equation 
        modelpath = Absyn.cref_to_path(model_);
        failure(_ = get_pathed_class_in_program(modelpath, p)); then (p,"false\n");
    case (name,tp,(model_ = Absyn.CREF_QUAL(_,_,_)),nargs,(p = Absyn.PROGRAM(_,w))) " Adding component to model that resides inside package "
      equation 
        modelpath = Absyn.cref_to_path(model_);
        modelwithin = Absyn.strip_last(modelpath);
        cdef = get_pathed_class_in_program(modelpath, p);
        tppath = Absyn.cref_to_path(tp);
        annotation_ = annotation_list_to_absyn_comment(nargs, NONE);
        modification = modification_to_absyn(nargs, NONE);
        newcdef = add_to_public(cdef, 
          Absyn.ELEMENTITEM(
          Absyn.ELEMENT(false,false,Absyn.UNSPECIFIED(),"",
          Absyn.COMPONENTS(Absyn.ATTR(false,Absyn.VAR(),Absyn.BIDIR(),{}),tppath,
          {
          Absyn.COMPONENTITEM(Absyn.COMPONENT(name,{},modification),annotation_)}),"",0,NONE)));
        newp = update_program(Absyn.PROGRAM({newcdef},Absyn.WITHIN(modelwithin)), p); then (newp,"Ok\n");
    case (name,tp,(model_ = Absyn.CREF_IDENT(_,_)),nargs,(p = Absyn.PROGRAM(_,w))) " Adding component to model that resides on top level "
      equation 
        modelpath = Absyn.cref_to_path(model_);
        cdef = get_pathed_class_in_program(modelpath, p);
        tppath = Absyn.cref_to_path(tp);
        annotation_ = annotation_list_to_absyn_comment(nargs, NONE);
        modification = modification_to_absyn(nargs, NONE);
        newcdef = add_to_public(cdef, 
          Absyn.ELEMENTITEM(
          Absyn.ELEMENT(false,false,Absyn.UNSPECIFIED(),"",
          Absyn.COMPONENTS(Absyn.ATTR(false,Absyn.VAR(),Absyn.BIDIR(),{}),tppath,
          {
          Absyn.COMPONENTITEM(Absyn.COMPONENT(name,{},modification),annotation_)}),"",0,NONE)));
        newp = update_program(Absyn.PROGRAM({newcdef},Absyn.TOP()), p); then (newp,"Ok\n");
    case (_,_,_,_,p) then (p,"Error"); 
  end matchcontinue;
end add_component;

protected function update_component
  input String in_string1;
  input Absyn.ComponentRef in_componentref2;
  input Absyn.ComponentRef in_componentref3;
  input Absyn_NamedArgList in_absyn_namedarglist4;
  input Absyn.Program in_program5;
  output Absyn.Program out_program;
  output String out_string;
protected 
  type Absyn_NamedArgList = list<Absyn.NamedArg>;
algorithm 
  (out_program,out_string):=
  matchcontinue (in_string1,in_componentref2,in_componentref3,in_absyn_namedarglist4,in_program5)
    local
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
      type Absyn_ModificationOption = Option<Absyn.Modification>;
      type Absyn_CommentOption = Option<Absyn.Comment>;
      Absyn.Path modelpath,modelwithin,tp,tppath;
      Absyn.Program p_1,newp,p;
      Absyn_ClassPartList parts;
      Absyn.Class cdef,newcdef;
      Absyn_ElementItemList publst,protlst;
      Absyn.ElementAttributes attr;
      Absyn_ComponentItemList items;
      Absyn_ModificationOption mod,modification;
      Absyn_CommentOption ann,annotation_;
      String name;
      Absyn.ComponentRef model_;
      Absyn_NamedArgList nargs;
      Absyn.Within w;
    case (name,tp,(model_ = Absyn.CREF_QUAL(_,_,_)),nargs,(p = Absyn.PROGRAM(_,w)))
      equation 
        modelpath = Absyn.cref_to_path(model_) " Updating a public component to model that resides inside package ";
        modelwithin = Absyn.strip_last(modelpath);
        (p_1,_) = delete_component(name, model_, p);
        Absyn.CLASS(_,_,_,_,_,Absyn.PARTS(parts,_),_) = get_pathed_class_in_program(modelpath, p);
        cdef = get_pathed_class_in_program(modelpath, p_1);
        publst = get_public_list(parts);
        Absyn.COMPONENTS(attr,tp,items) = get_components_contains_name(Absyn.CREF_IDENT(name,{}), publst);
        Absyn.COMPONENTITEM(Absyn.COMPONENT(_,_,mod),ann) = get_compitem_named(Absyn.CREF_IDENT(name,{}), items);
        annotation_ = annotation_list_to_absyn_comment(nargs, ann);
        modification = modification_to_absyn(nargs, mod);
        newcdef = add_to_public(cdef, 
          Absyn.ELEMENTITEM(
          Absyn.ELEMENT(false,false,Absyn.UNSPECIFIED(),"",
          Absyn.COMPONENTS(attr,tp,
          {
          Absyn.COMPONENTITEM(Absyn.COMPONENT(name,{},modification),annotation_)}),"",0,NONE)));
        newp = update_program(Absyn.PROGRAM({newcdef},Absyn.WITHIN(modelwithin)), p); then (newp,"true");
    case (name,tp,(model_ = Absyn.CREF_QUAL(_,_,_)),nargs,(p = Absyn.PROGRAM(_,w)))
      equation 
        modelpath = Absyn.cref_to_path(model_) " Updating a protected component to model that resides inside package";
        modelwithin = Absyn.strip_last(modelpath);
        (p_1,_) = delete_component(name, model_, p);
        Absyn.CLASS(_,_,_,_,_,Absyn.PARTS(parts,_),_) = get_pathed_class_in_program(modelpath, p);
        cdef = get_pathed_class_in_program(modelpath, p_1);
        protlst = get_protected_list(parts);
        Absyn.COMPONENTS(attr,tp,items) = get_components_contains_name(Absyn.CREF_IDENT(name,{}), protlst);
        Absyn.COMPONENTITEM(Absyn.COMPONENT(_,_,mod),ann) = get_compitem_named(Absyn.CREF_IDENT(name,{}), items);
        annotation_ = annotation_list_to_absyn_comment(nargs, ann);
        modification = modification_to_absyn(nargs, mod);
        newcdef = add_to_protected(cdef, 
          Absyn.ELEMENTITEM(
          Absyn.ELEMENT(false,false,Absyn.UNSPECIFIED(),"",
          Absyn.COMPONENTS(attr,tp,
          {
          Absyn.COMPONENTITEM(Absyn.COMPONENT(name,{},modification),annotation_)}),"",0,NONE)));
        newp = update_program(Absyn.PROGRAM({newcdef},Absyn.WITHIN(modelwithin)), p); then (newp,"true\n");
    case (name,tp,(model_ = Absyn.CREF_IDENT(_,_)),nargs,(p = Absyn.PROGRAM(_,w)))
      equation 
        modelpath = Absyn.cref_to_path(model_) " Updating a public component to model that resides on top level ";
        cdef = get_pathed_class_in_program(modelpath, p);
        tppath = Absyn.cref_to_path(tp);
        (p_1,_) = delete_component(name, model_, p);
        cdef = get_pathed_class_in_program(modelpath, p_1);
        Absyn.CLASS(_,_,_,_,_,Absyn.PARTS(parts,_),_) = get_pathed_class_in_program(modelpath, p);
        publst = get_public_list(parts);
        Absyn.COMPONENTS(attr,tp,items) = get_components_contains_name(Absyn.CREF_IDENT(name,{}), publst);
        Absyn.COMPONENTITEM(Absyn.COMPONENT(_,_,mod),ann) = get_compitem_named(Absyn.CREF_IDENT(name,{}), items);
        annotation_ = annotation_list_to_absyn_comment(nargs, ann);
        modification = modification_to_absyn(nargs, mod);
        newcdef = add_to_public(cdef, 
          Absyn.ELEMENTITEM(
          Absyn.ELEMENT(false,false,Absyn.UNSPECIFIED(),"",
          Absyn.COMPONENTS(attr,tppath,
          {
          Absyn.COMPONENTITEM(Absyn.COMPONENT(name,{},modification),annotation_)}),"",0,NONE)));
        newp = update_program(Absyn.PROGRAM({newcdef},Absyn.TOP()), p); then (newp,"true\n");
    case (name,tp,(model_ = Absyn.CREF_IDENT(_,_)),nargs,(p = Absyn.PROGRAM(_,w)))
      equation 
        modelpath = Absyn.cref_to_path(model_) " Updating a protected component to model that resides on top level ";
        cdef = get_pathed_class_in_program(modelpath, p);
        tppath = Absyn.cref_to_path(tp);
        (p_1,_) = delete_component(name, model_, p);
        cdef = get_pathed_class_in_program(modelpath, p_1);
        Absyn.CLASS(_,_,_,_,_,Absyn.PARTS(parts,_),_) = get_pathed_class_in_program(modelpath, p);
        protlst = get_protected_list(parts);
        Absyn.COMPONENTS(attr,tp,items) = get_components_contains_name(Absyn.CREF_IDENT(name,{}), protlst);
        Absyn.COMPONENTITEM(Absyn.COMPONENT(_,_,mod),ann) = get_compitem_named(Absyn.CREF_IDENT(name,{}), items);
        annotation_ = annotation_list_to_absyn_comment(nargs, ann);
        modification = modification_to_absyn(nargs, mod);
        newcdef = add_to_protected(cdef, 
          Absyn.ELEMENTITEM(
          Absyn.ELEMENT(false,false,Absyn.UNSPECIFIED(),"",
          Absyn.COMPONENTS(attr,tppath,
          {
          Absyn.COMPONENTITEM(Absyn.COMPONENT(name,{},modification),annotation_)}),"",0,NONE)));
        newp = update_program(Absyn.PROGRAM({newcdef},Absyn.TOP()), p); then (newp,"true\n");
  end matchcontinue;
end update_component;

protected function add_class_annotation "* function:add_class_annotation 
 ** This function takes a `ComponentRef' and an `Exp' expression and a `Program' and adds the 
 ** expression as a annotation to the specified model in the program, 
 ** returning the updated program.
*"
  input Absyn.ComponentRef in_componentref;
  input Absyn_NamedArgList in_absyn_namedarglist;
  input Absyn.Program in_program;
  output Absyn.Program out_program;
protected 
  type Absyn_NamedArgList = list<Absyn.NamedArg>;
algorithm 
  out_program:=
  matchcontinue (in_componentref,in_absyn_namedarglist,in_program)
    local
      Absyn.Path modelpath,modelwithin;
      Absyn.Class cdef,cdef_1;
      Absyn.Program newp,p;
      Absyn.ComponentRef model_;
      Absyn_NamedArgList nargs;
    case ((model_ = Absyn.CREF_QUAL(_,_,_)),nargs,p)
      equation 
        modelpath = Absyn.cref_to_path(model_) " Class inside other class ";
        modelwithin = Absyn.strip_last(modelpath);
        cdef = get_pathed_class_in_program(modelpath, p);
        cdef_1 = add_class_annotation_to_class(cdef, nargs);
        newp = update_program(Absyn.PROGRAM({cdef_1},Absyn.WITHIN(modelwithin)), p); then newp;
    case ((model_ = Absyn.CREF_IDENT(_,_)),nargs,p)
      equation 
        modelpath = Absyn.cref_to_path(model_) " Class on top level ";
        cdef = get_pathed_class_in_program(modelpath, p);
        cdef_1 = add_class_annotation_to_class(cdef, nargs);
        newp = update_program(Absyn.PROGRAM({cdef_1},Absyn.TOP()), p); then newp;
  end matchcontinue;
end add_class_annotation;

protected function add_class_annotation_to_class
  input Absyn.Class in_class;
  input Absyn_NamedArgList in_absyn_namedarglist;
  output Absyn.Class out_class;
protected 
  type Absyn_NamedArgList = list<Absyn.NamedArg>;
algorithm 
  out_class:=
  matchcontinue (in_class,in_absyn_namedarglist)
    local
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      type StringOption = Option<String>;
      Absyn_ElementItemList publst,publst2;
      Absyn.Annotation annotation_,oldann,newann,newann_1;
      Absyn.Class cdef_1,cdef;
      Absyn_ClassPartList parts,parts2;
      Absyn_NamedArgList nargs;
      String i,file;
      Boolean p,f,e;
      Absyn.Restriction r;
      StringOption cmt;
    case ((cdef = Absyn.CLASS(_,_,_,_,_,Absyn.PARTS(parts,_),_)),nargs)
      equation 
        publst = get_public_list(parts) " No annotation element found in class ";
        failure(_ = get_element_annotation_in_elements(publst));
        annotation_ = annotation_list_to_absyn(nargs);
        cdef_1 = add_to_public(cdef, Absyn.ANNOTATIONITEM(annotation_)); then cdef_1;
    case ((cdef = Absyn.CLASS(i,p,f,e,r,Absyn.PARTS(parts,cmt),file)),nargs)
      equation 
        publst = get_public_list(parts);
        Absyn.ANNOTATIONITEM(oldann) = get_element_annotation_in_elements(publst);
        newann = annotation_list_to_absyn(nargs);
        newann_1 = merge_annotations(oldann, newann);
        publst2 = replace_element_annotation_in_elements(publst, newann_1);
        parts2 = replace_public_list(parts, publst2); then Absyn.CLASS(i,p,f,e,r,Absyn.PARTS(parts2,cmt),file);
  end matchcontinue;
end add_class_annotation_to_class;

protected function replace_element_annotation_in_elements
  input Absyn_ElementItemList in_absyn_elementitemlist;
  input Absyn.Annotation in_annotation;
  output Absyn_ElementItemList out_absyn_elementitemlist;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
algorithm 
  out_absyn_elementitemlist:=
  matchcontinue (in_absyn_elementitemlist,in_annotation)
    local
      Absyn_ElementItemList xs,res;
      Absyn.Annotation a,a2;
    case (Absyn.ANNOTATIONITEM(_) :: xs,a) then Absyn.ANNOTATIONITEM(a) :: xs; 
    case (a :: xs,a2)
      local Absyn.ElementItem a;
      equation 
        res = replace_element_annotation_in_elements(xs, a2); then a :: res;
    case ({},_) then {}; 
  end matchcontinue;
end replace_element_annotation_in_elements;

protected function get_element_annotation_in_elements
  input Absyn_ElementItemList in_absyn_elementitemlist;
  output Absyn.ElementItem out_elementitem;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
algorithm 
  out_elementitem:=
  matchcontinue (in_absyn_elementitemlist)
    local
      Absyn.ElementItem a;
      Absyn_ElementItemList xs;
    case ((a = Absyn.ANNOTATIONITEM(_)) :: xs) then a; 
    case (_ :: xs)
      equation 
        a = get_element_annotation_in_elements(xs); then a;
  end matchcontinue;
end get_element_annotation_in_elements;

protected function merge_annotations "* function: merge_annotations
 *
 * This function takes an old annotation as first argument and a new annotation as 
 * second argument and merges the two.
 * Annotation \"parts\" that exist in both the old and the new annotation will be changed
 * according to the new definition. For instance,
 * merge_annotations(annotation(x=1,y=2),annotation(x=3)) => annotation(x=3,y=2)
 *"
  input Absyn.Annotation in_annotation1;
  input Absyn.Annotation in_annotation2;
  output Absyn.Annotation out_annotation;
algorithm 
  out_annotation:=
  matchcontinue (in_annotation1,in_annotation2)
    local
      type Absyn_ElementArgList = list<Absyn.ElementArg>;
      Absyn_ElementArgList neweltargs,oldrest,eltargs,eltargs_1;
      Absyn.ElementArg mod;
      Absyn.ComponentRef cr;
      Absyn.Annotation a;
    case (Absyn.ANNOTATION((mod = Absyn.MODIFICATION(_,_,cr,_,_)) :: oldrest),Absyn.ANNOTATION(eltargs))
      equation 
        failure(_ = remove_modification_in_elementargs(eltargs, cr));
        Absyn.ANNOTATION(neweltargs) = merge_annotations(Absyn.ANNOTATION(oldrest), Absyn.ANNOTATION(eltargs)); then Absyn.ANNOTATION(mod :: neweltargs);
    case (Absyn.ANNOTATION((mod = Absyn.MODIFICATION(_,_,cr,_,_)) :: oldrest),Absyn.ANNOTATION(eltargs))
      equation 
        eltargs_1 = remove_modification_in_elementargs(eltargs, cr);
        Absyn.ANNOTATION(neweltargs) = merge_annotations(Absyn.ANNOTATION(oldrest), Absyn.ANNOTATION(eltargs)); then Absyn.ANNOTATION(neweltargs);
    case (Absyn.ANNOTATION({}),a) then a; 
  end matchcontinue;
end merge_annotations;

protected function remove_modification_in_elementargs
  input Absyn_ElementArgList in_absyn_elementarglist;
  input Absyn.ComponentRef in_componentref;
  output Absyn_ElementArgList out_absyn_elementarglist;
protected 
  type Absyn_ElementArgList = list<Absyn.ElementArg>;
algorithm 
  out_absyn_elementarglist:=
  matchcontinue (in_absyn_elementarglist,in_componentref)
    local
      String id,id2;
      Absyn.ComponentRef cr;
      Absyn.ElementArg m;
      Absyn_ElementArgList res,xs;
    case ({Absyn.MODIFICATION(_,_,Absyn.CREF_IDENT(id,_),_,_)},(cr = Absyn.CREF_IDENT(id2,_)))
      equation 
        equality(id = id2); then {};
    case ({(m = Absyn.MODIFICATION(_,_,Absyn.CREF_IDENT(id,_),_,_))},(cr = Absyn.CREF_IDENT(id2,_)))
      equation 
        failure(equality(id = id2)); then fail();
    case (Absyn.MODIFICATION(_,_,Absyn.CREF_IDENT(id,_),_,_) :: xs,(cr = Absyn.CREF_IDENT(id2,_)))
      equation 
        equality(id = id2);
        res = remove_modification_in_elementargs(xs, cr); then res;
    case ((m = Absyn.MODIFICATION(_,_,Absyn.CREF_IDENT(id,_),_,_)) :: xs,(cr = Absyn.CREF_IDENT(id2,_)))
      equation 
        failure(equality(id = id2));
        res = remove_modification_in_elementargs(xs, cr); then m :: res;
    case ((m = Absyn.MODIFICATION(_,_,Absyn.CREF_IDENT(id,_),_,_)) :: xs,(cr = Absyn.CREF_IDENT(id2,_)))
      equation 
        res = remove_modification_in_elementargs(xs, cr); then m :: res;
  end matchcontinue;
end remove_modification_in_elementargs;

protected function get_inheritance_count "* function: get_inheritance_count
 ** This function takes a `ComponentRef' and a `Program' and returns the number of inherited classes in the 
 ** class referenced by the `ComponentRef'.
   *"
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  output Integer out_integer;
algorithm 
  out_integer:=
  matchcontinue (in_componentref,in_program)
    local
      Absyn.Path modelpath;
      Absyn.Class cdef;
      Integer count;
      Absyn.ComponentRef model_;
      Absyn.Program p;
    case (model_,p)
      equation 
        modelpath = Absyn.cref_to_path(model_);
        cdef = get_pathed_class_in_program(modelpath, p);
        count = count_base_classes(cdef); then count;
    case (_,_) then 0; 
  end matchcontinue;
end get_inheritance_count;

protected function get_nth_inherited_class "* function: get_inheritance_count
 ** This function takes a `ComponentRef' and a `Program' and returns the number of inherited classes in the 
 ** class referenced by the `ComponentRef'.
   *"
  input Absyn.ComponentRef in_componentref;
  input Integer in_integer;
  input Absyn.Program in_program;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_componentref,in_integer,in_program)
    local
      type SCode_ClassList = list<SCode.Class>;
      type Env_FrameList = list<Env.Frame>;
      Absyn.Path modelpath;
      Absyn.Class cdef;
      SCode_ClassList p_1;
      Env_FrameList env,env_1;
      SCode.Class c;
      String id,str;
      Boolean encflag;
      SCode.Restriction restr;
      Absyn.ComponentRef model_;
      Integer n;
      Absyn.Program p;
    case (model_,n,p)
      equation 
        modelpath = Absyn.cref_to_path(model_);
        cdef = get_pathed_class_in_program(modelpath, p);
        p_1 = SCode.elaborate(p);
        env = Inst.make_env_from_program(p_1, Absyn.IDENT(""));
        (c = SCode.CLASS(id,_,encflag,restr,_),env_1) = Lookup.lookup_class(env, modelpath, false);
        str = get_nth_inherited_class2(c, cdef, n, env_1); then str;
    case (_,_,_) then "Error"; 
  end matchcontinue;
end get_nth_inherited_class;

protected function get_nth_inherited_class2
  input SCode.Class in_class1;
  input Absyn.Class in_class2;
  input Integer in_integer3;
  input Env.Env in_env4;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_class1,in_class2,in_integer3,in_env4)
    local
      type Absyn_ComponentRefList = list<Absyn.ComponentRef>;
      type Env_FrameList = list<Env.Frame>;
      Absyn_ComponentRefList lst;
      Integer n_1,n;
      Absyn.ComponentRef cref;
      Absyn.Path path;
      String str,id;
      SCode.Class c;
      Absyn.Class cdef;
      Env_FrameList env,env2,env_2;
      ClassInf.State ci_state;
      Boolean encflag;
      SCode.Restriction restr;
    case ((c = SCode.CLASS(_,_,_,_,_)),cdef,n,env)
      equation 
        lst = get_base_classes(cdef, env) " First try without instantiating, if class is in parents";
        n_1 = n - 1;
        cref = list_nth(lst, n_1);
        path = Absyn.cref_to_path(cref);
        str = Absyn.path_string(path); then str;
    case ((c = SCode.CLASS(id,_,encflag,restr,_)),cdef,n,env)
      equation 
        env2 = Env.open_scope(env, encflag, SOME(id)) " If that fails, instantiate, which takes more time";
        ci_state = ClassInf.start(restr, id);
        (env_2,_) = Inst.partial_inst_class_in(env2, Types.NOMOD(), Prefix.NOPRE, {}, ci_state, c, false, 
          {});
        lst = get_base_classes(cdef, env_2);
        n_1 = n - 1;
        cref = list_nth(lst, n_1);
        path = Absyn.cref_to_path(cref);
        str = Absyn.path_string(path); then str;
  end matchcontinue;
end get_nth_inherited_class2;

public function get_component_count "* function: get_component_count
 ** This function takes a `ComponentRef' and a `Program' and returns the number of public components in the 
 ** class referenced by the `ComponentRef'.
   *"
  input Absyn.ComponentRef model_;
  input Absyn.Program p;
  output Integer count;
protected 
  Absyn.Path modelpath;
  Absyn.Class cdef;
algorithm 
  modelpath := Absyn.cref_to_path(model_);
  cdef := get_pathed_class_in_program(modelpath, p);
  count := count_components(cdef);
end get_component_count;

protected function count_components "* function: count_components
 ** This function takes a `Class' and returns the number of components in that class
*"
  input Absyn.Class in_class;
  output Integer out_integer;
algorithm 
  out_integer:=
  matchcontinue (in_class)
    local
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      type StringOption = Option<String>;
      Integer c1,c2,res;
      String a,file;
      Boolean b,c,d;
      Absyn.Restriction e;
      Absyn_ElementItemList elt;
      Absyn_ClassPartList lst;
      StringOption cmt;
    case Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(Absyn.PUBLIC(elt) :: lst,cmt),file)
      equation 
        c1 = count_components(Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(lst,cmt),file));
        c2 = count_components_in_elts(elt); then c1 + c2;
    case Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(Absyn.PROTECTED(elt) :: lst,cmt),file)
      equation 
        c1 = count_components(Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(lst,cmt),file));
        c2 = count_components_in_elts(elt); then c1 + c2;
    case Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(_ :: lst,cmt),file)
      equation 
        res = count_components(Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(lst,cmt),file)); then res;
    case Absyn.CLASS(a,b,c,d,e,Absyn.PARTS({},_),_) then 0; 
    case Absyn.CLASS(a,b,c,d,e,Absyn.DERIVED(_,_,_,_,_),_) then -1; 
  end matchcontinue;
end count_components;

protected function count_components_in_elts
  input Absyn_ElementItemList in_absyn_elementitemlist;
  output Integer out_integer;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
algorithm 
  out_integer:=
  matchcontinue (in_absyn_elementitemlist)
    local
      type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
      Integer c1,ncomps,res;
      Absyn_ComponentItemList complst;
      Absyn_ElementItemList lst;
    case (Absyn.ELEMENTITEM(Absyn.ELEMENT(_,_,_,_,Absyn.COMPONENTS(_,_,complst),_,_,NONE)) :: lst)
      equation 
        c1 = count_components_in_elts(lst);
        ncomps = list_length(complst); then c1 + ncomps;
    case (_ :: lst)
      equation 
        res = count_components_in_elts(lst); then res;
    case ({}) then 0; 
  end matchcontinue;
end count_components_in_elts;

protected function get_nth_component "* function: get_nth_component
 ** This function takes a `ComponentRef', a `Program' and an int and  returns a comma separated string of names
 ** containing the name, type and comment of that component.
 *"
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  input Integer in_integer;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_componentref,in_program,in_integer)
    local
      type SCode_ClassList = list<SCode.Class>;
      type Env_FrameList = list<Env.Frame>;
      Absyn.Path modelpath;
      SCode_ClassList p_1;
      Env_FrameList env,env_1;
      SCode.Class c;
      String id,str;
      Boolean encflag;
      SCode.Restriction restr;
      Absyn.Class cdef;
      Absyn.ComponentRef model_;
      Absyn.Program p;
      Integer n;
    case (model_,p,n)
      equation 
        modelpath = Absyn.cref_to_path(model_);
        p_1 = SCode.elaborate(p);
        env = Inst.make_env_from_program(p_1, Absyn.IDENT(""));
        (c = SCode.CLASS(id,_,encflag,restr,_),env_1) = Lookup.lookup_class(env, modelpath, false);
        cdef = get_pathed_class_in_program(modelpath, p);
        str = get_nth_component2(c, cdef, n, env_1); then str;
    case (_,_,_) then "Error"; 
  end matchcontinue;
end get_nth_component;

protected function get_nth_component2
  input SCode.Class in_class1;
  input Absyn.Class in_class2;
  input Integer in_integer3;
  input Env.Env in_env4;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_class1,in_class2,in_integer3,in_env4)
    local
      type Env_FrameList = list<Env.Frame>;
      Env_FrameList env2,env_2,env;
      ClassInf.State ci_state;
      Absyn.Element comp;
      String s1,s2,str,id;
      SCode.Class c;
      Boolean encflag;
      SCode.Restriction restr;
      Absyn.Class cdef;
      Integer n;
    case ((c = SCode.CLASS(id,_,encflag,restr,_)),cdef,n,env) "rule	
	get_nth_component_in_class(cdef,n) => comp &
	get_component_name_and_type_and_comment (comp,env) => {s1} &
	string_append(\"{\",s1) => s2 &
	string_append(s2,\"}\") => str 
	--------------------------
	get_nth_component2 (c as SCode.CLASS(id,_,encflag,restr,_),cdef,n,env)
	  => str
"
      equation 
        env2 = Env.open_scope(env, encflag, SOME(id)) " If fails, instantiate class and search... ";
        ci_state = ClassInf.start(restr, id);
        (env_2,_) = Inst.partial_inst_class_in(env2, Types.NOMOD(), Prefix.NOPRE, {}, ci_state, c, false, 
          {});
        comp = get_nth_component_in_class(cdef, n);
        {s1} = get_component_name_and_type_and_comment(comp, env_2);
        s2 = string_append("{", s1);
        str = string_append(s2, "}"); then str;
    case (_,_,_,_)
      equation 
        print("get_nth_component2 failed\n"); then fail();
  end matchcontinue;
end get_nth_component2;

public function get_components "* function: get_components
 ** This function takes a `ComponentRef', a `Program' and an int and  returns 
 ** a list of all components, as returned by get_nth_component.
 *"
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_componentref,in_program)
    local
      type SCode_ClassList = list<SCode.Class>;
      type Env_FrameList = list<Env.Frame>;
      type Absyn_ElementList = list<Absyn.Element>;
      Absyn.Path modelpath;
      Absyn.Class cdef;
      SCode_ClassList p_1;
      Env_FrameList env,env_1,env2,env_2;
      SCode.Class c;
      String id,s1,s2,str;
      Boolean encflag;
      SCode.Restriction restr;
      ClassInf.State ci_state;
      Absyn_ElementList comps;
      Absyn.ComponentRef model_;
      Absyn.Program p;
    case (model_,p)
      equation 
        modelpath = Absyn.cref_to_path(model_);
        cdef = get_pathed_class_in_program(modelpath, p);
        p_1 = SCode.elaborate(p);
        env = Inst.make_env_from_program(p_1, Absyn.IDENT(""));
        (c = SCode.CLASS(id,_,encflag,restr,_),env_1) = Lookup.lookup_class(env, modelpath, false);
        env2 = Env.open_scope(env_1, encflag, SOME(id));
        ci_state = ClassInf.start(restr, id);
        (env_2,_) = Inst.partial_inst_class_in(env2, Types.NOMOD(), Prefix.NOPRE, {}, ci_state, c, false, 
          {});
        comps = get_components_in_class(cdef);
        s1 = get_components_name_and_type_and_comment(comps, env_2);
        s2 = string_append("{", s1);
        str = string_append(s2, "}"); then str;
    case (_,_) then "Error"; 
  end matchcontinue;
end get_components;

protected function get_component_annotations
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_componentref,in_program)
    local
      type Absyn_ElementList = list<Absyn.Element>;
      Absyn.Path modelpath;
      Absyn.Class cdef;
      Absyn_ElementList comps;
      String s1,s2,str;
      Absyn.ComponentRef model_;
      Absyn.Program p;
    case (model_,p)
      equation 
        modelpath = Absyn.cref_to_path(model_);
        cdef = get_pathed_class_in_program(modelpath, p);
        comps = get_components_in_class(cdef);
        s1 = get_component_annotations_from_elts(comps);
        s2 = string_append("{", s1);
        str = string_append(s2, "}"); then str;
    case (_,_) then "Error"; 
  end matchcontinue;
end get_component_annotations;

protected function get_nth_component_annotation "* function: get_nth_component_annotation
 ** This function takes a `ComponentRef', a `Program' and an int and  returns a comma separated string of values
 ** corresponding to the flat record for component annotations.
 *"
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  input Integer in_integer;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_componentref,in_program,in_integer)
    local
      Absyn.Path modelpath;
      Absyn.Class cdef;
      Absyn.Element comp;
      String s1,s2,str;
      Absyn.ComponentRef model_;
      Absyn.Program p;
      Integer n;
    case (model_,p,n)
      equation 
        modelpath = Absyn.cref_to_path(model_);
        cdef = get_pathed_class_in_program(modelpath, p);
        comp = get_nth_component_in_class(cdef, n);
        s1 = get_component_annotation(comp);
        s2 = string_append("{", s1);
        str = string_append(s2, "}"); then str;
    case (_,_,_) then "Error"; 
  end matchcontinue;
end get_nth_component_annotation;

protected function get_nth_component_modification
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  input Integer in_integer;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_componentref,in_program,in_integer)
    local
      Absyn.Path modelpath;
      Absyn.Class cdef;
      Absyn.Element comp;
      String str,str_1;
      Absyn.ComponentRef model_;
      Absyn.Program p;
      Integer n;
    case (model_,p,n)
      equation 
        modelpath = Absyn.cref_to_path(model_);
        cdef = get_pathed_class_in_program(modelpath, p);
        comp = get_nth_component_in_class(cdef, n);
        str = get_component_modification(comp);
        str_1 = Util.string_append_list({"{",str,"}"}); then str_1;
    case (_,_,_) then "Error"; 
  end matchcontinue;
end get_nth_component_modification;

protected function get_connection_count "* function: get_connection_count
 ** This function takes a `ComponentRef' and a `Program' and returns a string containing
 ** the number of connections in the model identified by the `ComponentRef'.
*"
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_componentref,in_program)
    local
      Absyn.Path modelpath;
      Absyn.Class cdef;
      Integer numconn;
      String res;
      Absyn.ComponentRef model_;
      Absyn.Program p;
    case (model_,p)
      equation 
        modelpath = Absyn.cref_to_path(model_);
        cdef = get_pathed_class_in_program(modelpath, p);
        numconn = count_connections(cdef);
        res = int_string(numconn); then res;
    case (_,_) then "Error"; 
  end matchcontinue;
end get_connection_count;

protected function get_nth_connection "* function: get_nth_connection
 ** This function takes a `ComponentRef' and a `Program' and an int and 
** returns a comma separated string for the nth connection, e.g. \"R1.n,C.p\".
*"
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  input Integer in_integer;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_componentref,in_program,in_integer)
    local
      Absyn.Path modelpath;
      Absyn.Class cdef;
      Absyn.Equation eq;
      String str;
      Absyn.ComponentRef model_;
      Absyn.Program p;
      Integer n;
    case (model_,p,n)
      equation 
        modelpath = Absyn.cref_to_path(model_);
        cdef = get_pathed_class_in_program(modelpath, p);
        Absyn.EQUATIONITEM(eq,_) = get_nth_connectionitem_in_class(cdef, n);
        str = get_connection_str(eq); then str;
    case (_,_,_) then "Error"; 
  end matchcontinue;
end get_nth_connection;

protected function add_connection
  input Absyn.ComponentRef in_componentref1;
  input Absyn.ComponentRef in_componentref2;
  input Absyn.ComponentRef in_componentref3;
  input Absyn_NamedArgList in_absyn_namedarglist4;
  input Absyn.Program in_program5;
  output String out_string;
  output Absyn.Program out_program;
protected 
  type Absyn_NamedArgList = list<Absyn.NamedArg>;
algorithm 
  (out_string,out_program):=
  matchcontinue (in_componentref1,in_componentref2,in_componentref3,in_absyn_namedarglist4,in_program5)
    local
      Absyn.Path modelpath,package_;
      Absyn.Class cdef,newcdef;
      Absyn.Program newp,p;
      Absyn.ComponentRef model_,c1,c2;
      Absyn.Within w;
      Absyn.Annotation annotation_;
      Absyn_NamedArgList nargs;
    case ((model_ = Absyn.CREF_IDENT(_,_)),c1,c2,{},(p = Absyn.PROGRAM(_,w)))
      equation 
        modelpath = Absyn.cref_to_path(model_);
        cdef = get_pathed_class_in_program(modelpath, p);
        newcdef = add_to_equation(cdef, Absyn.EQUATIONITEM(Absyn.EQ_CONNECT(c1,c2),NONE));
        newp = update_program(Absyn.PROGRAM({newcdef},w), p); then ("Ok",newp);
    case ((model_ = Absyn.CREF_QUAL(_,_,_)),c1,c2,{},(p = Absyn.PROGRAM(_,w)))
      equation 
        modelpath = Absyn.cref_to_path(model_);
        cdef = get_pathed_class_in_program(modelpath, p);
        package_ = Absyn.strip_last(modelpath);
        newcdef = add_to_equation(cdef, Absyn.EQUATIONITEM(Absyn.EQ_CONNECT(c1,c2),NONE));
        newp = update_program(Absyn.PROGRAM({newcdef},Absyn.WITHIN(package_)), p); then ("Ok",newp);
    case ((model_ = Absyn.CREF_IDENT(_,_)),c1,c2,nargs,(p = Absyn.PROGRAM(_,w)))
      equation 
        modelpath = Absyn.cref_to_path(model_);
        cdef = get_pathed_class_in_program(modelpath, p);
        annotation_ = annotation_list_to_absyn(nargs);
        newcdef = add_to_equation(cdef, 
          Absyn.EQUATIONITEM(Absyn.EQ_CONNECT(c1,c2),
          SOME(Absyn.COMMENT(SOME(annotation_),NONE))));
        newp = update_program(Absyn.PROGRAM({newcdef},w), p); then ("Ok",newp);
    case ((model_ = Absyn.CREF_QUAL(_,_,_)),c1,c2,nargs,(p = Absyn.PROGRAM(_,w)))
      equation 
        modelpath = Absyn.cref_to_path(model_);
        cdef = get_pathed_class_in_program(modelpath, p);
        package_ = Absyn.strip_last(modelpath);
        annotation_ = annotation_list_to_absyn(nargs);
        newcdef = add_to_equation(cdef, 
          Absyn.EQUATIONITEM(Absyn.EQ_CONNECT(c1,c2),
          SOME(Absyn.COMMENT(SOME(annotation_),NONE))));
        newp = update_program(Absyn.PROGRAM({newcdef},Absyn.WITHIN(package_)), p); then ("Ok",newp);
  end matchcontinue;
end add_connection;

protected function delete_connection
  input Absyn.ComponentRef in_componentref1;
  input Absyn.ComponentRef in_componentref2;
  input Absyn.ComponentRef in_componentref3;
  input Absyn.Program in_program4;
  output String out_string;
  output Absyn.Program out_program;
algorithm 
  (out_string,out_program):=
  matchcontinue (in_componentref1,in_componentref2,in_componentref3,in_program4)
    local
      Absyn.Path modelpath,modelwithin;
      Absyn.Class cdef,newcdef;
      Absyn.Program newp,p;
      Absyn.ComponentRef model_,c1,c2;
      Absyn.Within w;
    case (model_,c1,c2,(p = Absyn.PROGRAM(_,w)))
      equation 
        modelpath = Absyn.cref_to_path(model_);
        modelwithin = Absyn.strip_last(modelpath);
        cdef = get_pathed_class_in_program(modelpath, p);
        newcdef = delete_equation_in_class(cdef, c1, c2);
        newp = update_program(Absyn.PROGRAM({newcdef},Absyn.WITHIN(modelwithin)), p); then ("Ok",newp);
    case (model_,c1,c2,(p = Absyn.PROGRAM(_,w)))
      equation 
        modelpath = Absyn.cref_to_path(model_);
        cdef = get_pathed_class_in_program(modelpath, p);
        newcdef = delete_equation_in_class(cdef, c1, c2);
        newp = update_program(Absyn.PROGRAM({newcdef},Absyn.TOP()), p); then ("Ok",newp);
    case (model_,c1,c2,(p = Absyn.PROGRAM(_,w))) then ("Error",p); 
  end matchcontinue;
end delete_connection;

protected function delete_equation_in_class
  input Absyn.Class in_class1;
  input Absyn.ComponentRef in_componentref2;
  input Absyn.ComponentRef in_componentref3;
  output Absyn.Class out_class;
algorithm 
  out_class:=
  matchcontinue (in_class1,in_componentref2,in_componentref3)
    local
      type Absyn_EquationItemList = list<Absyn.EquationItem>;
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      type StringOption = Option<String>;
      Absyn_EquationItemList eqlst,eqlst_1;
      Absyn_ClassPartList parts2,parts;
      String i,file;
      Boolean p,f,e;
      Absyn.Restriction r;
      StringOption cmt;
      Absyn.ComponentRef c1,c2;
    case (Absyn.CLASS(i,p,f,e,r,Absyn.PARTS(parts,cmt),file),c1,c2)
      equation 
        eqlst = get_equation_list(parts);
        eqlst_1 = delete_equation_in_eqlist(eqlst, c1, c2);
        parts2 = replace_equation_list(parts, eqlst_1); then Absyn.CLASS(i,p,f,e,r,Absyn.PARTS(parts2,cmt),file);
  end matchcontinue;
end delete_equation_in_class;

protected function delete_equation_in_eqlist
  input Absyn_EquationItemList in_absyn_equationitemlist1;
  input Absyn.ComponentRef in_componentref2;
  input Absyn.ComponentRef in_componentref3;
  output Absyn_EquationItemList out_absyn_equationitemlist;
protected 
  type Absyn_EquationItemList = list<Absyn.EquationItem>;
algorithm 
  out_absyn_equationitemlist:=
  matchcontinue (in_absyn_equationitemlist1,in_componentref2,in_componentref3)
    local
      Absyn.Path p1,p2,pn1,pn2;
      String s1,s2,sn1,sn2;
      Absyn_EquationItemList res,xs;
      Absyn.ComponentRef cn1,cn2,c1,c2;
      Absyn.EquationItem x;
    case ({},_,_) then {}; 
    case (Absyn.EQUATIONITEM(Absyn.EQ_CONNECT(cn1,cn2),_) :: xs,c1,c2)
      equation 
        p1 = Absyn.cref_to_path(c1);
        s1 = Absyn.path_string(p1);
        p2 = Absyn.cref_to_path(c2);
        s2 = Absyn.path_string(p2);
        pn1 = Absyn.cref_to_path(cn1);
        sn1 = Absyn.path_string(pn1);
        pn2 = Absyn.cref_to_path(cn2);
        sn2 = Absyn.path_string(pn2);
        equality(s1 = sn1);
        equality(s2 = sn2);
        res = delete_equation_in_eqlist(xs, c1, c2); then res;
    case (x :: xs,c1,c2)
      equation 
        res = delete_equation_in_eqlist(xs, c1, c2); then x :: res;
  end matchcontinue;
end delete_equation_in_eqlist;

protected function get_nth_connection_annotation "* function: get_nth_connection_annotation
 ** This function takes a `ComponentRef' and a `Program' and an int and 
** returns a comma separated string  of values for the annotation of the the nth connection.
*"
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  input Integer in_integer;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_componentref,in_program,in_integer)
    local
      Absyn.Path modelpath;
      Absyn.Class cdef;
      Absyn.EquationItem citem;
      String s1,s2,str;
      Absyn.ComponentRef model_;
      Absyn.Program p;
      Integer n;
    case (model_,p,n)
      equation 
        modelpath = Absyn.cref_to_path(model_);
        cdef = get_pathed_class_in_program(modelpath, p);
        citem = get_nth_connectionitem_in_class(cdef, n);
        s1 = get_connection_annotation_str(citem);
        s2 = string_append("{", s1);
        str = string_append(s2, "}"); then str;
    case (_,_,_) then "{}"; 
  end matchcontinue;
end get_nth_connection_annotation;

protected function get_connector_count "* function: get_connector_count
 ** This function takes a ComponentRef and a Program and returns the number
 ** of connector components in the class given by the classname in the 
 ** ComponentRef. A partial instantiation of the inheritance structure is performed in order 
 ** to find all connectors of the class
 *"
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_componentref,in_program)
    local
      Absyn.Path modelpath;
      Absyn.Class cdef;
      Integer count;
      String countstr;
      Absyn.ComponentRef model_;
      Absyn.Program p;
    case (model_,p) " Env.Env,  * A complete elaboration is far too expensive. Instead we only look through the components of
	 ** the class for types declared using the \"connector\" restricted class keyword. We also look in
	 ** base classes  (recursively)  
	 *"
      equation 
        modelpath = Absyn.cref_to_path(model_);
        cdef = get_pathed_class_in_program(modelpath, p);
        count = count_public_connectors(modelpath, p, cdef);
        countstr = int_string(count); then countstr;
    case (_,_) then "Error"; 
  end matchcontinue;
end get_connector_count;

protected function get_nth_connector "* function: get_nth_connector
 ** This function takes a ComponentRef and a Program and an int and returns a string with the name of the nth
 ** connector component in the class given by ComponentRef in the Program.
 *"
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  input Integer in_integer;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_componentref,in_program,in_integer)
    local
      Absyn.Path modelpath,tp;
      Absyn.Class cdef;
      String str,tpstr,s1,resstr;
      Absyn.ComponentRef model_;
      Absyn.Program p;
      Integer n;
    case (model_,p,n)
      equation 
        modelpath = Absyn.cref_to_path(model_);
        cdef = get_pathed_class_in_program(modelpath, p);
        (str,tp) = get_nth_public_connector_str(modelpath, cdef, p, n);
        tpstr = Absyn.path_string(tp);
        s1 = string_append(str, ",");
        resstr = string_append(s1, tpstr); then resstr;
    case (_,_,_) then "Error"; 
  end matchcontinue;
end get_nth_connector;

protected function get_nth_connector_icon_annotation "* function: get_nth_connector
 ** This function takes a ComponentRef and a Program and an int and returns a string with the name of the nth
 ** connectors icon annotation in the class given by ComponentRef in the Program.
 *"
  input Absyn.ComponentRef in_componentref;
  input Absyn.Program in_program;
  input Integer in_integer;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_componentref,in_program,in_integer)
    local
      Absyn.Path modelpath,tp;
      Absyn.Class cdef;
      String resstr;
      Absyn.ComponentRef model_;
      Absyn.Program p;
      Integer n;
    case (model_,p,n)
      equation 
        modelpath = Absyn.cref_to_path(model_);
        cdef = get_pathed_class_in_program(modelpath, p);
        (resstr,tp) = get_nth_public_connector_str(modelpath, cdef, p, n);
        resstr = get_icon_annotation(tp, p); then resstr;
    case (_,_,_) then "Error"; 
  end matchcontinue;
end get_nth_connector_icon_annotation;

protected function get_diagram_annotation "* function: get_diagram_annotation
 ** This function takes a Path and a Program and returns a comma separated 
   string of values for the diagram annotation for the class named by the first argument.
*"
  input Absyn.Path in_path;
  input Absyn.Program in_program;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_path,in_program)
    local
      Absyn.Class cdef;
      String str;
      Absyn.Path modelpath;
      Absyn.Program p;
    case (modelpath,p)
      equation 
        cdef = get_pathed_class_in_program(modelpath, p);
        str = get_diagram_annotation_in_class(cdef); then str;
    case (_,_) then "get_diagram_annotation failed!"; 
  end matchcontinue;
end get_diagram_annotation;

protected function get_icon_annotation "* function: get_icon_annotation
 ** This function takes a Path and a Program and returns a comma separated
 ** string of values for the icon annotation for the class named by the first argument. 	
 "
  input Absyn.Path in_path;
  input Absyn.Program in_program;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_path,in_program)
    local
      Absyn.Class cdef;
      String str;
      Absyn.Path modelpath;
      Absyn.Program p;
    case (modelpath,p)
      equation 
        cdef = get_pathed_class_in_program(modelpath, p);
        str = get_icon_annotation_in_class(cdef); then str;
    case (_,_) then ""; 
  end matchcontinue;
end get_icon_annotation;

protected function get_packages_in_path "* function: get_packages_in_path
 ** This function takes a Path and a Program and returns a list of the names of the packages found in the Path.
*"
  input Absyn.Path in_path;
  input Absyn.Program in_program;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_path,in_program)
    local
      Absyn.Class cdef;
      String str,s1,res;
      Absyn.Path modelpath;
      Absyn.Program p;
    case (modelpath,p)
      equation 
        cdef = get_pathed_class_in_program(modelpath, p);
        str = get_packages_in_class(modelpath, p, cdef);
        s1 = string_append("{", str);
        res = string_append(s1, "}"); then res;
    case (_,_) then "Error"; 
  end matchcontinue;
end get_packages_in_path;

protected function get_top_packages "* function: get_top_packages
 ** This function takes a Path and a Program and returns a list of the names of the packages found in the Path.
 *"
  input Absyn.Program in_program;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_program)
    local
      type StringList = list<String>;
      StringList strlist;
      String str,s1,res;
      Absyn.Program p;
    case (p)
      equation 
        strlist = get_top_packages_in_program(p);
        str = Util.string_delimit_list(strlist, ",");
        s1 = string_append("{", str);
        res = string_append(s1, "}"); then res;
    case (_) then "Error"; 
  end matchcontinue;
end get_top_packages;

protected function get_top_packages_in_program
  input Absyn.Program in_program;
  output StringList out_stringlist;
protected 
  type StringList = list<String>;
algorithm 
  out_stringlist:=
  matchcontinue (in_program)
    local
      type Absyn_ClassList = list<Absyn.Class>;
      StringList res;
      String id;
      Absyn_ClassList rest;
      Absyn.Within w;
    case Absyn.PROGRAM({},_) then {}; 
    case (Absyn.PROGRAM(Absyn.CLASS(id,_,_,_,Absyn.R_PACKAGE(),_,_) :: rest,w))
      equation 
        res = get_top_packages_in_program(Absyn.PROGRAM(rest,w)); then id :: res;
    case (Absyn.PROGRAM(_ :: rest,w))
      equation 
        res = get_top_packages_in_program(Absyn.PROGRAM(rest,w)); then res;
  end matchcontinue;
end get_top_packages_in_program;

protected function get_packages_in_class "* function: get_packages_in_class
 ** This function takes a `Class' definition and a Path identifying th elcass. 
 ** It returns a string containing comma separated package names found 
 ** in the class definition.
 *"
  input Absyn.Path in_path;
  input Absyn.Program in_program;
  input Absyn.Class in_class;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_path,in_program,in_class)
    local
      type StringList = list<String>;
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      type StringOption = Option<String>;
      StringList strlist;
      String res;
      Absyn_ClassPartList parts;
      StringOption cmt;
      Absyn.Class cdef;
      Absyn.Path newpath,inmodel,path;
      Absyn.Program p;
    case (_,_,Absyn.CLASS(_,_,_,_,_,Absyn.PARTS(parts,cmt),_))
      equation 
        strlist = get_packages_in_parts(parts);
        res = Util.string_delimit_list(strlist, ","); then res;
    case (inmodel,p,Absyn.CLASS(_,_,_,_,_,Absyn.DERIVED(path,_,_,_,_),_))
      equation 
        (cdef,newpath) = lookup_classdef(path, inmodel, p);
        res = get_packages_in_class(newpath, p, cdef); then res;
  end matchcontinue;
end get_packages_in_class;

protected function get_packages_in_parts
  input Absyn_ClassPartList in_absyn_classpartlist;
  output StringList out_stringlist;
protected 
  type Absyn_ClassPartList = list<Absyn.ClassPart>;
  type StringList = list<String>;
algorithm 
  out_stringlist:=
  matchcontinue (in_absyn_classpartlist)
    local
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      StringList l1,l2,res;
      Absyn_ElementItemList elts;
      Absyn_ClassPartList rest;
    case {} then {}; 
    case (Absyn.PUBLIC(elts) :: rest)
      equation 
        l1 = get_packages_in_elts(elts);
        l2 = get_packages_in_parts(rest);
        res = list_append(l1, l2); then res;
    case (_ :: rest)
      equation 
        res = get_packages_in_parts(rest); then res;
  end matchcontinue;
end get_packages_in_parts;

protected function get_packages_in_elts
  input Absyn_ElementItemList in_absyn_elementitemlist;
  output StringList out_stringlist;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
  type StringList = list<String>;
algorithm 
  out_stringlist:=
  matchcontinue (in_absyn_elementitemlist)
    local
      StringList res;
      String id;
      Absyn_ElementItemList rest;
    case {} then {}; 
    case (Absyn.ELEMENTITEM(Absyn.ELEMENT(_,_,_,_,Absyn.CLASSDEF(_,Absyn.CLASS(id,_,_,_,Absyn.R_PACKAGE(),_,_)),_,_,NONE)) :: rest)
      equation 
        res = get_packages_in_elts(rest); then id :: res;
    case (_ :: rest)
      equation 
        res = get_packages_in_elts(rest); then res;
  end matchcontinue;
end get_packages_in_elts;

protected function get_classnames_in_path
  input Absyn.Path in_path;
  input Absyn.Program in_program;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_path,in_program)
    local
      Absyn.Class cdef;
      String str,s1,res;
      Absyn.Path modelpath;
      Absyn.Program p;
    case (modelpath,p)
      equation 
        cdef = get_pathed_class_in_program(modelpath, p);
        str = get_classnames_in_class(modelpath, p, cdef);
        s1 = string_append("{", str);
        res = string_append(s1, "}"); then res;
    case (_,_) then "Error"; 
  end matchcontinue;
end get_classnames_in_path;

public function get_top_classnames "* function: get_top_classnames
 ** This function takes a Path and a Program and returns a list of the names of the packages found in the Path.
 *"
  input Absyn.Program in_program;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_program)
    local
      type StringList = list<String>;
      StringList strlist;
      String str,s1,res;
      Absyn.Program p;
    case (p)
      equation 
        strlist = get_top_classnames_in_program(p);
        str = Util.string_delimit_list(strlist, ",") "	Print.print_buf \"program =\" &
	Dump.dump p & Print.print_buf \"\n\" &";
        s1 = string_append("{", str);
        res = string_append(s1, "}"); then res;
    case (_) then "Error"; 
  end matchcontinue;
end get_top_classnames;

protected function get_top_classnames_in_program
  input Absyn.Program in_program;
  output StringList out_stringlist;
protected 
  type StringList = list<String>;
algorithm 
  out_stringlist:=
  matchcontinue (in_program)
    local
      type Absyn_ClassList = list<Absyn.Class>;
      StringList res;
      String id;
      Absyn_ClassList rest;
      Absyn.Within w;
    case Absyn.PROGRAM({},_) then {}; 
    case (Absyn.PROGRAM(Absyn.CLASS(id,_,_,_,_,_,_) :: rest,w))
      equation 
        res = get_top_classnames_in_program(Absyn.PROGRAM(rest,w)); then id :: res;
    case (Absyn.PROGRAM(_ :: rest,w))
      equation 
        res = get_top_classnames_in_program(Absyn.PROGRAM(rest,w)); then res;
  end matchcontinue;
end get_top_classnames_in_program;

protected function get_classnames_in_class "* function: get_classnames_in_class
 ** This function takes a `Class' definition and a Path identifying th elcass. 
 ** It returns a string containing comma separated package names found 
 ** in the class definition.
 *"
  input Absyn.Path in_path;
  input Absyn.Program in_program;
  input Absyn.Class in_class;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_path,in_program,in_class)
    local
      type StringList = list<String>;
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      StringList strlist;
      String res;
      Absyn_ClassPartList parts;
      Absyn.Class cdef;
      Absyn.Path newpath,inmodel,path;
      Absyn.Program p;
    case (_,_,Absyn.CLASS(_,_,_,_,_,Absyn.PARTS(parts,_),_))
      equation 
        strlist = get_classnames_in_parts(parts);
        res = Util.string_delimit_list(strlist, ","); then res;
    case (inmodel,p,Absyn.CLASS(_,_,_,_,_,Absyn.DERIVED(path,_,_,_,_),_))
      equation 
        (cdef,newpath) = lookup_classdef(path, inmodel, p);
        res = get_classnames_in_class(newpath, p, cdef); then res;
  end matchcontinue;
end get_classnames_in_class;

protected function get_classnames_in_parts
  input Absyn_ClassPartList in_absyn_classpartlist;
  output StringList out_stringlist;
protected 
  type Absyn_ClassPartList = list<Absyn.ClassPart>;
  type StringList = list<String>;
algorithm 
  out_stringlist:=
  matchcontinue (in_absyn_classpartlist)
    local
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      StringList l1,l2,res;
      Absyn_ElementItemList elts;
      Absyn_ClassPartList rest;
    case {} then {}; 
    case (Absyn.PUBLIC(elts) :: rest)
      equation 
        l1 = get_classnames_in_elts(elts);
        l2 = get_classnames_in_parts(rest);
        res = list_append(l1, l2); then res;
    case (_ :: rest)
      equation 
        res = get_classnames_in_parts(rest); then res;
  end matchcontinue;
end get_classnames_in_parts;

public function get_classnames_in_elts
  input Absyn_ElementItemList in_absyn_elementitemlist;
  output StringList out_stringlist;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
  type StringList = list<String>;
algorithm 
  out_stringlist:=
  matchcontinue (in_absyn_elementitemlist)
    local
      StringList res;
      String id;
      Absyn_ElementItemList rest;
    case {} then {}; 
    case (Absyn.ELEMENTITEM(Absyn.ELEMENT(_,_,_,_,Absyn.CLASSDEF(_,Absyn.CLASS(id,_,_,_,_,_,_)),_,_,NONE)) :: rest)
      equation 
        res = get_classnames_in_elts(rest); then id :: res;
    case (_ :: rest)
      equation 
        res = get_classnames_in_elts(rest); then res;
  end matchcontinue;
end get_classnames_in_elts;

protected function get_base_classes
  input Absyn.Class in_class;
  input Env.Env in_env;
  output Absyn_ComponentRefList out_absyn_componentreflist;
protected 
  type Absyn_ComponentRefList = list<Absyn.ComponentRef>;
algorithm 
  out_absyn_componentreflist:=
  matchcontinue (in_class,in_env)
    local
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      type Env_FrameList = list<Env.Frame>;
      Absyn_ComponentRefList res;
      Absyn_ClassPartList parts;
      Env_FrameList env;
    case (Absyn.CLASS(_,_,_,_,_,Absyn.PARTS(parts,_),_),env)
      equation 
        res = get_base_classes_from_parts(parts, env); then res;
    case (_,_) then {}; 
  end matchcontinue;
end get_base_classes;

protected function get_base_classes_from_parts
  input Absyn_ClassPartList in_absyn_classpartlist;
  input Env.Env in_env;
  output Absyn_ComponentRefList out_absyn_componentreflist;
protected 
  type Absyn_ClassPartList = list<Absyn.ClassPart>;
  type Absyn_ComponentRefList = list<Absyn.ComponentRef>;
algorithm 
  out_absyn_componentreflist:=
  matchcontinue (in_absyn_classpartlist,in_env)
    local
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      type Env_FrameList = list<Env.Frame>;
      Absyn_ComponentRefList c1,c2,res;
      Absyn_ElementItemList elts;
      Absyn_ClassPartList rest;
      Env_FrameList env;
    case (Absyn.PUBLIC(elts) :: rest,env)
      equation 
        c1 = get_base_classes_from_elts(elts, env);
        c2 = get_base_classes_from_parts(rest, env);
        res = list_append(c1, c2); then res;
    case (_ :: rest,env)
      equation 
        res = get_base_classes_from_parts(rest, env); then res;
    case ({},env) then {}; 
  end matchcontinue;
end get_base_classes_from_parts;

protected function get_base_classes_from_elts
  input Absyn_ElementItemList in_absyn_elementitemlist;
  input Env.Env in_env;
  output Absyn_ComponentRefList out_absyn_componentreflist;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
  type Absyn_ComponentRefList = list<Absyn.ComponentRef>;
algorithm 
  out_absyn_componentreflist:=
  matchcontinue (in_absyn_elementitemlist,in_env)
    local
      type Env_FrameList = list<Env.Frame>;
      Env_FrameList env,env_1;
      Absyn_ComponentRefList cl;
      SCode.Class c;
      Absyn.Path envpath,p_1,path;
      String tpname;
      Absyn.ComponentRef cref;
      Absyn_ElementItemList rest;
    case ({},env) then {}; 
    case (Absyn.ELEMENTITEM(Absyn.ELEMENT(_,_,_,_,Absyn.EXTENDS(path,_),_,_,_)) :: rest,env)
      equation 
        cl = get_base_classes_from_elts(rest, env) " Inherited class is defined inside package ";
        (c,env_1) = Lookup.lookup_class(env, path, true);
        SOME(envpath) = Env.get_env_path(env_1);
        tpname = Absyn.path_last_ident(path);
        p_1 = Absyn.join_paths(envpath, Absyn.IDENT(tpname));
        cref = Absyn.path_to_cref(p_1); then cref :: cl;
    case (Absyn.ELEMENTITEM(Absyn.ELEMENT(_,_,_,_,Absyn.EXTENDS(path,_),_,_,_)) :: rest,env)
      equation 
        cl = get_base_classes_from_elts(rest, env) " Inherited class defined on top level scope";
        (c,env_1) = Lookup.lookup_class(env, path, true);
        NONE = Env.get_env_path(env_1);
        cref = Absyn.path_to_cref(path); then cref :: cl;
    case (_ :: rest,env)
      equation 
        cl = get_base_classes_from_elts(rest, env); then cl;
  end matchcontinue;
end get_base_classes_from_elts;

protected function count_base_classes
  input Absyn.Class in_class;
  output Integer out_integer;
algorithm 
  out_integer:=
  matchcontinue (in_class)
    local
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      Integer res;
      Absyn_ClassPartList parts;
    case (Absyn.CLASS(_,_,_,_,_,Absyn.PARTS(parts,_),_))
      equation 
        res = count_base_classes_from_parts(parts); then res;
    case (_) then 0; 
  end matchcontinue;
end count_base_classes;

protected function count_base_classes_from_parts
  input Absyn_ClassPartList in_absyn_classpartlist;
  output Integer out_integer;
protected 
  type Absyn_ClassPartList = list<Absyn.ClassPart>;
algorithm 
  out_integer:=
  matchcontinue (in_absyn_classpartlist)
    local
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      Integer c1,c2,res;
      Absyn_ElementItemList elts;
      Absyn_ClassPartList rest;
    case (Absyn.PUBLIC(elts) :: rest)
      equation 
        c1 = count_base_classes_from_elts(elts);
        c2 = count_base_classes_from_parts(rest); then c1 + c2;
    case (_ :: rest)
      equation 
        res = count_base_classes_from_parts(rest); then res;
    case ({}) then 0; 
  end matchcontinue;
end count_base_classes_from_parts;

protected function count_base_classes_from_elts
  input Absyn_ElementItemList in_absyn_elementitemlist;
  output Integer out_integer;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
algorithm 
  out_integer:=
  matchcontinue (in_absyn_elementitemlist)
    local
      Integer cl;
      Absyn.Path path;
      Absyn_ElementItemList rest;
    case ({}) then 0; 
    case (Absyn.ELEMENTITEM(Absyn.ELEMENT(_,_,_,_,Absyn.EXTENDS(path,_),_,_,_)) :: rest)
      equation 
        cl = count_base_classes_from_elts(rest) " Inherited class "; then cl + 1;
    case (_ :: rest)
      equation 
        cl = count_base_classes_from_elts(rest); then cl;
  end matchcontinue;
end count_base_classes_from_elts;

protected function get_icon_annotation_from_baseclasses
  input Absyn_ComponentRefList in_absyn_componentreflist;
  input Absyn.Program in_program;
  output StringList out_stringlist;
protected 
  type Absyn_ComponentRefList = list<Absyn.ComponentRef>;
  type StringList = list<String>;
algorithm 
  out_stringlist:=
  matchcontinue (in_absyn_componentreflist,in_program)
    local
      Absyn.Program p;
      Absyn.Path path;
      String s1;
      StringList s2;
      Absyn.ComponentRef cr;
      Absyn_ComponentRefList rest;
    case ({},p) then {}; 
    case (cr :: rest,p)
      equation 
        path = Absyn.cref_to_path(cr);
        s1 = get_icon_annotation(path, p);
        s2 = get_icon_annotation_from_baseclasses(rest, p); then s1 :: s2;
  end matchcontinue;
end get_icon_annotation_from_baseclasses;

protected function get_icon_annotation_in_class
  input Absyn.Class in_class;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_class)
    local
      type Absyn_ElementArgList = list<Absyn.ElementArg>;
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      Absyn_ElementArgList annlst;
      String s1,s2,str;
      Absyn_ClassPartList parts;
    case (Absyn.CLASS(_,_,_,_,_,Absyn.PARTS(parts,_),_))
      equation 
        annlst = get_icon_annotation_from_parts(parts);
        s1 = get_icon_annotation_str(annlst);
        s2 = string_append("{", s1);
        str = string_append(s2, "}"); then str;
  end matchcontinue;
end get_icon_annotation_in_class;

protected function get_icon_annotation_from_parts
  input Absyn_ClassPartList in_absyn_classpartlist;
  output Absyn_ElementArgList out_absyn_elementarglist;
protected 
  type Absyn_ClassPartList = list<Absyn.ClassPart>;
  type Absyn_ElementArgList = list<Absyn.ElementArg>;
algorithm 
  out_absyn_elementarglist:=
  matchcontinue (in_absyn_classpartlist)
    local
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      type Absyn_EquationItemList = list<Absyn.EquationItem>;
      type Absyn_AlgorithmItemList = list<Absyn.AlgorithmItem>;
      Absyn_ElementArgList res;
      Absyn_ElementItemList elts;
      Absyn_ClassPartList rest;
      Absyn_EquationItemList eqns;
      Absyn_AlgorithmItemList algs;
    case {} then {}; 
    case (Absyn.PUBLIC(elts) :: rest)
      equation 
        res = get_icon_annotation_from_elts(elts); then res;
    case (Absyn.PROTECTED(elts) :: rest)
      equation 
        res = get_icon_annotation_from_elts(elts); then res;
    case (Absyn.EQUATIONS(eqns) :: rest)
      equation 
        res = get_icon_annotation_from_eqns(eqns); then res;
    case (Absyn.INITIALEQUATIONS(eqns) :: rest)
      equation 
        res = get_icon_annotation_from_eqns(eqns); then res;
    case (Absyn.ALGORITHMS(algs) :: rest)
      equation 
        res = get_icon_annotation_from_algs(algs); then res;
    case (Absyn.INITIALALGORITHMS(algs) :: rest)
      equation 
        res = get_icon_annotation_from_algs(algs); then res;
    case (_ :: rest)
      equation 
        res = get_icon_annotation_from_parts(rest); then res;
  end matchcontinue;
end get_icon_annotation_from_parts;

protected function get_icon_annotation_from_elts
  input Absyn_ElementItemList in_absyn_elementitemlist;
  output Absyn_ElementArgList out_absyn_elementarglist;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
  type Absyn_ElementArgList = list<Absyn.ElementArg>;
algorithm 
  out_absyn_elementarglist:=
  matchcontinue (in_absyn_elementitemlist)
    local
      Absyn_ElementArgList lst,res;
      Absyn_ElementItemList rest;
    case (Absyn.ANNOTATIONITEM(Absyn.ANNOTATION(lst)) :: rest)
      equation 
        contain_icon_annotation(lst); then lst;
    case (_ :: rest)
      equation 
        res = get_icon_annotation_from_elts(rest); then res;
  end matchcontinue;
end get_icon_annotation_from_elts;

protected function contain_icon_annotation
  input Absyn_ElementArgList in_absyn_elementarglist;
protected 
  type Absyn_ElementArgList = list<Absyn.ElementArg>;
algorithm 
  _:=
  matchcontinue (in_absyn_elementarglist)
    local Absyn_ElementArgList lst;
    case (Absyn.MODIFICATION(_,_,Absyn.CREF_IDENT("Icon",_),_,_) :: _) then (); 
    case (_ :: lst)
      equation 
        contain_icon_annotation(lst); then ();
  end matchcontinue;
end contain_icon_annotation;

protected function get_icon_annotation_from_eqns
  input Absyn_EquationItemList in_absyn_equationitemlist;
  output Absyn_ElementArgList out_absyn_elementarglist;
protected 
  type Absyn_EquationItemList = list<Absyn.EquationItem>;
  type Absyn_ElementArgList = list<Absyn.ElementArg>;
algorithm 
  out_absyn_elementarglist:=
  matchcontinue (in_absyn_equationitemlist)
    local
      Absyn_ElementArgList lst,res;
      Absyn_EquationItemList rest;
    case {} then {}; 
    case (Absyn.EQUATIONITEMANN(Absyn.ANNOTATION(lst)) :: rest)
      equation 
        contain_icon_annotation(lst); then lst;
    case (_ :: rest)
      equation 
        res = get_icon_annotation_from_eqns(rest); then res;
  end matchcontinue;
end get_icon_annotation_from_eqns;

protected function get_icon_annotation_from_algs
  input Absyn_AlgorithmItemList in_absyn_algorithmitemlist;
  output Absyn_ElementArgList out_absyn_elementarglist;
protected 
  type Absyn_AlgorithmItemList = list<Absyn.AlgorithmItem>;
  type Absyn_ElementArgList = list<Absyn.ElementArg>;
algorithm 
  out_absyn_elementarglist:=
  matchcontinue (in_absyn_algorithmitemlist)
    local
      Absyn_ElementArgList lst,res;
      Absyn_AlgorithmItemList rest;
    case {} then {}; 
    case (Absyn.ALGORITHMITEMANN(Absyn.ANNOTATION(lst)) :: rest)
      equation 
        contain_icon_annotation(lst); then lst;
    case (_ :: rest)
      equation 
        res = get_icon_annotation_from_algs(rest); then res;
  end matchcontinue;
end get_icon_annotation_from_algs;

protected function get_icon_annotation_in_elementitemlist
  input Absyn_ElementItemList in_absyn_elementitemlist;
  output String out_string;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
algorithm 
  out_string:=
  matchcontinue (in_absyn_elementitemlist)
    local
      type Absyn_ElementArgList = list<Absyn.ElementArg>;
      String s1,s2,str;
      Absyn_ElementArgList annlst;
      Absyn_ElementItemList xs;
    case {} then ""; 
    case (Absyn.ANNOTATIONITEM(Absyn.ANNOTATION(annlst)) :: _)
      equation 
        s1 = get_icon_annotation_str(annlst);
        s2 = string_append("{", s1);
        str = string_append(s2, "}"); then str;
    case (_ :: xs)
      equation 
        str = get_icon_annotation_in_elementitemlist(xs); then str;
  end matchcontinue;
end get_icon_annotation_in_elementitemlist;

protected function get_icon_annotation_str
  input Absyn_ElementArgList in_absyn_elementarglist;
  output String out_string;
protected 
  type Absyn_ElementArgList = list<Absyn.ElementArg>;
algorithm 
  out_string:=
  matchcontinue (in_absyn_elementarglist)
    local
      type Absyn_ModificationOption = Option<Absyn.Modification>;
      String records,str;
      Absyn.ElementArg ann;
      Absyn_ModificationOption mod;
      Absyn_ElementArgList xs;
    case ((ann = Absyn.MODIFICATION(_,_,Absyn.CREF_IDENT("Icon",_),mod,_)) :: _)
      equation 
        records = string_append(Icon_records, Graphics_records);
        str = get_annotation_string(records, Absyn.ANNOTATION({ann})); then str;
    case (_ :: xs)
      equation 
        str = get_icon_annotation_str(xs); then str;
  end matchcontinue;
end get_icon_annotation_str;

protected function get_diagram_annotation_from_baseclasses
  input Absyn_ComponentRefList in_absyn_componentreflist;
  input Absyn.Program in_program;
  output StringList out_stringlist;
protected 
  type Absyn_ComponentRefList = list<Absyn.ComponentRef>;
  type StringList = list<String>;
algorithm 
  out_stringlist:=
  matchcontinue (in_absyn_componentreflist,in_program)
    local
      Absyn.Program p;
      Absyn.Path path;
      String s1;
      StringList s2;
      Absyn.ComponentRef cr;
      Absyn_ComponentRefList rest;
    case ({},p) then {}; 
    case (cr :: rest,p)
      equation 
        path = Absyn.cref_to_path(cr);
        s1 = get_diagram_annotation(path, p);
        s2 = get_diagram_annotation_from_baseclasses(rest, p); then s1 :: s2;
  end matchcontinue;
end get_diagram_annotation_from_baseclasses;

protected function get_diagram_annotation_in_class
  input Absyn.Class in_class;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_class)
    local
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      Absyn_ElementItemList publst,protlst,lst;
      String str;
      Absyn_ClassPartList parts;
    case (Absyn.CLASS(_,_,_,_,_,Absyn.PARTS(parts,_),_))
      equation 
        publst = get_public_list(parts);
        protlst = get_protected_list(parts);
        lst = list_append(publst, protlst);
        str = get_diagram_annotation_in_elementitemlist(lst); then str;
  end matchcontinue;
end get_diagram_annotation_in_class;

protected function get_diagram_annotation_in_elementitemlist
  input Absyn_ElementItemList in_absyn_elementitemlist;
  output String out_string;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
algorithm 
  out_string:=
  matchcontinue (in_absyn_elementitemlist)
    local
      type Absyn_ElementArgList = list<Absyn.ElementArg>;
      String s1,s2,str;
      Absyn_ElementArgList annlst;
      Absyn_ElementItemList xs;
    case {} then ""; 
    case (Absyn.ANNOTATIONITEM(Absyn.ANNOTATION(annlst)) :: _)
      equation 
        s1 = get_diagram_annotation_str(annlst);
        s2 = string_append("{", s1);
        str = string_append(s2, "}"); then str;
    case (_ :: xs)
      equation 
        str = get_diagram_annotation_in_elementitemlist(xs); then str;
  end matchcontinue;
end get_diagram_annotation_in_elementitemlist;

protected function get_diagram_annotation_str
  input Absyn_ElementArgList in_absyn_elementarglist;
  output String out_string;
protected 
  type Absyn_ElementArgList = list<Absyn.ElementArg>;
algorithm 
  out_string:=
  matchcontinue (in_absyn_elementarglist)
    local
      type Absyn_ModificationOption = Option<Absyn.Modification>;
      String records,str;
      Absyn.ElementArg ann;
      Absyn_ModificationOption mod;
      Absyn_ElementArgList xs;
    case ((ann = Absyn.MODIFICATION(_,_,Absyn.CREF_IDENT("Diagram",_),mod,_)) :: _)
      equation 
        records = string_append(Diagram_records, Graphics_records);
        str = get_annotation_string(records, Absyn.ANNOTATION({ann})); then str;
    case (_ :: xs)
      equation 
        str = get_diagram_annotation_str(xs); then str;
  end matchcontinue;
end get_diagram_annotation_str;

protected function get_nth_public_connector_str
  input Absyn.Path in_path;
  input Absyn.Class in_class;
  input Absyn.Program in_program;
  input Integer in_integer;
  output String out_string;
  output Absyn.Path out_path;
algorithm 
  (out_string,out_path):=
  matchcontinue (in_path,in_class,in_program,in_integer)
    local
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      type StringOption = Option<String>;
      String str,a,file;
      Absyn.Path tp,modelpath;
      Boolean b,c,d;
      Absyn.Restriction e;
      Absyn_ElementItemList elt;
      Absyn_ClassPartList lst;
      Absyn.Program p;
      Integer n,c1,c2;
      StringOption cmt;
    case (modelpath,Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(Absyn.PUBLIC(elt) :: lst,_),_),p,n)
      equation 
        (str,tp) = get_nth_connector_str(p, modelpath, elt, n); then (str,tp);
    case (modelpath,Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(Absyn.PUBLIC(elt) :: lst,cmt),file),p,n) "* The rule above failed, count the number of connectors in the first public list, subtract the number 
	 ** and try the rest of the list *"
      equation 
        c1 = count_public_connectors(modelpath, p, 
          Absyn.CLASS(a,b,c,d,e,Absyn.PARTS({Absyn.PUBLIC(elt)},cmt),file));
        c2 = n - c1;
        (str,tp) = get_nth_public_connector_str(modelpath, 
          Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(lst,cmt),file), p, c2); then (str,tp);
    case (modelpath,Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(_ :: lst,cmt),file),p,n)
      equation 
        (str,tp) = get_nth_public_connector_str(modelpath, 
          Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(lst,cmt),file), p, n); then (str,tp);
  end matchcontinue;
end get_nth_public_connector_str;

protected function get_nth_connector_str
  input Absyn.Program in_program;
  input Absyn.Path in_path;
  input Absyn_ElementItemList in_absyn_elementitemlist;
  input Integer in_integer;
  output String out_string;
  output Absyn.Path out_path;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
algorithm 
  (out_string,out_path):=
  matchcontinue (in_program,in_path,in_absyn_elementitemlist,in_integer)
    local
      type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
      Absyn.Class cdef;
      Absyn.Path newmodelpath,tp,modelpath;
      String str;
      Absyn.Program p;
      Absyn_ElementItemList lst;
      Integer n,c1,c2,newn;
      Absyn_ComponentItemList complst;
    case (p,modelpath,Absyn.ELEMENTITEM(Absyn.ELEMENT(_,_,_,_,Absyn.EXTENDS(tp,_),_,_,NONE)) :: lst,n)
      equation 
        (cdef,newmodelpath) = lookup_classdef(tp, modelpath, p);
        (str,tp) = get_nth_public_connector_str(newmodelpath, cdef, p, n); then (str,tp);
    case (p,modelpath,Absyn.ELEMENTITEM(Absyn.ELEMENT(_,_,_,_,Absyn.EXTENDS(tp,_),_,_,NONE)) :: lst,n)
      equation 
        (cdef,newmodelpath) = lookup_classdef(tp, modelpath, p);
        c1 = count_public_connectors(newmodelpath, p, cdef);
        c2 = n - c1;
        (str,tp) = get_nth_connector_str(p, modelpath, lst, c2); then (str,tp);
    case (p,modelpath,Absyn.ELEMENTITEM(Absyn.ELEMENT(_,_,_,_,Absyn.COMPONENTS(_,tp,complst),_,_,NONE)) :: lst,n)
      equation 
        (Absyn.CLASS(_,_,_,_,Absyn.R_CONNECTOR(),_,_),newmodelpath) = lookup_classdef(tp, modelpath, p);
        str = get_nth_compname(complst, n); then (str,tp);
    case (p,modelpath,Absyn.ELEMENTITEM(Absyn.ELEMENT(_,_,_,_,Absyn.COMPONENTS(_,tp,complst),_,_,NONE)) :: lst,n)
      equation 
        (Absyn.CLASS(_,_,_,_,Absyn.R_CONNECTOR(),_,_),newmodelpath) = lookup_classdef(tp, modelpath, p) "* Not so fast, since we lookup and instantiate two times just because this was not 
	 ** the connector we were looking for. *";
        c1 = list_length(complst);
        newn = n - c1;
        (str,tp) = get_nth_connector_str(p, modelpath, lst, newn); then (str,tp);
    case (p,modelpath,_ :: lst,n)
      equation 
        (str,tp) = get_nth_connector_str(p, modelpath, lst, n); then (str,tp);
    case (p,modelpath,{},n) then fail(); 
  end matchcontinue;
end get_nth_connector_str;

protected function get_nth_compname
  input Absyn_ComponentItemList in_absyn_componentitemlist;
  input Integer in_integer;
  output String out_string;
protected 
  type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
algorithm 
  out_string:=
  matchcontinue (in_absyn_componentitemlist,in_integer)
    local
      String id,res;
      Absyn_ComponentItemList lst,xs;
      Integer n1,n;
    case (Absyn.COMPONENTITEM(Absyn.COMPONENT(id,_,_),_) :: lst,1) then id; 
    case (_ :: xs,n)
      equation 
        n1 = n - 1;
        res = get_nth_compname(xs, n1); then res;
    case ({},_) then fail(); 
  end matchcontinue;
end get_nth_compname;

protected function count_public_connectors " This function takes a Class and counts the number of connector components in the class. 
 * This also includes counting in inherited classes.
 "
  input Absyn.Path in_path;
  input Absyn.Program in_program;
  input Absyn.Class in_class;
  output Integer out_integer;
algorithm 
  out_integer:=
  matchcontinue (in_path,in_program,in_class)
    local
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      type StringOption = Option<String>;
      Integer c1,c2,res;
      Absyn.Path modelpath,newmodelpath,cname;
      Absyn.Program p;
      String a,file;
      Boolean b,c,d;
      Absyn.Restriction e;
      Absyn_ElementItemList elt;
      Absyn_ClassPartList lst;
      StringOption cmt;
      Absyn.Class cdef;
    case (modelpath,p,Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(Absyn.PUBLIC(elt) :: lst,cmt),file))
      equation 
        c1 = count_public_connectors(modelpath, p, 
          Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(lst,cmt),file));
        c2 = count_connectors(modelpath, p, elt); then c1 + c2;
    case (modelpath,p,Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(_ :: lst,cmt),file))
      equation 
        res = count_public_connectors(modelpath, p, 
          Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(lst,cmt),file)); then res;
    case (modelpath,p,Absyn.CLASS(a,b,c,d,e,Absyn.PARTS({},cmt),file)) then 0; 
    case (modelpath,p,Absyn.CLASS(a,b,c,d,e,Absyn.DERIVED(cname,_,_,_,_),_))
      equation 
        (cdef,newmodelpath) = lookup_classdef(cname, modelpath, p);
        res = count_public_connectors(newmodelpath, p, cdef); then res;
  end matchcontinue;
end count_public_connectors;

protected function count_connectors "* function: count_connectors
 ** This function takes a Path to the current model and a ElementItem list and returns the number 
 ** of connector components in that list.
 *"
  input Absyn.Path in_path;
  input Absyn.Program in_program;
  input Absyn_ElementItemList in_absyn_elementitemlist;
  output Integer out_integer;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
algorithm 
  out_integer:=
  matchcontinue (in_path,in_program,in_absyn_elementitemlist)
    local
      type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
      Absyn.Class cdef;
      Absyn.Path newmodelpath,modelpath,tp;
      Integer c1,c2,res;
      Absyn.Program p;
      Absyn_ElementItemList lst;
      Absyn_ComponentItemList complst;
    case (modelpath,p,Absyn.ELEMENTITEM(Absyn.ELEMENT(_,_,_,_,Absyn.EXTENDS(tp,_),_,_,NONE)) :: lst)
      equation 
        (cdef,newmodelpath) = lookup_classdef(tp, modelpath, p);
        c1 = count_public_connectors(newmodelpath, p, cdef);
        c2 = count_connectors(modelpath, p, lst); then c1 + c2;
    case (modelpath,p,Absyn.ELEMENTITEM(Absyn.ELEMENT(_,_,_,_,Absyn.COMPONENTS(_,tp,complst),_,_,NONE)) :: lst)
      equation 
        (Absyn.CLASS(_,_,_,_,Absyn.R_CONNECTOR(),_,_),newmodelpath) = lookup_classdef(tp, modelpath, p);
        c1 = list_length(complst);
        c2 = count_connectors(modelpath, p, lst); then c1 + c2;
    case (modelpath,p,_ :: lst) " Rule above didn't match => element not connector components, try rest of list"
      equation 
        res = count_connectors(modelpath, p, lst); then res;
    case (_,_,{}) then 0; 
  end matchcontinue;
end count_connectors;

protected function get_connection_annotation_str "* function: get_connection_annotation_str
 ** This function takes an `EquationItem' and returns a comma separated string of values 
 ** from the flat record of a connection annotation that is found in the `EquationItem'.
 *"
  input Absyn.EquationItem in_equationitem;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_equationitem)
    local
      type SCode_ClassList = list<SCode.Class>;
      type Env_FrameList = list<Env.Frame>;
      type Absyn_ElementArgList = list<Absyn.ElementArg>;
      Absyn.FunctionArgs fargs;
      Absyn.Program p;
      String parsestr,gexpstr;
      SCode_ClassList p_1;
      Env_FrameList env;
      Exp.Exp newexp;
      Absyn_ElementArgList elts;
    case (Absyn.EQUATIONITEM(Absyn.EQ_CONNECT(_,_),SOME(Absyn.COMMENT(SOME(Absyn.ANNOTATION({Absyn.MODIFICATION(_,_,Absyn.CREF_IDENT("Line",_),SOME(Absyn.CLASSMOD(elts,NONE)),_)})),_))))
      equation 
        fargs = create_funcargs_from_elementargs(elts);
        (p,parsestr) = Parser.parsestring(Line_records);
        equality(parsestr = "Ok") " Always succeeds, check parsestr for errors ";
        p_1 = SCode.elaborate(p);
        env = Inst.make_env_from_program(p_1, Absyn.IDENT(""));
        (newexp,_) = Static.elab_graphics_exp(env, Absyn.CALL(Absyn.CREF_IDENT("Line",{}),fargs), false) "impl";
        gexpstr = Exp.print_exp_str(newexp); then gexpstr;
    case (Absyn.EQUATIONITEM(Absyn.EQ_CONNECT(_,_),NONE)) then fail(); 
  end matchcontinue;
end get_connection_annotation_str;

protected function create_funcargs_from_elementargs
  input Absyn_ElementArgList in_absyn_elementarglist;
  output Absyn.FunctionArgs out_functionargs;
protected 
  type Absyn_ElementArgList = list<Absyn.ElementArg>;
algorithm 
  out_functionargs:=
  matchcontinue (in_absyn_elementarglist)
    local
      type Absyn_ExpList = list<Absyn.Exp>;
      type Absyn_NamedArgList = list<Absyn.NamedArg>;
      Absyn_ExpList expl;
      Absyn_NamedArgList narg;
      String id;
      Absyn.Exp exp;
      Absyn_ElementArgList xs;
    case ({}) then Absyn.FUNCTIONARGS({},{}); 
    case (Absyn.MODIFICATION(_,_,Absyn.CREF_IDENT(id,_),SOME(Absyn.CLASSMOD(_,SOME(exp))),_) :: xs)
      equation 
        Absyn.FUNCTIONARGS(expl,narg) = create_funcargs_from_elementargs(xs); then Absyn.FUNCTIONARGS(expl,Absyn.NAMEDARG(id,exp) :: narg);
    case (_ :: xs)
      equation 
        Absyn.FUNCTIONARGS(expl,narg) = create_funcargs_from_elementargs(xs); then Absyn.FUNCTIONARGS(expl,narg);
  end matchcontinue;
end create_funcargs_from_elementargs;

protected function get_nth_connectionitem_in_class "*relation: get_nth_connectionitem_in_class
 ** This function takes a `Class'and  an int ane returns the nth `EquationItem' containing a
 ** connect statement in that class.
*"
  input Absyn.Class in_class;
  input Integer in_integer;
  output Absyn.EquationItem out_equationitem;
algorithm 
  out_equationitem:=
  matchcontinue (in_class,in_integer)
    local
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      Absyn.EquationItem eq;
      Absyn_ClassPartList parts;
      Integer n;
    case (Absyn.CLASS(_,_,_,_,_,Absyn.PARTS(parts,_),_),n)
      equation 
        eq = get_nth_connectionitem_in_classparts(parts, n); then eq;
  end matchcontinue;
end get_nth_connectionitem_in_class;

protected function get_nth_connectionitem_in_classparts "*relation: get_nth_connectionitem_in_classparts
 ** This function takes a `ClassPart' list and an int and returns 
** the nth connections as an `EquationItem'.
*"
  input Absyn_ClassPartList in_absyn_classpartlist;
  input Integer in_integer;
  output Absyn.EquationItem out_equationitem;
protected 
  type Absyn_ClassPartList = list<Absyn.ClassPart>;
algorithm 
  out_equationitem:=
  matchcontinue (in_absyn_classpartlist,in_integer)
    local
      type Absyn_EquationItemList = list<Absyn.EquationItem>;
      Absyn.EquationItem eq;
      Absyn_EquationItemList e;
      Absyn_ClassPartList xs;
      Integer n,c1,newn;
    case (Absyn.EQUATIONS(e) :: xs,n)
      equation 
        eq = get_nth_connectionitem_in_equations(e, n); then eq;
    case (Absyn.EQUATIONS(e) :: xs,n) " The rule above failed, subtract the number of connections in the first equation section   and try with the rest of the classparts"
      equation 
        c1 = count_connections_in_equations(e);
        newn = n - c1;
        eq = get_nth_connectionitem_in_classparts(xs, newn); then eq;
    case (_ :: xs,n)
      equation 
        eq = get_nth_connectionitem_in_classparts(xs, n); then eq;
  end matchcontinue;
end get_nth_connectionitem_in_classparts;

protected function get_nth_connectionitem_in_equations "* function: get_nth_connection_in_equations
   ** This function takes  an `Equation' list and an int and 
   ** returns the nth connection as an `Equation'. If the number is larger than the number of connections
 ** in the list, the function fails.
*"
  input Absyn_EquationItemList in_absyn_equationitemlist;
  input Integer in_integer;
  output Absyn.EquationItem out_equationitem;
protected 
  type Absyn_EquationItemList = list<Absyn.EquationItem>;
algorithm 
  out_equationitem:=
  matchcontinue (in_absyn_equationitemlist,in_integer)
    local
      Absyn.EquationItem eq;
      Absyn_EquationItemList xs;
      Integer newn,n;
    case ((eq = Absyn.EQUATIONITEM(Absyn.EQ_CONNECT(_,_),_)) :: xs,1) then eq; 
    case (Absyn.EQUATIONITEM(Absyn.EQ_CONNECT(_,_),_) :: xs,n)
      equation 
        newn = n - 1;
        eq = get_nth_connectionitem_in_equations(xs, newn); then eq;
    case (_ :: xs,n)
      equation 
        eq = get_nth_connectionitem_in_equations(xs, n); then eq;
    case ({},_) then fail(); 
  end matchcontinue;
end get_nth_connectionitem_in_equations;

protected function get_connection_str "* function: get_connection_str
   ** This function takes an `Equation' assumed to contain a connection and returns a 
   ** comma separated string of componentreferences, e.g \"R1.n,C.p\" for  connect(R1.n,C.p).
   *"
  input Absyn.Equation in_equation;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_equation)
    local
      Absyn.Path p1,p2;
      String s1,s2,s3,str;
      Absyn.ComponentRef cr1,cr2;
    case Absyn.EQ_CONNECT(cr1,cr2)
      equation 
        p1 = Absyn.cref_to_path(cr1);
        p2 = Absyn.cref_to_path(cr2);
        s1 = Absyn.path_string(p1);
        s2 = Absyn.path_string(p2);
        s3 = string_append(s1, ",");
        str = string_append(s3, s2); then str;
  end matchcontinue;
end get_connection_str;

protected function count_connections "* function: count_connections
 ** This function takes a `Class' and returns an int with the number of connections
** in the `Class'.
*"
  input Absyn.Class in_class;
  output Integer out_integer;
algorithm 
  out_integer:=
  matchcontinue (in_class)
    local
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      Integer count;
      Absyn_ClassPartList parts;
    case Absyn.CLASS(_,_,_,_,_,Absyn.PARTS(parts,_),_)
      equation 
        count = count_connections_in_classparts(parts); then count;
  end matchcontinue;
end count_connections;

protected function count_connections_in_classparts "* function: count_connections_in_classparts
 ** This function takes a `ClassPart' list and returns an int with the number of connections in that list.
*"
  input Absyn_ClassPartList in_absyn_classpartlist;
  output Integer out_integer;
protected 
  type Absyn_ClassPartList = list<Absyn.ClassPart>;
algorithm 
  out_integer:=
  matchcontinue (in_absyn_classpartlist)
    local
      type Absyn_EquationItemList = list<Absyn.EquationItem>;
      Integer r1,r2,res;
      Absyn_EquationItemList eqlist;
      Absyn_ClassPartList xs;
    case (Absyn.EQUATIONS(eqlist) :: xs)
      equation 
        r1 = count_connections_in_equations(eqlist);
        r2 = count_connections_in_classparts(xs); then r1 + r2;
    case (_ :: xs)
      equation 
        res = count_connections_in_classparts(xs); then res;
    case ({}) then 0; 
  end matchcontinue;
end count_connections_in_classparts;

protected function count_connections_in_equations "* function: count_connections_in_equations
 ** This function takes an `Equation' list and returns  an int 
 ** with the number of connect statements in that list.
*"
  input Absyn_EquationItemList in_absyn_equationitemlist;
  output Integer out_integer;
protected 
  type Absyn_EquationItemList = list<Absyn.EquationItem>;
algorithm 
  out_integer:=
  matchcontinue (in_absyn_equationitemlist)
    local
      Integer r1,res;
      Absyn_EquationItemList xs;
    case (Absyn.EQUATIONITEM(Absyn.EQ_CONNECT(_,_),_) :: xs)
      equation 
        r1 = count_connections_in_equations(xs); then r1 + 1;
    case (_ :: xs)
      equation 
        res = count_connections_in_equations(xs); then res;
    case ({}) then 0; 
  end matchcontinue;
end count_connections_in_equations;

protected function get_component_annotations_from_elts
  input Absyn_ElementList comps;
  output String res_1;
protected 
  type Absyn_ElementList = list<Absyn.Element>;
  type SCode_ClassList = list<SCode.Class>;
  type Env_FrameList = list<Env.Frame>;
  type StringList = list<String>;
  Absyn.Program p;
  String parsestr,res_1;
  SCode_ClassList p_1;
  Env_FrameList env;
  StringList res;
algorithm 
  (p,parsestr) := Parser.parsestring(Placement_records);
  equality(parsestr := "Ok") " Always succeeds, check parsestr for errors ";
  p_1 := SCode.elaborate(p);
  env := Inst.make_env_from_program(p_1, Absyn.IDENT(""));
  res := get_componentitems_annotations(comps, env);
  res_1 := Util.string_delimit_list(res, ",");
end get_component_annotations_from_elts;

protected function get_componentitems_annotations
  input Absyn_ElementList in_absyn_elementlist;
  input Env.Env in_env;
  output StringList out_stringlist;
protected 
  type Absyn_ElementList = list<Absyn.Element>;
  type StringList = list<String>;
algorithm 
  out_stringlist:=
  matchcontinue (in_absyn_elementlist,in_env)
    local
      type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
      type Env_FrameList = list<Env.Frame>;
      StringList res1,res2,res;
      Absyn_ComponentItemList items;
      Absyn_ElementList rest;
      Env_FrameList env;
    case ({},_) then {}; 
    case (Absyn.ELEMENT(_,_,_,_,Absyn.COMPONENTS(_,_,items),_,_,_) :: rest,env)
      equation 
        res1 = get_componentitems_annotations_from_items(items, env);
        res2 = get_componentitems_annotations(rest, env);
        res = list_append(res1, res2); then res;
    case (Absyn.ELEMENT(_,_,_,_,Absyn.COMPONENTS(_,_,_),_,_,_) :: rest,env)
      equation 
        res2 = get_componentitems_annotations(rest, env);
        res = list_append({"{}"}, res2); then res;
    case (_ :: rest,env)
      equation 
        res = get_componentitems_annotations(rest, env); then res;
  end matchcontinue;
end get_componentitems_annotations;

protected function get_componentitems_annotations_from_items
  input Absyn_ComponentItemList in_absyn_componentitemlist;
  input Env.Env in_env;
  output StringList out_stringlist;
protected 
  type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
  type StringList = list<String>;
algorithm 
  out_stringlist:=
  matchcontinue (in_absyn_componentitemlist,in_env)
    local
      type Env_FrameList = list<Env.Frame>;
      type Connect_SetList = list<Connect.Set>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Absyn_ElementArgList = list<Absyn.ElementArg>;
      Env_FrameList env,env_1;
      SCode.Class c;
      SCode.Mod mod_1;
      Types.Mod mod_2;
      DAE_ElementList dae,dae_1;
      Connect_SetList cs;
      TupleTypes_TTypeAbsyn_PathOption t;
      ClassInf.State state;
      String gexpstr,gexpstr_1;
      StringList res;
      Absyn_ElementArgList mod;
      Absyn_ComponentItemList rest;
    case ({},env) then {}; 
    case (Absyn.COMPONENTITEM(_,SOME(Absyn.COMMENT(SOME(Absyn.ANNOTATION(Absyn.MODIFICATION(_,_,Absyn.CREF_IDENT("Placement",_),SOME(Absyn.CLASSMOD(mod,NONE)),_) :: _)),_))) :: rest,env)
      equation 
        (c,env_1) = Lookup.lookup_class(env, Absyn.IDENT("Placement"), false);
        mod_1 = SCode.build_mod(SOME(Absyn.CLASSMOD(mod,NONE)), false);
        mod_2 = Mod.elab_mod(env_1, Prefix.NOPRE, mod_1, false);
        (dae,_,cs,t,state) = Inst.inst_class(env_1, mod_2, Prefix.NOPRE, {}, c, {}, false, Inst.TOP_CALL);
        dae_1 = Inst.init_vars_modelica_output(dae) " Put bindings of variables as expressions inside variable elements of the dae instead of equations ";
        gexpstr = DAE.get_variable_bindings_str(dae_1);
        gexpstr_1 = Util.string_append_list({"{",gexpstr,"}"});
        res = get_componentitems_annotations_from_items(rest, env); then gexpstr_1 :: res;
    case (Absyn.COMPONENTITEM(_,NONE) :: (rest = _ :: _),env)
      equation 
        res = get_componentitems_annotations_from_items(rest, env); then "{}," :: res;
    case ({Absyn.COMPONENTITEM(_,NONE)},env) then {"{}"}; 
  end matchcontinue;
end get_componentitems_annotations_from_items;

protected function get_component_annotation "* function: get_component_annotation
 ** This function takes an `Element' and returns a comma separated string of values
 ** corresponding to the flat record for a component annotation. If several components are declared 
 ** within the eleement, a list of values is given for each of them.
*"
  input Absyn.Element in_element;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_element)
    local
      type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
      String str;
      Absyn_ComponentItemList lst;
    case (Absyn.ELEMENT(_,_,_,_,Absyn.COMPONENTS(_,_,lst),_,_,NONE))
      equation 
        str = get_componentitems_annotation(lst); then str;
    case _ then ""; 
  end matchcontinue;
end get_component_annotation;

protected function get_componentitems_annotation
  input Absyn_ComponentItemList in_absyn_componentitemlist;
  output String out_string;
protected 
  type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
algorithm 
  out_string:=
  matchcontinue (in_absyn_componentitemlist)
    local
      type Absyn_ElementArgList = list<Absyn.ElementArg>;
      String s1,s2,s3,str,res;
      Absyn_ElementArgList mod;
      Absyn_ComponentItemList rest;
    case (Absyn.COMPONENTITEM(_,SOME(Absyn.COMMENT(SOME(Absyn.ANNOTATION((mod = Absyn.MODIFICATION(_,_,Absyn.CREF_IDENT("Placement",_),_,_) :: _))),_))) :: (rest = _ :: _))
      equation 
        s1 = get_annotation_string(Placement_records, Absyn.ANNOTATION(mod));
        s2 = string_append("{", s1);
        s3 = string_append(s2, "},");
        str = get_componentitems_annotation(rest);
        res = string_append(s3, str); then res;
    case ({Absyn.COMPONENTITEM(_,SOME(Absyn.COMMENT(SOME(Absyn.ANNOTATION((mod = Absyn.MODIFICATION(_,_,Absyn.CREF_IDENT("Placement",_),_,_) :: _))),_)))})
      equation 
        s1 = get_annotation_string(Placement_records, Absyn.ANNOTATION(mod));
        s2 = string_append("{", s1);
        res = string_append(s2, "}"); then res;
    case (Absyn.COMPONENTITEM(_,SOME(Absyn.COMMENT(NONE,_))) :: (rest = _ :: _))
      equation 
        str = get_componentitems_annotation(rest);
        res = string_append("{nada},", str); then res;
    case (Absyn.COMPONENTITEM(_,NONE) :: (rest = _ :: _))
      equation 
        str = get_componentitems_annotation(rest);
        res = string_append("{},", str); then res;
    case ({Absyn.COMPONENTITEM(_,NONE)})
      equation 
        res = "{}"; then res;
  end matchcontinue;
end get_componentitems_annotation;

public function get_component_modification "* function: get_component_modification
 ** This function takes an `Element' and returns a comma separated list of 
 ** Code expression for the modification of the component.
*"
  input Absyn.Element in_element;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_element)
    local
      type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
      String str;
      Absyn_ComponentItemList lst;
    case (Absyn.ELEMENT(_,_,_,_,Absyn.COMPONENTS(_,_,lst),_,_,NONE))
      equation 
        str = get_componentitems_modification(lst); then str;
    case _ then ""; 
  end matchcontinue;
end get_component_modification;

protected function get_componentitems_modification
  input Absyn_ComponentItemList in_absyn_componentitemlist;
  output String out_string;
protected 
  type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
algorithm 
  out_string:=
  matchcontinue (in_absyn_componentitemlist)
    local
      String s1,s2,res,str;
      Absyn.Modification mod;
      Absyn_ComponentItemList rest;
    case (Absyn.COMPONENTITEM(Absyn.COMPONENT(_,_,SOME(mod)),_) :: (rest = _ :: _))
      equation 
        s1 = Dump.print_exp_str(Absyn.CODE(Absyn.C_MODIFICATION(mod)));
        s2 = get_componentitems_modification(rest);
        res = Util.string_append_list({s1,",",s2}); then res;
    case ({Absyn.COMPONENTITEM(Absyn.COMPONENT(_,_,SOME(mod)),_)})
      equation 
        res = Dump.print_exp_str(Absyn.CODE(Absyn.C_MODIFICATION(mod))); then res;
    case (Absyn.COMPONENTITEM(Absyn.COMPONENT(_,_,NONE),_) :: (rest = _ :: _))
      equation 
        str = get_componentitems_modification(rest);
        res = string_append("Code(),", str); then res;
    case (Absyn.COMPONENTITEM(Absyn.COMPONENT(_,_,NONE),_) :: (rest = _ :: _))
      equation 
        str = get_componentitems_modification(rest);
        res = string_append("Code(),", str); then res;
    case ({Absyn.COMPONENTITEM(_,NONE)})
      equation 
        res = "Code()"; then res;
  end matchcontinue;
end get_componentitems_modification;

protected function get_annotation_string "* function_ get_annotation_string
 ** This function takes an annotation and returns a comma separates string of values
 ** representing the flat record of the specific annotation.
 ** The function as two special rules for handling of Icon and Diagram annotations since these 
 ** two contain graphic primitives, which must be handled specially because Modelica does not have 
 ** the possibility to store polymorphic values (e.g. different record classes with the same base class)
 ** in for instance an array.
*"
  input String in_string;
  input Absyn.Annotation in_annotation;
  output String out_string;
algorithm 
  out_string:=
  matchcontinue (in_string,in_annotation)
    local
      type Absyn_ElementArgList = list<Absyn.ElementArg>;
      type SCode_ClassList = list<SCode.Class>;
      type Env_FrameList = list<Env.Frame>;
      type Connect_SetList = list<Connect.Set>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      Absyn_ElementArgList stripmod,mod,gxmods;
      Absyn.Exp graphicexp;
      SCode.Mod mod_1;
      Absyn.Program p;
      String parsestr,str,gexpstr,s1,totstr,totstring,anncname;
      SCode_ClassList p_1;
      Env_FrameList env;
      Absyn.Class placementc;
      SCode.Class placementclass;
      Types.Mod mod_2;
      DAE_ElementList dae,dae_1;
      Connect_SetList cs;
      TupleTypes_TTypeAbsyn_PathOption t;
      ClassInf.State state;
      Exp.Exp graphicexp2;
      Types.Properties prop;
    case (totstring,Absyn.ANNOTATION({Absyn.MODIFICATION(_,_,Absyn.CREF_IDENT("Icon",_),SOME(Absyn.CLASSMOD(mod,_)),_)}))
      equation 
        (stripmod,{Absyn.MODIFICATION(_,_,_,SOME(Absyn.CLASSMOD(_,SOME(graphicexp))),_)}) = strip_graphics_modification(mod);
        mod_1 = SCode.build_mod(SOME(Absyn.CLASSMOD(stripmod,NONE)), false);
        (p,parsestr) = Parser.parsestring(totstring);
        equality(parsestr = "Ok") " Always succeeds, check parsestr for errors ";
        p_1 = SCode.elaborate(p);
        env = Inst.make_simple_env_from_program(p_1, Absyn.IDENT("Icon"));
        placementc = get_class_in_program("Icon", p);
        placementclass = SCode.elab_class(placementc);
        mod_2 = Mod.elab_mod(env, Prefix.NOPRE, mod_1, false);
        (dae,_,cs,t,state) = Inst.inst_class(env, mod_2, Prefix.NOPRE, {}, placementclass, {}, false, 
          Inst.TOP_CALL);
        dae_1 = Inst.init_vars_modelica_output(dae) " Put bindings of variables as expressions inside variable elements of the dae instead of equations ";
        str = DAE.get_variable_bindings_str(dae_1);
        (graphicexp2,prop) = Static.elab_graphics_exp(env, graphicexp, false) "impl";
        gexpstr = Exp.print_exp_str(graphicexp2);
        s1 = string_append(str, ",");
        totstr = string_append(s1, gexpstr); then totstr;
    case (totstring,Absyn.ANNOTATION({Absyn.MODIFICATION(_,_,Absyn.CREF_IDENT("Icon",_),SOME(Absyn.CLASSMOD(mod,_)),_)})) " First line in the first rule above fails if return value from strip_graphics_modification doesn't match the rhs of => "
      equation 
        (stripmod,gxmods) = strip_graphics_modification(mod);
        mod_1 = SCode.build_mod(SOME(Absyn.CLASSMOD(stripmod,NONE)), false);
        (p,parsestr) = Parser.parsestring(totstring);
        equality(parsestr = "Ok") " Always succeeds, check parsestr for errors ";
        p_1 = SCode.elaborate(p);
        env = Inst.make_simple_env_from_program(p_1, Absyn.IDENT("Icon"));
        placementc = get_class_in_program("Icon", p);
        placementclass = SCode.elab_class(placementc);
        mod_2 = Mod.elab_mod(env, Prefix.NOPRE, mod_1, true);
        (dae,_,cs,t,state) = Inst.inst_class(env, mod_2, Prefix.NOPRE, {}, placementclass, {}, false, 
          Inst.TOP_CALL);
        dae_1 = Inst.init_vars_modelica_output(dae) " Put bindings of variables as expressions inside variable elements of the dae instead of equations ";
        str = DAE.get_variable_bindings_str(dae_1); then str;
    case (totstring,Absyn.ANNOTATION({Absyn.MODIFICATION(_,_,Absyn.CREF_IDENT("Diagram",_),SOME(Absyn.CLASSMOD(mod,_)),_)}))
      equation 
        (stripmod,{Absyn.MODIFICATION(_,_,_,SOME(Absyn.CLASSMOD(_,SOME(graphicexp))),_)}) = strip_graphics_modification(mod);
        mod_1 = SCode.build_mod(SOME(Absyn.CLASSMOD(stripmod,NONE)), false);
        (p,parsestr) = Parser.parsestring(totstring);
        equality(parsestr = "Ok") " Always succeeds, check parsestr for errors ";
        p_1 = SCode.elaborate(p);
        env = Inst.make_env_from_program(p_1, Absyn.IDENT("Diagram"));
        placementc = get_class_in_program("Diagram", p);
        placementclass = SCode.elab_class(placementc);
        mod_2 = Mod.elab_mod(env, Prefix.NOPRE, mod_1, false);
        (dae,_,cs,t,state) = Inst.inst_class(env, mod_2, Prefix.NOPRE, {}, placementclass, {}, false, 
          Inst.TOP_CALL);
        dae_1 = Inst.init_vars_modelica_output(dae) " Put bindings of variables as expressions inside variable elements of the dae instead of equations ";
        str = DAE.get_variable_bindings_str(dae_1);
        (graphicexp2,prop) = Static.elab_graphics_exp(env, graphicexp, false) "impl";
        gexpstr = Exp.print_exp_str(graphicexp2);
        s1 = string_append(str, ",");
        totstr = string_append(s1, gexpstr); then totstr;
    case (totstring,Absyn.ANNOTATION({Absyn.MODIFICATION(_,_,Absyn.CREF_IDENT(anncname,_),SOME(Absyn.CLASSMOD(mod,_)),_)}))
      equation 
        mod_1 = SCode.build_mod(SOME(Absyn.CLASSMOD(mod,NONE)), false);
        (p,parsestr) = Parser.parsestring(totstring);
        equality(parsestr = "Ok") " Always succeeds, check parsestr for errors ";
        p_1 = SCode.elaborate(p);
        env = Inst.make_env_from_program(p_1, Absyn.IDENT(anncname));
        placementc = get_class_in_program(anncname, p);
        placementclass = SCode.elab_class(placementc);
        mod_2 = Mod.elab_mod(env, Prefix.NOPRE, mod_1, false);
        (dae,_,cs,t,state) = Inst.inst_class(env, mod_2, Prefix.NOPRE, {}, placementclass, {}, false, 
          Inst.TOP_CALL);
        dae_1 = Inst.init_vars_modelica_output(dae) " Put bindings of variables as expressions inside variable elements of the dae instead of equations ";
        str = DAE.get_variable_bindings_str(dae_1); then str;
    case (_,_)
      equation 
        Print.print_buf("get_annotation_string failed!\n"); then fail();
  end matchcontinue;
end get_annotation_string;

protected function strip_graphics_modification "* function: strip_graphics_modification
 ** This function strips out the `graphics' modification from an ElementArg list and return 
 ** two lists, one with the other modifications and the second with the `graphics' modification
*"
  input Absyn_ElementArgList in_absyn_elementarglist;
  output Absyn_ElementArgList out_absyn_elementarglist1;
  output Absyn_ElementArgList out_absyn_elementarglist2;
protected 
  type Absyn_ElementArgList = list<Absyn.ElementArg>;
algorithm 
  (out_absyn_elementarglist1,out_absyn_elementarglist2):=
  matchcontinue (in_absyn_elementarglist)
    local
      Absyn.ElementArg mod;
      Absyn_ElementArgList rest,l1,l2;
    case ((mod = Absyn.MODIFICATION(_,_,Absyn.CREF_IDENT("graphics",_),_,_)) :: rest) then (rest,{mod}); 
    case ((mod = Absyn.MODIFICATION(_,_,_,_,_)) :: rest)
      equation 
        (l1,l2) = strip_graphics_modification(rest); then (mod :: l1,l2);
    case ({}) then ({},{}); 
  end matchcontinue;
end strip_graphics_modification;

public function get_components_in_class "* function: get_components_in_class
 ** 
 ** Both public and protected lists are searched.
*"
  input Absyn.Class in_class;
  output Absyn_ElementList out_absyn_elementlist;
protected 
  type Absyn_ElementList = list<Absyn.Element>;
algorithm 
  out_absyn_elementlist:=
  matchcontinue (in_class)
    local
      type StringOption = Option<String>;
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      String a,file;
      Boolean b,c,d;
      Absyn.Restriction e;
      StringOption cmt;
      Absyn_ElementList lst1,lst2,res;
      Absyn_ElementItemList elts;
      Absyn_ClassPartList lst;
    case (Absyn.CLASS(a,b,c,d,e,Absyn.PARTS({},cmt),_)) then {}; 
    case (Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(Absyn.PUBLIC(elts) :: lst,cmt),file)) " Search in public list "
      equation 
        lst1 = get_components_in_class(Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(lst,cmt),file));
        lst2 = get_components_in_elementitems(elts);
        res = list_append(lst2, lst1); then res;
    case (Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(Absyn.PROTECTED(elts) :: lst,cmt),file)) " Search in protected list "
      equation 
        lst1 = get_components_in_class(Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(lst,cmt),file));
        lst2 = get_components_in_elementitems(elts);
        res = list_append(lst2, lst1); then res;
    case (_) then {}; 
  end matchcontinue;
end get_components_in_class;

protected function get_components_in_elementitems
  input Absyn_ElementItemList in_absyn_elementitemlist;
  output Absyn_ElementList out_absyn_elementlist;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
  type Absyn_ElementList = list<Absyn.Element>;
algorithm 
  out_absyn_elementlist:=
  matchcontinue (in_absyn_elementitemlist)
    local
      Absyn_ElementList res;
      Absyn.Element elt;
      Absyn_ElementItemList rest;
    case ({}) then {}; 
    case (Absyn.ELEMENTITEM(elt) :: rest)
      equation 
        res = get_components_in_elementitems(rest); then elt :: res;
    case (_ :: rest)
      equation 
        res = get_components_in_elementitems(rest); then res;
  end matchcontinue;
end get_components_in_elementitems;

protected function get_nth_component_in_class
  input Absyn.Class in_class;
  input Integer in_integer;
  output Absyn.Element out_element;
algorithm 
  out_element:=
  matchcontinue (in_class,in_integer)
    local
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      type StringOption = Option<String>;
      Integer count,n,c1,newn;
      Absyn.Element res;
      String a,file,newnstr;
      Boolean b,c,d;
      Absyn.Restriction e;
      Absyn_ElementItemList elt;
      Absyn_ClassPartList lst,rest;
      StringOption cmt;
    case (Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(Absyn.PUBLIC(elt) :: lst,cmt),file),n)
      equation 
        count = count_components_in_elts(elt);
        (n <= count) = true;
        res = get_nth_component_in_elementitems(elt, n); then res;
    case (Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(Absyn.PUBLIC(elt) :: rest,cmt),file),n) " The rule above failed, i.e the nth number is larger than # elements in first public list   subtract and try next public list "
      equation 
        c1 = count_components_in_elts(elt);
        newn = n - c1;
        newnstr = int_string(newn);
        (newn > 0) = true;
        res = get_nth_component_in_class(Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(rest,cmt),file), newn); then res;
    case (Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(Absyn.PROTECTED(elt) :: lst,cmt),file),n)
      equation 
        res = get_nth_component_in_elementitems(elt, n); then res;
    case (Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(Absyn.PROTECTED(elt) :: rest,cmt),file),n) " The rule above failed, i.e the nth number is larger than # elements in first public list   subtract and try next public list "
      equation 
        c1 = count_components_in_elts(elt);
        newn = n - c1;
        (newn > 0) = true;
        res = get_nth_component_in_class(Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(rest,cmt),file), newn); then res;
    case (Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(_ :: lst,cmt),file),n)
      equation 
        res = get_nth_component_in_class(Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(lst,cmt),file), n); then res;
    case (Absyn.CLASS(a,b,c,d,e,Absyn.PARTS({},cmt),file),_) then fail(); 
    case (Absyn.CLASS(a,b,c,d,e,Absyn.DERIVED(_,_,_,_,_),file),_) then fail(); 
  end matchcontinue;
end get_nth_component_in_class;

public function get_elementitems_in_class "* function: get_elementitems_in_class
 ** 
 ** Both public and protected lists are searched.
*"
  input Absyn.Class in_class;
  output Absyn_ElementItemList out_absyn_elementitemlist;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
algorithm 
  out_absyn_elementitemlist:=
  matchcontinue (in_class)
    local
      type StringOption = Option<String>;
      String a,file;
      Boolean b,c,d;
      Absyn.Restriction e;
      StringOption cmt;
      Absyn_ElementItemList lst1,lst,elts;
    case (Absyn.CLASS(a,b,c,d,e,Absyn.PARTS({},cmt),_)) then {}; 
    case (Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(Absyn.PUBLIC(elts) :: lst,cmt),file)) " Search in public list "
      equation 
        lst1 = get_elementitems_in_class(Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(lst,cmt),file));
        lst = list_append(elts, lst1); then lst;
    case (Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(Absyn.PROTECTED(elts) :: lst,cmt),file)) " Search in protected list "
      equation 
        lst1 = get_elementitems_in_class(Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(lst,cmt),file));
        lst = list_append(elts, lst1); then lst;
    case (_) then {}; 
  end matchcontinue;
end get_elementitems_in_class;

protected function get_nth_component_in_elementitems
  input Absyn_ElementItemList in_absyn_elementitemlist;
  input Integer in_integer;
  output Absyn.Element out_element;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
algorithm 
  out_element:=
  matchcontinue (in_absyn_elementitemlist,in_integer)
    local
      type Absyn_ElementSpecOption = Option<Absyn.ElementSpec>;
      type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
      Boolean a,b;
      Absyn.InnerOuter c;
      String d,g;
      Absyn.ElementAttributes e;
      Absyn.Path f;
      Absyn.ComponentItem elt;
      Integer h,numcomps,newn,n,n_1;
      Absyn_ElementSpecOption i;
      Absyn.Element res;
      Absyn_ComponentItemList lst;
      Absyn_ElementItemList rest;
    case (Absyn.ELEMENTITEM(Absyn.ELEMENT(a,b,c,d,Absyn.COMPONENTS(e,f,elt :: _),g,h,i)) :: _,1) then Absyn.ELEMENT(a,b,c,d,Absyn.COMPONENTS(e,f,{elt}),g,h,i); 
    case (Absyn.ELEMENTITEM(Absyn.ELEMENT(_,_,_,_,Absyn.COMPONENTS(_,_,lst),_,_,_)) :: rest,n)
      equation 
        numcomps = list_length(lst);
        (n > numcomps) = true;
        newn = n - numcomps;
        res = get_nth_component_in_elementitems(rest, newn); then res;
    case (Absyn.ELEMENTITEM((elt = Absyn.ELEMENT(a,b,c,d,Absyn.COMPONENTS(e,f,lst),g,h,i))) :: rest,n)
      equation 
        numcomps = list_length(lst);
        (n <= numcomps) = true;
        n_1 = n - 1;
        elt = list_nth(lst, n_1); then Absyn.ELEMENT(a,b,c,d,Absyn.COMPONENTS(e,f,{elt}),g,h,i);
    case (_ :: rest,n) "  rule	int_ge(n,1) => true &
	get_nth_component_in_elementitems(rest,n) => res
	---------------------------------------------------
	get_nth_component_in_elementitems(Absyn.ELEMENTITEM(elt)::rest,n) => res"
      equation 
        res = get_nth_component_in_elementitems(rest, n); then res;
    case ({},_) then fail(); 
  end matchcontinue;
end get_nth_component_in_elementitems;

protected function get_components_name_and_type_and_comment
  input Absyn_ElementList elts;
  input Env.Env env;
  output String res;
protected 
  type Absyn_ElementList = list<Absyn.Element>;
  type StringList = list<String>;
  StringList lst;
  String lst_1;
algorithm 
  lst := get_components_name_and_type_and_comment2(elts, env);
  lst_1 := Util.string_delimit_list(lst, "},{");
  res := Util.string_append_list({"{",lst_1,"}"});
end get_components_name_and_type_and_comment;

protected function get_components_name_and_type_and_comment2
  input Absyn_ElementList in_absyn_elementlist;
  input Env.Env in_env;
  output StringList out_stringlist;
protected 
  type Absyn_ElementList = list<Absyn.Element>;
  type StringList = list<String>;
algorithm 
  out_stringlist:=
  matchcontinue (in_absyn_elementlist,in_env)
    local
      type Env_FrameList = list<Env.Frame>;
      StringList lst1,lst2,res;
      Absyn.Element elt;
      Absyn_ElementList rest;
      Env_FrameList env;
    case ({},_) then {}; 
    case (elt :: rest,env)
      equation 
        lst1 = get_component_name_and_type_and_comment(elt, env);
        lst2 = get_components_name_and_type_and_comment2(rest, env);
        res = list_append(lst1, lst2); then res;
  end matchcontinue;
end get_components_name_and_type_and_comment2;

public function get_component_name_and_type_and_comment
  input Absyn.Element in_element;
  input Env.Env in_env;
  output StringList out_stringlist;
protected 
  type StringList = list<String>;
algorithm 
  out_stringlist:=
  matchcontinue (in_element,in_env)
    local
      type Env_FrameList = list<Env.Frame>;
      SCode.Class c;
      Env_FrameList env_1,env;
      Absyn.Path envpath,p_1,p;
      String tpname,typename;
      StringList names,lst;
    case (Absyn.ELEMENT(_,_,_,_,Absyn.COMPONENTS(_,p,lst),_,_,_),env)
      equation 
        (c,env_1) = Lookup.lookup_class(env, p, true);
        SOME(envpath) = Env.get_env_path(env_1);
        tpname = Absyn.path_last_ident(p);
        p_1 = Absyn.join_paths(envpath, Absyn.IDENT(tpname));
        typename = Absyn.path_string(p_1);
        names = get_componentitems_name(lst);
        lst = prefix_typename(typename, names); then lst;
    case (Absyn.ELEMENT(_,_,_,_,Absyn.COMPONENTS(_,p,lst),_,_,_),env)
      equation 
        typename = Absyn.path_string(p);
        names = get_componentitems_name(lst);
        lst = prefix_typename(typename, names); then lst;
    case (_,env) then {}; 
    case (_,_)
      equation 
        print("get_component_name_and_type_and_comment failed\n"); then fail();
  end matchcontinue;
end get_component_name_and_type_and_comment;

protected function prefix_typename
  input String in_string;
  input StringList in_stringlist;
  output StringList out_stringlist;
protected 
  type StringList = list<String>;
algorithm 
  out_stringlist:=
  matchcontinue (in_string,in_stringlist)
    local
      StringList res,rest;
      String str_1,tp,str;
    case (_,{}) then {}; 
    case (tp,str :: rest)
      equation 
        res = prefix_typename(tp, rest);
        str_1 = Util.string_append_list({tp,",",str}); then str_1 :: res;
  end matchcontinue;
end prefix_typename;

public function get_componentitems_name "* function_get_componentitems_name
 ** This function takes a `ComponentItems' list and returns a 
 ** comma separated list of all
 ** component names and comments (if any).
*"
  input Absyn_ComponentItemList in_absyn_componentitemlist;
  output StringList out_stringlist;
protected 
  type Absyn_ComponentItemList = list<Absyn.ComponentItem>;
  type StringList = list<String>;
algorithm 
  out_stringlist:=
  matchcontinue (in_absyn_componentitemlist)
    local
      String s1,str,c1,s2;
      StringList lst,res;
      Absyn.ComponentItem c2;
      Absyn_ComponentItemList rest;
    case (Absyn.COMPONENTITEM(Absyn.COMPONENT(c1,_,_),SOME(Absyn.COMMENT(_,SOME(s2)))) :: c2 :: rest)
      equation 
        s1 = string_append(c1, ",");
        lst = get_componentitems_name(c2 :: rest);
        str = Util.string_append_list({s1,"\"",s2,"\""}); then str :: lst;
    case (Absyn.COMPONENTITEM(Absyn.COMPONENT(c1,_,_),NONE) :: c2 :: rest)
      equation 
        s1 = string_append(c1, ",");
        lst = get_componentitems_name(c2 :: rest);
        str = Util.string_append_list({s1,"\"\""}); then str :: lst;
    case (_ :: rest)
      equation 
        res = get_componentitems_name(rest); then res;
    case ({Absyn.COMPONENTITEM(Absyn.COMPONENT(c1,_,_),SOME(Absyn.COMMENT(_,SOME(s2))))})
      local String res;
      equation 
        res = Util.string_append_list({c1,",\"",s2,"\""}); then {res};
    case ({Absyn.COMPONENTITEM(Absyn.COMPONENT(c1,_,_),_)})
      local String res;
      equation 
        res = Util.string_append_list({c1,",\"\""}); then {res};
    case ({_}) then {}; 
  end matchcontinue;
end get_componentitems_name;

public function add_to_public "* function: add_to_public
 ** This function takes a 'Class' definition and adds an `ElementItem' to the first public list in the class.
 ** If no public list is available in the class one is created.
 *"
  input Absyn.Class in_class;
  input Absyn.ElementItem in_elementitem;
  output Absyn.Class out_class;
algorithm 
  out_class:=
  matchcontinue (in_class,in_elementitem)
    local
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      type StringOption = Option<String>;
      Absyn_ElementItemList publst,publst2;
      Absyn_ClassPartList parts2,parts;
      String i,file;
      Boolean p,f,e;
      Absyn.Restriction r;
      StringOption cmt;
      Absyn.ElementItem eitem;
    case (Absyn.CLASS(i,p,f,e,r,Absyn.PARTS(parts,cmt),file),eitem)
      equation 
        publst = get_public_list(parts);
        publst2 = list_append(publst, {eitem});
        parts2 = replace_public_list(parts, publst2); then Absyn.CLASS(i,p,f,e,r,Absyn.PARTS(parts2,cmt),file);
    case (Absyn.CLASS(i,p,f,e,r,Absyn.DERIVED(_,_,_,_,_),file),eitem) then fail(); 
    case (Absyn.CLASS(i,p,f,e,r,Absyn.PARTS(parts,cmt),file),eitem) then Absyn.CLASS(i,p,f,e,r,Absyn.PARTS(Absyn.PUBLIC({eitem}) :: parts,cmt),
          file); 
  end matchcontinue;
end add_to_public;

protected function add_to_protected "* function: add_to_protected
 ** This function takes a 'Class' definition and adds an `ElementItem' to the first protected list in the class.
 ** If no protected list is available in the class one is created.
 *"
  input Absyn.Class in_class;
  input Absyn.ElementItem in_elementitem;
  output Absyn.Class out_class;
algorithm 
  out_class:=
  matchcontinue (in_class,in_elementitem)
    local
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      type StringOption = Option<String>;
      Absyn_ElementItemList protlst,protlst2;
      Absyn_ClassPartList parts2,parts;
      String i,file;
      Boolean p,f,e;
      Absyn.Restriction r;
      StringOption cmt;
      Absyn.ElementItem eitem;
    case (Absyn.CLASS(i,p,f,e,r,Absyn.PARTS(parts,cmt),file),eitem)
      equation 
        protlst = get_protected_list(parts);
        protlst2 = list_append(protlst, {eitem});
        parts2 = replace_protected_list(parts, protlst2); then Absyn.CLASS(i,p,f,e,r,Absyn.PARTS(parts2,cmt),file);
    case (Absyn.CLASS(i,p,f,e,r,Absyn.DERIVED(_,_,_,_,_),file),eitem) then fail(); 
    case (Absyn.CLASS(i,p,f,e,r,Absyn.PARTS(parts,cmt),file),eitem) then Absyn.CLASS(i,p,f,e,r,
          Absyn.PARTS(Absyn.PROTECTED({eitem}) :: parts,cmt),file); 
  end matchcontinue;
end add_to_protected;

protected function add_to_equation "* function: add_to_equation
 ** This function takes a 'Class' definition and adds an `EquationItem' to the first equation list in the class.
 ** If no public list is available in the class one is created.
 *"
  input Absyn.Class in_class;
  input Absyn.EquationItem in_equationitem;
  output Absyn.Class out_class;
algorithm 
  out_class:=
  matchcontinue (in_class,in_equationitem)
    local
      type Absyn_EquationItemList = list<Absyn.EquationItem>;
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      type StringOption = Option<String>;
      Absyn_EquationItemList eqlst,eqlst2;
      Absyn_ClassPartList parts2,parts,newparts;
      String i,file;
      Boolean p,f,e;
      Absyn.Restriction r;
      StringOption cmt;
      Absyn.EquationItem eitem;
    case (Absyn.CLASS(i,p,f,e,r,Absyn.PARTS(parts,cmt),file),eitem)
      equation 
        eqlst = get_equation_list(parts);
        eqlst2 = eitem :: eqlst;
        parts2 = replace_equation_list(parts, eqlst2); then Absyn.CLASS(i,p,f,e,r,Absyn.PARTS(parts2,cmt),file);
    case (Absyn.CLASS(i,p,f,e,r,Absyn.DERIVED(_,_,_,_,_),file),eitem) then fail(); 
    case (Absyn.CLASS(i,p,f,e,r,Absyn.PARTS(parts,cmt),file),eitem)
      equation 
        newparts = list_append(parts, {Absyn.EQUATIONS({eitem})}) " Add the equations last, to make nicer output if public section present "; then Absyn.CLASS(i,p,f,e,r,Absyn.PARTS(newparts,cmt),file);
  end matchcontinue;
end add_to_equation;

protected function replace_class_in_program "* function: replace_class_in_program
 ** This function takes a `Class' and a `Program' and replaces the class definition 
 ** at the top level in the program by the class definition of the `Class'.
 *"
  input Absyn.Class in_class;
  input Absyn.Program in_program;
  output Absyn.Program out_program;
algorithm 
  out_program:=
  matchcontinue (in_class,in_program)
    local
      type Absyn_ClassList = list<Absyn.Class>;
      Absyn.Class c,c1;
      Absyn.Within w;
      String name1,name2;
      Absyn_ClassList clst,newclst;
      Absyn.Program p;
    case (c,Absyn.PROGRAM({},w)) then Absyn.PROGRAM({c},w); 
    case ((c = Absyn.CLASS(name1,_,_,_,_,_,_)),Absyn.PROGRAM(Absyn.CLASS(name2,_,_,_,_,_,_) :: clst,w))
      equation 
        equality(name1 = name2); then Absyn.PROGRAM(c :: clst,w);
    case ((c = Absyn.CLASS(name1,_,_,_,_,_,_)),Absyn.PROGRAM((c1 = Absyn.CLASS(name2,_,_,_,_,_,_)) :: clst,w))
      equation 
        failure(equality(name1 = name2));
        Absyn.PROGRAM(newclst,w) = replace_class_in_program(c, Absyn.PROGRAM(clst,w)); then Absyn.PROGRAM(c1 :: newclst,w);
    case (c,p)
      equation 
        Print.print_buf("replace_class_in_program failed \n class:");
        Debug.fcall("dump", Dump.dump, Absyn.PROGRAM({c},Absyn.TOP()));
        Print.print_buf("\nprogram: \n");
        Debug.fcall("dump", Dump.dump, p); then fail();
  end matchcontinue;
end replace_class_in_program;

protected function insert_class_in_program "* function: insert_class_in_program 
 ** This function inserts the class into the Program at the scope given by the 
 ** within argument. If the class referenced by the within argument is not 
 ** defined, the function prints an error message and fails.
 *"
  input Absyn.Class in_class;
  input Absyn.Within in_within;
  input Absyn.Program in_program;
  output Absyn.Program out_program;
algorithm 
  out_program:=
  matchcontinue (in_class,in_within,in_program)
    local
      Absyn.Class c2,c3,c1;
      Absyn.Program pnew,p;
      Absyn.Within w;
      String n1,s1,name;
    case (c1,(w = Absyn.WITHIN(Absyn.QUALIFIED(n1,_))),p)
      equation 
        c2 = get_class_in_program(n1, p);
        c3 = insert_class_in_class(c1, w, c2);
        pnew = update_program(Absyn.PROGRAM({c3},Absyn.TOP()), p); then pnew;
    case (c1,(w = Absyn.WITHIN(Absyn.IDENT(n1))),p)
      equation 
        c2 = get_class_in_program(n1, p);
        c3 = insert_class_in_class(c1, w, c2);
        pnew = update_program(Absyn.PROGRAM({c3},Absyn.TOP()), p); then pnew;
    case ((c1 = Absyn.CLASS(name,_,_,_,_,_,_)),w,p)
      equation 
        print("Error inserting in class. (");
        s1 = Dump.unparse_within(0, w);
        print(s1);
        print(") program = \n") "& Dump.unparse_str p => pstr & print pstr & print \"\n\"  "; then fail();
  end matchcontinue;
end insert_class_in_program;

protected function insert_class_in_class "* function: insert_class_in_class
 ** This function takes a class to update (the first argument)  and an inner 
 ** class (which is either replacing
 ** an earlier class or is a new inner definition) and a within statement
 ** pointing inside the class (including the class itself in the reference), and updates the class with the inner class.
 *"
  input Absyn.Class in_class1;
  input Absyn.Within in_within2;
  input Absyn.Class in_class3;
  output Absyn.Class out_class;
algorithm 
  out_class:=
  matchcontinue (in_class1,in_within2,in_class3)
    local
      Absyn.Class cnew,c1,c2,cinner,cnew_1;
      String name,name2;
      Absyn.Path path;
    case (c1,Absyn.WITHIN(Absyn.IDENT(name)),c2)
      equation 
        cnew = replace_inner_class(c1, c2); then cnew;
    case (c1,Absyn.WITHIN(Absyn.QUALIFIED(name,path)),c2)
      equation 
        name2 = get_first_ident_from_path(path);
        cinner = get_inner_class(c2, name2);
        cnew = insert_class_in_class(c1, Absyn.WITHIN(path), cinner);
        cnew_1 = replace_inner_class(cnew, c2); then cnew_1;
    case (_,_,_)
      equation 
        Print.print_buf("insert_class_in_class failed\n"); then fail();
  end matchcontinue;
end insert_class_in_class;

protected function get_first_ident_from_path "* function: get_first_ident_from_path
 ** This function takes a `Path` as argument and returns the first `Ident' of the path.
 *"
  input Absyn.Path in_path;
  output Absyn.Ident out_ident;
algorithm 
  out_ident:=
  matchcontinue (in_path)
    local
      String name;
      Absyn.Path path;
    case (Absyn.IDENT(name)) then name; 
    case (Absyn.QUALIFIED(name,path)) then name; 
  end matchcontinue;
end get_first_ident_from_path;

protected function remove_inner_class "* function: remove_inner_class 
 ** This function takes two class definitions. The first one is the local class that should be
 ** removed from the second one.
 *"
  input Absyn.Class in_class1;
  input Absyn.Class in_class2;
  output Absyn.Class out_class;
algorithm 
  out_class:=
  matchcontinue (in_class1,in_class2)
    local
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      type StringOption = Option<String>;
      Absyn_ElementItemList publst,publst2;
      Absyn_ClassPartList parts2,parts;
      Absyn.Class c1;
      String a,file;
      Boolean b,c,d;
      Absyn.Restriction e;
      StringOption cmt;
    case (c1,Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(parts,cmt),file))
      equation 
        publst = get_public_list(parts);
        publst2 = remove_class_in_elementitemlist(publst, c1);
        parts2 = replace_public_list(parts, publst2); then Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(parts2,cmt),file);
    case (_,_)
      equation 
        Print.print_buf("Failed in remove_inner_class\n"); then fail();
  end matchcontinue;
end remove_inner_class;

protected function remove_class_in_elementitemlist "* function: remove_class_in_elementitemlist
 ** This function takes an Element list and a Class and returns a modified element list
 ** where the class definition of the class is removed.
  *"
  input Absyn_ElementItemList in_absyn_elementitemlist;
  input Absyn.Class in_class;
  output Absyn_ElementItemList out_absyn_elementitemlist;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
algorithm 
  out_absyn_elementitemlist:=
  matchcontinue (in_absyn_elementitemlist,in_class)
    local
      type Absyn_ElementSpecOption = Option<Absyn.ElementSpec>;
      Absyn_ElementItemList res,xs;
      Absyn.ElementItem a1,e1;
      Absyn.Class c,c1,c2;
      String name1,name,d,f;
      Boolean a,b,e;
      Integer g;
      Absyn_ElementSpecOption h;
    case ((a1 = Absyn.ANNOTATIONITEM(_)) :: xs,c)
      equation 
        res = remove_class_in_elementitemlist(xs, c); then a1 :: res;
    case ((e1 = Absyn.ELEMENTITEM(Absyn.ELEMENT(_,_,_,_,Absyn.CLASSDEF(_,(c1 = Absyn.CLASS(name1,_,_,_,_,_,_))),_,_,NONE))) :: xs,(c = Absyn.CLASS(name,_,_,_,_,_,_)))
      equation 
        failure(equality(name1 = name));
        res = remove_class_in_elementitemlist(xs, c); then e1 :: res;
    case ((e1 = Absyn.ELEMENTITEM(Absyn.ELEMENT(a,b,c,d,Absyn.CLASSDEF(e,Absyn.CLASS(name1,_,_,_,_,_,_)),f,g,h))) :: xs,(c2 = Absyn.CLASS(name,_,_,_,_,_,_)))
      local Absyn.InnerOuter c;
      equation 
        equality(name1 = name); then xs;
    case (c1 :: xs,c)
      local Absyn.ElementItem c1;
      equation 
        res = remove_class_in_elementitemlist(xs, c); then c1 :: res;
    case ({},c) then {
          Absyn.ELEMENTITEM(
          Absyn.ELEMENT(false,false,Absyn.UNSPECIFIED(),"",Absyn.CLASSDEF(false,c),
          "",0,NONE))}; 
  end matchcontinue;
end remove_class_in_elementitemlist;

protected function replace_inner_class "* function: replace_inner_class 
 ** This function takes two class definitions. The first one is inserted/replaced as a local class inside the second one.
 *"
  input Absyn.Class in_class1;
  input Absyn.Class in_class2;
  output Absyn.Class out_class;
algorithm 
  out_class:=
  matchcontinue (in_class1,in_class2)
    local
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      type StringOption = Option<String>;
      Absyn_ElementItemList publst,publst2;
      Absyn_ClassPartList parts2,parts;
      Absyn.Class c1;
      String a,file;
      Boolean b,c,d;
      Absyn.Restriction e;
      StringOption cmt;
    case (c1,Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(parts,cmt),file))
      equation 
        publst = get_public_list(parts);
        publst2 = replace_class_in_elementitemlist(publst, c1);
        parts2 = replace_public_list(parts, publst2); then Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(parts2,cmt),file);
    case (_,_)
      equation 
        Print.print_buf("Failed in replace_inner_class\n"); then fail();
  end matchcontinue;
end replace_inner_class;

protected function replace_class_in_elementitemlist "* function: replace_class_in_elementitemlist
 ** This function takes an Element list and a Class and returns a modified element list
 ** where the class definition of the class is updated or added.
  *"
  input Absyn_ElementItemList in_absyn_elementitemlist;
  input Absyn.Class in_class;
  output Absyn_ElementItemList out_absyn_elementitemlist;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
algorithm 
  out_absyn_elementitemlist:=
  matchcontinue (in_absyn_elementitemlist,in_class)
    local
      type Absyn_ElementSpecOption = Option<Absyn.ElementSpec>;
      Absyn_ElementItemList res,xs;
      Absyn.ElementItem a1,e1;
      Absyn.Class c,c1,c2;
      String name1,name,d,f;
      Boolean a,b,e;
      Integer g;
      Absyn_ElementSpecOption h;
    case ((a1 = Absyn.ANNOTATIONITEM(_)) :: xs,c)
      equation 
        res = replace_class_in_elementitemlist(xs, c); then a1 :: res;
    case ((e1 = Absyn.ELEMENTITEM(Absyn.ELEMENT(_,_,_,_,Absyn.CLASSDEF(_,(c1 = Absyn.CLASS(name1,_,_,_,_,_,_))),_,_,NONE))) :: xs,(c = Absyn.CLASS(name,_,_,_,_,_,_)))
      equation 
        failure(equality(name1 = name));
        res = replace_class_in_elementitemlist(xs, c); then e1 :: res;
    case ((e1 = Absyn.ELEMENTITEM(Absyn.ELEMENT(a,b,c,d,Absyn.CLASSDEF(e,Absyn.CLASS(name1,_,_,_,_,_,_)),f,g,h))) :: xs,(c2 = Absyn.CLASS(name,_,_,_,_,_,_)))
      local Absyn.InnerOuter c;
      equation 
        equality(name1 = name); then Absyn.ELEMENTITEM(Absyn.ELEMENT(a,b,c,d,Absyn.CLASSDEF(e,c2),f,g,h)) :: xs;
    case (c1 :: xs,c)
      local Absyn.ElementItem c1;
      equation 
        res = replace_class_in_elementitemlist(xs, c); then c1 :: res;
    case ({},c) then {
          Absyn.ELEMENTITEM(
          Absyn.ELEMENT(false,false,Absyn.UNSPECIFIED(),"",Absyn.CLASSDEF(false,c),
          "",0,NONE))}; 
  end matchcontinue;
end replace_class_in_elementitemlist;

protected function get_inner_class "* function: get_inner_class
 ** This function takes a class name and a class and return the inner class definition 
 ** having that name.
*"
  input Absyn.Class in_class;
  input Absyn.Ident in_ident;
  output Absyn.Class out_class;
algorithm 
  out_class:=
  matchcontinue (in_class,in_ident)
    local
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      Absyn_ElementItemList publst;
      Absyn.Class c1,c;
      Absyn_ClassPartList parts;
      String name,str,s1;
    case (Absyn.CLASS(_,_,_,_,_,Absyn.PARTS(parts,_),_),name)
      equation 
        publst = get_public_list(parts);
        c1 = get_class_from_elementitemlist(publst, name); then c1;
    case (c,name)
      equation 
        str = Print.get_string();
        Print.clear_buf();
        Print.print_buf("get_inner_class failed, c:");
        Dump.dump(Absyn.PROGRAM({c},Absyn.TOP()));
        Print.print_buf("name :");
        Print.print_buf(name);
        s1 = Print.get_string();
        Print.clear_buf();
        Print.print_buf(str); then fail();
  end matchcontinue;
end get_inner_class;

protected function replace_public_list "* function: replace_public_list
 ** This function replaces the `ElementItem' list in the `ClassPart' list, and returns the updated
 ** list.
"
  input Absyn_ClassPartList in_absyn_classpartlist;
  input Absyn_ElementItemList in_absyn_elementitemlist;
  output Absyn_ClassPartList out_absyn_classpartlist;
protected 
  type Absyn_ClassPartList = list<Absyn.ClassPart>;
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
algorithm 
  out_absyn_classpartlist:=
  matchcontinue (in_absyn_classpartlist,in_absyn_elementitemlist)
    local
      Absyn_ClassPartList rest_1,rest,ys,xs;
      Absyn.ClassPart lst,x;
      Absyn_ElementItemList newpublst,new;
    case ((lst = Absyn.PUBLIC(_)) :: rest,newpublst)
      equation 
        rest_1 = delete_public_list(rest); then Absyn.PUBLIC(newpublst) :: rest_1;
    case (x :: xs,new)
      equation 
        ys = replace_public_list(xs, new); then x :: ys;
    case ({},_) then {}; 
  end matchcontinue;
end replace_public_list;

protected function replace_protected_list "* function: replace_public_list
 ** This function replaces the `ElementItem' list in the `ClassPart' list, and returns the updated
 ** list.
"
  input Absyn_ClassPartList in_absyn_classpartlist;
  input Absyn_ElementItemList in_absyn_elementitemlist;
  output Absyn_ClassPartList out_absyn_classpartlist;
protected 
  type Absyn_ClassPartList = list<Absyn.ClassPart>;
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
algorithm 
  out_absyn_classpartlist:=
  matchcontinue (in_absyn_classpartlist,in_absyn_elementitemlist)
    local
      Absyn_ClassPartList rest_1,rest,ys,xs;
      Absyn.ClassPart lst,x;
      Absyn_ElementItemList newprotlist,new;
    case ((lst = Absyn.PROTECTED(_)) :: rest,newprotlist)
      equation 
        rest_1 = delete_protected_list(rest); then Absyn.PROTECTED(newprotlist) :: rest_1;
    case (x :: xs,new)
      equation 
        ys = replace_protected_list(xs, new); then x :: ys;
    case ({},_) then {}; 
  end matchcontinue;
end replace_protected_list;

protected function delete_public_list
  input Absyn_ClassPartList in_absyn_classpartlist;
  output Absyn_ClassPartList out_absyn_classpartlist;
protected 
  type Absyn_ClassPartList = list<Absyn.ClassPart>;
algorithm 
  out_absyn_classpartlist:=
  matchcontinue (in_absyn_classpartlist)
    local
      Absyn_ClassPartList res,xs;
      Absyn.ClassPart x;
    case ({}) then {}; 
    case (Absyn.PUBLIC(_) :: xs)
      equation 
        res = delete_public_list(xs); then res;
    case (x :: xs)
      equation 
        res = delete_public_list(xs); then x :: res;
  end matchcontinue;
end delete_public_list;

protected function delete_protected_list
  input Absyn_ClassPartList in_absyn_classpartlist;
  output Absyn_ClassPartList out_absyn_classpartlist;
protected 
  type Absyn_ClassPartList = list<Absyn.ClassPart>;
algorithm 
  out_absyn_classpartlist:=
  matchcontinue (in_absyn_classpartlist)
    local
      Absyn_ClassPartList res,xs;
      Absyn.ClassPart x;
    case ({}) then {}; 
    case (Absyn.PROTECTED(_) :: xs)
      equation 
        res = delete_protected_list(xs); then res;
    case (x :: xs)
      equation 
        res = delete_protected_list(xs); then x :: res;
  end matchcontinue;
end delete_protected_list;

protected function replace_equation_list "* function: replace_equation_list
 ** This function replaces the `EquationItem' list in the `ClassPart' list, and returns the updated
 ** list.
"
  input Absyn_ClassPartList in_absyn_classpartlist;
  input Absyn_EquationItemList in_absyn_equationitemlist;
  output Absyn_ClassPartList out_absyn_classpartlist;
protected 
  type Absyn_ClassPartList = list<Absyn.ClassPart>;
  type Absyn_EquationItemList = list<Absyn.EquationItem>;
algorithm 
  out_absyn_classpartlist:=
  matchcontinue (in_absyn_classpartlist,in_absyn_equationitemlist)
    local
      Absyn.ClassPart lst,x;
      Absyn_ClassPartList rest,ys,xs;
      Absyn_EquationItemList newpublst,new;
    case ((lst = Absyn.EQUATIONS(_)) :: rest,newpublst) then Absyn.EQUATIONS(newpublst) :: rest; 
    case (x :: xs,new)
      equation 
        ys = replace_equation_list(xs, new); then x :: ys;
    case ({},_) then {}; 
  end matchcontinue;
end replace_equation_list;

protected function get_public_list "* function: get_public_list
 ** This function takes a ClassPart List and returns an appended list of all public lists.
 **
"
  input Absyn_ClassPartList in_absyn_classpartlist;
  output Absyn_ElementItemList out_absyn_elementitemlist;
protected 
  type Absyn_ClassPartList = list<Absyn.ClassPart>;
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
algorithm 
  out_absyn_elementitemlist:=
  matchcontinue (in_absyn_classpartlist)
    local
      Absyn_ElementItemList res2,res,res1,ys;
      Absyn_ClassPartList rest,xs;
      Absyn.ClassPart x;
    case ({}) then {}; 
    case Absyn.PUBLIC(res1) :: rest
      equation 
        res2 = get_public_list(rest);
        res = list_append(res1, res2); then res;
    case (x :: xs)
      equation 
        ys = get_public_list(xs); then ys;
  end matchcontinue;
end get_public_list;

protected function get_protected_list "* function: get_protected_list
 ** This function takes a ClassPart List and returns an appended list of all protected lists.
 **
"
  input Absyn_ClassPartList in_absyn_classpartlist;
  output Absyn_ElementItemList out_absyn_elementitemlist;
protected 
  type Absyn_ClassPartList = list<Absyn.ClassPart>;
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
algorithm 
  out_absyn_elementitemlist:=
  matchcontinue (in_absyn_classpartlist)
    local
      Absyn_ElementItemList res2,res,res1,ys;
      Absyn_ClassPartList rest,xs;
      Absyn.ClassPart x;
    case ({}) then {}; 
    case Absyn.PROTECTED(res1) :: rest
      equation 
        res2 = get_protected_list(rest);
        res = list_append(res1, res2); then res;
    case (x :: xs)
      equation 
        ys = get_protected_list(xs); then ys;
  end matchcontinue;
end get_protected_list;

protected function get_equation_list "* function: get_equation_list
 ** This function takes a ClassPart List and returns the first EquationItem list of the class.
 **
"
  input Absyn_ClassPartList in_absyn_classpartlist;
  output Absyn_EquationItemList out_absyn_equationitemlist;
protected 
  type Absyn_ClassPartList = list<Absyn.ClassPart>;
  type Absyn_EquationItemList = list<Absyn.EquationItem>;
algorithm 
  out_absyn_equationitemlist:=
  matchcontinue (in_absyn_classpartlist)
    local
      Absyn_EquationItemList lst,ys;
      Absyn_ClassPartList rest,xs;
      Absyn.ClassPart x;
    case Absyn.EQUATIONS(lst) :: rest then lst; 
    case (x :: xs)
      equation 
        ys = get_equation_list(xs); then ys;
    case (_) then fail(); 
  end matchcontinue;
end get_equation_list;

protected function get_class_from_elementitemlist "* function: get_class_from_elementitemlist
 ** This function takes an ElementItem list and an Ident and returns the class definition
 ** among the element list having that identifier
  *"
  input Absyn_ElementItemList in_absyn_elementitemlist;
  input Absyn.Ident in_ident;
  output Absyn.Class out_class;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
algorithm 
  out_class:=
  matchcontinue (in_absyn_elementitemlist,in_ident)
    local
      Absyn.Class res,c1;
      Absyn_ElementItemList xs;
      String name,name1,name2;
    case (Absyn.ANNOTATIONITEM(_) :: xs,name)
      equation 
        res = get_class_from_elementitemlist(xs, name); then res;
    case (Absyn.ELEMENTITEM(Absyn.ELEMENT(_,_,_,_,Absyn.CLASSDEF(_,(c1 = Absyn.CLASS(name1,_,_,_,_,_,_))),_,_,NONE)) :: xs,name2)
      equation 
        equality(name1 = name2); then c1;
    case (Absyn.ELEMENTITEM(Absyn.ELEMENT(_,_,_,_,Absyn.CLASSDEF(_,(c1 = Absyn.CLASS(name1,_,_,_,_,_,_))),_,_,NONE)) :: xs,name)
      equation 
        failure(equality(name1 = name));
        res = get_class_from_elementitemlist(xs, name); then res;
    case (_ :: xs,name)
      equation 
        res = get_class_from_elementitemlist(xs, name); then res;
    case ({},_) then fail(); 
  end matchcontinue;
end get_class_from_elementitemlist;

protected function class_in_program "* function: class_in_program
 ** This function takes a name and a Program and returns true if the name exists as a top class in the program.
*"
  input String in_string;
  input Absyn.Program in_program;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_string,in_program)
    local
      type Absyn_ClassList = list<Absyn.Class>;
      String str,c1;
      Boolean res;
      Absyn_ClassList p;
      Absyn.Within w;
    case (str,Absyn.PROGRAM({},_)) then false; 
    case (str,Absyn.PROGRAM(Absyn.CLASS(c1,_,_,_,_,_,_) :: p,w))
      equation 
        failure(equality(str = c1));
        res = class_in_program(str, Absyn.PROGRAM(p,w)); then res;
    case (_,_) then true; 
  end matchcontinue;
end class_in_program;

public function get_pathed_class_in_program "* function: get_pathed_class_in_program
 ** This function takes a `Path' and a `Program` and retrieves the class definition referenced 
 ** by the `Path' from the `Program'.
"
  input Absyn.Path in_path;
  input Absyn.Program in_program;
  output Absyn.Class out_class;
algorithm 
  out_class:=
  matchcontinue (in_path,in_program)
    local
      type Absyn_ClassList = list<Absyn.Class>;
      Absyn.Class c1,c1def,res;
      String str;
      Absyn.Program p;
      Absyn_ClassList classes;
      Absyn.Path path,prest;
      Absyn.Within w;
    case (Absyn.IDENT(str),p)
      equation 
        c1 = get_class_in_program(str, p); then c1;
    case ((path = Absyn.QUALIFIED(c1,prest)),(p = Absyn.PROGRAM(_,w)))
      local String c1;
      equation 
        c1def = get_class_in_program(c1, p);
        classes = get_classes_in_class(Absyn.IDENT(c1), p, c1def);
        res = get_pathed_class_in_program(prest, Absyn.PROGRAM(classes,w)); then res;
  end matchcontinue;
end get_pathed_class_in_program;

protected function get_classes_in_class "* function: get_classes_in_class
 ** This function takes a `Class' definition and returns a list of local `Class' definitions of that class.-
 *"
  input Absyn.Path in_path;
  input Absyn.Program in_program;
  input Absyn.Class in_class;
  output Absyn_ClassList out_absyn_classlist;
protected 
  type Absyn_ClassList = list<Absyn.Class>;
algorithm 
  out_absyn_classlist:=
  matchcontinue (in_path,in_program,in_class)
    local
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      Absyn_ClassList res;
      Absyn.Path modelpath,newpath,path;
      Absyn.Program p;
      Absyn_ClassPartList parts;
      Absyn.Class cdef;
    case (modelpath,p,Absyn.CLASS(_,_,_,_,_,Absyn.PARTS(parts,_),_))
      equation 
        res = get_classes_in_parts(parts); then res;
    case (modelpath,p,Absyn.CLASS(_,_,_,_,_,Absyn.DERIVED(path,_,_,_,_),_))
      equation 
        (cdef,newpath) = lookup_classdef(path, modelpath, p);
        res = get_classes_in_class(newpath, p, cdef); then res;
  end matchcontinue;
end get_classes_in_class;

protected function get_classes_in_parts
  input Absyn_ClassPartList in_absyn_classpartlist;
  output Absyn_ClassList out_absyn_classlist;
protected 
  type Absyn_ClassPartList = list<Absyn.ClassPart>;
  type Absyn_ClassList = list<Absyn.Class>;
algorithm 
  out_absyn_classlist:=
  matchcontinue (in_absyn_classpartlist)
    local
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      Absyn_ClassList l1,l2,res;
      Absyn_ElementItemList elts;
      Absyn_ClassPartList rest;
    case {} then {}; 
    case (Absyn.PUBLIC(elts) :: rest)
      equation 
        l1 = get_classes_in_parts(rest);
        l2 = get_classes_in_elts(elts);
        res = list_append(l1, l2); then res;
    case (Absyn.PROTECTED(elts) :: rest)
      equation 
        l1 = get_classes_in_parts(rest);
        l2 = get_classes_in_elts(elts);
        res = list_append(l1, l2); then res;
    case (_ :: rest)
      equation 
        res = get_classes_in_parts(rest); then res;
  end matchcontinue;
end get_classes_in_parts;

protected function get_classes_in_elts
  input Absyn_ElementItemList in_absyn_elementitemlist;
  output Absyn_ClassList out_absyn_classlist;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
  type Absyn_ClassList = list<Absyn.Class>;
algorithm 
  out_absyn_classlist:=
  matchcontinue (in_absyn_elementitemlist)
    local
      Absyn_ClassList res;
      Absyn.Class class_;
      Absyn_ElementItemList rest;
    case {} then {}; 
    case (Absyn.ELEMENTITEM(Absyn.ELEMENT(_,_,_,_,Absyn.CLASSDEF(_,class_),_,_,NONE)) :: rest)
      equation 
        res = get_classes_in_elts(rest); then class_ :: res;
    case (_ :: rest)
      equation 
        res = get_classes_in_elts(rest); then res;
  end matchcontinue;
end get_classes_in_elts;

protected function get_class_in_program "* function: get_class_in_program
 ** This function takes a Path and a Program and returns the class with the name `Path'.
 ** If that class does not exist, the function fail
*"
  input String in_string;
  input Absyn.Program in_program;
  output Absyn.Class out_class;
algorithm 
  out_class:=
  matchcontinue (in_string,in_program)
    local
      type Absyn_ClassList = list<Absyn.Class>;
      String str,c1,c1name;
      Absyn.Class res;
      Absyn_ClassList p;
      Absyn.Within w;
    case (str,Absyn.PROGRAM({},_)) then fail(); 
    case (str,Absyn.PROGRAM(Absyn.CLASS(c1,_,_,_,_,_,_) :: p,w))
      equation 
        failure(equality(str = c1));
        res = get_class_in_program(str, Absyn.PROGRAM(p,w)); then res;
    case (str,Absyn.PROGRAM((c1 = Absyn.CLASS(c1name,_,_,_,_,_,_)) :: p,w))
      local Absyn.Class c1;
      equation 
        equality(str = c1name); then c1;
  end matchcontinue;
end get_class_in_program;

protected function modification_to_absyn
  input Absyn_NamedArgList in_absyn_namedarglist;
  input Absyn_ModificationOption in_absyn_modificationoption;
  output Absyn_ModificationOption out_absyn_modificationoption;
protected 
  type Absyn_NamedArgList = list<Absyn.NamedArg>;
  type Absyn_ModificationOption = Option<Absyn.Modification>;
algorithm 
  out_absyn_modificationoption:=
  matchcontinue (in_absyn_namedarglist,in_absyn_modificationoption)
    local
      Absyn.Modification mod;
      Absyn_NamedArgList nargs;
      Absyn_ModificationOption oldmod;
    case (nargs,oldmod)
      equation 
        SOME(mod) = modification_to_absyn2(nargs); then SOME(mod);
    case (nargs,oldmod) then oldmod; 
  end matchcontinue;
end modification_to_absyn;

protected function modification_to_absyn2
  input Absyn_NamedArgList in_absyn_namedarglist;
  output Absyn_ModificationOption out_absyn_modificationoption;
protected 
  type Absyn_NamedArgList = list<Absyn.NamedArg>;
  type Absyn_ModificationOption = Option<Absyn.Modification>;
algorithm 
  out_absyn_modificationoption:=
  matchcontinue (in_absyn_namedarglist)
    local
      Absyn.Exp exp;
      Absyn_NamedArgList xs;
      Absyn.Modification mod;
      Absyn_ModificationOption res;
      Absyn.NamedArg x;
    case ({}) then NONE; 
    case (Absyn.NAMEDARG("binding",exp) :: xs) then SOME(Absyn.CLASSMOD({},SOME(exp))); 
    case (Absyn.NAMEDARG("modification",Absyn.CODE(Absyn.C_MODIFICATION(mod))) :: xs) then SOME(mod); 
    case (x :: xs)
      equation 
        res = modification_to_absyn2(xs); then res;
  end matchcontinue;
end modification_to_absyn2;

protected function annotation_list_to_absyn_comment
  input Absyn_NamedArgList in_absyn_namedarglist;
  input Absyn_CommentOption in_absyn_commentoption;
  output Absyn_CommentOption out_absyn_commentoption;
protected 
  type Absyn_NamedArgList = list<Absyn.NamedArg>;
  type Absyn_CommentOption = Option<Absyn.Comment>;
algorithm 
  out_absyn_commentoption:=
  matchcontinue (in_absyn_namedarglist,in_absyn_commentoption)
    local
      Absyn.Comment ann;
      Absyn_NamedArgList nargs;
      Absyn_CommentOption oldann;
    case (nargs,oldann)
      equation 
        SOME(ann) = annotation_list_to_absyn_comment2(nargs); then SOME(ann);
    case (nargs,oldann) then oldann; 
  end matchcontinue;
end annotation_list_to_absyn_comment;

protected function annotation_list_to_absyn_comment2
  input Absyn_NamedArgList in_absyn_namedarglist;
  output Absyn_CommentOption out_absyn_commentoption;
protected 
  type Absyn_NamedArgList = list<Absyn.NamedArg>;
  type Absyn_CommentOption = Option<Absyn.Comment>;
algorithm 
  out_absyn_commentoption:=
  matchcontinue (in_absyn_namedarglist)
    local
      type StringOption = Option<String>;
      String strcmt;
      Absyn_NamedArgList nargs;
      Absyn.Annotation annotation_;
    case (nargs)
      equation 
        Absyn.ANNOTATION({}) = annotation_list_to_absyn(nargs);
        SOME(strcmt) = comment_to_absyn(nargs); then SOME(Absyn.COMMENT(NONE,SOME(strcmt)));
    case (nargs)
      equation 
        Absyn.ANNOTATION({}) = annotation_list_to_absyn(nargs);
        NONE = comment_to_absyn(nargs); then NONE;
    case (nargs)
      local StringOption strcmt;
      equation 
        annotation_ = annotation_list_to_absyn(nargs);
        strcmt = comment_to_absyn(nargs); then SOME(Absyn.COMMENT(SOME(annotation_),strcmt));
    case (_) then NONE; 
  end matchcontinue;
end annotation_list_to_absyn_comment2;

protected function comment_to_absyn
  input Absyn_NamedArgList in_absyn_namedarglist;
  output StringOption out_stringoption;
protected 
  type Absyn_NamedArgList = list<Absyn.NamedArg>;
  type StringOption = Option<String>;
algorithm 
  out_stringoption:=
  matchcontinue (in_absyn_namedarglist)
    local
      String str;
      StringOption res;
      Absyn_NamedArgList rest;
    case (Absyn.NAMEDARG("comment",Absyn.STRING(str)) :: _) then SOME(str); 
    case (_ :: rest)
      equation 
        res = comment_to_absyn(rest); then res;
    case (_) then NONE; 
  end matchcontinue;
end comment_to_absyn;

protected function annotation_list_to_absyn "* function: annotation_list_to_absyn
 ** This function takes a list of `NamedArg' and returns an absyn `Annotation'.
 ** for instance {annotation = Placement( ...) } is converted to ANNOTATION(Placement(...)) 
 *"
  input Absyn_NamedArgList in_absyn_namedarglist;
  output Absyn.Annotation out_annotation;
protected 
  type Absyn_NamedArgList = list<Absyn.NamedArg>;
algorithm 
  out_annotation:=
  matchcontinue (in_absyn_namedarglist)
    local
      Absyn.ElementArg eltarg;
      Absyn.Exp e;
      Absyn.Annotation annres;
      Absyn.NamedArg a;
      Absyn_NamedArgList al;
    case ({}) then Absyn.ANNOTATION({}); 
    case (Absyn.NAMEDARG("annotate",e) :: _)
      equation 
        eltarg = record_constructor_to_modification(e); then Absyn.ANNOTATION({eltarg});
    case (a :: al)
      equation 
        annres = annotation_list_to_absyn(al); then annres;
  end matchcontinue;
end annotation_list_to_absyn;

protected function record_constructor_to_modification "* function:record_constructor_to_modification
 ** This function takes a record constructor expression and translates it into a `ElementArg'.
 ** Since modifications must be named, only named arguments are treated in the record constructor.
 *"
  input Absyn.Exp in_exp;
  output Absyn.ElementArg out_elementarg;
algorithm 
  out_elementarg:=
  matchcontinue (in_exp)
    local
      type Absyn_ElementArgList = list<Absyn.ElementArg>;
      type Absyn_NamedArgList = list<Absyn.NamedArg>;
      Absyn_ElementArgList eltarglst;
      Absyn.ElementArg res,emod;
      Absyn.ComponentRef cr;
      Absyn_NamedArgList nargs;
      Absyn.Exp e;
    case (Absyn.CALL(cr,Absyn.FUNCTIONARGS({},nargs)))
      equation 
        eltarglst = Util.list_map(nargs, namedarg_to_modification) " Covers the case annotate=Diagram(x=1,y=2) ";
        res = Absyn.MODIFICATION(false,Absyn.NON_EACH(),cr,
          SOME(Absyn.CLASSMOD(eltarglst,NONE)),NONE); then res;
    case (Absyn.CALL(cr,Absyn.FUNCTIONARGS({(e = Absyn.CALL(_,_))},nargs)))
      equation 
        eltarglst = Util.list_map(nargs, namedarg_to_modification) " Covers the case annotate=Diagram(SOMETHING(x=1,y=2)) ";
        emod = record_constructor_to_modification(e);
        res = Absyn.MODIFICATION(false,Absyn.NON_EACH(),cr,
          SOME(Absyn.CLASSMOD(emod :: eltarglst,NONE)),NONE); then res;
    case (e)
      equation 
        Print.print_buf("record_constructor_to_modification failed, exp=");
        Absyn.print_absyn_exp(e);
        Print.print_buf("\n"); then fail();
  end matchcontinue;
end record_constructor_to_modification;

protected function namedarg_to_modification "* function: namedarg_to_modification
 ** This function takes a `NamedArg' and translates it into a `ElementArg'.
 *"
  input Absyn.NamedArg in_namedarg;
  output Absyn.ElementArg out_elementarg;
algorithm 
  out_elementarg:=
  matchcontinue (in_namedarg)
    local
      type Absyn_ElementArgList = list<Absyn.ElementArg>;
      type Absyn_NamedArgList = list<Absyn.NamedArg>;
      Absyn_ElementArgList elts;
      Absyn.ComponentRef cr_1,cr;
      Absyn.ElementArg res;
      String id;
      Absyn.Exp c,e;
      Absyn_NamedArgList nargs;
    case (Absyn.NAMEDARG(id,(c = Absyn.CALL(cr,Absyn.FUNCTIONARGS({},nargs)))))
      equation 
        Absyn.MODIFICATION(_,_,_,SOME(Absyn.CLASSMOD(elts,_)),NONE) = record_constructor_to_modification(c);
        cr_1 = Absyn.CREF_IDENT(id,{});
        res = Absyn.MODIFICATION(false,Absyn.NON_EACH(),cr_1,
          SOME(Absyn.CLASSMOD(elts,NONE)),NONE); then res;
    case (Absyn.NAMEDARG(id,e))
      equation 
        res = Absyn.MODIFICATION(false,Absyn.NON_EACH(),Absyn.CREF_IDENT(id,{}),
          SOME(Absyn.CLASSMOD({},SOME(e))),NONE); then res;
    case (_)
      equation 
        Print.print_buf("- namedarg_to_modification failed\n"); then fail();
  end matchcontinue;
end namedarg_to_modification;

public function add_instantiated_class "* function: add_instantiated_class
 ** This function adds an instantiated class to the list of instantiated classes.
 ** If the class path already exists, the class is replaced. 
 *"
  input InstantiatedClassList in_instantiatedclasslist;
  input InstantiatedClass in_instantiatedclass;
  output InstantiatedClassList out_instantiatedclasslist;
algorithm 
  out_instantiatedclasslist:=
  matchcontinue (in_instantiatedclasslist,in_instantiatedclass)
    local
      type Env_FrameList = list<Env.Frame>;
      InstantiatedClass cl,newc,x;
      Absyn.Path path,path2;
      DAE_ElementList dae,dae_1;
      Env_FrameList env,env_1;
      InstantiatedClassList xs,res;
    case ({},cl) then {cl}; 
    case (INSTCLASS(path,dae,env) :: xs,(newc = INSTCLASS(path2,dae_1,env_1)))
      equation 
        true = ModUtil.path_equal(path, path2); then newc :: xs;
    case ((x = INSTCLASS(path,_,_)) :: xs,(newc = INSTCLASS(path2,_,_)))
      equation 
        false = ModUtil.path_equal(path, path2);
        res = add_instantiated_class(xs, newc); then x :: res;
  end matchcontinue;
end add_instantiated_class;

public function get_instantiated_class "* function: get_instantiated_class
 ** This function get an instantiated class from the list of instantiated classes.
 *"
  input InstantiatedClassList in_instantiatedclasslist;
  input Absyn.Path in_path;
  output InstantiatedClass out_instantiatedclass;
algorithm 
  out_instantiatedclass:=
  matchcontinue (in_instantiatedclasslist,in_path)
    local
      type Env_FrameList = list<Env.Frame>;
      InstantiatedClass x,res;
      Absyn.Path path,path2;
      DAE_ElementList dae;
      Env_FrameList env;
      InstantiatedClassList xs;
    case ((x = INSTCLASS(path,dae,env)) :: xs,path2)
      equation 
        true = ModUtil.path_equal(path, path2); then x;
    case ((x = INSTCLASS(path,_,_)) :: xs,path2)
      equation 
        false = ModUtil.path_equal(path, path2);
        res = get_instantiated_class(xs, path2); then res;
  end matchcontinue;
end get_instantiated_class;

public function get_contained_class_and_file " program
   package as a program"
  input Absyn.Path in_path;
  input Absyn.Program in_program;
  output Absyn.Program out_program;
  output String out_string;
algorithm 
  (out_program,out_string):=
  matchcontinue (in_path,in_program)
    local
      Absyn.Class cdef;
      String filename;
      Absyn.Program p_1,p_2,p;
      Absyn.Path classname;
    case (classname,p) " class name  program  package as a program  filename "
      equation 
        cdef = get_pathed_class_in_program(classname, p);
        filename = Absyn.class_filename(cdef);
        p_1 = get_surrounding_package(classname, p);
        p_2 = remove_inner_diff_filed_classes(p_1); then (p_2,filename);
  end matchcontinue;
end get_contained_class_and_file;

protected function remove_inner_diff_filed_classes " package as program. "
  input Absyn.Program in_program;
  output Absyn.Program out_program;
algorithm 
  out_program:=
  matchcontinue (in_program)
    local
      type Absyn_ClassList = list<Absyn.Class>;
      Absyn_ClassList classlst_1,classlst;
      Absyn.Within within_;
    case (Absyn.PROGRAM(classlst,within_)) " package as program. "
      equation 
        classlst_1 = Util.list_map(classlst, remove_inner_diff_filed_class); then Absyn.PROGRAM(classlst_1,within_);
  end matchcontinue;
end remove_inner_diff_filed_classes;

protected function remove_inner_diff_filed_class "* function: remove_inner_diff_filed_class
 ** author: PA
 ** 
 ** Helper function to remove_inner_diff_filed_classes, removes all local classes in class
 ** that does not have the same filename as the class iteself.
*"
  input Absyn.Class in_class;
  output Absyn.Class out_class;
algorithm 
  out_class:=
  matchcontinue (in_class)
    local
      type Absyn_ElementItemList = list<Absyn.ElementItem>;
      type Absyn_ClassPartList = list<Absyn.ClassPart>;
      type StringOption = Option<String>;
      Absyn_ElementItemList publst,publst2;
      Absyn_ClassPartList parts2,parts;
      String a,file;
      Boolean b,c,d;
      Absyn.Restriction e;
      StringOption cmt;
    case (Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(parts,cmt),file))
      equation 
        publst = get_public_list(parts);
        publst2 = remove_class_diff_filed_in_elementitemlist(publst, file);
        parts2 = replace_public_list(parts, publst2); then Absyn.CLASS(a,b,c,d,e,Absyn.PARTS(parts2,cmt),file);
  end matchcontinue;
end remove_inner_diff_filed_class;

protected function remove_class_diff_filed_in_elementitemlist " filename "
  input Absyn_ElementItemList in_absyn_elementitemlist;
  input String in_string;
  output Absyn_ElementItemList out_absyn_elementitemlist;
protected 
  type Absyn_ElementItemList = list<Absyn.ElementItem>;
algorithm 
  out_absyn_elementitemlist:=
  matchcontinue (in_absyn_elementitemlist,in_string)
    local
      Absyn_ElementItemList res,xs;
      Absyn.ElementItem a1,e1,c1;
      String c,filename2,filename1,filename;
    case ((a1 = Absyn.ANNOTATIONITEM(_)) :: xs,c) " filename "
      equation 
        res = remove_class_diff_filed_in_elementitemlist(xs, c) " annotations are kept "; then a1 :: res;
    case ((e1 = Absyn.ELEMENTITEM(Absyn.ELEMENT(_,_,_,_,Absyn.CLASSDEF(_,Absyn.CLASS(_,_,_,_,_,_,filename2)),_,_,_))) :: xs,filename1)
      equation 
        failure(equality(filename1 = filename2));
        res = remove_class_diff_filed_in_elementitemlist(xs, filename1); then res;
    case ((e1 = Absyn.ELEMENTITEM(Absyn.ELEMENT(_,_,_,_,Absyn.CLASSDEF(_,Absyn.CLASS(_,_,_,_,_,_,filename2)),_,_,_))) :: xs,filename1)
      equation 
        equality(filename1 = filename2);
        res = remove_class_diff_filed_in_elementitemlist(xs, filename1); then e1 :: res;
    case (c1 :: xs,filename)
      equation 
        res = remove_class_diff_filed_in_elementitemlist(xs, filename); then c1 :: res;
    case ({},filename) then {}; 
  end matchcontinue;
end remove_class_diff_filed_in_elementitemlist;

protected function get_surrounding_package "* function: get_surrounding_package
 ** author: PA
 **
 ** This function investigates the surrounding packages and returns the outermost package
 ** that has the same filename as the class
 *"
  input Absyn.Path in_path;
  input Absyn.Program in_program;
  output Absyn.Program out_program;
algorithm 
  out_program:=
  matchcontinue (in_path,in_program)
    local
      Absyn.Class cdef,pdef;
      String filename1,filename2;
      Absyn.Path ppath,classpath;
      Absyn.Program res,p;
      Absyn.Within within_;
    case (classpath,p)
      equation 
        cdef = get_pathed_class_in_program(classpath, p);
        filename1 = Absyn.class_filename(cdef);
        ppath = Absyn.strip_last(classpath);
        pdef = get_pathed_class_in_program(ppath, p);
        filename2 = Absyn.class_filename(pdef);
        equality(filename1 = filename2);
        res = get_surrounding_package(ppath, p); then res;
    case (classpath,p)
      equation 
        cdef = get_pathed_class_in_program(classpath, p) " No package with same filename ";
        within_ = build_within(classpath); then Absyn.PROGRAM({cdef},within_);
  end matchcontinue;
end get_surrounding_package;
end Interactive;

