package Static "
    Copyright (c) PELAB, Dept. Computer and Information Science, Linkoping University,
    Sweden, in the following denoted PELAB/LIU.

   
    This file is part of OpenModelica, in the following denoted OSM.


    The OSM License, Version 1.1, March 4, 2005.


    1 Preface

    The aim of this license is to lay down the conditions enabling you to use, 
    modify and circulate OSM. However, PELAB/LIU remain the authors of 
    OSM and so retain property rights and the use of all ancillary rights.

    2 Definitions

    OSM is defined as all successive versions of the OSM software and their 
    documentation that have been developed by PELAB/LIU and including
    accepted contributions from other contributors according to this license.

    OSM DERIVED SOFTWARE is defined as all or part of OSM that you have 
    modified and/or translated and/or adapted.


    3 Dual License

    OSM is made available under the OSM licensing scheme, which is a dual
    licensing scheme with two options, a) and b):

    a) OSM OPEN SOURCE LICENSE:

    If you wish to write Open Source software you can use the Open Source version 
    of OSM, released under the OSM license which include GPL as its open source 
    licensing option. If you use the OSM Open Source version you must release your 
    Application using OSM including this Application's source code under the GPL as well.

    This OSM license text, and Copyright (c) PELAB/Linkoping University, must
    be present in your copy of OSM and in OSM DERIVED SOFTWARE.

    You should have received a copy of the GPL - GNU General Public License
    along with OpenModelica; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.


    b) OSM COMMERCIAL LICENSE

    If you are using OSM commercially - that is, for commercial usage or
    for creating proprietary software for sale or use in a commercial setting
    - you must purchase a commercial license of OSM from PELAB/L
  *
 ** file:	 static.rml
 ** module:      Static
 ** description: Static analysis of expressions
 **
 ** RCS: $Id: Static.rml 1798 2005-05-31 06:32:26Z petar $
 ** 
 ** This module does static analysis on expressions.
 ** The analyzed expressions are built using the
 ** constructors in the `Exp' module from expressions defined in 'Absyn'.  
 ** Also, a set of properties of the expressions is calculated during analysis.
 ** Properties of expressions include type information and a boolean indicating if the
 ** expression is constant or not.
 ** If the expression is constant, the 'Ceval' module is used to evaluate the expression
 ** value. A value of an expression is described using the 'Values' module.

 ** The main function in this module is 'eval_exp' which takes an Absyn.Exp and transform it 
 ** into an Exp.Exp, while performing type checking and automatic type conversions, etc.
 ** To determine types of builtin functions and operators, the module also contain an elaboration
 ** handler for functions and operators. This function is called 'elab_builtin_handler'. 
 ** NOTE: These functions should only determine the type and properties of the builtin functions and
 ** operators and not evaluate them. Constant evaluation is performed by the 'Ceval' module.
 ** The module also contain a function for deoverloading of operators, in the 'deoverload' function.
 ** It transforms operators like '+' to its specific form, ADD, ADD_ARR, etc.
 **
 ** Interactive function calls are also given their types by 'elab_exp', which calls 
 ** 'elab_call_interactive'.
 **
 ** Elaboration for functions involve checking the types of the arguments by filling slots of the
 ** argument list with first positional and then named arguments to find a matching function. The 
 ** details of this mechanism can be found in the Modelica specification.
 ** The elaboration also contain function deoverloading which will be added to Modelica in the future.
 *"

public import OpenModelica.Compiler.Absyn;

public import OpenModelica.Compiler.Exp;

public import OpenModelica.Compiler.SCode;

public import OpenModelica.Compiler.Types;

public import OpenModelica.Compiler.Env;

public import OpenModelica.Compiler.Values;

public import OpenModelica.Compiler.Interactive;

public 
type Ident = String;

public 
type Exp_ExpOption = Option<Exp.Exp>;

uniontype Slot
  record SLOT
    Types.FuncArg x1;
    Boolean x2 " An argument to a function ";
    Exp_ExpOption x3 " True if the slot has been filled,   i.e. argument has been given a value";
  end SLOT;
end Slot;

protected import OpenModelica.Compiler.ClassInf;

protected import OpenModelica.Compiler.Dump;

protected import OpenModelica.Compiler.Print;

protected import OpenModelica.Compiler.System;

protected import OpenModelica.Compiler.Lookup;

protected import OpenModelica.Compiler.Debug;

protected import OpenModelica.Compiler.Inst;

protected import OpenModelica.Compiler.Codegen;

protected import OpenModelica.Compiler.ModUtil;

protected import OpenModelica.Compiler.DAE;

protected import OpenModelica.Compiler.Util;

protected import OpenModelica.Compiler.RTOpts;

protected import OpenModelica.Compiler.Parser;

protected import OpenModelica.Compiler.ClassLoader;

protected import OpenModelica.Compiler.Mod;

protected import OpenModelica.Compiler.Prefix;

protected import OpenModelica.Compiler.Ceval;

public function elab_exp_list
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Exp_ExpList out_exp_explist;
  output Types_PropertiesList out_types_propertieslist;
  output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
  type Exp_ExpList = list<Exp.Exp>;
  type Types_PropertiesList = list<Types.Properties>;
algorithm 
  (out_exp_explist,out_types_propertieslist,out_interactive_interactivesymboltableoption):=
  matchcontinue (in_env,in_absyn_explist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st,st_1,st_2;
      Exp.Exp exp;
      Types.Properties p;
      Exp_ExpList exps;
      Types_PropertiesList props;
      Env_FrameList env;
      Absyn.Exp e;
      Absyn_ExpList rest;
    case (_,{},impl,st) then ({},{},st); 
    case (env,e :: rest,impl,st)
      equation 
        (exp,p,st_1) = elab_exp(env, e, impl, st);
        (exps,props,st_2) = elab_exp_list(env, rest, impl, st_1); then (exp :: exps,p :: props,st_2);
  end matchcontinue;
end elab_exp_list;

public function elab_exp_list_list
  input Env.Env in_env;
  input Absyn_ExpListList in_absyn_explistlist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Exp_ExpListList out_exp_explistlist;
  output Types_PropertiesListList out_types_propertieslistlist;
  output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
  type Absyn_ExpListList = list<Absyn_ExpList>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
  type Exp_ExpList = list<Exp.Exp>;
  type Exp_ExpListList = list<Exp_ExpList>;
  type Types_PropertiesList = list<Types.Properties>;
  type Types_PropertiesListList = list<Types_PropertiesList>;
algorithm 
  (out_exp_explistlist,out_types_propertieslistlist,out_interactive_interactivesymboltableoption):=
  matchcontinue (in_env,in_absyn_explistlist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      Boolean impl;
      Interactive_InteractiveSymbolTableOption st,st_1,st_2;
      Exp_ExpList exp;
      Types_PropertiesList p;
      Exp_ExpListList exps;
      Types_PropertiesListList props;
      Env_FrameList env;
      Absyn_ExpList e;
      Absyn_ExpListList rest;
    case (_,{},impl,st) then ({},{},st); 
    case (env,e :: rest,impl,st)
      equation 
        (exp,p,st_1) = elab_exp_list(env, e, impl, st);
        (exps,props,st_2) = elab_exp_list_list(env, rest, impl, st_1); then (exp :: exps,p :: props,st_2);
  end matchcontinue;
end elab_exp_list_list;

protected function ceval_if_constant
  input Exp.Exp in_exp1;
  input Types.Properties in_properties2;
  input Boolean in_boolean3;
  input Boolean in_boolean4;
  input Env.Env in_env5;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_exp1,in_properties2,in_boolean3,in_boolean4,in_env5)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp e,e_1;
      Types.Properties prop;
      Boolean cnst,impl,c,const_;
      Values.Value v;
      TupleTypes_TTypeAbsyn_PathOption vt;
      Env_FrameList env;
    case (e,prop,(cnst = false),_,_) then (e,prop); 
    case (e,prop,(cnst = true),(impl = true),_) then (e,prop); 
    case (e,(prop = Types.PROP(_,c)),(cnst = true),impl,env) "as false"
      equation 
        (v,_) = Ceval.ceval(env, e, impl, NONE, NONE);
        e_1 = value_exp(v);
        vt = value_type(v); then (e_1,Types.PROP(vt,c));
    case (e,(prop = Types.PROP_TUPLE(_,c)),(cnst = true),impl,env) "as false"
      local Types.Const c;
      equation 
        (v,_) = Ceval.ceval(env, e, impl, NONE, NONE);
        e_1 = value_exp(v);
        vt = value_type(v); then (e_1,Types.PROP_TUPLE(vt,c));
    case (e,prop,const_,impl,env) then (e,prop); 
  end matchcontinue;
end ceval_if_constant;

public function elab_exp
  input Env.Env in_env;
  input Absyn.Exp in_exp;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
  output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
protected 
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  (out_exp,out_properties,out_interactive_interactivesymboltableoption):=
  matchcontinue (in_env,in_exp,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleTypes_TTypeAbsyn_PathOptionList = list<TupleTypes_TTypeAbsyn_PathOption>;
      type TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOption = tuple<Exp.Operator,TupleTypes_TTypeAbsyn_PathOptionList,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOptionList = list<TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOption>;
      type Absyn_ExpList = list<Absyn.Exp>;
      type Absyn_NamedArgList = list<Absyn.NamedArg>;
      type Exp_ExpList = list<Exp.Exp>;
      type Types_PropertiesList = list<Types.Properties>;
      type Types_ConstList = list<Types.Const>;
      type Types_PropertiesListList = list<Types_PropertiesList>;
      type TupleTypes_TTypeAbsyn_PathOptionListList = list<TupleTypes_TTypeAbsyn_PathOptionList>;
      type TupleExp_ExpBoolean = tuple<Exp.Exp,Boolean>;
      type TupleExp_ExpBooleanList = list<TupleExp_ExpBoolean>;
      type TupleExp_ExpBooleanListList = list<TupleExp_ExpBooleanList>;
      type Absyn_ExpListList = list<Absyn_ExpList>;
      Integer x,l,dim1,dim2,nmax;
      Boolean impl,c1,c2,c,c_start,c_stop,const_,c_step,a,havereal;
      Interactive_InteractiveSymbolTableOption st,st_1,st_2,st_3;
      Ident id,expstr;
      Exp.Exp exp,e1_1,e2_1,e1_2,e2_2,exp_1,exp_2,e_1,e_2,e3_1,start_1,stop_1,start_2,stop_2,step_1,step_2;
      Types.Properties prop,prop_1,prop1,prop2,prop3;
      Env_FrameList env;
      Absyn.ComponentRef cr,fn;
      TupleTypes_TTypeAbsyn_PathOption t1,t2,rtype,t,start_t,stop_t,step_t,tp;
      TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOptionList ops;
      Exp.Operator op_1;
      Absyn.Exp e1,e2,e,e3,iterexp,start,stop,step;
      Absyn.Operator op;
      Absyn_ExpList args,rest,es;
      Absyn_NamedArgList nargs;
      Exp_ExpList es_1;
      Types_PropertiesList props;
      TupleTypes_TTypeAbsyn_PathOptionList types,tps_2;
      Types_ConstList consts;
      Exp.Type rt,at,tp_1;
      Types_PropertiesListList tps;
      TupleTypes_TTypeAbsyn_PathOptionListList tps_1;
    case (_,Absyn.INTEGER(x),impl,st) then (Exp.ICONST(x),Types.PROP((Types.T_INTEGER({}),NONE),true),st); 
    case (_,Absyn.REAL(x),impl,st)
      local Real x; then (Exp.RCONST(x),Types.PROP((Types.T_REAL({}),NONE),true),st);
    case (_,Absyn.STRING(x),impl,st)
      local Ident x; then (Exp.SCONST(x),Types.PROP((Types.T_STRING({}),NONE),true),st);
    case (_,Absyn.BOOL(x),impl,st)
      local Boolean x; then (Exp.BCONST(x),Types.PROP((Types.T_BOOL({}),NONE),true),st);
    case (_,Absyn.END(),impl,st) then (Exp.END(),Types.PROP((Types.T_INTEGER({}),NONE),true),st); 
    case (env,Absyn.CREF(cr),impl,st)
      equation 
        (exp,prop,_) = elab_cref(env, cr, impl); then (exp,prop,st);
    case (env,(exp = Absyn.BINARY(e1,op,e2)),impl,st) "*  Binary and unary operations "
      local Absyn.Exp exp;
      equation 
        (e1_1,Types.PROP(t1,c1),st_1) = elab_exp(env, e1, impl, st);
        (e2_1,Types.PROP(t2,c2),st_2) = elab_exp(env, e2, impl, st_1);
        c = bool_and(c1, c2);
        ops = operators(op, env, t1, t2);
        (op_1,{e1_2,e2_2},rtype) = deoverload(ops, {(e1_1,t1),(e2_1,t2)}, exp);
        exp_1 = replace_operator_with_fcall(Exp.BINARY(e1_2,op_1,e2_2), c);
        prop = Types.PROP(rtype,c);
        (exp_2,prop_1) = ceval_if_constant(exp_1, prop, c, impl, env); then (exp_2,prop_1,st_2);
    case (env,(exp = Absyn.UNARY(op,e)),impl,st)
      local Absyn.Exp exp;
      equation 
        (e_1,Types.PROP(t,c),st_1) = elab_exp(env, e, impl, st);
        ops = operators(op, env, t, (Types.T_NOTYPE(),NONE));
        (op_1,{e_2},rtype) = deoverload(ops, {(e_1,t)}, exp);
        exp_1 = replace_operator_with_fcall(Exp.UNARY(op_1,e_2), c);
        prop = Types.PROP(rtype,c);
        (exp_2,prop_1) = ceval_if_constant(exp_1, prop, c, impl, env); then (exp_2,prop_1,st_1);
    case (env,(exp = Absyn.LBINARY(e1,op,e2)),impl,st)
      local Absyn.Exp exp;
      equation 
        (e1_1,Types.PROP(t1,c1),st_1) = elab_exp(env, e1, impl, st) " Logical binary expressions ";
        (e2_1,Types.PROP(t2,c2),st_2) = elab_exp(env, e2, impl, st_1);
        c = bool_and(c1, c2);
        ops = operators(op, env, t1, t2);
        (op_1,{e1_2,e2_2},rtype) = deoverload(ops, {(e1_1,t1),(e2_1,t2)}, exp);
        exp_1 = replace_operator_with_fcall(Exp.LBINARY(e1_2,op_1,e2_2), c);
        prop = Types.PROP(rtype,c);
        (exp_2,prop_1) = ceval_if_constant(exp_1, prop, c, impl, env); then (exp_2,prop_1,st_2);
    case (env,(exp = Absyn.LUNARY(op,e)),impl,st)
      local Absyn.Exp exp;
      equation 
        (e_1,Types.PROP(t,c),st_1) = elab_exp(env, e, impl, st) " Logical unary expressions ";
        ops = operators(op, env, t, (Types.T_NOTYPE(),NONE));
        (op_1,{e_2},rtype) = deoverload(ops, {(e_1,t)}, exp);
        exp_1 = replace_operator_with_fcall(Exp.LUNARY(op_1,e_2), c);
        prop = Types.PROP(rtype,c);
        (exp_2,prop_1) = ceval_if_constant(exp_1, prop, c, impl, env); then (exp_2,prop_1,st_1);
    case (env,(exp = Absyn.RELATION(e1,op,e2)),impl,st)
      local Absyn.Exp exp;
      equation 
        (e1_1,Types.PROP(t1,c1),st_1) = elab_exp(env, e1, impl, st) " Relations, e.g. a < b ";
        (e2_1,Types.PROP(t2,c2),st_2) = elab_exp(env, e2, impl, st_1);
        c = bool_and(c1, c2);
        ops = operators(op, env, t1, t2);
        (op_1,{e1_2,e2_2},rtype) = deoverload(ops, {(e1_1,t1),(e2_1,t2)}, exp);
        exp_1 = replace_operator_with_fcall(Exp.RELATION(e1_2,op_1,e2_2), c);
        prop = Types.PROP(rtype,c);
        (exp_2,prop_1) = ceval_if_constant(exp_1, prop, c, impl, env); then (exp_2,prop_1,st_2);
    case (env,Absyn.IFEXP(e1,e2,e3,_),impl,st) "*  Conditional expressions "
      local Exp.Exp e;
      equation 
        (e1_1,prop1,st_1) = elab_exp(env, e1, impl, st) " if expressions ";
        (e2_1,prop2,st_2) = elab_exp(env, e2, impl, st_1);
        (e3_1,prop3,st_3) = elab_exp(env, e3, impl, st_2);
        (e,prop) = elab_ifexp(e1_1, prop1, e2_1, prop2, e3_1, prop3); then (e,prop,st_3);
    case (env,Absyn.CALL(fn,Absyn.FUNCTIONARGS(args,nargs)),impl,st)
      local Exp.Exp e;
      equation 
        Debug.fprintln("sei", "elab_exp CALL...");
        (e,prop,st_1) = elab_call(env, fn, args, nargs, impl, st);
        c = Types.is_prop_all_const(prop);
        (e_1,prop_1) = ceval_if_constant(e, prop, c, impl, env);
        Debug.fprintln("sei", "elab_exp CALL done"); then (e_1,prop_1,st_1);
    case (env,Absyn.TUPLE((e = e1 :: rest)),impl,st) "PR. Get the properties for each expression in the tuple. 
	* Each expression has its own constflag.
	* !!The output from functions does just have one const flag. 
	* Fix this!!
	"
      local
        Exp_ExpList e_1;
        Absyn_ExpList e;
      equation 
        (e_1,props) = elab_tuple(env, e, impl) " Tuple function calls ";
        (types,consts) = split_props(props); then (Exp.TUPLE(e_1),Types.PROP_TUPLE((Types.T_TUPLE(types),NONE),Types.TUPLE_CONST(consts)),st);
    case (env,Absyn.CALL(fn,Absyn.FOR_ITER_FARG(exp,id,iterexp)),impl,st) "*  Array-related expressions  * Elab reduction expressions, including array() constructor *"
      local
        Exp.Exp e;
        Absyn.Exp exp;
      equation 
        (e,prop,st_1) = elab_call_reduction(env, fn, exp, id, iterexp, impl, st); then (e,prop,st_1);
    case (env,Absyn.RANGE(start,NONE,stop),impl,st)
      equation 
        (start_1,Types.PROP(start_t,c_start),st_1) = elab_exp(env, start, impl, st) " Range expressions without step value, e.g. 1:5 ";
        (stop_1,Types.PROP(stop_t,c_stop),st_2) = elab_exp(env, stop, impl, st_1);
        (start_2,NONE,stop_2,rt) = deoverload_range((start_1,start_t), NONE, (stop_1,stop_t));
        const_ = bool_and(c_start, c_stop);
        t = elab_range_type(env, start_2, NONE, stop_2, const_, rt, impl); then (Exp.RANGE(rt,start_1,NONE,stop_1),Types.PROP(t,const_),st_2);
    case (env,Absyn.RANGE(start,SOME(step),stop),impl,st)
      equation 
        (start_1,Types.PROP(start_t,c_start),st_1) = elab_exp(env, start, impl, st) " Range expressions with step value, e.g. 1:0.5:4 ";
        (step_1,Types.PROP(step_t,c_step),st_2) = elab_exp(env, step, impl, st_1);
        (stop_1,Types.PROP(stop_t,c_stop),st_3) = elab_exp(env, stop, impl, st_2);
        (start_2,SOME(step_2),stop_2,rt) = deoverload_range((start_1,start_t), SOME((step_1,step_t)), (stop_1,stop_t));
        c1 = bool_and(c_start, c_step);
        const_ = bool_and(c1, c_stop);
        t = elab_range_type(env, start_2, SOME(step_2), stop_2, const_, rt, impl); then (Exp.RANGE(rt,start_2,SOME(step_2),stop_2),Types.PROP(t,const_),st_3);
    case (env,Absyn.ARRAY(es),impl,st)
      equation 
        (es_1,Types.PROP(t,const_)) = elab_array(env, es, impl, st) " array expressions, e.g. {1,2,3} ";
        l = list_length(es_1);
        at = Types.elab_type(t);
        a = Types.is_array(t); then (Exp.ARRAY(at,a,es_1),Types.PROP((Types.T_ARRAY(Types.DIM(SOME(l)),t),NONE),const_),st);
    case (env,Absyn.MATRIX(es),impl,st)
      local
        TupleExp_ExpBooleanListList es_1;
        Absyn_ExpListList es;
      equation 
        (_,tps,_) = elab_exp_list_list(env, es, impl, st) " matrix expressions, e.g. {1,0;0,1} ";
        tps_1 = Util.list_list_map(tps, Types.get_prop_type);
        tps_2 = Util.list_flatten(tps_1);
        havereal = Types.contain_real(tps_2);
        (es_1,Types.PROP(t,c),dim1,dim2,nmax) = elab_matrix_semi(env, es, impl, st, havereal);
        at = Types.elab_type(t); then (Exp.MATRIX(at,nmax,es_1),Types.PROP(
          (
          Types.T_ARRAY(Types.DIM(SOME(dim1)),
          (Types.T_ARRAY(Types.DIM(SOME(dim2)),t),NONE)),NONE),c),st);
    case (env,Absyn.CODE(c),impl,st)
      local Absyn.Code c;
      equation 
        tp = elab_code_type(env, c) " Code expressions ";
        tp_1 = Types.elab_type(tp); then (Exp.CODE(c,tp_1),Types.PROP(tp,true),st);
    case (_,e,_,_)
      equation 
        Debug.fprint("failtrace", "- elab_exp failed: ");
        expstr = Debug.fcallret("failtrace", Dump.print_exp_str, e, "");
        Debug.fprintln("failtrace", expstr); then fail();
  end matchcontinue;
end elab_exp;

protected function add_for_loop_scope_const "* function: add_for_loop_scope_const 
 ** Creates a new scope on the environment used for loops and adds a loop variable 
 ** which is named by the second argument. The variable is given the value 1 (one) such that
 ** elaboration of expressions of containing the loop variable become constant.
 *"
  input Env.Env env;
  input Ident i;
  input Types.Type typ;
  output Env.Env env_2;
protected 
  type Env_FrameList = list<Env.Frame>;
  Env_FrameList env_1,env_2;
algorithm 
  env_1 := Env.open_scope(env, false, SOME("$for loop scope$")) "encapsulated?";
  env_2 := Env.extend_frame_v(env_1, 
          Types.VAR(i,Types.ATTR(false,SCode.RW(),SCode.PARAM(),Absyn.BIDIR()),
          false,typ,Types.VALBOUND(Values.INTEGER(1))), NONE, false);
end add_for_loop_scope_const;

protected function elab_call_reduction
  input Env.Env in_env1;
  input Absyn.ComponentRef in_componentref2;
  input Absyn.Exp in_exp3;
  input Ident in_ident4;
  input Absyn.Exp in_exp5;
  input Boolean in_boolean6;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption7;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
  output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
protected 
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  (out_exp,out_properties,out_interactive_interactivesymboltableoption):=
  matchcontinue (in_env1,in_componentref2,in_exp3,in_ident4,in_exp5,in_boolean6,in_interactive_interactivesymboltableoption7)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp iterexp_1,exp_1;
      Types.ArrayDim arraydim;
      TupleTypes_TTypeAbsyn_PathOption iterty,expty;
      Boolean iterconst,expconst,const_,impl;
      Env_FrameList env_1,env;
      Interactive_InteractiveSymbolTableOption st;
      Types.Properties prop;
      Absyn.Path fn_1;
      Absyn.ComponentRef fn;
      Absyn.Exp exp,iterexp;
      Ident iter;
    case (env,fn,exp,iter,iterexp,impl,st)
      equation 
        (iterexp_1,Types.PROP((Types.T_ARRAY(arraydim = Types.DIM(_),iterty),_),iterconst),_) = elab_exp(env, iterexp, impl, st);
        env_1 = add_for_loop_scope_const(env, iter, iterty);
        (exp_1,Types.PROP(expty,expconst),st) = elab_exp(env_1, exp, impl, st) "* const so that expr is elaborated to const *";
        const_ = bool_and(expconst, iterconst);
        prop = Types.PROP((Types.T_ARRAY(arraydim,expty),NONE),const_);
        fn_1 = Absyn.cref_to_path(fn); then (Exp.REDUCTION(fn_1,exp_1,iter,iterexp_1),prop,st);
  end matchcontinue;
end elab_call_reduction;

protected function replace_operator_with_fcall "* function: replace_operator_with_fcall
 ** Replaces a userdefined operator expression with a corresponding function call expression.
 ** Other expressions just passes through.
*"
  input Exp.Exp in_exp;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
algorithm 
  out_exp:=
  matchcontinue (in_exp,in_boolean)
    local
      Exp.Exp e1,e2,e;
      Absyn.Path funcname;
      Boolean c;
    case (Exp.BINARY(e1,Exp.USERDEFINED(funcname),e2),c) then Exp.CALL(funcname,{e1,e2},false,c); 
    case (Exp.UNARY(Exp.USERDEFINED(funcname),e1),c) then Exp.CALL(funcname,{e1},false,c); 
    case (Exp.LBINARY(e1,Exp.USERDEFINED(funcname),e2),c) then Exp.CALL(funcname,{e1,e2},false,c); 
    case (Exp.LUNARY(Exp.USERDEFINED(funcname),e1),c) then Exp.CALL(funcname,{e1},false,c); 
    case (Exp.RELATION(e1,Exp.USERDEFINED(funcname),e2),c) then Exp.CALL(funcname,{e1,e2},false,c); 
    case (e,_) then e; 
  end matchcontinue;
end replace_operator_with_fcall;

protected function elab_code_type "* function: elab_code_type
 ** This function will construct the correct type for the given Code expression.
 ** The types are built-in classes of different types. E.g. the class TypeName is the type
 ** of Code expressions corresponding to a type name Code expression. 
*"
  input Env.Env in_env;
  input Absyn.Code in_code;
  output Types.Type out_type;
algorithm 
  out_type:=
  matchcontinue (in_env,in_code)
    local
      type Env_FrameList = list<Env.Frame>;
      Env_FrameList env;
    case (env,Absyn.C_TYPENAME(_)) then ((Types.T_COMPLEX(ClassInf.UNKNOWN("TypeName"),{}),NONE)); 
    case (env,Absyn.C_VARIABLENAME(_)) then ((Types.T_COMPLEX(ClassInf.UNKNOWN("VariableName"),{}),NONE)); 
    case (env,Absyn.C_EQUATIONSECTION(_,_)) then ((Types.T_COMPLEX(ClassInf.UNKNOWN("EquationSection"),{}),
          NONE)); 
    case (env,Absyn.C_ALGORITHMSECTION(_,_)) then ((Types.T_COMPLEX(ClassInf.UNKNOWN("AlgorithmSection"),{}),
          NONE)); 
    case (env,Absyn.C_ELEMENT(_)) then ((Types.T_COMPLEX(ClassInf.UNKNOWN("Element"),{}),NONE)); 
    case (env,Absyn.C_EXPRESSION(_)) then ((Types.T_COMPLEX(ClassInf.UNKNOWN("Expression"),{}),NONE)); 
    case (env,Absyn.C_MODIFICATION(_)) then ((Types.T_COMPLEX(ClassInf.UNKNOWN("Modification"),{}),NONE)); 
  end matchcontinue;
end elab_code_type;

public function elab_graphics_exp "* function elab_graphics_exp
 ** This function is specially designed for elaboration of expressions when
 ** investigating Modelica 2.0 graphical annotations.
 ** These have an array of records representing graphical objects. These elements can
 ** have different types, therefore elab_graphic_exp will allow arrays with elements of 
 ** varying types. 
 *"
  input Env.Env in_env;
  input Absyn.Exp in_exp;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_exp,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleTypes_TTypeAbsyn_PathOptionList = list<TupleTypes_TTypeAbsyn_PathOption>;
      type TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOption = tuple<Exp.Operator,TupleTypes_TTypeAbsyn_PathOptionList,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOptionList = list<TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOption>;
      type Absyn_ExpList = list<Absyn.Exp>;
      type Absyn_NamedArgList = list<Absyn.NamedArg>;
      type Exp_ExpList = list<Exp.Exp>;
      type Types_PropertiesList = list<Types.Properties>;
      type Types_ConstList = list<Types.Const>;
      type Types_PropertiesListList = list<Types_PropertiesList>;
      type TupleTypes_TTypeAbsyn_PathOptionListList = list<TupleTypes_TTypeAbsyn_PathOptionList>;
      type TupleExp_ExpBoolean = tuple<Exp.Exp,Boolean>;
      type TupleExp_ExpBooleanList = list<TupleExp_ExpBoolean>;
      type TupleExp_ExpBooleanListList = list<TupleExp_ExpBooleanList>;
      type Absyn_ExpListList = list<Absyn_ExpList>;
      Integer x,l,dim1,dim2,nmax;
      Boolean impl,c1,c2,c,c_start,c_stop,const_,c_step,a,havereal;
      Ident fnstr;
      Exp.Exp exp,e1_1,e2_1,e1_2,e2_2,e_1,e_2,e3_1,start_1,stop_1,start_2,stop_2,step_1,step_2;
      Types.Properties prop,prop1,prop2,prop3;
      Env_FrameList env;
      Absyn.ComponentRef cr,fn;
      TupleTypes_TTypeAbsyn_PathOption t1,t2,rtype,t,start_t,stop_t,step_t;
      TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOptionList ops;
      Exp.Operator op_1;
      Absyn.Exp e1,e2,e,e3,start,stop,step;
      Absyn.Operator op;
      Absyn_ExpList args,rest,es;
      Absyn_NamedArgList nargs;
      Exp_ExpList es_1;
      Types_PropertiesList props;
      TupleTypes_TTypeAbsyn_PathOptionList types,tps_2;
      Types_ConstList consts;
      Exp.Type rt,at;
      Types_PropertiesListList tps;
      TupleTypes_TTypeAbsyn_PathOptionListList tps_1;
    case (_,Absyn.INTEGER(x),impl) then (Exp.ICONST(x),Types.PROP((Types.T_INTEGER({}),NONE),true)); 
    case (_,Absyn.REAL(x),impl)
      local Real x; then (Exp.RCONST(x),Types.PROP((Types.T_REAL({}),NONE),true));
    case (_,Absyn.STRING(x),impl)
      local Ident x; then (Exp.SCONST(x),Types.PROP((Types.T_STRING({}),NONE),true));
    case (_,Absyn.BOOL(x),impl)
      local Boolean x; then (Exp.BCONST(x),Types.PROP((Types.T_BOOL({}),NONE),true));
    case (env,Absyn.CREF(cr),impl)
      equation 
        (exp,prop,_) = elab_cref(env, cr, impl); then (exp,prop);
    case (env,(exp = Absyn.BINARY(e1,op,e2)),impl) "*  Binary and unary operations "
      local Absyn.Exp exp;
      equation 
        (e1_1,Types.PROP(t1,c1)) = elab_graphics_exp(env, e1, impl);
        (e2_1,Types.PROP(t2,c2)) = elab_graphics_exp(env, e2, impl);
        c = bool_and(c1, c2);
        ops = operators(op, env, t1, t2);
        (op_1,{e1_2,e2_2},rtype) = deoverload(ops, {(e1_1,t1),(e2_1,t2)}, exp); then (Exp.BINARY(e1_2,op_1,e2_2),Types.PROP(rtype,c));
    case (env,(exp = Absyn.UNARY(op,e)),impl)
      local Absyn.Exp exp;
      equation 
        (e_1,Types.PROP(t,c)) = elab_graphics_exp(env, e, impl);
        ops = operators(op, env, t, (Types.T_NOTYPE(),NONE));
        (op_1,{e_2},rtype) = deoverload(ops, {(e_1,t)}, exp); then (Exp.UNARY(op_1,e_2),Types.PROP(rtype,c));
    case (env,(exp = Absyn.LBINARY(e1,op,e2)),impl)
      local Absyn.Exp exp;
      equation 
        (e1_1,Types.PROP(t1,c1)) = elab_graphics_exp(env, e1, impl) " Logical binary expressions ";
        (e2_1,Types.PROP(t2,c2)) = elab_graphics_exp(env, e2, impl);
        c = bool_and(c1, c2);
        ops = operators(op, env, t1, t2);
        (op_1,{e1_2,e2_2},rtype) = deoverload(ops, {(e1_1,t1),(e2_1,t2)}, exp); then (Exp.LBINARY(e1_2,op_1,e2_2),Types.PROP(rtype,c));
    case (env,(exp = Absyn.LUNARY(op,e)),impl)
      local Absyn.Exp exp;
      equation 
        (e_1,Types.PROP(t,c)) = elab_graphics_exp(env, e, impl) " Logical unary expressions ";
        ops = operators(op, env, t, (Types.T_NOTYPE(),NONE));
        (op_1,{e_2},rtype) = deoverload(ops, {(e_1,t)}, exp); then (Exp.LUNARY(op_1,e_2),Types.PROP(rtype,c));
    case (env,(exp = Absyn.RELATION(e1,op,e2)),impl)
      local Absyn.Exp exp;
      equation 
        (e1_1,Types.PROP(t1,c1)) = elab_graphics_exp(env, e1, impl) " Relation expressions ";
        (e2_1,Types.PROP(t2,c2)) = elab_graphics_exp(env, e2, impl);
        c = bool_and(c1, c2);
        ops = operators(op, env, t1, t2);
        (op_1,{e1_2,e2_2},rtype) = deoverload(ops, {(e1_1,t1),(e2_1,t2)}, exp); then (Exp.RELATION(e1_2,op_1,e2_2),Types.PROP(rtype,c));
    case (env,Absyn.IFEXP(e1,e2,e3,_),impl) "*  Conditional expressions "
      local Exp.Exp e;
      equation 
        (e1_1,prop1) = elab_graphics_exp(env, e1, impl);
        (e2_1,prop2) = elab_graphics_exp(env, e2, impl);
        (e3_1,prop3) = elab_graphics_exp(env, e3, impl);
        (e,prop) = elab_ifexp(e1_1, prop1, e2_1, prop2, e3_1, prop3); then (e,prop);
    case (env,Absyn.CALL(fn,Absyn.FUNCTIONARGS(args,nargs)),impl) "*  Function calls  * PA. Only positional arguments are elaborated for now. * TODO: Implement elaboration of named arguments. "
      local Exp.Exp e;
      equation 
        fnstr = Dump.print_component_ref_str(fn);
        (e,prop,_) = elab_call(env, fn, args, nargs, true, NONE); then (e,prop);
    case (env,Absyn.TUPLE((e = e1 :: rest)),impl) "PR. Get the properties for each expression in the tuple. 
	* Each expression has its own constflag.
	* !!The output from functions does just have one const flag. 
	* Fix this!!
	"
      local
        Exp_ExpList e_1;
        Absyn_ExpList e;
      equation 
        (e_1,props) = elab_tuple(env, e, impl);
        (types,consts) = split_props(props); then (Exp.TUPLE(e_1),Types.PROP_TUPLE((Types.T_TUPLE(types),NONE),Types.TUPLE_CONST(consts)));
    case (env,Absyn.RANGE(start,NONE,stop),impl) "*  Array-related expressions "
      equation 
        (start_1,Types.PROP(start_t,c_start)) = elab_graphics_exp(env, start, impl);
        (stop_1,Types.PROP(stop_t,c_stop)) = elab_graphics_exp(env, stop, impl);
        (start_2,NONE,stop_2,rt) = deoverload_range((start_1,start_t), NONE, (stop_1,stop_t));
        const_ = bool_and(c_start, c_stop);
        t = elab_range_type(env, start_2, NONE, stop_2, const_, rt, impl); then (Exp.RANGE(rt,start_1,NONE,stop_1),Types.PROP(t,const_));
    case (env,Absyn.RANGE(start,SOME(step),stop),impl)
      equation 
        (start_1,Types.PROP(start_t,c_start)) = elab_graphics_exp(env, start, impl) "Debug.fprintln(\"setr\", \"elab_graphics_exp_range2\") &";
        (step_1,Types.PROP(step_t,c_step)) = elab_graphics_exp(env, step, impl);
        (stop_1,Types.PROP(stop_t,c_stop)) = elab_graphics_exp(env, stop, impl);
        (start_2,SOME(step_2),stop_2,rt) = deoverload_range((start_1,start_t), SOME((step_1,step_t)), (stop_1,stop_t));
        c1 = bool_and(c_start, c_step);
        const_ = bool_and(c1, c_stop);
        t = elab_range_type(env, start_2, SOME(step_2), stop_2, const_, rt, impl); then (Exp.RANGE(rt,start_2,SOME(step_2),stop_2),Types.PROP(t,const_));
    case (env,Absyn.ARRAY(es),impl) "* FIXME: what is a below? Why true if is array? *"
      equation 
        (es_1,Types.PROP(t,const_)) = elab_graphics_array(env, es, impl) "Debug.fprintln(\"setr\", \"elab_graphics_exp_array\") &";
        l = list_length(es_1);
        at = Types.elab_type(t) "	Ceval.ceval_list(env,es',impl,NONE) => vl &
	Util.list_map(vl, value_exp) => es'' &";
        a = Types.is_array(t); then (Exp.ARRAY(at,a,es_1),Types.PROP((Types.T_ARRAY(Types.DIM(SOME(l)),t),NONE),const_));
    case (env,Absyn.MATRIX(es),impl)
      local
        TupleExp_ExpBooleanListList es_1;
        Absyn_ExpListList es;
      equation 
        (_,tps,_) = elab_exp_list_list(env, es, impl, NONE);
        tps_1 = Util.list_list_map(tps, Types.get_prop_type);
        tps_2 = Util.list_flatten(tps_1);
        havereal = Types.contain_real(tps_2);
        (es_1,Types.PROP(t,c),dim1,dim2,nmax) = elab_matrix_semi(env, es, impl, NONE, havereal);
        at = Types.elab_type(t); then (Exp.MATRIX(at,nmax,es_1),Types.PROP(
          (
          Types.T_ARRAY(Types.DIM(SOME(dim1)),
          (Types.T_ARRAY(Types.DIM(SOME(dim2)),t),NONE)),NONE),c));
    case (_,e,impl)
      local Ident es;
      equation 
        Print.print_error_buf("- elab_graphics_exp failed: ");
        es = Dump.print_exp_str(e);
        Print.print_error_buf(es);
        Print.print_error_buf("\n"); then fail();
  end matchcontinue;
end elab_graphics_exp;

protected function deoverload_range
  input TupleExp_ExpTypes_Type in_tupleexp_exptypes_type1;
  input TupleExp_ExpTypes_TypeOption in_tupleexp_exptypes_typeoption2;
  input TupleExp_ExpTypes_Type in_tupleexp_exptypes_type3;
  output Exp.Exp out_exp1;
  output Exp_ExpOption out_exp_expoption2;
  output Exp.Exp out_exp3;
  output Exp.Type out_type4;
protected 
  type TupleExp_ExpTypes_Type = tuple<Exp.Exp,Types.Type>;
  type TupleExp_ExpTypes_TypeOption = Option<TupleExp_ExpTypes_Type>;
algorithm 
  (out_exp1,out_exp_expoption2,out_exp3,out_type4):=
  matchcontinue (in_tupleexp_exptypes_type1,in_tupleexp_exptypes_typeoption2,in_tupleexp_exptypes_type3)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      Exp.Exp e1,e3,e2,e1_1,e3_1,e2_1;
      TupleTypes_TTypeAbsyn_PathOption t1,t3,t2;
    case ((e1,(Types.T_INTEGER(_),_)),NONE,(e3,(Types.T_INTEGER(_),_))) then (e1,NONE,e3,Exp.INT()); 
    case ((e1,(Types.T_INTEGER(_),_)),SOME((e2,(Types.T_INTEGER(_),_))),(e3,(Types.T_INTEGER(_),_))) then (e1,SOME(e2),e3,Exp.INT()); 
    case ((e1,t1),NONE,(e3,t3))
      equation 
        ({e1_1,e3_1},_) = elab_arglist({(Types.T_REAL({}),NONE),(Types.T_REAL({}),NONE)}, 
          {(e1,t1),(e3,t3)}); then (e1_1,NONE,e3_1,Exp.REAL());
    case ((e1,t1),SOME((e2,t2)),(e3,t3))
      equation 
        ({e1_1,e2_1,e3_1},_) = elab_arglist(
          {(Types.T_REAL({}),NONE),(Types.T_REAL({}),NONE),
          (Types.T_REAL({}),NONE)}, {(e1,t1),(e2,t2),(e3,t3)}); then (e1_1,SOME(e2_1),e3_1,Exp.REAL());
  end matchcontinue;
end deoverload_range;

protected function elab_range_type "impl"
  input Env.Env in_env1;
  input Exp.Exp in_exp2;
  input Exp_ExpOption in_exp_expoption3;
  input Exp.Exp in_exp4;
  input Boolean in_boolean5;
  input Exp.Type in_type6;
  input Boolean in_boolean7;
  output Types.Type out_type;
algorithm 
  out_type:=
  matchcontinue (in_env1,in_exp2,in_exp_expoption3,in_exp4,in_boolean5,in_type6,in_boolean7)
    local
      type Env_FrameList = list<Env.Frame>;
      type IdentOption = Option<Ident>;
      Integer startv,stopv,n,n_1,stepv,n_2,n_3,n_4;
      Env_FrameList env;
      Exp.Exp start,stop,step;
      Boolean const_,impl;
      Ident s1,s2,s3,s4,s5,s6,str;
      IdentOption s2opt;
      Exp.Type expty;
    case (env,start,NONE,stop,(const_ = true),_,impl) "const impl as false"
      equation 
        (Values.INTEGER(startv),_) = Ceval.ceval(env, start, impl, NONE, NONE);
        (Values.INTEGER(stopv),_) = Ceval.ceval(env, stop, impl, NONE, NONE);
        n = stopv - startv;
        n_1 = n + 1; then ((
          Types.T_ARRAY(Types.DIM(SOME(n_1)),(Types.T_INTEGER({}),NONE)),NONE));
    case (env,start,SOME(step),stop,(const_ = true),_,impl) "as false"
      equation 
        (Values.INTEGER(startv),_) = Ceval.ceval(env, start, impl, NONE, NONE);
        (Values.INTEGER(stepv),_) = Ceval.ceval(env, step, impl, NONE, NONE);
        (Values.INTEGER(stopv),_) = Ceval.ceval(env, stop, impl, NONE, NONE);
        n = stopv - startv;
        n_1 = n/stepv;
        n_2 = n_1 + 1; then ((
          Types.T_ARRAY(Types.DIM(SOME(n_2)),(Types.T_INTEGER({}),NONE)),NONE));
    case (env,start,NONE,stop,(const_ = true),_,impl) "as false"
      local Real startv,stopv,n,n_2;
      equation 
        (Values.REAL(startv),_) = Ceval.ceval(env, start, impl, NONE, NONE);
        (Values.REAL(stopv),_) = Ceval.ceval(env, stop, impl, NONE, NONE);
        n = stopv -. startv;
        n_2 = real_floor(n);
        n_3 = real_int(n_2);
        n_1 = n_3 + 1; then ((
          Types.T_ARRAY(Types.DIM(SOME(n_1)),(Types.T_REAL({}),NONE)),NONE));
    case (env,start,SOME(step),stop,(const_ = true),_,impl) "as false"
      local Real startv,stepv,stopv,n,n_1,n_3;
      equation 
        (Values.REAL(startv),_) = Ceval.ceval(env, start, impl, NONE, NONE);
        (Values.REAL(stepv),_) = Ceval.ceval(env, step, impl, NONE, NONE);
        (Values.REAL(stopv),_) = Ceval.ceval(env, stop, impl, NONE, NONE);
        n = stopv -. startv;
        n_1 = n/.stepv;
        n_3 = real_floor(n_1);
        n_4 = real_int(n_3);
        n_2 = n_4 + 1; then ((
          Types.T_ARRAY(Types.DIM(SOME(n_2)),(Types.T_REAL({}),NONE)),NONE));
    case (_,_,_,_,(const_ = false),Exp.INT(),(impl = true)) then ((Types.T_ARRAY(Types.DIM(NONE),(Types.T_INTEGER({}),NONE)),
          NONE)); 
    case (_,_,_,_,(const_ = false),Exp.REAL(),(impl = true)) then ((Types.T_ARRAY(Types.DIM(NONE),(Types.T_REAL({}),NONE)),NONE)); 
    case (env,start,step,stop,const_,expty,impl)
      local Exp_ExpOption step;
      equation 
        Debug.fprint("failtrace", "- elab_range_type failed: ");
        s1 = Exp.print_exp_str(start);
        s2opt = Util.apply_option(step, Exp.print_exp_str);
        s2 = Util.flatten_option(s2opt, "none");
        s3 = Exp.print_exp_str(stop);
        s4 = Util.if_(const_, "const", "non-const");
        s5 = Util.if_(impl, "impl", "expl");
        s6 = Exp.type_string(expty);
        str = Util.string_append_list({"(",s1,":",s2,":",s3,") ",s4," ",s5," ",s6});
        Debug.fprintln("failtrace", str); then fail();
  end matchcontinue;
end elab_range_type;

protected function elab_tuple
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp_ExpList out_exp_explist;
  output Types_PropertiesList out_types_propertieslist;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
  type Exp_ExpList = list<Exp.Exp>;
  type Types_PropertiesList = list<Types.Properties>;
algorithm 
  (out_exp_explist,out_types_propertieslist):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp e_1;
      Types.Properties p;
      Exp_ExpList exps_1;
      Types_PropertiesList props;
      Env_FrameList env;
      Absyn.Exp e;
      Absyn_ExpList exps;
      Boolean impl;
    case (env,e :: exps,impl) "impl"
      equation 
        (e_1,p,_) = elab_exp(env, e, impl, NONE) "Debug.print \"\nEntered elab_tuple.\" &";
        (exps_1,props) = elab_tuple(env, exps, impl) "	Debug.print \"\nElaborated expression.\" &"; then (e_1 :: exps_1,p :: props);
    case (env,{},impl) then ({},{});  "PR.  rule	elab_exp (env,e) => (e',p) 
	-----------------------------
	elab_tuple(env,e::{}) => (e', p) 
"
  end matchcontinue;
end elab_tuple;

protected function elab_array
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Exp_ExpList out_exp_explist;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
  type Exp_ExpList = list<Exp.Exp>;
algorithm 
  (out_exp_explist,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type IdentList = list<Ident>;
      Exp.Exp e_1;
      Types.Properties prop;
      Env_FrameList env;
      Absyn.Exp e;
      Boolean impl,c1,c2,c;
      Interactive_InteractiveSymbolTableOption st;
      TupleTypes_TTypeAbsyn_PathOption t1,t2;
      Exp_ExpList es_1;
      Absyn_ExpList es;
      Ident str;
      IdentList strs;
    case (env,{e},impl,st) "impl"
      equation 
        (e_1,prop,_) = elab_exp(env, e, impl, st); then ({e_1},prop);
    case (env,e :: es,impl,st)
      equation 
        (e_1,Types.PROP(t1,c1),_) = elab_exp(env, e, impl, st);
        (es_1,Types.PROP(t2,c2)) = elab_array(env, es, impl, st);
        equality(t1 = t2);
        c = bool_and(c1, c2); then (e_1 :: es_1,Types.PROP(t1,c));
    case (env,e :: es,impl,st)
      equation 
        (e_1,Types.PROP(t1,c1),_) = elab_exp(env, e, impl, st);
        (es_1,Types.PROP(t2,c2)) = elab_array(env, es, impl, st);
        failure(equality(t1 = t2));
        Print.print_error_buf("# Incompatible types in array expression\n");
        Print.print_error_buf("  ");
        str = Dump.print_exp_str(e);
        Print.print_error_buf(str);
        Print.print_error_buf(" is of type ");
        str = Types.unparse_type(t1);
        Print.print_error_buf(str);
        Print.print_error_buf("\n  while the elements [");
        strs = Util.list_map(es, Dump.print_exp_str);
        str = Util.string_delimit_list(strs, ",");
        Print.print_error_buf(str);
        Print.print_error_buf("] are of type ");
        str = Types.unparse_type(t2);
        Print.print_error_buf(str);
        Print.print_error_buf("\n"); then fail();
    case (_,_,_,_)
      equation 
        Print.print_error_buf("- elab_array failed\n"); then fail();
  end matchcontinue;
end elab_array;

protected function elab_graphics_array
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp_ExpList out_exp_explist;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
  type Exp_ExpList = list<Exp.Exp>;
algorithm 
  (out_exp_explist,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      Exp.Exp e_1;
      Types.Properties prop;
      Env_FrameList env;
      Absyn.Exp e;
      Boolean impl,c1,c2,c;
      TupleTypes_TTypeAbsyn_PathOption t1,t2;
      Exp_ExpList es_1;
      Absyn_ExpList es;
    case (env,{e},impl) "impl"
      equation 
        (e_1,prop) = elab_graphics_exp(env, e, impl); then ({e_1},prop);
    case (env,e :: es,impl)
      equation 
        (e_1,Types.PROP(t1,c1)) = elab_graphics_exp(env, e, impl);
        (es_1,Types.PROP(t2,c2)) = elab_graphics_array(env, es, impl);
        c = bool_and(c1, c2); then (e_1 :: es_1,Types.PROP(t1,c));
    case (_,_,impl)
      equation 
        Print.print_error_buf("- elab_graphics_array failed\n"); then fail();
  end matchcontinue;
end elab_graphics_array;

protected function elab_matrix_comma "have real"
  input Env.Env in_env1;
  input Absyn_ExpList in_absyn_explist2;
  input Boolean in_boolean3;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption4;
  input Boolean in_boolean5;
  output TupleExp_ExpBooleanList out_tupleexp_expbooleanlist1;
  output Types.Properties out_properties2;
  output Integer out_integer3;
  output Integer out_integer4;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
  type TupleExp_ExpBoolean = tuple<Exp.Exp,Boolean>;
  type TupleExp_ExpBooleanList = list<TupleExp_ExpBoolean>;
algorithm 
  (out_tupleexp_expbooleanlist1,out_properties2,out_integer3,out_integer4):=
  matchcontinue (in_env1,in_absyn_explist2,in_boolean3,in_interactive_interactivesymboltableoption4,in_boolean5)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp el_1;
      TupleTypes_TTypeAbsyn_PathOption t,t1,t2;
      Boolean c,array,impl,havereal,c1,c2;
      Integer ndims,maxn,maxn2,dim,maxn1;
      Env_FrameList env;
      Absyn.Exp el;
      Interactive_InteractiveSymbolTableOption st;
      TupleExp_ExpBooleanList els_1;
      Types.Properties props;
      Absyn_ExpList els;
    case (env,{el},impl,st,havereal)
      equation 
        (el_1,Types.PROP(t,c),_) = elab_exp(env, el, impl, st);
        ndims = Types.ndims(t);
        maxn = int_max(2, ndims);
        array = (ndims > 0); then ({(el_1,array)},Types.PROP(t,c),1,maxn);
    case (env,el :: els,impl,st,havereal)
      equation 
        (el_1,Types.PROP(t1,c1),_) = elab_exp(env, el, impl, st);
        (els_1,Types.PROP(t2,c2),_,maxn2) = elab_matrix_comma(env, els, impl, st, havereal);
        props = Types.match_with_promote(Types.PROP(t1,c1), Types.PROP(t2,c2), havereal);
        dim = list_length(el :: els);
        maxn1 = Types.ndims(t1);
        maxn = int_max(maxn1, maxn2);
        array = (maxn1 > 0); then ((el_1,array) :: els_1,props,dim,maxn);
    case (_,_,_,_,_) then fail(); 
  end matchcontinue;
end elab_matrix_comma;

protected function elab_matrix_semi "contain real"
  input Env.Env in_env1;
  input Absyn_ExpListList in_absyn_explistlist2;
  input Boolean in_boolean3;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption4;
  input Boolean in_boolean5;
  output TupleExp_ExpBooleanListList out_tupleexp_expbooleanlistlist1;
  output Types.Properties out_properties2;
  output Integer out_integer3;
  output Integer out_integer4;
  output Integer out_integer5;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
  type Absyn_ExpListList = list<Absyn_ExpList>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
  type TupleExp_ExpBoolean = tuple<Exp.Exp,Boolean>;
  type TupleExp_ExpBooleanList = list<TupleExp_ExpBoolean>;
  type TupleExp_ExpBooleanListList = list<TupleExp_ExpBooleanList>;
algorithm 
  (out_tupleexp_expbooleanlistlist1,out_properties2,out_integer3,out_integer4,out_integer5):=
  matchcontinue (in_env1,in_absyn_explistlist2,in_boolean3,in_interactive_interactivesymboltableoption4,in_boolean5)
    local
      type Env_FrameList = list<Env.Frame>;
      TupleExp_ExpBooleanList el_1;
      Types.Properties props,props1,props2;
      Integer dim,maxn,dim1,maxn1,dim2,maxn2;
      Env_FrameList env;
      Absyn_ExpList el;
      Boolean impl,havereal;
      Interactive_InteractiveSymbolTableOption st;
      TupleExp_ExpBooleanListList els_1;
      Absyn_ExpListList els;
    case (env,{el},impl,st,havereal)
      equation 
        (el_1,props,dim,maxn) = elab_matrix_comma(env, el, impl, st, havereal); then ({el_1},props,1,dim,maxn);
    case (env,el :: els,impl,st,havereal)
      equation 
        dim = list_length(el);
        (el_1,props1,dim1,maxn1) = elab_matrix_comma(env, el, impl, st, havereal);
        (els_1,props2,_,dim2,maxn2) = elab_matrix_semi(env, els, impl, st, havereal);
        equality(dim1 = dim2);
        props = Types.match_with_promote(props1, props2, havereal);
        maxn = int_max(maxn1, maxn2); then (el_1 :: els_1,props,dim,dim1,maxn);
    case (_,_,_,_,_) then fail(); 
  end matchcontinue;
end elab_matrix_semi;

protected function elab_builtin_size "* function: elab_builtin_size
 ** This function elaborates the size operator.
 ** Input is the list of arguments to size as Absyn.Exp expressions and the 
 ** environment, Env.Env.
 *"
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp dimp,arraycrefe,exp;
      Boolean c1,c2,c,impl;
      TupleTypes_TTypeAbsyn_PathOption arrtp;
      Env_FrameList env;
      Absyn.Exp arraycr,dim;
      Absyn_ExpList expl;
    case (env,{arraycr,dim},impl) "impl  FIXME: Why false? "
      equation 
        (dimp,Types.PROP(_,c1),_) = elab_exp(env, dim, impl, NONE) "* size(A,x) that returns size of x:th dimension ";
        (arraycrefe,Types.PROP(arrtp,_),_) = elab_exp(env, arraycr, impl, NONE);
        c2 = Types.dimensions_known(arrtp);
        c = bool_and(c1, c2);
        exp = Exp.SIZE(arraycrefe,SOME(dimp)); then (exp,Types.PROP((Types.T_INTEGER({}),NONE),c));
    case (env,{arraycr},impl)
      equation 
        (arraycrefe,Types.PROP(arrtp,_),_) = elab_exp(env, arraycr, impl, NONE) " size(A) ";
        c = Types.dimensions_known(arrtp);
        exp = Exp.SIZE(arraycrefe,NONE); then (exp,Types.PROP(
          (
          Types.T_ARRAY(Types.DIM(SOME(1)),(Types.T_INTEGER({}),NONE)),NONE),c));
    case (env,expl,impl)
      equation 
        Debug.fprint("failtrace", "- elab_builtin_size failed\n"); then fail();
  end matchcontinue;
end elab_builtin_size;

protected function elab_builtin_fill "* function: elab_builtin_fill
 ** This function elaborates the builtin operator fill.
 ** The input is the arguments to fill as Absyn.Exp expressions and the environment Env.Env
 *"
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Exp_ExpList = list<Exp.Exp>;
      type Types_PropertiesList = list<Types.Properties>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Values_ValueList = list<Values.Value>;
      type Env_FrameList = list<Env.Frame>;
      type IdentList = list<Ident>;
      Exp.Exp s_1,exp;
      Types.Properties prop;
      Exp_ExpList dims_1;
      Types_PropertiesList dimprops;
      TupleTypes_TTypeAbsyn_PathOption sty;
      Values_ValueList dimvals;
      Env_FrameList env;
      Absyn.Exp s;
      Absyn_ExpList dims;
      Boolean impl;
      Ident implstr,expstr,str;
      IdentList expstrs;
    case (env,s :: dims,impl) "impl"
      equation 
        (s_1,prop,_) = elab_exp(env, s, impl, NONE);
        (dims_1,dimprops,_) = elab_exp_list(env, dims, impl, NONE);
        sty = Types.get_prop_type(prop);
        dimvals = Ceval.ceval_list(env, dims_1, impl, NONE);
        (exp,prop) = elab_builtin_fill_2(env, s_1, sty, dimvals); then (exp,prop);
    case (env,dims,impl)
      equation 
        Debug.fprint("failtrace", 
          "- elab_builtin_fill: Couldn't elaborate fill(): ");
        implstr = Util.bool_string(impl);
        expstrs = Util.list_map(dims, Dump.print_exp_str);
        expstr = Util.string_delimit_list(expstrs, ", ");
        str = Util.string_append_list({expstr," impl=",implstr});
        Debug.fprintln("failtrace", str); then fail();
  end matchcontinue;
end elab_builtin_fill;

protected function elab_builtin_fill_2
  input Env.Env in_env;
  input Exp.Exp in_exp;
  input Types.Type in_type;
  input Values_ValueList in_values_valuelist;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Values_ValueList = list<Values.Value>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_exp,in_type,in_values_valuelist)
    local
      type Exp_ExpList = list<Exp.Exp>;
      type Env_FrameList = list<Env.Frame>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      Exp_ExpList arraylist;
      Ident dimension;
      Exp.Type at;
      Boolean a,con;
      Env_FrameList env;
      Exp.Exp s,exp;
      TupleTypes_TTypeAbsyn_PathOption sty,ty;
      Integer v;
      Values_ValueList rest;
    case (env,s,sty,{Values.INTEGER(v)})
      equation 
        arraylist = build_exp_list(s, v);
        dimension = int_string(v);
        at = Types.elab_type(sty);
        a = Types.is_array(sty); then (Exp.ARRAY(at,a,arraylist),Types.PROP((Types.T_ARRAY(Types.DIM(SOME(v)),sty),NONE),true));
    case (env,s,sty,Values.INTEGER(v) :: rest)
      equation 
        (exp,Types.PROP(ty,con)) = elab_builtin_fill_2(env, s, sty, rest);
        arraylist = build_exp_list(exp, v);
        dimension = int_string(v);
        at = Types.elab_type(ty);
        a = Types.is_array(ty); then (Exp.ARRAY(at,a,arraylist),Types.PROP((Types.T_ARRAY(Types.DIM(SOME(v)),ty),NONE),true));
    case (_,_,_,_)
      equation 
        Print.print_error_buf("#-- elab_builtin_fill_2: Failed to elaborate fill()\n"); then fail();
  end matchcontinue;
end elab_builtin_fill_2;

protected function elab_builtin_transpose
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp matexp_1,exp,exp_1;
      TupleTypes_TTypeAbsyn_PathOption matty,eltp,newtp;
      Types.Properties prop;
      Env_FrameList env;
      Absyn.Exp matexp;
      Boolean impl;
      Types.ArrayDim d1,d2;
    case (env,{matexp},impl) "impl"
      equation 
        (matexp_1,Types.PROP(matty,true),_) = elab_exp(env, matexp, impl, NONE);
        (exp,prop) = elab_builtin_transpose_2(env, matexp_1, matty); then (exp,prop);
    case (env,{matexp},impl)
      equation 
        (exp_1,Types.PROP((Types.T_ARRAY(d1,(Types.T_ARRAY(d2,eltp),_)),_),_),_) = elab_exp(env, matexp, impl, NONE);
        newtp = (Types.T_ARRAY(d2,(Types.T_ARRAY(d1,eltp),NONE)),NONE);
        exp = Exp.CALL(Absyn.IDENT("transpose"),{exp_1},false,true);
        prop = Types.PROP(newtp,false); then (exp,prop);
  end matchcontinue;
end elab_builtin_transpose;

protected function elab_builtin_transpose_2
  input Env.Env in_env;
  input Exp.Exp in_exp;
  input Types.Type in_type;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_exp,in_type)
    local
      type Env_FrameList = list<Env.Frame>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      Env_FrameList env;
      Exp.Exp exp;
      TupleTypes_TTypeAbsyn_PathOption tp;
    case (env,exp,tp)
      equation 
        print("- elab_builtin_transpose_2 not implemented yet\n");
        Print.print_error_buf("- elab_builtin_transpose_2 not implemented yet\n"); then fail();
  end matchcontinue;
end elab_builtin_transpose_2;

protected function build_exp_list "* function: build_exp_list
 ** Helper function to e.g. elab_builtin_fill_2. Creates n copies of the same 
 ** expression given as input.
 *"
  input Exp.Exp in_exp;
  input Integer in_integer;
  output Exp_ExpList out_exp_explist;
protected 
  type Exp_ExpList = list<Exp.Exp>;
algorithm 
  out_exp_explist:=
  matchcontinue (in_exp,in_integer)
    local
      Exp.Exp e;
      Integer c_1,c;
      Exp_ExpList rest;
    case (e,0) then {}; 
    case (e,1) then {e}; 
    case (e,c)
      equation 
        c_1 = c - 1;
        rest = build_exp_list(e, c_1); then e :: rest;
  end matchcontinue;
end build_exp_list;

protected function elab_builtin_sum
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Values_ValueList = list<Values.Value>;
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp exp_1,exp_2;
      Types.ArrayDim dim;
      TupleTypes_TTypeAbsyn_PathOption tp;
      Boolean c,impl;
      Values_ValueList vals;
      Values.Value value;
      Env_FrameList env;
      Absyn.Exp arrexp;
    case (env,{arrexp},impl) "impl"
      equation 
        (exp_1,Types.PROP((Types.T_ARRAY(dim,tp),_),c = true),_) = elab_exp(env, arrexp, impl, NONE) " For constant expressions ";
        (Values.ARRAY(vals),_) = Ceval.ceval(env, exp_1, impl, NONE, NONE);
        value = Values.sum_arrayelt(vals);
        exp_2 = value_exp(value); then (exp_2,Types.PROP(tp,c));
    case (env,{arrexp},impl)
      equation 
        (exp_1,Types.PROP((Types.T_ARRAY(dim,tp),_),c = false),_) = elab_exp(env, arrexp, impl, NONE) " For non-constant expressions ";
        exp_2 = Exp.CALL(Absyn.IDENT("sum"),{exp_1},false,true); then (exp_2,Types.PROP(tp,c));
  end matchcontinue;
end elab_builtin_sum;

protected function elab_builtin_pre
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp exp_1,exp_2;
      TupleTypes_TTypeAbsyn_PathOption tp;
      Boolean c,impl;
      Env_FrameList env;
      Absyn.Exp exp;
    case (env,{exp},impl) "impl"
      equation 
        (exp_1,Types.PROP(tp,c),_) = elab_exp(env, exp, impl, NONE);
        true = Types.basic_type(tp);
        exp_2 = Exp.CALL(Absyn.IDENT("pre"),{exp_1},false,true); then (exp_2,Types.PROP(tp,c));
    case (env,{exp},impl)
      local Exp.Exp exp;
      equation 
        (exp,Types.PROP(tp,c),_) = elab_exp(env, exp, impl, NONE);
        false = Types.basic_type(tp);
        Print.print_error_buf("#Error, operand of pre must be of built-in type.\n"); then fail();
    case (env,_,_)
      equation 
        Print.print_error_buf(
          "#Error, wrong type or wrong number of arguments to pre operator.\n"); then fail();
  end matchcontinue;
end elab_builtin_pre;

protected function elab_builtin_initial
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      Env_FrameList env;
      Boolean impl;
    case (env,{},impl) then (Exp.CALL(Absyn.IDENT("initial"),{},false,true),Types.PROP((Types.T_BOOL({}),NONE),false)); 
    case (env,_,_)
      equation 
        Print.print_error_buf("#Error, initial takes no arguments.\n"); then fail();
  end matchcontinue;
end elab_builtin_initial;

protected function elab_builtin_terminal
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      Env_FrameList env;
      Boolean impl;
    case (env,{},impl) then (Exp.CALL(Absyn.IDENT("terminal"),{},false,true),Types.PROP((Types.T_BOOL({}),NONE),false)); 
    case (env,_,impl)
      equation 
        Print.print_error_buf("#Error, terminal takes no arguments.\n"); then (Exp.CALL(Absyn.IDENT("terminal"),{},false,true),Types.PROP((Types.T_BOOL({}),NONE),false));
    case (env,_,impl)
      equation 
        Print.print_error_buf("#Error, terminal takes no arguments.\n"); then fail();
  end matchcontinue;
end elab_builtin_terminal;

protected function elab_builtin_array
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Exp_ExpList = list<Exp.Exp>;
      type Types_PropertiesList = list<Types.Properties>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Env_FrameList = list<Env.Frame>;
      Exp_ExpList exp_1,exp_2;
      Types_PropertiesList typel;
      TupleTypes_TTypeAbsyn_PathOption tp,newtp;
      Boolean c,impl;
      Integer len;
      Exp.Type newtp_1;
      Exp.Exp exp;
      Env_FrameList env;
      Absyn_ExpList expl;
    case (env,expl,impl) "impl"
      equation 
        (exp_1,typel,_) = elab_exp_list(env, expl, impl, NONE);
        (exp_2,Types.PROP(tp,c)) = elab_builtin_array2(exp_1, typel);
        len = list_length(expl);
        newtp = (Types.T_ARRAY(Types.DIM(SOME(len)),tp),NONE);
        newtp_1 = Types.elab_type(newtp);
        exp = Exp.ARRAY(newtp_1,c,exp_1); then (exp,Types.PROP(newtp,c));
  end matchcontinue;
end elab_builtin_array;

protected function elab_builtin_array2
  input Exp_ExpList in_exp_explist;
  input Types_PropertiesList in_types_propertieslist;
  output Exp_ExpList out_exp_explist;
  output Types.Properties out_properties;
protected 
  type Exp_ExpList = list<Exp.Exp>;
  type Types_PropertiesList = list<Types.Properties>;
algorithm 
  (out_exp_explist,out_properties):=
  matchcontinue (in_exp_explist,in_types_propertieslist)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleTypes_TTypeAbsyn_PathOptionList = list<TupleTypes_TTypeAbsyn_PathOption>;
      Exp_ExpList expl,expl_1;
      Types_PropertiesList tpl;
      TupleTypes_TTypeAbsyn_PathOptionList tpl_1;
      Types.Properties tp;
    case (expl,tpl)
      equation 
        false = same_dimensions(tpl);
        Print.print_error_buf(
          "#Error, different dimension sizes in arguments to array.\n"); then fail();
    case (expl,tpl)
      equation 
        tpl_1 = Util.list_map(tpl, Types.get_prop_type) " If first elt is Integer but arguments contain Real, convert all to Real ";
        true = Types.contain_real(tpl_1);
        (expl_1,tp) = elab_builtin_array3(expl, tpl, Types.PROP((Types.T_REAL({}),NONE),false)); then (expl_1,tp);
    case (expl,(tpl = tp :: _))
      equation 
        (expl_1,tp) = elab_builtin_array3(expl, tpl, tp); then (expl_1,tp);
  end matchcontinue;
end elab_builtin_array2;

protected function elab_builtin_array3
  input Exp_ExpList in_exp_explist;
  input Types_PropertiesList in_types_propertieslist;
  input Types.Properties in_properties;
  output Exp_ExpList out_exp_explist;
  output Types.Properties out_properties;
protected 
  type Exp_ExpList = list<Exp.Exp>;
  type Types_PropertiesList = list<Types.Properties>;
algorithm 
  (out_exp_explist,out_properties):=
  matchcontinue (in_exp_explist,in_types_propertieslist,in_properties)
    local
      Types.Properties tp,t1;
      Exp.Exp e1_1,e1;
      Exp_ExpList expl_1,expl;
      Types_PropertiesList tpl;
    case ({},{},tp) then ({},tp); 
    case (e1 :: expl,t1 :: tpl,tp)
      equation 
        e1_1 = Types.match_prop(e1, t1, tp);
        (expl_1,_) = elab_builtin_array3(expl, tpl, tp); then (e1_1 :: expl_1,t1);
  end matchcontinue;
end elab_builtin_array3;

protected function elab_builtin_zeros "* function: elab_builtin_zeros
 ** This function elaborates the builtin operator 'zeros(n)'.
 *"
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp e;
      Types.Properties p;
      Env_FrameList env;
      Absyn_ExpList args;
      Boolean impl;
    case (env,args,impl) "impl"
      equation 
        (e,p) = elab_builtin_fill(env, Absyn.INTEGER(0) :: args, impl); then (e,p);
  end matchcontinue;
end elab_builtin_zeros;

protected function same_dimensions "* function: same_dimensions
 ** This function returns true of all the properties, containing types, have the same 
 ** dimensions, otherwise false. 
 *"
  input Types_PropertiesList tpl;
  output Boolean res;
protected 
  type Types_PropertiesList = list<Types.Properties>;
  type Absyn_PathOption = Option<Absyn.Path>;
  type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
  type TupleTypes_TTypeAbsyn_PathOptionList = list<TupleTypes_TTypeAbsyn_PathOption>;
  type IntegerList = list<Integer>;
  type IntegerListList = list<IntegerList>;
  TupleTypes_TTypeAbsyn_PathOptionList tpl_1;
  IntegerListList dimsizes;
algorithm 
  tpl_1 := Util.list_map(tpl, Types.get_prop_type);
  dimsizes := Util.list_map(tpl_1, Types.get_dimension_sizes);
  res := same_dimensions2(dimsizes);
end same_dimensions;

protected function same_dimensions2
  input IntegerListList in_integerlistlist;
  output Boolean out_boolean;
protected 
  type IntegerList = list<Integer>;
  type IntegerListList = list<IntegerList>;
algorithm 
  out_boolean:=
  matchcontinue (in_integerlistlist)
    local
      IntegerListList l,restelts;
      IntegerList elts;
    case (l)
      equation 
        {} = Util.list_flatten(l); then true;
    case (l)
      equation 
        elts = Util.list_map(l, Util.list_first);
        restelts = Util.list_map(l, Util.list_rest);
        true = same_dimensions3(elts);
        true = same_dimensions2(restelts); then true;
    case (_) then false; 
  end matchcontinue;
end same_dimensions2;

protected function same_dimensions3
  input IntegerList in_integerlist;
  output Boolean out_boolean;
protected 
  type IntegerList = list<Integer>;
algorithm 
  out_boolean:=
  matchcontinue (in_integerlist)
    local
      Integer i1,i2;
      Boolean res,res2,res_1;
      IntegerList rest;
    case ({}) then true; 
    case ({_}) then true; 
    case ({i1,i2}) then i1 == i2; 
    case (i1 :: i2 :: rest)
      equation 
        res = same_dimensions3(i2 :: rest);
        res2 = (i1 == i2);
        res_1 = bool_and(res, res2); then res_1;
    case (_) then false; 
  end matchcontinue;
end same_dimensions3;

protected function elab_builtin_ones "* function: elab_builtin_ones
 ** This function elaborates on the builtin opeator 'ones(n)'.
 *"
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp e;
      Types.Properties p;
      Env_FrameList env;
      Absyn_ExpList args;
      Boolean impl;
    case (env,args,impl) "impl"
      equation 
        (e,p) = elab_builtin_fill(env, Absyn.INTEGER(1) :: args, impl); then (e,p);
  end matchcontinue;
end elab_builtin_ones;

protected function elab_builtin_max "* function: elab_builtin_max
 **  This function elaborates on the builtin operator 'max(v1,v2)'
 *"
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Values_ValueList = list<Values.Value>;
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp arrexp_1,s1_1,s2_1;
      TupleTypes_TTypeAbsyn_PathOption arrty,ty;
      Boolean arrconst,impl;
      Values_ValueList vals;
      Integer i,i1,i2,res;
      Env_FrameList env;
      Absyn.Exp arrexp,s1,s2;
      Real r,r1,r2;
    case (env,{arrexp},impl) "impl"
      equation 
        (arrexp_1,Types.PROP(arrty,arrconst),_) = elab_exp(env, arrexp, impl, NONE) " max(vector) of type Integer ";
        (Values.ARRAY(vals),_) = Ceval.ceval(env, arrexp_1, impl, NONE, NONE);
        (Values.INTEGER(i),ty) = elab_builtin_max_2(Values.ARRAY(vals)); then (Exp.ICONST(i),Types.PROP(ty,true));
    case (env,{arrexp},impl)
      equation 
        (arrexp_1,Types.PROP(arrty,arrconst),_) = elab_exp(env, arrexp, impl, NONE) " max(vector) of type Real ";
        (Values.ARRAY(vals),_) = Ceval.ceval(env, arrexp_1, impl, NONE, NONE);
        (Values.REAL(r),ty) = elab_builtin_max_2(Values.ARRAY(vals)); then (Exp.RCONST(r),Types.PROP(ty,true));
    case (env,{s1,s2},impl)
      equation 
        (s1_1,Types.PROP(arrty,arrconst),_) = elab_exp(env, s1, impl, NONE) " max(x,y) where x & y are scalar Integers ";
        (Values.INTEGER(i1),_) = Ceval.ceval(env, s1_1, impl, NONE, NONE);
        (s2_1,Types.PROP(arrty,arrconst),_) = elab_exp(env, s2, impl, NONE);
        (Values.INTEGER(i2),_) = Ceval.ceval(env, s2_1, impl, NONE, NONE);
        res = int_max(i1, i2); then (Exp.ICONST(res),Types.PROP((Types.T_INTEGER({}),NONE),true));
    case (env,{s1,s2},impl)
      local Real res;
      equation 
        (s1_1,Types.PROP(arrty,arrconst),_) = elab_exp(env, s1, impl, NONE) " max(x,y) where x & y are scalar Reals ";
        (Values.REAL(r1),_) = Ceval.ceval(env, s1_1, impl, NONE, NONE);
        (s2_1,Types.PROP(arrty,arrconst),_) = elab_exp(env, s2, impl, NONE);
        (Values.REAL(r2),_) = Ceval.ceval(env, s2_1, impl, NONE, NONE);
        res = real_max(r1, r2); then (Exp.RCONST(res),Types.PROP((Types.T_REAL({}),NONE),true));
  end matchcontinue;
end elab_builtin_max;

protected function elab_builtin_max_2 "* function: elab_builtin_max_2
 ** Helper function to elab_builtin_max.
 *"
  input Values.Value in_value;
  output Values.Value out_value;
  output Types.Type out_type;
algorithm 
  (out_value,out_type):=
  matchcontinue (in_value)
    local
      type Values_ValueList = list<Values.Value>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      Integer i1,i2,res,i;
      Values.Value v1,v,vl;
      Values_ValueList vls;
      TupleTypes_TTypeAbsyn_PathOption t;
    case (Values.ARRAY(v1 :: (vls = _ :: _)))
      equation 
        (Values.INTEGER(i1),(Types.T_INTEGER(_),_)) = elab_builtin_max_2(v1);
        (Values.INTEGER(i2),(Types.T_INTEGER(_),_)) = elab_builtin_max_2(Values.ARRAY(vls));
        res = int_max(i1, i2); then (Values.INTEGER(res),(Types.T_INTEGER({}),NONE));
    case (Values.ARRAY(v1 :: (vls = _ :: _)))
      local Real i1,i2,res;
      equation 
        (Values.REAL(i1),(Types.T_REAL(_),_)) = elab_builtin_max_2(v1);
        (Values.REAL(i2),(Types.T_REAL(_),_)) = elab_builtin_max_2(Values.ARRAY(vls));
        res = real_max(i1, i2); then (Values.REAL(res),(Types.T_REAL({}),NONE));
    case (Values.ARRAY({vl}))
      equation 
        (v,t) = elab_builtin_max_2(vl); then (v,t);
    case (Values.INTEGER(i)) then (Values.INTEGER(i),(Types.T_INTEGER({}),NONE)); 
    case (Values.REAL(i))
      local Real i; then (Values.REAL(i),(Types.T_REAL({}),NONE));
  end matchcontinue;
end elab_builtin_max_2;

protected function elab_builtin_min "* function: elab_builtin_min
 ** This function elaborates the builtin operator 'min(a,b)'
 *"
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Values_ValueList = list<Values.Value>;
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp arrexp_1;
      TupleTypes_TTypeAbsyn_PathOption arrty,ty;
      Boolean arrconst,impl;
      Values_ValueList vals;
      Integer i;
      Env_FrameList env;
      Absyn.Exp arrexp;
      Real r;
    case (env,{arrexp},impl) "impl"
      equation 
        (arrexp_1,Types.PROP(arrty,arrconst),_) = elab_exp(env, arrexp, impl, NONE) " min(v) for an Integer vector v ";
        (Values.ARRAY(vals),_) = Ceval.ceval(env, arrexp_1, impl, NONE, NONE);
        (Values.INTEGER(i),ty) = elab_builtin_min_2(Values.ARRAY(vals)); then (Exp.ICONST(i),Types.PROP(ty,true));
    case (env,{arrexp},impl)
      equation 
        (arrexp_1,Types.PROP(arrty,arrconst),_) = elab_exp(env, arrexp, impl, NONE) " min(v) for a Real vector v ";
        (Values.ARRAY(vals),_) = Ceval.ceval(env, arrexp_1, impl, NONE, NONE);
        (Values.REAL(r),ty) = elab_builtin_min_2(Values.ARRAY(vals)); then (Exp.RCONST(r),Types.PROP(ty,true));
  end matchcontinue;
end elab_builtin_min;

protected function elab_builtin_min_2
  input Values.Value in_value;
  output Values.Value out_value;
  output Types.Type out_type;
algorithm 
  (out_value,out_type):=
  matchcontinue (in_value)
    local
      type Values_ValueList = list<Values.Value>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      Integer i1,i2,res,i;
      Values.Value v1,v,vl;
      Values_ValueList vls;
      TupleTypes_TTypeAbsyn_PathOption t;
    case (Values.ARRAY(v1 :: (vls = _ :: _)))
      equation 
        (Values.INTEGER(i1),(Types.T_INTEGER(_),_)) = elab_builtin_min_2(v1);
        (Values.INTEGER(i2),(Types.T_INTEGER(_),_)) = elab_builtin_min_2(Values.ARRAY(vls));
        res = int_min(i1, i2); then (Values.INTEGER(res),(Types.T_INTEGER({}),NONE));
    case (Values.ARRAY(v1 :: (vls = _ :: _)))
      local Real i1,i2,res;
      equation 
        (Values.REAL(i1),(Types.T_REAL(_),_)) = elab_builtin_min_2(v1);
        (Values.REAL(i2),(Types.T_REAL(_),_)) = elab_builtin_min_2(Values.ARRAY(vls));
        res = real_min(i1, i2); then (Values.REAL(res),(Types.T_REAL({}),NONE));
    case (Values.ARRAY({vl}))
      equation 
        (v,t) = elab_builtin_min_2(vl); then (v,t);
    case (Values.INTEGER(i)) then (Values.INTEGER(i),(Types.T_INTEGER({}),NONE)); 
    case (Values.REAL(i))
      local Real i; then (Values.REAL(i),(Types.T_REAL({}),NONE));
  end matchcontinue;
end elab_builtin_min_2;

protected function elab_builtin_floor "* function: elab_builtin_floor
 ** This function elaborates on the builtin operator floor.
 *"
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp s1_1;
      Boolean c,impl;
      Env_FrameList env;
      Absyn.Exp s1;
    case (env,{s1},impl) "impl"
      equation 
        (s1_1,Types.PROP((Types.T_REAL({}),NONE),c),_) = elab_exp(env, s1, impl, NONE) "print \"# floor function not implemented yet\n\" &"; then (Exp.CALL(Absyn.IDENT("floor"),{s1_1},false,true),Types.PROP((Types.T_REAL({}),NONE),c));
  end matchcontinue;
end elab_builtin_floor;

protected function elab_builtin_ceil "* function: elab_builtin_ceil
 ** This function elaborates on the builtin operator ceil.
 *"
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp s1_1;
      Boolean c,impl;
      Env_FrameList env;
      Absyn.Exp s1;
    case (env,{s1},impl) "impl"
      equation 
        (s1_1,Types.PROP((Types.T_REAL({}),NONE),c),_) = elab_exp(env, s1, impl, NONE) "*print \"# ceil function not implemented yet\n\" &*"; then (Exp.CALL(Absyn.IDENT("ceil"),{s1_1},false,true),Types.PROP((Types.T_REAL({}),NONE),c));
  end matchcontinue;
end elab_builtin_ceil;

protected function elab_builtin_abs "* function: elab_builtin_abs
 ** This function elaborates on the builtin operator abs
 *"
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp s1_1;
      Boolean c,impl;
      Env_FrameList env;
      Absyn.Exp s1;
    case (env,{s1},impl) "impl"
      equation 
        (s1_1,Types.PROP((Types.T_REAL({}),NONE),c),_) = elab_exp(env, s1, impl, NONE); then (Exp.CALL(Absyn.IDENT("abs"),{s1_1},false,true),Types.PROP((Types.T_REAL({}),NONE),c));
    case (env,{s1},impl)
      equation 
        (s1_1,Types.PROP((Types.T_INTEGER({}),NONE),c),_) = elab_exp(env, s1, impl, NONE); then (Exp.CALL(Absyn.IDENT("abs"),{s1_1},false,true),Types.PROP((Types.T_INTEGER({}),NONE),c));
  end matchcontinue;
end elab_builtin_abs;

protected function elab_builtin_sqrt "* function: elab_builtin_sqrt
 ** This function elaborates on the builtin operator sqrt.
 *"
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp s1_1;
      Boolean c,impl;
      Env_FrameList env;
      Absyn.Exp s1;
    case (env,{s1},impl) "impl"
      equation 
        (s1_1,Types.PROP((Types.T_REAL({}),NONE),c),_) = elab_exp(env, s1, impl, NONE); then (Exp.CALL(Absyn.IDENT("sqrt"),{s1_1},false,true),Types.PROP((Types.T_REAL({}),NONE),c));
  end matchcontinue;
end elab_builtin_sqrt;

protected function elab_builtin_div "* function: elab_builtin_div
 ** This function elaborates on the builtin operator div.
 *"
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp s1_1,s2_1;
      Boolean c,impl;
      Env_FrameList env;
      Absyn.Exp s1,s2;
    case (env,{s1,s2},impl) "impl"
      equation 
        (s1_1,Types.PROP((Types.T_REAL({}),NONE),c),_) = elab_exp(env, s1, impl, NONE);
        (s2_1,Types.PROP((Types.T_REAL({}),NONE),c),_) = elab_exp(env, s2, impl, NONE); then (Exp.CALL(Absyn.IDENT("div"),{s1_1,s2_1},false,true),Types.PROP((Types.T_REAL({}),NONE),c));
    case (env,{s1,s2},impl)
      equation 
        (s1_1,Types.PROP((Types.T_INTEGER({}),NONE),c),_) = elab_exp(env, s1, impl, NONE);
        (s2_1,Types.PROP((Types.T_REAL({}),NONE),c),_) = elab_exp(env, s2, impl, NONE); then (Exp.CALL(Absyn.IDENT("div"),{s1_1,s2_1},false,true),Types.PROP((Types.T_REAL({}),NONE),c));
    case (env,{s1,s2},impl)
      equation 
        (s1_1,Types.PROP((Types.T_REAL({}),NONE),c),_) = elab_exp(env, s1, impl, NONE);
        (s2_1,Types.PROP((Types.T_INTEGER({}),NONE),c),_) = elab_exp(env, s2, impl, NONE); then (Exp.CALL(Absyn.IDENT("div"),{s1_1,s2_1},false,true),Types.PROP((Types.T_REAL({}),NONE),c));
    case (env,{s1,s2},impl)
      equation 
        (s1_1,Types.PROP((Types.T_INTEGER({}),NONE),c),_) = elab_exp(env, s1, impl, NONE);
        (s2_1,Types.PROP((Types.T_INTEGER({}),NONE),c),_) = elab_exp(env, s2, impl, NONE); then (Exp.CALL(Absyn.IDENT("div"),{s1_1,s2_1},false,true),Types.PROP((Types.T_INTEGER({}),NONE),c));
  end matchcontinue;
end elab_builtin_div;

protected function elab_builtin_mod "* function: elab_builtin_mod
 ** This function elaborates on the builtin operator mod.
 *"
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp s1_1,s2_1;
      Boolean c,impl;
      Env_FrameList env;
      Absyn.Exp s1,s2;
    case (env,{s1,s2},impl) "impl"
      equation 
        (s1_1,Types.PROP((Types.T_REAL({}),NONE),c),_) = elab_exp(env, s1, impl, NONE);
        (s2_1,Types.PROP((Types.T_REAL({}),NONE),c),_) = elab_exp(env, s2, impl, NONE); then (Exp.CALL(Absyn.IDENT("mod"),{s1_1,s2_1},false,true),Types.PROP((Types.T_REAL({}),NONE),c));
    case (env,{s1,s2},impl)
      equation 
        (s1_1,Types.PROP((Types.T_INTEGER({}),NONE),c),_) = elab_exp(env, s1, impl, NONE);
        (s2_1,Types.PROP((Types.T_REAL({}),NONE),c),_) = elab_exp(env, s2, impl, NONE); then (Exp.CALL(Absyn.IDENT("mod"),{s1_1,s2_1},false,true),Types.PROP((Types.T_REAL({}),NONE),c));
    case (env,{s1,s2},impl)
      equation 
        (s1_1,Types.PROP((Types.T_REAL({}),NONE),c),_) = elab_exp(env, s1, impl, NONE);
        (s2_1,Types.PROP((Types.T_INTEGER({}),NONE),c),_) = elab_exp(env, s2, impl, NONE); then (Exp.CALL(Absyn.IDENT("mod"),{s1_1,s2_1},false,true),Types.PROP((Types.T_REAL({}),NONE),c));
    case (env,{s1,s2},impl)
      equation 
        (s1_1,Types.PROP((Types.T_INTEGER({}),NONE),c),_) = elab_exp(env, s1, impl, NONE);
        (s2_1,Types.PROP((Types.T_INTEGER({}),NONE),c),_) = elab_exp(env, s2, impl, NONE); then (Exp.CALL(Absyn.IDENT("mod"),{s1_1,s2_1},false,true),Types.PROP((Types.T_INTEGER({}),NONE),c));
  end matchcontinue;
end elab_builtin_mod;

protected function elab_builtin_rem "* function: elab_builtin_sqrt
 ** This function elaborates on the builtin operator rem.
 *"
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp s1_1,s2_1;
      Boolean c,impl;
      Env_FrameList env;
      Absyn.Exp s1,s2;
    case (env,{s1,s2},impl) "impl"
      equation 
        (s1_1,Types.PROP((Types.T_REAL({}),NONE),c),_) = elab_exp(env, s1, impl, NONE);
        (s2_1,Types.PROP((Types.T_REAL({}),NONE),c),_) = elab_exp(env, s2, impl, NONE); then (Exp.CALL(Absyn.IDENT("rem"),{s1_1,s2_1},false,true),Types.PROP((Types.T_REAL({}),NONE),c));
    case (env,{s1,s2},impl)
      equation 
        (s1_1,Types.PROP((Types.T_INTEGER({}),NONE),c),_) = elab_exp(env, s1, impl, NONE);
        (s2_1,Types.PROP((Types.T_REAL({}),NONE),c),_) = elab_exp(env, s2, impl, NONE); then (Exp.CALL(Absyn.IDENT("rem"),{s1_1,s2_1},false,true),Types.PROP((Types.T_REAL({}),NONE),c));
    case (env,{s1,s2},impl)
      equation 
        (s1_1,Types.PROP((Types.T_REAL({}),NONE),c),_) = elab_exp(env, s1, impl, NONE);
        (s2_1,Types.PROP((Types.T_INTEGER({}),NONE),c),_) = elab_exp(env, s2, impl, NONE); then (Exp.CALL(Absyn.IDENT("rem"),{s1_1,s2_1},false,true),Types.PROP((Types.T_REAL({}),NONE),c));
    case (env,{s1,s2},impl)
      equation 
        (s1_1,Types.PROP((Types.T_INTEGER({}),NONE),c),_) = elab_exp(env, s1, impl, NONE);
        (s2_1,Types.PROP((Types.T_INTEGER({}),NONE),c),_) = elab_exp(env, s2, impl, NONE); then (Exp.CALL(Absyn.IDENT("rem"),{s1_1,s2_1},false,true),Types.PROP((Types.T_INTEGER({}),NONE),c));
  end matchcontinue;
end elab_builtin_rem;

protected function elab_builtin_integer "* function: elab_builtin_integer
 ** This function elaborates on the builtin operator integer, which extracts the 
 ** Integer value of a Real value.
 *"
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp s1_1;
      Boolean c,impl;
      Env_FrameList env;
      Absyn.Exp s1;
    case (env,{s1},impl) "impl"
      equation 
        (s1_1,Types.PROP((Types.T_REAL({}),NONE),c),_) = elab_exp(env, s1, impl, NONE); then (Exp.CALL(Absyn.IDENT("integer"),{s1_1},false,true),Types.PROP((Types.T_INTEGER({}),NONE),c));
  end matchcontinue;
end elab_builtin_integer;

protected function elab_builtin_diagonal "* function: elab_builtin_diagonal
 ** This function elaborates on the builtin operator diagonal, creating a matrix with 
 ** a value of the diagonal. The other elements are zero.
 *"
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp s1_1;
      Types.ArrayDim dim;
      Integer dimension;
      TupleTypes_TTypeAbsyn_PathOption arrType;
      Boolean c,impl;
      Env_FrameList env;
      Absyn.Exp s1;
    case (env,{s1},impl) "impl"
      equation 
        (s1_1,Types.PROP((Types.T_ARRAY(dim = Types.DIM(SOME(dimension)),arrType),NONE),c),_) = elab_exp(env, s1, impl, NONE); then (Exp.CALL(Absyn.IDENT("diagonal"),{s1_1},false,true),Types.PROP(
          (Types.T_ARRAY(dim,(Types.T_ARRAY(dim,arrType),NONE)),NONE),c));
    case (_,_,_)
      equation 
        print(
          "#-- elab_builtin_diagonal: Couldn't elaborate diagonal()\n"); then fail();
  end matchcontinue;
end elab_builtin_diagonal;

protected function elab_builtin_differentiate "* function: elab_builtin_differentiate
 ** This function elaborates on the builtin operator differentiate, 
 ** by deriving the Exp
 *"
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Absyn_ComponentRefList = list<Absyn.ComponentRef>;
      type Env_FrameList = list<Env.Frame>;
      Absyn_ComponentRefList cref_list1,cref_list2,cref_list;
      Interactive.InteractiveSymbolTable symbol_table;
      Env_FrameList gen_env,env;
      Exp.Exp s1_1,s2_1;
      Types.Properties st;
      Absyn.Exp s1,s2;
      Boolean impl;
    case (env,{s1,s2},impl) "impl"
      equation 
        cref_list1 = Absyn.get_cref_from_exp(s1);
        cref_list2 = Absyn.get_cref_from_exp(s2);
        cref_list = list_append(cref_list1, cref_list2);
        symbol_table = absyn_cref_list_to_interactive_var_list(cref_list, 
          Interactive.SYMBOLTABLE(Absyn.PROGRAM({},Absyn.TOP()),{},{},{},{}), (Types.T_REAL({}),NONE));
        gen_env = Interactive.build_env_from_symboltable(symbol_table);
        (s1_1,st,_) = elab_exp(gen_env, s1, impl, NONE);
        (s2_1,st,_) = elab_exp(gen_env, s2, impl, NONE); then (Exp.CALL(Absyn.IDENT("differentiate"),{s1_1,s2_1},false,true),st);
    case (_,_,_)
      equation 
        print(
          "#-- elab_builtin_differentiate: Couldn't elaborate differentiate()\n"); then fail();
  end matchcontinue;
end elab_builtin_differentiate;

protected function elab_builtin_simplify "* function: elab_builtin_simplify
 ** This function elaborates on the builtin operator simplify, 
 ** The call in mosh is: simplify(x+y*x-x,\"Real\") if the variable should be Real
 ** or simplify(x+y*x-x,\"Integer\") if the variable should be Integer
 ** This function is only for testing simplify
 *"
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Absyn_ComponentRefList = list<Absyn.ComponentRef>;
      type Env_FrameList = list<Env.Frame>;
      Absyn_ComponentRefList cref_list;
      Interactive.InteractiveSymbolTable symbol_table;
      Env_FrameList gen_env,env;
      Exp.Exp s1_1;
      Types.Properties st;
      Absyn.Exp s1;
      Boolean impl;
    case (env,{s1,Absyn.STRING("Real")},impl) "impl"
      equation 
        cref_list = Absyn.get_cref_from_exp(s1);
        symbol_table = absyn_cref_list_to_interactive_var_list(cref_list, 
          Interactive.SYMBOLTABLE(Absyn.PROGRAM({},Absyn.TOP()),{},{},{},{}), (Types.T_REAL({}),NONE));
        gen_env = Interactive.build_env_from_symboltable(symbol_table);
        (s1_1,st,_) = elab_exp(gen_env, s1, impl, NONE); then (Exp.CALL(Absyn.IDENT("simplify"),{s1_1},false,true),st);
    case (env,{s1,Absyn.STRING("Integer")},impl)
      equation 
        cref_list = Absyn.get_cref_from_exp(s1);
        symbol_table = absyn_cref_list_to_interactive_var_list(cref_list, 
          Interactive.SYMBOLTABLE(Absyn.PROGRAM({},Absyn.TOP()),{},{},{},{}), (Types.T_INTEGER({}),NONE));
        gen_env = Interactive.build_env_from_symboltable(symbol_table);
        (s1_1,st,_) = elab_exp(gen_env, s1, impl, NONE); then (Exp.CALL(Absyn.IDENT("simplify"),{s1_1},false,true),st);
    case (_,_,_)
      equation 
        print(
          "#-- elab_builtin_simplify: Couldn't elaborate simplify()\n"); then fail();
  end matchcontinue;
end elab_builtin_simplify;

protected function absyn_cref_list_to_interactive_var_list
  input Absyn_ComponentRefList in_absyn_componentreflist;
  input Interactive.InteractiveSymbolTable in_interactivesymboltable;
  input Types.Type in_type;
  output Interactive.InteractiveSymbolTable out_interactivesymboltable;
protected 
  type Absyn_ComponentRefList = list<Absyn.ComponentRef>;
algorithm 
  out_interactivesymboltable:=
  matchcontinue (in_absyn_componentreflist,in_interactivesymboltable,in_type)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      Interactive.InteractiveSymbolTable symbol_table,symbol_table_1,symbol_table_2;
      Absyn.Path path;
      Ident path_str;
      Absyn.ComponentRef cr;
      Absyn_ComponentRefList rest;
      TupleTypes_TTypeAbsyn_PathOption tp;
    case ({},symbol_table,_) then symbol_table; 
    case (cr :: rest,symbol_table,tp)
      equation 
        path = Absyn.cref_to_path(cr);
        path_str = Absyn.path_string(path);
        symbol_table_1 = Interactive.add_var_to_symboltable(path_str, Values.CODE(Absyn.C_VARIABLENAME(cr)), tp, 
          symbol_table);
        symbol_table_2 = absyn_cref_list_to_interactive_var_list(rest, symbol_table_1, tp); then symbol_table_2;
    case (_,_,_)
      equation 
        Print.print_error_buf("error in absyn_cref_list_to_interactive_var_list"); then fail();
  end matchcontinue;
end absyn_cref_list_to_interactive_var_list;

protected function elab_builtin_dymtabletimeini
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp e1_1,e2_1,e3_1,e4_1,e5_1,e6_1;
      Boolean c,impl;
      Types.ArrayDim dim;
      TupleTypes_TTypeAbsyn_PathOption arrType;
      Env_FrameList env;
      Absyn.Exp e1,e2,e3,e4,e5,e6;
    case (env,{e1,e2,e3,e4,e5,e6},impl) "impl"
      equation 
        (e1_1,Types.PROP((Types.T_REAL({}),NONE),c),_) = elab_exp(env, e1, impl, NONE);
        (e2_1,Types.PROP((Types.T_INTEGER({}),NONE),c),_) = elab_exp(env, e2, impl, NONE);
        (e3_1,Types.PROP((Types.T_STRING({}),NONE),c),_) = elab_exp(env, e3, impl, NONE);
        (e4_1,Types.PROP((Types.T_STRING({}),NONE),c),_) = elab_exp(env, e4, impl, NONE);
        (e5_1,Types.PROP((Types.T_ARRAY(dim,arrType),NONE),c),_) = elab_exp(env, e5, impl, NONE);
        (e6_1,Types.PROP((Types.T_INTEGER({}),NONE),c),_) = elab_exp(env, e6, impl, NONE); then (Exp.CALL(Absyn.IDENT("dymTableTimeIni"),
          {e1_1,e2_1,e3_1,e4_1,e5_1,e6_1},false,true),Types.PROP((Types.T_REAL({}),NONE),false));
    case (_,_,_)
      equation 
        print(
          "#-- elab_builtin_dymtabletimeini: Couldn't elaborate diagonal()\n"); then fail();
  end matchcontinue;
end elab_builtin_dymtabletimeini;

protected function elab_builtin_noevent "* function: elab_builtin_noevent
 ** The builtin operator noevent makes sure that events are not generated
 ** for the expression.
 *"
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp exp_1;
      Types.Properties prop;
      Env_FrameList env;
      Absyn.Exp exp;
      Boolean impl;
    case (env,{exp},impl) "impl"
      equation 
        (exp_1,prop,_) = elab_exp(env, exp, impl, NONE); then (Exp.CALL(Absyn.IDENT("noEvent"),{exp_1},false,true),prop);
  end matchcontinue;
end elab_builtin_noevent;

protected function elab_builtin_edge "* function: elab_builtin_edge
 ** This function handles the built in edge operator. If the operand is constant edge is always false
 *"
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_absyn_explist,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp exp_1,exp_2;
      Boolean c,impl;
      Env_FrameList env;
      Absyn.Exp exp;
    case (env,{exp},impl) "impl"
      equation 
        (exp_1,Types.PROP((Types.T_BOOL({}),_),c = false),_) = elab_exp(env, exp, impl, NONE); then (Exp.CALL(Absyn.IDENT("edge"),{exp_1},false,true),Types.PROP((Types.T_BOOL({}),NONE),false));
    case (env,{exp},impl)
      equation 
        (exp_1,Types.PROP((Types.T_BOOL({}),_),c = true),_) = elab_exp(env, exp, impl, NONE);
        exp_2 = value_exp(Values.BOOL(false)); then (exp_2,Types.PROP((Types.T_BOOL({}),NONE),true));
    case (env,_,_)
      equation 
        Print.print_error_buf(
          "#Error, wrong type or wrong number of arguments to edge operator.\n"); then fail();
  end matchcontinue;
end elab_builtin_edge;

public function elab_builtin_handler
  input Ident in_ident;
  output FuncType1 out_functype1;
protected 
  function FuncType1
    input Env.Env in_env;
    input Absyn_ExpList in_absyn_explist;
    input Boolean in_boolean;
    output Exp.Exp out_exp;
    output Types.Properties out_properties;
  protected 
    type Absyn_ExpList = list<Absyn.Exp>;
  end FuncType1;
algorithm 
  out_functype1:=
  matchcontinue (in_ident)
    case "size" then elab_builtin_size; 
    case "zeros" then elab_builtin_zeros; 
    case "ones" then elab_builtin_ones; 
    case "fill" then elab_builtin_fill; 
    case "max" then elab_builtin_max; 
    case "min" then elab_builtin_min; 
    case "transpose" then elab_builtin_transpose; 
    case "array" then elab_builtin_array; 
    case "sum" then elab_builtin_sum; 
    case "pre" then elab_builtin_pre; 
    case "initial" then elab_builtin_initial; 
    case "terminal" then elab_builtin_terminal; 
    case "floor" then elab_builtin_floor; 
    case "ceil" then elab_builtin_ceil; 
    case "abs" then elab_builtin_abs; 
    case "sqrt" then elab_builtin_sqrt; 
    case "div" then elab_builtin_div; 
    case "integer" then elab_builtin_integer; 
    case "mod" then elab_builtin_mod; 
    case "rem" then elab_builtin_rem; 
    case "diagonal" then elab_builtin_diagonal; 
    case "differentiate" then elab_builtin_differentiate; 
    case "simplify" then elab_builtin_simplify; 
    case "noEvent" then elab_builtin_noevent; 
    case "edge" then elab_builtin_edge; 
    case "dymTableTimeIni" then elab_builtin_dymtabletimeini; 
  end matchcontinue;
end elab_builtin_handler;

protected function is_builtin_func
  input Absyn.Path in_path;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_path)
    local
      Ident id;
      Absyn.Path path;
    case Absyn.IDENT(id)
      equation 
        _ = elab_builtin_handler(id); then true;
    case path
      equation 
        true = Lookup.is_in_builtin_env(path); then true;
    case _ then false; 
  end matchcontinue;
end is_builtin_func;

protected function elab_call_builtin
  input Env.Env in_env;
  input Absyn.ComponentRef in_componentref;
  input Absyn_ExpList in_absyn_explist;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_componentref,in_absyn_explist,in_boolean)
    local
      function FuncType1
        input Env_FrameList in_env_framelist;
        input Absyn_ExpList in_absyn_explist;
        input Boolean in_boolean;
        output Exp.Exp out_exp;
        output Types.Properties out_properties;
      protected 
        type Env_FrameList = list<Env.Frame>;
      end FuncType1;
      type Env_FrameList = list<Env.Frame>;
      FuncType1 handler;
      Exp.Exp exp;
      Types.Properties prop;
      Env_FrameList env;
      Ident name;
      Absyn_ExpList args;
      Boolean impl;
    case (env,Absyn.CREF_IDENT(name,{}),args,impl) "impl"
      equation 
        handler = elab_builtin_handler(name);
        (exp,prop) = handler(env, args, impl); then (exp,prop);
  end matchcontinue;
end elab_call_builtin;

protected function elab_call "impl"
  input Env.Env in_env;
  input Absyn.ComponentRef in_componentref;
  input Absyn_ExpList in_absyn_explist;
  input Absyn_NamedArgList in_absyn_namedarglist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
  output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
  type Absyn_NamedArgList = list<Absyn.NamedArg>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  (out_exp,out_properties,out_interactive_interactivesymboltableoption):=
  matchcontinue (in_env,in_componentref,in_absyn_explist,in_absyn_namedarglist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      type IdentList = list<Ident>;
      Exp.Exp e;
      Types.Properties prop;
      Interactive_InteractiveSymbolTableOption st,st_1;
      Env_FrameList env;
      Absyn.ComponentRef fn;
      Absyn_ExpList args;
      Absyn_NamedArgList nargs;
      Boolean impl;
      Absyn.Path fn_1;
      Ident fnstr,argstr;
      IdentList argstrs;
    case (env,fn,args,nargs,impl,st) "impl * LS: Check if a builtin function call, e.g. size()
	      and calculate if so *"
      equation 
        (e,prop,st) = elab_call_interactive(env, fn, args, nargs, impl, st) " Elaborate interactive function calls "; then (e,prop,st);
    case (env,fn,args,nargs,impl,st)
      equation 
        (e,prop) = elab_call_builtin(env, fn, args, impl) " PA. Built in functions have only possitional arguments "; then (e,prop,st);
    case (env,fn,args,nargs,(impl = true),st) " Interactive mode "
      equation 
        Debug.fprintln("sei", "elab_call 3");
        fn_1 = Absyn.cref_to_path(fn);
        (e,prop) = elab_call_args(env, fn_1, args, nargs, impl, st);
        st_1 = generate_compiled_function(env, fn, e, prop, st);
        Debug.fprint("sei", "elab_call 3 succeeded: ");
        fnstr = Dump.print_component_ref_str(fn);
        Debug.fprintln("sei", fnstr); then (e,prop,st_1);
    case (env,fn,args,nargs,(impl = false),st) " Non-interactive mode "
      equation 
        Debug.fprint("sei", "elab_call 4: ");
        fnstr = Dump.print_component_ref_str(fn);
        Debug.fprintln("sei", fnstr);
        fn_1 = Absyn.cref_to_path(fn);
        (e,prop) = elab_call_args(env, fn_1, args, nargs, impl, st);
        st_1 = generate_compiled_function(env, fn, e, prop, st);
        Debug.fprint("sei", "elab_call 4 succeeded: ");
        Debug.fprintln("sei", fnstr); then (e,prop,st_1);
    case (env,fn,args,nargs,impl,st)
      equation 
        Debug.fprint("failtrace", "- elab_call failed\n");
        Debug.fprint("failtrace", " function: ");
        fnstr = Dump.print_component_ref_str(fn);
        Debug.fprint("failtrace", fnstr);
        Debug.fprint("failtrace", "   posargs: ");
        argstrs = Util.list_map(args, Dump.print_exp_str);
        argstr = Util.string_delimit_list(argstrs, ", ");
        Debug.fprintln("failtrace", argstr); then fail();
  end matchcontinue;
end elab_call;

protected function elab_call_interactive
  input Env.Env in_env;
  input Absyn.ComponentRef in_componentref;
  input Absyn_ExpList in_absyn_explist;
  input Absyn_NamedArgList in_absyn_namedarglist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
  output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
  type Absyn_NamedArgList = list<Absyn.NamedArg>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  (out_exp,out_properties,out_interactive_interactivesymboltableoption):=
  matchcontinue (in_env,in_componentref,in_absyn_explist,in_absyn_namedarglist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Exp_ExpList = list<Exp.Exp>;
      Absyn.Path path;
      Exp.ComponentRef cr_1,cr2_1;
      Env_FrameList env;
      Absyn.ComponentRef cr,cr2;
      Boolean impl;
      Interactive.InteractiveSymbolTable st,st_2,st_3;
      Ident varid,filename,str;
      TupleTypes_TTypeAbsyn_PathOption recordtype;
      Exp.Exp startTime,stopTime,numberOfIntervals,exp_1,exp1_1,exp2_1,exp3_1;
      Absyn_NamedArgList args;
      Exp_ExpList vars_1;
      Absyn_ExpList vars;
      Integer size;
      Types.Properties prop,prop1,prop2,prop3;
      Interactive_InteractiveSymbolTableOption st_1;
      Absyn.Exp exp,aexp,acref,aexp1,aexp2;
    case (env,Absyn.CREF_IDENT("lookupClass",_),{Absyn.CREF(cr)},{},impl,SOME(st))
      equation 
        path = Absyn.cref_to_path(cr);
        cr_1 = path_to_component_ref(path); then (Exp.CALL(Absyn.IDENT("lookupClass"),{Exp.CREF(cr_1,Exp.OTHER())},
          false,true),Types.PROP((Types.T_STRING({}),NONE),false),SOME(st));
    case (env,Absyn.CREF_IDENT("typeOf",_),{Absyn.CREF(Absyn.CREF_IDENT(varid,{}))},{},impl,SOME(st)) then (Exp.CALL(Absyn.IDENT("typeOf"),
          {Exp.CREF(Exp.CREF_IDENT(varid,{}),Exp.OTHER())},false,true),Types.PROP((Types.T_STRING({}),NONE),false),SOME(st)); 
    case (env,Absyn.CREF_IDENT("clear",_),{},{},impl,SOME(st)) then (Exp.CALL(Absyn.IDENT("clear"),{},false,true),Types.PROP((Types.T_BOOL({}),NONE),false),SOME(st)); 
    case (env,Absyn.CREF_IDENT("clearVariables",_),{},{},impl,SOME(st)) then (Exp.CALL(Absyn.IDENT("clearVariables"),{},false,true),Types.PROP((Types.T_BOOL({}),NONE),false),SOME(st)); 
    case (env,Absyn.CREF_IDENT("list",_),{},{},impl,SOME(st)) then (Exp.CALL(Absyn.IDENT("list"),{},false,true),Types.PROP((Types.T_STRING({}),NONE),false),SOME(st)); 
    case (env,Absyn.CREF_IDENT("list",_),{Absyn.CREF(cr)},{},impl,SOME(st))
      equation 
        cr_1 = elab_untyped_cref(env, cr, impl); then (Exp.CALL(Absyn.IDENT("list"),{Exp.CREF(cr_1,Exp.OTHER())},false,true),Types.PROP((Types.T_STRING({}),NONE),false),SOME(st));
    case (env,Absyn.CREF_IDENT("translateModel",_),{Absyn.CREF(cr)},{},impl,SOME(st))
      equation 
        cr_1 = elab_untyped_cref(env, cr, impl);
        recordtype = (
          Types.T_COMPLEX(ClassInf.RECORD("SimulationObject"),
          {
          Types.VAR("flatClass",
          Types.ATTR(false,SCode.RO(),SCode.VAR(),Absyn.BIDIR()),false,(Types.T_STRING({}),NONE),Types.UNBOUND()),
          Types.VAR("exeFile",
          Types.ATTR(false,SCode.RO(),SCode.VAR(),Absyn.BIDIR()),false,(Types.T_STRING({}),NONE),Types.UNBOUND())}),NONE); then (Exp.CALL(Absyn.IDENT("translateModel"),{Exp.CREF(cr_1,Exp.OTHER())},
          false,true),Types.PROP(recordtype,false),SOME(st));
    case (env,Absyn.CREF_IDENT("instantiateModel",_),{Absyn.CREF(cr)},{},impl,SOME(st))
      equation 
        cr_1 = elab_untyped_cref(env, cr, impl); then (Exp.CALL(Absyn.IDENT("instantiateModel"),
          {Exp.CREF(cr_1,Exp.OTHER())},false,true),Types.PROP((Types.T_STRING({}),NONE),false),SOME(st));
    case (env,Absyn.CREF_IDENT("simulate",_),{Absyn.CREF(cr)},args,impl,SOME(st))
      equation 
        cr_1 = elab_untyped_cref(env, cr, impl);
        startTime = get_optional_named_arg(env, SOME(st), impl, "startTime", (Types.T_REAL({}),NONE), 
          args, Exp.RCONST(0.0));
        stopTime = get_optional_named_arg(env, SOME(st), impl, "stopTime", (Types.T_REAL({}),NONE), 
          args, Exp.RCONST(1.0));
        numberOfIntervals = get_optional_named_arg(env, SOME(st), impl, "numberOfIntervals", 
          (Types.T_INTEGER({}),NONE), args, Exp.ICONST(500));
        recordtype = (
          Types.T_COMPLEX(ClassInf.RECORD("SimulationResult"),
          {
          Types.VAR("resultFile",
          Types.ATTR(false,SCode.RO(),SCode.VAR(),Absyn.BIDIR()),false,(Types.T_STRING({}),NONE),Types.UNBOUND())}),NONE); then (Exp.CALL(Absyn.IDENT("simulate"),
          {Exp.CREF(cr_1,Exp.OTHER()),startTime,stopTime,
          numberOfIntervals},false,true),Types.PROP(recordtype,false),SOME(st));
    case (env,Absyn.CREF_IDENT("jacobian",_),{Absyn.CREF(cr)},args,impl,SOME(st)) " Fill in rest of defaults here"
      equation 
        cr_1 = elab_untyped_cref(env, cr, impl); then (Exp.CALL(Absyn.IDENT("jacobian"),{Exp.CREF(cr_1,Exp.OTHER())},false,
          true),Types.PROP((Types.T_STRING({}),NONE),false),SOME(st));
    case (env,Absyn.CREF_IDENT("readSimulationResult",_),{Absyn.STRING(filename),Absyn.ARRAY(vars),Absyn.INTEGER(size)},args,impl,SOME(st))
      equation 
        vars_1 = elab_variablenames(vars); then (Exp.CALL(Absyn.IDENT("readSimulationResult"),
          {Exp.SCONST(filename),Exp.ARRAY(Exp.OTHER(),false,vars_1),
          Exp.ICONST(size)},false,true),Types.PROP((Types.T_STRING({}),NONE),false),SOME(st));
    case (env,Absyn.CREF_IDENT("plot",_),{Absyn.ARRAY(vars)},{},impl,SOME(st))
      equation 
        vars_1 = elab_variablenames(vars); then (Exp.CALL(Absyn.IDENT("plot"),{Exp.ARRAY(Exp.OTHER(),false,vars_1)},
          false,true),Types.PROP((Types.T_BOOL({}),NONE),false),SOME(st));
    case (env,Absyn.CREF_IDENT("timing",_),{exp},{},impl,SOME(st))
      equation 
        (exp_1,prop,st_1) = elab_exp(env, exp, impl, SOME(st)); then (Exp.CALL(Absyn.IDENT("timing"),{exp_1},false,true),Types.PROP((Types.T_REAL({}),NONE),false),st_1);
    case (env,Absyn.CREF_IDENT("generateCode",_),{Absyn.CREF(cr)},{},impl,SOME(st))
      equation 
        cr_1 = elab_untyped_cref(env, cr, impl); then (Exp.CALL(Absyn.IDENT("generateCode"),{Exp.CREF(cr_1,Exp.OTHER())},
          false,true),Types.PROP((Types.T_BOOL({}),NONE),false),SOME(st));
    case (env,Absyn.CREF_IDENT("setCompiler",_),{Absyn.STRING(str)},{},impl,SOME(st)) then (Exp.CALL(Absyn.IDENT("setCompiler"),{Exp.SCONST(str)},false,true),Types.PROP((Types.T_BOOL({}),NONE),false),SOME(st)); 
    case (env,Absyn.CREF_IDENT("setCompilerFlags",_),{Absyn.STRING(str)},{},impl,SOME(st)) then (Exp.CALL(Absyn.IDENT("setCompilerFlags"),{Exp.SCONST(str)},false,
          true),Types.PROP((Types.T_BOOL({}),NONE),false),SOME(st)); 
    case (env,Absyn.CREF_IDENT("setDebugFlags",_),{Absyn.STRING(str)},{},impl,SOME(st)) then (Exp.CALL(Absyn.IDENT("setDebugFlags"),{Exp.SCONST(str)},false,true),Types.PROP((Types.T_BOOL({}),NONE),false),SOME(st)); 
    case (env,Absyn.CREF_IDENT("cd",_),{Absyn.STRING(str)},{},impl,SOME(st)) then (Exp.CALL(Absyn.IDENT("cd"),{Exp.SCONST(str)},false,true),Types.PROP((Types.T_BOOL({}),NONE),false),SOME(st)); 
    case (env,Absyn.CREF_IDENT("cd",_),{},{},impl,SOME(st)) then (Exp.CALL(Absyn.IDENT("cd"),{},false,true),Types.PROP((Types.T_STRING({}),NONE),false),SOME(st)); 
    case (env,Absyn.CREF_IDENT("system",_),{Absyn.STRING(str)},{},impl,SOME(st)) then (Exp.CALL(Absyn.IDENT("system"),{Exp.SCONST(str)},false,true),Types.PROP((Types.T_INTEGER({}),NONE),false),SOME(st)); 
    case (env,Absyn.CREF_IDENT("readFile",_),{Absyn.STRING(str)},{},impl,SOME(st)) then (Exp.CALL(Absyn.IDENT("readFile"),{Exp.SCONST(str)},false,true),Types.PROP((Types.T_STRING({}),NONE),false),SOME(st)); 
    case (env,Absyn.CREF_IDENT("listVariables",_),{},{},impl,SOME(st)) then (Exp.CALL(Absyn.IDENT("listVariables"),{},false,true),Types.PROP(
          (Types.T_ARRAY(Types.DIM(NONE),(Types.T_NOTYPE(),NONE)),NONE),false),SOME(st)); 
    case (env,Absyn.CREF_IDENT("getErrorString",_),{},{},impl,SOME(st)) then (Exp.CALL(Absyn.IDENT("getErrorString"),{},false,true),Types.PROP((Types.T_STRING({}),NONE),false),SOME(st)); 
    case (env,Absyn.CREF_IDENT("runScript",_),{Absyn.STRING(str)},{},impl,SOME(st)) then (Exp.CALL(Absyn.IDENT("runScript"),{Exp.SCONST(str)},false,true),Types.PROP((Types.T_STRING({}),NONE),false),SOME(st)); 
    case (env,Absyn.CREF_IDENT("loadModel",_),{Absyn.CREF(cr)},{},impl,SOME(st))
      equation 
        cr_1 = elab_untyped_cref(env, cr, impl); then (Exp.CALL(Absyn.IDENT("loadModel"),{Exp.CREF(cr_1,Exp.OTHER())},
          false,true),Types.PROP((Types.T_BOOL({}),NONE),false),SOME(st));
    case (env,Absyn.CREF_IDENT("loadFile",_),{Absyn.STRING(str)},{},impl,SOME(st)) then (Exp.CALL(Absyn.IDENT("loadFile"),{Exp.SCONST(str)},false,true),Types.PROP((Types.T_BOOL({}),NONE),false),SOME(st)); 
    case (env,Absyn.CREF_IDENT("saveModel",_),{Absyn.STRING(str),Absyn.CREF(cr)},{},impl,SOME(st))
      equation 
        cr_1 = elab_untyped_cref(env, cr, impl); then (Exp.CALL(Absyn.IDENT("saveModel"),
          {Exp.SCONST(str),Exp.CREF(cr_1,Exp.OTHER())},false,true),Types.PROP((Types.T_BOOL({}),NONE),false),SOME(st));
    case (env,Absyn.CREF_IDENT("save",_),{Absyn.CREF(cr)},{},impl,SOME(st))
      equation 
        cr_1 = elab_untyped_cref(env, cr, impl); then (Exp.CALL(Absyn.IDENT("save"),{Exp.CREF(cr_1,Exp.OTHER())},false,true),Types.PROP((Types.T_BOOL({}),NONE),false),SOME(st));
    case (env,Absyn.CREF_IDENT("saveAll",_),{Absyn.STRING(str)},{},impl,SOME(st)) then (Exp.CALL(Absyn.IDENT("saveAll"),{Exp.SCONST(str)},false,true),Types.PROP((Types.T_BOOL({}),NONE),false),SOME(st)); 
    case (env,Absyn.CREF_IDENT("help",_),{},{},impl,SOME(st)) then (Exp.CALL(Absyn.IDENT("help"),{},false,true),Types.PROP((Types.T_STRING({}),NONE),false),SOME(st)); 
    case (env,Absyn.CREF_IDENT("getUnit",_),{Absyn.CREF(cr),Absyn.CREF(cr2)},{},impl,SOME(st))
      equation 
        cr_1 = elab_untyped_cref(env, cr, impl);
        cr2_1 = elab_untyped_cref(env, cr2, impl); then (Exp.CALL(Absyn.IDENT("getUnit"),
          {Exp.CREF(cr_1,Exp.OTHER()),Exp.CREF(cr2_1,Exp.OTHER())},false,true),Types.PROP((Types.T_STRING({}),NONE),false),SOME(st));
    case (env,Absyn.CREF_IDENT("getQuantity",_),{Absyn.CREF(cr),Absyn.CREF(cr2)},{},impl,SOME(st))
      equation 
        cr_1 = elab_untyped_cref(env, cr, impl);
        cr2_1 = elab_untyped_cref(env, cr2, impl); then (Exp.CALL(Absyn.IDENT("getQuantity"),
          {Exp.CREF(cr_1,Exp.OTHER()),Exp.CREF(cr2_1,Exp.OTHER())},false,true),Types.PROP((Types.T_STRING({}),NONE),false),SOME(st));
    case (env,Absyn.CREF_IDENT("getDisplayUnit",_),{Absyn.CREF(cr),Absyn.CREF(cr2)},{},impl,SOME(st))
      equation 
        cr_1 = elab_untyped_cref(env, cr, impl);
        cr2_1 = elab_untyped_cref(env, cr2, impl); then (Exp.CALL(Absyn.IDENT("getDisplayUnit"),
          {Exp.CREF(cr_1,Exp.OTHER()),Exp.CREF(cr2_1,Exp.OTHER())},false,true),Types.PROP((Types.T_STRING({}),NONE),false),SOME(st));
    case (env,Absyn.CREF_IDENT("getMin",_),{Absyn.CREF(cr),Absyn.CREF(cr2)},{},impl,SOME(st))
      equation 
        cr_1 = elab_untyped_cref(env, cr, impl);
        cr2_1 = elab_untyped_cref(env, cr2, impl); then (Exp.CALL(Absyn.IDENT("getMin"),
          {Exp.CREF(cr_1,Exp.OTHER()),Exp.CREF(cr2_1,Exp.OTHER())},false,true),Types.PROP((Types.T_REAL({}),NONE),false),SOME(st));
    case (env,Absyn.CREF_IDENT("getMax",_),{Absyn.CREF(cr),Absyn.CREF(cr2)},{},impl,SOME(st))
      equation 
        cr_1 = elab_untyped_cref(env, cr, impl);
        cr2_1 = elab_untyped_cref(env, cr2, impl); then (Exp.CALL(Absyn.IDENT("getMax"),
          {Exp.CREF(cr_1,Exp.OTHER()),Exp.CREF(cr2_1,Exp.OTHER())},false,true),Types.PROP((Types.T_REAL({}),NONE),false),SOME(st));
    case (env,Absyn.CREF_IDENT("getStart",_),{Absyn.CREF(cr),Absyn.CREF(cr2)},{},impl,SOME(st))
      equation 
        cr_1 = elab_untyped_cref(env, cr, impl);
        cr2_1 = elab_untyped_cref(env, cr2, impl); then (Exp.CALL(Absyn.IDENT("getStart"),
          {Exp.CREF(cr_1,Exp.OTHER()),Exp.CREF(cr2_1,Exp.OTHER())},false,true),Types.PROP((Types.T_REAL({}),NONE),false),SOME(st));
    case (env,Absyn.CREF_IDENT("getFixed",_),{Absyn.CREF(cr),Absyn.CREF(cr2)},{},impl,SOME(st))
      equation 
        cr_1 = elab_untyped_cref(env, cr, impl);
        cr2_1 = elab_untyped_cref(env, cr2, impl); then (Exp.CALL(Absyn.IDENT("getFixed"),
          {Exp.CREF(cr_1,Exp.OTHER()),Exp.CREF(cr2_1,Exp.OTHER())},false,true),Types.PROP((Types.T_BOOL({}),NONE),false),SOME(st));
    case (env,Absyn.CREF_IDENT("getNominal",_),{Absyn.CREF(cr),Absyn.CREF(cr2)},{},impl,SOME(st))
      equation 
        cr_1 = elab_untyped_cref(env, cr, impl);
        cr2_1 = elab_untyped_cref(env, cr2, impl); then (Exp.CALL(Absyn.IDENT("getNominal"),
          {Exp.CREF(cr_1,Exp.OTHER()),Exp.CREF(cr2_1,Exp.OTHER())},false,true),Types.PROP((Types.T_REAL({}),NONE),false),SOME(st));
    case (env,Absyn.CREF_IDENT("getStateSelect",_),{Absyn.CREF(cr),Absyn.CREF(cr2)},{},impl,SOME(st))
      equation 
        cr_1 = elab_untyped_cref(env, cr, impl);
        cr2_1 = elab_untyped_cref(env, cr2, impl); then (Exp.CALL(Absyn.IDENT("getStateSelect"),
          {Exp.CREF(cr_1,Exp.OTHER()),Exp.CREF(cr2_1,Exp.OTHER())},false,true),Types.PROP(
          (
          Types.T_ENUMERATION({"never","avoid","default","prefer","always"},{}),NONE),false),SOME(st));
    case (env,Absyn.CREF_IDENT("exp_contains",_),{aexp,acref},{},impl,SOME(st)) " FIXME: test code: remove "
      local Interactive.InteractiveSymbolTable st_1;
      equation 
        (exp1_1,prop1,SOME(st_1)) = elab_exp(env, aexp, impl, SOME(st));
        (exp2_1,prop2,SOME(st_2)) = elab_exp(env, acref, impl, SOME(st_1)); then (Exp.CALL(Absyn.IDENT("exp_contains"),{exp1_1,exp2_1},false,true),Types.PROP((Types.T_BOOL({}),NONE),false),SOME(st_2));
    case (env,Absyn.CREF_IDENT("solve",_),{aexp1,aexp2,acref},{},impl,SOME(st))
      local Interactive.InteractiveSymbolTable st_1;
      equation 
        (exp1_1,prop1,SOME(st_1)) = elab_exp(env, aexp1, impl, SOME(st));
        (exp2_1,prop2,SOME(st_2)) = elab_exp(env, aexp2, impl, SOME(st_1));
        (exp3_1,prop3,SOME(st_3)) = elab_exp(env, acref, impl, SOME(st_2)); then (Exp.CALL(Absyn.IDENT("solve"),{exp1_1,exp2_1,exp3_1},false,true),prop3,SOME(st_3));
  end matchcontinue;
end elab_call_interactive;

protected function elab_variablenames "* function: elab_variablenames
 ** This function elaborates variablenames to Exp.Exp. A variablename can
 ** be used in e.g. plot(model,{v1{3},v2.t}) It should only be used in interactive 
 ** functions that uses variablenames as componentreferences.
*"
  input Absyn_ExpList in_absyn_explist;
  output Exp_ExpList out_exp_explist;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
  type Exp_ExpList = list<Exp.Exp>;
algorithm 
  out_exp_explist:=
  matchcontinue (in_absyn_explist)
    local
      Exp.ComponentRef cr_1;
      Exp_ExpList xs_1;
      Absyn.ComponentRef cr;
      Absyn_ExpList xs;
    case {} then {}; 
    case (Absyn.CREF(cr) :: xs)
      equation 
        cr_1 = Exp.to_exp_cref(cr) " Use simplified conversion, all indexes integer constants ";
        xs_1 = elab_variablenames(xs); then Exp.CREF(cr_1,Exp.OTHER()) :: xs_1;
  end matchcontinue;
end elab_variablenames;

protected function get_optional_named_arg " The expected type "
  input Env.Env in_env;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  input Boolean in_boolean;
  input Ident in_ident;
  input Types.Type in_type;
  input Absyn_NamedArgList in_absyn_namedarglist;
  input Exp.Exp in_exp;
  output Exp.Exp out_exp;
protected 
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
  type Absyn_NamedArgList = list<Absyn.NamedArg>;
algorithm 
  out_exp:=
  matchcontinue (in_env,in_interactive_interactivesymboltableoption,in_boolean,in_ident,in_type,in_absyn_namedarglist,in_exp)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Env_FrameList = list<Env.Frame>;
      Exp.Exp exp,exp_1,exp_2,dexp;
      TupleTypes_TTypeAbsyn_PathOption t,tp;
      Boolean c1,impl;
      Env_FrameList env;
      Interactive_InteractiveSymbolTableOption st;
      Ident id,id2;
      Absyn_NamedArgList xs;
    case (_,_,_,_,_,{},exp) then exp; 
    case (env,st,impl,id,tp,Absyn.NAMEDARG(id2,exp) :: xs,dexp)
      local Absyn.Exp exp;
      equation 
        equality(id = id2);
        (exp_1,Types.PROP(t,c1),_) = elab_exp(env, exp, impl, st);
        (exp_2,_) = Types.match_type(exp_1, t, tp); then exp_2;
    case (env,st,impl,id,tp,Absyn.NAMEDARG(id2,exp) :: xs,dexp)
      local Absyn.Exp exp;
      equation 
        exp_1 = get_optional_named_arg(env, st, impl, id, tp, xs, dexp); then exp_1;
  end matchcontinue;
end get_optional_named_arg;

protected function elab_untyped_cref "* function: elab_untyped_cref
 ** This function elaborates a ComponentRef without adding type information. 
 **  Environment is passed along, such that constant subscripts can be elabed using existing
 ** functions
 *"
  input Env.Env in_env;
  input Absyn.ComponentRef in_componentref;
  input Boolean in_boolean;
  output Exp.ComponentRef out_componentref;
algorithm 
  out_componentref:=
  matchcontinue (in_env,in_componentref,in_boolean)
    local
      type Exp_SubscriptList = list<Exp.Subscript>;
      type Env_FrameList = list<Env.Frame>;
      type Absyn_SubscriptList = list<Absyn.Subscript>;
      Exp_SubscriptList subs_1;
      Env_FrameList env;
      Ident id;
      Absyn_SubscriptList subs;
      Boolean impl;
      Exp.ComponentRef cr_1;
      Absyn.ComponentRef cr;
    case (env,Absyn.CREF_IDENT(id,subs),impl) "impl"
      equation 
        (subs_1,_) = elab_subscripts(env, subs, impl); then Exp.CREF_IDENT(id,subs_1);
    case (env,Absyn.CREF_QUAL(id,subs,cr),impl)
      equation 
        (subs_1,_) = elab_subscripts(env, subs, impl);
        cr_1 = elab_untyped_cref(env, cr, impl); then Exp.CREF_QUAL(id,subs_1,cr_1);
  end matchcontinue;
end elab_untyped_cref;

protected function path_to_component_ref "* function: path_to_component_ref
 ** This function tranlates a typename to a variable name.
 *"
  input Absyn.Path in_path;
  output Exp.ComponentRef out_componentref;
algorithm 
  out_componentref:=
  matchcontinue (in_path)
    local
      Ident s,id;
      Exp.ComponentRef cref;
      Absyn.Path path;
    case (Absyn.IDENT(s)) then Exp.CREF_IDENT(s,{}); 
    case (Absyn.QUALIFIED(id,path))
      equation 
        cref = path_to_component_ref(path); then Exp.CREF_QUAL(id,{},cref);
  end matchcontinue;
end path_to_component_ref;

public function component_ref_to_path "* function: component_ref_to_path
 ** This function translates a variable name to a type name.
 *"
  input Exp.ComponentRef in_componentref;
  output Absyn.Path out_path;
algorithm 
  out_path:=
  matchcontinue (in_componentref)
    local
      Ident s,id;
      Absyn.Path path;
      Exp.ComponentRef cref;
    case (Exp.CREF_IDENT(s,{})) then Absyn.IDENT(s); 
    case (Exp.CREF_QUAL(id,_,cref))
      equation 
        path = component_ref_to_path(cref); then Absyn.QUALIFIED(id,path);
  end matchcontinue;
end component_ref_to_path;

protected function generate_compiled_function
  input Env.Env in_env;
  input Absyn.ComponentRef in_componentref;
  input Exp.Exp in_exp;
  input Types.Properties in_properties;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Interactive_InteractiveSymbolTableOption out_interactive_interactivesymboltableoption;
protected 
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  out_interactive_interactivesymboltableoption:=
  matchcontinue (in_env,in_componentref,in_exp,in_properties,in_interactive_interactivesymboltableoption)
    local
      type Env_FrameList = list<Env.Frame>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOption = tuple<Absyn.Path,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOptionList = list<TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOption>;
      type SCode_ClassList = list<SCode.Class>;
      type DAE_ElementList = list<DAE.Element>;
      type Interactive_InstantiatedClassList = list<Interactive.InstantiatedClass>;
      type Interactive_InteractiveVariableList = list<Interactive.InteractiveVariable>;
      Absyn.Path pfn,path;
      Env_FrameList env,env_1,env_2;
      Absyn.ComponentRef fn,cr;
      Exp.Exp e,exp;
      Types.Properties prop;
      Interactive.InteractiveSymbolTable st;
      Absyn.Program p;
      TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOptionList cflist;
      SCode_ClassList p_1,a;
      SCode.Class cls;
      DAE_ElementList d;
      DAE.DAElist d_1;
      Ident pathstr,filename,str1,str2;
      TupleTypes_TTypeAbsyn_PathOption t;
      Interactive_InstantiatedClassList b;
      Interactive_InteractiveVariableList c;
    case (env,fn,e,prop,SOME((st = Interactive.SYMBOLTABLE(p,_,_,_,cflist)))) "  axiom generate_compiled_function(_,_,_,_,NONE) => NONE "
      equation 
        Debug.fprintln("sei", "generate_compiled_function: start1") "  axiom generate_compiled_function(_,_,_,_,NONE) => NONE ";
        pfn = Absyn.cref_to_path(fn);
        true = is_function_in_cflist(cflist, pfn); then SOME(st);
    case (env,fn,e,prop,SOME((st = Interactive.SYMBOLTABLE(p,a,b,c,cflist))))
      equation 
        Debug.fprintln("sei", "generate_compiled_function: start2");
        path = Absyn.cref_to_path(fn);
        false = is_function_in_cflist(cflist, path);
        p_1 = SCode.elaborate(p);
        Debug.fprintln("sei", "generate_compiled_function: elaborated");
        (cls,env_1) = Lookup.lookup_class(env, path, false) "	Inst.instantiate_implicit(p') => d &  message";
        Debug.fprintln("sei", "generate_compiled_function: class looked up") "	Inst.instantiate_implicit(p') => d &  message";
        (env_2,d) = Inst.implicit_function_instantiation(env_1, Types.NOMOD(), Prefix.NOPRE, {}, cls, {}, false);
        Debug.fprintln("sei", "generate_compiled_function: function instantiated");
        Print.clear_buf();
        d_1 = ModUtil.string_prefix_params(DAE.DAE(d));
        Codegen.generate_functions(d_1);
        Debug.fprintln("sei", "generate_compiled_function: function generated");
        pathstr = ModUtil.path_string2(path, "_");
        filename = string_append(pathstr, ".c");
        Print.print_buf(
          "#include \"modelica.h\"\n#include <stdio.h>\n#include <stdlib.h>\n#include <errno.h>\n");
        Print.print_buf(
          "\nint main(int argc, char** argv)\n{\n\n  if (argc != 3)\n    {\n      fprintf(stderr,\"# Incorrrect number of arguments\\n\");\n      return 1;\n    }\n");
        Print.print_buf(pathstr);
        Print.print_buf("_read_call_write(argv[1],argv[2]);\n  return 0;\n}\n");
        Print.write_buf(filename);
        Print.clear_buf();
        System.compile_c_file(filename);
        t = Types.get_prop_type(prop) "	& Debug.fprintln(\"sei\", \"generate_compiled_function: compiled\")"; then SOME(Interactive.SYMBOLTABLE(p,a,b,c,(path,t) :: cflist));
    case (env,fn,e,prop,NONE) " PROP_TUPLE? "
      equation 
        Debug.fprintln("sei", "generate_compiled_function: start3"); then NONE;
    case (env,cr,exp,prop,st) "
  rule	(* If fails, skip it. *)
	--------------------------------------------------
	generate_compiled_function(_,_,_,_,st) => st
"
      local Interactive_InteractiveSymbolTableOption st;
      equation 
        Debug.fprintln("failtrace", "- generate_compiled_function failed4") " PROP_TUPLE? ";
        str1 = Dump.print_component_ref_str(cr);
        str2 = Exp.print_exp_str(exp);
        Debug.fprint("failtrace", str1);
        Debug.fprint("failtrace", " -- ");
        Debug.fprintln("failtrace", str2); then st;
    case (_,_,_,_,st)
      local Interactive_InteractiveSymbolTableOption st; then st;
  end matchcontinue;
end generate_compiled_function;

public function is_function_in_cflist " list of compiled functions
   function name "
  input TupleAbsyn_PathTypes_TypeList in_tupleabsyn_pathtypes_typelist;
  input Absyn.Path in_path;
  output Boolean out_boolean;
protected 
  type TupleAbsyn_PathTypes_Type = tuple<Absyn.Path,Types.Type>;
  type TupleAbsyn_PathTypes_TypeList = list<TupleAbsyn_PathTypes_Type>;
algorithm 
  out_boolean:=
  matchcontinue (in_tupleabsyn_pathtypes_typelist,in_path)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOption = tuple<Absyn.Path,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOptionList = list<TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOption>;
      Absyn.Path path1,path2;
      TupleTypes_TTypeAbsyn_PathOption ty;
      TupleAbsyn_PathTupleTypes_TTypeAbsyn_PathOptionList rest;
      Boolean res;
    case ({},_) then false; 
    case ((path1,ty) :: rest,path2)
      equation 
        true = ModUtil.path_equal(path1, path2); then true;
    case ((path1,ty) :: rest,path2)
      equation 
        false = ModUtil.path_equal(path1, path2);
        res = is_function_in_cflist(rest, path2); then res;
  end matchcontinue;
end is_function_in_cflist;

protected function elab_call_args " impl "
  input Env.Env in_env;
  input Absyn.Path in_path;
  input Absyn_ExpList in_absyn_explist;
  input Absyn_NamedArgList in_absyn_namedarglist;
  input Boolean in_boolean;
  input Interactive_InteractiveSymbolTableOption in_interactive_interactivesymboltableoption;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
  type Absyn_NamedArgList = list<Absyn.NamedArg>;
  type Interactive_InteractiveSymbolTableOption = Option<Interactive.InteractiveSymbolTable>;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_env,in_path,in_absyn_explist,in_absyn_namedarglist,in_boolean,in_interactive_interactivesymboltableoption)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleIdentTupleTypes_TTypeAbsyn_PathOption = tuple<Ident,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleIdentTupleTypes_TTypeAbsyn_PathOptionList = list<TupleIdentTupleTypes_TTypeAbsyn_PathOption>;
      type Env_FrameList = list<Env.Frame>;
      type SlotList = list<Slot>;
      type Exp_ExpList = list<Exp.Exp>;
      type BooleanList = list<Boolean>;
      type TupleTypes_TTypeAbsyn_PathOptionList = list<TupleTypes_TTypeAbsyn_PathOption>;
      TupleTypes_TTypeAbsyn_PathOption t,outtype,restype,functype;
      TupleIdentTupleTypes_TTypeAbsyn_PathOptionList fargs;
      Env_FrameList env_1,env_2,env;
      SlotList slots,newslots,newslots2;
      Exp_ExpList args_1,args_2;
      BooleanList constlist;
      Boolean const_,impl,tuple_,builtin;
      Types.Const tyconst;
      Types.Properties prop;
      SCode.Class cl;
      Absyn.Path fn,fn_1;
      Absyn_ExpList args;
      Absyn_NamedArgList nargs;
      Interactive_InteractiveSymbolTableOption st;
      TupleTypes_TTypeAbsyn_PathOptionList typelist,ktypelist;
      Ident s;
    case (env,fn,args,nargs,impl,st) " impl  tuple builtin"
      equation 
        (t = (Types.T_FUNCTION(fargs,outtype = (Types.T_COMPLEX(ClassInf.RECORD(_),_),_)),_),env_1) = Lookup.lookup_type(env, fn, true) " This rule finds record constructors, user defined or implicit ";
        slots = make_empty_slots(fargs);
        (args_1,newslots,constlist) = elab_input_args(env, args, nargs, slots, impl);
        const_ = Util.bool_and_list(constlist);
        tyconst = elab_consts(outtype, const_);
        prop = get_properties(outtype, tyconst);
        (cl,env_2) = Lookup.lookup_record_constructor_class(env_1, fn);
        newslots2 = fill_default_slots(newslots, cl, env_2, impl);
        args_2 = exp_list_from_slots(newslots2); then (Exp.CALL(fn,args_2,false,false),prop);
    case (env,fn,args,nargs,impl,st) " This rule finds the built-in functions   FIXME: LS: How do we know that? Anyway, if true, I've changed false   to true in Exp.CALL() where last flag stands for \"builtin\" "
      equation 
        typelist = Lookup.lookup_functions_in_env(env, fn) " PR. A function can have several types. Taking an array with
	 different dimensions as parameter for example. Because of this we
	 cannot just lookup the function name and trust that it
	 returns the correct function. It returns just one
	 functiontype of several possibilites. The solution is to send
	 in the funktion type of the user function and check both the
	 funktion name and the function's type. 
	 ";
        (args_1,constlist,restype,functype) = elab_types(env, args, typelist, impl) " The constness of a function depends on the inputs. If all inputs are
	 * constant the call itself is constant.
	 ";
        fn_1 = deoverload_funcname(fn, functype);
        tuple_ = is_tuple(restype);
        builtin = is_builtin_func(fn_1);
        const_ = Util.bool_and_list(constlist);
        tyconst = elab_consts(restype, const_);
        prop = get_properties(restype, tyconst); then (Exp.CALL(fn_1,args_1,tuple_,builtin),prop);
    case (env,fn,args,nargs,impl,st)
      equation 
        ktypelist = get_koening_function_types(env, fn, args, nargs, impl) " Rule above failed. Also consider koening lookup. ";
        (args_1,constlist,restype,functype) = elab_types(env, args, ktypelist, impl);
        fn_1 = deoverload_funcname(fn, functype);
        tuple_ = is_tuple(restype);
        builtin = is_builtin_func(fn_1);
        const_ = Util.bool_and_list(constlist);
        tyconst = elab_consts(restype, const_);
        prop = get_properties(restype, tyconst); then (Exp.CALL(fn_1,args_1,tuple_,builtin),prop);
    case (env,fn,args,nargs,impl,st)
      equation 
        failure((_,_) = Lookup.lookup_type(env, fn, false)) "msg";
        Print.print_error_buf("# Couldn't find function ") "msg";
        s = Absyn.path_string(fn);
        Print.print_error_buf(s);
        Print.print_error_buf("\n"); then fail();
    case (env,fn,args,nargs,impl,st)
      equation 
        Debug.fprint("failtrace", "- elab_call_args failed\n"); then fail();
  end matchcontinue;
end elab_call_args;

protected function deoverload_funcname "* function: deoverload_funcname
 ** This function is used to deoverload function calls. It investigates the type of the function
 ** to see if it has the optional functionname set. If so this is returned. 
 ** Otherwise return input.
*"
  input Absyn.Path in_path;
  input Types.Type in_type;
  output Absyn.Path out_path;
algorithm 
  out_path:=
  matchcontinue (in_path,in_type)
    local Absyn.Path fn,fn_1;
    case (fn,(Types.T_FUNCTION(_,_),SOME(fn_1))) then fn_1; 
    case (fn,(_,_)) then fn; 
  end matchcontinue;
end deoverload_funcname;

protected function is_tuple
  input Types.Type in_type;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_type)
    case ((Types.T_TUPLE(_),_)) then true; 
    case ((_,_)) then false; 
  end matchcontinue;
end is_tuple;

protected function elab_types "const
  result type
  function type"
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Types_TypeList in_types_typelist;
  input Boolean in_boolean;
  output Exp_ExpList out_exp_explist1;
  output BooleanList out_booleanlist2;
  output Types.Type out_type3;
  output Types.Type out_type4;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
  type Types_TypeList = list<Types.Type>;
  type Exp_ExpList = list<Exp.Exp>;
  type BooleanList = list<Boolean>;
algorithm 
  (out_exp_explist1,out_booleanlist2,out_type3,out_type4):=
  matchcontinue (in_env,in_absyn_explist,in_types_typelist,in_boolean)
    local
      type SlotList = list<Slot>;
      type Env_FrameList = list<Env.Frame>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleIdentTupleTypes_TTypeAbsyn_PathOption = tuple<Ident,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleIdentTupleTypes_TTypeAbsyn_PathOptionList = list<TupleIdentTupleTypes_TTypeAbsyn_PathOption>;
      type TupleTypes_TTypeAbsyn_PathOptionList = list<TupleTypes_TTypeAbsyn_PathOption>;
      SlotList slots,newslots;
      Exp_ExpList args_1;
      BooleanList clist;
      Env_FrameList env;
      Absyn_ExpList args;
      TupleTypes_TTypeAbsyn_PathOption t,restype;
      TupleIdentTupleTypes_TTypeAbsyn_PathOptionList params;
      TupleTypes_TTypeAbsyn_PathOptionList trest;
      Boolean impl;
    case (env,args,(t = (Types.T_FUNCTION(params,restype),_)) :: trest,impl) "impl const result type function type  
	 We found nothing. This is an error. return to the calling
	 function that writes an error message that makes sense (the
	 name of the funtion etc). 
	   
	 If trest gets empty then we will return with a missmatch.
	 and write an error message in the calling function. 
	   We found a match."
      equation 
        slots = make_empty_slots(params) "Debug.fprintln (\"sei\", \"We try the elab_input\") &
	Debug.fprintln (\"sei\", \"testing type:\") &
	Debug.fcall (\"sei\", Types.print_type,t) &";
        (args_1,newslots,clist) = elab_input_args(env, args, {}, slots, impl); then (args_1,clist,restype,t);
    case (env,args,(Types.T_FUNCTION(params,restype),_) :: trest,impl) " We did not found a match, try next. "
      equation 
        (args_1,clist,restype,t) = elab_types(env, args, trest, impl) "Debug.fprintln (\"sei\", \"The elab_input_args failed, try next.\") &"; then (args_1,clist,restype,t);
    case (env,_,_,_)
      equation 
        Debug.fprintln("failtrace", "- elab_types failed."); then fail();
  end matchcontinue;
end elab_types;

protected function get_properties "* function: get_properties
 ** This function creates a Properties object from a Types.Type and a Types.Const value
 *"
  input Types.Type in_type;
  input Types.Const in_const;
  output Types.Properties out_properties;
algorithm 
  out_properties:=
  matchcontinue (in_type,in_const)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      TupleTypes_TTypeAbsyn_PathOption tt,t,ty;
      Types.Const const_;
      Boolean b;
      Ident tystr,conststr;
    case ((tt = (Types.T_TUPLE(_),_)),const_) then Types.PROP_TUPLE(tt,const_); 
    case (t,Types.TUPLE_CONST(Types.CONST(b) :: {})) then Types.PROP(t,b); 
    case (t,Types.TUPLE_CONST(Types.CONST(b) :: {})) then Types.PROP(t,b); 
    case (t,Types.CONST(b)) then Types.PROP(t,b); 
    case (ty,const_)
      equation 
        Debug.fprint("failtrace", "- get_properties failed: ");
        tystr = Types.unparse_type(ty);
        conststr = Types.unparse_const(const_);
        Debug.fprint("failtrace", tystr);
        Debug.fprint("failtrace", ", ");
        Debug.fprintln("failtrace", conststr); then fail();
  end matchcontinue;
end get_properties;

protected function build_tuple_const "* LS:
 ** 
 ** Build a TUPLE_CONST (Types.Const) for a PROP_TUPLE for a function call
 ** from a list of bools derived from arguments
 **
 ** We should check functions actual arguments instead of their formal
 ** parameters as done below
 *"
  input BooleanList blist;
  output Types.Const out_const;
protected 
  type BooleanList = list<Boolean>;
  type Types_ConstList = list<Types.Const>;
  Types_ConstList clist;
algorithm 
  clist := build_tuple_const_list(blist);
  out_const := Types.TUPLE_CONST(clist);
end build_tuple_const;

protected function build_tuple_const_list
  input BooleanList in_booleanlist;
  output Types_ConstList out_types_constlist;
protected 
  type BooleanList = list<Boolean>;
  type Types_ConstList = list<Types.Const>;
algorithm 
  out_types_constlist:=
  matchcontinue (in_booleanlist)
    local
      Types_ConstList restlist;
      Boolean c;
      BooleanList crest;
    case {} then {}; 
    case c :: crest
      equation 
        restlist = build_tuple_const_list(crest); then Types.CONST(c) :: restlist;
  end matchcontinue;
end build_tuple_const_list;

protected function elab_consts "* PR. assisting routines
 ** This just splits the properties list into a type list and a const list. 
 *
   LS: Changed to take a Type, which is the functions return type 
   LS: Update: const is derived from the input arguments and sent here "
  input Types.Type in_type;
  input Boolean in_boolean;
  output Types.Const out_const;
algorithm 
  out_const:=
  matchcontinue (in_type,in_boolean)
    local
      type Types_ConstList = list<Types.Const>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleTypes_TTypeAbsyn_PathOptionList = list<TupleTypes_TTypeAbsyn_PathOption>;
      Types_ConstList consts;
      TupleTypes_TTypeAbsyn_PathOptionList tys;
      Boolean c;
      TupleTypes_TTypeAbsyn_PathOption ty;
    case ((Types.T_TUPLE(tys),_),c) "const"
      equation 
        consts = check_consts(tys, c); then Types.TUPLE_CONST(consts);
    case (ty,c) " LS: If not a tuple then one normal type, T_INTEGER etc, but we make a list of types
     with one element and call the same check_consts, so that we always have Types.TUPLE_CONST as result
   "
      equation 
        consts = check_consts({ty}, c); then Types.TUPLE_CONST(consts);
  end matchcontinue;
end elab_consts;

protected function check_consts " LS: Changed to take a Type list, which is the functions return type. Only
   for functions returning a tuple 
   LS: Update: const is derived from the input arguments and sent here "
  input Types_TypeList in_types_typelist;
  input Boolean in_boolean;
  output Types_ConstList out_types_constlist;
protected 
  type Types_TypeList = list<Types.Type>;
  type Types_ConstList = list<Types.Const>;
algorithm 
  out_types_constlist:=
  matchcontinue (in_types_typelist,in_boolean)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleTypes_TTypeAbsyn_PathOptionList = list<TupleTypes_TTypeAbsyn_PathOption>;
      Types.Const c;
      Types_ConstList rest_1;
      TupleTypes_TTypeAbsyn_PathOption a;
      TupleTypes_TTypeAbsyn_PathOptionList rest;
      Boolean const_;
    case ({},_) then {}; 
    case (a :: rest,const_)
      equation 
        c = check_const(a, const_);
        rest_1 = check_consts(rest, const_); then c :: rest_1;
  end matchcontinue;
end check_consts;

protected function check_const "PR. !! At the moment this make all outputs non cons.
* All ouputs sholud be checked in the function body for constness. 

   LS: but it says true? 
   LS: Adapted to check one type instead of funcarg, since it just checks return type 
   LS: Update: const is derived from the input arguments and sent here "
  input Types.Type in_type;
  input Boolean in_boolean;
  output Types.Const out_const;
algorithm 
  out_const:=
  matchcontinue (in_type,in_boolean)
    local Boolean c;
    case ((Types.T_TUPLE(_),_),c) "const"
      equation 
        Print.print_error_buf("\n Error. No support for tuples built by tuples. Yet.") "const"; then fail();
    case ((_,_),c) then Types.CONST(c); 
  end matchcontinue;
end check_const;

protected function split_props " Splits the properties list into the separated types list and const list. "
  input Types_PropertiesList in_types_propertieslist;
  output Types_TypeList out_types_typelist;
  output Types_ConstList out_types_constlist;
protected 
  type Types_PropertiesList = list<Types.Properties>;
  type Types_TypeList = list<Types.Type>;
  type Types_ConstList = list<Types.Const>;
algorithm 
  (out_types_typelist,out_types_constlist):=
  matchcontinue (in_types_propertieslist)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleTypes_TTypeAbsyn_PathOptionList = list<TupleTypes_TTypeAbsyn_PathOption>;
      TupleTypes_TTypeAbsyn_PathOptionList types;
      Types_ConstList consts;
      TupleTypes_TTypeAbsyn_PathOption t;
      Boolean c;
      Types_PropertiesList props;
      Types.Const t_c;
    case (Types.PROP(t,c) :: props)
      equation 
        (types,consts) = split_props(props) "list_append(ts,t::{}) => t1 &
	list_append(cs,Types.CONST(c)::{}) => t2 &
"; then (t :: types,Types.CONST(c) :: consts);
    case (Types.PROP_TUPLE(t,t_c) :: props)
      equation 
        (types,consts) = split_props(props) "list_append(ts,t::{}) => ts' &  list_append(cs, t_c::{}) => cs' & 
"; then (t :: types,t_c :: consts);
    case ({}) then ({},{}); 
  end matchcontinue;
end split_props;

protected function get_types "* function: get_types
 ** This relatoin returns the types of a Types.FuncArg list.
 *"
  input Types_FuncArgList in_types_funcarglist;
  output Types_TypeList out_types_typelist;
protected 
  type Types_FuncArgList = list<Types.FuncArg>;
  type Types_TypeList = list<Types.Type>;
algorithm 
  out_types_typelist:=
  matchcontinue (in_types_funcarglist)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleTypes_TTypeAbsyn_PathOptionList = list<TupleTypes_TTypeAbsyn_PathOption>;
      type TupleIdentTupleTypes_TTypeAbsyn_PathOption = tuple<Ident,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleIdentTupleTypes_TTypeAbsyn_PathOptionList = list<TupleIdentTupleTypes_TTypeAbsyn_PathOption>;
      TupleTypes_TTypeAbsyn_PathOptionList types;
      Ident n;
      TupleTypes_TTypeAbsyn_PathOption t;
      TupleIdentTupleTypes_TTypeAbsyn_PathOptionList rest;
    case ((n,t) :: rest)
      equation 
        types = get_types(rest) " print(\"\nDebug: Got a type for output of function. \") & "; then t :: types;
    case ({}) then {}; 
  end matchcontinue;
end get_types;

protected function function_params
  input Types_VarList in_types_varlist;
  output Types_FuncArgList out_types_funcarglist1;
  output Types_FuncArgList out_types_funcarglist2;
protected 
  type Types_VarList = list<Types.Var>;
  type Types_FuncArgList = list<Types.FuncArg>;
algorithm 
  (out_types_funcarglist1,out_types_funcarglist2):=
  matchcontinue (in_types_varlist)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleIdentTupleTypes_TTypeAbsyn_PathOption = tuple<Ident,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleIdentTupleTypes_TTypeAbsyn_PathOptionList = list<TupleIdentTupleTypes_TTypeAbsyn_PathOption>;
      TupleIdentTupleTypes_TTypeAbsyn_PathOptionList in_,out;
      Types_VarList vs;
      Ident n;
      TupleTypes_TTypeAbsyn_PathOption t;
      Types.Var v;
    case {} then ({},{}); 
    case (Types.VAR(_,_,true,_,_) :: vs) " Ignore protected components "
      equation 
        (in_,out) = function_params(vs) "Debug.print(\"protected\") &"; then (in_,out);
    case (Types.VAR(n,Types.ATTR(_,_,_,Absyn.INPUT()),false,t,Types.UNBOUND()) :: vs)
      equation 
        (in_,out) = function_params(vs) "Debug.print(\"not protected. intput\") &"; then ((n,t) :: in_,out);
    case (Types.VAR(n,Types.ATTR(_,_,_,Absyn.OUTPUT()),false,t,Types.UNBOUND()) :: vs)
      equation 
        (in_,out) = function_params(vs) "Debug.print(\"not protected. output\") &"; then (in_,(n,t) :: out);
    case ((v = Types.VAR(n,Types.ATTR(_,_,_,Absyn.BIDIR()),_,_,_)) :: vs)
      equation 
        Print.print_error_buf("# Components in functions must be INPUT or OUTPUT\n");
        Print.print_error_buf("    component: ");
        Print.print_error_buf(n);
        Print.print_error_buf("\n"); then fail();
    case _
      equation 
        Print.print_error_buf("- function_params failed\n"); then fail();
  end matchcontinue;
end function_params;

protected function elab_input_args " impl 
  const"
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Absyn_NamedArgList in_absyn_namedarglist;
  input SlotList in_slotlist;
  input Boolean in_boolean;
  output Exp_ExpList out_exp_explist;
  output SlotList out_slotlist;
  output BooleanList out_booleanlist;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
  type Absyn_NamedArgList = list<Absyn.NamedArg>;
  type SlotList = list<Slot>;
  type Exp_ExpList = list<Exp.Exp>;
  type BooleanList = list<Boolean>;
algorithm 
  (out_exp_explist,out_slotlist,out_booleanlist):=
  matchcontinue (in_env,in_absyn_explist,in_absyn_namedarglist,in_slotlist,in_boolean)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleIdentTupleTypes_TTypeAbsyn_PathOption = tuple<Ident,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleIdentTupleTypes_TTypeAbsyn_PathOptionList = list<TupleIdentTupleTypes_TTypeAbsyn_PathOption>;
      type Env_FrameList = list<Env.Frame>;
      TupleIdentTupleTypes_TTypeAbsyn_PathOptionList farg;
      SlotList slots_1,newslots,slots;
      BooleanList clist1,clist2,clist;
      Exp_ExpList explst,newexp;
      Env_FrameList env;
      Absyn_ExpList exp;
      Absyn_NamedArgList narg;
      Boolean impl;
    case (env,(exp = _ :: _),narg,slots,impl) " impl  const"
      equation 
        farg = funcarg_lst_from_slots(slots) "Debug.fprint (\"sei\",\"elab_input_args, positional arguments\n\") &";
        (slots_1,clist1) = elab_positional_input_args(env, exp, farg, slots, impl);
        (_,newslots,clist2) = elab_input_args(env, {}, narg, slots_1, impl);
        clist = list_append(clist1, clist2);
        explst = exp_list_from_slots(newslots); then (explst,newslots,clist);
    case (env,{},(narg = _ :: _),slots,impl)
      equation 
        farg = funcarg_lst_from_slots(slots) "Debug.fprint (\"sei\",\"elab_input_args, named arguments\n\") &";
        (newslots,clist) = elab_named_input_args(env, narg, farg, slots, impl) "Debug.fprint (\"sei\",\"elab_input_args, got funcarg\n\") &";
        newexp = exp_list_from_slots(newslots) "& Debug.fprint (\"sei\",\"elab_input_args, got expressions: \") &
 	Debug.fprint_list (\"sei\",newexp,Exp.print_exp,\", \")"; then (newexp,newslots,clist);
    case (env,{},{},slots,impl) then ({},slots,{}); 
  end matchcontinue;
end elab_input_args;

protected function make_empty_slots "* function: make_empty_slots
 ** Helper relatoin to elab_input_args.
 ** Creates the slots to be filled with arguments. Intially they are empty.
 *"
  input Types_FuncArgList in_types_funcarglist;
  output SlotList out_slotlist;
protected 
  type Types_FuncArgList = list<Types.FuncArg>;
  type SlotList = list<Slot>;
algorithm 
  out_slotlist:=
  matchcontinue (in_types_funcarglist)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleIdentTupleTypes_TTypeAbsyn_PathOption = tuple<Ident,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleIdentTupleTypes_TTypeAbsyn_PathOptionList = list<TupleIdentTupleTypes_TTypeAbsyn_PathOption>;
      SlotList ss;
      TupleIdentTupleTypes_TTypeAbsyn_PathOption fa;
      TupleIdentTupleTypes_TTypeAbsyn_PathOptionList fs;
    case ({}) then {}; 
    case (fa :: fs)
      equation 
        ss = make_empty_slots(fs); then SLOT(fa,false,NONE) :: ss;
  end matchcontinue;
end make_empty_slots;

protected function funcarg_lst_from_slots "* function: funcarg_lst_from_slots
 ** Converts slots to Types.Funcarg
 *"
  input SlotList in_slotlist;
  output Types_FuncArgList out_types_funcarglist;
protected 
  type SlotList = list<Slot>;
  type Types_FuncArgList = list<Types.FuncArg>;
algorithm 
  out_types_funcarglist:=
  matchcontinue (in_slotlist)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleIdentTupleTypes_TTypeAbsyn_PathOption = tuple<Ident,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleIdentTupleTypes_TTypeAbsyn_PathOptionList = list<TupleIdentTupleTypes_TTypeAbsyn_PathOption>;
      TupleIdentTupleTypes_TTypeAbsyn_PathOptionList fs;
      TupleIdentTupleTypes_TTypeAbsyn_PathOption fa;
      SlotList xs;
    case {} then {}; 
    case (SLOT(fa,_,_) :: xs)
      equation 
        fs = funcarg_lst_from_slots(xs); then fa :: fs;
  end matchcontinue;
end funcarg_lst_from_slots;

protected function exp_list_from_slots "* function exp_list_from_slots
 ** Convers slots to expressions 
 *"
  input SlotList in_slotlist;
  output Exp_ExpList out_exp_explist;
protected 
  type SlotList = list<Slot>;
  type Exp_ExpList = list<Exp.Exp>;
algorithm 
  out_exp_explist:=
  matchcontinue (in_slotlist)
    local
      Exp_ExpList lst;
      Exp.Exp e;
      SlotList xs;
    case {} then {}; 
    case (SLOT(_,_,SOME(e)) :: xs)
      equation 
        lst = exp_list_from_slots(xs); then e :: lst;
    case (SLOT(_,_,NONE) :: xs)
      equation 
        lst = exp_list_from_slots(xs); then lst;
  end matchcontinue;
end exp_list_from_slots;

protected function fill_default_slots "* function: fill_default_slots
 ** This function takes a slot list and a class definition of a function and fills 
 ** default values into slots which have not been filled.
 *"
  input SlotList in_slotlist;
  input SCode.Class in_class;
  input Env.Env in_env;
  input Boolean in_boolean;
  output SlotList out_slotlist;
protected 
  type SlotList = list<Slot>;
algorithm 
  out_slotlist:=
  matchcontinue (in_slotlist,in_class,in_env,in_boolean)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleIdentTupleTypes_TTypeAbsyn_PathOption = tuple<Ident,TupleTypes_TTypeAbsyn_PathOption>;
      type Env_FrameList = list<Env.Frame>;
      SlotList res,xs;
      TupleIdentTupleTypes_TTypeAbsyn_PathOption fa;
      Exp_ExpOption e;
      SCode.Class class_;
      Env_FrameList env;
      Boolean impl,c1;
      Absyn.Exp dexp;
      Exp.Exp exp,exp_1;
      TupleTypes_TTypeAbsyn_PathOption t,tp;
      Ident id;
    case (SLOT(fa,true,e) :: xs,class_,env,impl) "impl"
      equation 
        res = fill_default_slots(xs, class_, env, impl); then SLOT(fa,true,e) :: res;
    case (SLOT((id,tp),false,e) :: xs,class_,env,impl)
      equation 
        res = fill_default_slots(xs, class_, env, impl);
        SCode.COMPONENT(_,_,_,_,_,_,SCode.MOD(_,_,SOME(dexp)),_,_) = SCode.get_element_named(id, class_);
        (exp,Types.PROP(t,c1),_) = elab_exp(env, dexp, impl, NONE);
        (exp_1,_) = Types.match_type(exp, t, tp); then SLOT((id,tp),true,SOME(exp_1)) :: res;
    case (SLOT((id,tp),false,e) :: xs,class_,env,impl)
      equation 
        res = fill_default_slots(xs, class_, env, impl);
        Print.print_error_buf("#Error filling slot, id =");
        Print.print_error_buf(id);
        Print.print_error_buf("\n"); then SLOT((id,tp),true,e) :: xs;
    case ({},_,_,_) then {}; 
  end matchcontinue;
end fill_default_slots;

protected function print_slots "* function print_slots
 ** prints the slots on Print buffer
 *"
  input SlotList in_slotlist;
protected 
  type SlotList = list<Slot>;
algorithm 
  _:=
  matchcontinue (in_slotlist)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleIdentTupleTypes_TTypeAbsyn_PathOption = tuple<Ident,TupleTypes_TTypeAbsyn_PathOption>;
      Ident str;
      TupleIdentTupleTypes_TTypeAbsyn_PathOption farg;
      Boolean filled;
      Exp_ExpOption exp;
      SlotList xs;
    case (SLOT(farg,filled,exp) :: xs)
      equation 
        Print.print_buf("SLOT(");
        Types.print_farg(farg);
        Print.print_buf(", ");
        Dump.print_select(filled, "filled", "not filled");
        Print.print_buf(", ");
        str = Dump.get_option_str(exp, Exp.print_exp_str);
        Print.print_buf(str);
        Print.print_buf(")\n");
        print_slots(xs); then ();
    case ({}) then (); 
  end matchcontinue;
end print_slots;

protected function elab_positional_input_args " impl 
  const"
  input Env.Env in_env;
  input Absyn_ExpList in_absyn_explist;
  input Types_FuncArgList in_types_funcarglist;
  input SlotList in_slotlist;
  input Boolean in_boolean;
  output SlotList out_slotlist;
  output BooleanList out_booleanlist;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
  type Types_FuncArgList = list<Types.FuncArg>;
  type SlotList = list<Slot>;
  type BooleanList = list<Boolean>;
algorithm 
  (out_slotlist,out_booleanlist):=
  matchcontinue (in_env,in_absyn_explist,in_types_funcarglist,in_slotlist,in_boolean)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Env_FrameList = list<Env.Frame>;
      type TupleIdentTupleTypes_TTypeAbsyn_PathOption = tuple<Ident,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleIdentTupleTypes_TTypeAbsyn_PathOptionList = list<TupleIdentTupleTypes_TTypeAbsyn_PathOption>;
      SlotList slots,slots_1,newslots;
      Boolean impl,c1;
      Exp.Exp e_1,e_2;
      TupleTypes_TTypeAbsyn_PathOption t,vt;
      BooleanList clist;
      Env_FrameList env;
      Absyn.Exp e;
      Absyn_ExpList es;
      TupleIdentTupleTypes_TTypeAbsyn_PathOption farg;
      TupleIdentTupleTypes_TTypeAbsyn_PathOptionList vs;
    case (_,{},_,slots,impl) then (slots,{}); 
    case (env,e :: es,(farg = (_,vt)) :: vs,slots,impl)
      equation 
        (e_1,Types.PROP(t,c1),_) = elab_exp(env, e, impl, NONE);
        (e_2,_) = Types.match_type(e_1, t, vt);
        (slots_1,clist) = elab_positional_input_args(env, es, vs, slots, impl);
        newslots = fill_slot(farg, e_2, slots_1); then (newslots,c1 :: clist);
  end matchcontinue;
end elab_positional_input_args;

protected function elab_named_input_args "impl
  const"
  input Env.Env in_env;
  input Absyn_NamedArgList in_absyn_namedarglist;
  input Types_FuncArgList in_types_funcarglist;
  input SlotList in_slotlist;
  input Boolean in_boolean;
  output SlotList out_slotlist;
  output BooleanList out_booleanlist;
protected 
  type Absyn_NamedArgList = list<Absyn.NamedArg>;
  type Types_FuncArgList = list<Types.FuncArg>;
  type SlotList = list<Slot>;
  type BooleanList = list<Boolean>;
algorithm 
  (out_slotlist,out_booleanlist):=
  matchcontinue (in_env,in_absyn_namedarglist,in_types_funcarglist,in_slotlist,in_boolean)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Env_FrameList = list<Env.Frame>;
      type TupleIdentTupleTypes_TTypeAbsyn_PathOption = tuple<Ident,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleIdentTupleTypes_TTypeAbsyn_PathOptionList = list<TupleIdentTupleTypes_TTypeAbsyn_PathOption>;
      Exp.Exp e_1,e_2;
      TupleTypes_TTypeAbsyn_PathOption t,vt;
      Boolean c1,impl;
      SlotList slots_1,newslots,slots;
      BooleanList clist;
      Env_FrameList env;
      Ident id;
      Absyn.Exp e;
      Absyn_NamedArgList nas,narg;
      TupleIdentTupleTypes_TTypeAbsyn_PathOptionList farg;
    case (env,Absyn.NAMEDARG(id,e) :: nas,farg,slots,impl) "impl const   rule	check_slots_filled(env,slots) 
	----------------------------
	elab_named_input_args(env,{},farg,slots) => ({},slots,true) "
      equation 
        (e_1,Types.PROP(t,c1),_) = elab_exp(env, e, impl, NONE);
        vt = find_named_arg_type(id, farg);
        (e_2,_) = Types.match_type(e_1, t, vt);
        slots_1 = fill_slot((id,vt), e_2, slots);
        (newslots,clist) = elab_named_input_args(env, nas, farg, slots_1, impl); then (newslots,c1 :: clist);
    case (_,{},_,slots,impl) then (slots,{}); 
    case (env,narg,farg,_,impl)
      equation 
        Debug.fprint("failtrace", "- elab_named_input_args failed\n"); then fail();
  end matchcontinue;
end elab_named_input_args;

protected function find_named_arg_type "* function find_named_arg_type
 ** This function takes an Ident and a FuncArg list, and returns the FuncArg which has 
 ** that identifier.
 ** Used for instance when looking up named arguments from the function type.
 *"
  input Ident in_ident;
  input Types_FuncArgList in_types_funcarglist;
  output Types.Type out_type;
protected 
  type Types_FuncArgList = list<Types.FuncArg>;
algorithm 
  out_type:=
  matchcontinue (in_ident,in_types_funcarglist)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleIdentTupleTypes_TTypeAbsyn_PathOption = tuple<Ident,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleIdentTupleTypes_TTypeAbsyn_PathOptionList = list<TupleIdentTupleTypes_TTypeAbsyn_PathOption>;
      Ident id,id2;
      TupleTypes_TTypeAbsyn_PathOption farg;
      TupleIdentTupleTypes_TTypeAbsyn_PathOptionList ts;
    case (id,(id2,farg) :: ts)
      equation 
        equality(id = id2); then farg;
    case (id,(farg = (id2,_)) :: ts)
      equation 
        failure(equality(id = id2));
        farg = find_named_arg_type(id, ts); then farg;
  end matchcontinue;
end find_named_arg_type;

protected function fill_slot "* function: fill_slot
 ** This function takses a `FuncArg' and an Exp.Exp and a Slot list and fills the slot holding the FuncArg, by setting
 ** the boolean value of the slot and setting the expression. The function fails if the slot is allready set.
 *"
  input Types.FuncArg in_funcarg;
  input Exp.Exp in_exp;
  input SlotList in_slotlist;
  output SlotList out_slotlist;
protected 
  type SlotList = list<Slot>;
algorithm 
  out_slotlist:=
  matchcontinue (in_funcarg,in_exp,in_slotlist)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleIdentTupleTypes_TTypeAbsyn_PathOption = tuple<Ident,TupleTypes_TTypeAbsyn_PathOption>;
      Ident fa1,fa2;
      Exp.Exp exp;
      TupleTypes_TTypeAbsyn_PathOption b;
      SlotList xs,newslots;
      TupleIdentTupleTypes_TTypeAbsyn_PathOption farg;
      Slot s1;
    case ((fa1,_),exp,SLOT((fa2,b),false,_) :: xs)
      equation 
        equality(fa1 = fa2); then SLOT((fa2,b),true,SOME(exp)) :: xs;
    case ((fa1,_),exp,SLOT((fa2,b),true,_) :: xs)
      equation 
        equality(fa1 = fa2);
        Print.print_error_buf("#Error, slot in functional argument allready filled.\b"); then fail();
    case ((farg = (fa1,_)),exp,(s1 = SLOT((fa2,_),_,_)) :: xs)
      equation 
        failure(equality(fa1 = fa2));
        newslots = fill_slot(farg, exp, xs); then s1 :: newslots;
    case (_,_,_)
      equation 
        Print.print_error_buf("#Error, slot not found in function type.\n"); then fail();
  end matchcontinue;
end fill_slot;

public function elab_cref
  input Env.Env in_env;
  input Absyn.ComponentRef in_componentref;
  input Boolean in_boolean;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
  output SCode.Accessibility out_accessibility;
algorithm 
  (out_exp,out_properties,out_accessibility):=
  matchcontinue (in_env,in_componentref,in_boolean)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Env_FrameList = list<Env.Frame>;
      Exp.ComponentRef c_1;
      Boolean const_,impl;
      SCode.Accessibility acc,acc_1;
      SCode.Variability variability;
      TupleTypes_TTypeAbsyn_PathOption t;
      Types.Binding binding;
      Exp.Exp exp;
      Env_FrameList env;
      Absyn.ComponentRef c;
      Ident s;
    case (env,c,impl) "impl"
      equation 
        (c_1,const_) = elab_cref_subs(env, c, impl);
        (Types.ATTR(_,acc,variability,_),t,binding) = Lookup.lookup_var(env, c_1);
        (exp,const_,acc_1) = elab_cref2(env, c_1, acc, variability, t, binding); then (exp,Types.PROP(t,const_),acc_1);
    case (env,c,impl)
      equation 
        (c_1,const_) = elab_cref_subs(env, c, impl);
        Print.print_error_buf("# Unknown component: ");
        s = Dump.print_component_ref_str(c);
        Print.print_error_buf(s);
        Print.print_error_buf("\n"); then fail();
    case (env,c,impl)
      equation 
        Debug.fprint("failtrace", "- elab_cref failed: ");
        Debug.fcall("failtrace", Dump.print_component_ref, c);
        Debug.fprint("failtrace", "\n"); then fail();
  end matchcontinue;
end elab_cref;

protected function fill_cref_subscripts
  input Exp.ComponentRef in_componentref;
  input Types.Type in_type;
  output Exp.ComponentRef out_componentref;
algorithm 
  out_componentref:=
  matchcontinue (in_componentref,in_type)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Exp_SubscriptList = list<Exp.Subscript>;
      Exp.ComponentRef e,cref_1,cref;
      TupleTypes_TTypeAbsyn_PathOption t;
      Exp_SubscriptList subs_1,subs;
      Ident id;
    case ((e = Exp.CREF_IDENT(_,{})),t) then e; 
    case (Exp.CREF_IDENT(id,subs),t)
      equation 
        subs_1 = fill_subscripts(subs, t); then Exp.CREF_IDENT(id,subs_1);
    case (Exp.CREF_QUAL(id,subs,cref),t)
      equation 
        cref_1 = fill_cref_subscripts(cref, t); then Exp.CREF_QUAL(id,subs,cref_1);
  end matchcontinue;
end fill_cref_subscripts;

protected function fill_subscripts
  input Exp_SubscriptList in_exp_subscriptlist;
  input Types.Type in_type;
  output Exp_SubscriptList out_exp_subscriptlist;
protected 
  type Exp_SubscriptList = list<Exp.Subscript>;
algorithm 
  out_exp_subscriptlist:=
  matchcontinue (in_exp_subscriptlist,in_type)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      Exp_SubscriptList subs_1,subs_2,subs;
      TupleTypes_TTypeAbsyn_PathOption t;
      Exp.Subscript fs;
    case ({},(Types.T_ARRAY(_,t),_))
      equation 
        subs_1 = fill_subscripts({}, t);
        subs_2 = list_append({Exp.WHOLEDIM()}, subs_1); then subs_2;
    case (fs :: subs,(Types.T_ARRAY(_,t),_))
      equation 
        subs_1 = fill_subscripts(subs, t); then fs :: subs_1;
    case (subs,_) then subs; 
  end matchcontinue;
end fill_subscripts;

protected function elab_cref2
  input Env.Env in_env;
  input Exp.ComponentRef in_componentref;
  input SCode.Accessibility in_accessibility;
  input SCode.Variability in_variability;
  input Types.Type in_type;
  input Types.Binding in_binding;
  output Exp.Exp out_exp;
  output Boolean out_boolean;
  output SCode.Accessibility out_accessibility;
algorithm 
  (out_exp,out_boolean,out_accessibility):=
  matchcontinue (in_env,in_componentref,in_accessibility,in_variability,in_type,in_binding)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Env_FrameList = list<Env.Frame>;
      Exp.Type t;
      Exp.ComponentRef cr_1,cr,cref;
      SCode.Accessibility acc,acc_1;
      TupleTypes_TTypeAbsyn_PathOption tt,t_1,tp;
      Exp.Exp e,e_1,exp;
      Values.Value v;
      Env_FrameList env;
      Boolean const_;
      SCode.Variability variability_1,variability,var;
      Types.Binding binding_1,bind;
      Ident s,str,ts,binds,vs;
    case (_,cr,acc,SCode.VAR(),tt,_)
      equation 
        t = Types.elab_type(tt);
        cr_1 = fill_cref_subscripts(cr, tt); then (Exp.CREF(cr_1,t),false,acc);
    case (_,cr,acc,SCode.DISCRETE(),tt,_)
      equation 
        t = Types.elab_type(tt);
        cr_1 = fill_cref_subscripts(cr, tt); then (Exp.CREF(cr_1,t),false,acc);
    case (_,cr,_,SCode.CONST(),t,Types.VALBOUND(v))
      local TupleTypes_TTypeAbsyn_PathOption t;
      equation 
        e = value_exp(v); then (e,true,SCode.RO());
    case (env,cr,acc,SCode.PARAM(),tt,Types.VALBOUND(v))
      equation 
        t = Types.elab_type(tt);
        cr_1 = fill_cref_subscripts(cr, tt);
        e_1 = cref_vectorize(Exp.CREF(cr_1,t), tt); then (e_1,false,acc);
    case (env,cr,acc,SCode.STRUCTPARAM(),tt,Types.VALBOUND(v))
      equation 
        t = Types.elab_type(tt);
        cr_1 = fill_cref_subscripts(cr, tt);
        e_1 = value_exp(v); then (e_1,true,acc);
    case (env,cr,acc,SCode.CONST(),tt,Types.EQBOUND(exp,const_))
      equation 
        t = Types.elab_type(tt) " Constants with equal binings should be constant, i.e. true
	 but const is passed on, allowing constants to have wrong bindings
	 This must be caught later on.";
        cr_1 = fill_cref_subscripts(cr, tt);
        e_1 = cref_vectorize(Exp.CREF(cr_1,t), tt); then (e_1,const_,acc);
    case (env,cr,acc,SCode.STRUCTPARAM(),tt,Types.EQBOUND(exp,const_))
      equation 
        t = Types.elab_type(tt) " ...the same goes for structural parameters.";
        cr_1 = fill_cref_subscripts(cr, tt);
        e_1 = cref_vectorize(Exp.CREF(cr_1,t), tt); then (e_1,const_,acc);
    case (env,cr,acc,_,tt,Types.EQBOUND(exp,const_))
      equation 
        t = Types.elab_type(tt) " ..the rest should be non constant, even if they have a 
	 constant binding.";
        cr_1 = fill_cref_subscripts(cr, tt);
        e_1 = cref_vectorize(Exp.CREF(cr_1,t), tt); then (e_1,false,acc);
    case (env,cr,acc,_,(tt = (Types.T_ENUM(),_)),_) " Enum constants does not have a value expression "
      equation 
        t = Types.elab_type(tt); then (Exp.CREF(cr,t),true,acc);
    case (env,cr,acc,variability,tp,Types.EQBOUND(Exp.CREF(cref,t),false))
      equation 
        (Types.ATTR(_,acc_1,variability_1,_),t_1,binding_1) = Lookup.lookup_var(env, cref) " If value not constant, but references another parameter, which has a value   We need to perform value propagation. ";
        (e,const_,acc) = elab_cref2(env, cref, acc_1, variability_1, t_1, binding_1); then (e,const_,acc);
    case (_,cr,_,_,_,Types.EQBOUND(exp,false))
      equation 
        Print.print_error_buf("# Constant or parameter with a non-constant initializer\n");
        Print.print_error_buf("# component: ");
        s = Exp.print_component_ref_str(cr);
        Print.print_error_buf(" = ");
        str = Exp.print_exp_str(exp);
        Print.print_error_buf(str);
        Print.print_error_buf("\n"); then fail();
    case (env,cr,_,SCode.CONST(),_,Types.UNBOUND())
      equation 
        Print.print_error_buf("# Constant  without a value\n");
        Print.print_error_buf("# component: ");
        s = Exp.print_component_ref_str(cr);
        Print.print_error_buf(s);
        Print.print_error_buf("\n"); then fail();
    case (env,cr,_,SCode.PARAM(),_,Types.UNBOUND())
      equation 
        Print.print_error_buf("# Constant  without a value\n");
        Print.print_error_buf("# component: ");
        s = Exp.print_component_ref_str(cr);
        Print.print_error_buf(s);
        Print.print_error_buf("\n"); then fail();
    case (env,cr,acc,var,tp,bind)
      equation 
        print("- elab_cref2 failed (component: ");
        str = Exp.print_component_ref_str(cr);
        print(str);
        print("\n");
        Print.print_buf("- elab_cref2 failed (component: ");
        Exp.print_component_ref(cr);
        Print.print_buf(")\n");
        ts = Types.unparse_type(tp);
        print("type = ");
        print(ts);
        print(", ");
        binds = Types.print_binding_str(bind);
        print("binding =");
        print(binds);
        print("\n");
        vs = SCode.variability_string(var);
        print("var = ");
        print(vs);
        print("\n");
        Env.print_env(env); then fail();
  end matchcontinue;
end elab_cref2;

protected function cref_vectorize "* function: cref_vectorize
 ** This function takes a 'Exp.Exp' and a 'Types.Type' and if the expression
 ** is a ComponentRef and the type is an array it returns an array of 
 ** component references with subscripts for each index.
 ** For instance, parameter Real x{3};   
 ** gives cref_vectorize('x', <arraytype>) => '{x{1},x{2},x{3}}'
 ** This is needed since the DAE does not know what the variable 'x' is, it only
 ** knows the variables 'x{1}', 'x{2}' and 'x{3}'.
*"
  input Exp.Exp in_exp;
  input Types.Type in_type;
  output Exp.Exp out_exp;
algorithm 
  out_exp:=
  matchcontinue (in_exp,in_type)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      Exp.Exp e;
      Exp.ComponentRef cr;
      Exp.Type exptp;
      Integer ds;
      TupleTypes_TTypeAbsyn_PathOption t;
    case (Exp.CREF(cr,exptp),(Types.T_ARRAY(Types.DIM(SOME(ds)),t),_))
      equation 
        e = create_cref_array(cr, 1, ds, exptp, t); then e;
    case (e,_) then e; 
  end matchcontinue;
end cref_vectorize;

protected function create_cref_array " index iterator 
   dimension size "
  input Exp.ComponentRef in_componentref1;
  input Integer in_integer2;
  input Integer in_integer3;
  input Exp.Type in_type4;
  input Types.Type in_type5;
  output Exp.Exp out_exp;
algorithm 
  out_exp:=
  matchcontinue (in_componentref1,in_integer2,in_integer3,in_type4,in_type5)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Exp_ExpList = list<Exp.Exp>;
      Exp.ComponentRef cr,cr_1;
      Integer indx,ds,indx_1;
      Exp.Type et;
      TupleTypes_TTypeAbsyn_PathOption t;
      Exp_ExpList expl;
      Exp.Exp e_1;
    case (cr,indx,ds,et,t) " index iterator   dimension size "
      equation 
        (indx > ds) = true; then Exp.ARRAY(et,false,{});
    case (cr,indx,ds,et,t)
      equation 
        indx_1 = indx + 1;
        Exp.ARRAY(_,_,expl) = create_cref_array(cr, indx_1, ds, et, t);
        cr_1 = Exp.subscript_cref(cr, {Exp.INDEX(Exp.ICONST(indx))});
        e_1 = cref_vectorize(Exp.CREF(cr_1,et), t); then Exp.ARRAY(et,false,e_1 :: expl);
    case (cr,indx,ds,et,t)
      equation 
        Debug.fprint("failtrace", "create_cref_array failed\n"); then fail();
  end matchcontinue;
end create_cref_array;

protected function elab_cref_subs
  input Env.Env in_env;
  input Absyn.ComponentRef in_componentref;
  input Boolean in_boolean;
  output Exp.ComponentRef out_componentref;
  output Boolean out_boolean;
algorithm 
  (out_componentref,out_boolean):=
  matchcontinue (in_env,in_componentref,in_boolean)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type IntegerList = list<Integer>;
      type Exp_SubscriptList = list<Exp.Subscript>;
      type Env_FrameList = list<Env.Frame>;
      type Absyn_SubscriptList = list<Absyn.Subscript>;
      TupleTypes_TTypeAbsyn_PathOption t;
      IntegerList sl;
      Exp_SubscriptList ss_1;
      Boolean const_,impl,const1,const2;
      Env_FrameList env;
      Ident id;
      Absyn_SubscriptList ss;
      Exp.ComponentRef cr;
      Absyn.ComponentRef subs;
    case (env,Absyn.CREF_IDENT(id,ss),impl) "impl"
      equation 
        (_,t,_) = Lookup.lookup_var(env, Exp.CREF_IDENT(id,{}));
        sl = Types.get_dimension_sizes(t);
        (ss_1,const_) = elab_subscripts_dims(env, ss, sl, impl); then (Exp.CREF_IDENT(id,ss_1),const_);
    case (env,Absyn.CREF_IDENT(id,ss),impl)
      equation 
        (ss_1,const_) = elab_subscripts(env, ss, impl); then (Exp.CREF_IDENT(id,ss_1),const_);
    case (env,Absyn.CREF_QUAL(id,ss,subs),impl)
      equation 
        (ss_1,const1) = elab_subscripts(env, ss, impl);
        (cr,const2) = elab_cref_subs(env, subs, impl);
        const_ = bool_and(const1, const2); then (Exp.CREF_QUAL(id,ss_1,cr),const_);
    case (_,_,impl)
      equation 
        print("- elab_cref_subs failed\n"); then fail();
  end matchcontinue;
end elab_cref_subs;

public function elab_subscripts
  input Env.Env in_env;
  input Absyn_SubscriptList in_absyn_subscriptlist;
  input Boolean in_boolean;
  output Exp_SubscriptList out_exp_subscriptlist;
  output Boolean out_boolean;
protected 
  type Absyn_SubscriptList = list<Absyn.Subscript>;
  type Exp_SubscriptList = list<Exp.Subscript>;
algorithm 
  (out_exp_subscriptlist,out_boolean):=
  matchcontinue (in_env,in_absyn_subscriptlist,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      Exp.Subscript sub_1;
      Boolean const1,const2,impl;
      Exp_SubscriptList subs_1;
      Env_FrameList env;
      Absyn.Subscript sub;
      Absyn_SubscriptList subs;
    case (_,{},_) then ({},true); 
    case (env,sub :: subs,impl)
      equation 
        (sub_1,const1) = elab_subscript(env, sub, impl);
        (subs_1,const2) = elab_subscripts(env, subs, impl);
        true = bool_and(const1, const2) " HL &
	Ceval.ceval_subscripts(env, sub'::subs',impl) => ss "; then (sub_1 :: subs_1,true);
    case (env,sub :: subs,impl)
      equation 
        (sub_1,const1) = elab_subscript(env, sub, impl);
        (subs_1,const2) = elab_subscripts(env, subs, impl);
        false = bool_and(const1, const2); then (sub_1 :: subs_1,false);
  end matchcontinue;
end elab_subscripts;

protected function elab_subscripts_dims
  input Env.Env in_env;
  input Absyn_SubscriptList in_absyn_subscriptlist;
  input IntegerList in_integerlist;
  input Boolean in_boolean;
  output Exp_SubscriptList out_exp_subscriptlist;
  output Boolean out_boolean;
protected 
  type Absyn_SubscriptList = list<Absyn.Subscript>;
  type IntegerList = list<Integer>;
  type Exp_SubscriptList = list<Exp.Subscript>;
algorithm 
  (out_exp_subscriptlist,out_boolean):=
  matchcontinue (in_env,in_absyn_subscriptlist,in_integerlist,in_boolean)
    local
      type Env_FrameList = list<Env.Frame>;
      Exp.Subscript sub_1;
      Boolean const1,const2,impl;
      Exp_SubscriptList subs_1,ss;
      Env_FrameList env;
      Absyn.Subscript sub;
      Absyn_SubscriptList subs;
      Integer dim;
      IntegerList restdims;
    case (_,{},_,_) then ({},true); 
    case (env,sub :: subs,dim :: restdims,impl)
      equation 
        (sub_1,const1) = elab_subscript(env, sub, impl);
        (subs_1,const2) = elab_subscripts_dims(env, subs, restdims, impl);
        true = bool_and(const1, const2);
        ss = Ceval.ceval_subscripts(env, sub_1 :: subs_1, dim :: restdims, impl); then (ss,true);
    case (env,sub :: subs,dim :: restdims,impl)
      equation 
        (sub_1,const1) = elab_subscript(env, sub, impl);
        (subs_1,const2) = elab_subscripts_dims(env, subs, restdims, impl);
        false = bool_and(const1, const2); then (sub_1 :: subs_1,false);
  end matchcontinue;
end elab_subscripts_dims;

protected function elab_subscript
  input Env.Env in_env;
  input Absyn.Subscript in_subscript;
  input Boolean in_boolean;
  output Exp.Subscript out_subscript;
  output Boolean out_boolean;
algorithm 
  (out_subscript,out_boolean):=
  matchcontinue (in_env,in_subscript,in_boolean)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Env_FrameList = list<Env.Frame>;
      Boolean impl,const_;
      Exp.Exp sub_1;
      TupleTypes_TTypeAbsyn_PathOption ty;
      Exp.Subscript sub_2;
      Env_FrameList env;
      Absyn.Exp sub;
    case (_,Absyn.NOSUB(),impl) then (Exp.WHOLEDIM(),true); 
    case (env,Absyn.SUBSCRIPT(sub),impl)
      equation 
        (sub_1,Types.PROP(ty,const_),_) = elab_exp(env, sub, impl, NONE);
        sub_2 = elab_subscript_type(ty, sub, sub_1); then (sub_2,const_);
  end matchcontinue;
end elab_subscript;

protected function elab_subscript_type
  input Types.Type in_type1;
  input Absyn.Exp in_exp2;
  input Exp.Exp in_exp3;
  output Exp.Subscript out_subscript;
algorithm 
  out_subscript:=
  matchcontinue (in_type1,in_exp2,in_exp3)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      Exp.Exp sub;
      Ident s;
      TupleTypes_TTypeAbsyn_PathOption t;
      Absyn.Exp e;
    case ((Types.T_INTEGER(_),_),_,sub) then Exp.INDEX(sub); 
    case ((Types.T_ARRAY(_,(Types.T_INTEGER(_),_)),_),_,sub) then Exp.SLICE(sub); 
    case (t,e,_)
      equation 
        Print.print_error_buf("# Subscript is not an integer or integer array\n");
        Print.print_error_buf("    expression: ");
        s = Dump.print_exp_str(e);
        Print.print_error_buf(s);
        Print.print_error_buf(" :: ");
        s = Types.unparse_type(t);
        Print.print_error_buf(s);
        Print.print_error_buf("\n"); then fail();
  end matchcontinue;
end elab_subscript_type;

protected function subscript_cref_type "* function: subscript_cref_type
 **
 ** If a component of an array type is subscripted, the type of the
 ** component reference is of lower dimensionality than the
 ** component.  This function shows the function between the component
 ** type and the component reference expression type.
 **
 ** This function might actually not be needed.
 *"
  input Exp.Exp in_exp;
  input Types.Type in_type;
  output Types.Type out_type;
algorithm 
  out_type:=
  matchcontinue (in_exp,in_type)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      TupleTypes_TTypeAbsyn_PathOption t_1,t;
      Exp.ComponentRef c;
      Exp.Exp e;
    case (Exp.CREF(c,_),t)
      equation 
        t_1 = subscript_cref_type2(c, t); then t_1;
    case (e,t) then t; 
  end matchcontinue;
end subscript_cref_type;

protected function subscript_cref_type2
  input Exp.ComponentRef in_componentref;
  input Types.Type in_type;
  output Types.Type out_type;
algorithm 
  out_type:=
  matchcontinue (in_componentref,in_type)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Exp_SubscriptList = list<Exp.Subscript>;
      TupleTypes_TTypeAbsyn_PathOption t,t_1;
      Exp_SubscriptList subs;
      Exp.ComponentRef c;
    case (Exp.CREF_IDENT(_,{}),t) then t; 
    case (Exp.CREF_IDENT(_,subs),t)
      equation 
        t_1 = subscript_type(t, subs); then t_1;
    case (Exp.CREF_QUAL(_,_,c),t)
      equation 
        t_1 = subscript_cref_type2(c, t); then t_1;
  end matchcontinue;
end subscript_cref_type2;

protected function subscript_type
  input Types.Type in_type;
  input Exp_SubscriptList in_exp_subscriptlist;
  output Types.Type out_type;
protected 
  type Exp_SubscriptList = list<Exp.Subscript>;
algorithm 
  out_type:=
  matchcontinue (in_type,in_exp_subscriptlist)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      TupleTypes_TTypeAbsyn_PathOption t,t_1;
      Exp_SubscriptList subs;
      Types.ArrayDim dim;
      Absyn_PathOption p;
    case (t,{}) then t; 
    case ((Types.T_ARRAY(Types.DIM(_),t),_),Exp.INDEX(_) :: subs)
      equation 
        t_1 = subscript_type(t, subs); then t_1;
    case ((Types.T_ARRAY(dim,t),p),Exp.SLICE(_) :: subs)
      equation 
        t_1 = subscript_type(t, subs); then ((Types.T_ARRAY(dim,t_1),p));
    case ((Types.T_ARRAY(dim,t),p),Exp.WHOLEDIM() :: subs)
      equation 
        t_1 = subscript_type(t, subs); then ((Types.T_ARRAY(dim,t_1),p));
    case (t,_)
      equation 
        Print.print_buf("- subscript_type failed (");
        Types.print_type(t);
        Print.print_buf(" , [...])\n"); then fail();
  end matchcontinue;
end subscript_type;

protected function elab_ifexp
  input Exp.Exp in_exp1;
  input Types.Properties in_properties2;
  input Exp.Exp in_exp3;
  input Types.Properties in_properties4;
  input Exp.Exp in_exp5;
  input Types.Properties in_properties6;
  output Exp.Exp out_exp;
  output Types.Properties out_properties;
algorithm 
  (out_exp,out_properties):=
  matchcontinue (in_exp1,in_properties2,in_exp3,in_properties4,in_exp5,in_properties6)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      Boolean c,c1,c2,c3;
      Exp.Exp e1,e2,e3,e2_1,e3_1;
      TupleTypes_TTypeAbsyn_PathOption t2,t3,t2_1,t3_1,t1;
      Ident s;
    case (e1,Types.PROP((Types.T_BOOL(_),_),c1),e2,Types.PROP(t2,c2),e3,Types.PROP(t3,c3))
      equation 
        true = Types.equivtypes(t2, t3);
        c = const_ifexp(e1, c1, c2, c3); then (Exp.IFEXP(e1,e2,e3),Types.PROP(t2,c));
    case (e1,Types.PROP((Types.T_BOOL(_),_),c1),e2,Types.PROP(t2,c2),e3,Types.PROP(t3,c3))
      equation 
        (e2_1,t2_1) = Types.match_type(e2, t2, t3);
        c = const_ifexp(e1, c1, c2, c3) " then-part type converted to match else-part "; then (Exp.IFEXP(e1,e2_1,e3),Types.PROP(t2_1,c));
    case (e1,Types.PROP((Types.T_BOOL(_),_),c1),e2,Types.PROP(t2,c2),e3,Types.PROP(t3,c3))
      equation 
        (e3_1,t3_1) = Types.match_type(e3, t3, t2);
        c = const_ifexp(e1, c1, c2, c3) " else-part type converted to match then-part "; then (Exp.IFEXP(e1,e2,e3_1),Types.PROP(t2,c));
    case (e1,Types.PROP(t1,c1),e2,Types.PROP(t2,c2),e3,Types.PROP(t3,c3))
      equation 
        failure(equality(t1 = (Types.T_BOOL({}),NONE)));
        Print.print_error_buf("# conditional in if expression has to be boolean\n");
        Print.print_error_buf("    got type: ");
        s = Types.unparse_type(t1);
        Print.print_error_buf("\n");
        Print.print_error_buf("    expression: ");
        s = Exp.print_exp_str(e1);
        Print.print_error_buf(s);
        Print.print_error_buf("\n"); then fail();
    case (e1,Types.PROP((Types.T_BOOL(_),_),c1),e2,Types.PROP(t2,c2),e3,Types.PROP(t3,c3))
      equation 
        false = Types.equivtypes(t2, t3);
        Print.print_error_buf("# Type mismatch in if-expression\n");
        Print.print_error_buf("    true branch : ");
        s = Exp.print_exp_str(e2);
        Print.print_error_buf(s);
        Print.print_error_buf(" :: ");
        s = Types.unparse_type(t2);
        Print.print_error_buf(s);
        Print.print_error_buf("\n");
        Print.print_error_buf("    false branch: ");
        s = Exp.print_exp_str(e3);
        Print.print_error_buf(s);
        Print.print_error_buf(" :: ");
        s = Types.unparse_type(t3);
        Print.print_error_buf(s);
        Print.print_buf("\n"); then fail();
    case (_,_,_,_,_,_)
      equation 
        Print.print_buf("- elab_ifexp failed\n"); then fail();
  end matchcontinue;
end elab_ifexp;

protected function const_ifexp "* function: const_ifexp
 **
 ** Tests wether an `if' expression is constant.  This is done by
 ** first testing if the conditional is constant, and if so evaluating
 ** it to see which branch should be tested for constant-ness.
 **
 ** This will miss some occations where the expression actually is
 ** constant, as in the expression `if x then 1.0 else 1.0'.
 *"
  input Exp.Exp in_exp1;
  input Boolean in_boolean2;
  input Boolean in_boolean3;
  input Boolean in_boolean4;
  output Boolean out_boolean;
algorithm 
  out_boolean:=
  matchcontinue (in_exp1,in_boolean2,in_boolean3,in_boolean4)
    case (_,true,true,true) then true; 
    case (_,_,_,_) then false; 
  end matchcontinue;
end const_ifexp;

public function value_exp ""
  input Values.Value in_value;
  output Exp.Exp out_exp;
algorithm 
  out_exp:=
  matchcontinue (in_value)
    local
      type Exp_ExpList = list<Exp.Exp>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type Values_ValueList = list<Values.Value>;
      Integer x;
      Ident vs;
      Boolean a;
      Exp_ExpList explist;
      TupleTypes_TTypeAbsyn_PathOption vt;
      Exp.Type t;
      Values.Value v;
      Values_ValueList xs,vallist;
    case (Values.INTEGER(x)) then Exp.ICONST(x); 
    case (Values.REAL(x))
      local Real x; then Exp.RCONST(x);
    case (Values.STRING(x))
      local Ident x; then Exp.SCONST(x);
    case (Values.BOOL(x))
      local Boolean x; then Exp.BCONST(x);
    case (Values.ARRAY({})) then Exp.ARRAY(Exp.OTHER(),false,{}); 
    case (Values.ARRAY(x :: xs))
      local Values.Value x;
      equation 
        explist = Util.list_map(x :: xs, value_exp);
        vt = value_type(x);
        t = Types.elab_type(vt);
        a = Types.is_array(vt); then Exp.ARRAY(t,a,explist);
    case (Values.TUPLE(vallist))
      equation 
        explist = Util.list_map(vallist, value_exp); then Exp.TUPLE(explist);
    case v
      equation 
        Print.print_error_buf("- value_exp failed: ");
        vs = Values.val_string(v);
        Print.print_error_buf(vs);
        Print.print_error_buf("\n"); then fail();
  end matchcontinue;
end value_exp;

protected function value_type " FIXME: Already a function in Types called type_of_value 
  * function: value_type
 ** This function investigates a Value and return the Type of this value.
 *
   LS: Removed. Using Types.type_of_value instead 
  
relation value_type : Values.Value => Types.Type =

  axiom	value_type Values.INTEGER(x) => ((Types.T_INTEGER({}),NONE))
  axiom	value_type Values.REAL(x)    => ((Types.T_REAL({}),NONE))
  axiom value_type Values.STRING(x)  => ((Types.T_STRING({}),NONE))
  axiom value_type Values.BOOL(x)    => ((Types.T_BOOL({}),NONE))

  axiom	value_type Values.ENUM(x) => ((Types.T_ENUM,NONE))

  rule	value_type vlfirst => eltype &
	list_length(vlrest) => dim &
	int_add(dim,1) => dim'
	----------------------------
	value_type Values.ARRAY(vlfirst::vlrest) => ((Types.T_ARRAY(Types.DIM(SOME(dim')), eltype),NONE))

  rule	Util.list_map(vl, value_type) => tylist
	---------------------------------------
	value_type Values.TUPLE(vl) => ((Types.T_TUPLE(tylist),NONE))

  rule	Types.values_to_vars(vl,ids) => vars &
	Absyn.path_string(cname) => cname_str
	---------------------------------------
	value_type Values.RECORD(cname,vl,ids) => ((Types.T_COMPLEX(ClassInf.RECORD(cname_str),vars),NONE))

  rule	Print.print_error_buf \"- value_type failed: \" &	
	Values.val_string v => vs &
	Print.print_error_buf vs &
	Print.print_error_buf \"\n\"
	----------------------------
	value_type v => fail

end
"
  input Values.Value v;
  output Types.Type t;
algorithm 
  t := Types.type_of_value(v);
end value_type;

protected function canon_cref2 "* function: canon_cref
 **
 ** This function relates a `Exp.ComponentRef' to its canonical form,
 ** which is when all subscripts are evaluated to constant values.  If
 ** Such an evaluation is not possible, there is no canonical form and
 ** this function fails.
 *"
  input Env.Env in_env;
  input Exp.ComponentRef in_componentref;
  input IntegerList in_integerlist;
  input Boolean in_boolean;
  output Exp.ComponentRef out_componentref;
protected 
  type IntegerList = list<Integer>;
algorithm 
  out_componentref:=
  matchcontinue (in_env,in_componentref,in_integerlist,in_boolean)
    local
      type Exp_SubscriptList = list<Exp.Subscript>;
      type Env_FrameList = list<Env.Frame>;
      Exp_SubscriptList ss_1,ss;
      Env_FrameList env;
      Ident n;
      IntegerList sl;
      Boolean impl;
    case (env,Exp.CREF_IDENT(n,ss),sl,impl) "impl"
      equation 
        ss_1 = Ceval.ceval_subscripts(env, ss, sl, impl); then Exp.CREF_IDENT(n,ss_1);
  end matchcontinue;
end canon_cref2;

public function canon_cref
  input Env.Env in_env;
  input Exp.ComponentRef in_componentref;
  input Boolean in_boolean;
  output Exp.ComponentRef out_componentref;
algorithm 
  out_componentref:=
  matchcontinue (in_env,in_componentref,in_boolean)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type IntegerList = list<Integer>;
      type Exp_SubscriptList = list<Exp.Subscript>;
      type Env_FrameList = list<Env.Frame>;
      TupleTypes_TTypeAbsyn_PathOption t;
      IntegerList sl;
      Exp_SubscriptList ss_1,ss;
      Env_FrameList env;
      Ident n;
      Boolean impl;
      Exp.ComponentRef c_1,c;
    case (env,Exp.CREF_IDENT(n,ss),impl) "impl"
      equation 
        (_,t,_) = Lookup.lookup_var(env, Exp.CREF_IDENT(n,{}));
        sl = Types.get_dimension_sizes(t);
        ss_1 = Ceval.ceval_subscripts(env, ss, sl, impl); then Exp.CREF_IDENT(n,ss_1);
    case (env,Exp.CREF_QUAL(n,ss,c),impl)
      equation 
        (_,t,_) = Lookup.lookup_var(env, Exp.CREF_QUAL(n,{},c));
        sl = Types.get_dimension_sizes(t);
        ss_1 = Ceval.ceval_subscripts(env, ss, sl, impl);
        c_1 = canon_cref2(env, c, sl, impl); then Exp.CREF_QUAL(n,ss_1,c_1);
  end matchcontinue;
end canon_cref;

public function eq_cref "* - Equality functions 
  * function: eq_cref
 **
 ** This function checks if two component references can be considered
 ** equal and fails if not.  Two component references are equal if all
 ** corresponding identifiers are the same, and if the subscripts are
 ** equal, according to the function `eq_subscripts'.
 *"
  input Exp.ComponentRef in_componentref1;
  input Exp.ComponentRef in_componentref2;
algorithm 
  _:=
  matchcontinue (in_componentref1,in_componentref2)
    local
      type Exp_SubscriptList = list<Exp.Subscript>;
      Ident n1,n2;
      Exp_SubscriptList s1,s2;
      Exp.ComponentRef c1,c2;
    case (Exp.CREF_IDENT(n1,s1),Exp.CREF_IDENT(n2,s2))
      equation 
        equality(n1 = n2);
        eq_subscripts(s1, s2); then ();
    case (Exp.CREF_QUAL(n1,s1,c1),Exp.CREF_QUAL(n2,s2,c2))
      equation 
        equality(n1 = n2);
        eq_subscripts(s1, s2);
        eq_cref(c1, c2); then ();
  end matchcontinue;
end eq_cref;

protected function eq_subscripts "* function: eq_subscripts
 **
 ** Two list of subscripts are equal if they are of equal length and
 ** all their elements are pairwise equal according to the function
 ** `eq_subscript'.
 *"
  input Exp_SubscriptList in_exp_subscriptlist1;
  input Exp_SubscriptList in_exp_subscriptlist2;
protected 
  type Exp_SubscriptList = list<Exp.Subscript>;
algorithm 
  _:=
  matchcontinue (in_exp_subscriptlist1,in_exp_subscriptlist2)
    local
      Exp.Subscript s1,s2;
      Exp_SubscriptList ss1,ss2;
    case ({},{}) then (); 
    case (s1 :: ss1,s2 :: ss2)
      equation 
        eq_subscript(s1, s2);
        eq_subscripts(ss1, ss2); then ();
  end matchcontinue;
end eq_subscripts;

protected function eq_subscript "* function: eq_subscript
 **
 ** This function test whether two subscripts are equal.  Two
 ** subscripts are equal if they have the same constructor, and if all
 ** corresponding expressions are either syntactically equal, or if
 ** they have the same constant value.
 *"
  input Exp.Subscript in_subscript1;
  input Exp.Subscript in_subscript2;
algorithm 
  _:=
  matchcontinue (in_subscript1,in_subscript2)
    local Exp.Exp s1,s2;
    case (Exp.WHOLEDIM(),Exp.WHOLEDIM()) then (); 
    case (Exp.INDEX(s1),Exp.INDEX(s2))
      equation 
        equality(s1 = s2); then ();
  end matchcontinue;
end eq_subscript;

protected function elab_arglist
  input Types_TypeList in_types_typelist;
  input TupleExp_ExpTypes_TypeList in_tupleexp_exptypes_typelist;
  output Exp_ExpList out_exp_explist;
  output Types_TypeList out_types_typelist;
protected 
  type Types_TypeList = list<Types.Type>;
  type TupleExp_ExpTypes_Type = tuple<Exp.Exp,Types.Type>;
  type TupleExp_ExpTypes_TypeList = list<TupleExp_ExpTypes_Type>;
  type Exp_ExpList = list<Exp.Exp>;
algorithm 
  (out_exp_explist,out_types_typelist):=
  matchcontinue (in_types_typelist,in_tupleexp_exptypes_typelist)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleTypes_TTypeAbsyn_PathOptionList = list<TupleTypes_TTypeAbsyn_PathOption>;
      type TupleExp_ExpTupleTypes_TTypeAbsyn_PathOption = tuple<Exp.Exp,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleExp_ExpTupleTypes_TTypeAbsyn_PathOptionList = list<TupleExp_ExpTupleTypes_TTypeAbsyn_PathOption>;
      Exp.Exp arg_1,arg;
      TupleTypes_TTypeAbsyn_PathOption atype_1,pt,atype;
      Exp_ExpList args_1;
      TupleTypes_TTypeAbsyn_PathOptionList atypes_1,pts;
      TupleExp_ExpTupleTypes_TTypeAbsyn_PathOptionList args;
    case ({},{}) then ({},{}); 
    case (pt :: pts,(arg,atype) :: args)
      equation 
        (arg_1,atype_1) = Types.match_type(arg, atype, pt);
        (args_1,atypes_1) = elab_arglist(pts, args); then (arg_1 :: args_1,atype_1 :: atypes_1);
  end matchcontinue;
end elab_arglist;

public function deoverload
  input TupleExp_OperatorTypes_TypeListTypes_TypeList in_tupleexp_operatortypes_typelisttypes_typelist;
  input TupleExp_ExpTypes_TypeList in_tupleexp_exptypes_typelist;
  input Absyn.Exp in_exp;
  output Exp.Operator out_operator;
  output Exp_ExpList out_exp_explist;
  output Types.Type out_type;
protected 
  type Types_TypeList = list<Types.Type>;
  type TupleExp_OperatorTypes_TypeListTypes_Type = tuple<Exp.Operator,Types_TypeList,Types.Type>;
  type TupleExp_OperatorTypes_TypeListTypes_TypeList = list<TupleExp_OperatorTypes_TypeListTypes_Type>;
  type TupleExp_ExpTypes_Type = tuple<Exp.Exp,Types.Type>;
  type TupleExp_ExpTypes_TypeList = list<TupleExp_ExpTypes_Type>;
  type Exp_ExpList = list<Exp.Exp>;
algorithm 
  (out_operator,out_exp_explist,out_type):=
  matchcontinue (in_tupleexp_operatortypes_typelisttypes_typelist,in_tupleexp_exptypes_typelist,in_exp)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleTypes_TTypeAbsyn_PathOptionList = list<TupleTypes_TTypeAbsyn_PathOption>;
      type TupleExp_ExpTupleTypes_TTypeAbsyn_PathOption = tuple<Exp.Exp,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleExp_ExpTupleTypes_TTypeAbsyn_PathOptionList = list<TupleExp_ExpTupleTypes_TTypeAbsyn_PathOption>;
      type TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOption = tuple<Exp.Operator,TupleTypes_TTypeAbsyn_PathOptionList,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOptionList = list<TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOption>;
      Exp_ExpList args_1;
      TupleTypes_TTypeAbsyn_PathOptionList types_1,params;
      TupleTypes_TTypeAbsyn_PathOption rtype_1,rtype;
      Exp.Operator op;
      TupleExp_ExpTupleTypes_TTypeAbsyn_PathOptionList args;
      TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOptionList xs;
      Absyn.Exp exp;
      Ident s;
    case ((op,params,rtype) :: _,args,_)
      equation 
        Debug.fprint_list("dovl", params, Types.print_type, "\n");
        Debug.fprint("dovl", "\n===\n");
        (args_1,types_1) = elab_arglist(params, args);
        rtype_1 = compute_return_type(op, types_1, rtype); then (op,args_1,rtype_1);
    case (_ :: xs,args,exp)
      equation 
        (op,args_1,rtype) = deoverload(xs, args, exp); then (op,args_1,rtype);
    case ({},args,exp)
      equation 
        Print.print_error_buf("# Can't resolve type of expression: ");
        s = Dump.print_exp_str(exp);
        Print.print_error_buf(s);
        Print.print_error_buf("\n"); then fail();
  end matchcontinue;
end deoverload;

protected function compute_return_type "* function: compute_return_type
 ** This function determines the return type of an operator and the types of 
 ** the operands.
 *"
  input Exp.Operator in_operator;
  input Types_TypeList in_types_typelist;
  input Types.Type in_type;
  output Types.Type out_type;
protected 
  type Types_TypeList = list<Types.Type>;
algorithm 
  out_type:=
  matchcontinue (in_operator,in_types_typelist,in_type)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      TupleTypes_TTypeAbsyn_PathOption typ1,typ2,rtype,etype,typ;
      Ident s;
      Integer n1,n2,m,n,m1,m2,p;
    case (Exp.ADD_ARR(_),{typ1,typ2},rtype)
      equation 
        true = Types.subtype(typ1, typ2); then typ1;
    case (Exp.ADD_ARR(_),{typ1,typ2},rtype)
      equation 
        true = Types.subtype(typ1, typ2); then typ1;
    case (Exp.ADD_ARR(_),{typ1,typ2},_)
      equation 
        Print.print_error_buf("# +: Incompatible argument types ");
        s = Types.unparse_type(typ1);
        Print.print_error_buf(s);
        Print.print_error_buf(", ");
        s = Types.unparse_type(typ2);
        Print.print_error_buf(s);
        Print.print_error_buf("\n"); then fail();
    case (Exp.SUB_ARR(_),{typ1,typ2},rtype)
      equation 
        true = Types.subtype(typ1, typ2); then typ1;
    case (Exp.SUB_ARR(_),{typ1,typ2},rtype)
      equation 
        true = Types.subtype(typ1, typ2); then typ1;
    case (Exp.SUB_ARR(_),{typ1,typ2},_)
      equation 
        Print.print_error_buf("# -: Incompatible argument types ");
        s = Types.unparse_type(typ1);
        Print.print_error_buf(s);
        Print.print_error_buf(", ");
        s = Types.unparse_type(typ2);
        Print.print_error_buf(s);
        Print.print_error_buf("\n"); then fail();
    case (Exp.MUL_SCALAR_PRODUCT(_),{typ1,typ2},rtype)
      equation 
        true = Types.subtype(typ1, typ2); then rtype;
    case (Exp.MUL_SCALAR_PRODUCT(_),{typ1,typ2},rtype)
      equation 
        true = Types.subtype(typ1, typ2); then rtype;
    case (Exp.MUL_SCALAR_PRODUCT(_),{typ1,typ2},rtype)
      equation 
        Print.print_error_buf("# *-: Incompatible argument types ");
        s = Types.unparse_type(typ1);
        Print.print_error_buf("ltype: ");
        Print.print_error_buf(s);
        s = Types.unparse_type(typ2);
        Print.print_error_buf("rtype: ");
        Print.print_error_buf(s);
        Print.print_error_buf("\n"); then fail();
    case (Exp.MUL_MATRIX_PRODUCT(_),{typ1,typ2},_)
      equation 
        1 = n_dims(typ1);
        2 = n_dims(typ2);
        n1 = dim_size(typ1, 1);
        n2 = dim_size(typ2, 1);
        m = dim_size(typ2, 2);
        equality(n1 = n2);
        etype = element_type(typ1);
        rtype = (Types.T_ARRAY(Types.DIM(SOME(m)),etype),NONE); then rtype;
    case (Exp.MUL_MATRIX_PRODUCT(_),{typ1,typ2},_)
      equation 
        2 = n_dims(typ1);
        1 = n_dims(typ2);
        n = dim_size(typ1, 1);
        m1 = dim_size(typ1, 2);
        m2 = dim_size(typ2, 1);
        equality(m1 = m2);
        etype = element_type(typ2);
        rtype = (Types.T_ARRAY(Types.DIM(SOME(n)),etype),NONE); then rtype;
    case (Exp.MUL_MATRIX_PRODUCT(_),{typ1,typ2},_)
      equation 
        2 = n_dims(typ1);
        2 = n_dims(typ2);
        n = dim_size(typ1, 1);
        m1 = dim_size(typ1, 2);
        m2 = dim_size(typ2, 1);
        p = dim_size(typ2, 2);
        equality(m1 = m2);
        etype = element_type(typ1);
        rtype = (
          Types.T_ARRAY(Types.DIM(SOME(n)),
          (Types.T_ARRAY(Types.DIM(SOME(p)),etype),NONE)),NONE); then rtype;
    case (Exp.MUL_MATRIX_PRODUCT(_),{typ1,typ2},rtype)
      equation 
        Print.print_error_buf("# *-----: Incompatible argument types ");
        s = Types.unparse_type(typ1);
        Print.print_error_buf("ltype: ");
        Print.print_error_buf(s);
        s = Types.unparse_type(typ2);
        Print.print_error_buf("rtype: ");
        Print.print_error_buf(s);
        Print.print_error_buf("\n"); then fail();
    case (Exp.MUL_SCALAR_ARRAY(_),{typ1,typ2},rtype) then typ2;  "rtype"
    case (Exp.MUL_ARRAY_SCALAR(_),{typ1,typ2},rtype) then typ1;  " rtype"
    case (Exp.DIV_ARRAY_SCALAR(_),{typ1,typ2},rtype) then typ1;  " rtype"
    case (Exp.ADD(_),_,typ) then typ; 
    case (Exp.SUB(_),_,typ) then typ; 
    case (Exp.MUL(_),_,typ) then typ; 
    case (Exp.DIV(_),_,typ) then typ; 
    case (Exp.POW(_),_,typ) then typ; 
    case (Exp.UMINUS(_),_,typ) then typ; 
    case (Exp.UPLUS(_),_,typ) then typ; 
    case (Exp.AND(),_,typ) then typ; 
    case (Exp.OR(),_,typ) then typ; 
    case (Exp.NOT(),_,typ) then typ; 
    case (Exp.LESS(_),_,typ) then typ; 
    case (Exp.LESSEQ(_),_,typ) then typ; 
    case (Exp.GREATER(_),_,typ) then typ; 
    case (Exp.GREATEREQ(_),_,typ) then typ; 
    case (Exp.EQUAL(_),_,typ) then typ; 
    case (Exp.NEQUAL(_),_,typ) then typ; 
    case (Exp.USERDEFINED(_),_,typ) then typ; 
  end matchcontinue;
end compute_return_type;

protected function n_dims "* function n_dims
 ** Returns the number of dimensions of a Type.
 *"
  input Types.Type in_type;
  output Integer out_integer;
algorithm 
  out_integer:=
  matchcontinue (in_type)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      Integer ns;
      TupleTypes_TTypeAbsyn_PathOption t;
    case ((Types.T_INTEGER(_),_)) then 0; 
    case ((Types.T_REAL(_),_)) then 0; 
    case ((Types.T_STRING(_),_)) then 0; 
    case ((Types.T_BOOL(_),_)) then 0; 
    case ((Types.T_ARRAY(_,t),_))
      equation 
        ns = n_dims(t); then ns + 1;
  end matchcontinue;
end n_dims;

protected function dim_size " size of n:nth dimension "
  input Types.Type in_type;
  input Integer in_integer;
  output Integer out_integer;
algorithm 
  out_integer:=
  matchcontinue (in_type,in_integer)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      Integer n,d_1,d;
      TupleTypes_TTypeAbsyn_PathOption t;
    case ((Types.T_ARRAY(Types.DIM(SOME(n)),_),_),1) then n; 
    case ((Types.T_ARRAY(_,t),_),d)
      equation 
        (d > 1) = true;
        d_1 = d - 1;
        n = dim_size(t, d_1); then n;
  end matchcontinue;
end dim_size;

protected function element_type "* function: element_type
 ** Returns the element type of a type, i.e. for arrays, return the element type, and for 
 ** bulitin scalar types return the type itself.
 *"
  input Types.Type in_type;
  output Types.Type out_type;
algorithm 
  out_type:=
  matchcontinue (in_type)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      TupleTypes_TTypeAbsyn_PathOption t,t_1;
    case ((t = (Types.T_INTEGER(_),_))) then t; 
    case ((t = (Types.T_REAL(_),_))) then t; 
    case ((t = (Types.T_STRING(_),_))) then t; 
    case ((t = (Types.T_BOOL(_),_))) then t; 
    case ((Types.T_ARRAY(_,t),_))
      equation 
        t_1 = element_type(t); then t_1;
  end matchcontinue;
end element_type;

public function operators
  input Absyn.Operator in_operator1;
  input Env.Env in_env2;
  input Types.Type in_type3;
  input Types.Type in_type4;
  output TupleExp_OperatorTypes_TypeListTypes_TypeList out_tupleexp_operatortypes_typelisttypes_typelist;
protected 
  type Types_TypeList = list<Types.Type>;
  type TupleExp_OperatorTypes_TypeListTypes_Type = tuple<Exp.Operator,Types_TypeList,Types.Type>;
  type TupleExp_OperatorTypes_TypeListTypes_TypeList = list<TupleExp_OperatorTypes_TypeListTypes_Type>;
algorithm 
  out_tupleexp_operatortypes_typelisttypes_typelist:=
  matchcontinue (in_operator1,in_env2,in_type3,in_type4)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleTypes_TTypeAbsyn_PathOptionList = list<TupleTypes_TTypeAbsyn_PathOption>;
      type TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOption = tuple<Exp.Operator,TupleTypes_TTypeAbsyn_PathOptionList,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOptionList = list<TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOption>;
      type Env_FrameList = list<Env.Frame>;
      TupleTypes_TTypeAbsyn_PathOptionList intarrtypes,realarrtypes,stringarrtypes,inttypes,realtypes;
      TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOptionList intarrs,realarrs,stringarrs,scalars,userops,arrays,types,scalarprod,matrixprod,intscalararrs,realscalararrs,intarrsscalar,realarrsscalar,realarrscalar,arrscalar;
      Env_FrameList env;
      TupleTypes_TTypeAbsyn_PathOption t1,t2,int_scalar,int_vector,int_matrix,real_scalar,real_vector,real_matrix;
      Exp.Operator int_mul,real_mul,int_mul_sp,real_mul_sp,int_mul_mp,real_mul_mp,real_div,real_pow,int_pow;
      Ident s;
      Absyn.Operator op;
    case (Absyn.ADD(),env,t1,t2) "*  Arithmetical operators "
      equation 
        intarrtypes = array_type_list(9, (Types.T_INTEGER({}),NONE)) " The ADD operator ";
        realarrtypes = array_type_list(9, (Types.T_REAL({}),NONE));
        stringarrtypes = array_type_list(9, (Types.T_STRING({}),NONE));
        intarrs = operator_return(Exp.ADD_ARR(Exp.INT()), intarrtypes, intarrtypes, 
          intarrtypes);
        realarrs = operator_return(Exp.ADD_ARR(Exp.REAL()), realarrtypes, realarrtypes, 
          realarrtypes);
        stringarrs = operator_return(Exp.ADD_ARR(Exp.STRING()), stringarrtypes, stringarrtypes, 
          stringarrtypes);
        scalars = {
          (Exp.ADD(Exp.INT()),
          {(Types.T_INTEGER({}),NONE),(Types.T_INTEGER({}),NONE)},(Types.T_INTEGER({}),NONE)),
          (Exp.ADD(Exp.REAL()),
          {(Types.T_REAL({}),NONE),(Types.T_REAL({}),NONE)},(Types.T_REAL({}),NONE)),
          (Exp.ADD(Exp.STRING()),
          {(Types.T_STRING({}),NONE),(Types.T_STRING({}),NONE)},(Types.T_STRING({}),NONE))};
        userops = get_koening_operator_types("plus", env, t1, t2);
        arrays = Util.list_flatten({intarrs,realarrs,stringarrs});
        types = Util.list_flatten({scalars,arrays,userops}); then types;
    case (Absyn.SUB(),env,t1,t2)
      equation 
        intarrtypes = array_type_list(9, (Types.T_INTEGER({}),NONE)) " the SUB operator ";
        realarrtypes = array_type_list(9, (Types.T_REAL({}),NONE));
        intarrs = operator_return(Exp.SUB_ARR(Exp.INT()), intarrtypes, intarrtypes, 
          intarrtypes);
        realarrs = operator_return(Exp.SUB_ARR(Exp.REAL()), realarrtypes, realarrtypes, 
          realarrtypes);
        scalars = {
          (Exp.SUB(Exp.INT()),
          {(Types.T_INTEGER({}),NONE),(Types.T_INTEGER({}),NONE)},(Types.T_INTEGER({}),NONE)),
          (Exp.SUB(Exp.REAL()),
          {(Types.T_REAL({}),NONE),(Types.T_REAL({}),NONE)},(Types.T_REAL({}),NONE))};
        userops = get_koening_operator_types("minus", env, t1, t2);
        types = Util.list_flatten({scalars,intarrs,realarrs,userops}); then types;
    case (Absyn.MUL(),env,t1,t2)
      equation 
        intarrtypes = array_type_list(9, (Types.T_INTEGER({}),NONE)) " The MUL operator ";
        realarrtypes = array_type_list(9, (Types.T_REAL({}),NONE));
        inttypes = n_types(9, (Types.T_INTEGER({}),NONE));
        realtypes = n_types(9, (Types.T_REAL({}),NONE));
        int_mul = Exp.MUL(Exp.INT());
        real_mul = Exp.MUL(Exp.REAL());
        int_mul_sp = Exp.MUL_SCALAR_PRODUCT(Exp.INT());
        real_mul_sp = Exp.MUL_SCALAR_PRODUCT(Exp.REAL());
        int_mul_mp = Exp.MUL_MATRIX_PRODUCT(Exp.INT());
        real_mul_mp = Exp.MUL_MATRIX_PRODUCT(Exp.REAL());
        int_scalar = (Types.T_INTEGER({}),NONE);
        int_vector = (Types.T_ARRAY(Types.DIM(NONE),int_scalar),NONE);
        int_matrix = (Types.T_ARRAY(Types.DIM(NONE),int_vector),NONE);
        real_scalar = (Types.T_REAL({}),NONE);
        real_vector = (Types.T_ARRAY(Types.DIM(NONE),real_scalar),NONE);
        real_matrix = (Types.T_ARRAY(Types.DIM(NONE),real_vector),NONE);
        scalars = {(int_mul,{int_scalar,int_scalar},int_scalar),
          (real_mul,{real_scalar,real_scalar},real_scalar)};
        scalarprod = {(int_mul_sp,{int_vector,int_vector},int_scalar),
          (real_mul_sp,{real_vector,real_vector},real_scalar)};
        matrixprod = {(int_mul_mp,{int_vector,int_matrix},int_vector),
          (int_mul_mp,{int_matrix,int_vector},int_vector),(int_mul_mp,{int_matrix,int_matrix},int_matrix),
          (real_mul_mp,{real_vector,real_matrix},real_vector),(real_mul_mp,{real_matrix,real_vector},real_vector),
          (real_mul_mp,{real_matrix,real_matrix},real_matrix)};
        intscalararrs = operator_return(Exp.MUL_SCALAR_ARRAY(Exp.INT()), inttypes, intarrtypes, 
          intarrtypes);
        realscalararrs = operator_return(Exp.MUL_SCALAR_ARRAY(Exp.REAL()), realtypes, realarrtypes, 
          realarrtypes);
        intarrsscalar = operator_return(Exp.MUL_ARRAY_SCALAR(Exp.INT()), intarrtypes, inttypes, 
          intarrtypes);
        realarrsscalar = operator_return(Exp.MUL_ARRAY_SCALAR(Exp.REAL()), realarrtypes, realtypes, 
          realarrtypes);
        userops = get_koening_operator_types("times", env, t1, t2);
        types = Util.list_flatten(
          {scalars,intscalararrs,realscalararrs,intarrsscalar,
          realarrsscalar,scalarprod,matrixprod,userops}); then types;
    case (Absyn.DIV(),env,t1,t2)
      equation 
        realarrtypes = array_type_list(9, (Types.T_REAL({}),NONE)) " The DIV operator ";
        realtypes = n_types(9, (Types.T_REAL({}),NONE));
        real_div = Exp.DIV(Exp.REAL());
        real_scalar = (Types.T_REAL({}),NONE);
        scalars = {(real_div,{real_scalar,real_scalar},real_scalar)};
        realarrscalar = operator_return(Exp.DIV_ARRAY_SCALAR(Exp.REAL()), realarrtypes, realtypes, 
          realarrtypes);
        userops = get_koening_operator_types("divide", env, t1, t2);
        types = Util.list_flatten({scalars,realarrscalar,userops}); then types;
    case (Absyn.POW(),env,t1,t2)
      equation 
        real_scalar = (Types.T_REAL({}),NONE) " The POW operator. a^b is only defined for integer exponents, i.e. b must
	 * be of type Integer ";
        int_scalar = (Types.T_INTEGER({}),NONE);
        real_vector = (Types.T_ARRAY(Types.DIM(NONE),real_scalar),NONE);
        real_matrix = (Types.T_ARRAY(Types.DIM(NONE),real_vector),NONE);
        real_pow = Exp.POW(Exp.REAL());
        int_pow = Exp.POW(Exp.INT());
        scalars = {(int_pow,{int_scalar,int_scalar},int_scalar),
          (real_pow,{real_scalar,real_scalar},real_scalar)};
        arrscalar = {
          (Exp.POW_ARR(Exp.REAL()),{real_matrix,int_scalar},
          real_matrix)};
        userops = get_koening_operator_types("power", env, t1, t2);
        types = Util.list_flatten({scalars,arrscalar,userops}); then types;
    case (Absyn.UMINUS(),env,t1,t2)
      equation 
        scalars = {
          (Exp.UMINUS(Exp.INT()),{(Types.T_INTEGER({}),NONE)},
          (Types.T_INTEGER({}),NONE)),
          (Exp.UMINUS(Exp.REAL()),{(Types.T_REAL({}),NONE)},
          (Types.T_REAL({}),NONE))} " The UMINUS operator, unary minus ";
        userops = get_koening_operator_types("unaryMinus", env, t1, t2);
        types = Util.list_flatten({scalars,userops}); then types;
    case (Absyn.UPLUS(),env,t1,t2)
      equation 
        scalars = {
          (Exp.UPLUS(Exp.INT()),{(Types.T_INTEGER({}),NONE)},
          (Types.T_INTEGER({}),NONE)),
          (Exp.UPLUS(Exp.REAL()),{(Types.T_REAL({}),NONE)},
          (Types.T_REAL({}),NONE))} " The UPLUS operator, unary plus.";
        userops = get_koening_operator_types("unaryPlus", env, t1, t2);
        types = Util.list_flatten({scalars,userops}); then types;
    case (Absyn.AND(),env,t1,t2) then {
          (Exp.AND(),
          {(Types.T_BOOL({}),NONE),(Types.T_BOOL({}),NONE)},(Types.T_BOOL({}),NONE))}; 
    case (Absyn.OR(),env,t1,t2) then {
          (Exp.OR(),{(Types.T_BOOL({}),NONE),(Types.T_BOOL({}),NONE)},
          (Types.T_BOOL({}),NONE))}; 
    case (Absyn.NOT(),env,t1,t2) then {
          (Exp.NOT(),{(Types.T_BOOL({}),NONE)},(Types.T_BOOL({}),NONE))}; 
    case (Absyn.LESS(),env,t1,t2) "*  Relational operators "
      equation 
        scalars = {
          (Exp.LESS(Exp.INT()),
          {(Types.T_INTEGER({}),NONE),(Types.T_INTEGER({}),NONE)},(Types.T_BOOL({}),NONE)),
          (Exp.LESS(Exp.REAL()),
          {(Types.T_REAL({}),NONE),(Types.T_REAL({}),NONE)},(Types.T_BOOL({}),NONE))} " '<' operator ";
        userops = get_koening_operator_types("less", env, t1, t2);
        types = Util.list_flatten({scalars,userops}); then types;
    case (Absyn.LESSEQ(),env,t1,t2)
      equation 
        scalars = {
          (Exp.LESSEQ(Exp.INT()),
          {(Types.T_INTEGER({}),NONE),(Types.T_INTEGER({}),NONE)},(Types.T_BOOL({}),NONE)),
          (Exp.LESSEQ(Exp.REAL()),
          {(Types.T_REAL({}),NONE),(Types.T_REAL({}),NONE)},(Types.T_BOOL({}),NONE))} " '<=' operator ";
        userops = get_koening_operator_types("lessEqual", env, t1, t2);
        types = Util.list_flatten({scalars,userops}); then types;
    case (Absyn.GREATER(),env,t1,t2)
      equation 
        scalars = {
          (Exp.GREATER(Exp.INT()),
          {(Types.T_INTEGER({}),NONE),(Types.T_INTEGER({}),NONE)},(Types.T_BOOL({}),NONE)),
          (Exp.GREATER(Exp.REAL()),
          {(Types.T_REAL({}),NONE),(Types.T_REAL({}),NONE)},(Types.T_BOOL({}),NONE))} " '>' operator ";
        userops = get_koening_operator_types("greater", env, t1, t2);
        types = Util.list_flatten({scalars,userops}); then types;
    case (Absyn.GREATEREQ(),env,t1,t2)
      equation 
        scalars = {
          (Exp.GREATEREQ(Exp.INT()),
          {(Types.T_INTEGER({}),NONE),(Types.T_INTEGER({}),NONE)},(Types.T_BOOL({}),NONE)),
          (Exp.GREATEREQ(Exp.REAL()),
          {(Types.T_REAL({}),NONE),(Types.T_REAL({}),NONE)},(Types.T_BOOL({}),NONE))} " '>=' operator ";
        userops = get_koening_operator_types("greaterEqual", env, t1, t2);
        types = Util.list_flatten({scalars,userops}); then types;
    case (Absyn.EQUAL(),env,t1,t2)
      equation 
        scalars = {
          (Exp.EQUAL(Exp.INT()),
          {(Types.T_INTEGER({}),NONE),(Types.T_INTEGER({}),NONE)},(Types.T_BOOL({}),NONE)),
          (Exp.EQUAL(Exp.REAL()),
          {(Types.T_REAL({}),NONE),(Types.T_REAL({}),NONE)},(Types.T_BOOL({}),NONE)),
          (Exp.EQUAL(Exp.STRING()),
          {(Types.T_STRING({}),NONE),(Types.T_STRING({}),NONE)},(Types.T_BOOL({}),NONE)),
          (Exp.EQUAL(Exp.BOOL()),
          {(Types.T_BOOL({}),NONE),(Types.T_BOOL({}),NONE)},(Types.T_BOOL({}),NONE))} " '==' operator ";
        userops = get_koening_operator_types("equal", env, t1, t2);
        types = Util.list_flatten({scalars,userops}); then types;
    case (Absyn.NEQUAL(),env,t1,t2)
      equation 
        scalars = {
          (Exp.NEQUAL(Exp.INT()),
          {(Types.T_INTEGER({}),NONE),(Types.T_INTEGER({}),NONE)},(Types.T_BOOL({}),NONE)),
          (Exp.NEQUAL(Exp.REAL()),
          {(Types.T_REAL({}),NONE),(Types.T_REAL({}),NONE)},(Types.T_BOOL({}),NONE)),
          (Exp.NEQUAL(Exp.STRING()),
          {(Types.T_STRING({}),NONE),(Types.T_STRING({}),NONE)},(Types.T_BOOL({}),NONE)),
          (Exp.NEQUAL(Exp.BOOL()),
          {(Types.T_BOOL({}),NONE),(Types.T_BOOL({}),NONE)},(Types.T_BOOL({}),NONE))} " '!=' operator ";
        userops = get_koening_operator_types("notEqual", env, t1, t2);
        types = Util.list_flatten({scalars,userops}); then types;
    case (op,env,t1,t2)
      equation 
        Debug.fprint("failtrace", "-operators failed, op: ");
        s = Dump.op_symbol(op);
        Debug.fprint("failtrace", s);
        Debug.fprint("failtrace", "\n"); then fail();
  end matchcontinue;
end operators;

protected function get_koening_function_types "impl"
  input Env.Env in_env;
  input Absyn.Path in_path;
  input Absyn_ExpList in_absyn_explist;
  input Absyn_NamedArgList in_absyn_namedarglist;
  input Boolean in_boolean;
  output Types_TypeList out_types_typelist;
protected 
  type Absyn_ExpList = list<Absyn.Exp>;
  type Absyn_NamedArgList = list<Absyn.NamedArg>;
  type Types_TypeList = list<Types.Type>;
algorithm 
  out_types_typelist:=
  matchcontinue (in_env,in_path,in_absyn_explist,in_absyn_namedarglist,in_boolean)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleTypes_TTypeAbsyn_PathOptionList = list<TupleTypes_TTypeAbsyn_PathOption>;
      type Env_FrameList = list<Env.Frame>;
      TupleTypes_TTypeAbsyn_PathOption t;
      Absyn.Path p1,fn;
      SCode.Class c;
      Env.Frame f,f_1;
      TupleTypes_TTypeAbsyn_PathOptionList typelist,typelist2,res;
      Env_FrameList env;
      Absyn.Exp e1,exp;
      Absyn_ExpList exps;
      Absyn_NamedArgList na;
      Boolean impl;
      Ident id,fnstr;
    case (env,(fn = Absyn.IDENT(_)),e1 :: exps,na,impl) "impl"
      equation 
        (_,Types.PROP(t,_),_) = elab_exp(env, e1, impl, NONE);
        p1 = Types.get_classname(t);
        (c,f :: _) = Lookup.lookup_class(env, p1, false) "msg";
        (_,f_1 :: _) = Lookup.lookup_type({f}, fn, false) " To make sure the function is implicitly instantiated.";
        typelist = Lookup.lookup_functions_in_env({f_1}, fn);
        typelist2 = get_koening_function_types(env, fn, exps, na, impl);
        res = list_append(typelist, typelist2); then res;
    case (env,(fn = Absyn.IDENT(_)),e1 :: exps,na,impl)
      equation 
        typelist = get_koening_function_types(env, fn, exps, na, impl); then typelist;
    case (env,(fn = Absyn.IDENT(_)),{},Absyn.NAMEDARG(id,exp) :: na,impl)
      equation 
        (_,Types.PROP(t,_),_) = elab_exp(env, exp, impl, NONE);
        (p1 = Absyn.QUALIFIED(_,_)) = Types.get_classname(t);
        (c,f :: _) = Lookup.lookup_class(env, p1, false);
        (_,f_1 :: _) = Lookup.lookup_type({f}, fn, false) " To make sure the function is implicitly instantiated.";
        typelist = Lookup.lookup_functions_in_env({f_1}, fn);
        typelist2 = get_koening_function_types(env, fn, {}, na, impl);
        res = list_append(typelist, typelist2); then res;
    case (env,(fn = Absyn.IDENT(_)),{},_ :: na,impl)
      equation 
        res = get_koening_function_types(env, fn, {}, na, impl); then res;
    case (env,(fn = Absyn.IDENT(_)),{},{},impl) then {}; 
    case (env,(fn = Absyn.QUALIFIED(_,_)),_,_,impl)
      equation 
        Print.print_error_buf("# Error, koening lookup of non-simple function name, ");
        fnstr = Absyn.path_string(fn);
        Print.print_error_buf(fnstr);
        Print.print_error_buf("\n"); then fail();
  end matchcontinue;
end get_koening_function_types;

protected function get_koening_operator_types
  input String in_string1;
  input Env.Env in_env2;
  input Types.Type in_type3;
  input Types.Type in_type4;
  output TupleExp_OperatorTypes_TypeListTypes_TypeList out_tupleexp_operatortypes_typelisttypes_typelist;
protected 
  type Types_TypeList = list<Types.Type>;
  type TupleExp_OperatorTypes_TypeListTypes_Type = tuple<Exp.Operator,Types_TypeList,Types.Type>;
  type TupleExp_OperatorTypes_TypeListTypes_TypeList = list<TupleExp_OperatorTypes_TypeListTypes_Type>;
algorithm 
  out_tupleexp_operatortypes_typelisttypes_typelist:=
  matchcontinue (in_string1,in_env2,in_type3,in_type4)
    local
      type Env_FrameList = list<Env.Frame>;
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleTypes_TTypeAbsyn_PathOptionList = list<TupleTypes_TTypeAbsyn_PathOption>;
      type TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOption = tuple<Exp.Operator,TupleTypes_TTypeAbsyn_PathOptionList,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOptionList = list<TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOption>;
      Absyn.Path p1,p2;
      SCode.Class c;
      Env_FrameList env1,env2,env;
      TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOptionList res1,res2,res;
      Ident op;
      TupleTypes_TTypeAbsyn_PathOption t1,t2;
    case (op,env,t1,t2)
      equation 
        (p1 = Absyn.QUALIFIED(_,_)) = Types.get_classname(t1) " Both types user defined ";
        (c,env1) = Lookup.lookup_class(env, p1, false);
        res1 = get_koening_operator_types_in_scope(op, env1);
        (p2 = Absyn.QUALIFIED(_,_)) = Types.get_classname(t2);
        (c,env2) = Lookup.lookup_class(env, p2, false);
        res2 = get_koening_operator_types_in_scope(op, env2);
        res = list_append(res1, res2); then res;
    case (op,env,t1,t2)
      equation 
        failure(Absyn.QUALIFIED(_,_) = Types.get_classname(t1)) " User defined types only in t2";
        (p2 = Absyn.QUALIFIED(_,_)) = Types.get_classname(t2);
        (c,env2) = Lookup.lookup_class(env, p2, false);
        res = get_koening_operator_types_in_scope(op, env2); then res;
    case (op,env,t1,t2)
      equation 
        failure(Absyn.QUALIFIED(_,_) = Types.get_classname(t2)) " User defined types only in t1";
        (p1 = Absyn.QUALIFIED(_,_)) = Types.get_classname(t1);
        (c,env1) = Lookup.lookup_class(env, p1, false);
        res = get_koening_operator_types_in_scope(op, env1); then res;
    case (op,env,t1,t2)
      equation 
        failure(Absyn.QUALIFIED(_,_) = Types.get_classname(t1)) " No User defined types at all. ";
        failure(Absyn.QUALIFIED(_,_) = Types.get_classname(t2)); then {};
    case (op,env,t1,t2) then {}; 
  end matchcontinue;
end get_koening_operator_types;

protected function get_koening_operator_types_in_scope
  input String in_string;
  input Env.Env in_env;
  output TupleExp_OperatorTypes_TypeListTypes_TypeList out_tupleexp_operatortypes_typelisttypes_typelist;
protected 
  type Types_TypeList = list<Types.Type>;
  type TupleExp_OperatorTypes_TypeListTypes_Type = tuple<Exp.Operator,Types_TypeList,Types.Type>;
  type TupleExp_OperatorTypes_TypeListTypes_TypeList = list<TupleExp_OperatorTypes_TypeListTypes_Type>;
algorithm 
  out_tupleexp_operatortypes_typelisttypes_typelist:=
  matchcontinue (in_string,in_env)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleTypes_TTypeAbsyn_PathOptionList = list<TupleTypes_TTypeAbsyn_PathOption>;
      type TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOption = tuple<Exp.Operator,TupleTypes_TTypeAbsyn_PathOptionList,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOptionList = list<TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOption>;
      type Env_FrameList = list<Env.Frame>;
      Env.Frame f_1,f;
      TupleTypes_TTypeAbsyn_PathOptionList tplst;
      Integer tplen;
      Absyn.Path fullfuncname;
      TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOptionList res;
      Ident funcname;
      Env_FrameList fs;
    case (funcname,f :: fs)
      equation 
        (_,f_1 :: _) = Lookup.lookup_type({f}, Absyn.IDENT(funcname), false) " To make sure the function is implicitly instantiated.";
        tplst = Lookup.lookup_functions_in_env({f_1}, Absyn.IDENT(funcname)) " TODO: Fix so lookup_functions_in_env also does instantiation to get type ";
        tplen = list_length(tplst);
        fullfuncname = Inst.make_fully_qualified(f_1 :: fs, Absyn.IDENT(funcname));
        res = build_operator_types(tplst, fullfuncname); then res;
  end matchcontinue;
end get_koening_operator_types_in_scope;

protected function build_operator_types
  input Types_TypeList in_types_typelist;
  input Absyn.Path in_path;
  output TupleExp_OperatorTypes_TypeListTypes_TypeList out_tupleexp_operatortypes_typelisttypes_typelist;
protected 
  type Types_TypeList = list<Types.Type>;
  type TupleExp_OperatorTypes_TypeListTypes_Type = tuple<Exp.Operator,Types_TypeList,Types.Type>;
  type TupleExp_OperatorTypes_TypeListTypes_TypeList = list<TupleExp_OperatorTypes_TypeListTypes_Type>;
algorithm 
  out_tupleexp_operatortypes_typelisttypes_typelist:=
  matchcontinue (in_types_typelist,in_path)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleTypes_TTypeAbsyn_PathOptionList = list<TupleTypes_TTypeAbsyn_PathOption>;
      type TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOption = tuple<Exp.Operator,TupleTypes_TTypeAbsyn_PathOptionList,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOptionList = list<TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOption>;
      type TupleIdentTupleTypes_TTypeAbsyn_PathOption = tuple<Ident,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleIdentTupleTypes_TTypeAbsyn_PathOptionList = list<TupleIdentTupleTypes_TTypeAbsyn_PathOption>;
      TupleTypes_TTypeAbsyn_PathOptionList argtypes,tps;
      TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOptionList rest;
      TupleIdentTupleTypes_TTypeAbsyn_PathOptionList args;
      TupleTypes_TTypeAbsyn_PathOption tp;
      Absyn.Path funcname;
    case ({},_) then {}; 
    case ((Types.T_FUNCTION(args,tp),_) :: tps,funcname)
      equation 
        argtypes = Util.list_map(args, Util.tuple2_2);
        rest = build_operator_types(tps, funcname); then (Exp.USERDEFINED(funcname),argtypes,tp) :: rest;
  end matchcontinue;
end build_operator_types;

protected function n_dim_array " array type of n dimensions with element type = orig type "
  input Integer in_integer;
  input Types.Type in_type;
  output Types.Type out_type;
algorithm 
  out_type:=
  matchcontinue (in_integer,in_type)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      TupleTypes_TTypeAbsyn_PathOption t,t_1;
      Integer n_1,n;
    case (0,t) then t; 
    case (n,t)
      equation 
        n_1 = n - 1;
        t_1 = n_dim_array(n_1, t); then ((Types.T_ARRAY(Types.DIM(NONE),t_1),NONE));
  end matchcontinue;
end n_dim_array;

protected function n_types "* function: n_types
 ** Creates n copies of the type type.
 ** This could instead be accomplished with Util.list_fill...
 *"
  input Integer in_integer;
  input Types.Type in_type;
  output Types_TypeList out_types_typelist;
protected 
  type Types_TypeList = list<Types.Type>;
algorithm 
  out_types_typelist:=
  matchcontinue (in_integer,in_type)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleTypes_TTypeAbsyn_PathOptionList = list<TupleTypes_TTypeAbsyn_PathOption>;
      Integer n_1,n;
      TupleTypes_TTypeAbsyn_PathOptionList l;
      TupleTypes_TTypeAbsyn_PathOption t;
    case (0,_) then {}; 
    case (n,t)
      equation 
        n_1 = n - 1;
        l = n_types(n_1, t); then t :: l;
  end matchcontinue;
end n_types;

protected function operator_return
  input Exp.Operator in_operator1;
  input Types_TypeList in_types_typelist2;
  input Types_TypeList in_types_typelist3;
  input Types_TypeList in_types_typelist4;
  output TupleExp_OperatorTypes_TypeListTypes_TypeList out_tupleexp_operatortypes_typelisttypes_typelist;
protected 
  type Types_TypeList = list<Types.Type>;
  type TupleExp_OperatorTypes_TypeListTypes_Type = tuple<Exp.Operator,Types_TypeList,Types.Type>;
  type TupleExp_OperatorTypes_TypeListTypes_TypeList = list<TupleExp_OperatorTypes_TypeListTypes_Type>;
algorithm 
  out_tupleexp_operatortypes_typelisttypes_typelist:=
  matchcontinue (in_operator1,in_types_typelist2,in_types_typelist3,in_types_typelist4)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleTypes_TTypeAbsyn_PathOptionList = list<TupleTypes_TTypeAbsyn_PathOption>;
      type TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOption = tuple<Exp.Operator,TupleTypes_TTypeAbsyn_PathOptionList,TupleTypes_TTypeAbsyn_PathOption>;
      type TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOptionList = list<TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOption>;
      TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOptionList rest;
      TupleExp_OperatorTupleTypes_TTypeAbsyn_PathOptionListTupleTypes_TTypeAbsyn_PathOption t;
      Exp.Operator op;
      TupleTypes_TTypeAbsyn_PathOption l,r,re;
      TupleTypes_TTypeAbsyn_PathOptionList lr,rr,rer;
    case (_,{},{},{}) then {}; 
    case (op,l :: lr,r :: rr,re :: rer)
      equation 
        rest = operator_return(op, lr, rr, rer);
        t = (op,{l,r},re); then t :: rest;
  end matchcontinue;
end operator_return;

protected function array_type_list " array types "
  input Integer in_integer;
  input Types.Type in_type;
  output Types_TypeList out_types_typelist;
protected 
  type Types_TypeList = list<Types.Type>;
algorithm 
  out_types_typelist:=
  matchcontinue (in_integer,in_type)
    local
      type Absyn_PathOption = Option<Absyn.Path>;
      type TupleTypes_TTypeAbsyn_PathOption = tuple<Types.TType,Absyn_PathOption>;
      type TupleTypes_TTypeAbsyn_PathOptionList = list<TupleTypes_TTypeAbsyn_PathOption>;
      Integer n_1,n;
      TupleTypes_TTypeAbsyn_PathOption f,t;
      TupleTypes_TTypeAbsyn_PathOptionList r;
    case (0,_) then {}; 
    case (n,t)
      equation 
        n_1 = n - 1;
        f = n_dim_array(n, t);
        r = array_type_list(n_1, t); then f :: r;
  end matchcontinue;
end array_type_list;
end Static;

